import { CloseIcon as _CloseIcon } from "@muya-ui/theme-light/es/icons/CloseIcon";
import _extends from "@babel/runtime/helpers/extends";
import { ClearIcon as _ClearIcon } from "@muya-ui/theme-light/es/icons/ClearIcon";
import { ReminderIcon as _ReminderIcon } from "@muya-ui/theme-light/es/icons/ReminderIcon";
import { InformIcon as _InformIcon } from "@muya-ui/theme-light/es/icons/InformIcon";
import { SuccessIcon as _SuccessIcon } from "@muya-ui/theme-light/es/icons/SuccessIcon";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["children", "type", "showClose", "title", "description", "showIcon", "onClose", "visible", "styles", "size", "className", "style", "appear", "center", "timeout", "childrenAsTitle", "TransitionComponent"];
import React, { useCallback, useMemo, useState } from 'react';
import Animation from '../Animation';
import IconButton from '../IconButton';
import addPx from '../utils/addPx';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { StyledAlert, StyledAlertContainer, StyledClose, StyledDesc, StyledIcon, StyledTitle } from './styled';
var defaultStyles = {
  wrapper: '',
  container: '',
  icon: '',
  title: '',
  desc: '',
  close: ''
};
var Alert = memoForwardRef(function (props, ref) {
  var theme = useTheme();
  var token = theme.components.Alert;

  var children = props.children,
      _props$type = props.type,
      type = _props$type === void 0 ? 'info' : _props$type,
      showClose = props.showClose,
      title = props.title,
      description = props.description,
      _props$showIcon = props.showIcon,
      showIcon = _props$showIcon === void 0 ? token.defaultShowIcon : _props$showIcon,
      onClose = props.onClose,
      _props$visible = props.visible,
      visible = _props$visible === void 0 ? true : _props$visible,
      styles = props.styles,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      className = props.className,
      style = props.style,
      _props$appear = props.appear,
      appear = _props$appear === void 0 ? true : _props$appear,
      _props$center = props.center,
      center = _props$center === void 0 ? !!token.defaultCenter && !props.showClose : _props$center,
      timeout = props.timeout,
      childrenAsTitle = props.childrenAsTitle,
      _props$TransitionComp = props.TransitionComponent,
      TransitionComponent = _props$TransitionComp === void 0 ? Animation.Collapse : _props$TransitionComp,
      otherProps = _objectWithoutProperties(props, _excluded);

  var innerStyles = useStyles('alert', defaultStyles, styles);
  var wrapperStyleItem = useMemo(function () {
    return mergeStyleItem({
      className: className,
      style: style
    }, innerStyles.wrapper);
  }, [className, innerStyles.wrapper, style]);
  var isControlled = ('visible' in props);
  var innerTitle = title;

  if (!title && (typeof children === 'string' || children && childrenAsTitle)) {
    innerTitle = children;
  }

  var hasTitle = !!innerTitle;
  var hasDesc = !!description; // 同时存在 title 和 desc 是两种样式

  var hasTitleAndDesc = hasTitle && hasDesc;

  var _useState = useState(true),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var finalVisible = isControlled ? visible : state;
  var iconNode = useMemo(function () {
    if (showIcon) {
      var typeIcon = token.typeIcon || {
        success: _SuccessIcon,
        info: _InformIcon,
        warning: _ReminderIcon,
        error: _ClearIcon
      };
      var Icon = typeIcon[type];
      var iconBgColor = token.icon.bgColor[type];
      var fontSize = hasTitleAndDesc ? token.icon.hasTitleAndDescSize[size] : token.icon.size[size];
      return /*#__PURE__*/React.createElement(StyledIcon, _extends({}, innerStyles.icon, {
        theme: theme,
        $type: type,
        $hasTitleAndDesc: hasTitleAndDesc,
        $size: size
      }), /*#__PURE__*/React.createElement(Icon, {
        bgColor: iconBgColor,
        fontSize: addPx(fontSize)
      }));
    }
  }, [hasTitleAndDesc, innerStyles.icon, showIcon, size, theme, token.icon.bgColor, token.icon.hasTitleAndDescSize, token.icon.size, token.typeIcon, type]);
  var titleNode = useMemo(function () {
    if (hasTitle) {
      return /*#__PURE__*/React.createElement(StyledTitle, _extends({}, innerStyles.title, {
        theme: theme,
        $size: size,
        $hasDesc: hasDesc,
        $type: type
      }), innerTitle);
    } else if (children) {
      return children;
    }
  }, [children, hasDesc, hasTitle, innerStyles.title, innerTitle, size, theme, type]);
  var contentNode = useMemo(function () {
    if (hasDesc) {
      return /*#__PURE__*/React.createElement(StyledDesc, _extends({
        theme: theme,
        $type: type
      }, innerStyles.desc), description);
    }
  }, [description, hasDesc, innerStyles.desc, theme, type]);
  var handleClose = useCallback(function () {
    if (!isControlled) {
      setState(false);
    }

    onClose && onClose();
  }, [isControlled, onClose]);
  var closeNode = useMemo(function () {
    if (showClose) {
      var Icon = theme.components.Alert.closeIcon || _CloseIcon;
      var closeIconButtonSize = theme.components.Alert.closeIconButtonSize;
      return /*#__PURE__*/React.createElement(StyledClose, _extends({
        theme: theme,
        $size: size,
        $type: type,
        $hasTitleAndDesc: hasTitleAndDesc
      }, innerStyles.close), /*#__PURE__*/React.createElement(IconButton, {
        size: closeIconButtonSize[size],
        onClick: handleClose
      }, /*#__PURE__*/React.createElement(Icon, null)));
    }
  }, [handleClose, hasTitleAndDesc, innerStyles.close, showClose, size, theme, type]);
  return /*#__PURE__*/React.createElement(TransitionComponent, {
    "in": finalVisible,
    appear: appear,
    timeout: timeout
  }, /*#__PURE__*/React.createElement(StyledAlert, _extends({
    theme: theme,
    ref: ref,
    $hasTitle: hasTitle,
    $hasDesc: hasDesc,
    $showIcon: showIcon,
    $showClose: showClose,
    $type: type,
    $size: size,
    $center: center
  }, otherProps, wrapperStyleItem), /*#__PURE__*/React.createElement(StyledAlertContainer, innerStyles.container, iconNode, titleNode, contentNode, closeNode)));
});
export default Alert;