import _extends from "@babel/runtime/helpers/extends";
import { forkRef as _forkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["index", "href", "title", "styles", "children", "onClick"];
import React, { useCallback, useEffect, useRef } from 'react';
import styled from 'styled-components';
import InlineButton from '../InlineButton';
import { Tab } from '../Tabs';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import { useAnchorContext } from './context';
var StyledVerticalTab = styled(Tab).withConfig({
  displayName: "StyledVerticalTab"
})(["margin:0;& ", "{padding:0;text-align:left;}"], InlineButton);
var StyledVerticalTabWrapper = styled.div.withConfig({
  displayName: "StyledVerticalTabWrapper"
})(["padding-left:", "px;"], function (props) {
  return props.theme.spacing.spec.s5;
});
export default memoForwardRef(function (props, ref) {
  var index = props.index,
      href = props.href,
      title = props.title,
      styles = props.styles,
      children = props.children,
      onClick = props.onClick,
      other = _objectWithoutProperties(props, _excluded);

  var tabRef = useRef(null);
  var theme = useTheme();

  var handleRef = _forkRef(tabRef, ref);

  var _useAnchorContext = useAnchorContext(),
      disableHash = _useAnchorContext.disableHash,
      direction = _useAnchorContext.direction,
      activeLink = _useAnchorContext.activeLink,
      registerLink = _useAnchorContext.registerLink,
      unregisterLink = _useAnchorContext.unregisterLink,
      handleScrollTo = _useAnchorContext.handleScrollTo,
      updateIndicator = _useAnchorContext.updateIndicator;

  var handleClick = useCallback(function (e) {
    if (index) {
      handleScrollTo(index);
    }

    if (onClick) {
      onClick(e);
    }
  }, [handleScrollTo, index, onClick]);
  useEffect(function () {
    if (index) {
      registerLink(index);
    }

    return function () {
      if (index) {
        unregisterLink(index);
      }
    };
  }, [index, registerLink, unregisterLink]);
  useEffect(function () {
    if (direction === 'vertical' && activeLink === index && tabRef.current) {
      var height = tabRef.current.clientHeight / 2;
      var top = tabRef.current.offsetTop + height / 2;
      updateIndicator(top, height);
    }
  }, [activeLink, direction, index, updateIndicator]);

  if (direction === 'vertical') {
    var selected = index === activeLink;
    return /*#__PURE__*/React.createElement(StyledVerticalTabWrapper, {
      theme: theme
    }, /*#__PURE__*/React.createElement(StyledVerticalTab, _extends({
      ref: handleRef,
      theme: theme,
      selected: selected,
      buttonType: "normal",
      onClick: handleClick,
      index: index,
      href: disableHash ? undefined : index
    }, other), title), children);
  }

  return /*#__PURE__*/React.createElement(Tab, _extends({
    ref: ref,
    onClick: handleClick,
    index: index,
    styles: styles,
    href: disableHash ? undefined : index
  }, other), title || children);
});