import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useRef, useCallback, useState, useEffect } from 'react';
import { getOffsetTop, getScroll, scrollTo } from './utils';
var sharpMatcherRegx = /#(\S+)$/;
export default function useAnchor(props) {
  var onChange = props.onChange,
      _props$offsetTop = props.offsetTop,
      offsetTop = _props$offsetTop === void 0 ? 0 : _props$offsetTop,
      _props$disableHash = props.disableHash,
      disableHash = _props$disableHash === void 0 ? false : _props$disableHash,
      _props$duration = props.duration,
      duration = _props$duration === void 0 ? 300 : _props$duration,
      bounds = props.bounds,
      _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'vertical' : _props$direction;
  var linksRef = useRef([]);
  var animatingRef = useRef(false);

  var _useState = useState(window),
      _useState2 = _slicedToArray(_useState, 2),
      container = _useState2[0],
      setContainer = _useState2[1];

  var _useState3 = useState(null),
      _useState4 = _slicedToArray(_useState3, 2),
      activeLink = _useState4[0],
      setActiveLink = _useState4[1];

  var _useState5 = useState(0),
      _useState6 = _slicedToArray(_useState5, 2),
      indicatorTop = _useState6[0],
      setIndicatorTop = _useState6[1];

  var _useState7 = useState(0),
      _useState8 = _slicedToArray(_useState7, 2),
      indicatorHeight = _useState8[0],
      setIndicatorHeight = _useState8[1];

  var registerLink = useCallback(function (link) {
    if (linksRef.current.indexOf(link) < 0) {
      linksRef.current.push(link);
    } else {
      _devWarning("[Anchor]: ".concat(link, " is already registered !"));
    }
  }, []);
  var unregisterLink = useCallback(function (link) {
    var index = linksRef.current.indexOf(link);

    if (index > 0) {
      linksRef.current.splice(index, 1);
    }
  }, []);
  var getContainer = useCallback(function (node) {
    if (node && container !== node) {
      setContainer(node);
    }
  }, [container]);
  var setCurrentActiveLink = useCallback(function (link) {
    if (activeLink !== link) {
      setActiveLink(link);

      if (onChange) {
        onChange(link);
      }
    }
  }, [activeLink, onChange]);
  var getCurrentAnchor = useCallback(function (offsetTop) {
    var bounds = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;
    var linkSections = [];
    linksRef.current.forEach(function (link) {
      var sharpLinkMatch = sharpMatcherRegx.exec(link.toString());

      if (!sharpLinkMatch) {
        return;
      }

      var target = document.getElementById(sharpLinkMatch[1]);

      if (target) {
        var top = getOffsetTop(target, container);

        if (top < offsetTop + bounds) {
          linkSections.push({
            link: link,
            top: top
          });
        }
      }
    });

    if (linkSections.length) {
      var maxSection = linkSections.reduce(function (prev, curr) {
        return curr.top > prev.top ? curr : prev;
      });
      return maxSection.link;
    }

    return '';
  }, [container]);
  var handleScrollTo = useCallback(function (link) {
    if (animatingRef.current) return;
    setCurrentActiveLink(link);
    var scrollTop = getScroll(container, true);
    var sharpLinkMatch = sharpMatcherRegx.exec(link);

    if (!sharpLinkMatch) {
      return;
    }

    var targetElement = document.getElementById(sharpLinkMatch[1]);

    if (!targetElement) {
      return;
    }

    var eleOffsetTop = getOffsetTop(targetElement, container);
    var y = scrollTop + eleOffsetTop;
    y -= offsetTop;
    animatingRef.current = true;
    scrollTo(y, {
      callback: function callback() {
        animatingRef.current = false;
      },
      duration: duration,
      container: container
    });
  }, [container, duration, offsetTop, setCurrentActiveLink]);
  var handleScroll = useCallback(function () {
    if (animatingRef.current) {
      return;
    }

    var currentActiveLink = getCurrentAnchor(offsetTop, bounds);
    setCurrentActiveLink(currentActiveLink);
  }, [bounds, getCurrentAnchor, offsetTop, setCurrentActiveLink]);
  var updateIndicator = useCallback(function (top, height) {
    setIndicatorTop(top);
    setIndicatorHeight(height);
  }, []);
  useEffect(function () {
    container.addEventListener('scroll', handleScroll);
    return function () {
      container.removeEventListener('scroll', handleScroll);
    };
  }, [container, handleScroll]);
  useEffect(function () {
    handleScroll();
    var hash = decodeURIComponent(location.hash);

    if (hash && linksRef.current.indexOf(hash) > 0) {
      handleScrollTo(hash);
    }
  }, [container]); // eslint-disable-line

  return {
    activeLink: activeLink,
    disableHash: disableHash,
    direction: direction,
    indicatorHeight: indicatorHeight,
    indicatorTop: indicatorTop,
    registerLink: registerLink,
    unregisterLink: unregisterLink,
    getContainer: getContainer,
    setCurrentActiveLink: setCurrentActiveLink,
    getCurrentAnchor: getCurrentAnchor,
    handleScrollTo: handleScrollTo,
    handleScroll: handleScroll,
    updateIndicator: updateIndicator
  };
}