import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _extends from "@babel/runtime/helpers/extends";
var _excluded = ["children", "timeout", "easing", "style", "onEnter", "onExit", "minHeight", "appear"];
import React, { useRef } from 'react';
import Transition from 'react-transition-group/Transition';
import styled, { css } from 'styled-components';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
var StyledCollapse = styled.div.withConfig({
  displayName: "StyledCollapse"
})(["", ""], function (props) {
  return css(["height:", "px;overflow:hidden;> div{display:flex;> div{width:100%;}}"], props.minHeight);
});

function makeStyles(state, props, wrapperRef, minHeight) {
  var transitionPattern = props.theme.transition.pattern,
      timeout = props.timeout,
      easing = props.easing;
  var wrapperHeight = wrapperRef.current ? wrapperRef.current.clientHeight : minHeight;
  var defaultStyle = {
    height: "".concat(minHeight, "px"),
    transitionProperty: 'height',
    transitionTimingFunction: easing || transitionPattern.easing.status,
    transitionDuration: "".concat(timeout, "ms")
  };
  var transitionStyles = {
    entering: {
      transitionTimingFunction: easing || transitionPattern.easing.enter,
      height: "".concat(wrapperHeight, "px")
    },
    entered: {
      height: 'auto',
      overflow: 'visible'
    },
    exiting: {
      transitionTimingFunction: easing || transitionPattern.easing.leave
    },
    exited: {// visibility: 'hidden',
    }
  };
  return _extends({}, defaultStyle, transitionStyles[state]);
}

var Collapse = memoForwardRef(function (props, ref) {
  var children = props.children,
      timeout = props.timeout,
      easing = props.easing,
      style = props.style,
      onEnterProp = props.onEnter,
      onExitProp = props.onExit,
      _props$minHeight = props.minHeight,
      minHeight = _props$minHeight === void 0 ? 0 : _props$minHeight,
      _props$appear = props.appear,
      appear = _props$appear === void 0 ? true : _props$appear,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var timeoutWithDefault = timeout || theme.transition.spec.duration.normal;
  var wrapperRef = useRef(null);

  var onEnter = function onEnter(node) {
    if (node && node.style) {
      node.style.height = "".concat(minHeight, "px");
    }
    /* istanbul ignore else */


    if (onEnterProp) {
      onEnterProp(node, true);
    }
  };

  var onExit = function onExit(node) {
    var wrapperHeight = wrapperRef.current ? wrapperRef.current.clientHeight : minHeight;

    if (node && node.style) {
      node.style.height = "".concat(wrapperHeight, "px");
    }
    /* istanbul ignore else */


    if (onExitProp) {
      onExitProp(node);
    }
  };

  return /*#__PURE__*/React.createElement(Transition, _extends({
    appear: appear,
    timeout: timeoutWithDefault,
    style: style,
    onEnter: onEnter,
    onExit: onExit
  }, other), function (state) {
    return /*#__PURE__*/React.createElement(StyledCollapse, {
      className: state,
      ref: ref,
      minHeight: minHeight,
      style: _extends({}, style, makeStyles(state, {
        theme: theme,
        timeout: timeoutWithDefault,
        easing: easing
      }, wrapperRef, minHeight))
    }, /*#__PURE__*/React.createElement("div", {
      ref: wrapperRef
    }, /*#__PURE__*/React.createElement("div", null, children)));
  });
});
export default Collapse;