import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["children", "timeout", "easing", "direction", "minScale"];
import React, { useMemo } from 'react';
import Transition from 'react-transition-group/Transition';
import reflow from '../utils/reflow';
import useTheme from '../utils/useTheme';
var Slide = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var children = props.children,
      timeout = props.timeout,
      easing = props.easing,
      _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'up' : _props$direction,
      _props$minScale = props.minScale,
      minScale = _props$minScale === void 0 ? 0.8 : _props$minScale,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var timeoutWithDefault = timeout || theme.transition.spec.duration.normal;

  var handleRef = _useForkRef(children.ref, ref);

  var transitionPattern = theme.transition.pattern;
  var styleWithDirectionMap = useMemo(function () {
    return {
      up: {
        "default": {
          transformOrigin: '0% 0%',
          transform: "scaleY(".concat(minScale, ")")
        },
        entering: {
          transform: 'scaleY(1)'
        },
        exiting: {
          transform: "scaleY(".concat(minScale, ")")
        }
      },
      down: {
        "default": {
          transformOrigin: '100% 100%',
          transform: "scaleY(".concat(minScale, ")")
        },
        entering: {
          transform: 'scaleY(1)'
        },
        exiting: {
          transform: "scaleY(".concat(minScale, ")")
        }
      },
      left: {
        "default": {
          transformOrigin: '0% 0%',
          transform: "scaleX(".concat(minScale, ")")
        },
        entering: {
          transform: 'scaleX(1)'
        },
        exiting: {
          transform: "scaleX(".concat(minScale, ")")
        }
      },
      right: {
        "default": {
          transformOrigin: '100% 0%',
          transform: "scaleX(".concat(minScale, ")")
        },
        entering: {
          transform: 'scaleX(1)'
        },
        exiting: {
          transform: "scaleX(".concat(minScale, ")")
        }
      }
    };
  }, [minScale]);

  function makeStyles(state, direction) {
    var defaultStyle = _extends({
      opacity: 0
    }, styleWithDirectionMap[direction]["default"]);

    var transitionStyles = {
      entering: _extends({
        opacity: 1
      }, styleWithDirectionMap[direction].entering),
      entered: {
        opacity: 1,
        transform: 'none'
      },
      exited: {
        visibility: 'hidden'
      }
    };
    return _extends({}, defaultStyle, transitionStyles[state]);
  }

  var onEnter = function onEnter(node, isAppearing) {
    // 进入动画之前，重置当前direction对应的默认状态
    if (node.style) {
      node.style.transition = 'none';
      node.style.webkitTransition = 'none';
      node.style.transform = styleWithDirectionMap[direction]["default"].transform;
      node.style.webkitTransform = styleWithDirectionMap[direction]["default"].transform;
    }

    reflow(node);
    var innerEasing = easing || transitionPattern.easing.enter; // 设置transition

    if (node.style) {
      node.style.transition = "opacity ".concat(timeoutWithDefault, "ms ").concat(innerEasing, ", transform ").concat(timeoutWithDefault, "ms ").concat(innerEasing);
      node.style.webkitTransition = "opacity ".concat(timeoutWithDefault, "ms ").concat(innerEasing, ", transform ").concat(timeoutWithDefault, "ms ").concat(innerEasing);
    }

    if (props.onEnter) {
      props.onEnter(node, isAppearing);
    }
  };

  var onExit = function onExit(node) {
    node.style.opacity = '0';
    node.style.transform = styleWithDirectionMap[direction].exiting.transform;
    node.style.webkitTransform = styleWithDirectionMap[direction].exiting.transform;
    reflow(node);
    var innerEasing = easing || transitionPattern.easing.leave;
    node.style.transition = "opacity ".concat(timeoutWithDefault, "ms ").concat(innerEasing, ",transform ").concat(timeoutWithDefault, "ms ").concat(innerEasing);
    node.style.webkitTransition = "opacity ".concat(timeoutWithDefault, "ms ").concat(innerEasing, ",transform ").concat(timeoutWithDefault, "ms ").concat(innerEasing);

    if (props.onExit) {
      props.onExit(node);
    }
  };

  return /*#__PURE__*/React.createElement(Transition, _extends({
    appear: true
  }, other, {
    timeout: timeoutWithDefault,
    onEnter: onEnter,
    onExit: onExit
  }), function (state) {
    return /*#__PURE__*/React.cloneElement(children, _extends({}, children.props, {
      style: _extends({}, makeStyles(state, direction), children.props.style),
      ref: handleRef
    }));
  });
});
export default Slide;