import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["dot", "isStroke", "showZero", "detached", "value", "color", "size", "max", "styles", "children"];
import React, { useMemo, useRef } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useAdjustMiniFont from '../utils/useAdjustMiniFont';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { StyledBadge, StyledBadgeContainer, StyledBadgeInnerText, StyledIndependentBadge } from './styled';
import { getDisplayText } from './utils';
var Badge = memoForwardRef(function (props, ref) {
  var _props$dot = props.dot,
      dot = _props$dot === void 0 ? false : _props$dot,
      _props$isStroke = props.isStroke,
      isStroke = _props$isStroke === void 0 ? false : _props$isStroke,
      _props$showZero = props.showZero,
      showZero = _props$showZero === void 0 ? false : _props$showZero,
      _props$detached = props.detached,
      detached = _props$detached === void 0 ? false : _props$detached,
      value = props.value,
      color = props.color,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$max = props.max,
      max = _props$max === void 0 ? 99 : _props$max,
      styles = props.styles,
      children = props.children,
      rest = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var token = theme.components.Badge;
  var fontSize = token.fontSize[size];
  var textRef = useRef(null);
  var defaultStyles = useMemo(function () {
    return {
      container: '',
      badge: ''
    };
  }, []);
  var innerStyles = useStyles('badge', defaultStyles, styles);
  var styledBadgeProps = useMemo(function () {
    return {
      theme: theme,
      $detached: detached,
      $isStroke: isStroke,
      $value: value,
      $showZero: showZero,
      $dot: dot,
      $size: size,
      $color: color,
      $text: ''
    };
  }, [color, detached, dot, isStroke, showZero, size, theme, value]);
  var innerText = useMemo(function () {
    if (dot) {
      return '';
    }

    var displayText = getDisplayText(value, max); // 12px 以下的字需要 transform 一下

    if (fontSize < 12) {
      return /*#__PURE__*/React.createElement(StyledBadgeInnerText, {
        ref: textRef,
        theme: theme
      }, displayText);
    }

    return displayText;
  }, [dot, fontSize, max, theme, value]);
  useAdjustMiniFont(textRef, fontSize);
  var finalChildren = useMemo(function () {
    return children ? /*#__PURE__*/React.createElement(StyledBadgeContainer, innerStyles.container, /*#__PURE__*/React.createElement(StyledBadge, _extends({
      ref: ref
    }, styledBadgeProps, innerStyles.badge, rest), innerText), children) : /*#__PURE__*/React.createElement(StyledIndependentBadge, _extends({
      ref: ref
    }, styledBadgeProps, innerStyles.badge, rest), innerText);
  }, [children, innerStyles.badge, innerStyles.container, innerText, ref, rest, styledBadgeProps]);
  return finalChildren;
});
export default Badge;