import _extends from "@babel/runtime/helpers/extends";
import _typeof from "@babel/runtime/helpers/typeof";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["maxItemCountPerPage", "width", "size", "children", "onScroll", "noShadow"];
import React, { useMemo } from 'react';
import useTheme from '../utils/useTheme';
import { StyledBaseMenu, StyledBaseMenuWrapper } from './styled';
import { getMenuItemCountFromChildren } from './utils';
var Menu = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var maxItemCountPerPage = props.maxItemCountPerPage,
      width = props.width,
      size = props.size,
      children = props.children,
      onScroll = props.onScroll,
      noShadow = props.noShadow,
      restProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var hasScrollBar = useMemo(function () {
    if (maxItemCountPerPage) {
      var count = getMenuItemCountFromChildren(children);
      return count > maxItemCountPerPage;
    } else {
      return false;
    }
  }, [children, maxItemCountPerPage]);
  var wrapperWidth = useMemo(function () {
    if (width && _typeof(width) === 'object') {
      return width[size];
    } else {
      return width;
    }
  }, [size, width]);
  return /*#__PURE__*/React.createElement(StyledBaseMenu, _extends({
    ref: ref,
    theme: theme,
    $size: size,
    $width: wrapperWidth,
    $hasScrollBar: hasScrollBar,
    $maxItemCountPerPage: maxItemCountPerPage,
    $noShadow: noShadow
  }, restProps), /*#__PURE__*/React.createElement(StyledBaseMenuWrapper, {
    onScroll: onScroll
  }, children));
});
Menu.defaultProps = {
  size: 'm'
};
export default /*#__PURE__*/React.memo(Menu);