import _extends from "@babel/runtime/helpers/extends";
import { SelectIcon as _SelectIcon } from "@muya-ui/theme-light/es/icons/SelectIcon";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["size", "children", "selected", "disabled", "active", "loading", "checkable", "loadingIcon", "selectedIcon", "onClick"];
import React, { cloneElement, useEffect, useMemo, useRef } from 'react';
import { isElement } from 'react-is';
import { Checkbox } from '../Checkbox';
import Spin from '../Spin';
import memoForwardRef from '../utils/memoForwardRef';
import scrollIntoView from '../utils/scrollIntoView';
import useTheme from '../utils/useTheme';
import { StyledBaseMenuItem } from './styled';
var MenuItem = memoForwardRef(function (props, ref) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      children = props.children,
      _props$selected = props.selected,
      selected = _props$selected === void 0 ? false : _props$selected,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      _props$active = props.active,
      active = _props$active === void 0 ? false : _props$active,
      _props$loading = props.loading,
      loading = _props$loading === void 0 ? false : _props$loading,
      _props$checkable = props.checkable,
      checkable = _props$checkable === void 0 ? false : _props$checkable,
      loadingIcon = props.loadingIcon,
      selectedIcon = props.selectedIcon,
      onClick = props.onClick,
      restProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var itemRef = useRef(null);

  var handleRef = _useForkRef(ref, itemRef);

  var token = theme.components.BaseMenu;
  var suffixIconNode;
  var SuffixIcon;

  if (loading) {
    suffixIconNode = loadingIcon || /*#__PURE__*/React.createElement(Spin, {
      style: {
        flexShrink: 0,
        marginLeft: token.iconMarginLeft,
        color: token.iconColor
      }
    });
  } else if (selected && !checkable) {
    SuffixIcon = token.selectedIcon || _SelectIcon;
    suffixIconNode = selectedIcon || /*#__PURE__*/React.createElement(SuffixIcon, {
      style: {
        flexShrink: 0,
        marginLeft: token.iconMarginLeft,
        color: token.iconColor
      }
    });
  }

  var finalChildren = useMemo(function () {
    return isElement(children) ? /*#__PURE__*/cloneElement(children, {
      size: size,
      theme: theme
    }) : /*#__PURE__*/React.createElement("span", null, children);
  }, [children, size, theme]);
  useEffect(function () {
    if (active && itemRef.current) {
      scrollIntoView(itemRef.current, {
        onlyFirstScrollableParent: true,
        behavior: 'auto',
        isScrollable: function isScrollable(element) {
          return getComputedStyle(element).overflowY === 'auto';
        },
        align: {
          top: 0.5
        }
      });
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [active]);
  return /*#__PURE__*/React.createElement(StyledBaseMenuItem, _extends({
    ref: handleRef,
    theme: theme,
    $size: size,
    $active: active,
    $selected: selected,
    $checkable: checkable,
    $disabled: disabled,
    onClick: onClick
  }, restProps), checkable ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Checkbox, {
    checked: selected,
    onClick: onClick,
    disabled: disabled
  }), finalChildren) : finalChildren, suffixIconNode);
});
MenuItem.displayName = 'MenuItem';
export default MenuItem;