import { GuildRightIcon as _GuildRightIcon } from "@muya-ui/theme-light/es/icons/GuildRightIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["key", "style"],
    _excluded2 = ["url", "label"],
    _excluded3 = ["size", "fontWeight", "separators", "children", "items", "headItem", "styles", "maxNode", "ellipsisRule"],
    _excluded4 = ["label", "url"];
import React, { useCallback, useMemo } from 'react';
import styled, { css } from 'styled-components';
import Dropdown from '../Dropdown';
import IconButton from '../IconButton';
import InlineButton from '../InlineButton';
import { Menu, MenuItem } from '../Menu';
import { arraySpliceWithFn } from '../utils/array';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
import { BaseNode, SeparatorHeadWrapper, SeparatorWrapper, StyledHeadSeparator, StyledMoreIcon } from './styled';
var defaultPreCount = 1;
var defaultSuffixCount = 2;

var DropdownContainer = function DropdownContainer(_ref) {
  var children = _ref.children,
      size = _ref.size;
  return /*#__PURE__*/React.createElement(Dropdown, {
    size: size,
    overlay: /*#__PURE__*/React.createElement(Menu, null, children)
  }, /*#__PURE__*/React.createElement(IconButton, {
    size: size
  }, /*#__PURE__*/React.createElement(StyledMoreIcon, null)));
};

var genEllipsisChildrenNode = function genEllipsisChildrenNode(size, children) {
  return /*#__PURE__*/React.createElement(DropdownContainer, {
    size: size
  }, children.map(function (child, index) {
    var props = child.props || {};

    var key = props.key,
        _props$style = props.style,
        style = _props$style === void 0 ? {} : _props$style,
        otherProps = _objectWithoutProperties(props, _excluded);

    return /*#__PURE__*/React.createElement(MenuItem, {
      key: key || index
    }, /*#__PURE__*/React.cloneElement(child, _extends({
      style: _extends({
        width: '100%',
        justifyContent: 'flex-start'
      }, style),
      size: size
    }, otherProps)));
  }));
};

var genEllipsisPropNode = function genEllipsisPropNode(size, items) {
  return /*#__PURE__*/React.createElement(DropdownContainer, {
    size: size
  }, items.map(function (item, index) {
    var url = item.url,
        label = item.label,
        otherProps = _objectWithoutProperties(item, _excluded2);

    return /*#__PURE__*/React.createElement(MenuItem, {
      key: index
    }, /*#__PURE__*/React.createElement(InlineButton, _extends({}, otherProps, {
      fontWeight: "lighter",
      type: "secondary",
      href: item.url,
      size: size
    }), item.label));
  }));
};

var defaultStyles = {
  item: '',
  headSeparator: '',
  separator: ''
};
var BreadcrumbsPure = memoForwardRef(function (props, ref) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      fontWeightProp = props.fontWeight,
      separators = props.separators,
      children = props.children,
      items = props.items,
      headItem = props.headItem,
      styles = props.styles,
      _props$maxNode = props.maxNode,
      maxNode = _props$maxNode === void 0 ? 4 : _props$maxNode,
      _props$ellipsisRule = props.ellipsisRule,
      ellipsisRule = _props$ellipsisRule === void 0 ? [defaultPreCount, defaultSuffixCount] : _props$ellipsisRule,
      otherProps = _objectWithoutProperties(props, _excluded3);

  var theme = useTheme();
  var token = theme.components.Breadcrumbs;
  var DefaultSeparator = token.separator || _GuildRightIcon;
  var childNum = React.Children.count(children);
  var separatorSize = token.separatorSize[size];
  var innerStyles = useStyles('breadcrumbs', defaultStyles, styles);
  var innerSeparators = useMemo(function () {
    if (!separators && headItem) {
      return ['|', '>'];
    }

    return separators || ['>'];
  }, [headItem, separators]);
  var innerItems = useMemo(function () {
    if (headItem && items) {
      items.unshift(headItem);
    }

    return items;
  }, [headItem, items]);
  var renderSeparator = useCallback(function (key) {
    if (key === '|') {
      return /*#__PURE__*/React.createElement(SeparatorHeadWrapper, innerStyles.headSeparator, /*#__PURE__*/React.createElement(StyledHeadSeparator, null));
    } else if (key === '>') {
      return /*#__PURE__*/React.createElement(SeparatorWrapper, innerStyles.separator, /*#__PURE__*/React.createElement(DefaultSeparator, {
        style: {
          width: separatorSize,
          height: separatorSize
        }
      }));
    }

    return key;
  }, [innerStyles.headSeparator, innerStyles.separator, separatorSize]);
  var separatorLastIndex = innerSeparators.length - 1;
  var finalNode = useMemo(function () {
    if (childNum) {
      var childrenArr = React.Children.toArray(children); // 省略的相关处理

      if (childrenArr.length > maxNode) {
        arraySpliceWithFn(childrenArr, ellipsisRule[0] || defaultPreCount, ellipsisRule[1] || defaultSuffixCount, genEllipsisChildrenNode.bind(null, size));
      }

      return childrenArr.map(function (child, childIndex) {
        if (! /*#__PURE__*/React.isValidElement(child)) {
          return null;
        }

        var isLast = childIndex === childrenArr.length - 1;
        var cloneNode = /*#__PURE__*/React.cloneElement(child, _extends({
          size: size,
          fontWeight: isLast ? token.currentFontWeight : fontWeightProp || 'lighter'
        }, child.props));
        var innerNode;

        if (!isLast) {
          var separatorKey = innerSeparators[childIndex] || innerSeparators[separatorLastIndex];
          innerNode = renderSeparator(separatorKey);
        }

        return /*#__PURE__*/React.createElement(React.Fragment, {
          key: childIndex
        }, cloneNode, innerNode);
      });
    } else if (innerItems && innerItems.length) {
      // 省略的相关处理
      if (innerItems.length > maxNode) {
        arraySpliceWithFn(innerItems, ellipsisRule[0] || defaultPreCount, ellipsisRule[1] || defaultSuffixCount, genEllipsisPropNode.bind(null, size));
      }

      return innerItems.map(function (item, itemIndex) {
        var innerNode;
        var isLast = itemIndex === innerItems.length - 1;

        if (!isLast) {
          var separatorKey = innerSeparators[itemIndex] || innerSeparators[separatorLastIndex];
          innerNode = renderSeparator(separatorKey);
        } // 如果是React节点则是省略号节点, 只需要处理下separator


        if ( /*#__PURE__*/React.isValidElement(item)) {
          return /*#__PURE__*/React.createElement(React.Fragment, {
            key: itemIndex
          }, item, innerNode);
        } else {
          var itemLabel = item.label,
              itemUrl = item.url,
              otherItemProps = _objectWithoutProperties(item, _excluded4);

          var href;
          var buttonType = 'strong';
          var fontWeight = fontWeightProp || token.currentFontWeight;
          var constant = isLast;

          if (!isLast) {
            href = itemUrl;
            buttonType = 'secondary';
            fontWeight = 'lighter';
          }

          return /*#__PURE__*/React.createElement(React.Fragment, {
            key: itemIndex
          }, /*#__PURE__*/React.createElement(InlineButton, _extends({}, innerStyles.item, otherItemProps, {
            fontWeight: fontWeight,
            type: buttonType,
            constant: constant,
            size: size,
            href: href
          }), itemLabel), innerNode);
        }
      });
    }
  }, [childNum, children, ellipsisRule, fontWeightProp, innerItems, innerSeparators, innerStyles.item, maxNode, renderSeparator, separatorLastIndex, size, token.currentFontWeight]);
  return /*#__PURE__*/React.createElement(BaseNode, _extends({}, otherProps, {
    ref: ref
  }), finalNode);
});
var BreadcrumbsWithoutDefaultTheme = styled(BreadcrumbsPure).withConfig({
  displayName: "BreadcrumbsWithoutDefaultTheme"
})(["position:relative;display:flex;align-items:center;", ""], function (props) {
  var size = props.size || 'm';
  var token = props.theme.components.Breadcrumbs;
  var btnToken = props.theme.components.Button;
  var separatorSize = token.separatorSize[size];
  return css(["color:", ";& ", "{width:", "px;}& ", "{width:", "px;}& ", "{border-left:1px solid ", ";height:", "px;}& ", "{font-size:", "px;}"], token.separatorColor, SeparatorHeadWrapper, token.separatorHeadWidth, SeparatorWrapper, token.separatorWidth, StyledHeadSeparator, token.separatorColor, separatorSize, StyledMoreIcon, btnToken.fontSize[size]);
});
var Breadcrumbs = withThemeForStyled(BreadcrumbsWithoutDefaultTheme);
export default Breadcrumbs;