import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
var _excluded = ["size", "htmlType", "disabled", "component", "children", "textFine", "fontWeight", "onClick", "type", "width", "loading", "busy", "suffixNode", "prefixNode", "styles", "disableSiblingMargin", "shape"];
import React, { useMemo } from 'react';
import SpinIcon from '../Spin/SpinIcon';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { StyledButton, StyledLink, StyledSide } from './styled';
var defaultStyles = {
  prefix: '',
  suffix: ''
};
var ButtonCore = memoForwardRef(function (props, ref) {
  // @TODO remove next breaking change
  _devWarning('[Button]: prop `textFine` is deprecated, use `fontWeight="lighter" instead.`', 'textFine' in props);

  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$htmlType = props.htmlType,
      htmlType = _props$htmlType === void 0 ? 'button' : _props$htmlType,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      _props$component = props.component,
      asProp = _props$component === void 0 ? 'button' : _props$component,
      children = props.children,
      textFine = props.textFine,
      fontWeight = props.fontWeight,
      onClick = props.onClick,
      type = props.type,
      width = props.width,
      _props$loading = props.loading,
      loading = _props$loading === void 0 ? false : _props$loading,
      _props$busy = props.busy,
      busy = _props$busy === void 0 ? false : _props$busy,
      suffixNode = props.suffixNode,
      prefixNode = props.prefixNode,
      styles = props.styles,
      disableSiblingMargin = props.disableSiblingMargin,
      shape = props.shape,
      otherProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('button', defaultStyles, styles);
  var buttonProps = useMemo(function () {
    var buttonProps = {};

    if (asProp === 'button') {
      buttonProps = {
        disabled: disabled,
        type: htmlType
      };
    }

    return buttonProps;
  }, [asProp, disabled, htmlType]);
  var innerBusy = busy;

  if (loading) {
    innerBusy = true;
  }

  var childrenNode = useMemo(function () {
    var innerSuffixNode = suffixNode;
    var innerPrefixNode = prefixNode;
    var isCircleOrSquare = shape === 'circle' || shape === 'square';
    var showChildren = !loading || !isCircleOrSquare;

    if (loading && theme.components.Button.loadingPosition === 'suffix') {
      innerSuffixNode = /*#__PURE__*/React.createElement(SpinIcon, {
        color: "currentColor"
      });
    } else if (loading) {
      innerPrefixNode = /*#__PURE__*/React.createElement(SpinIcon, {
        color: "currentColor"
      });
    }

    return /*#__PURE__*/React.createElement(React.Fragment, null, innerPrefixNode ? /*#__PURE__*/React.createElement(StyledSide, _extends({}, innerStyles.prefix, {
      $isCircleOrSquare: isCircleOrSquare,
      shape: shape,
      theme: theme,
      prefixNode: innerPrefixNode
    }), innerPrefixNode) : null, showChildren && children, innerSuffixNode ? /*#__PURE__*/React.createElement(StyledSide, _extends({}, innerStyles.suffix, {
      $isCircleOrSquare: isCircleOrSquare,
      shape: shape,
      theme: theme,
      suffixNode: innerSuffixNode
    }), innerSuffixNode) : null);
  }, [children, innerStyles.prefix, innerStyles.suffix, loading, prefixNode, shape, suffixNode, theme]);

  var handleClick = _useEventCallback(function (e) {
    if (!innerBusy && !disabled) {
      onClick && onClick(e);
    } else {
      e.preventDefault();
    }
  });

  if (otherProps.href || asProp === 'a') {
    return /*#__PURE__*/React.createElement(StyledLink, _extends({
      ref: ref,
      role: "button",
      "aria-disabled": disabled,
      theme: theme,
      onClick: handleClick
    }, otherProps), childrenNode);
  }

  return /*#__PURE__*/React.createElement(StyledButton, _extends({
    ref: ref,
    as: asProp,
    theme: theme,
    onClick: handleClick
  }, buttonProps, otherProps), childrenNode);
});
export default ButtonCore;