import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["type", "size", "htmlType", "disabled", "busy", "shape", "block", "children", "plain", "outline", "lightShadow"],
    _excluded2 = ["disabled", "busy", "shape", "size", "block", "type", "plain", "outline", "lightShadow", "offBorder"];
import React from 'react';
import styled, { css } from 'styled-components';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
import { siblingCss } from './mixins';
var BaseNode = styled.div.withConfig({
  displayName: "BaseNode"
})([""]);

/* eslint-disable complexity */

/**
 * 在 Group 中需要根据 z-index 来决定谁的 border 在上面
 * @param childIndex
 * @param configMap
 * @param disabled
 */
export function getOffBorder(childIndex, configMap, disabled) {
  var _configMap$childIndex = configMap[childIndex],
      groupType = _configMap$childIndex.groupType,
      zIndex = _configMap$childIndex.zIndex;
  var offBorder;
  var prevItem = configMap[childIndex - 1];
  var nextItem = configMap[childIndex + 1];

  if (disabled && (groupType === 'head' || groupType === 'group')) {
    offBorder = 'right';
  } else if (disabled && groupType === 'tail') {
    offBorder = 'no';
  } // 是第一个的时候，有两种情况
  else if (groupType === 'head' && nextItem && nextItem.zIndex < zIndex) {
    offBorder = 'no';
  } else if (groupType === 'head') {
    offBorder = 'right';
  } // 是最后的一个的时候，也有两种情况
  else if (groupType === 'tail' && prevItem && prevItem.zIndex > zIndex) {
    offBorder = 'left';
  } else if (groupType === 'tail') {
    offBorder = 'no';
  } // 在中间的时候，有四种情况
  else if (groupType === 'group' && nextItem.zIndex > zIndex && prevItem.zIndex > zIndex) {
    offBorder = 'both';
  } else if (groupType === 'group' && nextItem.zIndex < zIndex && prevItem.zIndex < zIndex) {
    offBorder = 'no';
  } else if (groupType === 'group' && nextItem.zIndex < zIndex && prevItem.zIndex > zIndex) {
    offBorder = 'left';
  } else {
    offBorder = 'right';
  }

  return offBorder;
}
/* eslint-enable complexity */

var ButtonGroupWithoutStyle = /*#__PURE__*/React.memo( /*#__PURE__*/React.forwardRef(function (props, ref) {
  var type = props.type,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      htmlType = props.htmlType,
      disabled = props.disabled,
      busy = props.busy,
      shape = props.shape,
      block = props.block,
      children = props.children,
      plain = props.plain,
      outline = props.outline,
      lightShadow = props.lightShadow,
      otherProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var childrenLen = React.Children.count(children);
  var notOnlyOne = childrenLen > 1;
  var resultChildren;
  var _theme$components$But = theme.components.Button,
      zIndexOrder = _theme$components$But.zIndexOrder,
      plainIsLight = _theme$components$But.plainIsLight;

  if (children) {
    var configMap = {};
    React.Children.forEach(children, function (child, childIndex) {
      if (! /*#__PURE__*/React.isValidElement(child)) {
        return;
      }

      var currentType = child.props.type || type;
      var groupType;

      if (notOnlyOne && !childIndex) {
        groupType = 'head';
      } else if (notOnlyOne && childIndex === childrenLen - 1) {
        groupType = 'tail';
      } else if (notOnlyOne) {
        groupType = 'group';
      }

      configMap[childIndex] = {
        groupType: groupType,
        zIndex: zIndexOrder[currentType] || 0
      };
    });
    resultChildren = React.Children.map(children, function (child, childIndex) {
      if (! /*#__PURE__*/React.isValidElement(child)) {
        return null;
      }

      var _child$props = child.props,
          childDisabled = _child$props.disabled,
          childBusy = _child$props.busy,
          childShape = _child$props.shape,
          childSize = _child$props.size,
          childBlock = _child$props.block,
          childType = _child$props.type,
          childPlain = _child$props.plain,
          childOutline = _child$props.outline,
          childLightShadow = _child$props.lightShadow,
          childOffBorder = _child$props.offBorder,
          otherChildProps = _objectWithoutProperties(_child$props, _excluded2); // 布尔值需要注意一下


      var currentDisabled = childDisabled !== undefined ? childDisabled : disabled;
      var offBorder = childOffBorder || getOffBorder(childIndex, configMap, currentDisabled);
      var currentType = childType || type;
      var groupType = configMap[childIndex].groupType;
      var finalPlain = typeof childPlain !== 'undefined' ? childPlain : plain;
      var finalOutline = typeof childOutline !== 'undefined' ? childOutline : outline;
      var finalLightShadow = typeof childLightShadow !== 'undefined' ? childLightShadow : lightShadow;

      if (typeof finalOutline === 'undefined' && finalPlain && !plainIsLight) {
        finalOutline = true;
      }

      if (typeof finalLightShadow === 'undefined' && finalPlain && plainIsLight) {
        finalLightShadow = true;
      }

      return /*#__PURE__*/React.cloneElement(child, _extends({}, otherChildProps, {
        disabled: currentDisabled,
        busy: typeof childBusy !== 'undefined' ? childBusy : busy,
        shape: childShape || shape,
        outline: finalOutline,
        lightShadow: finalLightShadow,
        type: currentType,
        offBorder: offBorder,
        groupType: groupType,
        size: size,
        block: block
      }));
    });
  }

  return /*#__PURE__*/React.createElement(BaseNode, _extends({
    ref: ref
  }, otherProps), resultChildren);
}));
var StyledButtonGroup = styled(ButtonGroupWithoutStyle).withConfig({
  displayName: "StyledButtonGroup"
})(["", ""], function (props) {
  var _props$size2 = props.size,
      size = _props$size2 === void 0 ? 'm' : _props$size2,
      block = props.block,
      theme = props.theme,
      _props$disableSibling = props.disableSiblingMargin,
      disableSiblingMargin = _props$disableSibling === void 0 ? false : _props$disableSibling;
  return css(["position:relative;display:inline-flex;", " ", ""], block && 'width: 100%;', siblingCss({
    size: size,
    theme: theme,
    inGroup: false,
    disableSiblingMargin: disableSiblingMargin
  }));
});
export var ButtonGroup = withThemeForStyled(StyledButtonGroup);
export default ButtonGroup;