import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import { useEffectOnce as _useEffectOnce } from "@muya-ui/utils/es/hooks/useEffectOnce";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["type", "constant"],
    _excluded2 = ["loading", "autoLoading", "onClick"];
import React from 'react';
import styled, { css } from 'styled-components';
import addPx from '../utils/addPx';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
import ButtonCore from './ButtonCore';
import { highlightNeutralColorButtonType, siblingCss } from './mixins';
import useBtnAutoLoading from './useBtnAutoLoading';
var InlineButtonPure = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var type = props.type,
      constant = props.constant,
      otherProps = _objectWithoutProperties(props, _excluded);

  return /*#__PURE__*/React.createElement(ButtonCore, _extends({
    ref: ref
  }, otherProps));
});

function buttonCSS(props) {
  var _props$type = props.type,
      type = _props$type === void 0 ? 'normal' : _props$type,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      theme = props.theme,
      busy = props.busy,
      disabled = props.disabled,
      innerFontWeight = props.fontWeight,
      constant = props.constant,
      width = props.width,
      loading = props.loading,
      _props$disableSibling = props.disableSiblingMargin,
      disableSiblingMargin = _props$disableSibling === void 0 ? false : _props$disableSibling;
  var btnToken = theme.components.Button;
  var fontSize = btnToken.fontSize,
      typeColor = btnToken.typeColor,
      inlineButtonTypeColor = btnToken.inlineButtonTypeColor;
  var fontWeight = btnToken.fontWeight[innerFontWeight || 'normal'];
  var colorToken = inlineButtonTypeColor[type] || typeColor[type];
  var cssObj = {};
  cssObj.color = colorToken.normal;

  if (disabled) {
    cssObj.color = theme.colors.pattern.text.disabled;
  }

  var interactive = !(busy || disabled || loading);
  var interactiveCSS;

  if (interactive && !constant) {
    var colorItem = colorToken;

    if (highlightNeutralColorButtonType(type)) {
      colorItem = typeColor.primary;
    }

    interactiveCSS = css(["cursor:pointer;&:hover{color:", ";}&:active{color:", ";}"], colorItem.hover, colorItem.click);
  } else if (!constant) {
    interactiveCSS = css(["cursor:not-allowed;"]);
  } else {
    interactiveCSS = css(["cursor:default;"]);
  }

  if (!disabled && (busy || loading)) {
    cssObj.opacity = theme.opacity.pattern.disabled;
  }

  if (width) {
    cssObj.width = addPx(width);
  }

  var baseCSS = css(cssObj);
  return css(["", " font-weight:", ";font-size:", "px;", " ", ""], baseCSS, fontWeight, fontSize[size], interactiveCSS, siblingCss({
    size: size,
    theme: theme,
    inGroup: false,
    disableSiblingMargin: disableSiblingMargin
  }));
}

export var StyledInlineButton = styled(InlineButtonPure).withConfig({
  displayName: "StyledInlineButton"
})(["", ""], function (props) {
  var textFine = props.textFine,
      propFontWeight = props.fontWeight;
  var innerFontWeight = propFontWeight;

  if (!innerFontWeight && textFine) {
    innerFontWeight = 'lighter';
  }

  var newProps = _extends({}, props, {
    fontWeight: innerFontWeight
  });

  return css(["background:transparent;border:none;padding:0;", ""], buttonCSS(newProps));
});
var InlineButton = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var loading = props.loading,
      autoLoading = props.autoLoading,
      onClick = props.onClick,
      otherProps = _objectWithoutProperties(props, _excluded2);

  var _useBtnAutoLoading = useBtnAutoLoading(props),
      _useBtnAutoLoading2 = _slicedToArray(_useBtnAutoLoading, 2),
      realLoading = _useBtnAutoLoading2[0],
      handleClick = _useBtnAutoLoading2[1];

  var theme = useTheme();

  _useEffectOnce(function () {
    _devWarning('[InlineButton]: prop `weakLevel` is deprecated.', 'weakLevel' in props);
  });

  return /*#__PURE__*/React.createElement(StyledInlineButton, _extends({
    ref: ref,
    onClick: handleClick,
    loading: realLoading
  }, otherProps, {
    theme: theme
  }));
});
var WrapInlineButton = withThemeForStyled(styled(InlineButton).withConfig({
  displayName: "WrapInlineButton"
})([""]));
WrapInlineButton.__MUYA_BUTTON = true;
export default WrapInlineButton;