import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo } from 'react';
import styled from 'styled-components';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import CalendarFooter from './CalendarFooter';
import CalendarHead from './CalendarHead';
import CalendarPanel from './CalendarPanel';
import useCalendar from './useCalendar';
var Root = styled.div.withConfig({
  displayName: "Root"
})(["width:", "px;background-color:", ";"], function (props) {
  return props.theme.components.Calendar.defaultWidth;
}, function (props) {
  return props.theme.colors.pattern.background.higher;
});
var defaultStyles = {
  head: '',
  panel: '',
  footer: ''
};
var Calendar = memoForwardRef(function (props, ref) {
  var _useCalendar = useCalendar(props),
      finalUIState = _useCalendar.finalUIState,
      finalSelectedDate = _useCalendar.finalSelectedDate,
      finalOptions = _useCalendar.finalOptions,
      handleItemClick = _useCalendar.handleItemClick,
      handlePrevMonth = _useCalendar.handlePrevMonth,
      handleNextMonth = _useCalendar.handleNextMonth,
      handlePrevYear = _useCalendar.handlePrevYear,
      handleNextYear = _useCalendar.handleNextYear,
      handleSwitch = _useCalendar.handleSwitch,
      finalDisableRange = _useCalendar.finalDisableRange,
      finalTodayDate = _useCalendar.finalTodayDate,
      readOnly = _useCalendar.readOnly,
      otherProps = _useCalendar.otherProps,
      styles = _useCalendar.styles,
      hideItemOutside = _useCalendar.hideItemOutside,
      renderDecadeItem = _useCalendar.renderDecadeItem,
      renderMonthItem = _useCalendar.renderMonthItem,
      renderYearItem = _useCalendar.renderYearItem,
      renderMonthLabel = _useCalendar.renderMonthLabel,
      disableDate = _useCalendar.disableDate;

  var theme = useTheme();
  var pickStyles = useMemo(function () {
    if (styles) {
      var head = styles.head,
          panel = styles.panel,
          footer = styles.footer;
      return {
        head: head,
        panel: panel,
        footer: footer
      };
    }
  }, [styles]);
  var innerStyles = useStyles('calendar', defaultStyles, pickStyles); // CalendarFooter 本身就 memo了，单个节点不需要 memo 了

  var footerNode;

  if (finalOptions && finalOptions.length) {
    footerNode = /*#__PURE__*/React.createElement(CalendarFooter, _extends({}, innerStyles.footer, {
      styles: styles,
      options: finalOptions
    }));
  } // 本就是解构了传给 pure component，所以不需要 memo


  var panelProps = {
    styles: styles,
    todayDate: finalTodayDate,
    disableRange: finalDisableRange,
    disableDate: disableDate,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    readOnly: readOnly
  };
  return /*#__PURE__*/React.createElement(Root, _extends({}, otherProps, {
    theme: theme,
    ref: ref
  }), /*#__PURE__*/React.createElement(CalendarHead, _extends({}, innerStyles.head, {
    styles: styles,
    viewType: finalUIState.viewType,
    viewDate: finalUIState.viewDate,
    onPrevMonth: handlePrevMonth,
    onNextMonth: handleNextMonth,
    onPrevYear: handlePrevYear,
    onNextYear: handleNextYear,
    onSwitch: handleSwitch
  })), /*#__PURE__*/React.createElement(CalendarPanel, _extends({}, innerStyles.panel, {
    viewType: finalUIState.viewType,
    selected: finalSelectedDate,
    viewDate: finalUIState.viewDate,
    onItemClick: handleItemClick
  }, panelProps)), footerNode);
});
export default Calendar;