import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["viewType", "viewDate", "selected", "range", "start", "end", "isRange", "disableRange", "disableDate", "onItemClick", "onItemHover", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "todayDate", "styles", "min", "max", "fixedStart", "fixedEnd", "readOnly", "panelBlockProps", "isWeek", "hoveredWeek", "selectedWeek", "disableWeek"];
import React, { useMemo } from 'react';
import styled, { css } from 'styled-components';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import CalendarItem from './CalendarItem';
import { createCheckDisabled, createGetStatus, setOnlyStart } from './calendarPanelUtils';
import { getDateType, getDaysOfMonth, getDecadeYears, getMonthsOfYear } from './utils';
var Container = styled.div.withConfig({
  displayName: "Container"
})(["display:flex;flex-wrap:wrap;position:relative;"]);
var StyledItem = styled(CalendarItem).withConfig({
  displayName: "StyledItem"
})([""]);
var StyledItemM = /*#__PURE__*/React.memo(StyledItem);
var defaultStyles = {
  panelMonthBlock: '',
  panelYearBlock: '',
  panelDecadeBlock: '',
  panelItem: '',
  panelLabelItem: '',
  panelLabelBlock: ''
};
var Root = styled.div.withConfig({
  displayName: "Root"
})(["width:100%;", ""], function (props) {
  var viewType = props.$type,
      theme = props.theme;
  var panelToken = theme.components.Calendar.panel;
  var typeCss;

  if (viewType === 'month') {
    typeCss = css(["padding:", ";& ", "{padding:", ";}& ", "{width:", "%;margin-bottom:", "px;}"], panelToken.containerMonthPadding, Container, panelToken.monthPadding, StyledItem, 1 / 7 * 100, panelToken.monthItemMarginBottom);
  } else {
    typeCss = css(["padding:", ";& ", "{margin:", ";}& ", "{width:", "%;margin-bottom:", "px;}"], panelToken.containerOtherPadding, Container, panelToken.otherMargin, StyledItem, 1 / 3 * 100, panelToken.otherItemMarginBottom);
  }

  return css(["", ""], typeCss);
});
var CalendarPanel = memoForwardRef(function (props, ref) {
  var viewType = props.viewType,
      viewDate = props.viewDate,
      selected = props.selected,
      range = props.range,
      start = props.start,
      end = props.end,
      isRange = props.isRange,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      onItemClick = props.onItemClick,
      onItemHover = props.onItemHover,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      todayDate = props.todayDate,
      styles = props.styles,
      min = props.min,
      max = props.max,
      fixedStart = props.fixedStart,
      fixedEnd = props.fixedEnd,
      readOnly = props.readOnly,
      panelBlockProps = props.panelBlockProps,
      _props$isWeek = props.isWeek,
      isWeek = _props$isWeek === void 0 ? false : _props$isWeek,
      hoveredWeek = props.hoveredWeek,
      selectedWeek = props.selectedWeek,
      disableWeek = props.disableWeek,
      otherProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var pickStyles = useMemo(function () {
    if (styles) {
      var panelMonthBlock = styles.panelMonthBlock,
          panelYearBlock = styles.panelYearBlock,
          panelDecadeBlock = styles.panelDecadeBlock,
          panelItem = styles.panelItem,
          panelLabelItem = styles.panelLabelItem,
          panelLabelBlock = styles.panelLabelBlock;
      return {
        panelMonthBlock: panelMonthBlock,
        panelYearBlock: panelYearBlock,
        panelDecadeBlock: panelDecadeBlock,
        panelItem: panelItem,
        panelLabelItem: panelLabelItem,
        panelLabelBlock: panelLabelBlock
      };
    }
  }, [styles]);
  var innerStyles = useStyles('calendar', defaultStyles, pickStyles);
  var dates = useMemo(function () {
    if (viewType === 'month') {
      return getDaysOfMonth(viewDate);
    }

    if (viewType === 'year') {
      return getMonthsOfYear(viewDate);
    }

    return getDecadeYears(viewDate);
  }, [viewType, viewDate]); // 标签，只有月视图有

  var labelBlock = useMemo(function () {
    if (viewType !== 'month') {
      return;
    }

    var weekLabels = dates.slice(0, 7);
    return /*#__PURE__*/React.createElement(Container, innerStyles.panelLabelBlock, weekLabels.map(function (labelDate) {
      return /*#__PURE__*/React.createElement(StyledItemM, _extends({}, innerStyles.panelLabelItem, {
        styles: styles,
        renderMonthLabel: renderMonthLabel,
        date: labelDate,
        isLabel: true,
        viewType: "month",
        key: labelDate.unix()
      }));
    }));
  }, [dates, innerStyles.panelLabelBlock, innerStyles.panelLabelItem, renderMonthLabel, styles, viewType]);
  var blockStyleItem = innerStyles.panelMonthBlock;

  if (viewType === 'year') {
    blockStyleItem = innerStyles.panelYearBlock;
  } else if (viewType === 'decade') {
    blockStyleItem = innerStyles.panelYearBlock;
  }

  var dateItems = useMemo(function () {
    var getStatus = createGetStatus({
      start: start,
      end: end,
      range: range,
      selected: selected,
      viewType: viewType,
      isRange: isRange,
      viewDate: viewDate,
      // week
      isWeek: isWeek,
      hoveredWeek: hoveredWeek,
      selectedWeek: selectedWeek
    });
    var checkDisabled = createCheckDisabled({
      disableRange: disableRange,
      disableDate: disableDate,
      viewType: viewType,
      min: min,
      max: max,
      start: start,
      end: end,
      fixedStart: fixedStart,
      fixedEnd: fixedEnd,
      isRange: isRange,
      // week
      isWeek: isWeek,
      disableWeek: disableWeek
    });
    var rowNum = viewType === 'month' ? 7 : 3;
    var dateType = getDateType(viewType);
    var itemPropsList = dates.map(function (date, dateIndex) {
      var status = getStatus(date, dateIndex);
      var disabled = checkDisabled(date);
      var rowType = 'normal';
      var isCurrent = date.isSame(todayDate, dateType);
      var remain = dateIndex % rowNum;

      if (remain === 0) {
        rowType = 'head';
      } else if (remain === rowNum - 1) {
        rowType = 'tail';
      }

      return {
        hideItemOutside: hideItemOutside,
        renderDecadeItem: renderDecadeItem,
        renderMonthItem: renderMonthItem,
        renderYearItem: renderYearItem,
        onItemClick: onItemClick,
        onItemHover: onItemHover,
        status: status,
        disabled: disabled,
        readOnly: viewType === 'month' && readOnly,
        rowType: rowType,
        date: date,
        isCurrent: isCurrent,
        styles: styles
      };
    });

    if (isRange) {
      setOnlyStart(itemPropsList);
    }

    return itemPropsList.map(function (dateItem) {
      return /*#__PURE__*/React.createElement(StyledItemM, _extends({}, innerStyles.panelItem, dateItem, {
        viewType: viewType,
        key: dateItem.date.unix()
      }));
    });
  }, [dates, disableDate, disableRange, disableWeek, end, fixedEnd, fixedStart, hideItemOutside, hoveredWeek, innerStyles.panelItem, isRange, isWeek, max, min, onItemClick, onItemHover, range, readOnly, renderDecadeItem, renderMonthItem, renderYearItem, selected, selectedWeek, start, styles, todayDate, viewDate, viewType]);
  return /*#__PURE__*/React.createElement(Root, _extends({
    theme: theme,
    $type: viewType
  }, otherProps, {
    ref: ref
  }), labelBlock, /*#__PURE__*/React.createElement(Container, _extends({}, blockStyleItem, panelBlockProps), dateItems));
});
export default CalendarPanel;