import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo } from 'react';
import styled from 'styled-components';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import CalendarFooter from './CalendarFooter';
import CalendarHead from './CalendarHead';
import CalendarPanel from './CalendarPanel';
import useMultiSelectCalendar from './useMultiSelectCalendar';
var Root = styled.div.withConfig({
  displayName: "Root"
})(["width:", "px;background-color:", ";"], function (props) {
  return props.theme.components.Calendar.defaultWidth;
}, function (props) {
  return props.theme.colors.pattern.background.higher;
});
var defaultStyles = {
  head: '',
  panel: '',
  footer: ''
};
var MultiSelectCalendar = memoForwardRef(function (props, ref) {
  var _useMultiSelectCalend = useMultiSelectCalendar(props),
      finalUIState = _useMultiSelectCalend.finalUIState,
      finalSelectedDate = _useMultiSelectCalend.finalSelectedDate,
      finalOptions = _useMultiSelectCalend.finalOptions,
      handleItemClick = _useMultiSelectCalend.handleItemClick,
      handlePrevMonth = _useMultiSelectCalend.handlePrevMonth,
      handleNextMonth = _useMultiSelectCalend.handleNextMonth,
      handlePrevYear = _useMultiSelectCalend.handlePrevYear,
      handleNextYear = _useMultiSelectCalend.handleNextYear,
      handleSwitch = _useMultiSelectCalend.handleSwitch,
      finalDisableRange = _useMultiSelectCalend.finalDisableRange,
      finalTodayDate = _useMultiSelectCalend.finalTodayDate,
      readOnly = _useMultiSelectCalend.readOnly,
      otherProps = _useMultiSelectCalend.otherProps,
      styles = _useMultiSelectCalend.styles,
      hideItemOutside = _useMultiSelectCalend.hideItemOutside,
      renderDecadeItem = _useMultiSelectCalend.renderDecadeItem,
      renderMonthItem = _useMultiSelectCalend.renderMonthItem,
      renderYearItem = _useMultiSelectCalend.renderYearItem,
      renderMonthLabel = _useMultiSelectCalend.renderMonthLabel,
      disableDate = _useMultiSelectCalend.disableDate;

  var theme = useTheme();
  var pickStyles = useMemo(function () {
    if (styles) {
      var head = styles.head,
          panel = styles.panel,
          footer = styles.footer;
      return {
        head: head,
        panel: panel,
        footer: footer
      };
    }
  }, [styles]);
  var innerStyles = useStyles('calendar', defaultStyles, pickStyles); // CalendarFooter 本身就 memo了，单个节点不需要 memo 了

  var footerNode;

  if (finalOptions && finalOptions.length) {
    footerNode = /*#__PURE__*/React.createElement(CalendarFooter, _extends({}, innerStyles.footer, {
      styles: styles,
      options: finalOptions
    }));
  } // 本就是解构了传给 pure component，所以不需要 memo


  var panelProps = {
    styles: styles,
    todayDate: finalTodayDate,
    disableRange: finalDisableRange,
    disableDate: disableDate,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel
  };
  return /*#__PURE__*/React.createElement(Root, _extends({}, otherProps, {
    theme: theme,
    ref: ref
  }), /*#__PURE__*/React.createElement(CalendarHead, _extends({}, innerStyles.head, {
    styles: styles,
    viewType: finalUIState.viewType,
    viewDate: finalUIState.viewDate,
    onPrevMonth: handlePrevMonth,
    onNextMonth: handleNextMonth,
    onPrevYear: handlePrevYear,
    onNextYear: handleNextYear,
    onSwitch: handleSwitch
  })), /*#__PURE__*/React.createElement(CalendarPanel, _extends({}, innerStyles.panel, {
    readOnly: readOnly,
    viewType: finalUIState.viewType,
    selected: finalSelectedDate,
    viewDate: finalUIState.viewDate,
    onItemClick: handleItemClick
  }, panelProps)), footerNode);
});
export default MultiSelectCalendar;