import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React, { useMemo } from 'react';
import styled from 'styled-components';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import CalendarFooter from './CalendarFooter';
import CalendarHead from './CalendarHead';
import CalendarPanel from './CalendarPanel';
import useRangeCalendar from './useRangeCalendar';
var LeftCalendar = styled.div.withConfig({
  displayName: "LeftCalendar"
})(["width:", "px;border-right:1px solid ", ";"], function (props) {
  return props.theme.components.Calendar.defaultWidth;
}, function (props) {
  return props.theme.colors.pattern.border.normal;
});
var RightCalendar = styled.div.withConfig({
  displayName: "RightCalendar"
})(["width:", "px;"], function (props) {
  return props.theme.components.Calendar.defaultWidth;
});
var Container = styled.div.withConfig({
  displayName: "Container"
})(["display:flex;"]);
var Root = styled.div.withConfig({
  displayName: "Root"
})(["position:relative;background-color:", ";"], function (props) {
  return props.theme.colors.pattern.background.higher;
});
var defaultStyles = {
  head: '',
  panel: '',
  footer: '',
  container: '',
  leftCalendar: '',
  rightCalendar: ''
};
var RangeCalendar = memoForwardRef(function (props, ref) {
  var theme = useTheme();

  var _useRangeCalendar = useRangeCalendar(props),
      finalState = _useRangeCalendar.finalState,
      finalUIState = _useRangeCalendar.finalUIState,
      finalFixedEnd = _useRangeCalendar.finalFixedEnd,
      finalFixedStart = _useRangeCalendar.finalFixedStart,
      finalOptions = _useRangeCalendar.finalOptions,
      readOnly = _useRangeCalendar.readOnly,
      otherProps = _useRangeCalendar.otherProps,
      handleItemHover = _useRangeCalendar.handleItemHover,
      handleLeftSwitch = _useRangeCalendar.handleLeftSwitch,
      handleLeftClick = _useRangeCalendar.handleLeftClick,
      handleLeftPrevMonth = _useRangeCalendar.handleLeftPrevMonth,
      handleLeftPrevYear = _useRangeCalendar.handleLeftPrevYear,
      handleLeftNextMonth = _useRangeCalendar.handleLeftNextMonth,
      handleLeftNextYear = _useRangeCalendar.handleLeftNextYear,
      handleRightSwitch = _useRangeCalendar.handleRightSwitch,
      handleRightClick = _useRangeCalendar.handleRightClick,
      handleRightPrevMonth = _useRangeCalendar.handleRightPrevMonth,
      handleRightPrevYear = _useRangeCalendar.handleRightPrevYear,
      handleRightNextMonth = _useRangeCalendar.handleRightNextMonth,
      handleRightNextYear = _useRangeCalendar.handleRightNextYear,
      finalDisableRange = _useRangeCalendar.finalDisableRange,
      finalTodayDate = _useRangeCalendar.finalTodayDate,
      styles = _useRangeCalendar.styles,
      renderDecadeItem = _useRangeCalendar.renderDecadeItem,
      renderMonthItem = _useRangeCalendar.renderMonthItem,
      renderYearItem = _useRangeCalendar.renderYearItem,
      renderMonthLabel = _useRangeCalendar.renderMonthLabel,
      hideItemOutside = _useRangeCalendar.hideItemOutside,
      disableDate = _useRangeCalendar.disableDate,
      minRangeLength = _useRangeCalendar.minRangeLength,
      maxRangeLength = _useRangeCalendar.maxRangeLength;

  var pickStyles = useMemo(function () {
    if (styles) {
      var head = styles.head,
          panel = styles.panel,
          footer = styles.footer,
          container = styles.container,
          leftCalendar = styles.leftCalendar,
          rightCalendar = styles.rightCalendar;
      return {
        head: head,
        panel: panel,
        footer: footer,
        container: container,
        leftCalendar: leftCalendar,
        rightCalendar: rightCalendar
      };
    }
  }, [styles]);
  var innerStyles = useStyles('calendar', defaultStyles, pickStyles);

  var _finalUIState$viewTyp = _slicedToArray(finalUIState.viewType, 2),
      viewLeftType = _finalUIState$viewTyp[0],
      viewRightType = _finalUIState$viewTyp[1];

  var _finalUIState$viewDat = _slicedToArray(finalUIState.viewDate, 2),
      viewLeftDate = _finalUIState$viewDat[0],
      viewRightDate = _finalUIState$viewDat[1]; // CalendarFooter 本身就 memo了，单个节点不需要 memo 了


  var footerNode;

  if (finalOptions && finalOptions.length) {
    footerNode = /*#__PURE__*/React.createElement(CalendarFooter, _extends({}, innerStyles.footer, {
      styles: styles,
      options: finalOptions
    }));
  } // 本就是解构了传给 pure component，所以不需要 memo


  var basePanelProps = {
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    disableDate: disableDate,
    fixedStart: finalFixedStart,
    fixedEnd: finalFixedEnd,
    disableRange: finalDisableRange,
    range: finalState.range,
    start: finalState.start,
    end: finalState.end,
    todayDate: finalTodayDate,
    isRange: true,
    onItemHover: handleItemHover,
    min: minRangeLength,
    max: maxRangeLength,
    hideItemOutside: hideItemOutside
  };
  return /*#__PURE__*/React.createElement(Root, _extends({
    theme: theme
  }, otherProps, {
    ref: ref
  }), /*#__PURE__*/React.createElement(Container, innerStyles.container, /*#__PURE__*/React.createElement(LeftCalendar, _extends({
    theme: theme
  }, innerStyles.leftCalendar), /*#__PURE__*/React.createElement(CalendarHead, _extends({}, innerStyles.head, {
    styles: styles,
    viewType: viewLeftType,
    viewDate: viewLeftDate,
    onSwitch: handleLeftSwitch,
    onPrevMonth: handleLeftPrevMonth,
    onPrevYear: handleLeftPrevYear,
    onNextMonth: handleLeftNextMonth,
    onNextYear: handleLeftNextYear
  })), /*#__PURE__*/React.createElement(CalendarPanel, _extends({}, innerStyles.panel, basePanelProps, {
    readOnly: readOnly,
    styles: styles,
    viewType: viewLeftType,
    viewDate: viewLeftDate,
    onItemClick: handleLeftClick
  }))), /*#__PURE__*/React.createElement(RightCalendar, _extends({
    theme: theme
  }, innerStyles.rightCalendar), /*#__PURE__*/React.createElement(CalendarHead, _extends({}, innerStyles.head, {
    styles: styles,
    viewType: viewRightType,
    viewDate: viewRightDate,
    onSwitch: handleRightSwitch,
    onPrevMonth: handleRightPrevMonth,
    onPrevYear: handleRightPrevYear,
    onNextMonth: handleRightNextMonth,
    onNextYear: handleRightNextYear
  })), /*#__PURE__*/React.createElement(CalendarPanel, _extends({}, innerStyles.panel, basePanelProps, {
    readOnly: readOnly,
    styles: styles,
    viewType: viewRightType,
    viewDate: viewRightDate,
    onItemClick: handleRightClick
  })))), footerNode);
});
export default RangeCalendar;