import { getDateType, inRange } from './utils';
export function dateInMinRange(currentDate, compareDate, dateType, min) {
  if (currentDate.isSame(compareDate, dateType)) {
    return true;
  }

  if (currentDate.isAfter(compareDate, dateType) && currentDate.subtract(min, dateType).isBefore(compareDate, dateType)) {
    return true;
  }

  if (currentDate.isBefore(compareDate, dateType) && currentDate.add(min, dateType).isAfter(compareDate, dateType)) {
    return true;
  }

  return false;
}
export function dateOutMaxRange(currentDate, compareDate, dateType, max) {
  if (currentDate.isAfter(compareDate, dateType) && currentDate.subtract(max, dateType).isAfter(compareDate, dateType)) {
    return true;
  }

  if (currentDate.isBefore(compareDate, dateType) && currentDate.add(max, dateType).isBefore(compareDate, dateType)) {
    return true;
  }

  return false;
}

/**
 * 检查当前日期是否是禁用的
 * @param args
 */
export function createCheckDisabled(args) {
  var disableRange = args.disableRange,
      disableDate = args.disableDate,
      viewType = args.viewType,
      min = args.min,
      max = args.max,
      start = args.start,
      end = args.end,
      fixedStart = args.fixedStart,
      fixedEnd = args.fixedEnd,
      isRange = args.isRange,
      isWeek = args.isWeek,
      disableWeek = args.disableWeek;
  var innerDisableRange;

  if (disableRange && disableRange.length > 1) {
    innerDisableRange = disableRange;
  }

  var dateType = getDateType(viewType);

  var normalCheckDisabled = function normalCheckDisabled(currentDate) {
    if (innerDisableRange && inRange(currentDate, innerDisableRange)) {
      return true;
    }

    if (disableDate && disableDate(currentDate, dateType)) {
      return true;
    }

    return false;
  };

  var rangeCheckDisabled = function rangeCheckDisabled(currentDate) {
    if (normalCheckDisabled(currentDate)) {
      return true;
    }

    if (fixedStart && currentDate.isBefore(fixedStart, dateType)) {
      return true;
    }

    if (fixedEnd && currentDate.isAfter(fixedEnd, dateType)) {
      return true;
    }

    var compareDate = start || end;
    var notBothStartEnd = !(start && end);

    if (notBothStartEnd && compareDate && min && min >= 1 && dateInMinRange(currentDate, compareDate, dateType, min)) {
      return true;
    }

    if (notBothStartEnd && compareDate && max && dateOutMaxRange(currentDate, compareDate, dateType, max)) {
      return true;
    }

    return false;
  };

  var weekCheckDisabled = function weekCheckDisabled(currentDate) {
    if (disableWeek && disableWeek([currentDate.startOf('w'), currentDate.endOf('w')])) {
      return true;
    }

    return false;
  };

  if (isRange) {
    return rangeCheckDisabled;
  }

  if (isWeek) {
    return weekCheckDisabled;
  }

  return normalCheckDisabled;
}
export function createGetStatus(args) {
  var start = args.start,
      end = args.end,
      _args$range = args.range,
      range = _args$range === void 0 ? [] : _args$range,
      selected = args.selected,
      viewType = args.viewType,
      isRange = args.isRange,
      viewDate = args.viewDate,
      isWeek = args.isWeek,
      _args$hoveredWeek = args.hoveredWeek,
      hoveredWeek = _args$hoveredWeek === void 0 ? [] : _args$hoveredWeek,
      _args$selectedWeek = args.selectedWeek,
      selectedWeek = _args$selectedWeek === void 0 ? [] : _args$selectedWeek;
  var dateType = getDateType(viewType);
  var firstDay = viewDate.startOf('month');
  var lastDay = viewDate.endOf('month');

  var normalGetStatus = function normalGetStatus(currentDate) {
    var dateIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var isOutsideWhenInMonthPanel = viewType === 'month' && (currentDate.isBefore(firstDay) || currentDate.isAfter(lastDay));
    var isOutsideWhenInDecadePanel = viewType === 'decade' && (dateIndex === 0 || dateIndex === 11);

    if (isOutsideWhenInMonthPanel || isOutsideWhenInDecadePanel) {
      return 'outside';
    } // 处理选中的逻辑


    if (selected) {
      if (Array.isArray(selected)) {
        return selected.some(function (d) {
          return currentDate.isSame(d, dateType);
        }) ? 'selected' : 'normal';
      } else if (currentDate.isSame(selected, dateType)) {
        return 'selected';
      }
    }

    return 'normal';
  };

  var rangeGetStatus = function rangeGetStatus(currentDate) {
    var dateIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

    if (normalGetStatus(currentDate, dateIndex) === 'outside') {
      return 'outside';
    }

    var innerStart = start || range[0];
    var innerEnd = end || range[1];

    if (innerStart && currentDate.isSame(innerStart, dateType)) {
      return 'range-start';
    }

    if (innerEnd && currentDate.isSame(innerEnd, dateType)) {
      return 'range-end';
    }

    if (range.length === 2 && inRange(currentDate, range)) {
      return 'range';
    }

    return 'normal';
  };

  var weekGetStatus = function weekGetStatus(currentDate) {
    var dateIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    var isHovered = hoveredWeek.length === 2 && inRange(currentDate, hoveredWeek);

    if (selectedWeek.length === 2 && inRange(currentDate, selectedWeek)) {
      return isHovered ? 'range-hovered' : 'range';
    }

    if (isHovered) {
      return 'hovered';
    }

    if (normalGetStatus(currentDate, dateIndex) === 'outside') {
      return 'outside';
    }

    return 'normal';
  };

  if (isRange) {
    return rangeGetStatus;
  }

  if (isWeek) {
    return weekGetStatus;
  }

  return normalGetStatus;
} // 当区间只有 start 的时候，需要特殊处理样式

export function setOnlyStart(dates) {
  var startIndex = -1;
  var onlyOneStart = true;
  dates.forEach(function (item, itemIndex) {
    if (item.status === 'range-start') {
      startIndex = itemIndex;
    } else if (item.status === 'range' || item.status === 'range-end') {
      onlyOneStart = false;
    }
  });

  if (onlyOneStart && startIndex !== -1) {
    dates[startIndex].status = 'selected';
  }

  return startIndex;
}