import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["selectType", "allowClear", "selectedDate", "todayDate", "viewType", "viewDate", "defaultDate", "disableRange", "disableDate", "options", "onUIChange", "onChange", "onSelect", "styles", "defaultViewDate", "readOnly", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel"];
import dayjs from 'dayjs';
import { useEffect, useMemo, useRef, useState } from 'react';
import { useLocale } from '../Locale';
import useCalendarBase from './useCalendarBase';
import { formatCalendarOptions, moveOne, transformType } from './utils';
export default function useCalendar(props) {
  var baseData = useCalendarBase(props);

  var _props$selectType = props.selectType,
      selectType = _props$selectType === void 0 ? 'date' : _props$selectType,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      selectedDate = props.selectedDate,
      todayDate = props.todayDate,
      viewType = props.viewType,
      viewDate = props.viewDate,
      defaultDate = props.defaultDate,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      onUIChange = props.onUIChange,
      onChange = props.onChange,
      onSelect = props.onSelect,
      styles = props.styles,
      defaultViewDate = props.defaultViewDate,
      readOnly = props.readOnly,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      otherProps = _objectWithoutProperties(props, _excluded);

  var isControlled = ('selectedDate' in props);
  var defaultSelectedDate;

  if (selectedDate) {
    defaultSelectedDate = dayjs(selectedDate);
  } else if (defaultDate) {
    defaultSelectedDate = dayjs(defaultDate);
  }

  var selectTypeRef = useRef(selectType);
  var defaultTransformViewType = transformType(selectType);
  var viewDateFromProps = useMemo(function () {
    var inputViewDate = viewDate || selectedDate || defaultDate || defaultViewDate;

    if (inputViewDate) {
      return dayjs(inputViewDate);
    }

    return baseData.finalTodayDate;
  }, [baseData.finalTodayDate, defaultDate, defaultViewDate, selectedDate, viewDate]);
  var uiStateFromProps = {
    viewDate: viewDateFromProps,
    viewType: viewType || defaultTransformViewType
  };

  var _useState = useState(uiStateFromProps),
      _useState2 = _slicedToArray(_useState, 2),
      uiState = _useState2[0],
      setUIState = _useState2[1];

  var _useState3 = useState(defaultSelectedDate),
      _useState4 = _slicedToArray(_useState3, 2),
      stateSelectedDate = _useState4[0],
      setSelectedDate = _useState4[1];

  var finalUIState = _extends({}, uiState);

  if ('viewDate' in props) {
    finalUIState.viewDate = uiStateFromProps.viewDate;
  }

  if ('viewType' in props) {
    finalUIState.viewType = uiStateFromProps.viewType;
  }

  if (selectTypeRef.current !== selectType) {
    selectTypeRef.current = selectType;
    setUIState(uiStateFromProps);
    setSelectedDate(undefined);
  }

  var updateUI = function updateUI(inputState) {
    if (!inputState.viewDate && !inputState.viewType) {
      return;
    }

    var newState = _extends({}, finalUIState, inputState);

    setUIState(newState);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var updateUIBySelectedDate = function updateUIBySelectedDate(date) {
    var backToDefaultViewType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    var newUIState = {
      viewDate: date,
      viewType: finalUIState.viewType
    };

    if (backToDefaultViewType && finalUIState.viewType !== defaultTransformViewType) {
      newUIState.viewType = defaultTransformViewType;
    }

    updateUI(newUIState);
  };

  var updateSelectedDate = function updateSelectedDate(date) {
    var syncView = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    var backToDefaultViewType = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    setSelectedDate(date);

    if (onChange) {
      onChange(date);
    }

    if (onSelect && date) {
      onSelect(date);
    }

    if (syncView && date) {
      updateUIBySelectedDate(date, backToDefaultViewType);
    }
  };

  var handleItemClick = _useEventCallback(function (date, status) {
    if (finalUIState.viewType === 'year' && selectType !== 'month') {
      updateUI({
        viewType: 'month',
        viewDate: finalUIState.viewDate.set('month', date.get('month'))
      });
      return;
    }

    if (finalUIState.viewType === 'decade' && selectType !== 'year') {
      updateUI({
        viewType: selectType === 'date' ? 'month' : 'year',
        viewDate: finalUIState.viewDate.set('year', date.get('year'))
      });
      return;
    }

    if (stateSelectedDate && stateSelectedDate.isSame(date, selectType) && allowClear) {
      updateSelectedDate(undefined);
    } else {
      updateSelectedDate(date, status === 'outside');
    }
  });

  var handleOptionClick = _useEventCallback(function (date) {
    updateSelectedDate(date, true, true);
  });

  var handleMove = function handleMove(step, type) {
    return function () {
      updateUI({
        viewDate: moveOne(finalUIState.viewDate, type, step)
      });
    };
  };

  var handlePrevMonth = _useEventCallback(handleMove(-1, 'month'));

  var handleNextMonth = _useEventCallback(handleMove(1, 'month'));

  var yearBtnType = finalUIState.viewType === 'decade' ? 'decade' : 'year';

  var handlePrevYear = _useEventCallback(handleMove(-1, yearBtnType));

  var handleNextYear = _useEventCallback(handleMove(1, yearBtnType));

  var handleSwitch = _useEventCallback(function (type) {
    updateUI({
      viewType: type
    });
  });

  var locale = useLocale();
  var finalOptions = useMemo(function () {
    return formatCalendarOptions(options, handleOptionClick, function (opt) {
      if (typeof opt.date === 'function') {
        _devWarning('[Calendar]: option.date() return invalid date.');
      } else {
        _devWarning('[Calendar]: options passed invalid date.');
      }
    });
  }, [handleOptionClick, options]); // 当国际化变了，得更新内存中 dayjs 对象的国际化

  var updateLocale = function updateLocale() {
    var localeKey = locale.dayjsLocaleKey;
    updateUI({
      viewDate: finalUIState.viewDate.locale(localeKey)
    });

    if (stateSelectedDate) {
      updateSelectedDate(stateSelectedDate.locale(localeKey));
    }
  };

  useEffect(function () {
    var localeKey = locale.dayjsLocaleKey; // 任一变了就可以了

    var localeChange = localeKey !== finalUIState.viewDate.locale();

    if (!localeChange) {
      return;
    }

    updateLocale();
  });
  useEffect(function () {
    if (isControlled) {
      setSelectedDate(selectedDate ? dayjs(selectedDate) : dayjs(''));
    }
  }, [isControlled, selectedDate]);
  return _extends({}, baseData, {
    otherProps: otherProps,
    readOnly: readOnly,
    handleItemClick: handleItemClick,
    handlePrevMonth: handlePrevMonth,
    handleNextMonth: handleNextMonth,
    handlePrevYear: handlePrevYear,
    handleNextYear: handleNextYear,
    handleSwitch: handleSwitch,
    finalUIState: finalUIState,
    finalSelectedDate: stateSelectedDate,
    finalOptions: finalOptions,
    updateUI: updateUI,
    updateSelectedDate: updateSelectedDate,
    updateUIBySelectedDate: updateUIBySelectedDate,
    // 透传回去的属性
    styles: styles,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    disableDate: disableDate
  });
}