import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import dayjs from 'dayjs';
import minMax from 'dayjs/plugin/minMax';
dayjs.extend(minMax);
/**
 * 将选择类型转换为展示类型
 * @param selectType 选择类型
 */

export function transformType(selectType) {
  if (selectType === 'month') {
    return 'year';
  }

  if (selectType === 'year') {
    return 'decade';
  }

  return 'month';
}
export function getDateType(viewType) {
  if (viewType === 'month') {
    return 'day';
  }

  if (viewType === 'year') {
    return 'month';
  }

  return 'year';
}
export function getSelectDateType(selectType) {
  if (selectType === 'date') {
    return 'day';
  }

  return selectType;
}
export function getDaysOfMonth(date) {
  var firstDay = date.startOf('month').startOf('week');
  var i = 0;
  var days = [];
  days.push(firstDay);

  while (i < 41) {
    i++;
    days.push(firstDay.add(i, 'day'));
  }

  return days;
}
export function getMonthsOfYear(date) {
  var num = date.get('y');
  var i = 0;
  var months = [];

  while (i < 12) {
    months.push(dayjs("".concat(num, "-").concat(i + 1, "-1")));
    i++;
  }

  return months;
}
/**
 * 获取对应时间的十年，加上首位的两年
 * @param date 日期
 */

export function getDecadeYears(date) {
  var num = date.get('y');
  var firstYear = num - num % 10 - 1;
  var years = [];
  var i = 0;

  while (i < 12) {
    years.push(dayjs("".concat(firstYear + i, "-01-01")));
    i++;
  }

  return years;
}
/**
 * 获取对应时间的十年，加上首位的两年
 * @param date 日期
 */

export function getDecade(date) {
  var num = date.get('y');
  var firstYear = num - num % 10;
  return [dayjs("".concat(firstYear, "-01-01")), dayjs("".concat(firstYear + 10, "-01-01"))];
}
export function moveOne(date, type) {
  var num = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;

  if (type !== 'decade') {
    return date.add(num, type);
  } else {
    return date.add(10 * num, 'year');
  }
}
export function isSame(dateA, dateB, type) {
  if (type !== 'decade') {
    return dateA.isSame(dateB, type);
  } else {
    var _getDecade = getDecade(dateA),
        _getDecade2 = _slicedToArray(_getDecade, 1),
        decadeA = _getDecade2[0];

    var _getDecade3 = getDecade(dateB),
        _getDecade4 = _slicedToArray(_getDecade3, 1),
        decadeB = _getDecade4[0];

    return decadeA.isSame(decadeB, 'year');
  }
}
/**
 * 把用户输入转换成一个有序日期区间
 * @param range 日期区间
 */

export function getRangeInOrder(range) {
  if (!range || range.length < 2) {
    return;
  }

  var _range = _slicedToArray(range, 2),
      start = _range[0],
      end = _range[1];

  start = dayjs(start);
  end = dayjs(end);

  if (start.isValid() && end.isValid()) {
    var left = dayjs.min(start, end);
    var right = dayjs.max(start, end);
    return [left, right];
  }
}
export function inRange(date, range) {
  var _range2 = _slicedToArray(range, 2),
      left = _range2[0],
      right = _range2[1];

  return !date.isBefore(left) && !date.isAfter(right);
}
export function getRangeViewType(selectType, viewType) {
  if (viewType && viewType.length > 1) {
    return [viewType[0], viewType[1]];
  }

  var vType = transformType(selectType);
  return [vType, vType];
}
export function getRangeViewDate(viewType, viewDate) {
  var num = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 1;

  if (viewDate && viewDate.length > 1) {
    var _ref = getRangeInOrder(viewDate),
        _ref2 = _slicedToArray(_ref, 2),
        left = _ref2[0],
        right = _ref2[1];

    var newLeft = dayjs(left);
    var newRight = dayjs(right);
    var leftRightIsSame = isSame(newLeft, newRight, viewType[0]);

    if (leftRightIsSame && num === 1) {
      newRight = moveOne(newLeft, viewType[0]);
    } else if (leftRightIsSame && num === -1) {
      newLeft = moveOne(newLeft, viewType[0], -1);
    }

    return [newLeft, newRight];
  }

  var viewLeftDate = dayjs();
  return [viewLeftDate, moveOne(viewLeftDate, viewType[0])];
}
var disabledStatusMap = {
  selected: 'selected-disabled',
  normal: 'disabled',
  range: 'range-disabled',
  'range-hovered': 'range-hovered',
  'range-end': 'range-end-disabled',
  'range-start': 'range-start-disabled',
  outside: 'outside-disabled',
  hovered: 'hovered'
};
export function transformItemStatus(status, disabled) {
  if (disabled) {
    return disabledStatusMap[status];
  }

  return status;
}
export function formatStartEnd(date) {
  var isEnd = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var selectType = arguments.length > 2 ? arguments[2] : undefined;
  var dateType = 'd';

  if (selectType === 'month') {
    dateType = 'M';
  }

  if (selectType === 'year') {
    dateType = 'y';
  }

  return isEnd ? date.endOf(dateType) : date.startOf(dateType);
}
export function formatCalendarOptions(options, handleItemClick, onError) {
  if (!options || options.length < 1) {
    return;
  }

  var innerOpts = [];
  options.forEach(function (opt) {
    var dateFn;

    if (typeof opt.date === 'function') {
      var innerDate = opt.date();

      if (!innerDate.isValid()) {
        onError && onError(opt);
        return;
      }

      dateFn = opt.date;
    } else if (opt.date) {
      var _innerDate = dayjs(opt.date);

      if (!_innerDate.isValid()) {
        onError && onError(opt);
        return;
      }

      dateFn = function dateFn() {
        return _innerDate;
      };
    } else {
      dateFn = function dateFn() {
        return dayjs();
      };
    }

    innerOpts.push({
      label: opt.label,
      disabled: opt.disabled,
      onClick: function onClick() {
        handleItemClick && handleItemClick(dateFn());
      }
    });
  });
  return innerOpts;
}