import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import { useAfterEffect as _useAfterEffect2 } from "@muya-ui/utils/es/hooks/useAfterEffect";
import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["imgs", "index", "defaultIndex", "autoplay", "arrow", "arrowEnabled", "arrowSize", "indicator", "indicatorTrigger", "duration", "onChange", "animation", "lazy", "styles", "indicatorStyles", "enableDiffChildren", "children", "onMouseEnter", "onMouseLeave"];
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import styled from 'styled-components';
import Animation from '../Animation';
import Img, { ImgContainer } from '../Img';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import { withThemeForStyled } from '../utils/withTheme';
import PagerButton from './PagerButton';
import { carouselCss, StyledIndexIndicator, StyledPagerNext, StyledPagerPrev } from './styled';
import useMoveActive from './useMoveActive';
import { transformImgs } from './utils';
var StyledImg = styled(Img).withConfig({
  displayName: "StyledImg"
})(["width:100%;height:100%;position:absolute;top:0;left:0;"]);
var defaultStyles = {
  pagerPrev: '',
  pagerPrevBtn: '',
  pagerNext: '',
  pagerNextBtn: '',
  indicator: '',
  swipe: '',
  img: ''
};
var AnimCarouselPure = memoForwardRef(function (props, ref) {
  var imgs = props.imgs,
      index = props.index,
      _props$defaultIndex = props.defaultIndex,
      defaultIndex = _props$defaultIndex === void 0 ? 0 : _props$defaultIndex,
      _props$autoplay = props.autoplay,
      autoplay = _props$autoplay === void 0 ? 0 : _props$autoplay,
      _props$arrow = props.arrow,
      arrow = _props$arrow === void 0 ? 'always' : _props$arrow,
      _props$arrowEnabled = props.arrowEnabled,
      arrowEnabled = _props$arrowEnabled === void 0 ? 'always' : _props$arrowEnabled,
      _props$arrowSize = props.arrowSize,
      arrowSize = _props$arrowSize === void 0 ? 'm' : _props$arrowSize,
      _props$indicator = props.indicator,
      indicator = _props$indicator === void 0 ? 'center' : _props$indicator,
      indicatorTrigger = props.indicatorTrigger,
      _props$duration = props.duration,
      duration = _props$duration === void 0 ? 300 : _props$duration,
      onChange = props.onChange,
      _props$animation = props.animation,
      animation = _props$animation === void 0 ? 'fade' : _props$animation,
      lazy = props.lazy,
      styles = props.styles,
      indicatorStyles = props.indicatorStyles,
      enableDiffChildren = props.enableDiffChildren,
      children = props.children,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      otherProps = _objectWithoutProperties(props, _excluded);

  var _useMemo = useMemo(function () {
    return transformImgs(imgs);
  }, [imgs]),
      innerImgs = _useMemo.imgs,
      isItems = _useMemo.isItems,
      imgItems = _useMemo.imgItems;

  var innerStyles = useStyles('carousel', defaultStyles, styles);
  var isControlled = ('index' in props);

  var _useState = useState(defaultIndex),
      _useState2 = _slicedToArray(_useState, 2),
      indexState = _useState2[0],
      setIndexState = _useState2[1];

  var finalIndex = isControlled ? index || 0 : indexState;
  var canSwipe = innerImgs.length > 1;

  var _useMoveActive = useMoveActive(arrowEnabled === 'always'),
      shouldActive = _useMoveActive.shouldActive,
      setMoveActive = _useMoveActive.setMoveActive;

  var updateIndex = shouldActive(function (i) {
    setIndexState(i);
    onChange && onChange(innerImgs[i], i);
  });
  var onIndexChange = useCallback(function (i) {
    updateIndex(i);
  }, [updateIndex]);
  var onPrev = useCallback(function () {
    if (finalIndex === 0) {
      updateIndex(innerImgs.length - 1);
    } else {
      updateIndex(finalIndex - 1);
    }
  }, [finalIndex, innerImgs.length, updateIndex]);
  var onNext = useCallback(function () {
    if (finalIndex === innerImgs.length - 1) {
      updateIndex(0);
    } else {
      updateIndex(finalIndex + 1);
    }
  }, [finalIndex, innerImgs.length, updateIndex]);
  var pagerNode = useMemo(function () {
    return arrow !== 'none' && canSwipe ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(StyledPagerPrev, innerStyles.pagerPrev, /*#__PURE__*/React.createElement(PagerButton, _extends({}, innerStyles.pagerPrevBtn, {
      arrow: "left",
      size: arrowSize,
      onClick: onPrev
    }))), /*#__PURE__*/React.createElement(StyledPagerNext, innerStyles.pagerNext, /*#__PURE__*/React.createElement(PagerButton, _extends({}, innerStyles.pagerNextBtn, {
      arrow: "right",
      size: arrowSize,
      onClick: onNext
    })))) : null;
  }, [arrow, arrowSize, canSwipe, innerStyles.pagerNext, innerStyles.pagerNextBtn, innerStyles.pagerPrev, innerStyles.pagerPrevBtn, onNext, onPrev]);
  var indicatorNode = useMemo(function () {
    return indicator !== 'none' && canSwipe ? /*#__PURE__*/React.createElement(StyledIndexIndicator, _extends({}, innerStyles.indicator, {
      styles: indicatorStyles,
      trigger: indicatorTrigger,
      index: finalIndex,
      num: innerImgs.length,
      onChange: onIndexChange
    })) : null;
  }, [canSwipe, finalIndex, indicator, indicatorStyles, indicatorTrigger, innerImgs.length, innerStyles.indicator, onIndexChange]);

  var _useAfterEffect = _useAfterEffect2(onNext, autoplay * 1000),
      onActive = _useAfterEffect.onActive,
      onDisable = _useAfterEffect.onDisable;

  useEffect(function () {
    if (finalIndex > innerImgs.length) {
      updateIndex(innerImgs.length - 1);
    }
  });
  var checkRegion = useRef({
    top: -1,
    bottom: 1,
    left: -1,
    right: 1
  });

  var mouseEnterHandler = _useEventCallback(function (e) {
    onDisable();
    onMouseEnter && onMouseEnter(e);
  }, []);

  var mouseLeaveHandler = _useEventCallback(function (e) {
    onActive();
    onMouseLeave && onMouseLeave(e);
  }, []);

  var Anim;

  if (animation === 'grow') {
    Anim = Animation.Grow;
  } else {
    Anim = Animation.Fade;
  }

  var settings = useMemo(function () {
    return {
      checkRegion: checkRegion.current
    };
  }, []);
  return /*#__PURE__*/React.createElement(ImgContainer, _extends({
    settings: settings
  }, otherProps, {
    ref: ref,
    onMouseEnter: mouseEnterHandler,
    onMouseLeave: mouseLeaveHandler
  }), innerImgs.map(function (imgSrc, i) {
    var selected = finalIndex === i;
    var imgNode;

    if (isItems) {
      var currentItem = imgItems[finalIndex];
      imgNode = /*#__PURE__*/React.createElement(StyledImg, _extends({}, innerStyles.img, {
        src: imgSrc,
        lazy: "off"
      }, currentItem.imgProps), currentItem.children);
    } else {
      imgNode = /*#__PURE__*/React.createElement(StyledImg, _extends({}, innerStyles.img, {
        src: imgSrc,
        lazy: "off"
      }));
    }

    return /*#__PURE__*/React.createElement(Anim, {
      key: i,
      timeout: duration,
      "in": selected,
      onEntered: function onEntered() {
        return setMoveActive(true);
      }
    }, imgNode);
  }), pagerNode, children, indicatorNode);
});
var AnimCarousel = styled(AnimCarouselPure).withConfig({
  displayName: "AnimCarousel"
})(["", ""], carouselCss);
export default withThemeForStyled(AnimCarousel);