import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo } from 'react';
import styled from 'styled-components';
import Img, { ImgContainer } from '../Img';
import { LoopSwipe } from '../Swipe';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import { withThemeForStyled } from '../utils/withTheme';
import PagerButton from './PagerButton';
import { carouselCss, StyledIndexIndicator, StyledPagerNext, StyledPagerPrev } from './styled';
import useSwipeCarousel from './useSwipeCarousel';
var StyledImg = styled(Img).withConfig({
  displayName: "StyledImg"
})(["height:100%;transform:translate3d(0,0,0);"]);
var StyledSwipe = styled(LoopSwipe).withConfig({
  displayName: "StyledSwipe"
})(["height:100%;"]);
var defaultStyles = {
  pagerPrev: '',
  pagerPrevBtn: '',
  pagerNext: '',
  pagerNextBtn: '',
  indicator: '',
  swipe: '',
  img: ''
};
var imgContainerSettings = {
  checkRegion: {
    top: -1,
    bottom: 1,
    left: -1,
    right: 1
  }
};
var SwipeCarouselPure = memoForwardRef(function (props, ref) {
  var _useSwipeCarousel = useSwipeCarousel(props),
      canSwipe = _useSwipeCarousel.canSwipe,
      isItems = _useSwipeCarousel.isItems,
      imgItems = _useSwipeCarousel.imgItems,
      innerImgs = _useSwipeCarousel.innerImgs,
      currentIndex = _useSwipeCarousel.currentIndex,
      imgLength = _useSwipeCarousel.imgLength,
      handlePrev = _useSwipeCarousel.handlePrev,
      handleNext = _useSwipeCarousel.handleNext,
      handleGoTo = _useSwipeCarousel.handleGoTo,
      handleMouseEnter = _useSwipeCarousel.handleMouseEnter,
      handleMouseLeave = _useSwipeCarousel.handleMouseLeave,
      handleTransitionEnd = _useSwipeCarousel.handleTransitionEnd,
      arrowSize = _useSwipeCarousel.arrowSize,
      arrow = _useSwipeCarousel.arrow,
      enableDiffChildren = _useSwipeCarousel.enableDiffChildren,
      lazy = _useSwipeCarousel.lazy,
      duration = _useSwipeCarousel.duration,
      indicatorTrigger = _useSwipeCarousel.indicatorTrigger,
      indicator = _useSwipeCarousel.indicator,
      children = _useSwipeCarousel.children,
      otherProps = _useSwipeCarousel.otherProps,
      styles = _useSwipeCarousel.styles,
      indicatorStyles = _useSwipeCarousel.indicatorStyles;

  var innerStyles = useStyles('carousel', defaultStyles, styles);
  var pagerNode = useMemo(function () {
    return arrow !== 'none' && canSwipe ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(StyledPagerPrev, innerStyles.pagerPrev, /*#__PURE__*/React.createElement(PagerButton, _extends({}, innerStyles.pagerPrevBtn, {
      arrow: "left",
      size: arrowSize,
      onClick: handlePrev
    }))), /*#__PURE__*/React.createElement(StyledPagerNext, innerStyles.pagerNext, /*#__PURE__*/React.createElement(PagerButton, _extends({}, innerStyles.pagerNextBtn, {
      arrow: "right",
      size: arrowSize,
      onClick: handleNext
    })))) : null;
  }, [arrow, arrowSize, canSwipe, handleNext, handlePrev, innerStyles.pagerNext, innerStyles.pagerNextBtn, innerStyles.pagerPrev, innerStyles.pagerPrevBtn]);
  var indicatorNode = useMemo(function () {
    return indicator !== 'none' && canSwipe ? /*#__PURE__*/React.createElement(StyledIndexIndicator, _extends({}, innerStyles.indicator, {
      styles: indicatorStyles,
      trigger: indicatorTrigger,
      index: currentIndex,
      num: imgLength,
      onChange: handleGoTo
    })) : null;
  }, [canSwipe, currentIndex, handleGoTo, imgLength, indicator, indicatorStyles, indicatorTrigger, innerStyles.indicator]);
  var imgNodes = useMemo(function () {
    if (innerImgs.length <= 0) {
      return;
    }

    return innerImgs.map(function (imgSrc, index) {
      if (isItems) {
        var currentItem = imgItems[index];
        return /*#__PURE__*/React.createElement(StyledImg, _extends({}, innerStyles.img, {
          key: index,
          src: imgSrc,
          lazy: lazy
        }, currentItem.imgProps), currentItem.children);
      }

      return /*#__PURE__*/React.createElement(StyledImg, _extends({}, innerStyles.img, {
        key: index,
        src: imgSrc,
        lazy: lazy
      }));
    });
  }, [imgItems, innerImgs, innerStyles.img, isItems, lazy]);
  var swipeNode = useMemo(function () {
    if (innerImgs.length > 0) {
      return /*#__PURE__*/React.createElement(StyledSwipe, _extends({}, innerStyles.swipe, {
        equalNum: 1,
        duration: duration,
        stepIndex: currentIndex,
        onTransitionEnd: handleTransitionEnd,
        enableDiffChildren: enableDiffChildren
      }), imgNodes);
    }
  }, [currentIndex, duration, enableDiffChildren, handleTransitionEnd, imgNodes, innerImgs.length, innerStyles.swipe]);
  return /*#__PURE__*/React.createElement(ImgContainer, _extends({
    settings: imgContainerSettings,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave
  }, otherProps, {
    ref: ref
  }), pagerNode, swipeNode, children, indicatorNode);
});
var SwipeCarousel = styled(SwipeCarouselPure).withConfig({
  displayName: "SwipeCarousel"
})(["", ""], carouselCss);
export default withThemeForStyled(SwipeCarousel);