import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _get from "lodash/get";
import _isEmpty from "lodash/isEmpty";
import React, { useMemo, useRef } from 'react';
import BaseMenu from '../BaseMenu';
import useLocale from '../Locale/useLocale';
import Result from '../Result';
import Trigger from '../Trigger';
import useStyles from '../utils/useStyles';
import { useTheme } from '../utils/useTheme';
import CascaderInput from './CascaderInput';
import CascaderPanel from './CascaderPanel';
import PureCascaderPanel from './PureCascaderPanel';
import { StyledNotFoundContent } from './styled';
import useCascader from './useCascader';
var defaultStyles = {
  inputWrapper: '',
  resultMenu: '',
  resultMenuItem: ''
};
var defaultMaxItemCountPerPage = 6.5;
var Cascader = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var locale = useLocale();
  var theme = useTheme();
  var styles = props.styles,
      inputRef = props.inputRef,
      className = props.className,
      style = props.style,
      children = props.children,
      showSearch = props.showSearch,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      notFoundContent = props.notFoundContent,
      _props$maxItemCountPe = props.maxItemCountPerPage,
      maxItemCountPerPage = _props$maxItemCountPe === void 0 ? defaultMaxItemCountPerPage : _props$maxItemCountPe;
  var pickStyles = useMemo(function () {
    if (styles) {
      var inputWrapper = styles.inputWrapper,
          resultMenu = styles.resultMenu,
          resultMenuItem = styles.resultMenuItem;
      return {
        inputWrapper: inputWrapper,
        resultMenu: resultMenu,
        resultMenuItem: resultMenuItem
      };
    }
  }, [styles]);
  var innerStyles = useStyles('cascader', defaultStyles, pickStyles);
  var nodeRef = useRef(null);
  var innerInputRef = useRef(null);

  var handleRef = _useForkRef(nodeRef, ref);

  var handleInputRef = _useForkRef(inputRef, innerInputRef);

  var menuRef = useRef(null);
  var firstSelectedItemRef = useRef(null);

  var _useCascader = useCascader(props, nodeRef, innerInputRef, menuRef, firstSelectedItemRef),
      inputProps = _useCascader.inputProps,
      panelProps = _useCascader.panelProps,
      triggerProps = _useCascader.triggerProps,
      searchResultProps = _useCascader.searchResultProps;

  var popupNode = useMemo(function () {
    var searchValue = searchResultProps.searchValue,
        activeKey = searchResultProps.activeKey,
        fieldNames = searchResultProps.fieldNames,
        filterFieldName = searchResultProps.filterFieldName,
        firstSelectedOption = searchResultProps.firstSelectedOption,
        menuWidth = searchResultProps.menuWidth,
        filterOptions = searchResultProps.filterOptions,
        optionSelectedIndex = searchResultProps.optionSelectedIndex,
        handleOptionClick = searchResultProps.handleOptionClick,
        handleOptionMouseEnter = searchResultProps.handleOptionMouseEnter;
    var renderOption = panelProps.renderOption;

    if (showSearch && searchValue) {
      var menuContent;

      var noResult = _isEmpty(filterOptions);

      if (noResult) {
        if (notFoundContent) {
          menuContent = notFoundContent;
        } else {
          menuContent = /*#__PURE__*/React.createElement(StyledNotFoundContent, {
            theme: theme
          }, /*#__PURE__*/React.createElement(Result, {
            subTitle: locale['Cascader.noResultText'],
            type: "emptySmall"
          }));
        }
      } else {
        menuContent = filterOptions.map(function (option) {
          var refProps = {};

          if (firstSelectedOption && firstSelectedOption === option) {
            refProps.ref = firstSelectedItemRef;
          }

          var label = option[fieldNames.label] || option[filterFieldName];
          return /*#__PURE__*/React.createElement(BaseMenu.Item, _extends({}, refProps, {
            key: option[filterFieldName].toString(),
            disabled: option.disabled,
            selected: optionSelectedIndex(option) !== -1,
            active: activeKey === option[filterFieldName].toString(),
            onClick: function onClick(e) {
              return handleOptionClick(option, e);
            },
            onMouseEnter: handleOptionMouseEnter
          }, innerStyles.resultMenuItem), renderOption ? renderOption(option) : label);
        });
      }

      return /*#__PURE__*/React.createElement(BaseMenu.Menu, _extends({
        ref: menuRef,
        maxItemCountPerPage: maxItemCountPerPage,
        width: _get(showSearch, 'matchInputWidth', true) || noResult ? menuWidth : undefined,
        size: size,
        tabIndex: -1
      }, innerStyles.resultMenu), menuContent);
    }

    return /*#__PURE__*/React.createElement(PureCascaderPanel, _extends({
      styles: styles
    }, panelProps));
  }, [innerStyles.resultMenu, innerStyles.resultMenuItem, locale, maxItemCountPerPage, notFoundContent, panelProps, searchResultProps, showSearch, size, styles, theme]);
  return /*#__PURE__*/React.createElement(Trigger, _extends({}, triggerProps, {
    popup: popupNode
  }), /*#__PURE__*/React.isValidElement(children) ? children : /*#__PURE__*/React.createElement(CascaderInput, _extends({
    ref: handleRef,
    inputRef: handleInputRef,
    className: [innerStyles.inputWrapper.className, className].join(' ').trim(),
    style: _extends({}, innerStyles.inputWrapper.style, style)
  }, inputProps)));
});
Cascader.__MUYA_CASCADER = true;
Cascader.Panel = CascaderPanel;
export default Cascader;