import _extends from "@babel/runtime/helpers/extends";
import { FoldIcon as _FoldIcon } from "@muya-ui/theme-light/es/icons/FoldIcon";
import _omit from "lodash/omit";
import _endsWith from "lodash/endsWith";
import React, { useMemo } from 'react';
import ExpandIcon from '../ExpandIcon';
import Input, { TagsInput } from '../Input';
import { StyledPopperInputWrapper } from '../Input/styled';
import ExpandWrapper from '../styled/components/ExpandWrapper';
import addPx from '../utils/addPx';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import { allCascaderInputProps } from './const';
var CascaderInput = memoForwardRef(function (props, ref) {
  var id = props.id,
      status = props.status,
      disabled = props.disabled,
      readOnly = props.readOnly,
      placeholder = props.placeholder,
      _props$value = props.value,
      value = _props$value === void 0 ? [] : _props$value,
      inputValue = props.inputValue,
      disabledValue = props.disabledValue,
      inputRef = props.inputRef,
      expandIcon = props.expandIcon,
      popupVisible = props.popupVisible,
      multiple = props.multiple,
      width = props.width,
      maxTagTextLength = props.maxTagTextLength,
      maxVerticalTagCount = props.maxVerticalTagCount,
      maxTagWidth = props.maxTagWidth,
      hasRelatedValue = props.hasRelatedValue,
      style = props.style,
      className = props.className,
      showSearch = props.showSearch,
      onRemoveTag = props.onRemoveTag,
      onClear = props.onClear,
      onInput = props.onInput,
      onKeyDown = props.onKeyDown,
      renderTag = props.renderTag;
  var theme = useTheme();
  var token = theme.components.Select;

  var inputRestProps = _omit(props, allCascaderInputProps);

  var suffixNode = useMemo(function () {
    var DefaultExpandIcon = token.expandIcon || _FoldIcon;
    return /*#__PURE__*/React.createElement(ExpandWrapper, {
      theme: theme,
      $expanded: popupVisible
    }, /*#__PURE__*/React.createElement(ExpandIcon, {
      icon: expandIcon,
      DefaultIcon: DefaultExpandIcon,
      disabled: disabled
    }));
  }, [disabled, expandIcon, popupVisible, theme, token.expandIcon]);
  var singleModeNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(Input, _extends({}, inputRestProps, {
      ref: ref,
      inputRef: inputRef,
      suffixNode: suffixNode,
      type: "text",
      disabled: disabled,
      width: typeof width === 'string' && _endsWith(width, '%') ? '100%' : width,
      placeholder: placeholder,
      value: inputValue,
      hasRelatedValue: hasRelatedValue,
      clearReplace: true,
      readOnly: readOnly || showSearch && !popupVisible || !showSearch,
      onClear: onClear,
      onInput: onInput,
      onKeyDown: onKeyDown,
      focusWhenClear: false,
      title: inputValue
    }));
  }, [disabled, hasRelatedValue, inputRef, inputRestProps, inputValue, onClear, onInput, onKeyDown, placeholder, popupVisible, readOnly, ref, showSearch, suffixNode, width]); // 渲染 input 和 tag

  var multipleModeNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(TagsInput, _extends({}, inputRestProps, {
      ref: ref,
      suffixNode: suffixNode,
      inputRef: inputRef,
      width: typeof width === 'string' && _endsWith(width, '%') ? '100%' : width,
      disabled: disabled,
      value: value,
      inputValue: inputValue,
      disabledValue: disabledValue,
      hasRelatedValue: hasRelatedValue,
      placeholder: value.length > 0 ? '' : placeholder,
      scrollTopAfterEffect: popupVisible,
      maxTagTextLength: maxTagTextLength,
      maxVerticalTagCount: maxVerticalTagCount,
      maxTagWidth: maxTagWidth,
      onRemoveTag: onRemoveTag,
      onClear: onClear,
      readOnly: readOnly || !showSearch,
      allowInput: !!onInput,
      onInputKeyDown: onKeyDown,
      onInput: onInput,
      focusWhenClear: false,
      renderTag: renderTag
    }));
  }, [disabled, disabledValue, hasRelatedValue, inputRef, inputRestProps, inputValue, maxTagTextLength, maxTagWidth, maxVerticalTagCount, onClear, onInput, onKeyDown, onRemoveTag, placeholder, popupVisible, readOnly, ref, renderTag, showSearch, suffixNode, value, width]);
  return /*#__PURE__*/React.createElement(StyledPopperInputWrapper, {
    id: id,
    role: "combobox",
    key: "selection",
    $pointer: !disabled || readOnly,
    $popupVisible: popupVisible,
    $width: addPx(width),
    $status: status,
    theme: theme,
    style: style,
    className: className
  }, multiple ? multipleModeNode : singleModeNode);
});
export default CascaderInput;