import { useThrottle as _useThrottle } from "@muya-ui/utils/es/hooks/useThrottle";
import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo, useCallback } from 'react';
import useTheme from '../utils/useTheme';
import { StyledCascaderMenuBorderBox, StyledCascaderMenu } from './styled';
import CascaderMenuItem from './CascaderMenuItem';
import useVirtualList from '../utils/useVirtualList';
export default /*#__PURE__*/React.memo(function (props) {
  var size = props.size,
      menuReverse = props.menuReverse,
      expandTrigger = props.expandTrigger,
      checkedKeys = props.checkedKeys,
      fieldNames = props.fieldNames,
      multiple = props.multiple,
      placement = props.placement,
      expandIcon = props.expandIcon,
      loadingIcon = props.loadingIcon,
      loadingKeys = props.loadingKeys,
      loadedKeys = props.loadedKeys,
      onLoad = props.onLoad,
      onSelect = props.onSelect,
      onDeselect = props.onDeselect,
      renderOption = props.renderOption,
      virtual = props.virtual,
      options = props.options,
      menuIndex = props.menuIndex,
      menuWidth = props.menuWidth,
      activeOptions = props.activeOptions,
      activeOptionsRef = props.activeOptionsRef,
      styles = props.styles,
      onMenuScroll = props.onMenuScroll;
  var hasScrollBar = options.length > 6;
  var theme = useTheme();
  var token = theme.components.Cascader;
  var itemHeight = token.item.height[size];
  var activeValue = activeOptions[menuIndex] && activeOptions[menuIndex][fieldNames.value];

  var _useVirtualList = useVirtualList(virtual ? options : [], {
    itemHeight: itemHeight,
    overscan: token.overscan
  }),
      list = _useVirtualList.list,
      containerProps = _useVirtualList.containerProps,
      wrapperProps = _useVirtualList.wrapperProps;

  var containerStyle = useMemo(function () {
    var containerStyle = styles.menu.style;

    if (virtual) {
      return _extends({}, containerStyle, containerProps.style);
    }

    return containerStyle;
  }, [containerProps.style, styles.menu.style, virtual]);

  var handleScroll = _useThrottle(useCallback(function (e) {
    /* istanbul ignore else */
    if (virtual) {
      containerProps.onScroll(e);
    }
    /* istanbul ignore else */


    if (onMenuScroll) {
      onMenuScroll(e, menuIndex, activeOptions);
    }
  }, [activeOptions, containerProps, menuIndex, onMenuScroll, virtual]), 50);

  var renderItem = useCallback(function (option) {
    var value = option[fieldNames.value];
    return /*#__PURE__*/React.createElement(CascaderMenuItem, _extends({
      activeOptionsRef: activeOptionsRef,
      key: value,
      loading: loadingKeys.indexOf(value) > -1,
      loaded: loadedKeys.indexOf(value) > -1,
      menuIndex: menuIndex,
      size: size,
      option: option,
      active: activeValue === value,
      expandTrigger: expandTrigger,
      checkedKeys: checkedKeys,
      fieldNames: fieldNames,
      expandIcon: expandIcon,
      loadingIcon: loadingIcon,
      multiple: multiple,
      placement: placement,
      onLoad: onLoad,
      onSelect: onSelect,
      onDeselect: onDeselect,
      renderOption: renderOption
    }, styles.menuItem));
  }, [activeOptionsRef, activeValue, checkedKeys, expandIcon, expandTrigger, fieldNames, loadedKeys, loadingIcon, loadingKeys, menuIndex, multiple, onDeselect, onLoad, onSelect, placement, renderOption, size, styles.menuItem]);
  var virtualChildren = useMemo(function () {
    if (virtual) {
      return /*#__PURE__*/React.createElement("div", wrapperProps, list.map(function (_ref) {
        var option = _ref.data;
        return renderItem(option);
      }));
    }

    return null;
  }, [list, renderItem, virtual, wrapperProps]);
  var children = useMemo(function () {
    return options.map(renderItem);
  }, [options, renderItem]);
  return /*#__PURE__*/React.createElement(StyledCascaderMenuBorderBox, _extends({
    theme: theme,
    menuReverse: menuReverse,
    hasScrollBar: hasScrollBar,
    onScroll: function onScroll(e) {
      return e.stopPropagation();
    }
  }, styles.menuBox), /*#__PURE__*/React.createElement(StyledCascaderMenu, {
    ref: containerProps.ref,
    size: size,
    theme: theme,
    menuWidth: menuWidth,
    className: styles.menu.className,
    style: containerStyle,
    onScroll: handleScroll
  }, virtual ? virtualChildren : children));
});