import { RightIcon as _RightIcon } from "@muya-ui/theme-light/es/icons/RightIcon";
import React, { useCallback, useMemo } from 'react';
import Checkbox from '../Checkbox';
import EllipsisContent from '../EllipsisContent';
import ExpandIcon from '../ExpandIcon';
import Spin from '../Spin';
import { useTheme } from '../utils/useTheme';
import { StyledCascaderMenuItem, StyledCascaderMenuItemContent } from './styled';
import useCascaderMenuItem from './useCascaderMenuItem';
export default /*#__PURE__*/React.memo(function (props) {
  var size = props.size,
      menuIndex = props.menuIndex,
      option = props.option,
      active = props.active,
      expandIcon = props.expandIcon,
      loadingIcon = props.loadingIcon,
      loading = props.loading,
      multiple = props.multiple,
      style = props.style,
      className = props.className,
      activeOptionsRef = props.activeOptionsRef,
      renderOption = props.renderOption;
  var theme = useTheme();
  var token = theme.components.Cascader;
  var disabled = option.disabled,
      loadingFromOption = option.loading;

  var _useCascaderMenuItem = useCascaderMenuItem(props),
      label = _useCascaderMenuItem.label,
      checked = _useCascaderMenuItem.checked,
      isLeaf = _useCascaderMenuItem.isLeaf,
      halfChecked = _useCascaderMenuItem.halfChecked,
      hasChildren = _useCascaderMenuItem.hasChildren,
      handleSelect = _useCascaderMenuItem.handleSelect,
      handleCheck = _useCascaderMenuItem.handleCheck,
      handleMouseEvent = _useCascaderMenuItem.handleMouseEvent;

  var handleRef = useCallback(function (el) {
    if (active) {
      activeOptionsRef.current[menuIndex] = el;
    }
  }, [active, activeOptionsRef, menuIndex]);
  var suffixIconNode = useMemo(function () {
    var suffixIconNode = null;

    if (loadingFromOption || loading) {
      suffixIconNode = loadingIcon || /*#__PURE__*/React.createElement(Spin, null);
    } else if (hasChildren || isLeaf === false) {
      var DefaultExpandIcon = token.menuExpandIcon || _RightIcon;
      suffixIconNode = expandIcon || /*#__PURE__*/React.createElement(ExpandIcon, {
        icon: expandIcon,
        DefaultIcon: DefaultExpandIcon,
        style: {
          margin: token.item.iconMargin
        },
        disabled: disabled,
        buttonType: active ? 'primary' : 'weak'
      });
    }

    return suffixIconNode;
  }, [active, disabled, expandIcon, hasChildren, isLeaf, loading, loadingFromOption, loadingIcon, token.item.iconMargin, token.menuExpandIcon]);
  return /*#__PURE__*/React.createElement(StyledCascaderMenuItem, {
    ref: handleRef,
    role: "menuitem",
    size: size,
    $disabled: !!disabled,
    $selected: active,
    $multiple: multiple,
    onClick: handleSelect,
    onMouseEnter: handleMouseEvent,
    onMouseLeave: handleMouseEvent,
    theme: theme,
    style: style,
    className: className,
    onMouseDown: function onMouseDown(e) {
      return e.preventDefault();
    }
  }, /*#__PURE__*/React.createElement(StyledCascaderMenuItemContent, null, multiple && /*#__PURE__*/React.createElement(Checkbox, {
    key: "checkbox",
    size: size,
    indeterminate: !checked && halfChecked,
    checked: checked,
    onChange: handleCheck,
    disabled: disabled
  }), renderOption ? renderOption(option, menuIndex) : /*#__PURE__*/React.createElement(EllipsisContent, {
    key: "label",
    enableTitle: true
  }, label)), suffixIconNode);
});