import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo, useRef } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import CascaderMenu from './CascaderMenu';
import { StyledCascaderPanel } from './styled';
import useCascaderPanel from './useCascaderPanel';
var defaultStyles = {
  panel: '',
  menuBox: '',
  menu: '',
  menuItem: ''
};
var PureCascaderPanel = memoForwardRef(function (props, ref) {
  var size = props.size,
      menusWidth = props.menusWidth,
      menuReverse = props.menuReverse,
      expandTrigger = props.expandTrigger,
      checkedKeys = props.checkedKeys,
      styles = props.styles,
      fieldNames = props.fieldNames,
      multiple = props.multiple,
      placement = props.placement,
      expandIcon = props.expandIcon,
      loadingIcon = props.loadingIcon,
      loadingKeys = props.loadingKeys,
      loadedKeys = props.loadedKeys,
      onLoad = props.onLoad,
      onSelect = props.onSelect,
      onDeselect = props.onDeselect,
      renderOption = props.renderOption,
      virtual = props.virtual,
      onMenuScroll = props.onMenuScroll,
      pure = props.pure;
  var theme = useTheme();
  var activeOptionsRef = useRef([]);
  var pickStyles = useMemo(function () {
    if (styles) {
      var panel = styles.panel,
          menuBox = styles.menuBox,
          menu = styles.menu,
          menuItem = styles.menuItem;
      return {
        panel: panel,
        menuBox: menuBox,
        menu: menu,
        menuItem: menuItem
      };
    }
  }, [styles]);
  var innerStyles = useStyles('cascader', defaultStyles, pickStyles);

  var _useCascaderPanel = useCascaderPanel(props, activeOptionsRef),
      showOptions = _useCascaderPanel.showOptions,
      activeOptions = _useCascaderPanel.activeOptions;

  return /*#__PURE__*/React.createElement(StyledCascaderPanel, _extends({
    ref: ref,
    theme: theme,
    menuReverse: menuReverse,
    $pure: !!pure
  }, innerStyles.panel), showOptions.map(function (options, menuIndex) {
    var menuWidth = menusWidth;

    if (Array.isArray(menusWidth)) {
      menuWidth = menusWidth[menuIndex];
    }

    return /*#__PURE__*/React.createElement(CascaderMenu, {
      key: menuIndex,
      size: size,
      menuReverse: menuReverse,
      expandTrigger: expandTrigger,
      checkedKeys: checkedKeys,
      fieldNames: fieldNames,
      multiple: multiple,
      placement: placement,
      expandIcon: expandIcon,
      loadingIcon: loadingIcon,
      loadingKeys: loadingKeys,
      loadedKeys: loadedKeys,
      onLoad: onLoad,
      onSelect: onSelect,
      onDeselect: onDeselect,
      renderOption: renderOption,
      virtual: virtual,
      options: options,
      menuIndex: menuIndex,
      menuWidth: menuWidth,
      activeOptions: activeOptions,
      activeOptionsRef: activeOptionsRef,
      onMenuScroll: onMenuScroll,
      styles: innerStyles
    });
  }));
});
export default PureCascaderPanel;