import styled, { css } from 'styled-components';
import { StyledCheckboxLabel } from '../Checkbox/styled';
import { scrollBarStyle } from '../ScrollView/styled';
import { itemDisabledStyle, itemNormalStyle, itemSelectedStyle } from '../styled/mixins/item';
import listResetStyle from '../styled/mixins/listReset';

var menuItemCss = function menuItemCss(props) {
  var size = props.size,
      $selected = props.$selected,
      $disabled = props.$disabled,
      $multiple = props.$multiple,
      _props$theme = props.theme,
      fontSize = _props$theme.typography.spec.fontSize,
      _props$theme$transiti = _props$theme.transition.pattern,
      duration = _props$theme$transiti.duration,
      easing = _props$theme$transiti.easing;
  var token = props.theme.components.Cascader.item;
  var fontLevel = token.fontLevel[size];
  return css(["display:flex;justify-content:space-between;align-items:center;flex:1;box-sizing:border-box;font-size:", "px;line-height:1;height:", "px;cursor:pointer;transition:background-color ", "ms ", ",color ", "ms ", ";", " ", " ", " ", " ", "{margin-right:", "px;}"], fontSize[fontLevel], token.height[size], duration.status, easing.status, duration.status, easing.status, itemNormalStyle(token), $selected && itemSelectedStyle(token), $selected && $multiple && css(["font-weight:", ";"], token.multipleSelectFontWeight), $disabled && itemDisabledStyle(token), StyledCheckboxLabel, token.checkboxMarginRight);
};

export var StyledCascaderMenuItem = styled.li.withConfig({
  displayName: "StyledCascaderMenuItem"
})(["", ";", ";"], listResetStyle, menuItemCss);
export var StyledCascaderMenuItemContent = styled.div.withConfig({
  displayName: "StyledCascaderMenuItemContent"
})(["display:flex;justify-content:flex-start;align-items:center;overflow:hidden;"]);
/**
 * menu
 */

var menuCss = function menuCss(props) {
  var size = props.size,
      menuWidth = props.menuWidth,
      theme = props.theme;
  var token = theme.components.Cascader;
  return css(["position:relative;max-height:", "px;margin:", "px 0;background:", ";overflow-y:auto;overflow-x:hidden;", ";", ";"], token.maxHeight[size], token.menuMarginVertical, token.background, menuWidth ? "width: ".concat(menuWidth, "px") : 'min-width: 111px', scrollBarStyle({
    theme: theme
  }));
};

export var StyledCascaderMenu = styled.ul.withConfig({
  displayName: "StyledCascaderMenu"
})(["", ";", ";"], listResetStyle, menuCss); // 滚动条和菜单反向的样式

var menuReverseCss = css(["border-left:1px solid ", ";&:last-child{border-left:none;}"], function (props) {
  return props.theme.colors.pattern.border.normal;
});
var menuNoReverseCss = css(["border-right:1px solid ", ";&:last-child{border-right:none;}"], function (props) {
  return props.theme.colors.pattern.border.normal;
});

var menuBoxCss = function menuBoxCss(props) {
  var hasScrollBar = props.hasScrollBar,
      menuReverse = props.menuReverse,
      token = props.theme.components.Cascader;
  var scrollBarCss;

  if (hasScrollBar) {
    scrollBarCss = css(["", "{margin-right:", "px;}", "{padding-left:", "px;margin-right:", "px;}", ";"], StyledCascaderMenu, token.menuMarginRight - 1, StyledCascaderMenuItem, token.item.paddingHorizontal, token.item.marginRight, menuReverse ? css(["&:first-child{", "{margin-right:", "px;}}"], StyledCascaderMenu, token.menuMarginRight) : css(["&:last-child{", "{margin-right:", "px;}}"], StyledCascaderMenu, token.menuMarginRight));
  } else {
    scrollBarCss = css(["", ";"], menuReverse ? css(["", "{padding-left:", "px;padding-right:", "px;}&:last-child{", "{padding-left:", "px;}}"], StyledCascaderMenuItem, token.item.paddingHorizontal - 1, token.item.paddingHorizontal, StyledCascaderMenuItem, token.item.paddingHorizontal) : css(["", "{padding-left:", "px;padding-right:", "px;}&:last-child{", "{padding-right:", "px;}}"], StyledCascaderMenuItem, token.item.paddingHorizontal, token.item.paddingHorizontal - 1, StyledCascaderMenuItem, token.item.paddingHorizontal));
  }

  return css(["background:", ";&:first-child{border-top-left-radius:", ";border-bottom-left-radius:", ";}&:last-child{border-top-right-radius:", ";border-bottom-right-radius:", ";}", " ", ""], token.background, token.borderRadius, token.borderRadius, token.borderRadius, token.borderRadius, menuReverse ? menuReverseCss : menuNoReverseCss, scrollBarCss);
};

export var StyledCascaderMenuBorderBox = styled.div.withConfig({
  displayName: "StyledCascaderMenuBorderBox"
})(["", ";"], menuBoxCss);

var menuWrapperCss = function menuWrapperCss(props) {
  var menuReverse = props.menuReverse,
      _props$theme2 = props.theme,
      shadows = _props$theme2.shadows,
      colors = _props$theme2.colors,
      $pure = props.$pure;
  var pureCss;

  if ($pure) {
    pureCss = css(["box-shadow:none;border:1px solid ", ";"], colors.pattern.border.normal);
  }

  return css(["display:flex;flex-direction:", ";box-shadow:", ";", ";"], menuReverse ? 'row-reverse' : 'row', shadows.pattern.popper.normal, pureCss);
};

export var StyledCascaderPanel = styled.div.withConfig({
  displayName: "StyledCascaderPanel"
})(["", ";", ";"], listResetStyle, menuWrapperCss);
export var StyledNotFoundContent = styled.div.withConfig({
  displayName: "StyledNotFoundContent"
})(["display:flex;justify-content:center;align-items:center;height:", "px;"], function (props) {
  return props.theme.components.Cascader.notFoundPanelHeight;
});