import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _findIndex from "lodash/findIndex";
import _get from "lodash/get";
import _omit from "lodash/omit";
import _last from "lodash/last";
import _isEmpty from "lodash/isEmpty";

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

import React from 'react';
import Typography from '../Typography';

/**
 * 默认的展示规则
 */
export var defaultDisplayRender = function defaultDisplayRender(labels) {
  return labels.join('/');
};
export function getLeafValues(value, multiple) {
  if (multiple) {
    return value.filter(function (item) {
      return !_isEmpty(item);
    }).map(function (item) {
      return [_last(item)];
    });
  }

  if (_isEmpty(value)) {
    return [];
  }

  return [_last(value)];
}
export var getValueStateFromProps = function getValueStateFromProps(value, multiple, controlLeaf, keyEntities, keyPosMap) {
  if (controlLeaf) {
    return _toConsumableArray(value).filter(function (item) {
      return !_isEmpty(Array.isArray(item) ? item : item.toString());
    }) // 将数字类型的value转换成字符串判断，不然isEmpty会将他判别为空
    .reduce(function (result, item) {
      if (multiple) {
        var key = item[0];
        var poses = keyPosMap[key];

        if (poses) {
          if (poses.length > 1) {
            _devWarning("[Cascader]: value ".concat(key, " has duplicate options in cascader. when open controlLeaf option, please keep value uniq."));

            return result;
          }

          result.push(keyEntities["".concat(key, "|").concat(poses[0])].keyPath);
        } else {
          _devWarning('[Cascader]: value not exist in options');
        }

        return result;
      } else {
        var _key = item;
        var _poses = keyPosMap[_key];

        if (_poses) {
          if (_poses.length > 1) {
            _devWarning("[Cascader]: value ".concat(_key, " has duplicate options in cascader. when open controlLeaf option, please keep value uniq."));

            return result;
          }

          return result.concat(keyEntities["".concat(_key, "|").concat(_poses[0])].keyPath);
        } else {
          _devWarning('[Cascader]: value not exist in options');
        }
      }

      return result;
    }, []);
  }

  return value;
};
/**
 * 从 value 获取激活态的 value
 */

export var getActiveStateFromValue = function getActiveStateFromValue(value, multiple) {
  var activeState = [];

  if (multiple) {
    if (value.length > 0) {
      activeState = value[0];
    }
  } else {
    activeState = value;
  }

  return activeState;
};
/**
 * 遍历 options
 */

export function traverseOptions(options, fieldNames, callback) {
  function dfs(node, index, parent) {
    var children = node[fieldNames.children];
    var pos = "".concat(parent.pos, "-").concat(index);
    var key = typeof node[fieldNames.value] !== 'undefined' ? node[fieldNames.value] : pos;
    var data = {
      node: node,
      index: index,
      pos: pos,
      key: key,
      parentPos: parent.pos,
      level: parent.level + 1,
      keyPath: parent.keyPath.concat([key])
    };
    callback(data);

    if (Array.isArray(children) && children.length > 0) {
      children.forEach(function (subNode, subIndex) {
        dfs(subNode, subIndex, data);
      });
    }
  }

  var parent = {
    node: null,
    pos: '0',
    keyPath: [],
    level: -1
  };

  var _iterator = _createForOfIteratorHelper(options.entries()),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var _step$value = _slicedToArray(_step.value, 2),
          index = _step$value[0],
          item = _step$value[1];

      dfs(item, index, parent);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
}
/**
 * 遍历 options 生成 key 和 pos 的索引结构
 */

export function getEntityDataFromOptions(options, fieldNames) {
  var posEntities = {};
  var keyEntities = {};
  var keyPathEntities = {};
  var keyPosMap = {};
  var wrapper = {
    posEntities: posEntities,
    keyEntities: keyEntities,
    keyPathEntities: keyPathEntities,
    keyPosMap: keyPosMap
  };
  traverseOptions(options, fieldNames, function (item) {
    var parentPos = item.parentPos,
        pos = item.pos,
        key = item.key;

    var entity = _omit(item, 'parentPos');

    posEntities[pos] = entity; // 解决值重复情况下定位不准确的问题

    keyEntities["".concat(key, "|").concat(pos)] = entity;
    keyPathEntities[entity.keyPath.join('/')] = entity;

    if (!keyPosMap[key]) {
      keyPosMap[key] = [];
    }

    keyPosMap[key].push(pos);
    entity.parent = posEntities[parentPos];

    if (entity.parent) {
      entity.parent.children = entity.parent.children || [];
      entity.parent.children.push(entity);
    }
  });
  return wrapper;
}
/**
 * 根据之前的选中状态计算选中或者取消选中部分 key 后新的状态
 * @param keyList 需要改变的 key 列表
 * @param checked 改变为 true 还是 false
 * @param keyEntities key 索引的 option 数据
 * @param checkStatus 之前的选中状态
 */

export var calcCheckedKeys = function calcCheckedKeys(keyList, checked, checkStrictly, keyEntities) {
  var checkStatus = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
  var checkedKeys = {};
  var halfCheckedKeys = {};
  (checkStatus.checked || []).forEach(function (_ref) {
    var key = _ref.key,
        pos = _ref.pos;
    checkedKeys["".concat(key, "|").concat(pos)] = true;
  });
  (checkStatus.halfChecked || []).forEach(function (_ref2) {
    var key = _ref2.key,
        pos = _ref2.pos;
    halfCheckedKeys["".concat(key, "|").concat(pos)] = true;
  }); // Conduct up

  function conductUp(key, pos) {
    if (checked && checkedKeys["".concat(key, "|").concat(pos)] || !checked && !checkedKeys["".concat(key, "|").concat(pos)] && !halfCheckedKeys["".concat(key, "|").concat(pos)]) return;
    var entity = keyEntities["".concat(key, "|").concat(pos)];
    if (!entity) return;
    var children = entity.children,
        parent = entity.parent,
        node = entity.node;
    if (node.disabled) return; // Check child node checked status

    var everyChildChecked = true;
    var someChildChecked = false; // Child checked or half checked

    (children || []).filter(function (child) {
      return !child.node.disabled;
    }).forEach(function (_ref3) {
      var childKey = _ref3.key,
          childPos = _ref3.pos;
      var childChecked = checkedKeys["".concat(childKey, "|").concat(childPos)];
      var childHalfChecked = halfCheckedKeys["".concat(childKey, "|").concat(childPos)];
      if (childChecked || childHalfChecked) someChildChecked = true;
      if (!childChecked) everyChildChecked = false;
    }); // Update checked status

    if (checked) {
      checkedKeys["".concat(key, "|").concat(pos)] = everyChildChecked;
    } else {
      checkedKeys["".concat(key, "|").concat(pos)] = false;
    }

    halfCheckedKeys["".concat(key, "|").concat(pos)] = someChildChecked;

    if (parent) {
      conductUp(parent.key, parent.pos);
    }
  } // Conduct down


  function conductDown(key, pos) {
    if (checkedKeys["".concat(key, "|").concat(pos)] === checked) return;
    var entity = keyEntities["".concat(key, "|").concat(pos)];
    if (!entity) return;
    var children = entity.children,
        node = entity.node;
    if (node.disabled) return;
    checkedKeys["".concat(key, "|").concat(pos)] = checked;
    (children || []).forEach(function (child) {
      conductDown(child.key, child.pos);
    });
  }

  function conduct(key, pos) {
    var entity = keyEntities["".concat(key, "|").concat(pos)];

    if (!entity) {
      _devWarning("[Cascader]: '".concat(key, "' does not exist in the cascader."));

      return;
    }

    var children = entity.children,
        parent = entity.parent,
        node = entity.node;
    checkedKeys["".concat(key, "|").concat(pos)] = checked;
    if (node.disabled) return; // Conduct down

    (children || []).filter(function (child) {
      return !child.node.disabled;
    }).forEach(function (child) {
      conductDown(child.key, child.pos);
    }); // Conduct up

    if (parent) {
      conductUp(parent.key, parent.pos);
    }
  }

  (keyList || []).forEach(function (_ref4) {
    var key = _ref4.key,
        pos = _ref4.pos;

    if (checkStrictly) {
      if (Array.isArray(key)) {
        checkedKeys["".concat(key[key.length - 1], "|").concat(pos)] = checked;
      } else {
        checkedKeys["".concat(key, "|").concat(pos)] = checked;
      }
    } else {
      if (Array.isArray(key)) {
        conduct(key[key.length - 1], pos);
      } else {
        conduct(key, pos);
      }
    }
  });
  var checkedKeyList = [];
  var halfCheckedKeyList = []; // Fill checked list

  Object.keys(checkedKeys).forEach(function (key) {
    if (checkedKeys[key] && keyEntities[key]) {
      checkedKeyList.push(keyEntities[key]);
    }
  }); // Fill half checked list

  Object.keys(halfCheckedKeys).forEach(function (key) {
    if (!checkedKeys[key] && halfCheckedKeys[key] && keyEntities[key]) {
      halfCheckedKeyList.push(keyEntities[key]);
    }
  });
  return {
    checked: checkedKeyList,
    halfChecked: halfCheckedKeyList
  };
};
export var diffValue = function diffValue(newValue, oldValue, multiple, controlLeaf) {
  var realOldValue = oldValue;
  var realNewValue = newValue;

  if (controlLeaf) {
    realOldValue = getLeafValues(oldValue, multiple);
    realNewValue = getLeafValues(realNewValue, multiple);
  }

  if (realNewValue.length !== realOldValue.length) {
    return false;
  } else {
    return realNewValue.every(function (nv, idx) {
      if (Array.isArray(nv)) {
        var nvStr = nv.join('-');
        var ov = realOldValue[idx];

        if (Array.isArray(ov)) {
          return ov.join('-') === nvStr;
        } else {
          return ov === nvStr;
        }
      } else {
        return realOldValue[idx] === nv;
      }
    });
  }
};
export function findOption(options, value, fieldNames) {
  var _iterator2 = _createForOfIteratorHelper(options),
      _step2;

  try {
    for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
      var option = _step2.value;

      if (option[fieldNames.value] === value) {
        return option;
      }

      if (Array.isArray(option[fieldNames.children])) {
        var result = findOption(option[fieldNames.children], value, fieldNames);

        if (result) {
          return result;
        }
      }
    }
  } catch (err) {
    _iterator2.e(err);
  } finally {
    _iterator2.f();
  }
}
export function getPlaceholder(value, defaultPlaceholder) {
  var inputValue = !Array.isArray(value) ? value : value[0];

  if (inputValue) {
    return inputValue;
  } else {
    return defaultPlaceholder;
  }
}
export function highlightKeyword(str, keyword) {
  return str.split(keyword).map(function (node, index) {
    return index === 0 ? node : [/*#__PURE__*/React.createElement(Typography.Text, {
      key: "seperator",
      color: "highlight"
    }, keyword), node];
  });
}
export function defaultFilterOption(inputValue, path, names) {
  return path.some(function (option) {
    return option[names.label].indexOf(inputValue) > -1;
  });
}
export function defaultRenderFilteredOption(inputValue, path, names) {
  return path.map(function (option, index) {
    var label = option[names.label];
    var node = label.indexOf(inputValue) > -1 ? highlightKeyword(label, inputValue) : label;
    return index === 0 ? node : [' / ', node];
  });
}
export function defaultSortFilteredOption(a, b, inputValue, names) {
  function callback(elem) {
    return elem[names.label].indexOf(inputValue) > -1;
  }

  return _findIndex(a, callback) - _findIndex(b, callback);
}
export function flattenTree(props) {
  var options = props.options,
      changeOnSelect = props.changeOnSelect,
      _props$ancestor = props.ancestor,
      ancestor = _props$ancestor === void 0 ? [] : _props$ancestor,
      names = props.names;
  var flattenOptions = [];
  var childrenName = names.children;
  options.forEach(function (option) {
    var path = ancestor.concat(option);

    if (changeOnSelect || !option[childrenName] || !option[childrenName].length) {
      flattenOptions.push(path);
    }

    if (option[childrenName]) {
      flattenOptions = flattenOptions.concat(flattenTree({
        options: option[childrenName],
        changeOnSelect: changeOnSelect,
        ancestor: path,
        names: names
      }));
    }
  });
  return flattenOptions;
}
export function getActiveKeyFromValue(valueState) {
  var value = _get(valueState, '0', []);

  if (Array.isArray(value)) {
    return value.toString();
  } else {
    return value;
  }
}
export function calValueEntity(value, keyPathEntities) {
  if (!value || value.length === 0) return [];

  if (Array.isArray(value[0])) {
    return value.map(function (item) {
      var entity = keyPathEntities[item.join('/')];
      return entity;
    });
  }

  var entity = keyPathEntities[value.join('/')];
  return [entity];
}