import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["value", "disabled", "onChange", "ellipsis", "width", "children"];
import React, { useMemo } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import Checkbox from './Checkbox';
import { StyledCheckboxGroup } from './styled';
import useCheckboxGroup from './useCheckboxGroup';
var CheckboxGroup = memoForwardRef(function (props, ref) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      styles = props.styles;

  var _useCheckboxGroup = useCheckboxGroup(props),
      opts = _useCheckboxGroup.opts,
      restProps = _useCheckboxGroup.restProps,
      valueState = _useCheckboxGroup.valueState,
      children = _useCheckboxGroup.children,
      renderCheckbox = _useCheckboxGroup.renderCheckbox,
      checkboxEllipsis = _useCheckboxGroup.checkboxEllipsis,
      checkboxWidth = _useCheckboxGroup.checkboxWidth,
      handleChange = _useCheckboxGroup.handleChange;

  var theme = useTheme();
  var checkboxList = useMemo(function () {
    if (opts && opts.length) {
      return opts.map(function (option, optIndex) {
        var label = option.label,
            value = option.value,
            disabled = option.disabled,
            ellipsis = option.ellipsis,
            width = option.width;
        var checkboxNode = /*#__PURE__*/React.createElement(Checkbox, {
          key: value.toString(),
          styles: styles,
          size: size,
          value: value,
          ellipsis: ellipsis === undefined ? checkboxEllipsis : ellipsis,
          width: width || checkboxWidth,
          disabled: props.disabled || disabled,
          checked: valueState.indexOf(value) > -1,
          onChange: handleChange.bind(null, option)
        }, label);
        return renderCheckbox(checkboxNode, option, optIndex);
      });
    }

    return React.Children.map(children, function (child, childIndex) {
      if (! /*#__PURE__*/React.isValidElement(child)) {
        return null;
      }

      var _child$props = child.props,
          value = _child$props.value,
          disabled = _child$props.disabled,
          onChange = _child$props.onChange,
          ellipsis = _child$props.ellipsis,
          width = _child$props.width,
          label = _child$props.children,
          otherChildProps = _objectWithoutProperties(_child$props, _excluded);

      var option = {
        value: value,
        disabled: disabled,
        onChange: onChange,
        label: label
      };
      var checkboxNode = /*#__PURE__*/React.cloneElement(child, _extends({}, otherChildProps, {
        children: label,
        value: value,
        ellipsis: ellipsis === undefined ? checkboxEllipsis : ellipsis,
        width: width || checkboxWidth,
        checked: valueState.indexOf(value) > -1,
        disabled: props.disabled || disabled,
        styles: styles,
        size: size,
        onChange: handleChange.bind(null, option)
      }));
      return renderCheckbox(checkboxNode, option, childIndex);
    });
  }, [checkboxEllipsis, checkboxWidth, children, handleChange, opts, props.disabled, renderCheckbox, size, styles, valueState]);
  return /*#__PURE__*/React.createElement(StyledCheckboxGroup, _extends({}, restProps, {
    ref: ref,
    theme: theme
  }), checkboxList);
});
CheckboxGroup.useCheckboxGroup = useCheckboxGroup;
export default CheckboxGroup;