import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["defaultValue", "disabled", "value", "options", "onChange", "onClick", "children", "renderCheckbox", "checkboxEllipsis", "checkboxWidth"];
import { useCallback, useMemo, useState } from 'react';
import { shallowEqualArrays } from 'shallow-equal';

var defaultRenderCheckbox = function defaultRenderCheckbox(checkboxNode) {
  return checkboxNode;
};

export function useCheckboxGroup(props) {
  var _props$defaultValue = props.defaultValue,
      defaultValue = _props$defaultValue === void 0 ? [] : _props$defaultValue,
      disabled = props.disabled,
      value = props.value,
      options = props.options,
      onChange = props.onChange,
      onClick = props.onClick,
      children = props.children,
      _props$renderCheckbox = props.renderCheckbox,
      renderCheckbox = _props$renderCheckbox === void 0 ? defaultRenderCheckbox : _props$renderCheckbox,
      checkboxEllipsis = props.checkboxEllipsis,
      checkboxWidth = props.checkboxWidth,
      restProps = _objectWithoutProperties(props, _excluded);

  var isControlled = ('value' in props);

  var _useState = useState((isControlled ? value : defaultValue) || []),
      _useState2 = _slicedToArray(_useState, 2),
      valueState = _useState2[0],
      setValueState = _useState2[1];

  var opts = useMemo(function () {
    return (options || []).map(function (option) {
      if (typeof option === 'string') {
        return {
          label: option,
          value: option
        };
      }

      return option;
    });
  }, [options]);
  var handleLogic = useCallback(function (checked, newValueState, option) {
    if (checked) {
      newValueState.push(option.value);
    } else {
      var index = newValueState.indexOf(option.value);
      newValueState.splice(index, 1);
    }

    if (!isControlled) {
      setValueState(newValueState);
    }

    if (onChange) {
      onChange(newValueState);
    }
  }, [isControlled, onChange]);
  var handleChange = useCallback(function (option, e) {
    var checked = e.target.checked;

    var newValueState = _toConsumableArray(valueState);

    handleLogic(checked, newValueState, option);

    if (option.onChange) {
      option.onChange(e);
    }
  }, [handleLogic, valueState]);
  var handleClick = useCallback(function (option, e) {
    var newValueState = _toConsumableArray(valueState);

    var checked = newValueState.indexOf(option.value) > -1;
    handleLogic(!checked, newValueState, option);

    if (option.onClick) {
      option.onClick(e);
    }
  }, [handleLogic, valueState]);

  if (isControlled && !shallowEqualArrays(props.value || [], valueState)) {
    setValueState(props.value || []);
  }

  return {
    opts: opts,
    valueState: valueState,
    handleChange: handleChange,
    handleClick: handleClick,
    restProps: restProps,
    children: children,
    renderCheckbox: renderCheckbox,
    checkboxEllipsis: checkboxEllipsis,
    checkboxWidth: checkboxWidth
  };
}
export default useCheckboxGroup;