import { useDebounce as _useDebounce3 } from "@muya-ui/utils/es/hooks/useDebounce";
import _extends from "@babel/runtime/helpers/extends";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _pick from "lodash/pick";
var _excluded = ["selectType", "disableRange", "disableDate", "options", "todayDate", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "onSelect", "viewType", "viewDate", "selectedDate", "defaultDate", "defaultViewDate", "disabled", "readOnly", "defaultValue", "value", "onFocus", "onClear", "onPressEnter", "allowClear", "inputRef", "inputDelay", "format", "popupOpen", "onChange", "onUIChange", "onDateChange", "onCalendarChange", "onPopupVisibleChange", "suffixNode", "placement", "triggerId", "popperProps", "flip", "arrowPointAtCenter", "calendarStyles", "styles", "getPopupContainer", "inputReadOnly"];
import dayjs from 'dayjs';
import { useMemo, useRef, useState } from 'react';
import useCalendar from '../Calendar/useCalendar';
import forkHandler from '../utils/forkHandler';
import useFormatDate from './useFormatDate';
export default function useDatePicker(props) {
  var _props$selectType = props.selectType,
      selectType = _props$selectType === void 0 ? 'date' : _props$selectType,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      todayDate = props.todayDate,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      onSelect = props.onSelect,
      viewType = props.viewType,
      viewDate = props.viewDate,
      selectedDate = props.selectedDate,
      defaultDate = props.defaultDate,
      defaultViewDate = props.defaultViewDate,
      disabled = props.disabled,
      readOnly = props.readOnly,
      defaultValue = props.defaultValue,
      value = props.value,
      onFocus = props.onFocus,
      onClear = props.onClear,
      onPressEnter = props.onPressEnter,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      inputRef = props.inputRef,
      _props$inputDelay = props.inputDelay,
      inputDelay = _props$inputDelay === void 0 ? 200 : _props$inputDelay,
      format = props.format,
      popupOpen = props.popupOpen,
      onChange = props.onChange,
      onUIChange = props.onUIChange,
      onDateChange = props.onDateChange,
      onCalendarChange = props.onCalendarChange,
      onPopupVisibleChange = props.onPopupVisibleChange,
      suffixNode = props.suffixNode,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      triggerId = props.triggerId,
      popperProps = props.popperProps,
      flip = props.flip,
      arrowPointAtCenter = props.arrowPointAtCenter,
      calendarStyles = props.calendarStyles,
      styles = props.styles,
      getPopupContainer = props.getPopupContainer,
      inputReadOnly = props.inputReadOnly,
      otherProps = _objectWithoutProperties(props, _excluded);

  var innerInputRef = useRef(null);

  var handleInputRef = _useForkRef(innerInputRef, inputRef);

  var _useFormatDate = useFormatDate(selectType, format),
      formatDate = _useFormatDate.formatDate,
      formatHasChanged = _useFormatDate.formatHasChanged;

  var valueControlled = ('value' in props);
  var valuePropDate = useMemo(function () {
    return dayjs(value || defaultValue || '');
  }, [defaultValue, value]);
  var valuePropStr = useMemo(function () {
    return formatDate(valuePropDate);
  }, [formatDate, valuePropDate]); // 从 props 中获得的合法的 dayjs 对象

  var validValueDateFromProps = useMemo(function () {
    return valuePropDate.isValid() ? valuePropDate : undefined;
  }, [valuePropDate]);

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      statePopupOpen = _useState2[0],
      setPopupOpen = _useState2[1];

  var _useState3 = useState(valuePropStr),
      _useState4 = _slicedToArray(_useState3, 2),
      stateValue = _useState4[0],
      setStateValue = _useState4[1];

  var isOpenControlled = ('popupOpen' in props);
  var finalPopupOpen = isOpenControlled ? !!popupOpen : statePopupOpen;
  var valueInStateIsDifferentFromProps = stateValue !== valuePropStr; // 情况1：当用户受控使用时，可能外面会处理 value，重新传入的时候，需要判断一下需不需要更新状态里的 value
  // 情况2：字符串格式化发生变化的时候，需要判断一下状态里的 value 需不需要更新

  if ((valueControlled && !finalPopupOpen || formatHasChanged) && valueInStateIsDifferentFromProps) {
    setStateValue(valuePropStr);
  } // pick 相对于解构的优势在于一些受控的 props ，比如这里
  // selectedDate ，如果直接类似 { selectedDate } ，那么 Hooks 中对于受控的判断就不对了。


  var calendarProps = _pick(props, ['selectType', 'selectedDate', 'viewType', 'viewDate', 'disableDate', 'disableRange']);

  if (defaultValue) {
    calendarProps.defaultDate = defaultValue;
  }

  if (defaultDate || defaultViewDate) {
    calendarProps.defaultViewDate = defaultDate || defaultViewDate;
  }

  if (valueControlled && !('selectedDate' in props)) {
    calendarProps.selectedDate = validValueDateFromProps;
  }

  calendarProps.onUIChange = _useEventCallback(function (inputState) {
    var newState = _extends({
      popupOpen: finalPopupOpen
    }, inputState);

    if (onUIChange) {
      onUIChange(newState);
    }
  });
  calendarProps.onChange = _useEventCallback(function (date) {
    if (!onDateChange && !onChange) {
      return;
    }

    var innerDate = date ? date : dayjs('');

    if (onDateChange) {
      var innerValue = date ? formatDate(date) : '';
      onDateChange(innerValue, innerDate);
    }

    if (onChange) {
      onChange(innerDate);
    }
  });
  /**
   * 这里复用 hooks 的逻辑需要注意的点：
   * * 事件处理一定要在这个 hook 里处理
   */

  var calendarState = useCalendar(calendarProps);

  var finalUIState = _extends({}, calendarState.finalUIState, {
    popupOpen: finalPopupOpen
  });

  var finalValue = stateValue;
  var finalSelectedDate = calendarState.finalSelectedDate;

  if (valueControlled && !finalPopupOpen) {
    finalValue = valuePropStr;
  } // 切换日历面板的显示隐藏


  var updateUI = function updateUI(innerPopupOpen) {
    var newState = _extends({}, calendarState.finalUIState, {
      popupOpen: innerPopupOpen
    });

    handlePopupVisibleChange(innerPopupOpen);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var lastValidDate = useRef(validValueDateFromProps);

  var selectByValue = function selectByValue(newValue) {
    var valueDate = dayjs(newValue);
    var valid = valueDate.isValid();

    if (valid && calendarState.isDateDisabled(valueDate)) {
      valid = false;
    }

    if (valid) {
      lastValidDate.current = valueDate;
    }

    if (newValue === '' && allowClear) {
      calendarState.updateSelectedDate();
      return '';
    } else {
      calendarState.updateSelectedDate(lastValidDate.current, true);
      return formatDate(lastValidDate.current);
    }
  };

  var _useDebounce = _useDebounce3(selectByValue, inputDelay),
      _useDebounce2 = _slicedToArray(_useDebounce, 2),
      debounceUpdateInputSelected = _useDebounce2[0],
      clearUpdateInputSelected = _useDebounce2[1];
  /**
   * 关闭日历面板
   */


  var closePopup = function closePopup() {
    if (!finalUIState.popupOpen) {
      return;
    }

    updateUI(false);
    clearUpdateInputSelected();
  };

  var handleCalendarUIChange = _useEventCallback(function (inputState) {
    if (innerInputRef.current) {
      innerInputRef.current.focus();
    }

    calendarState.updateUI(inputState);
  });

  var handleCalendarChange = _useEventCallback(forkHandler(function (date) {
    calendarState.updateSelectedDate(date);

    if (date) {
      closePopup();
    }

    setStateValue(formatDate(date));
  }, onCalendarChange));

  var handleInputChange = _useEventCallback(function (e) {
    debounceUpdateInputSelected(e.target.value);
    setStateValue(e.target.value);
  });

  var handleFocus = _useEventCallback(forkHandler(function () {
    updateUI(true);
  }, onFocus));

  var handleClear = _useEventCallback(forkHandler(function () {
    calendarState.updateSelectedDate();
    clearUpdateInputSelected();
    setStateValue('');
  }, onClear));

  var handleClickAway = _useEventCallback(function () {
    if (finalPopupOpen) {
      closePopup();
    }
  });

  var handlePressEnter = _useEventCallback(forkHandler(function () {
    if (innerInputRef.current) {
      innerInputRef.current.blur();
    }

    var newValue = selectByValue(finalValue);
    closePopup();
    setStateValue(newValue);
  }, onPressEnter));

  var handlePopupVisibleChange = _useEventCallback(function (open) {
    if (!isOpenControlled) {
      setPopupOpen(open);
    }

    if (onPopupVisibleChange && finalPopupOpen !== open) {
      onPopupVisibleChange(open);
    }
  });

  return {
    finalUIState: finalUIState,
    finalValue: finalValue,
    finalSelectedDate: finalSelectedDate,
    otherProps: otherProps,
    // 事件处理
    handleFocus: handleFocus,
    handleCalendarChange: handleCalendarChange,
    handleCalendarUIChange: handleCalendarUIChange,
    handleInputChange: handleInputChange,
    handlePressEnter: handlePressEnter,
    handleClickAway: handleClickAway,
    handleClear: handleClear,
    handleInputRef: handleInputRef,
    handleAddDate: function handleAddDate() {},
    handleRemoveDate: function handleRemoveDate() {},
    // for test
    innerInputRef: innerInputRef,
    // 透传....
    styles: styles,
    disabled: disabled,
    readOnly: readOnly,
    allowClear: allowClear,
    calendarStyles: calendarStyles,
    onSelect: onSelect,
    container: getPopupContainer,
    inputReadOnly: inputReadOnly,
    // props from calendar 直接透传
    selectType: selectType,
    disableRange: disableRange,
    disableDate: disableDate,
    options: options,
    todayDate: todayDate,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // trigger props
    placement: placement,
    popperProps: popperProps,
    arrowPointAtCenter: arrowPointAtCenter,
    triggerId: triggerId,
    flip: flip
  };
}