import _typeof from "@babel/runtime/helpers/typeof";
import dayjs from 'dayjs';
import { useCallback, useMemo, useRef } from 'react';
var defaultFormatMap = {
  date: 'YYYY-MM-DD',
  month: 'YYYY-MM',
  year: 'YYYY'
};

function getDefaultFormat(selectType) {
  return defaultFormatMap[selectType];
}

var dateForCheck = dayjs();
export function formatIsChanged(formatA, formatB) {
  // 都不存在返回 false
  if (!formatA && !formatB) {
    return false;
  }

  if (!formatA || !formatB) {
    return true;
  }

  var typeA = _typeof(formatA);

  var typeB = _typeof(formatB);

  if (typeA !== typeB) {
    return true;
  }

  if (typeA === 'function') {
    return formatA(dateForCheck) !== formatB(dateForCheck);
  }

  return formatA !== formatB;
}
export default function useFormatDate(selectType, format) {
  var innerFormatDate = useMemo(function () {
    if (typeof format === 'function') {
      return format;
    }

    var formatStr = format || getDefaultFormat(selectType);
    return function (date) {
      return date.format(formatStr);
    };
  }, [format, selectType]);
  var finalFormatDate = useCallback(function (date) {
    if (date && date.isValid()) {
      return innerFormatDate(date);
    }

    return '';
  }, [innerFormatDate]);
  var lastSelectType = useRef(selectType);
  var lastFormat = useRef(format);
  var selectTypeHasChange = false;
  var formatHasChange = false;

  if (selectType !== lastSelectType.current) {
    lastSelectType.current = selectType;
    selectTypeHasChange = true;
  }

  if (formatIsChanged(format, lastFormat.current)) {
    lastFormat.current = format;
    formatHasChange = true;
  }

  return {
    formatDate: finalFormatDate,
    formatHasChanged: selectTypeHasChange || formatHasChange
  };
}