import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo } from 'react';
import styled from 'styled-components';
import Button from '../Button';
import CalendarFooter from '../Calendar/CalendarFooter';
import CalendarHead from '../Calendar/CalendarHead';
import CalendarPanel from '../Calendar/CalendarPanel';
import InlineButton from '../InlineButton';
import { useLocale } from '../Locale';
import TimePickerPanel from '../TimePicker/TimePickerPanel';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import useDateTimeCalendar from './useDateTimeCalendar';
var Root = styled.div.withConfig({
  displayName: "Root"
})(["background-color:", ";"], function (props) {
  return props.theme.colors.pattern.background.higher;
});
var StyledPanel = styled.div.withConfig({
  displayName: "StyledPanel"
})(["display:flex;"]);
var StyledFooter = styled.div.withConfig({
  displayName: "StyledFooter"
})(["display:flex;justify-content:space-between;align-items:center;border-top:1px solid ", ";height:", "px;"], function (props) {
  return props.theme.colors.pattern.border.normal;
}, function (props) {
  return props.theme.components.Calendar.dateTimeCalendar.footerHeight;
});
var StyledDatePanel = styled.div.withConfig({
  displayName: "StyledDatePanel"
})(["width:", "px;border-right:1px solid ", ";"], function (props) {
  return props.theme.components.Calendar.defaultWidth;
}, function (props) {
  return props.theme.colors.pattern.border.normal;
});
var StyledTimePanel = styled.div.withConfig({
  displayName: "StyledTimePanel"
})([""]);
var StyledTimeHead = styled.div.withConfig({
  displayName: "StyledTimeHead"
})(["border-bottom:1px solid ", ";height:", "px;display:flex;align-items:center;justify-content:center;margin-bottom:", "px;"], function (props) {
  return props.theme.colors.pattern.border.normal;
}, function (props) {
  return props.theme.components.Calendar.head.height;
}, function (props) {
  return props.theme.components.Calendar.dateTimeCalendar.timeHeadMarginBottom;
});
var StyledSpace = styled.span.withConfig({
  displayName: "StyledSpace"
})(["display:inline-flex;width:", "px;"], function (props) {
  return props.theme.spacing.spec.s6;
});
var StyledButtonContainer = styled.div.withConfig({
  displayName: "StyledButtonContainer"
})(["padding:0 ", "px;"], function (props) {
  return props.theme.spacing.spec.s3;
});
var defaultStyles = {
  head: '',
  panel: '',
  footer: '',
  dateTimePanel: '',
  dateCol: '',
  timeCol: '',
  timeHead: '',
  timeHeadText: '',
  timePanel: '',
  footerContainer: '',
  buttonContainer: '',
  clearButton: '',
  confirmButton: ''
};
var DateTimeCalendar = memoForwardRef(function (props, ref) {
  var _useDateTimeCalendar = useDateTimeCalendar(props),
      timePickerPanelProps = _useDateTimeCalendar.timePickerPanelProps,
      timePanelTitle = _useDateTimeCalendar.timePanelTitle,
      finalValue = _useDateTimeCalendar.finalValue,
      otherProps = _useDateTimeCalendar.otherProps,
      calendarState = _useDateTimeCalendar.calendarState,
      finalOptions = _useDateTimeCalendar.finalOptions,
      timeRowNum = _useDateTimeCalendar.timeRowNum,
      handleClear = _useDateTimeCalendar.handleClear,
      handleConfirm = _useDateTimeCalendar.handleConfirm,
      styles = _useDateTimeCalendar.styles,
      allowClear = _useDateTimeCalendar.allowClear,
      hideItemOutside = _useDateTimeCalendar.hideItemOutside,
      renderDecadeItem = _useDateTimeCalendar.renderDecadeItem,
      renderMonthItem = _useDateTimeCalendar.renderMonthItem,
      renderYearItem = _useDateTimeCalendar.renderYearItem,
      renderMonthLabel = _useDateTimeCalendar.renderMonthLabel;

  var theme = useTheme();
  var locale = useLocale();
  var pickStyles = useMemo(function () {
    if (styles) {
      var head = styles.head,
          panel = styles.panel,
          footer = styles.footer,
          dateTimePanel = styles.dateTimePanel,
          dateCol = styles.dateCol,
          timeCol = styles.timeCol,
          timeHead = styles.timeHead,
          timeHeadText = styles.timeHeadText,
          timePanel = styles.timePanel,
          footerContainer = styles.footerContainer,
          buttonContainer = styles.buttonContainer,
          clearButton = styles.clearButton,
          confirmButton = styles.confirmButton;
      return {
        head: head,
        panel: panel,
        footer: footer,
        dateTimePanel: dateTimePanel,
        dateCol: dateCol,
        timeCol: timeCol,
        timeHead: timeHead,
        timeHeadText: timeHeadText,
        timePanel: timePanel,
        footerContainer: footerContainer,
        buttonContainer: buttonContainer,
        clearButton: clearButton,
        confirmButton: confirmButton
      };
    }
  }, [styles]);
  var innerStyles = useStyles('datetime-calendar', defaultStyles, pickStyles); // 本就是解构了传给 pure component，所以不需要 memo

  var panelProps = {
    styles: styles,
    todayDate: calendarState.finalTodayDate,
    disableRange: calendarState.finalDisableRange,
    disableDate: calendarState.disableDate,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel
  };
  return /*#__PURE__*/React.createElement(Root, _extends({}, otherProps, {
    theme: theme,
    ref: ref
  }), /*#__PURE__*/React.createElement(StyledPanel, innerStyles.dateTimePanel, /*#__PURE__*/React.createElement(StyledDatePanel, _extends({
    theme: theme
  }, innerStyles.dateCol), /*#__PURE__*/React.createElement(CalendarHead, _extends({}, innerStyles.head, {
    styles: styles,
    viewType: calendarState.finalUIState.viewType,
    viewDate: calendarState.finalUIState.viewDate,
    onPrevMonth: calendarState.handlePrevMonth,
    onNextMonth: calendarState.handleNextMonth,
    onPrevYear: calendarState.handlePrevYear,
    onNextYear: calendarState.handleNextYear,
    onSwitch: calendarState.handleSwitch
  })), /*#__PURE__*/React.createElement(CalendarPanel, _extends({}, innerStyles.panel, {
    viewType: calendarState.finalUIState.viewType,
    selected: finalValue,
    viewDate: calendarState.finalUIState.viewDate,
    onItemClick: calendarState.handleItemClick
  }, panelProps))), /*#__PURE__*/React.createElement(StyledTimePanel, innerStyles.timeCol, /*#__PURE__*/React.createElement(StyledTimeHead, _extends({
    theme: theme
  }, innerStyles.timeHead), /*#__PURE__*/React.createElement(InlineButton, _extends({}, innerStyles.timeHeadText, {
    size: "s",
    component: "span",
    constant: true
  }), timePanelTitle)), /*#__PURE__*/React.createElement(TimePickerPanel, _extends({}, innerStyles.timePanel, {
    rowNum: timeRowNum || 5,
    defaultScrollBehavior: "smooth"
  }, timePickerPanelProps)))), /*#__PURE__*/React.createElement(StyledFooter, _extends({
    theme: theme
  }, innerStyles.footerContainer), /*#__PURE__*/React.createElement(CalendarFooter, _extends({}, innerStyles.footer, {
    type: "date-time",
    styles: styles,
    options: finalOptions
  })), /*#__PURE__*/React.createElement(StyledButtonContainer, _extends({
    theme: theme
  }, innerStyles.buttonContainer), allowClear && /*#__PURE__*/React.createElement(InlineButton, _extends({}, innerStyles.clearButton, {
    type: "primary",
    size: "s",
    onClick: handleClear,
    busy: !finalValue
  }), locale['Calendar.clearText']), allowClear && /*#__PURE__*/React.createElement(StyledSpace, {
    theme: theme
  }), /*#__PURE__*/React.createElement(Button, _extends({}, innerStyles.confirmButton, {
    onClick: handleConfirm,
    type: "primary",
    size: "s",
    busy: !finalValue
  }), locale['Calendar.confirmText']))));
});
export default DateTimeCalendar;