import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo } from 'react';
import usePickerBase from '../DatePicker/usePickerBase';
import { StyledPopPanel } from '../styled/components/PopPanel';
import Trigger from '../Trigger';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import DateTimeCalendar from './DateTimeCalendar';
import { StyledInput } from './styled';
import useDateTimePicker from './useDateTimePicker';
var defaultStyles = {
  popPanel: ''
};
var DateTimePicker = memoForwardRef(function (props, ref) {
  var theme = useTheme();

  var _useDateTimePicker = useDateTimePicker(props),
      finalUIState = _useDateTimePicker.finalUIState,
      calendarState = _useDateTimePicker.calendarState,
      finalValue = _useDateTimePicker.finalValue,
      otherProps = _useDateTimePicker.otherProps,
      finalTimeFormat = _useDateTimePicker.finalTimeFormat,
      handleFocus = _useDateTimePicker.handleFocus,
      handleCalendarChange = _useDateTimePicker.handleCalendarChange,
      handleCalendarUIChange = _useDateTimePicker.handleCalendarUIChange,
      handleClickAway = _useDateTimePicker.handleClickAway,
      handleClear = _useDateTimePicker.handleClear,
      handleConfirm = _useDateTimePicker.handleConfirm,
      styles = _useDateTimePicker.styles,
      disabled = _useDateTimePicker.disabled,
      allowClear = _useDateTimePicker.allowClear,
      calendarStyles = _useDateTimePicker.calendarStyles,
      disableTimeAfter = _useDateTimePicker.disableTimeAfter,
      disableTimeBefore = _useDateTimePicker.disableTimeBefore,
      defaultTime = _useDateTimePicker.defaultTime,
      todayDate = _useDateTimePicker.todayDate,
      disableRange = _useDateTimePicker.disableRange,
      disableDate = _useDateTimePicker.disableDate,
      options = _useDateTimePicker.options,
      container = _useDateTimePicker.container,
      hideItemOutside = _useDateTimePicker.hideItemOutside,
      renderDecadeItem = _useDateTimePicker.renderDecadeItem,
      renderMonthItem = _useDateTimePicker.renderMonthItem,
      renderYearItem = _useDateTimePicker.renderYearItem,
      renderMonthLabel = _useDateTimePicker.renderMonthLabel,
      timeRowNum = _useDateTimePicker.timeRowNum,
      hourStep = _useDateTimePicker.hourStep,
      minuteStep = _useDateTimePicker.minuteStep,
      secondStep = _useDateTimePicker.secondStep,
      disabledHours = _useDateTimePicker.disabledHours,
      disabledMinutes = _useDateTimePicker.disabledMinutes,
      disabledSeconds = _useDateTimePicker.disabledSeconds,
      hideDisabledOptions = _useDateTimePicker.hideDisabledOptions,
      placement = _useDateTimePicker.placement,
      popperProps = _useDateTimePicker.popperProps,
      arrowPointAtCenter = _useDateTimePicker.arrowPointAtCenter,
      triggerId = _useDateTimePicker.triggerId,
      flip = _useDateTimePicker.flip; // input 的 styles 直接传个 input


  var inputStyles = useMemo(function () {
    if (!styles) {
      return;
    }

    var inputWrapper = styles.inputWrapper,
        input = styles.input,
        nodeDivider = styles.nodeDivider,
        nodeWrapper = styles.nodeWrapper,
        clearWrapper = styles.clearWrapper,
        statusWrapper = styles.statusWrapper;
    return {
      inputWrapper: inputWrapper,
      input: input,
      nodeDivider: nodeDivider,
      nodeWrapper: nodeWrapper,
      clearWrapper: clearWrapper,
      statusWrapper: statusWrapper
    };
  }, [styles]);
  var currentStyles = useMemo(function () {
    if (!styles) {
      return;
    }

    var popPanel = styles.popPanel;
    return {
      popPanel: popPanel
    };
  }, [styles]);
  var innerStyles = useStyles('datetime-picker', defaultStyles, currentStyles);

  var _usePickerBase = usePickerBase(),
      innerSuffixNode = _usePickerBase.suffixNode;

  var realNode = /*#__PURE__*/React.createElement(StyledInput, _extends({
    ref: ref,
    value: finalValue,
    suffixNode: innerSuffixNode,
    clearReplace: true,
    focusWhenClear: false,
    theme: theme,
    styles: inputStyles,
    disabled: disabled,
    allowClear: allowClear,
    onFocus: handleFocus,
    onClear: handleClear,
    readOnly: true
  }, otherProps));

  if (disabled) {
    return realNode;
  }

  var calendarProps = {
    styles: calendarStyles,
    options: options,
    todayDate: todayDate,
    allowClear: allowClear,
    disableRange: disableRange,
    disableDate: disableDate,
    timeFormat: finalTimeFormat,
    defaultTime: defaultTime,
    disableTimeAfter: disableTimeAfter,
    disableTimeBefore: disableTimeBefore,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // ---- timepicker panel props
    timeRowNum: timeRowNum,
    hourStep: hourStep,
    minuteStep: minuteStep,
    secondStep: secondStep,
    disabledHours: disabledHours,
    disabledMinutes: disabledMinutes,
    disabledSeconds: disabledSeconds,
    hideDisabledOptions: hideDisabledOptions
  };
  var calendarNode = /*#__PURE__*/React.createElement(StyledPopPanel, _extends({}, innerStyles.popPanel, {
    theme: theme
  }), /*#__PURE__*/React.createElement(DateTimeCalendar, _extends({
    viewDate: finalUIState.viewDate,
    viewType: finalUIState.viewType,
    selectedDate: calendarState.finalValue,
    onChange: handleCalendarChange,
    onConfirm: handleConfirm,
    onUIChange: handleCalendarUIChange
  }, calendarProps)));
  var triggerProps = {
    placement: placement,
    popperProps: popperProps,
    arrowPointAtCenter: arrowPointAtCenter,
    triggerId: triggerId,
    flip: flip,
    hideArrow: true,
    container: container
  };
  return /*#__PURE__*/React.createElement(Trigger, _extends({
    popup: calendarNode,
    triggerAction: "click",
    open: finalUIState.popupOpen,
    onClickAway: handleClickAway
  }, triggerProps), realNode);
});
DateTimePicker.__MUYA_DATETIMEPICKER = true;
export default DateTimePicker;