import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["allowClear", "todayDate", "viewType", "viewDate", "disableRange", "disableDate", "options", "onUIChange", "defaultViewDate", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "timeRowNum", "hourStep", "minuteStep", "secondStep", "disabledHours", "disabledMinutes", "disabledSeconds", "hideDisabledOptions", "selectedDate", "disableTimeAfter", "disableTimeBefore", "defaultDate", "timeFormat", "defaultTime", "onChange", "onClear", "onSelect", "onConfirm", "styles"];
import dayjs from 'dayjs';
import { useMemo, useState } from 'react';
import useCalendar from '../Calendar/useCalendar';
import { formatCalendarOptions } from '../Calendar/utils';
import mergeBooleanFns from '../utils/mergeBooleanFns';
export function transformDateByTime(date, timeStr) {
  var time = timeStr === 'now' ? dayjs().format('HH:mm:ss') : timeStr;
  return dayjs("".concat(date.format('YYYY-MM-DD'), " ").concat(time));
}
export function transformDateByDate(date, otherDate) {
  return dayjs("".concat(date.format('YYYY-MM-DD'), " ").concat(otherDate.format('HH:mm:ss')));
}
/**
 * 根据禁用之前之后的时间来自定义时间的禁用
 * @param disableTimeBefore
 * @param disableTimeAfter
 */

export function getDisableDateFn(disableTimeBefore, disableTimeAfter) {
  if (!disableTimeAfter && !disableTimeBefore) {
    return;
  }

  var disableTimeAfterFn;
  var disableTimeBeforeFn;

  if (disableTimeAfter) {
    var time = dayjs(disableTimeAfter); // 注意：一定是开区间的

    disableTimeAfterFn = function disableTimeAfterFn(date) {
      return date.isAfter(time) && !date.isSame(time, 'd');
    };
  }

  if (disableTimeBefore) {
    var _time = dayjs(disableTimeBefore); // 注意：一定是开区间的


    disableTimeBeforeFn = function disableTimeBeforeFn(date) {
      return date.isBefore(_time) && !date.isSame(_time, 'd');
    };
  }

  var disableDateByTime = function disableDateByTime(date) {
    if (disableTimeAfterFn && disableTimeAfterFn(date)) {
      return true;
    }

    if (disableTimeBeforeFn && disableTimeBeforeFn(date)) {
      return true;
    }

    return false;
  };

  return disableDateByTime;
}
export default function useDateTimeCalendar(props) {
  var _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      todayDate = props.todayDate,
      viewType = props.viewType,
      viewDate = props.viewDate,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      onUIChange = props.onUIChange,
      defaultViewDate = props.defaultViewDate,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      timeRowNum = props.timeRowNum,
      hourStep = props.hourStep,
      minuteStep = props.minuteStep,
      secondStep = props.secondStep,
      disabledHours = props.disabledHours,
      disabledMinutes = props.disabledMinutes,
      disabledSeconds = props.disabledSeconds,
      hideDisabledOptions = props.hideDisabledOptions,
      selectedDate = props.selectedDate,
      disableTimeAfter = props.disableTimeAfter,
      disableTimeBefore = props.disableTimeBefore,
      defaultDate = props.defaultDate,
      _props$timeFormat = props.timeFormat,
      timeFormat = _props$timeFormat === void 0 ? 'HH:mm:ss' : _props$timeFormat,
      _props$defaultTime = props.defaultTime,
      defaultTime = _props$defaultTime === void 0 ? 'now' : _props$defaultTime,
      onChange = props.onChange,
      onClear = props.onClear,
      onSelect = props.onSelect,
      onConfirm = props.onConfirm,
      styles = props.styles,
      otherProps = _objectWithoutProperties(props, _excluded);

  var isControlled = ('selectedDate' in props);
  var valueFromProps = useMemo(function () {
    if (isControlled) {
      if (!selectedDate) {
        return;
      }

      var _innerDate = dayjs(selectedDate);

      if (_innerDate.isValid()) {
        return _innerDate;
      }

      return;
    }

    if (!defaultDate) {
      return;
    }

    var innerDate = dayjs(defaultDate);

    if (innerDate.isValid()) {
      return innerDate;
    }
  }, [defaultDate, isControlled, selectedDate]);

  var _useState = useState(valueFromProps),
      _useState2 = _slicedToArray(_useState, 2),
      stateValue = _useState2[0],
      setStateValue = _useState2[1];

  var finalValue = isControlled ? valueFromProps : stateValue;
  var memoDisableTimeBefore = useMemo(function () {
    if (disableTimeBefore) {
      var innerDate = dayjs(disableTimeBefore);
      return innerDate.isValid() ? innerDate : undefined;
    }
  }, [disableTimeBefore]);
  var memoDisableTimeAfter = useMemo(function () {
    if (disableTimeAfter) {
      var innerDate = dayjs(disableTimeAfter);
      return innerDate.isValid() ? innerDate : undefined;
    }
  }, [disableTimeAfter]);
  var disableTime = useMemo(function () {
    return getDisableDateFn(memoDisableTimeBefore, memoDisableTimeAfter);
  }, [memoDisableTimeAfter, memoDisableTimeBefore]); // disableTime 和 disableDate 同时存在的话要合并

  var finalDisableDate = useMemo(function () {
    return mergeBooleanFns(disableDate, disableTime);
  }, [disableDate, disableTime]);

  var updateValue = function updateValue(date) {
    var innerDate = date;

    if (memoDisableTimeAfter && innerDate && innerDate.isAfter(memoDisableTimeAfter)) {
      innerDate = memoDisableTimeAfter;
    }

    if (memoDisableTimeBefore && innerDate && innerDate.isBefore(memoDisableTimeBefore)) {
      innerDate = memoDisableTimeBefore;
    }

    setStateValue(innerDate);

    if (onChange) {
      onChange(innerDate);
    }

    if (innerDate && onSelect) {
      onSelect(innerDate);
    }

    if (!innerDate && onClear) {
      onClear();
    }
  };

  var handleDateChange = _useEventCallback(function (date) {
    if (!date) {
      updateValue();
      return;
    } // 如果有值，使用之前的值，如果没有使用设置的值


    var transformedDate = finalValue ? transformDateByDate(date, finalValue) : transformDateByTime(date, defaultTime);
    updateValue(transformedDate);
  });

  var handleClear = _useEventCallback(function () {
    updateValue();
  });

  var handleConfirm = _useEventCallback(function () {
    if (onConfirm && finalValue) {
      onConfirm(finalValue);
    }
  });

  var calendarProps = {
    allowClear: allowClear,
    todayDate: todayDate,
    disableRange: disableRange,
    disableDate: finalDisableDate,
    onUIChange: onUIChange,
    defaultViewDate: defaultViewDate,
    selectedDate: finalValue,
    onChange: handleDateChange
  };

  if ('viewType' in props) {
    calendarProps.viewType = viewType;
  }

  if ('viewDate' in props) {
    calendarProps.viewDate = viewDate;
  }

  var calendarState = useCalendar(calendarProps);

  var handleTimeChange = _useEventCallback(function (time) {
    updateValue(time);
  });

  var handleOptionClick = _useEventCallback(function (time) {
    calendarState.updateUIBySelectedDate(time);
    updateValue(time);
  });

  var timePickerPanelProps = {
    hourStep: hourStep,
    minuteStep: minuteStep,
    secondStep: secondStep,
    disabledHours: disabledHours,
    disabledMinutes: disabledMinutes,
    disabledSeconds: disabledSeconds,
    disableTimeAfter: memoDisableTimeAfter,
    disableTimeBefore: memoDisableTimeBefore,
    hideDisabledOptions: hideDisabledOptions,
    value: finalValue,
    onChange: handleTimeChange,
    format: timeFormat
  };
  var finalOptions = useMemo(function () {
    return formatCalendarOptions(options, handleOptionClick, function (opt) {
      if (typeof opt.date === 'function') {
        _devWarning('[DateTimeCalendar]: option.date() return invalid date.');
      } else {
        _devWarning('[DateTimeCalendar]: options passed invalid date.');
      }
    });
  }, [handleOptionClick, options]);
  return {
    calendarState: calendarState,
    otherProps: otherProps,
    finalValue: finalValue,
    timePanelTitle: finalValue ? finalValue.format(timeFormat) : '',
    timePickerPanelProps: timePickerPanelProps,
    finalOptions: finalOptions,
    timeRowNum: timeRowNum,
    // 直接透传
    styles: styles,
    allowClear: allowClear,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // 事件处理
    handleTimeChange: handleTimeChange,
    handleDateChange: handleDateChange,
    handleClear: handleClear,
    handleConfirm: handleConfirm,
    // 给其他 Hook 使用
    updateValue: updateValue,
    // for test
    handleOptionClick: handleOptionClick
  };
}