import _extends from "@babel/runtime/helpers/extends";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["todayDate", "viewType", "viewDate", "disableRange", "disableDate", "options", "defaultViewDate", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "getPopupContainer", "timeRowNum", "hourStep", "minuteStep", "secondStep", "disabledHours", "disabledMinutes", "disabledSeconds", "hideDisabledOptions", "disabled", "onFocus", "onClear", "defaultValue", "value", "format", "timeFormat", "allowClear", "popupOpen", "onConfirm", "onChange", "onUIChange", "onCalendarChange", "suffixNode", "placement", "triggerId", "popperProps", "flip", "arrowPointAtCenter", "calendarStyles", "styles", "defaultTime", "disableTimeAfter", "disableTimeBefore"];
import dayjs from 'dayjs';
import { useEffect, useMemo, useState } from 'react';
import forkHandler from '../utils/forkHandler';
import useDateTimeCalendar from './useDateTimeCalendar';
var defaultDateTimeFormat = 'YYYY-MM-DD HH:mm:ss';
var HReg = /H+/;
var HmReg = /H[^m]+m+/;
var HmsReg = /H[^s]+s+/;
export function getTimeFormatByFormat(format) {
  if (format === defaultDateTimeFormat || typeof format === 'function') {
    return 'HH:mm:ss';
  }

  var hmsResult = format.match(HmsReg);

  if (hmsResult && hmsResult[0]) {
    return hmsResult[0];
  }

  var hmResult = format.match(HmReg);

  if (hmResult && hmResult[0]) {
    return hmResult[0];
  }

  var hResult = format.match(HReg);

  if (hResult && hResult[0]) {
    return hResult[0];
  }

  return 'HH:mm:ss';
}
export default function useDateTimePicker(props) {
  var todayDate = props.todayDate,
      viewType = props.viewType,
      viewDate = props.viewDate,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      defaultViewDate = props.defaultViewDate,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      getPopupContainer = props.getPopupContainer,
      timeRowNum = props.timeRowNum,
      hourStep = props.hourStep,
      minuteStep = props.minuteStep,
      secondStep = props.secondStep,
      disabledHours = props.disabledHours,
      disabledMinutes = props.disabledMinutes,
      disabledSeconds = props.disabledSeconds,
      hideDisabledOptions = props.hideDisabledOptions,
      disabled = props.disabled,
      onFocus = props.onFocus,
      onClear = props.onClear,
      defaultValue = props.defaultValue,
      value = props.value,
      _props$format = props.format,
      format = _props$format === void 0 ? defaultDateTimeFormat : _props$format,
      timeFormat = props.timeFormat,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      popupOpen = props.popupOpen,
      onConfirm = props.onConfirm,
      onChange = props.onChange,
      onUIChange = props.onUIChange,
      onCalendarChange = props.onCalendarChange,
      suffixNode = props.suffixNode,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      triggerId = props.triggerId,
      popperProps = props.popperProps,
      flip = props.flip,
      arrowPointAtCenter = props.arrowPointAtCenter,
      calendarStyles = props.calendarStyles,
      styles = props.styles,
      defaultTime = props.defaultTime,
      disableTimeAfter = props.disableTimeAfter,
      disableTimeBefore = props.disableTimeBefore,
      otherProps = _objectWithoutProperties(props, _excluded);

  var isControlled = ('value' in props);
  var finalTimeFormat = timeFormat || getTimeFormatByFormat(format);
  var valueFromProps = useMemo(function () {
    if (isControlled) {
      if (!value) {
        return;
      }

      var _innerDate = dayjs(value);

      if (_innerDate.isValid()) {
        return _innerDate;
      }

      return;
    }

    if (!defaultValue) {
      return;
    }

    var innerDate = dayjs(defaultValue);

    if (innerDate.isValid()) {
      return innerDate;
    }
  }, [defaultValue, isControlled, value]);
  var formatDateStr = useMemo(function () {
    var tplFn;

    if (typeof format === 'function') {
      tplFn = format;
    } else {
      tplFn = function tplFn(date) {
        return date.format(format);
      };
    }

    return function (date) {
      if (date) {
        return tplFn(date);
      }

      return '';
    };
  }, [format]);

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      statePopupOpen = _useState2[0],
      setPopupOpen = _useState2[1]; // const [stateValue, setStateValue] = useState(formatDateStr(valuePropWeek));


  var finalPopupOpen = popupOpen !== undefined ? popupOpen : statePopupOpen;

  var _useState3 = useState(valueFromProps),
      _useState4 = _slicedToArray(_useState3, 2),
      calendarDate = _useState4[0],
      setCalendarDate = _useState4[1];

  var _useState5 = useState(valueFromProps),
      _useState6 = _slicedToArray(_useState5, 2),
      inputDate = _useState6[0],
      setInputDate = _useState6[1];

  var finalInputDate = isControlled ? valueFromProps : inputDate;
  var calendarProps = {
    allowClear: allowClear,
    todayDate: todayDate,
    disableRange: disableRange,
    disableDate: disableDate,
    options: options,
    defaultViewDate: defaultViewDate,
    defaultDate: defaultValue,
    selectedDate: calendarDate,
    defaultTime: defaultTime
  };

  if ('viewType' in props) {
    calendarProps.viewType = viewType;
  }

  if ('viewDate' in props) {
    calendarProps.viewDate = viewDate;
  }

  calendarProps.onUIChange = _useEventCallback(function (inputState) {
    var newState = _extends({
      popupOpen: finalPopupOpen
    }, inputState);

    if (onUIChange) {
      onUIChange(newState);
    }
  });
  calendarProps.onChange = _useEventCallback(function (date) {
    setCalendarDate(date);

    if (!date) {
      setInputDate(undefined);

      if (onChange) {
        onChange(dayjs(''));
      }
    }
  });
  /**
   * 这里复用 hooks 的逻辑需要注意的点：
   * * 事件处理一定要在这个 hook 里处理
   */

  var calendarState = useDateTimeCalendar(calendarProps);

  var finalUIState = _extends({}, calendarState.calendarState.finalUIState, {
    popupOpen: finalPopupOpen
  });

  var finalValue = finalPopupOpen ? formatDateStr(calendarDate) : formatDateStr(finalInputDate); // 切换日历面板的显示隐藏

  var updateUI = function updateUI(innerPopupOpen) {
    var newState = _extends({}, calendarState.calendarState.finalUIState, {
      popupOpen: innerPopupOpen
    });

    setPopupOpen(innerPopupOpen);

    if (onUIChange) {
      onUIChange(newState);
    }
  };
  /**
   * 关闭日历面板
   */


  function closePopup() {
    if (!finalUIState.popupOpen) {
      return;
    }

    updateUI(false);
  }

  var handleCalendarUIChange = _useEventCallback(function (inputState) {
    calendarState.calendarState.updateUI(inputState);
  });

  var handleCalendarChange = _useEventCallback(forkHandler(function (date) {
    calendarState.updateValue(date);
  }, onCalendarChange));

  var handleFocus = _useEventCallback(forkHandler(function () {
    updateUI(true);
  }, onFocus));

  var handleClear = _useEventCallback(forkHandler(function () {
    calendarState.updateValue();
  }, onClear));

  var handleClickAway = _useEventCallback(function () {
    if (finalPopupOpen) {
      closePopup();
    }
  });

  var handleConfirm = _useEventCallback(function (date) {
    setInputDate(date);
    closePopup();

    if (onConfirm) {
      onConfirm(date);
    }

    if (onChange) {
      onChange(date);
    }
  });

  useEffect(function () {
    if (isControlled) {
      setCalendarDate(valueFromProps);
    }
  }, [isControlled, valueFromProps]);
  return {
    // 处理后的数据
    finalUIState: finalUIState,
    finalInputDate: finalInputDate,
    calendarDate: calendarDate,
    calendarState: calendarState,
    finalValue: finalValue,
    otherProps: otherProps,
    finalTimeFormat: finalTimeFormat,
    // 事件处理
    handleFocus: handleFocus,
    handleCalendarChange: handleCalendarChange,
    handleCalendarUIChange: handleCalendarUIChange,
    handleClickAway: handleClickAway,
    handleClear: handleClear,
    handleConfirm: handleConfirm,
    // 直接透传
    styles: styles,
    disabled: disabled,
    allowClear: allowClear,
    calendarStyles: calendarStyles,
    defaultTime: defaultTime,
    container: getPopupContainer,
    // ---- calendar props
    todayDate: todayDate,
    disableRange: disableRange,
    disableDate: disableDate,
    options: options,
    disableTimeAfter: disableTimeAfter,
    disableTimeBefore: disableTimeBefore,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // ---- timepicker panel props
    timeRowNum: timeRowNum,
    hourStep: hourStep,
    minuteStep: minuteStep,
    secondStep: secondStep,
    disabledHours: disabledHours,
    disabledMinutes: disabledMinutes,
    disabledSeconds: disabledSeconds,
    hideDisabledOptions: hideDisabledOptions,
    // triggerProps
    placement: placement,
    popperProps: popperProps,
    arrowPointAtCenter: arrowPointAtCenter,
    triggerId: triggerId,
    flip: flip
  };
}