import { muyaThemeLight as _muyaThemeLight } from "@muya-ui/theme-light/es/theme";
import { CloseIcon as _CloseIcon } from "@muya-ui/theme-light/es/icons/CloseIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _remove from "lodash/remove";
var _excluded = ["confirmText", "cancelText", "confirmButtonType", "onConfirm", "onCancel", "title", "text", "icon", "type", "size", "open", "styles", "onClose", "closeIcon", "hideClose", "tipAction", "confirmButtonProps", "cancelButtonProps", "closeFn"],
    _excluded2 = ["theme", "locale", "styleSheet"];
import React, { useCallback } from 'react';
import ReactDOM from 'react-dom';
import styled, { css, ThemeProvider, StyleSheetManager } from 'styled-components';
import LocaleContext from '../Locale/LocaleContext';
import useLocale from '../Locale/useLocale';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import Actions from './Actions';
import { AlertActionButton, AlertActionInlineButton } from './AlertActionButton';
import Base from './Base';
import Content from './Content';
var StyledActions = styled(Actions).withConfig({
  displayName: "StyledActions"
})(["", ""], function (props) {
  return css(["padding:", ";"], props.theme.components.Dialog.actions.paddingForAlert);
});
var StyledIconWrapper = styled(AlertActionInlineButton).withConfig({
  displayName: "StyledIconWrapper"
})(["&&{position:absolute;right:", "px;top:", "px;padding:0;}"], function (props) {
  return props.theme.components.Dialog.title.closeIconRight;
}, function (props) {
  return props.theme.components.Dialog.title.closeIconTop;
});
var defaultStyles = {
  content: '',
  actions: ''
};
var Alert = memoForwardRef(function (props, ref) {
  var local = useLocale();

  var _props$confirmText = props.confirmText,
      confirmText = _props$confirmText === void 0 ? local['Modal.okText'] : _props$confirmText,
      cancelText = props.cancelText,
      _props$confirmButtonT = props.confirmButtonType,
      confirmButtonType = _props$confirmButtonT === void 0 ? 'primary' : _props$confirmButtonT,
      onConfirm = props.onConfirm,
      onCancel = props.onCancel,
      title = props.title,
      text = props.text,
      icon = props.icon,
      _props$type = props.type,
      type = _props$type === void 0 ? 'info' : _props$type,
      _props$size = props.size,
      size = _props$size === void 0 ? 's' : _props$size,
      open = props.open,
      styles = props.styles,
      onClose = props.onClose,
      closeIcon = props.closeIcon,
      _props$hideClose = props.hideClose,
      hideClose = _props$hideClose === void 0 ? true : _props$hideClose,
      tipAction = props.tipAction,
      confirmButtonProps = props.confirmButtonProps,
      cancelButtonProps = props.cancelButtonProps,
      closeFn = props.closeFn,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('dialog-alert', defaultStyles, styles);
  var handleClose = useCallback(function (e) {
    if (!onClose) {
      closeFn && closeFn();
      return;
    }

    var res = onClose(e);
    if (!res) return;

    if (res.then) {
      res.then(function () {
        if ((arguments.length <= 0 ? undefined : arguments[0]) !== false) {
          closeFn && closeFn();
        }
      }, function (e) {
        console.error(e);
      });
    } else {
      closeFn && closeFn();
    }
  }, [closeFn, onClose]);
  var confirmButton = confirmText ? /*#__PURE__*/React.createElement(AlertActionButton, _extends({
    closeAlert: closeFn,
    type: confirmButtonType,
    actionFn: onConfirm,
    open: open
  }, confirmButtonProps), confirmText) : null;
  var cancelButton = cancelText ? /*#__PURE__*/React.createElement(AlertActionButton, _extends({
    plain: true,
    closeAlert: closeFn,
    actionFn: onCancel,
    open: open
  }, cancelButtonProps), cancelText) : null;
  var Close = theme.components.Dialog.title.closeIcon || _CloseIcon;
  return /*#__PURE__*/React.createElement(Base, _extends({
    ref: ref,
    open: open,
    size: size,
    onClose: handleClose
  }, other), /*#__PURE__*/React.createElement(Content, _extends({}, innerStyles.content, {
    type: type,
    title: title,
    text: text,
    icon: icon,
    size: size
  })), /*#__PURE__*/React.createElement(StyledActions, _extends({}, innerStyles.actions, {
    theme: theme,
    tipAction: tipAction
  }), cancelButton, confirmButton), !hideClose && /*#__PURE__*/React.createElement(StyledIconWrapper, {
    closeAlert: closeFn,
    actionFn: onClose,
    size: size,
    theme: theme,
    open: open,
    type: "weak"
  }, closeIcon || /*#__PURE__*/React.createElement(Close, null)));
});
var dialogItemList = [];
var defaultConfig = {
  theme: _muyaThemeLight
};

var alert = function alert(config) {
  var div = document.createElement('div');

  var mergedConfig = _extends({}, defaultConfig, config);

  var currentConfig = mergedConfig;

  function render(props) {
    var innerTheme = props.theme,
        locale = props.locale,
        styleSheet = props.styleSheet,
        other = _objectWithoutProperties(props, _excluded2);

    var children = /*#__PURE__*/React.createElement(ThemeProvider, {
      theme: innerTheme
    }, /*#__PURE__*/React.createElement(Alert, _extends({
      open: true,
      closeFn: close,
      onExited: destroy
    }, other)));

    if (styleSheet) {
      children = /*#__PURE__*/React.createElement(StyleSheetManager, {
        target: styleSheet
      }, /*#__PURE__*/React.createElement(ThemeProvider, {
        theme: innerTheme
      }, /*#__PURE__*/React.createElement(Alert, _extends({
        open: true,
        closeFn: close,
        onExited: destroy
      }, other))));
    }

    if (locale) {
      ReactDOM.render( /*#__PURE__*/React.createElement(LocaleContext.Provider, {
        value: locale
      }, children), div);
      return;
    }

    ReactDOM.render(children, div);
  }

  function close() {
    currentConfig = _extends({}, currentConfig, {
      open: false
    });
    render(currentConfig);
  }

  function open() {
    currentConfig = _extends({}, currentConfig, {
      open: true
    });
    render(currentConfig);
  }

  function destroy() {
    var noCache = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

    if (noCache === true) {
      _remove(dialogItemList, item);
    } else {
      currentConfig = _extends({}, currentConfig, {
        open: false
      });
    }

    return ReactDOM.unmountComponentAtNode(div);
  }

  function update(newConfig) {
    currentConfig = _extends({}, currentConfig, newConfig);
    render(currentConfig);
  }

  render(currentConfig);
  var item = {
    update: update,
    close: close,
    open: open,
    destroy: destroy,
    getConfig: function getConfig() {
      return currentConfig;
    }
  };
  dialogItemList.push(item);
  return item;
};
/**
 *
 * 修改配置
 * @param {Partial<IDialogAlertProps>} newConfig
 */


var config = function config(newConfig) {
  // 首先修改默认配置，后续创建的实例都会使用新的默认配置
  defaultConfig = _extends({}, defaultConfig, newConfig); // 已经创建了的实例，也需要更新配置

  dialogItemList.forEach(function (item) {
    return item.update(newConfig);
  });
};

var success = function success(props) {
  return alert(_extends({
    type: 'success'
  }, props));
};

var info = function info(props) {
  return alert(_extends({
    type: 'info'
  }, props));
};

var warning = function warning(props) {
  return alert(_extends({
    type: 'warning'
  }, props));
};

var error = function error(props) {
  return alert(_extends({
    type: 'error'
  }, props));
};

export default Alert;
export { alert, config, success, info, warning, error };