import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
var _excluded = ["actionFn", "closeAlert", "loading", "onClick", "open"];
import React, { useCallback, useState } from 'react';
import { Button, InlineButton } from '../Button';

var useActionButton = function useActionButton(props) {
  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      loadingState = _useState2[0],
      setLoading = _useState2[1];

  var actionFn = props.actionFn,
      closeAlert = props.closeAlert,
      loadingProp = props.loading,
      onClick = props.onClick,
      open = props.open,
      other = _objectWithoutProperties(props, _excluded);

  var loading = 'loading' in props ? loadingProp : loadingState;
  var handleClick = useCallback(function (e) {
    // 防止重复点击，当弹窗被关闭，动画执行过程中，不执行以下动作
    if (!open) return; // 首先调用外部传入的onClick

    if (onClick) {
      onClick(e);
    } // 没有传入actionFn，关闭弹窗


    if (!actionFn) {
      closeAlert && closeAlert();
      return;
    }

    var res = actionFn(e); // 返回falsy不关闭

    if (!res) return; // Promise resolve后关闭

    if (res.then) {
      setLoading(true);
      res.then(function () {
        setLoading(false);

        if ((arguments.length <= 0 ? undefined : arguments[0]) !== false) {
          closeAlert && closeAlert.apply(void 0, arguments);
        }
      }, function (e) {
        // Emit error when catch promise reject
        console.error(e);
        setLoading(false);
      });
    } else {
      closeAlert && closeAlert(e);
    }
  }, [actionFn, closeAlert, onClick, open]);
  return _extends({}, other, {
    onClick: handleClick,
    loading: loading
  });
};

var AlertActionButton = /*#__PURE__*/React.memo(function (props) {
  var buttonProps = useActionButton(props);
  return /*#__PURE__*/React.createElement(Button, buttonProps);
});
var AlertActionInlineButton = /*#__PURE__*/React.memo(function (props) {
  var buttonProps = useActionButton(props);
  return /*#__PURE__*/React.createElement(InlineButton, buttonProps);
});
export { AlertActionButton, AlertActionInlineButton };