import _extends from "@babel/runtime/helpers/extends";
import { useLockScroll as _useLockScroll } from "@muya-ui/utils/es/hooks/useLockScroll";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["disablePortal", "open", "container", "children", "onRendered", "disableEscapeKeyDown", "disableMaskClick", "onClose", "onMaskClick", "onEscapeKeyDown", "hideMask", "onEntered", "onExited", "lazyMount", "destroyOnClose", "size", "fullWidth", "fullScreen", "disableSize", "dialogContainerProps", "customDialogContainer", "width", "height", "maxHeight", "zIndex", "rootRef"];
import React, { useCallback, useMemo, useRef, useState } from 'react';
import Animation from '../Animation';
import Portal, { portalUtils } from '../Portal';
import addPx from '../utils/addPx';
import forkHandler from '../utils/forkHandler';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import DialogContainer from './StyledContainer';
import DialogRoot from './StyledRoot';
export default memoForwardRef(function (props, ref) {
  var disablePortal = props.disablePortal,
      _props$open = props.open,
      open = _props$open === void 0 ? false : _props$open,
      container = props.container,
      children = props.children,
      onRendered = props.onRendered,
      disableEscapeKeyDown = props.disableEscapeKeyDown,
      disableMaskClick = props.disableMaskClick,
      onClose = props.onClose,
      onMaskClick = props.onMaskClick,
      onEscapeKeyDown = props.onEscapeKeyDown,
      hideMask = props.hideMask,
      onEntered = props.onEntered,
      onExited = props.onExited,
      _props$lazyMount = props.lazyMount,
      lazyMount = _props$lazyMount === void 0 ? false : _props$lazyMount,
      destroyOnClose = props.destroyOnClose,
      size = props.size,
      fullWidth = props.fullWidth,
      fullScreen = props.fullScreen,
      disableSize = props.disableSize,
      dialogContainerProps = props.dialogContainerProps,
      customDialogContainer = props.customDialogContainer,
      width = props.width,
      height = props.height,
      maxHeight = props.maxHeight,
      zIndex = props.zIndex,
      rootRef = props.rootRef,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();

  var _useState = useState(!open),
      _useState2 = _slicedToArray(_useState, 2),
      exited = _useState2[0],
      setExited = _useState2[1];

  var _useState3 = useState(),
      _useState4 = _slicedToArray(_useState3, 2),
      mouseDownNode = _useState4[0],
      setMouseDownNode = _useState4[1];

  var dialogRef = useRef(null);

  var handleRootRef = _useForkRef(dialogRef, rootRef);

  var firstMountRef = useRef(false);

  var handleMaskClick = _useEventCallback(function (event) {
    // 如果鼠标按下时的元素和当前元素不是一个东西，阻止弹窗关闭
    if (event.target !== mouseDownNode) {
      return;
    }

    if (event.target !== event.currentTarget) {
      return;
    } // 弹窗点击蒙层时的回调


    if (onMaskClick) {
      onMaskClick(event);
    }

    if (!disableMaskClick && onClose) {
      onClose(event, 'maskClick');
    }
  }, []);

  var handleKeyDown = _useEventCallback(function (event) {
    // We don't take event.defaultPrevented into account:
    //
    // event.preventDefault() is meant to stop default behaviours like
    // clicking a checkbox to check it, hitting a button to submit a form,
    // and hitting left arrow to move the cursor in a text input etc.
    // Only special HTML elements have these default behaviors.
    if (event.key !== 'Escape') {
      return;
    }

    event.stopPropagation(); // 按下ESC键触发的回调

    if (onEscapeKeyDown) {
      onEscapeKeyDown(event);
    }

    if (!disableEscapeKeyDown && onClose) {
      onClose(event, 'escapeKeyDown');
    }
  }, []);

  var handleEntered = useMemo(function () {
    return forkHandler(function () {
      if (open && dialogRef.current) {
        // 禁用ESC键时，不需要focus
        !disableEscapeKeyDown && dialogRef.current.focus();
      }
    }, onEntered);
  }, [disableEscapeKeyDown, onEntered, open]);
  var handleExited = useMemo(function () {
    return forkHandler(function () {
      setExited(true);
    }, onExited);
  }, [onExited]);
  var handleEnter = useCallback(function () {
    setExited(false);
  }, []);

  var handleMouseDown = _useEventCallback(function (event) {
    setMouseDownNode(event.target);
  }, []);
  /**
   * effect: dialog出现时隐藏container的滚动条
   * 1. 当前动画处于非exited状态，隐藏container的滚动条
   * 2. 此处使用exited，是希望关闭弹窗时，弹窗动画执行完毕之后，再隐藏滚动条
   * 3. 如果不使用exited，直接使用open，弹窗动画未执行完毕时，滚动条就隐藏了，此时页面会有跳动的感觉
   */


  _useLockScroll(!exited, portalUtils.getPortalContainer(container));

  var childrenWithProps = useMemo(function () {
    return React.Children.map(children, function (child) {
      var props = /*#__PURE__*/React.isValidElement(child) && typeof child.type === 'string' ? {} : {
        fullWidth: fullWidth
      };
      return child && /*#__PURE__*/React.cloneElement(child, props);
    });
  }, [children, fullWidth]);
  var dialogContainer = useMemo(function () {
    return customDialogContainer || /*#__PURE__*/React.createElement(DialogContainer, _extends({
      theme: theme,
      $size: size,
      $fullScreen: fullScreen,
      $fullWidth: fullWidth,
      $disableSize: disableSize,
      $width: addPx(width),
      $height: addPx(height),
      $maxHeight: addPx(maxHeight),
      ref: ref
    }, dialogContainerProps), typeof children === 'string' ? children : childrenWithProps);
  }, [children, childrenWithProps, customDialogContainer, dialogContainerProps, disableSize, fullScreen, fullWidth, height, maxHeight, ref, size, theme, width]);
  var rootChildren = useMemo(function () {
    // destroyOnClose逻辑，在弹窗关闭并且动画执行完毕之后，才会渲染null
    if (destroyOnClose && !open && exited) {
      return null;
    } // lazyMount开启 && 初次渲染，执行懒加载，不渲染子节点


    if (lazyMount && !firstMountRef.current && !open && exited) {
      return null;
    }

    return dialogContainer;
  }, [destroyOnClose, dialogContainer, exited, lazyMount, open]); // 第一次渲染后，将firstMountRef设置为true

  if (!firstMountRef.current) {
    firstMountRef.current = true;
  }

  return /*#__PURE__*/React.createElement(Portal, {
    disablePortal: disablePortal,
    container: container,
    onRendered: onRendered
  }, /*#__PURE__*/React.createElement(Animation.Fade, {
    "in": open,
    onEntered: handleEntered,
    onExited: handleExited,
    onEnter: handleEnter
  }, /*#__PURE__*/React.createElement(DialogRoot, _extends({
    theme: theme,
    role: "presentation",
    ref: handleRootRef,
    onKeyDown: handleKeyDown,
    tabIndex: -1,
    onClick: handleMaskClick,
    onMouseDown: handleMouseDown,
    $hideMask: hideMask,
    $container: portalUtils.getPortalContainer(container),
    $zIndex: zIndex
  }, other), rootChildren)));
});