import { ClearIcon as _ClearIcon } from "@muya-ui/theme-light/es/icons/ClearIcon";
import { ReminderIcon as _ReminderIcon } from "@muya-ui/theme-light/es/icons/ReminderIcon";
import { InformIcon as _InformIcon } from "@muya-ui/theme-light/es/icons/InformIcon";
import { SuccessIcon as _SuccessIcon } from "@muya-ui/theme-light/es/icons/SuccessIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["children", "hideDividers", "title", "text", "type", "icon", "fullWidth", "size"];
import React from 'react';
import styled, { css } from 'styled-components';
import Typography from '../Typography';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';

function makeContentStyle(props) {
  var token = props.theme.components.Dialog;
  var content = token.content;
  var borderStyle;
  var paddingStyle = css(["padding:", " ", " ", ";"], content.paddingTop, content.paddingHorizontal, content.paddingBottom);

  if (!props.$hideDividers) {
    borderStyle = css(["border-bottom:1px solid ", ";"], content.borderColor);
  }

  if (props.$type) {
    paddingStyle = css(["padding:", ";"], content.paddingWithType);
  }

  if (props.$fullWidth) {
    paddingStyle = css(["padding:", " ", " ", ";"], content.paddingTop, content.fullWidthPaddingHorizontal, content.paddingBottom);
  }

  return css(["flex:1;overflow-y:auto;box-sizing:border-box;", " ", ""], borderStyle, paddingStyle);
}

var StyledDialogContent = styled.div.withConfig({
  displayName: "StyledDialogContent"
})(["", ""], makeContentStyle);
var StyledContentTitleWrapper = styled(Typography.Title).withConfig({
  displayName: "StyledContentTitleWrapper"
})(["position:relative;margin-bottom:", "px;"], function (props) {
  return props.theme.components.Dialog.content.titleMarginBottom;
});
var StyledIconWrapper = styled.span.withConfig({
  displayName: "StyledIconWrapper"
})(["position:absolute;top:", "px;left:-", "px;transform:translateX(-100%);color:", ";"], function (props) {
  return props.theme.components.Dialog.content.iconMarginTop;
}, function (props) {
  return props.theme.components.Dialog.content.iconMarginLeft;
}, function (props) {
  return props.$type && props.theme.colors.pattern.feature[props.$type];
});
export default memoForwardRef(function (props, ref) {
  var children = props.children,
      hideDividers = props.hideDividers,
      title = props.title,
      text = props.text,
      type = props.type,
      icon = props.icon,
      fullWidth = props.fullWidth,
      size = props.size,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();

  if (!type) {
    return /*#__PURE__*/React.createElement(StyledDialogContent, _extends({
      ref: ref,
      theme: theme,
      $hideDividers: hideDividers,
      $type: type,
      $fullWidth: fullWidth
    }, other), children);
  }

  var typeIcon = _extends({
    success: _SuccessIcon,
    info: _InformIcon,
    warning: _ReminderIcon,
    error: _ClearIcon
  }, theme.components.Dialog.content.typeIcon);

  var _theme$components$Dia = theme.components.Dialog.content,
      typeIconBgColor = _theme$components$Dia.typeIconBgColor,
      defaultTitleLevel = _theme$components$Dia.defaultTitleLevel;
  var Icon = typeIcon[type];
  var iconBgColor = typeIconBgColor[type];
  return /*#__PURE__*/React.createElement(StyledDialogContent, _extends({
    ref: ref,
    theme: theme,
    $type: type,
    $fullWidth: fullWidth,
    $hideDividers: true
  }, other), /*#__PURE__*/React.createElement(StyledContentTitleWrapper, {
    theme: theme,
    level: defaultTitleLevel
  }, /*#__PURE__*/React.createElement(StyledIconWrapper, {
    theme: theme,
    $type: type
  }, icon || /*#__PURE__*/React.createElement(Icon, {
    bgColor: iconBgColor
  })), title), /*#__PURE__*/React.createElement(Typography.Paragraph, {
    as: "div",
    fontSize: "s2",
    color: "assistant"
  }, text));
});