import _extends from "@babel/runtime/helpers/extends";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["hideArrow", "placement", "visible", "defaultVisible", "disabled", "overlay", "size", "onOverlayClick", "TransitionComponent", "onVisibleChange", "children", "getPopupContainer"];
import React, { useCallback, useMemo, useRef, useState } from 'react';
import Animation from '../Animation';
import useTheme from '../utils/useTheme';
export default function useDropdown(props, ref) {
  var _props$hideArrow = props.hideArrow,
      hideArrow = _props$hideArrow === void 0 ? true : _props$hideArrow,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      visible = props.visible,
      _props$defaultVisible = props.defaultVisible,
      defaultVisible = _props$defaultVisible === void 0 ? false : _props$defaultVisible,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      overlay = props.overlay,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      onOverlayClick = props.onOverlayClick,
      _props$TransitionComp = props.TransitionComponent,
      TransitionComponent = _props$TransitionComp === void 0 ? Animation.Slide : _props$TransitionComp,
      onVisibleChangeProp = props.onVisibleChange,
      children = props.children,
      getPopupContainer = props.getPopupContainer,
      restProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var shadows = theme.shadows,
      DropdownToken = theme.components.Dropdown;

  var _useState = useState(defaultVisible),
      _useState2 = _slicedToArray(_useState, 2),
      openState = _useState2[0],
      setOpenState = _useState2[1];

  var child = useMemo(function () {
    return React.Children.only(children);
  }, [children]);
  var triggerRef = useRef(null);

  var handleRef = _useForkRef(triggerRef, child.ref);

  var _useRef = useRef('visible' in props),
      isControlled = _useRef.current;

  React.useImperativeHandle(ref, function () {
    return triggerRef.current;
  }, [triggerRef]);

  if (isControlled && openState !== visible) {
    setOpenState(visible);
  }

  var onVisibleChange = useCallback(function (visible) {
    if (!isControlled) {
      setOpenState(visible);
    }

    if (onVisibleChangeProp) {
      onVisibleChangeProp(visible);
    }
  }, [isControlled, onVisibleChangeProp]);
  var getOverlayElement = useCallback(function () {
    var overlayElement;

    if (typeof overlay === 'function') {
      overlayElement = overlay();
    } else {
      overlayElement = overlay;
    }

    overlayElement = React.Children.only(overlayElement);
    return overlayElement;
  }, [overlay]);

  var onClick = _useEventCallback(function (selectInfo) {
    var overlayProps = getOverlayElement().props;

    if (!isControlled) {
      setOpenState(false);
    }

    if (onOverlayClick) {
      onOverlayClick(selectInfo);
    }

    if (overlayProps.onClick) {
      overlayProps.onClick(selectInfo);
    }
  }, []);

  var menu = useMemo(function () {
    var overlayElement = getOverlayElement();
    var extraOverlayProps = {
      size: size,
      hideRootMenuSpacing: true,
      onClick: onClick,
      style: _extends({}, overlayElement.props.style, {
        boxShadow: shadows.pattern.popper.normal
      })
    };

    if (DropdownToken.menuItemAlign) {
      extraOverlayProps.menuItemAlign = DropdownToken.menuItemAlign;
    }

    return /*#__PURE__*/React.cloneElement(overlayElement, extraOverlayProps);
  }, [DropdownToken.menuItemAlign, getOverlayElement, onClick, shadows.pattern.popper.normal, size]);
  var dropdownTrigger = useMemo(function () {
    return /*#__PURE__*/React.cloneElement(child, {
      ref: handleRef,
      disabled: disabled
    });
  }, [child, disabled, handleRef]);
  var arrowStyle = useMemo(function () {
    return {
      color: theme.colors.pattern.background.higher
    };
  }, [theme.colors.pattern.background.higher]);
  var open = isControlled ? visible : openState;
  return _extends({}, restProps, {
    container: getPopupContainer,
    onVisibleChange: onVisibleChange,
    TransitionComponent: TransitionComponent,
    placement: placement,
    hideArrow: hideArrow,
    popup: menu,
    children: dropdownTrigger,
    arrowStyle: arrowStyle,
    offset: DropdownToken.offset[size],
    open: disabled ? false : open
  });
}