import _extends from "@babel/runtime/helpers/extends";
import React from 'react';
import { FormConsumer, FormProvider, useFormContext } from './FormContext';
import FormItem from './FormItem';
import useForm from './useForm';

function Form(props) {
  var children = props.children,
      formBagRef = props.formBagRef,
      _props$independent = props.independent,
      independent = _props$independent === void 0 ? false : _props$independent;
  var innerFormBag = useForm(props);
  var formBagFromContext = useFormContext(); // 如果Form被嵌套在另一个Form里面, 并且没有设置independent，此时使用外层Form的数据

  var isNestedForm = !!formBagFromContext && !independent;
  var formBag = isNestedForm ? formBagFromContext : innerFormBag;
  var finalChildren = typeof children === 'function' ? children(formBag) : children; // 通过Ref将formBag传递出去

  React.useImperativeHandle(formBagRef, function () {
    return formBag;
  }); // Form嵌套情况，不需要渲染Provider和form标签

  if (isNestedForm) {
    return finalChildren;
  }

  var handleReset = formBag.handleReset,
      handleSubmit = formBag.handleSubmit,
      formProps = formBag.formProps,
      style = formBag.style,
      className = formBag.className,
      _formBag$autoComplete = formBag.autoComplete,
      autoComplete = _formBag$autoComplete === void 0 ? 'off' : _formBag$autoComplete;
  return /*#__PURE__*/React.createElement(FormProvider, {
    value: formBag
  }, /*#__PURE__*/React.createElement("form", _extends({
    onSubmit: handleSubmit,
    onReset: handleReset,
    style: style,
    className: className,
    autoComplete: autoComplete
  }, formProps), finalChildren));
}

Form.Item = FormItem;
Form.useForm = useForm;
Form.Provider = FormProvider;
Form.Consumer = FormConsumer;
Form.useFormContext = useFormContext;
export default Form;