import { useEffectOnce as _useEffectOnce } from "@muya-ui/utils/es/hooks/useEffectOnce";
import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _extends from "@babel/runtime/helpers/extends";
import _isFunction from "lodash/isFunction";
import _isEmpty from "lodash/isEmpty";
import _isBoolean from "lodash/isBoolean";
import _hasIn from "lodash/hasIn";
import Schema from 'async-validator';
import deepmerge from 'deepmerge';
import { useCallback, useReducer, useRef } from 'react';
import scrollIntoView from '../utils/scrollIntoView';
import { getIn, overwriteArrayMerge, setIn, setNestedObjectValues } from './utils';

// State reducer
function formReducer(state, msg) {
  switch (msg.type) {
    case 'SET_VALUES':
      return _extends({}, state, {
        values: msg.payload
      });

    case 'SET_TOUCHED':
      return _extends({}, state, {
        touched: msg.payload
      });

    case 'SET_ERRORS':
      return _extends({}, state, {
        errors: msg.payload
      });

    case 'SET_STATUS':
      return _extends({}, state, {
        status: msg.payload
      });

    case 'SET_ISSUBMITTING':
      return _extends({}, state, {
        isSubmitting: msg.payload
      });

    case 'SET_ISVALIDATING':
      return _extends({}, state, {
        isValidating: msg.payload
      });

    case 'SET_FIELD_VALUE':
      return _extends({}, state, {
        values: setIn(state.values, msg.payload.field, msg.payload.value)
      });

    case 'SET_FIELD_TOUCHED':
      return _extends({}, state, {
        touched: setIn(state.touched, msg.payload.field, msg.payload.value)
      });

    case 'SET_FIELD_ERROR':
      {
        var newErrors = msg.payload.enableNestedErrors ? setIn(state.errors, msg.payload.field, msg.payload.value) : _extends({}, state.errors, _defineProperty({}, msg.payload.field, msg.payload.value));
        return _extends({}, state, {
          errors: newErrors
        });
      }

    case 'RESET_FORM':
    case 'SET_FORM_STATE':
      return _extends({}, state, msg.payload);

    case 'SUBMIT_ATTEMPT':
      return _extends({}, state, {
        touched: setNestedObjectValues(state.values, true),
        isSubmitting: true,
        submitCount: state.submitCount + 1
      });

    case 'SUBMIT_FAILURE':
      return _extends({}, state, {
        isSubmitting: false
      });

    case 'SUBMIT_SUCCESS':
      return _extends({}, state, {
        isSubmitting: false
      });

    default:
      return state;
  }
} // Initial empty states // objects


var emptyErrors = {};
var emptyTouched = {};
var emptyValues = {};
export default function useForm(props) {
  var onChange = props.onChange,
      valuesProp = props.values,
      _props$defaultValues = props.defaultValues,
      defaultValues = _props$defaultValues === void 0 ? emptyValues : _props$defaultValues,
      _props$defaultErrors = props.defaultErrors,
      defaultErrors = _props$defaultErrors === void 0 ? emptyErrors : _props$defaultErrors,
      _props$defaultTouched = props.defaultTouched,
      defaultTouched = _props$defaultTouched === void 0 ? emptyTouched : _props$defaultTouched,
      defaultStatus = props.defaultStatus,
      _props$validateOnBlur = props.validateOnBlur,
      validateOnBlur = _props$validateOnBlur === void 0 ? true : _props$validateOnBlur,
      _props$validateOnChan = props.validateOnChange,
      validateOnChange = _props$validateOnChan === void 0 ? true : _props$validateOnChan,
      _props$enableNestedEr = props.enableNestedErrors,
      enableNestedErrors = _props$enableNestedEr === void 0 ? false : _props$enableNestedEr,
      _props$scrollToFirstE = props.scrollToFirstError,
      scrollToFirstError = _props$scrollToFirstE === void 0 ? false : _props$scrollToFirstE,
      scrollSettings = props.scrollSettings,
      onSubmit = props.onSubmit,
      onReset = props.onReset,
      children = props.children;
  var defaultValuesRef = useRef(defaultValues);
  var defaultErrorsRef = useRef(defaultErrors);
  var defaultTouchedRef = useRef(defaultTouched);
  var defaultStatusRef = useRef(defaultStatus);
  var formItemsRef = useRef({});
  var isControlled = ('values' in props);

  var _useReducer = useReducer(formReducer, {
    values: defaultValues,
    errors: defaultErrors,
    touched: defaultTouched,
    isSubmitting: false,
    isValidating: false,
    submitCount: 0,
    status: defaultStatus
  }),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var fieldRules = useRef({});
  var fieldProps = useRef({}); // 受控模式情况使用props中的values，非受控使用state.values

  var currentValues = isControlled && valuesProp ? valuesProp : state.values;
  var handleFormItemRef = useCallback(function (name, ref) {
    formItemsRef.current[name] = ref;
  }, []);

  var resetForm = _useEventCallback(function (nextState) {
    var values = nextState && nextState.values ? nextState.values : defaultValuesRef.current;
    var errors = nextState && nextState.errors ? nextState.errors : defaultErrorsRef.current ? defaultErrorsRef.current : defaultErrors || {};
    var touched = nextState && nextState.touched ? nextState.touched : defaultTouchedRef.current ? defaultTouchedRef.current : defaultTouched || {};
    var status = nextState && nextState.status ? nextState.status : defaultStatusRef.current ? defaultStatusRef.current : defaultStatus;
    defaultValuesRef.current = values;
    defaultErrorsRef.current = errors;
    defaultTouchedRef.current = touched;
    defaultStatusRef.current = status;
    dispatch({
      type: 'RESET_FORM',
      payload: {
        isSubmitting: !!nextState && !!nextState.isSubmitting,
        errors: errors,
        touched: touched,
        status: status,
        values: values,
        isValidating: !!nextState && !!nextState.isValidating,
        submitCount: !!nextState && !!nextState.submitCount && typeof nextState.submitCount === 'number' ? nextState.submitCount : 0
      }
    });
    onChange && onChange(values, helpers);
  }, [defaultErrors, defaultTouched, defaultStatus, onChange]);

  var registerFieldRules = useCallback(function (name, rule) {
    fieldRules.current[name] = rule;
  }, []);
  var unregisterFieldRules = useCallback(function (name) {
    delete fieldRules.current[name];
  }, []);
  var registerFieldProps = useCallback(function (name, props) {
    fieldProps.current[name] = props;
  }, []);
  var unregisterFieldProps = useCallback(function (name) {
    delete fieldProps.current[name];
  }, []);
  var setStatus = useCallback(function (status) {
    dispatch({
      type: 'SET_STATUS',
      payload: status
    });
  }, []);
  var setSubmitting = useCallback(function (isSubmitting) {
    dispatch({
      type: 'SET_ISSUBMITTING',
      payload: isSubmitting
    });
  }, []);
  var setErrors = useCallback(function (errors) {
    dispatch({
      type: 'SET_ERRORS',
      payload: errors
    });
  }, []);
  var setFieldError = useCallback(function (field, value) {
    dispatch({
      type: 'SET_FIELD_ERROR',
      payload: {
        field: field,
        value: value,
        enableNestedErrors: enableNestedErrors
      }
    });
  }, [enableNestedErrors]); // 表单整体校验

  var validateForm = _useEventCallback(function (newValues) {
    var values = newValues || currentValues;
    dispatch({
      type: 'SET_ISVALIDATING',
      payload: true
    }); // 校验所有已注册过Rule的Field

    var allValues = {};
    Object.keys(fieldRules.current).forEach(function (field) {
      var value = getIn(values, field);
      allValues[field] = value;
    });
    var validator = new Schema(fieldRules.current);
    return validator.validate(allValues, {}, function (_errors, fieldErrors) {
      dispatch({
        type: 'SET_ISVALIDATING',
        payload: false
      });

      if (fieldErrors) {
        var newErrors = {};

        if (enableNestedErrors) {
          Object.keys(fieldErrors).forEach(function (field) {
            newErrors = setIn(newErrors, field, fieldErrors[field]);
          });
          setErrors(newErrors);
        } else {
          setErrors(fieldErrors);
        }
      } else {
        setErrors({});
      }
    });
  }, [currentValues, setErrors]);

  var validatePartialForm = _useEventCallback(function (fields, partialValues) {
    var values = partialValues || currentValues;
    dispatch({
      type: 'SET_ISVALIDATING',
      payload: true
    }); // 校验所有已注册过Rule的Field

    var partialRules = {};
    fields.forEach(function (field) {
      var rule = fieldRules.current[field];

      if (rule) {
        partialRules[field] = rule;
      }
    });
    var handledPartialValues = {};
    Object.keys(partialRules).forEach(function (field) {
      var value = getIn(values, field);
      handledPartialValues[field] = value;
    });
    var validator = new Schema(partialRules);
    return validator.validate(handledPartialValues, {}, function (_errors, fieldErrors) {
      dispatch({
        type: 'SET_ISVALIDATING',
        payload: false
      });

      if (fieldErrors) {
        var newErrors = {};

        if (enableNestedErrors) {
          Object.keys(fieldErrors).forEach(function (field) {
            newErrors = setIn(newErrors, field, fieldErrors[field]);
          });
          setErrors(newErrors);
        } else {
          setErrors(fieldErrors);
        }
      } else {
        setErrors({});
      }
    });
  }, [currentValues, setErrors]); // 校验单个表单项


  var validateField = _useEventCallback(function (field, outerValue) {
    var rule = fieldRules.current[field];
    var currentFieldProps = fieldProps.current[field];
    var validateFirst = currentFieldProps ? currentFieldProps.validateFirst : false;
    var value = outerValue === undefined ? getIn(currentValues, field) : outerValue;

    if (rule) {
      dispatch({
        type: 'SET_ISVALIDATING',
        payload: true
      });
      var validator = new Schema(_defineProperty({}, field, rule));
      return validator.validate(_defineProperty({}, field, value), {
        first: validateFirst
      }, function (errors) {
        dispatch({
          type: 'SET_ISVALIDATING',
          payload: false
        });
        setFieldError(field, errors);
      });
    }

    return Promise.resolve();
  }, [currentValues]);

  var setTouched = useCallback(function (touched) {
    dispatch({
      type: 'SET_TOUCHED',
      payload: touched
    });

    if (validateOnBlur) {
      validateForm();
    }
  }, [validateForm, validateOnBlur]);

  var setValues = _useEventCallback(function (values) {
    var shouldValidate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    dispatch({
      type: 'SET_VALUES',
      payload: values
    });
    onChange && onChange(values, helpers);

    if (validateOnChange && shouldValidate) {
      validateForm(values);
    }
  });

  var setPartialValues = _useEventCallback(function (values) {
    var shouldValidate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var deep = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var newValues;

    if (deep) {
      newValues = deepmerge(_extends({}, currentValues), values, _isBoolean(deep) ? {
        arrayMerge: overwriteArrayMerge
      } : deep);
    } else {
      newValues = _extends({}, currentValues, values);
    }

    dispatch({
      type: 'SET_VALUES',
      payload: newValues
    });
    onChange && onChange(newValues, helpers);

    if (validateOnChange && shouldValidate) {
      var validateFields = [];
      Object.keys(fieldRules.current).forEach(function (field) {
        if (_hasIn(values, field)) {
          validateFields.push(field);
        }
      });
      validatePartialForm(validateFields, values);
    }
  });

  var setFieldValue = _useEventCallback(function (field, value) {
    var shouldValidate = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    dispatch({
      type: 'SET_FIELD_VALUE',
      payload: {
        field: field,
        value: value
      }
    });
    onChange && onChange(setIn(currentValues, field, value), helpers, field, value);
    return validateOnChange && shouldValidate ? validateField(field, value) : Promise.resolve();
  }, [currentValues, onChange, validateField, validateOnChange]);

  var setFieldTouched = useCallback(function (field) {
    var touched = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var shouldValidate = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    dispatch({
      type: 'SET_FIELD_TOUCHED',
      payload: {
        field: field,
        value: touched
      }
    });
    return validateOnBlur && shouldValidate ? validateField(field) : Promise.resolve();
  }, [validateField, validateOnBlur]);
  var getFieldError = useCallback(function (field) {
    var error = enableNestedErrors ? getIn(state.errors, field) : state.errors[field] || [];

    if (error && error.length) {
      return error.map(function (item) {
        return item.message;
      });
    }
  }, [enableNestedErrors, state]);
  var getFieldsError = useCallback(function (fields) {
    var finalFields = fields || Object.keys(fieldRules.current);
    var allErrors = {};
    finalFields.forEach(function (field) {
      var error = getFieldError(field);

      if (error) {
        allErrors[field] = error;
      }
    });
    return allErrors;
  }, [getFieldError]);
  var scrollToField = useCallback(function (name) {
    var itemRef = formItemsRef.current[name];

    if (itemRef) {
      try {
        scrollIntoView(itemRef, _extends({
          onlyFirstScrollableParent: true,
          align: {
            top: 0
          }
        }, scrollSettings));
      } catch (_unused) {
        _devWarning('[Form]: form scrollToFieldError occur error');
      }
    } else {
      _devWarning("[Form]: form name: ".concat(name, " field not exist"));
    }
  }, [scrollSettings]);
  var scrollToFirstErrorField = useCallback(function (errors) {
    if (!_isEmpty(errors)) {
      var firstErrorField = errors[0].field;
      scrollToField(firstErrorField);
    }
  }, [scrollToField]);
  var helpers = {
    resetForm: resetForm,
    validateForm: validateForm,
    validatePartialForm: validatePartialForm,
    validateField: validateField,
    setErrors: setErrors,
    setFieldError: setFieldError,
    setFieldTouched: setFieldTouched,
    setFieldValue: setFieldValue,
    setStatus: setStatus,
    setSubmitting: setSubmitting,
    setTouched: setTouched,
    setValues: setValues,
    setPartialValues: setPartialValues,
    getFieldError: getFieldError,
    getFieldsError: getFieldsError,
    registerFieldRules: registerFieldRules,
    unregisterFieldRules: unregisterFieldRules,
    registerFieldProps: registerFieldProps,
    unregisterFieldProps: unregisterFieldProps,
    scrollToField: scrollToField,
    scrollToFirstErrorField: scrollToFirstErrorField
  };
  var submitForm = useCallback(function () {
    dispatch({
      type: 'SUBMIT_ATTEMPT'
    });
    return validateForm().then(function () {
      dispatch({
        type: 'SUBMIT_SUCCESS'
      });
      onSubmit && onSubmit(currentValues, helpers);
    })["catch"](function (data) {
      dispatch({
        type: 'SUBMIT_FAILURE'
      });

      if (scrollToFirstError) {
        scrollToFirstErrorField(data.errors);
      }
    });
  }, [currentValues, helpers, onSubmit, scrollToFirstError, scrollToFirstErrorField, validateForm]);
  var handleSubmit = useCallback(function (e) {
    if (e && e.preventDefault && _isFunction(e.preventDefault)) {
      e.preventDefault();
    }

    if (e && e.stopPropagation && _isFunction(e.stopPropagation)) {
      e.stopPropagation();
    }

    return submitForm();
  }, [submitForm]);
  var handleReset = useCallback(function (e) {
    if (e && e.preventDefault && _isFunction(e.preventDefault)) {
      e.preventDefault();
    }

    if (e && e.stopPropagation && _isFunction(e.stopPropagation)) {
      e.stopPropagation();
    }

    if (onReset) {
      onReset(currentValues, helpers);
    }

    resetForm();
  }, [currentValues, helpers, onReset, resetForm]);

  _useEffectOnce(function () {
    if ('scrollToFirstErrorSettings' in props) {
      _devWarning('[Form]: prop `scrollToFirstErrorSettings` is deprecated, please use `scrollSettings` instead.');
    }
  });

  return _extends({}, state, helpers, props, {
    handleFormItemRef: handleFormItemRef,
    handleReset: handleReset,
    handleSubmit: handleSubmit,
    children: children,
    defaultStatus: defaultStatusRef.current,
    defaultErrors: defaultErrorsRef.current,
    defaultTouched: defaultTouchedRef.current,
    defaultValues: defaultValuesRef.current,
    validateOnBlur: validateOnBlur,
    validateOnChange: validateOnChange,
    onSubmit: onSubmit,
    onReset: onReset
  });
}