import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import { useCallback, useEffect, useMemo, useState } from 'react';
import { getScroller } from '../Sticky/utils';
import { getCalculationsData, getWindow, inView, safe } from './utils';

var getData = function getData(nodes, padding) {
  var data = [];
  nodes.forEach(function (node) {
    data.push(getCalculationsData(node, padding));
  });
  return data;
};

var getNodeAndScroller = function getNodeAndScroller(selector, stepScroller, globalScroller) {
  var scroller;
  var node;

  if (typeof selector === 'string') {
    node = document.querySelector(selector);
    node && (scroller = getScroller(node));
  } else {
    node = selector;
    scroller = getScroller(selector);
  }

  return {
    node: node,
    scroller: stepScroller || globalScroller || scroller
  };
};

var getNodes = function getNodes(step, globalScroller) {
  var nodes = [];
  var scroller;
  var selector = step.selector;

  if (Array.isArray(selector)) {
    selector.forEach(function (selector) {
      var _getNodeAndScroller = getNodeAndScroller(selector, step.scroller, globalScroller),
          node = _getNodeAndScroller.node,
          newScroller = _getNodeAndScroller.scroller;

      _devWarning('引导框选多节点时只支持相同的滚动容器', !!(scroller && scroller !== newScroller));

      nodes.push(node);
      scroller = newScroller;
    });
  } else {
    var _getNodeAndScroller2 = getNodeAndScroller(selector, globalScroller),
        node = _getNodeAndScroller2.node,
        newScroller = _getNodeAndScroller2.scroller;

    nodes.push(node);
    scroller = newScroller;
  }

  return {
    nodes: nodes,
    scroller: scroller
  };
};

export default (function (props) {
  var _props$maskSpace = props.maskSpace,
      padding = _props$maskSpace === void 0 ? 0 : _props$maskSpace,
      currentIndex = props.currentIndex,
      _props$steps = props.steps,
      steps = _props$steps === void 0 ? [] : _props$steps,
      _props$closeWithMask = props.closeWithMask,
      closeWithMask = _props$closeWithMask === void 0 ? false : _props$closeWithMask,
      close = props.close,
      goTo = props.goTo,
      globalScroller = props.scroller;

  var _useState = useState([]),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var _useState3 = useState(),
      _useState4 = _slicedToArray(_useState3, 2),
      currentScroller = _useState4[0],
      setCurrentScroller = _useState4[1];

  var _useState5 = useState(false),
      _useState6 = _slicedToArray(_useState5, 2),
      disabled = _useState6[0],
      setDisabled = _useState6[1];

  var handleData = useCallback(function () {
    var step = steps[currentIndex];

    var _getNodes = getNodes(step, globalScroller),
        nodes = _getNodes.nodes,
        scroller = _getNodes.scroller;

    setState(getData(nodes, step.maskSpace || padding));
    setCurrentScroller(scroller);
  }, [currentIndex, globalScroller, padding, steps]);
  useEffect(function () {
    setDisabled(false);
    handleData();

    var handleNode = function handleNode() {
      handleData();
    };

    window.addEventListener('resize', handleNode);
    return function () {
      return window.removeEventListener('resize', handleNode);
    };
  }, [currentIndex, handleData]);
  var left = Infinity;
  var top = Infinity;
  var bottom = -Infinity;
  var right = -Infinity;
  state.forEach(function (item) {
    left = Math.min(item.left, left);
    top = Math.min(item.top, top);
    bottom = Math.max(item.bottom, bottom);
    right = Math.max(item.right, right);
  });
  var width = safe(right - left);
  var height = safe(bottom - top);

  var _getWindow = getWindow(),
      innerWidth = _getWindow.width,
      innerHeight = _getWindow.height;

  if (state.length && !inView({
    top: top,
    bottom: bottom,
    left: left,
    height: innerHeight
  })) {
    var windowOffset = top + window.scrollY + height / 2 - innerHeight / 2;
    var offset = height > innerHeight ? -25 : -(innerHeight / 2) + height / 2;

    if (currentScroller && currentScroller !== window) {
      currentScroller.scroll({
        top: Math.abs(offset)
      });
    }

    window.scroll({
      top: windowOffset
    });
    var step = steps[currentIndex];

    var _getNodes2 = getNodes(step),
        nodes = _getNodes2.nodes;

    var data = getData(nodes, step.maskSpace || padding);
    left = Infinity;
    top = Infinity;
    bottom = -Infinity;
    right = -Infinity;
    data.forEach(function (item) {
      left = Math.min(item.left, left);
      top = Math.min(item.top, top);
      bottom = Math.max(item.bottom, bottom);
      right = Math.max(item.right, right);
    });
    width = safe(right - left);
    height = safe(bottom - top);
  }

  top = safe(top);
  var _steps$currentIndex = steps[currentIndex],
      content = _steps$currentIndex.content,
      canNext = _steps$currentIndex.canNext;
  var handleMaskClick = useCallback(function () {
    if (closeWithMask) {
      close();
    }
  }, [close, closeWithMask]);
  useEffect(function () {
    if (canNext) {
      var res = canNext();

      if (typeof res === 'boolean') {
        setDisabled(!res);
      } else if (res.then) {
        setDisabled(true);
        res.then(function () {
          setDisabled(false);
        }, function (e) {
          console.error(e);
          setDisabled(false);
        });
      }
    }
  }, [canNext]);
  var hasCustomContent = !!(content && typeof content === 'function');
  var stepContent = useMemo(function () {
    return content && (typeof content === 'function' ? content({
      close: close,
      goTo: goTo,
      step: currentIndex + 1
    }) : content);
  }, [close, content, currentIndex, goTo]);
  return {
    stepContent: stepContent,
    hasCustomContent: hasCustomContent,
    handleMaskClick: handleMaskClick,
    left: left,
    right: right,
    height: height,
    bottom: bottom,
    top: top,
    width: width,
    disabled: disabled,
    state: state,
    setState: setState,
    innerWidth: innerWidth,
    innerHeight: innerHeight
  };
});