import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["width", "height", "suffixHeight", "suffixWidth", "webp", "format", "resizeMode", "ratio", "blur", "q", "Q", "external", "wait", "resize", "clean", "oss", "suffix", "lazy", "src", "aspectRatio", "onLoaded", "onError", "defaultImg", "component", "defaultImgStyle", "loadingType", "loadingDelay", "children", "style", "styles", "className"],
    _excluded2 = ["component"],
    _excluded3 = ["component"];
import React, { useMemo, useRef } from 'react';
import styled from 'styled-components';
import SpinIcon from '../Spin/SpinIcon';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyle from '../utils/mergeStyle';
import { withProperties } from '../utils/type-helper';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
import ImgContainer from './ImgContainer';
import { BaseDiv, BaseImg, BaseSpan, StyledSkeleton } from './styled';
import useImg from './useImg';
var defaultStyles = {
  loading: '',
  error: '',
  blank: '',
  loaded: ''
};
var ImgPure = memoForwardRef(function (props, ref) {
  var width = props.width,
      height = props.height,
      suffixHeight = props.suffixHeight,
      suffixWidth = props.suffixWidth,
      webp = props.webp,
      format = props.format,
      resizeMode = props.resizeMode,
      ratio = props.ratio,
      blur = props.blur,
      q = props.q,
      Q = props.Q,
      external = props.external,
      wait = props.wait,
      resize = props.resize,
      clean = props.clean,
      oss = props.oss,
      suffix = props.suffix,
      lazy = props.lazy,
      src = props.src,
      aspectRatio = props.aspectRatio,
      onLoaded = props.onLoaded,
      onError = props.onError,
      defaultImg = props.defaultImg,
      component = props.component,
      defaultImgStyle = props.defaultImgStyle,
      loadingType = props.loadingType,
      loadingDelay = props.loadingDelay,
      children = props.children,
      style = props.style,
      styles = props.styles,
      className = props.className,
      otherProps = _objectWithoutProperties(props, _excluded);

  var imgRef = useRef(null);

  var handleRef = _useForkRef(imgRef, ref);

  var theme = useTheme();

  var _useImg = useImg({
    imgRef: imgRef,
    suffixs: {
      width: suffixWidth,
      height: suffixHeight,
      webp: webp,
      format: format,
      resizeMode: resizeMode,
      ratio: ratio,
      blur: blur,
      q: q,
      Q: Q,
      external: external
    },
    options: {
      wait: wait,
      resize: resize,
      clean: clean,
      oss: oss,
      suffix: suffix,
      lazy: lazy
    },
    src: src,
    onError: onError,
    onLoaded: onLoaded,
    loadingDelay: loadingDelay
  }),
      imgState = _useImg.imgState,
      imgPool = _useImg.imgPool;

  var imgSrc = imgState.imgSrc,
      loadStatus = imgState.loadStatus;
  var innerStyles = useStyles('img', defaultStyles, styles);
  var loading = loadStatus === 'none';
  var finalDefaultImg = defaultImg || imgPool.settings.defaultImg || theme.components.Img.errorImg;
  var useDefaultImgAndIsFromTheme = false;
  var innerImgSrc = imgSrc;

  if (loading && loadingType === 'default-img' && finalDefaultImg || loadStatus === 'error' && finalDefaultImg) {
    innerImgSrc = finalDefaultImg;
    useDefaultImgAndIsFromTheme = finalDefaultImg === theme.components.Img.errorImg;
  }

  var styleItem;

  if (loading) {
    styleItem = innerStyles.loading;
  } else if (loadStatus === 'blank') {
    styleItem = innerStyles.blank;
  } else if (loadStatus === 'error') {
    styleItem = innerStyles.error;
  } else if (loadStatus === 'loaded') {
    styleItem = innerStyles.loaded;
  }

  var innerStyle = useMemo(function () {
    var moreStyle;

    if (defaultImg && defaultImgStyle) {
      moreStyle = defaultImgStyle;
    } else if (finalDefaultImg === imgPool.settings.defaultImg && imgPool.settings.defaultImgStyle) {
      moreStyle = imgPool.settings.defaultImgStyle;
    }

    var styleItemStyle = styleItem ? styleItem.style : undefined;
    return mergeStyle(moreStyle, styleItemStyle, style);
  }, [defaultImg, defaultImgStyle, finalDefaultImg, imgPool.settings.defaultImg, imgPool.settings.defaultImgStyle, style, styleItem]);
  var imgStyle = useMemo(function () {
    if (innerImgSrc) {
      return _extends({}, innerStyle, {
        backgroundImage: "url('".concat(innerImgSrc, "')")
      });
    }

    return innerStyle;
  }, [innerImgSrc, innerStyle]);
  var innerClassName = useMemo(function () {
    var classnames = [];

    if (className) {
      classnames.push(className);
    }

    if (styleItem) {
      classnames.push(styleItem.className);
    }

    return classnames.length > 0 ? classnames.join(' ') : undefined;
  }, [className, styleItem]);
  var finalChildren = useMemo(function () {
    if (loading && loadingType === 'skeleton') {
      return /*#__PURE__*/React.createElement(StyledSkeleton, null, children);
    } else if (loading && loadingType === 'spin') {
      return /*#__PURE__*/React.createElement(SpinIcon, {
        color: "currentColor"
      });
    }

    return children;
  }, [children, loading, loadingType]);

  if (component === 'img') {
    return /*#__PURE__*/React.createElement(BaseImg, _extends({}, otherProps, {
      className: innerClassName,
      theme: theme,
      $width: width,
      $height: height,
      style: innerStyle,
      "data-src": src,
      src: innerImgSrc,
      ref: handleRef
    }));
  }

  if (component === 'span') {
    return /*#__PURE__*/React.createElement(BaseSpan, _extends({}, otherProps, {
      className: innerClassName,
      theme: theme,
      $width: width,
      $height: height,
      $aspectRatio: aspectRatio,
      $loadingType: loadingType,
      $useDefaultImgAndIsFromTheme: useDefaultImgAndIsFromTheme,
      $status: loadStatus,
      style: imgStyle,
      ref: handleRef
    }), finalChildren);
  }

  return /*#__PURE__*/React.createElement(BaseDiv, _extends({}, otherProps, {
    className: innerClassName,
    theme: theme,
    $width: width,
    $height: height,
    $aspectRatio: aspectRatio,
    $loadingType: loadingType,
    $status: loadStatus,
    $useDefaultImgAndIsFromTheme: useDefaultImgAndIsFromTheme,
    style: imgStyle,
    ref: handleRef
  }), finalChildren);
});
var ImgNoTheme = styled(ImgPure).withConfig({
  displayName: "ImgNoTheme"
})([""]);
var Img = withThemeForStyled(ImgNoTheme);
var ImgDiv = Img;
var Picture = Img;
var ImgSpan = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var component = props.component,
      otherProps = _objectWithoutProperties(props, _excluded2);

  return /*#__PURE__*/React.createElement(Img, _extends({}, otherProps, {
    ref: ref,
    component: "span"
  }));
});
var ImgImg = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var component = props.component,
      otherProps = _objectWithoutProperties(props, _excluded3);

  return /*#__PURE__*/React.createElement(Img, _extends({}, otherProps, {
    ref: ref,
    component: "img"
  }));
});
export default withProperties(Img, {
  Span: ImgSpan,
  Img: ImgImg,
  Container: ImgContainer,
  useImg: useImg
});
export { ImgPure, ImgDiv, ImgSpan, Picture, ImgImg };