import styled, { css } from 'styled-components';
import Skeleton from '../Skeleton';
import { skeletonNormalCss } from '../styled/mixins/skeleton';
import addPx from '../utils/addPx';

function rectCss(props) {
  var $width = props.$width,
      $height = props.$height;
  var cssObj = {};

  if ($width) {
    cssObj.width = addPx($width);
  }

  if ($height) {
    cssObj.height = addPx($height);
  }

  return cssObj;
}

var AspectRatioMap = {
  '1:1': 1,
  '4:3': 3 / 4,
  '3:2': 2 / 3,
  '16:9': 9 / 16,
  '3:4': 4 / 3,
  '9:16': 16 / 9
};

function blockCss(props) {
  var $width = props.$width,
      $height = props.$height,
      $aspectRatio = props.$aspectRatio,
      $status = props.$status,
      $loadingType = props.$loadingType,
      $useDefaultImgAndIsFromTheme = props.$useDefaultImgAndIsFromTheme,
      theme = props.theme;
  var aspectCss;

  if ($aspectRatio) {
    var cssObj = {
      position: 'relative',
      height: 0
    };
    var heightRatio = AspectRatioMap[$aspectRatio];

    if (typeof $width === 'number' && $width > 0) {
      cssObj.height = Math.floor($width * heightRatio);
    } else {
      cssObj.paddingBottom = "".concat(heightRatio * 100, "%");
    }

    if ($width) {
      cssObj.width = addPx($width);
    }

    aspectCss = cssObj;
  } else if ($width !== undefined || $height !== undefined) {
    aspectCss = rectCss(props);
  }

  var loadingSkeletonCss;

  if ($status === 'none' && $loadingType === 'skeleton') {
    loadingSkeletonCss = css(["position:relative;"]);
  }

  var baseCss;

  if ($useDefaultImgAndIsFromTheme) {
    var token = theme.components.Img;
    baseCss = css(["background-repeat:", ";background-size:", ";"], token.errorBgRepeat, token.errorBgSize);
  } else {
    baseCss = css(["background-repeat:no-repeat;background-size:cover;"]);
  }

  return css(["", " ", " ", " background-position:center;transition:background-image ", "ms ", ";"], aspectCss, loadingSkeletonCss, baseCss, theme.transition.spec.duration.normal, theme.transition.pattern.easing.enter);
}

function divCss(props) {
  var $status = props.$status,
      $loadingType = props.$loadingType,
      theme = props.theme;

  if ($status === 'none' && $loadingType === 'spin') {
    return css(["display:flex;align-items:center;justify-content:center;color:", ";", ""], theme.colors.spec.brand, skeletonNormalCss);
  }
}

function spanCss(props) {
  var $status = props.$status,
      $loadingType = props.$loadingType,
      theme = props.theme;

  if ($status === 'none' && $loadingType === 'spin') {
    return css(["display:inline-flex;align-items:center;justify-content:center;color:", ";", ""], theme.colors.spec.brand, skeletonNormalCss);
  }

  return css(["display:inline-block;"]);
}

export var BaseDiv = styled.div.withConfig({
  displayName: "BaseDiv"
})(["", " ", ""], blockCss, divCss);
export var BaseImg = styled.img.withConfig({
  displayName: "BaseImg"
})(["", ""], rectCss);
export var BaseSpan = styled.span.withConfig({
  displayName: "BaseSpan"
})(["", " ", ""], blockCss, spanCss);
export var StyledSkeleton = styled(Skeleton).withConfig({
  displayName: "StyledSkeleton"
})(["width:100%;height:100%;"]);