import _extends from "@babel/runtime/helpers/extends";
import { wait as _wait } from "@muya-ui/utils/es/utils/wait";
import { useLockScroll as _useLockScroll } from "@muya-ui/utils/es/hooks/useLockScroll";
import { useAfterEffect as _useAfterEffect } from "@muya-ui/utils/es/hooks/useAfterEffect";
import { useThrottle as _useThrottle } from "@muya-ui/utils/es/hooks/useThrottle";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import forkHandler from '../utils/forkHandler';
import { useTheme } from '../utils/useTheme';
import { cropImg, getImgStyle, getNewTranslate } from './utils';
export default function useImgCropper(props, maskRef, imgRef) {
  var _useTheme = useTheme(),
      defaultSize = _useTheme.components.ImgCropper.defaultSize;

  var _props$size = props.size,
      size = _props$size === void 0 ? defaultSize : _props$size,
      defaultScale = props.defaultScale,
      _props$src = props.src,
      src = _props$src === void 0 ? '' : _props$src,
      _props$wheelScaleStep = props.wheelScaleStep,
      wheelScaleStep = _props$wheelScaleStep === void 0 ? 0.01 : _props$wheelScaleStep,
      _props$scaleStep = props.scaleStep,
      scaleStep = _props$scaleStep === void 0 ? 0.1 : _props$scaleStep,
      disableWheel = props.disableWheel,
      _props$scaleMax = props.scaleMax,
      scaleMax = _props$scaleMax === void 0 ? 4 : _props$scaleMax,
      _props$scaleMin = props.scaleMin,
      scaleMin = _props$scaleMin === void 0 ? 1 : _props$scaleMin,
      _props$backDelay = props.backDelay,
      backDelay = _props$backDelay === void 0 ? 200 : _props$backDelay;
  var innerWheelScaleStep = wheelScaleStep < 0 || wheelScaleStep > 1 ? 0.1 : wheelScaleStep;
  var finalScaleMin = scaleMin < 1 ? 1 : scaleMin;
  var finalScaleMax = scaleMax < 1 ? 1 : scaleMax;

  var _useState = useState(defaultScale || finalScaleMin),
      _useState2 = _slicedToArray(_useState, 2),
      stateScale = _useState2[0],
      setScale = _useState2[1];

  var finalScale = Math.max(stateScale, 0.01);

  var _useState3 = useState([0, 0]),
      _useState4 = _slicedToArray(_useState3, 2),
      finalTranslate = _useState4[0],
      setTranslate = _useState4[1];

  var _useState5 = useState(0),
      _useState6 = _slicedToArray(_useState5, 2),
      finalRotate = _useState6[0],
      setRotate = _useState6[1];

  var _useState7 = useState({
    src: src,
    status: 'none'
  }),
      _useState8 = _slicedToArray(_useState7, 2),
      state = _useState8[0],
      setState = _useState8[1];

  var _useState9 = useState(false),
      _useState10 = _slicedToArray(_useState9, 2),
      moving = _useState10[0],
      setMoving = _useState10[1];

  var _useState11 = useState(false),
      _useState12 = _slicedToArray(_useState11, 2),
      enter = _useState12[0],
      setEnter = _useState12[1];

  if (src !== state.src) {
    setState({
      src: src,
      status: 'none'
    });
    setScale(finalScaleMin);
    setTranslate([0, 0]);
    setRotate(0);
  }

  var imgStyle = useMemo(function () {
    var innerImgStyle = getImgStyle(size, finalRotate, state.img);

    if (state.src) {
      innerImgStyle.backgroundImage = "url(".concat(state.src, ")");
    }

    innerImgStyle.transform = ["translate(".concat(finalTranslate[0], "px, ").concat(finalTranslate[1], "px)"), "rotate(".concat(finalRotate, "deg)"), "scale(".concat(finalScale, ")"), 'translateZ(0)'].join(' ');
    return innerImgStyle;
  }, [finalRotate, finalScale, finalTranslate, size, state.img, state.src]); // 滚轮停止 或者 鼠标结束时，校验位置是否正确的
  // 不用包 memo 永远 debounce 使用了， 用了 useAfterEffect

  var backToRightTransform = function backToRightTransform() {
    if (moving) {
      return;
    }

    if (finalScale < finalScaleMin) {
      setScale(finalScaleMin);
      setTranslate([0, 0]);
      return;
    }

    if (finalScale > finalScaleMax) {
      setScale(finalScaleMax);
      return;
    }
    /* istanbul ignore if */


    if (!(maskRef.current && imgRef.current)) {
      return;
    }

    var newTranslate = getNewTranslate(maskRef.current.getBoundingClientRect(), imgRef.current.getBoundingClientRect(), finalTranslate);

    if (newTranslate) {
      setTranslate(newTranslate);
    }
  };

  var getImg = function getImg() {
    var radio = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;

    /* istanbul ignore if */
    if (!(maskRef.current && imgRef.current && state.img)) {
      return null;
    }

    var maskRect = maskRef.current.getBoundingClientRect();
    var imgRect = imgRef.current.getBoundingClientRect();
    return cropImg(state.img, imgRect, maskRect, finalRotate, radio);
  };

  var getInfo = function getInfo() {
    /* istanbul ignore if */
    if (!(maskRef.current && imgRef.current && state.img)) {
      return null;
    }

    var maskRect = maskRef.current.getBoundingClientRect();
    var imgRect = imgRef.current.getBoundingClientRect();
    return {
      maskRect: maskRect,
      imgRect: imgRect,
      rotate: finalRotate,
      scale: finalScale
    };
  };

  var beforeFn = useCallback(function () {
    return state.status === 'loaded';
  }, [state.status]);
  var tmp = useRef({
    originX: 0,
    originY: 0,
    lastX: 0,
    lastY: 0
  });
  var moveStart = useCallback(function (e) {
    setMoving(true);
    tmp.current.originX = e.clientX;
    tmp.current.originY = e.clientY;
    tmp.current.lastX = finalTranslate[0];
    tmp.current.lastY = finalTranslate[1];
  }, [finalTranslate]);
  var moveEnd = useCallback(function () {
    setMoving(false);
    setEnter(false);
  }, []);
  var move = useCallback(function (e) {
    if (!moving) {
      return;
    }

    var _tmp$current = tmp.current,
        lastX = _tmp$current.lastX,
        lastY = _tmp$current.lastY,
        originX = _tmp$current.originX,
        originY = _tmp$current.originY;
    var x = lastX + (e.clientX - originX);
    var y = lastY + (e.clientY - originY);
    setTranslate([x, y]);
  }, [moving]);
  var zoom = useCallback(function (e) {
    if (disableWheel || !enter) {
      return;
    }

    var diff = e.deltaY;

    if (diff < 0.1 && diff > -0.1) {
      return;
    }

    var diffScale = e.deltaY * innerWheelScaleStep;
    var newScale = finalScale + diffScale;
    setScale(newScale);
  }, [disableWheel, enter, finalScale, innerWheelScaleStep]);

  var zoomFn = _useThrottle(zoom, 50);

  var rotateImg = useCallback(function () {
    setRotate(finalRotate - 90);
  }, [finalRotate]);
  var handleMouseDown = useMemo(function () {
    return forkHandler(moveStart, undefined, beforeFn);
  }, [beforeFn, moveStart]);
  var handleMouseUp = useMemo(function () {
    return forkHandler(moveEnd, undefined, beforeFn);
  }, [beforeFn, moveEnd]);
  var handleMouseMove = useMemo(function () {
    return forkHandler(move, undefined, beforeFn);
  }, [beforeFn, move]);
  var handleWheel = useMemo(function () {
    return forkHandler(zoomFn, undefined, beforeFn);
  }, [beforeFn, zoomFn]);
  var handleMouseEnter = useCallback(function () {
    setEnter(true);
  }, []);
  var handleSliderChange = setScale;
  var handleZoom = useCallback(function () {
    setScale(finalScale + scaleStep);
  }, [finalScale, scaleStep]);
  var handleZoomOut = useCallback(function () {
    setScale(finalScale - scaleStep);
  }, [finalScale, scaleStep]);
  var resetImg = useCallback(function () {
    setScale(defaultScale || finalScaleMin);
    setRotate(0);
  }, [defaultScale, finalScaleMin]);

  _useAfterEffect(backToRightTransform, backDelay);

  _useLockScroll(enter && !disableWheel);

  useEffect(function () {
    if (state.src && state.status === 'none') {
      _wait.imgLoaded(state.src, 'Anonymous').then(function (img) {
        setState(function (prev) {
          return _extends({}, prev, {
            status: 'loaded',
            img: img
          });
        });
      });
    }
  }, [state.src, state.status]);
  return {
    moving: moving,
    handleMouseDown: handleMouseDown,
    handleMouseUp: handleMouseUp,
    handleMouseEnter: handleMouseEnter,
    handleMouseMove: handleMouseMove,
    handleWheel: handleWheel,
    handleSliderChange: handleSliderChange,
    handleZoom: handleZoom,
    handleZoomOut: handleZoomOut,
    rotateImg: rotateImg,
    resetImg: resetImg,
    imgStyle: imgStyle,
    finalTranslate: finalTranslate,
    finalScale: finalScale,
    finalScaleMin: finalScaleMin,
    finalScaleMax: finalScaleMax,
    finalRotate: finalRotate,
    imgSrc: state.src,
    getImg: getImg,
    getInfo: getInfo,
    backToRightTransform: backToRightTransform,
    moveStart: moveStart,
    moveEnd: moveEnd,
    move: move,
    zoom: zoom,
    enter: enter
  };
}