import _extends from "@babel/runtime/helpers/extends";
import { GuildRightIcon as _GuildRightIcon } from "@muya-ui/theme-light/es/icons/GuildRightIcon";
import { GuildLeftIcon as _GuildLeftIcon } from "@muya-ui/theme-light/es/icons/GuildLeftIcon";
import { CloseIcon as _CloseIcon } from "@muya-ui/theme-light/es/icons/CloseIcon";
import { FullscreenIcon as _FullscreenIcon } from "@muya-ui/theme-light/es/icons/FullscreenIcon";
import { ShrinkIcon as _ShrinkIcon } from "@muya-ui/theme-light/es/icons/ShrinkIcon";
import { MagnifyIcon as _MagnifyIcon } from "@muya-ui/theme-light/es/icons/MagnifyIcon";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["mode", "open", "loading", "zoomInIcon", "zoomOutIcon", "resetZoomIcon", "hideActions", "paginationImgWidth", "disableMaskClick", "renderInImgNode", "renderInImgContainer", "renderCustomPageButton", "renderCustomPagination", "renderCustomActionsNode", "onClose", "onExited", "styles", "style", "className", "suffix", "zIndex"];
import React, { cloneElement, useMemo, useRef } from 'react';
import Button from '../Button';
import ButtonGroup from '../ButtonGroup';
import Dialog from '../Dialog';
import Result from '../Result';
import Spin from '../Spin';
import Swipe from '../Swipe';
import Tooltip from '../Tooltip';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { StyledCloseIconWrapper, StyledImg, StyledImgActions, StyledImgContainer, StyledImgPaginationWrapper, StyledImgPreview, StyledImgPreviewContainer, StyledImgViewerLayer, StyledImgWrapper, StyledLoadStateLayer, StyledPageButton, StyledPaginationButton, StyledPaginationImg } from './styled';
import useImgPreview from './useImgPreview';
import memoForwardRef from '../utils/memoForwardRef';
var defaultStyles = {
  wrapper: '',
  closeIconWrapper: '',
  imgLayer: '',
  imgWrapper: '',
  imgContainer: '',
  img: '',
  paginationWrapper: '',
  paginationButton: '',
  paginationImg: ''
};
var ImgPreview = memoForwardRef(function (props, ref) {
  var theme = useTheme();

  var _props$mode = props.mode,
      mode = _props$mode === void 0 ? 'single' : _props$mode,
      open = props.open,
      loading = props.loading,
      zoomInIcon = props.zoomInIcon,
      zoomOutIcon = props.zoomOutIcon,
      resetZoomIcon = props.resetZoomIcon,
      _props$hideActions = props.hideActions,
      hideActions = _props$hideActions === void 0 ? false : _props$hideActions,
      paginationImgWidth = props.paginationImgWidth,
      _props$disableMaskCli = props.disableMaskClick,
      disableMaskClick = _props$disableMaskCli === void 0 ? false : _props$disableMaskCli,
      renderInImgNode = props.renderInImgNode,
      renderInImgContainer = props.renderInImgContainer,
      renderCustomPageButton = props.renderCustomPageButton,
      renderCustomPagination = props.renderCustomPagination,
      renderCustomActionsNode = props.renderCustomActionsNode,
      onClose = props.onClose,
      onExited = props.onExited,
      styles = props.styles,
      style = props.style,
      className = props.className,
      suffix = props.suffix,
      zIndex = props.zIndex,
      restProps = _objectWithoutProperties(props, _excluded);

  var imgRef = useRef(null);
  var imgScrollWrapperRef = useRef(null);
  var imgWrapperRef = useRef(null);
  var paginationRef = useRef();
  var innerStyles = useStyles('img-preview', defaultStyles, styles);
  var wrapperStyleItem = useMemo(function () {
    return mergeStyleItem({
      className: className,
      style: style
    }, innerStyles.wrapper);
  }, [className, innerStyles.wrapper, style]);

  var _useImgPreview = useImgPreview(props, theme, imgScrollWrapperRef, imgWrapperRef, imgRef),
      imgUrl = _useImgPreview.imgUrl,
      imgs = _useImgPreview.imgs,
      imgIndex = _useImgPreview.imgIndex,
      onStepsChange = _useImgPreview.onStepsChange,
      imgOverflow = _useImgPreview.imgOverflow,
      imgLoading = _useImgPreview.imgLoading,
      imgLoadError = _useImgPreview.imgLoadError,
      imgLoaded = _useImgPreview.imgLoaded,
      loadingText = _useImgPreview.loadingText,
      loadErrorText = _useImgPreview.loadErrorText,
      zoomInText = _useImgPreview.zoomInText,
      zoomOutText = _useImgPreview.zoomOutText,
      resetZoomText = _useImgPreview.resetZoomText,
      overflowOffset = _useImgPreview.overflowOffset,
      stepIndex = _useImgPreview.stepIndex,
      hasPrevStep = _useImgPreview.hasPrevStep,
      hasNextStep = _useImgPreview.hasNextStep,
      toPrevStep = _useImgPreview.toPrevStep,
      toNextStep = _useImgPreview.toNextStep,
      hasPrevImg = _useImgPreview.hasPrevImg,
      hasNextImg = _useImgPreview.hasNextImg,
      showPaginationButton = _useImgPreview.showPaginationButton,
      containerSize = _useImgPreview.containerSize,
      updateOccupyHeight = _useImgPreview.updateOccupyHeight,
      showPagination = _useImgPreview.showPagination,
      handleImgClick = _useImgPreview.handleImgClick,
      handleImgLoaded = _useImgPreview.handleImgLoaded,
      handleImgError = _useImgPreview.handleImgError,
      handleImgMouseDown = _useImgPreview.handleImgMouseDown,
      handleImgMouseMove = _useImgPreview.handleImgMouseMove,
      handleImgMouseUp = _useImgPreview.handleImgMouseUp,
      handleZoomIn = _useImgPreview.handleZoomIn,
      handleZoomOut = _useImgPreview.handleZoomOut,
      handleResetZoom = _useImgPreview.handleResetZoom,
      handleExited = _useImgPreview.handleExited,
      toImgIndex = _useImgPreview.toImgIndex,
      toPrevImg = _useImgPreview.toPrevImg,
      toNextImg = _useImgPreview.toNextImg,
      handleMaskClick = _useImgPreview.handleMaskClick,
      handleCloseIconClick = _useImgPreview.handleCloseIconClick;

  var _theme$components$Img = theme.components.ImgPreview,
      spinColor = _theme$components$Img.spinColor,
      _theme$components$Img2 = _theme$components$Img.imgActions,
      _theme$components$Img3 = _theme$components$Img2.zoomInIcon,
      ZoomInIcon = _theme$components$Img3 === void 0 ? _MagnifyIcon : _theme$components$Img3,
      _theme$components$Img4 = _theme$components$Img2.zoomOutIcon,
      ZoomOutIcon = _theme$components$Img4 === void 0 ? _ShrinkIcon : _theme$components$Img4,
      _theme$components$Img5 = _theme$components$Img2.resetZoomIcon,
      ResetZoomIcon = _theme$components$Img5 === void 0 ? _FullscreenIcon : _theme$components$Img5,
      _theme$components$Img6 = _theme$components$Img.closeIcon.icon,
      CloseIcon = _theme$components$Img6 === void 0 ? _CloseIcon : _theme$components$Img6,
      _theme$components$Img7 = _theme$components$Img.pageButton,
      _theme$components$Img8 = _theme$components$Img7.leftIcon,
      PageLeftIcon = _theme$components$Img8 === void 0 ? _GuildLeftIcon : _theme$components$Img8,
      _theme$components$Img9 = _theme$components$Img7.rightIcon,
      PageRightIcon = _theme$components$Img9 === void 0 ? _GuildRightIcon : _theme$components$Img9;
  var zoomInIconNode = useMemo(function () {
    return zoomInIcon || /*#__PURE__*/React.createElement(ZoomInIcon, null);
  }, [zoomInIcon]);
  var zoomOutIconNode = useMemo(function () {
    return zoomOutIcon || /*#__PURE__*/React.createElement(ZoomOutIcon, null);
  }, [zoomOutIcon]);
  var resetZoomNode = useMemo(function () {
    return resetZoomIcon || /*#__PURE__*/React.createElement(ResetZoomIcon, null);
  }, [resetZoomIcon]);
  var handelPaginationRef = React.useCallback(function (instance) {
    paginationRef.current = instance;

    if (instance) {
      updateOccupyHeight(parseInt(getComputedStyle(instance).height, 10) || 0);
    }
  }, [updateOccupyHeight]);
  var actionsNode = useMemo(function () {
    if (typeof renderCustomActionsNode === 'function') {
      return renderCustomActionsNode({
        zoomInText: zoomInText,
        handleZoomIn: handleZoomIn,
        zoomOutText: zoomOutText,
        handleZoomOut: handleZoomOut,
        resetZoomText: resetZoomText,
        handleResetZoom: handleResetZoom
      });
    }

    return /*#__PURE__*/React.createElement(StyledImgActions, {
      theme: theme
    }, /*#__PURE__*/React.createElement(ButtonGroup, null, /*#__PURE__*/React.createElement(Tooltip, {
      title: zoomInText,
      placement: "top"
    }, /*#__PURE__*/React.createElement(Button, {
      type: "strong",
      onClick: handleZoomIn
    }, zoomInIconNode)), /*#__PURE__*/React.createElement(Tooltip, {
      title: zoomOutText,
      placement: "top"
    }, /*#__PURE__*/React.createElement(Button, {
      type: "strong",
      onClick: handleZoomOut
    }, zoomOutIconNode)), /*#__PURE__*/React.createElement(Tooltip, {
      title: resetZoomText,
      placement: "top"
    }, /*#__PURE__*/React.createElement(Button, {
      type: "strong",
      onClick: handleResetZoom
    }, resetZoomNode))));
  }, [handleResetZoom, handleZoomIn, handleZoomOut, renderCustomActionsNode, resetZoomNode, resetZoomText, theme, zoomInIconNode, zoomInText, zoomOutIconNode, zoomOutText]);
  var pageButtonNode = useMemo(function () {
    if (typeof renderCustomPageButton === 'function') {
      return renderCustomPageButton({
        hasPrev: hasPrevImg,
        hasNext: hasNextImg,
        toPrevImg: toPrevImg,
        toNextImg: toNextImg
      });
    } else {
      return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(StyledPageButton, {
        theme: theme,
        onClick: toPrevImg,
        $disabled: !hasPrevImg,
        $direction: "left",
        $height: containerSize.height,
        $imgOverflow: imgOverflow
      }, /*#__PURE__*/React.createElement(PageLeftIcon, null)), /*#__PURE__*/React.createElement(StyledPageButton, {
        theme: theme,
        onClick: toNextImg,
        $disabled: !hasNextImg,
        $direction: "right",
        $height: containerSize.height,
        $imgOverflow: imgOverflow
      }, /*#__PURE__*/React.createElement(PageRightIcon, null)));
    }
  }, [containerSize.height, hasNextImg, hasPrevImg, imgOverflow, renderCustomPageButton, theme, toNextImg, toPrevImg]);
  var paginationNode = useMemo(function () {
    if (typeof renderCustomPagination === 'function') {
      var pagination = renderCustomPagination({
        imgIndex: imgIndex,
        toImgIndex: toImgIndex
      });
      return /*#__PURE__*/cloneElement(pagination, {
        ref: handelPaginationRef
      });
    } else {
      return /*#__PURE__*/React.createElement(StyledImgPaginationWrapper, _extends({
        ref: handelPaginationRef
      }, innerStyles.paginationWrapper, {
        theme: theme,
        $width: containerSize.width
      }), showPaginationButton && /*#__PURE__*/React.createElement(StyledPaginationButton, _extends({}, innerStyles.paginationButton, {
        theme: theme,
        onClick: toPrevStep,
        disabled: !hasPrevStep,
        arrow: "left",
        side: "right"
      }), /*#__PURE__*/React.createElement(PageLeftIcon, null)), /*#__PURE__*/React.createElement(Swipe, {
        stepIndex: stepIndex,
        onStepsChange: onStepsChange
      }, imgs.map(function (imgSrc, i) {
        return /*#__PURE__*/React.createElement(StyledPaginationImg, _extends({}, innerStyles.paginationImg, {
          key: i,
          "data-i": i,
          theme: theme,
          $selected: imgIndex === i,
          $width: paginationImgWidth,
          onClick: function onClick() {
            return toImgIndex(i);
          },
          src: imgSrc,
          suffix: suffix
        }));
      })), showPaginationButton && /*#__PURE__*/React.createElement(StyledPaginationButton, _extends({}, innerStyles.paginationButton, {
        theme: theme,
        onClick: toNextStep,
        disabled: !hasNextStep,
        arrow: "right",
        side: "left"
      }), /*#__PURE__*/React.createElement(PageRightIcon, null)));
    }
  }, [containerSize.width, handelPaginationRef, hasNextStep, hasPrevStep, imgIndex, imgs, innerStyles.paginationButton, innerStyles.paginationImg, innerStyles.paginationWrapper, onStepsChange, paginationImgWidth, renderCustomPagination, showPaginationButton, stepIndex, suffix, theme, toImgIndex, toNextStep, toPrevStep]);
  return /*#__PURE__*/React.createElement(Dialog.Base, {
    ref: ref,
    open: open,
    disableSize: true,
    lazyMount: true,
    disableMaskClick: disableMaskClick,
    onClose: onClose,
    onExited: handleExited,
    zIndex: zIndex,
    customDialogContainer: /*#__PURE__*/React.createElement(StyledImgPreviewContainer, _extends({}, wrapperStyleItem, {
      ref: ref
    }), /*#__PURE__*/React.createElement(StyledCloseIconWrapper, _extends({
      theme: theme,
      onClick: handleCloseIconClick
    }, innerStyles.closeIconWrapper), /*#__PURE__*/React.createElement(CloseIcon, {
      color: theme.components.ImgPreview.closeIcon.color
    })), !imgLoadError && /*#__PURE__*/React.createElement(StyledImgViewerLayer, _extends({}, innerStyles.imgLayer, {
      onClick: handleMaskClick
    }), /*#__PURE__*/React.createElement(StyledImgPreview, {
      ref: imgScrollWrapperRef,
      theme: theme
    }, /*#__PURE__*/React.createElement(StyledImgContainer, _extends({
      theme: theme,
      ref: imgWrapperRef,
      $imgOverflow: imgOverflow,
      $showPagination: showPagination,
      $width: containerSize.width,
      $height: containerSize.height
    }, innerStyles.imgContainer), /*#__PURE__*/React.createElement(StyledImgWrapper, innerStyles.imgWrapper, /*#__PURE__*/React.createElement(StyledImg, _extends({}, restProps, innerStyles.img, {
      ref: imgRef,
      $imgOverflow: imgOverflow,
      $overflowOffset: overflowOffset,
      resizeMode: "lfit",
      suffixWidth: containerSize.width,
      suffixHeight: "auto",
      suffix: suffix,
      wait: "on",
      theme: theme,
      src: imgUrl,
      onMouseDown: handleImgMouseDown,
      onMouseMove: handleImgMouseMove,
      onMouseUp: handleImgMouseUp,
      onClick: handleImgClick,
      onLoaded: handleImgLoaded,
      onError: handleImgError
    }), imgLoaded && renderInImgNode && renderInImgNode(imgUrl))), renderInImgContainer && renderInImgContainer(imgUrl)))), /*#__PURE__*/React.createElement(StyledLoadStateLayer, null, /*#__PURE__*/React.createElement(Spin, {
      spinning: imgLoading || !!loading,
      color: spinColor,
      desc: loadingText,
      styles: {
        desc: {
          color: spinColor
        }
      }
    }), imgLoadError && !loading && /*#__PURE__*/React.createElement(Result, {
      type: "empty",
      subTitle: loadErrorText
    })), !hideActions && !showPagination && imgLoaded && actionsNode, mode === 'multiple' && pageButtonNode, showPagination && paginationNode)
  });
});
export default ImgPreview;