import { ClearIcon as _ClearIcon } from "@muya-ui/theme-light/es/icons/ClearIcon";
import { SuccessIcon as _SuccessIcon } from "@muya-ui/theme-light/es/icons/SuccessIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _get from "lodash/get";
import _endsWith from "lodash/endsWith";
var _excluded = ["size", "disabled", "status", "allowClear", "clearReplace", "limit", "prefixNode", "suffixNode", "width", "height", "onMouseEnter", "onMouseLeave", "className", "style", "inputRef", "replacedInputNode", "multiline", "autosize", "minRows", "maxRows", "defaultValue", "value", "autoFocus", "onClear", "onFocus", "onBlur", "onClick", "onChange", "onPressEnter", "onKeyDown", "hasFeedback", "hasRelatedValue", "focusWhenClear", "styles", "clearIcon", "bordered", "formatter", "parser"];
import React, { Children, useMemo } from 'react';
import Spin from '../Spin';
import ClearIconWrapper from '../styled/components/ClearIconWrapper';
import addPx from '../utils/addPx';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { ExtraNodes, StyledInput, StyledInputWrapper, StyledStatusWrapper, StyledTextarea, StyledTextareaLimitWrapper, StyledTextareaWrapper } from './styled';
import TextLimit from './TextLimit';
import useInput from './useInput';
var defaultStyles = {
  inputWrapper: '',
  textareaWrapper: '',
  statusWrapper: '',
  clearWrapper: '',
  textarea: '',
  textLimitWrapper: '',
  textLimit: '',
  nodeWrapper: '',
  nodeDivider: '',
  input: ''
};
var Input = memoForwardRef(function Input(props, ref) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      status = props.status,
      allowClear = props.allowClear,
      clearReplace = props.clearReplace,
      _props$limit = props.limit,
      limit = _props$limit === void 0 ? 0 : _props$limit,
      prefixNode = props.prefixNode,
      suffixNode = props.suffixNode,
      width = props.width,
      height = props.height,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      className = props.className,
      style = props.style,
      inputRef = props.inputRef,
      replacedInputNode = props.replacedInputNode,
      multiline = props.multiline,
      autosize = props.autosize,
      minRows = props.minRows,
      maxRows = props.maxRows,
      defaultValue = props.defaultValue,
      value = props.value,
      autoFocus = props.autoFocus,
      onClear = props.onClear,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      onClick = props.onClick,
      onChange = props.onChange,
      onPressEnter = props.onPressEnter,
      onKeyDown = props.onKeyDown,
      _props$hasFeedback = props.hasFeedback,
      hasFeedback = _props$hasFeedback === void 0 ? true : _props$hasFeedback,
      hasRelatedValue = props.hasRelatedValue,
      _props$focusWhenClear = props.focusWhenClear,
      focusWhenClear = _props$focusWhenClear === void 0 ? true : _props$focusWhenClear,
      styles = props.styles,
      customClearIcon = props.clearIcon,
      _props$bordered = props.bordered,
      bordered = _props$bordered === void 0 ? true : _props$bordered,
      formatter = props.formatter,
      parser = props.parser,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('input', defaultStyles, styles);
  var inputWrapperStyleItem = useMemo(function () {
    return mergeStyleItem({
      className: className,
      style: style
    }, innerStyles.inputWrapper);
  }, [className, innerStyles.inputWrapper, style]);
  var textareaWrapperStyleItem = useMemo(function () {
    return mergeStyleItem({
      className: className,
      style: style
    }, innerStyles.textareaWrapper);
  }, [className, innerStyles.textareaWrapper, style]); // 受控模式使用内部state

  var isControlled = ('value' in props);

  var _useInput = useInput({
    defaultValue: defaultValue,
    value: value,
    autoFocus: autoFocus,
    disabled: disabled,
    formatter: formatter,
    parser: parser,
    onClear: onClear,
    onChange: onChange,
    onClick: onClick,
    onFocus: onFocus,
    onBlur: onBlur,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    onKeyDown: onKeyDown,
    onPressEnter: onPressEnter,
    focusWhenClear: focusWhenClear,
    inputRef: inputRef,
    autosize: autosize,
    multiline: multiline,
    minRows: minRows,
    maxRows: maxRows
  }, isControlled),
      finalValue = _useInput.finalValue,
      handleInputRef = _useInput.handleInputRef,
      handleClick = _useInput.handleClick,
      handleChange = _useInput.handleChange,
      handleClear = _useInput.handleClear,
      focus = _useInput.focus,
      entered = _useInput.entered,
      handleFocus = _useInput.handleFocus,
      handleBlur = _useInput.handleBlur,
      handleMouseEnter = _useInput.handleMouseEnter,
      handleMouseLeave = _useInput.handleMouseLeave,
      handleKeyDown = _useInput.handleKeyDown,
      autosizeStyles = _useInput.autosizeStyles;

  var textareaStyleItem = useMemo(function () {
    return mergeStyleItem({
      style: autosize ? autosizeStyles : {}
    }, innerStyles.textarea);
  }, [autosize, autosizeStyles, innerStyles.textarea]);
  var exceedLimit = limit > 0 && _get(finalValue, 'length', 0) > limit; // 文字超出limit

  var hasError = exceedLimit || status === 'error';
  var innerSuffixNodeList = useMemo(function () {
    var innerSuffixNode = [];
    Children.forEach(suffixNode, function (child) {
      innerSuffixNode.push(child);
    }); // 字数限制

    if (limit) {
      innerSuffixNode.unshift( /*#__PURE__*/React.createElement(TextLimit, {
        value: finalValue,
        limit: limit
      }));
    } // 状态反馈


    if (status && hasFeedback) {
      if (status === 'loading') {
        innerSuffixNode.unshift( /*#__PURE__*/React.createElement(StyledStatusWrapper, _extends({
          theme: theme,
          status: status,
          key: "status"
        }, innerStyles.statusWrapper), /*#__PURE__*/React.createElement(Spin, null)));
      } else {
        var statusIcon = _extends({
          success: _SuccessIcon,
          error: _ClearIcon
        }, theme.components.Input.statusIcon);

        var StatusIcon = statusIcon[status];
        innerSuffixNode.unshift( /*#__PURE__*/React.createElement(StyledStatusWrapper, _extends({
          theme: theme,
          status: status,
          key: "status"
        }, innerStyles.statusWrapper), /*#__PURE__*/React.createElement(StatusIcon, null)));
      }
    } // 清除输入框功能


    if (allowClear && (finalValue || hasRelatedValue || replacedInputNode) && entered && !disabled) {
      var token = theme.components.Input;
      var ClearIcon = token.clearIcon || _ClearIcon;
      var fontSize = theme.typography.spec.fontSize;
      var clearNode = /*#__PURE__*/React.createElement(ClearIconWrapper, _extends({
        key: "clear",
        $fontSize: fontSize[token.fontLevel[size]],
        onClick: handleClear
      }, innerStyles.clearWrapper), customClearIcon || /*#__PURE__*/React.createElement(ClearIcon, null));

      if (clearReplace) {
        innerSuffixNode.shift();
      }

      innerSuffixNode.unshift(clearNode);
    }

    return innerSuffixNode;
  }, [allowClear, clearReplace, customClearIcon, disabled, entered, finalValue, handleClear, hasFeedback, hasRelatedValue, innerStyles.clearWrapper, innerStyles.statusWrapper, limit, replacedInputNode, size, status, suffixNode, theme]);
  var finalChildren = useMemo(function () {
    if (multiline) {
      return /*#__PURE__*/React.createElement(StyledTextareaWrapper, _extends({
        theme: theme,
        disabled: disabled,
        hasError: hasError,
        focus: focus,
        $width: addPx(width),
        $bordered: bordered,
        size: size,
        onClick: handleClick,
        ref: ref
      }, textareaWrapperStyleItem), /*#__PURE__*/React.createElement(StyledTextarea, _extends({
        ref: handleInputRef,
        value: finalValue,
        disabled: disabled,
        theme: theme,
        $bordered: bordered,
        $width: typeof width === 'string' && _endsWith(width, '%') ? '100%' : addPx(width),
        onFocus: handleFocus,
        onBlur: handleBlur,
        onChange: handleChange,
        limit: limit,
        size: size,
        autoFocus: autoFocus,
        onKeyDown: handleKeyDown,
        autosize: autosize,
        $height: addPx(height)
      }, textareaStyleItem, other)), /*#__PURE__*/React.createElement(StyledTextareaLimitWrapper, _extends({
        theme: theme,
        size: size
      }, innerStyles.textLimitWrapper), /*#__PURE__*/React.createElement(TextLimit, _extends({
        value: finalValue,
        limit: limit
      }, innerStyles.textLimit))));
    }

    return /*#__PURE__*/React.createElement(StyledInputWrapper, _extends({
      theme: theme,
      disabled: disabled,
      hasError: hasError,
      focus: focus,
      $bordered: bordered,
      $width: addPx(width),
      $height: addPx(height),
      size: size,
      onClick: handleClick,
      onMouseEnter: handleMouseEnter,
      onMouseLeave: handleMouseLeave,
      ref: ref
    }, inputWrapperStyleItem), /*#__PURE__*/React.createElement(ExtraNodes, {
      $type: "prefix",
      theme: theme,
      size: size,
      nodes: prefixNode,
      $nodeStyleItem: innerStyles.nodeWrapper,
      $dividerStyleItem: innerStyles.nodeDivider,
      $disabled: disabled
    }), replacedInputNode ? replacedInputNode : /*#__PURE__*/React.createElement(StyledInput, _extends({
      ref: handleInputRef,
      value: finalValue,
      disabled: disabled,
      theme: theme,
      size: size,
      onFocus: handleFocus,
      onBlur: handleBlur,
      onChange: handleChange,
      onKeyDown: handleKeyDown,
      autoFocus: autoFocus
    }, innerStyles.input, other)), /*#__PURE__*/React.createElement(ExtraNodes, {
      $type: "suffix",
      theme: theme,
      size: size,
      nodes: innerSuffixNodeList,
      $nodeStyleItem: innerStyles.nodeWrapper,
      $dividerStyleItem: innerStyles.nodeDivider,
      $disabled: disabled
    }));
  }, [autoFocus, autosize, bordered, disabled, finalValue, focus, handleBlur, handleChange, handleClick, handleFocus, handleInputRef, handleKeyDown, handleMouseEnter, handleMouseLeave, hasError, height, innerStyles.input, innerStyles.nodeDivider, innerStyles.nodeWrapper, innerStyles.textLimit, innerStyles.textLimitWrapper, innerSuffixNodeList, inputWrapperStyleItem, limit, multiline, other, prefixNode, ref, replacedInputNode, size, textareaStyleItem, textareaWrapperStyleItem, theme, width]);
  return finalChildren;
});
Input.__MUYA_INPUT = true;
export default Input;