import _extends from "@babel/runtime/helpers/extends";
import { ClearIcon as _ClearIcon } from "@muya-ui/theme-light/es/icons/ClearIcon";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["size", "disabled", "readOnly", "status", "prefixNode", "suffixNode", "middleNode", "allowClear", "clearReplace", "width", "height", "onMouseEnter", "onMouseLeave", "className", "style", "styles", "value", "defaultValue", "placeholder", "autoFocus", "onFocus", "onBlur", "onClear", "onClick", "onChange", "onKeyDown", "onPressEnter", "hasRelatedValue", "focusWhenClear", "inputProps", "clearIcon", "bordered"];
import React, { Children, useMemo, useRef } from 'react';
import ClearIconWrapper from '../styled/components/ClearIconWrapper';
import addPx from '../utils/addPx';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { ExtraNodes, StyledInput, StyledInputWrapper } from './styled';
import useRangeInput from './useRangeInput';
var defaultStyles = {
  inputWrapper: '',
  headInput: '',
  tailInput: '',
  clearWrapper: '',
  nodeWrapper: '',
  nodeDivider: ''
};
var RangeInput = memoForwardRef(function (props, ref) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      readOnly = props.readOnly,
      status = props.status,
      prefixNode = props.prefixNode,
      suffixNode = props.suffixNode,
      middleNode = props.middleNode,
      allowClear = props.allowClear,
      clearReplace = props.clearReplace,
      width = props.width,
      height = props.height,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      className = props.className,
      style = props.style,
      styles = props.styles,
      value = props.value,
      defaultValue = props.defaultValue,
      _props$placeholder = props.placeholder,
      placeholder = _props$placeholder === void 0 ? [] : _props$placeholder,
      _props$autoFocus = props.autoFocus,
      autoFocus = _props$autoFocus === void 0 ? false : _props$autoFocus,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      onClear = props.onClear,
      onClick = props.onClick,
      onChange = props.onChange,
      onKeyDown = props.onKeyDown,
      onPressEnter = props.onPressEnter,
      hasRelatedValue = props.hasRelatedValue,
      _props$focusWhenClear = props.focusWhenClear,
      focusWhenClear = _props$focusWhenClear === void 0 ? true : _props$focusWhenClear,
      inputProps = props.inputProps,
      customClearIcon = props.clearIcon,
      _props$bordered = props.bordered,
      bordered = _props$bordered === void 0 ? true : _props$bordered,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('range-input', defaultStyles, styles);
  var inputWrapperStyleItem = useMemo(function () {
    return mergeStyleItem({
      className: className,
      style: style
    }, innerStyles.inputWrapper);
  }, [className, innerStyles.inputWrapper, style]);

  var _useRangeInput = useRangeInput(props),
      finalValue = _useRangeInput.finalValue,
      focus = _useRangeInput.focus,
      entered = _useRangeInput.entered,
      leftInputRef = _useRangeInput.leftInputRef,
      rightInputRef = _useRangeInput.rightInputRef,
      handleLeftFocus = _useRangeInput.handleLeftFocus,
      handleRightFocus = _useRangeInput.handleRightFocus,
      handleBlur = _useRangeInput.handleBlur,
      handleClick = _useRangeInput.handleClick,
      handleLeftChange = _useRangeInput.handleLeftChange,
      handleRightChange = _useRangeInput.handleRightChange,
      handleClear = _useRangeInput.handleClear,
      handleMouseEnter = _useRangeInput.handleMouseEnter,
      handleMouseLeave = _useRangeInput.handleMouseLeave,
      handleKeyDown = _useRangeInput.handleKeyDown;

  var innerRef = useRef(null);
  var more = useRef({
    focus: function focus() {
      var position = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var option = arguments.length > 1 ? arguments[1] : undefined;

      if (position === 1) {
        rightInputRef.current.focus(option);
      } else {
        leftInputRef.current.focus(option);
      }
    },
    blur: function blur() {
      var position = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

      if (position === 1) {
        rightInputRef.current.blur();
      } else {
        leftInputRef.current.blur();
      }
    }
  });

  var handleRef = _useForkRef(ref, innerRef, more.current);

  var innerSuffixNode = [];
  Children.forEach(suffixNode, function (child) {
    innerSuffixNode.push(child);
  });

  var _finalValue = _slicedToArray(finalValue, 2),
      leftValue = _finalValue[0],
      rightValue = _finalValue[1]; // 清除输入框功能


  if (allowClear && (leftValue || rightValue || hasRelatedValue) && entered && !disabled) {
    var fontSize = theme.typography.spec.fontSize,
        token = theme.components.Input;
    var ClearIcon = token.clearIcon || _ClearIcon;
    var clearNode = /*#__PURE__*/React.createElement(ClearIconWrapper, _extends({
      key: "clear",
      $fontSize: fontSize[token.fontLevel[size]],
      onClick: handleClear
    }, innerStyles.clearWrapper), customClearIcon || /*#__PURE__*/React.createElement(ClearIcon, null));

    if (clearReplace) {
      innerSuffixNode.shift();
    }

    innerSuffixNode.unshift(clearNode);
  }

  var inputWidth = width !== undefined ? width : theme.components.Input.rangeInputWidth[size];
  return /*#__PURE__*/React.createElement(StyledInputWrapper, _extends({
    theme: theme,
    disabled: disabled,
    focus: focus,
    $bordered: bordered,
    $width: addPx(inputWidth),
    $height: addPx(height),
    size: size,
    onClick: handleClick,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave,
    hasError: status === 'error',
    ref: handleRef
  }, inputWrapperStyleItem, other), /*#__PURE__*/React.createElement(ExtraNodes, {
    $type: "prefix",
    theme: theme,
    size: size,
    nodes: prefixNode,
    $nodeStyleItem: innerStyles.nodeWrapper,
    $dividerStyleItem: innerStyles.nodeDivider,
    $disabled: disabled
  }), /*#__PURE__*/React.createElement(StyledInput, _extends({
    value: leftValue,
    disabled: disabled,
    theme: theme,
    onFocus: handleLeftFocus,
    onBlur: handleBlur,
    onChange: handleLeftChange,
    onKeyDown: handleKeyDown,
    autoFocus: autoFocus,
    placeholder: placeholder[0],
    size: size,
    ref: leftInputRef,
    readOnly: readOnly
  }, inputProps)), /*#__PURE__*/React.createElement(ExtraNodes, {
    $type: "suffix",
    theme: theme,
    size: size,
    nodes: middleNode,
    full: true,
    $nodeStyleItem: innerStyles.nodeWrapper,
    $dividerStyleItem: innerStyles.nodeDivider,
    $disabled: disabled
  }), /*#__PURE__*/React.createElement(StyledInput, _extends({
    value: rightValue,
    disabled: disabled,
    theme: theme,
    onFocus: handleRightFocus,
    onBlur: handleBlur,
    onChange: handleRightChange,
    onKeyDown: handleKeyDown,
    autoFocus: autoFocus,
    size: size,
    placeholder: placeholder[1],
    ref: rightInputRef,
    readOnly: readOnly
  }, inputProps)), /*#__PURE__*/React.createElement(ExtraNodes, {
    $type: "suffix",
    theme: theme,
    size: size,
    nodes: innerSuffixNode,
    $nodeStyleItem: innerStyles.nodeWrapper,
    $dividerStyleItem: innerStyles.nodeDivider,
    $disabled: disabled
  }));
});
RangeInput.__MUYA_RANGEINPUT = true;
export default RangeInput;