import _extends from "@babel/runtime/helpers/extends";
import { ClearIcon as _ClearIcon } from "@muya-ui/theme-light/es/icons/ClearIcon";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _typeof from "@babel/runtime/helpers/typeof";
import _omit from "lodash/omit";
var _excluded = ["size", "width", "disabled", "readOnly", "allowClear", "hasRelatedValue", "autoFocus", "scrollTopAfterEffect", "maxTagWidth", "maxVerticalTagCount", "disabledValue", "readOnlyValue", "suffixNode", "clearIcon", "className", "style", "styles", "inputRef", "status", "getTagText", "renderTag", "horizontalScroll", "inputReadOnly", "bordered"];
import React, { useMemo, useRef } from 'react';
import ClearIconWrapper from '../styled/components/ClearIconWrapper';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import InputTag from './InputTag';
import { StyledInputSizer, StyledTagsInput, StyledTagsInputContentWrapper, StyledTagsInputSuffixNodeWrapper, StyledTagsInputWrapper, StyledTagsWrapper } from './styled';
import useTagsInput, { isTagDisabled } from './useTagsInput';

var noop = function noop() {};

var defaultGetTagText = function defaultGetTagText(value) {
  return value && _typeof(value) === 'object' ? value.label || value.value : value;
};

var defaultStyles = {
  inputWrapper: '',
  contentWrapper: '',
  tagsWrapper: '',
  input: '',
  tag: ''
};
var TagsInput = memoForwardRef(function (props, ref) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      width = props.width,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      _props$readOnly = props.readOnly,
      readOnly = _props$readOnly === void 0 ? false : _props$readOnly,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? false : _props$allowClear,
      hasRelatedValue = props.hasRelatedValue,
      _props$autoFocus = props.autoFocus,
      autoFocus = _props$autoFocus === void 0 ? false : _props$autoFocus,
      _props$scrollTopAfter = props.scrollTopAfterEffect,
      scrollTopAfterEffect = _props$scrollTopAfter === void 0 ? false : _props$scrollTopAfter,
      maxTagWidth = props.maxTagWidth,
      _props$maxVerticalTag = props.maxVerticalTagCount,
      maxVerticalTagCount = _props$maxVerticalTag === void 0 ? 2.5 : _props$maxVerticalTag,
      _props$disabledValue = props.disabledValue,
      disabledValue = _props$disabledValue === void 0 ? [] : _props$disabledValue,
      _props$readOnlyValue = props.readOnlyValue,
      readOnlyValue = _props$readOnlyValue === void 0 ? [] : _props$readOnlyValue,
      suffixNode = props.suffixNode,
      clearIcon = props.clearIcon,
      className = props.className,
      style = props.style,
      styles = props.styles,
      inputRef = props.inputRef,
      status = props.status,
      _props$getTagText = props.getTagText,
      getTagText = _props$getTagText === void 0 ? defaultGetTagText : _props$getTagText,
      renderTag = props.renderTag,
      _props$horizontalScro = props.horizontalScroll,
      horizontalScroll = _props$horizontalScro === void 0 ? false : _props$horizontalScro,
      _props$inputReadOnly = props.inputReadOnly,
      inputReadOnly = _props$inputReadOnly === void 0 ? false : _props$inputReadOnly,
      _props$bordered = props.bordered,
      bordered = _props$bordered === void 0 ? true : _props$bordered,
      restProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('tags-input', defaultStyles, styles);
  var inputWrapperStyleItem = useMemo(function () {
    return mergeStyleItem({
      className: className,
      style: style
    }, innerStyles.inputWrapper);
  }, [className, innerStyles.inputWrapper, style]);
  var fontSize = theme.typography.spec.fontSize,
      token = theme.components.Input;
  var sizerRef = useRef(null);
  var contentRef = useRef(null);
  var innerInputRef = useRef(null);

  var handleInputRef = _useForkRef(inputRef, innerInputRef);

  var _useTagsInput = useTagsInput(props, contentRef, sizerRef, innerInputRef),
      focus = _useTagsInput.focus,
      entered = _useTagsInput.entered,
      value = _useTagsInput.value,
      inputValue = _useTagsInput.inputValue,
      placeholder = _useTagsInput.placeholder,
      hasValue = _useTagsInput.hasValue,
      hasError = _useTagsInput.hasError,
      handleFocus = _useTagsInput.handleFocus,
      handleBlur = _useTagsInput.handleBlur,
      handleMouseEnter = _useTagsInput.handleMouseEnter,
      handleMouseLeave = _useTagsInput.handleMouseLeave,
      handleClick = _useTagsInput.handleClick,
      handleClear = _useTagsInput.handleClear,
      handleRemoveTag = _useTagsInput.handleRemoveTag,
      handleInputKeyDown = _useTagsInput.handleInputKeyDown,
      handleInput = _useTagsInput.handleInput;

  var renderSuffixNode = function renderSuffixNode() {
    var clearNode = null;

    if (allowClear && (hasValue || hasRelatedValue) && !disabled && entered) {
      var SuffixIcon = token.clearIcon || _ClearIcon;
      clearNode = /*#__PURE__*/React.createElement(ClearIconWrapper, {
        key: "clear",
        $fontSize: fontSize[token.fontLevel[size]],
        onClick: handleClear
      }, clearIcon || /*#__PURE__*/React.createElement(SuffixIcon, null));
    }

    if (clearNode) {
      return /*#__PURE__*/React.createElement(StyledTagsInputSuffixNodeWrapper, null, clearNode);
    }

    if (suffixNode) {
      return /*#__PURE__*/React.createElement(StyledTagsInputSuffixNodeWrapper, null, suffixNode);
    }
  };

  var inputProps = _omit(restProps, ['value', 'defaultValue', 'placeholder', 'inputValue', 'allowInput', 'focusWhenClear', 'defaultInputValue', 'onAddTag', 'onRemoveTag', 'onFocus', 'onBlur', 'onInputKeyDown', 'onKeyDown', 'onClear', 'onInput', 'onInputChange', 'onClick', 'onMouseEnter', 'onMouseLeave']);

  return /*#__PURE__*/React.createElement(StyledTagsInputWrapper, _extends({
    theme: theme,
    ref: ref,
    $disabled: disabled,
    $focus: focus,
    $width: width,
    $maxTagWidth: maxTagWidth,
    $maxVerticalTagCount: maxVerticalTagCount,
    $size: size,
    $hasValue: hasValue,
    $hasError: hasError,
    $horizontalScroll: horizontalScroll,
    $bordered: bordered,
    onClick: handleClick,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave
  }, inputWrapperStyleItem), /*#__PURE__*/React.createElement(StyledTagsInputContentWrapper, _extends({}, innerStyles.contentWrapper, {
    ref: contentRef
  }), /*#__PURE__*/React.createElement(StyledTagsWrapper, innerStyles.tagsWrapper, value.map(function (value, index) {
    var tagDisabled = disabled || isTagDisabled(disabledValue, value);
    var tagReadOnly = readOnly || isTagDisabled(readOnlyValue, value);
    var tagProps = {
      size: size,
      onClose: function onClose(e) {
        return handleRemoveTag(index, e);
      },
      closable: !tagDisabled && !tagReadOnly,
      disabled: tagDisabled,
      readOnly: tagReadOnly
    };

    if (typeof renderTag === 'function') {
      return renderTag(_extends({
        value: value,
        tagText: getTagText(value)
      }, tagProps, innerStyles.tag));
    }

    return /*#__PURE__*/React.createElement(InputTag, _extends({
      key: value && _typeof(value) === 'object' ? value.value : value,
      theme: theme,
      shape: "rect"
    }, tagProps, innerStyles.tag), getTagText(value));
  }), /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(StyledTagsInput, _extends({}, inputProps, {
    ref: handleInputRef,
    type: "text",
    placeholder: placeholder,
    value: inputValue,
    onChange: noop,
    onFocus: handleFocus,
    onBlur: handleBlur,
    onInput: handleInput,
    onKeyDown: handleInputKeyDown,
    autoFocus: autoFocus,
    disabled: disabled,
    readOnly: readOnly || inputReadOnly
  }, innerStyles.input)), /*#__PURE__*/React.createElement(StyledInputSizer, {
    ref: sizerRef
  }, inputValue || placeholder)))), renderSuffixNode());
});
TagsInput.__MUYA_TAGSINPUT = true;
export default TagsInput;