import _extends from "@babel/runtime/helpers/extends";
import _isInteger from "lodash/isInteger";
import React, { Children, useMemo } from 'react';
import styled, { css } from 'styled-components';
import { scrollBarStyle } from '../ScrollView/styled';
import ellipsisStyle from '../styled/mixins/ellipsis';
import addPx from '../utils/addPx';
import InputTag from './InputTag';
export function makeWrapperStyle(props) {
  var theme = props.theme,
      size = props.size,
      disabled = props.disabled,
      focus = props.focus,
      $width = props.$width,
      $height = props.$height,
      $bordered = props.$bordered,
      hasError = props.hasError;
  var colorsPattern = theme.colors.pattern,
      _theme$typography$spe = theme.typography.spec,
      fontSize = _theme$typography$spe.fontSize,
      lineHeight = _theme$typography$spe.lineHeight,
      _theme$transition$pat = theme.transition.pattern,
      easing = _theme$transition$pat.easing,
      duration = _theme$transition$pat.duration,
      token = theme.components.Input;
  var fontLevel = token.fontLevel[size]; // 错误样式

  var errorStyle = css(["border-color:", ";box-shadow:", ";"], colorsPattern.feature.error, token.errorFocusShadow); // 聚焦样式

  var focusStyle = css(["border-color:", ";box-shadow:", ";"], token.borderColor.focused, token.focusShadow); // 禁用样式

  var disabledStyle = css(["background:", ";cursor:not-allowed;input,svg{cursor:not-allowed;}"], colorsPattern.background.disabled);
  var widthStyle = $width && css(["width:", ";"], $width);
  var heightStyle = $height && css(["height:", ";"], $height);
  var borderStyle = $bordered && css(["border:1px solid ", ";"], token.borderColor.normal);
  return css(["display:inline-flex;position:relative;box-sizing:border-box;align-items:center;font-size:", "px;line-height:", "px;border-radius:", ";", " transition:border ", "ms ", ",width ", "ms ", ";vertical-align:middle;background-color:", ";&:hover{border-color:", ";}", " ", " ", " ", " ", ""], fontSize[fontLevel], lineHeight[fontLevel], token.borderRadius[size], borderStyle, duration.status, easing.status, duration.status, easing.status, token.bgColor, !(focus || disabled || hasError) && token.borderColor.hover, widthStyle, heightStyle, focus && focusStyle, hasError && errorStyle, disabled && disabledStyle);
}
export function makeInputWrapperStyle(props) {
  var theme = props.theme,
      size = props.size;
  var _theme$components$Inp = theme.components.Input,
      inputPadding = _theme$components$Inp.inputPadding,
      inputWidth = _theme$components$Inp.inputWidth,
      inputHeight = _theme$components$Inp.inputHeight;
  return css(["width:", "px;height:", "px;padding:0 ", "px;"], inputWidth[size], inputHeight[size], inputPadding[size]);
}
export function makeTextareaStyle(props) {
  var theme = props.theme,
      size = props.size,
      limit = props.limit,
      _props$resize = props.resize,
      resize = _props$resize === void 0 ? 'vertical' : _props$resize,
      $height = props.$height,
      autosize = props.autosize,
      $width = props.$width;
  var _theme$typography$spe2 = theme.typography.spec,
      fontSize = _theme$typography$spe2.fontSize,
      lineHeight = _theme$typography$spe2.lineHeight,
      token = theme.components.Input;
  var heightStyle = $height && css(["height:", ";"], $height);
  var fontLevel = token.fontLevel[size];
  var padding = autosize ? token.autosizeTextareaPadding : token.textareaPadding;
  var currentPadding = padding[size];
  var space = currentPadding[0];

  switch (currentPadding.length) {
    case 1:
    case 2:
      space *= 2;
      break;

    default:
      space += currentPadding[2];
      break;
  }

  var width = $width ? $width : "".concat(token.textareaWidth[size], "px");
  var limitStyle = limit && limit > 0 && css(["::-webkit-scrollbar{width:0;}border-bottom:", "px solid transparent;padding-bottom:0;-ms-overflow-style:none;resize:none;"], token.textareaLimitMarginBottom[size]);
  return css(["resize:", ";width:", ";height:", "px;font-size:", "px;line-height:", "px;min-height:", "px;padding:", ";box-sizing:border-box;", " ", ""], resize, width, token.textareaHeight[size], fontSize[fontLevel], lineHeight[fontLevel], lineHeight[fontLevel] + space, currentPadding.map(function (p) {
    return "".concat(p, "px ");
  }), heightStyle, limitStyle);
}
export function makeInputStyle(props) {
  var _props$theme = props.theme,
      colors = _props$theme.colors,
      _props$theme$typograp = _props$theme.typography.spec,
      fontSize = _props$theme$typograp.fontSize,
      lineHeight = _props$theme$typograp.lineHeight,
      token = _props$theme.components.Input,
      disabled = props.disabled,
      size = props.size;
  var fontLevel = token.fontLevel[size];
  return css(["display:inline-block;outline:none;border:0;padding:0;margin:0;flex:1;min-width:0;background:none;font-size:", "px;line-height:", "px;caret-color:", ";color:", ";", " &::selection{background:", ";}&::placeholder{color:", ";}"], fontSize[fontLevel], lineHeight[fontLevel], colors.spec.brand, disabled ? colors.pattern.text.disabled : token.color, disabled && css(["-webkit-text-fill-color:currentColor;opacity:1;"]), token.selectionBackground, colors.pattern.text.placeholder);
}
export function makeStatusWrapperStyle(props) {
  var theme = props.theme,
      status = props.status;

  if (status) {
    return css(["line-height:0;cursor:default;color:", ";"], theme.colors.pattern.feature[status]);
  }
}
export function makeTextareaLimitWrapper(props) {
  var theme = props.theme,
      size = props.size;
  var _theme$components$Inp2 = theme.components.Input.textareaLimitPosition[size],
      right = _theme$components$Inp2.right,
      bottom = _theme$components$Inp2.bottom;
  return css(["position:absolute;right:", "px;bottom:", "px;"], right, bottom);
}
export var StyledInputWrapper = styled.div.withConfig({
  displayName: "StyledInputWrapper"
})(["", " ", ""], makeInputWrapperStyle, makeWrapperStyle);
export var StyledTextareaWrapper = styled.div.withConfig({
  displayName: "StyledTextareaWrapper"
})(["", ""], makeWrapperStyle);
export var StyledInput = styled.input.withConfig({
  displayName: "StyledInput"
})(["", ""], makeInputStyle);
export var StyledTextarea = styled.textarea.withConfig({
  displayName: "StyledTextarea"
})(["", " ", ""], makeInputStyle, makeTextareaStyle);

function makeAddonNodeMarginStyle(props) {
  var theme = props.theme,
      size = props.size,
      $type = props.$type,
      $small = props.$small;
  var spacing = theme.components.Input.inputPadding[size];
  var left = theme.components.Input.inputAddonNodeMargin[size];
  var marginRight = spacing;
  var marginLeft = spacing;

  if ($small === 'prefix') {
    marginRight = left;
  } else if ($small === 'suffix') {
    marginLeft = left;
  }

  return $type === 'prefix' ? css(["margin-right:", "px;"], marginRight) : css(["margin-left:", "px;"], marginLeft);
}

export var StyledNodeWrapper = styled.div.withConfig({
  displayName: "StyledNodeWrapper"
})(["", ""], function (props) {
  var theme = props.theme,
      $disabled = props.$disabled,
      children = props.children;
  var token = theme.components.Input;
  var _theme$colors$pattern = theme.colors.pattern.text,
      darktip = _theme$colors$pattern.darktip,
      secondary = _theme$colors$pattern.secondary,
      assistant = _theme$colors$pattern.assistant; // 字符串节点固定颜色。React节点有特殊颜色

  var elementStyle = typeof children !== 'string' && css(["color:", ";&:hover{color:", ";}&:active{color:", ";}cursor:", ";"], darktip, secondary, assistant, $disabled ? 'not-allowed' : 'pointer');
  return css(["", " line-height:0;color:", ";flex-shrink:0;outline:none;cursor:default;user-select:none;display:flex;height:100%;align-items:center;", ""], makeAddonNodeMarginStyle(props), token.color, elementStyle);
});
export var StyledDivider = styled.div.withConfig({
  displayName: "StyledDivider"
})(["", ""], function (props) {
  var theme = props.theme,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size;
  var fontLevel = theme.components.Input.fontLevel[size];
  return css(["", " width:1px;height:", "px;flex-shrink:0;cursor:default;user-select:none;background:", ";"], makeAddonNodeMarginStyle(props), theme.typography.spec.fontSize[fontLevel], theme.colors.pattern.background.divider);
});
export function ExtraNodes(props) {
  var nodes = props.nodes,
      $type = props.$type,
      size = props.size,
      theme = props.theme,
      full = props.full,
      $nodeStyleItem = props.$nodeStyleItem,
      $dividerStyleItem = props.$dividerStyleItem,
      $disabled = props.$disabled;
  return useMemo(function () {
    if (!nodes) {
      return null;
    }

    var childNum = Children.count(nodes);
    return /*#__PURE__*/React.createElement(React.Fragment, null, Children.map(nodes, function (child, index) {
      if (!child) return null;
      var $small;

      if (!full && index === childNum - 1 && $type === 'prefix') {
        $small = 'prefix';
      }

      return /*#__PURE__*/React.createElement(React.Fragment, {
        key: index
      }, index > 0 && /*#__PURE__*/React.createElement(StyledDivider, _extends({
        $type: $type,
        $disabled: $disabled,
        theme: theme,
        size: size
      }, $dividerStyleItem)), /*#__PURE__*/React.createElement(StyledNodeWrapper, _extends({
        $type: $type,
        theme: theme,
        size: size
      }, $nodeStyleItem, {
        $small: $small,
        $disabled: $disabled
      }), child));
    }), full && /*#__PURE__*/React.createElement(StyledNodeWrapper, _extends({
      $type: $type,
      $disabled: $disabled,
      theme: theme,
      size: size
    }, $nodeStyleItem)));
  }, [$disabled, $dividerStyleItem, $nodeStyleItem, $type, full, nodes, size, theme]);
}
export var StyledStatusWrapper = styled.div.withConfig({
  displayName: "StyledStatusWrapper"
})(["", ""], makeStatusWrapperStyle);
export var StyledTextareaLimitWrapper = styled.div.withConfig({
  displayName: "StyledTextareaLimitWrapper"
})(["", ""], makeTextareaLimitWrapper); // tags input

export var StyledTagsInputContentWrapper = styled.div.withConfig({
  displayName: "StyledTagsInputContentWrapper"
})(["position:relative;box-sizing:border-box;overflow-x:hidden;overflow-y:auto;outline:none;"]);
export var StyledTagsWrapper = styled.div.withConfig({
  displayName: "StyledTagsWrapper"
})(["align-self:flex-start;display:flex;flex-wrap:wrap;align-items:center;"]);
export var StyledTagsInput = styled.input.withConfig({
  displayName: "StyledTagsInput"
})(["display:inline-block;overflow:hidden;background:transparent;border-width:0;margin:0;padding:0;outline:0;margin-bottom:4px;flex-shrink:0;"]);
export var StyledInputSizer = styled.div.withConfig({
  displayName: "StyledInputSizer"
})(["position:absolute;top:0;left:0;visibility:hidden;height:0;overflow:scroll;white-space:pre;line-height:1;"]);
export var StyledTagsInputSuffixNodeWrapper = styled.div.withConfig({
  displayName: "StyledTagsInputSuffixNodeWrapper"
})([""]); // TODO: 样式重构
// eslint-disable-next-line complexity

var tagsInputCss = function tagsInputCss(props) {
  var _props$theme2 = props.theme,
      colorsPattern = _props$theme2.colors.pattern,
      fontSize = _props$theme2.typography.spec.fontSize,
      sizeSpec = _props$theme2.size.spec,
      _props$theme2$transit = _props$theme2.transition.pattern,
      duration = _props$theme2$transit.duration,
      easing = _props$theme2$transit.easing,
      token = _props$theme2.components.Input,
      $size = props.$size,
      $width = props.$width,
      $focus = props.$focus,
      $disabled = props.$disabled,
      $hasValue = props.$hasValue,
      $maxVerticalTagCount = props.$maxVerticalTagCount,
      $maxTagWidth = props.$maxTagWidth,
      $hasError = props.$hasError,
      $horizontalScroll = props.$horizontalScroll,
      $bordered = props.$bordered;
  var hasStatus = $focus || $disabled || $hasError;
  var finalWidth = $width || sizeSpec.width[$size];
  var computedMaxTagWidth = $maxTagWidth;

  if (!$maxTagWidth && typeof finalWidth === 'number') {
    computedMaxTagWidth = finalWidth - token.tagsInput.contentPaddingRight[$size] - token.inputTag.outerPadding * 2;
  }

  var fontLevel = token.fontLevel[$size];
  var disabledCss = css(["background:", ";border:1px solid ", ";cursor:not-allowed;"], colorsPattern.background.disabled, token.borderColor.normal); // 错误样式

  var errorStyle = css(["border-color:", ";box-shadow:", ";"], colorsPattern.feature.error, $focus ? token.errorFocusShadow : 'none');
  var borderStyle = $bordered && css(["border:1px solid ", ";"], $focus ? token.borderColor.focused : token.borderColor.normal);
  return css(["position:relative;display:inline-flex;overflow:hidden;outline:none;box-sizing:border-box;width:", ";", " box-shadow:", ";border-radius:", ";background:", ";line-height:1;font-size:", "px;transition:border ", "ms ", ";cursor:text;&:hover{border-color:", ";}", ";", ";", " ", " ", "{padding-left:", "px;padding-right:", "px;", ";", ";", ";}", "{", ";}", "{width:1px;color:", ";font-size:", "px;}input{&[readonly]{cursor:", ";}&::placeholder{color:", ";}}", "{font-size:", "px;}", "{", " ", ";}", "{max-width:", "px;}"], addPx(finalWidth), borderStyle, $focus ? token.focusShadow : 'none', token.borderRadius[$size], token.bgColor, fontSize[fontLevel], duration.status, easing.status, !hasStatus && token.borderColor.hover, $hasValue && css(["padding:", ";"], token.tagsInput.wrapperPadding), $horizontalScroll && css(["justify-content:space-between;"]), $disabled && disabledCss, $hasError && errorStyle, StyledTagsInputContentWrapper, token.inputPadding[$size], token.tagsInput.contentPaddingRight[$size], $hasValue && css(["max-height:", "px;padding-left:0;"], $maxVerticalTagCount * (token.inputHeight[$size] - 10) + Math.ceil($maxVerticalTagCount - 1) * 4 + (_isInteger($maxVerticalTagCount) ? 5 : 0)), $horizontalScroll && css(["padding-right:0;overflow-x:auto;"]), scrollBarStyle({
    theme: props.theme,
    hideEndButton: true,
    hidden: $horizontalScroll
  }), StyledTagsInputSuffixNodeWrapper, $horizontalScroll ? css(["margin:0 ", "px ", "px ", "px;color:", ";flex-shrink:0;outline:none;cursor:default;user-select:none;display:inline-flex;align-items:center;line-height:1;"], $hasValue ? token.inputPadding[$size] - 4 : token.inputPadding[$size], $hasValue ? 4 : 0, token.inputPadding[$size], token.color) : css(["position:absolute;top:50%;right:", "px;transform:translateY(-50%);margin:auto;"], token.inputPadding[$size]), StyledTagsInput, colorsPattern.text.text, fontSize[fontLevel], $disabled ? 'not-allowed' : 'pointer', colorsPattern.text.placeholder, StyledInputSizer, fontSize[fontLevel], StyledTagsWrapper, !$hasValue && css(["height:", "px;", "{margin-bottom:0;}"], token.inputHeight[$size] - 2, StyledTagsInput), $horizontalScroll && "flex-wrap: nowrap", InputTag, computedMaxTagWidth);
};

export var StyledTagsInputWrapper = styled.div.withConfig({
  displayName: "StyledTagsInputWrapper"
})(["", ""], tagsInputCss);
export var StyledPopperInputWrapper = styled.div.withConfig({
  displayName: "StyledPopperInputWrapper"
})(["position:relative;display:inline-block;outline:none;", "{", ";}", ""], StyledInput, ellipsisStyle, function (props) {
  var $pointer = props.$pointer,
      $popupVisible = props.$popupVisible,
      $width = props.$width,
      $status = props.$status,
      $bordered = props.$bordered,
      theme = props.theme;
  var openStyle;
  var widthStyle;
  var pointerStyle;
  var borderStyle;

  if ($popupVisible && $status !== 'error') {
    openStyle = css(["", ",", "{border-color:", ";}"], StyledTagsInputWrapper, StyledInputWrapper, theme.components.Input.borderColor.focused);
  }

  if ($width) {
    widthStyle = css(["width:", ";"], $width);
  }

  if ($pointer) {
    pointerStyle = css(["", ",", "{cursor:pointer;input,svg{cursor:pointer;}}"], StyledInputWrapper, StyledTagsInputWrapper);
  }

  if ($bordered === false) {
    borderStyle = css(["", ",", "{border:none;}"], StyledInputWrapper, StyledTagsInputWrapper);
  }

  return css(["", ";", ";", ";", ";"], widthStyle, openStyle, pointerStyle, borderStyle);
});