import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import ResizeObserver from 'resize-observer-polyfill';
import forkHandler from '../utils/forkHandler';
import calculateNodeHeight from './calculateNodeHeight';
import useInputBase from './useInputBase';
export default function useInput(props, isControlled) {
  var _props$defaultValue = props.defaultValue,
      defaultValue = _props$defaultValue === void 0 ? '' : _props$defaultValue,
      _props$value = props.value,
      value = _props$value === void 0 ? '' : _props$value,
      _props$autoFocus = props.autoFocus,
      autoFocus = _props$autoFocus === void 0 ? false : _props$autoFocus,
      disabled = props.disabled,
      inputRefProp = props.inputRef,
      onClear = props.onClear,
      onChange = props.onChange,
      onClick = props.onClick,
      onKeyDown = props.onKeyDown,
      onPressEnter = props.onPressEnter,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      focusWhenClear = props.focusWhenClear,
      multiline = props.multiline,
      autosize = props.autosize,
      minRows = props.minRows,
      maxRows = props.maxRows,
      formatter = props.formatter,
      parser = props.parser;

  var _useState = useState(defaultValue),
      _useState2 = _slicedToArray(_useState, 2),
      innerValue = _useState2[0],
      setInnerValue = _useState2[1];

  var _useState3 = useState(defaultValue),
      _useState4 = _slicedToArray(_useState3, 2),
      showValue = _useState4[0],
      setShowValue = _useState4[1];

  var inputRef = useRef(null);

  var handleInputRef = _useForkRef(inputRef, inputRefProp);

  var observerRef = useRef();

  var _useState5 = useState({}),
      _useState6 = _slicedToArray(_useState5, 2),
      autosizeStyles = _useState6[0],
      setTextareaStyles = _useState6[1];

  var baseResult = useInputBase({
    autoFocus: autoFocus,
    disabled: disabled,
    onFocus: onFocus,
    onBlur: onBlur,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave
  });
  var handleClick = useMemo(function () {
    return forkHandler(function (e) {
      if (inputRef.current && e.currentTarget === e.target) {
        inputRef.current.focus();
      }
    }, onClick);
  }, [onClick]);
  var setInputValue = useCallback(function (value, e) {
    var isBlur = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var newValue = value;

    if (!isBlur) {
      newValue = parser ? parser(value) : value;
      setShowValue(value);

      if (!isControlled) {
        setInnerValue(newValue);
      }
    }

    if (onChange) {
      var event = e;

      if (e.type === 'click') {
        // click clear icon
        var inputTarget = inputRef.current;
        event = Object.create(e);
        event.target = inputTarget;
        event.currentTarget = inputTarget;
        var originalInputValue = inputTarget.value; // change input value cause e.target.value should be '' when clear input

        inputTarget.value = '';
        onChange(event); // reset input value

        inputTarget.value = originalInputValue;
        return;
      } else {
        // @ts-ignore
        event.target.value = value;
      }

      onChange(event);
    }
  }, [isControlled, onChange, parser]);
  var resizeTextarea = useCallback(function () {
    if (autosize && multiline && inputRef.current) {
      setTextareaStyles(calculateNodeHeight(inputRef.current, isControlled, minRows, maxRows));
    }
  }, [autosize, isControlled, maxRows, minRows, multiline]);
  var handleChange = useCallback(function (e) {
    setInputValue(e.target.value, e);
  }, [setInputValue]);
  var handleClear = useCallback(function (e) {
    e.stopPropagation();
    setInputValue('', e);

    if (onClear) {
      onClear(e);
    } // 重新聚焦的动作等清楚动作完成后再聚焦，不然会导致点一下没有办法清空值


    setTimeout(function () {
      if (focusWhenClear && inputRef.current) {
        inputRef.current.focus();
      }
    }, 0);
  }, [focusWhenClear, onClear, setInputValue]);
  var handleKeyDown = useMemo(function () {
    return forkHandler(function (e) {
      if (e.keyCode === 13 && onPressEnter) {
        onPressEnter(e);
      }
    }, onKeyDown);
  }, [onKeyDown, onPressEnter]);
  var handleFocus = useCallback(function (e) {
    var val = isControlled ? value : innerValue;
    var showVal = formatter ? formatter(val) : val;
    setShowValue(showVal);
    baseResult.handleFocus(e);
  }, [baseResult, formatter, innerValue, isControlled, value]);
  var handleBlur = useCallback(function (e) {
    if (parser) {
      setInputValue(parser(showValue), e, true);
    }

    baseResult.handleBlur(e);
  }, [baseResult, parser, setInputValue, showValue]);

  if (disabled) {
    inputRef.current && inputRef.current.blur();
  }

  useEffect(function () {
    // textarea autosize
    if (!observerRef.current && inputRef.current && multiline && autosize) {
      observerRef.current = new ResizeObserver(resizeTextarea);
      observerRef.current.observe(inputRef.current);
    }

    return function () {
      if (observerRef.current) {
        observerRef.current.disconnect();
        observerRef.current = undefined;
      }
    };
  }, [autosize, multiline, resizeTextarea]);
  useEffect(function () {
    resizeTextarea();
  }, [resizeTextarea]);
  useEffect(function () {
    // formatter 函数存在且受控的情况下，只有失焦的时候才更新showValue
    if (isControlled && formatter && !baseResult.focus) {
      setShowValue(formatter(value));
    } else if (isControlled && !formatter) {
      // 其他受控情况下直接更新值即可
      setShowValue(value);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [value]);
  var currentValue = isControlled ? value : innerValue;
  var finalValue = baseResult.focus ? showValue : formatter ? formatter(currentValue) : currentValue;
  useEffect(function () {
    resizeTextarea(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [finalValue]);
  return _extends({}, baseResult, {
    finalValue: finalValue,
    handleClick: handleClick,
    handleChange: handleChange,
    handleClear: handleClear,
    handleKeyDown: handleKeyDown,
    handleInputRef: handleInputRef,
    handleFocus: handleFocus,
    handleBlur: handleBlur,
    autosizeStyles: autosizeStyles
  });
}