import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useCallback, useMemo, useRef, useState } from 'react';
import forkHandler from '../utils/forkHandler';
import useInputBase from './useInputBase';
import { getRangeDefaultValue } from './utils';
export default function useRangeInput(props) {
  var autoFocus = props.autoFocus,
      disabled = props.disabled,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      onClick = props.onClick,
      value = props.value,
      defaultValue = props.defaultValue,
      onChange = props.onChange,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      focusWhenClear = props.focusWhenClear,
      onKeyDown = props.onKeyDown,
      onPressEnter = props.onPressEnter,
      onClear = props.onClear;
  var leftInputRef = useRef(null);
  var rightInputRef = useRef(null);
  var baseResult = useInputBase({
    autoFocus: autoFocus,
    disabled: disabled,
    onBlur: onBlur,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave
  });
  var valueFromProps = useMemo(function () {
    if (value) {
      return getRangeDefaultValue(value);
    }

    if (defaultValue) {
      return getRangeDefaultValue(defaultValue);
    }

    return ['', ''];
  }, [defaultValue, value]);

  var _useState = useState(valueFromProps),
      _useState2 = _slicedToArray(_useState, 2),
      innerValue = _useState2[0],
      setInnerValue = _useState2[1];

  var finalValue = 'value' in props ? valueFromProps : innerValue;
  var handleClick = useMemo(function () {
    return forkHandler(function (e) {
      if (leftInputRef.current && e.currentTarget === e.target) {
        leftInputRef.current.focus();
      }
    }, onClick);
  }, [onClick]);
  var updateValue = useCallback(function (newValue) {
    setInnerValue(newValue);

    if (onChange) {
      // TODO: 把
      onChange({
        target: {
          type: 'range-input'
        },
        value: newValue
      });
    }
  }, [onChange]);
  var setInputValue = useCallback(function (v, inputPos) {
    var _finalValue = _slicedToArray(finalValue, 2),
        oldLeft = _finalValue[0],
        oldRight = _finalValue[1];

    var newValue = [oldLeft, oldRight];
    newValue[inputPos] = v;
    updateValue(newValue);
  }, [finalValue, updateValue]);
  var handleLeftChange = useCallback(function (e) {
    setInputValue(e.target.value, 0);
  }, [setInputValue]);
  var handleRightChange = useCallback(function (e) {
    setInputValue(e.target.value, 1);
  }, [setInputValue]);

  if (disabled) {
    leftInputRef.current && leftInputRef.current.blur();
    rightInputRef.current && rightInputRef.current.blur();
  }

  var handleClear = useCallback(function (e) {
    e.stopPropagation();
    updateValue(['', '']);

    if (focusWhenClear && leftInputRef.current) {
      leftInputRef.current.focus();
    }

    if (onClear) {
      onClear({
        value: ['', '']
      });
    }
  }, [focusWhenClear, onClear, updateValue]);
  var handleKeyDown = useMemo(function () {
    return forkHandler(function (e) {
      if (e.keyCode === 13 && onPressEnter) {
        onPressEnter({
          value: finalValue
        });
      }
    }, onKeyDown);
  }, [finalValue, onKeyDown, onPressEnter]);
  var handleLeftFocus = useCallback(function (e) {
    onFocus && onFocus(e, 0);
  }, [onFocus]);
  var handleRightFocus = useCallback(function (e) {
    onFocus && onFocus(e, 1);
  }, [onFocus]);
  return _extends({}, baseResult, {
    finalValue: finalValue,
    leftInputRef: leftInputRef,
    rightInputRef: rightInputRef,
    handleClick: handleClick,
    handleLeftChange: handleLeftChange,
    handleRightChange: handleRightChange,
    handleClear: handleClear,
    handleKeyDown: handleKeyDown,
    handleLeftFocus: handleLeftFocus,
    handleRightFocus: handleRightFocus
  });
}