import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _get from "lodash/get";
var _excluded = ["children", "size", "disabled"];
import React, { useCallback, useMemo, useState } from 'react';
import styled, { css } from 'styled-components';
import { StyledInputWrapper, StyledTagsInputWrapper } from '../Input/styled';
import useTheme from '../utils/useTheme';

function ZIndexHoc(props) {
  var children = props.children,
      size = props.size;
  var childProps = children.props;
  var childType = children.type;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      focus = _useState2[0],
      setFocus = _useState2[1];

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      popupVisible = _useState4[0],
      setPopupVisible = _useState4[1];

  var theme = useTheme();
  var handleFocus = useCallback(function (e) {
    if (childProps.onFocus) {
      childProps.onFocus(e);
    }

    setFocus(true);
  }, [childProps]);
  var handleBlur = useCallback(function (e) {
    if (childProps.onBlur) {
      childProps.onBlur(e);
    }

    setFocus(false);
  }, [childProps]);
  var handlePopupVisibleChange = useCallback(function (open) {
    if (childProps.onPopupVisibleChange) {
      childProps.onPopupVisibleChange(open);
    }

    setPopupVisible(open);
  }, [childProps]); // 组件处于focus或者popupVisible状态，提升zIndex层级

  var shouldAddZIndex = focus || popupVisible;
  var childStyle = useMemo(function () {
    return {
      zIndex: shouldAddZIndex ? 1 : undefined
    };
  }, [shouldAddZIndex]);
  var finalChildProps = useMemo(function () {
    return {
      onFocus: handleFocus,
      onBlur: handleBlur,
      size: childProps.size || size
    };
  }, [childProps.size, handleBlur, handleFocus, size]); // 只针对Select相关组件添加此事件，否则会出现warning
  // Select相关组件blur后实际上还处于聚焦状态，需要特殊处理保证zIndex正确

  if (childType.__MUYA_TREESELECT || childType.__MUYA_SELECT || childType.__MUYA_CASCADER || childType.__MUYA_AUTOCOMPLETE) {
    finalChildProps.onPopupVisibleChange = handlePopupVisibleChange;
  } // 设置plain为true，即为线性按钮，带有border
  // type为normal的情况下，默认也是为线性按钮


  var isPlainButton = !theme.components.Button.plainIsLight && (childProps.plain || !childProps.type || childProps.type === 'normal') || childProps.outline; // 非线性按钮，不需要向右移动

  if (childType.__MUYA_BUTTON && !isPlainButton) {
    childStyle.marginRight = 0;
  }

  finalChildProps.style = _extends({}, childStyle, childProps.style);
  return /*#__PURE__*/React.cloneElement(children, finalChildProps);
}

var StyledInputGroup = styled.div.withConfig({
  displayName: "StyledInputGroup"
})(["", ""], function (props) {
  var compactStyle = props.disabled ? undefined : css(["& > *{position:relative;float:none;vertical-align:top;&:focus{z-index:1;}&:not(:first-child):not(:last-child){&,& ", ",& ", "{border-radius:0;}}&:first-child{&,& ", ",& ", "{border-top-right-radius:0;border-bottom-right-radius:0;}}&:last-child{&,& ", ",& ", "{border-top-left-radius:0;border-bottom-left-radius:0;}}&:not(:last-child){margin-right:-1px;border-right-width:1px;}}"], StyledInputWrapper, StyledTagsInputWrapper, StyledInputWrapper, StyledTagsInputWrapper, StyledInputWrapper, StyledTagsInputWrapper);
  return css(["position:relative;display:block;width:100%;", ""], compactStyle);
});
var InputGroup = /*#__PURE__*/React.memo(function (props) {
  var children = props.children,
      size = props.size,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      other = _objectWithoutProperties(props, _excluded);

  return /*#__PURE__*/React.createElement(StyledInputGroup, _extends({
    disabled: disabled
  }, other), React.Children.map(children, function (child, index) {
    if (! /*#__PURE__*/React.isValidElement(child)) {
      return null;
    } // 第一个 select 或者 autoComplete 自适应宽


    if (index === 0 && (child.type.__MUYA_SELECT || child.type.__MUYA_AUTOCOMPLETE) && typeof _get(child, 'props.autoResize') === 'undefined') {
      return /*#__PURE__*/React.createElement(ZIndexHoc, {
        size: size
      }, /*#__PURE__*/React.cloneElement(child, {
        autoResize: true
      }));
    }

    return /*#__PURE__*/React.createElement(ZIndexHoc, {
      size: size
    }, child);
  }));
});
export default InputGroup;