import _extends from "@babel/runtime/helpers/extends";
import { MinusIcon as _MinusIcon } from "@muya-ui/theme-light/es/icons/MinusIcon";
import { AddIcon as _AddIcon } from "@muya-ui/theme-light/es/icons/AddIcon";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["prefixNode", "suffixNode", "onMouseEnter", "onMouseLeave", "disabled", "size", "hiddenArrow", "disableBlurFormat", "mode", "fixPrecision", "min", "max", "precision", "step", "iconSize", "width", "defaultValue", "value", "onBlur", "onChange", "onKeyDown", "onFocus", "formatter", "parser", "inputRef", "readOnly", "showRightBorder", "hasFeedback", "bordered", "arrowBordered"];
import React, { useCallback, useMemo, useRef } from 'react';
import styled, { css } from 'styled-components';
import { Input } from '../Input';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import ArrowWrapper from './ArrowWrapper';
import { StyledNodeWrapper } from './styled';
import useInputNumber from './useInputNumber';
import IconButton from '../IconButton';
var InputNumberWrapper = styled(Input).withConfig({
  displayName: "InputNumberWrapper"
})(["", ""], function (props) {
  var $entered = props.$entered,
      $hiddenArrow = props.$hiddenArrow,
      $mode = props.$mode,
      size = props.size,
      theme = props.theme;
  var spacing = theme.components.Input.inputPadding[size];
  var left = theme.components.Input.inputAddonNodeMargin[size];
  var defaultStyle = $mode === 'default' && $entered && !$hiddenArrow && css(["padding-right:0;"]);
  var plusMinusStyle = $mode === 'plus-minus' && css(["padding-right:0;padding-left:0;& input{padding-left:", "px;text-align:center;}"], spacing - left);
  return css(["", " ", ""], defaultStyle, plusMinusStyle);
});
var InputNumber = memoForwardRef(function (props, ref) {
  var theme = useTheme();

  var prefixNode = props.prefixNode,
      suffixNode = props.suffixNode,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      disabled = props.disabled,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$hiddenArrow = props.hiddenArrow,
      hiddenArrow = _props$hiddenArrow === void 0 ? false : _props$hiddenArrow,
      disableBlurFormat = props.disableBlurFormat,
      _props$mode = props.mode,
      mode = _props$mode === void 0 ? 'default' : _props$mode,
      fixPrecision = props.fixPrecision,
      _props$min = props.min,
      min = _props$min === void 0 ? -Infinity : _props$min,
      _props$max = props.max,
      max = _props$max === void 0 ? Infinity : _props$max,
      _props$precision = props.precision,
      precision = _props$precision === void 0 ? 1 : _props$precision,
      _props$step = props.step,
      step = _props$step === void 0 ? 1 : _props$step,
      iconSize = props.iconSize,
      width = props.width,
      defaultValue = props.defaultValue,
      value = props.value,
      onBlur = props.onBlur,
      onChange = props.onChange,
      onKeyDown = props.onKeyDown,
      onFocus = props.onFocus,
      formatter = props.formatter,
      parser = props.parser,
      inputRefProp = props.inputRef,
      readOnly = props.readOnly,
      showRightBorder = props.showRightBorder,
      hasFeedback = props.hasFeedback,
      _props$bordered = props.bordered,
      bordered = _props$bordered === void 0 ? true : _props$bordered,
      _props$arrowBordered = props.arrowBordered,
      arrowBordered = _props$arrowBordered === void 0 ? false : _props$arrowBordered,
      other = _objectWithoutProperties(props, _excluded);

  var isControlled = ('value' in props);
  var finalWidth = width || (mode === 'plus-minus' ? theme.components.InputNumber.plusMinusWidth[size] : theme.components.InputNumber.width[size]);
  var inputRef = useRef(null);

  var handleInputRef = _useForkRef(inputRef, inputRefProp);

  var focus = useCallback(function () {
    inputRef.current && inputRef.current.focus();
  }, []); // 只有当同时隐藏输入框边框和步进器边框时才真正隐藏步进器边框

  var showArrowBorder = bordered || arrowBordered;

  var _useInputNumber = useInputNumber({
    defaultValue: defaultValue,
    onChange: onChange,
    formatter: formatter,
    parser: parser,
    onKeyDown: onKeyDown,
    onBlur: onBlur,
    onFocus: onFocus,
    value: value,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    focus: focus,
    step: step,
    max: max,
    min: min,
    precision: precision,
    fixPrecision: fixPrecision,
    disableBlurFormat: disableBlurFormat,
    disabled: disabled
  }, isControlled),
      handleBlur = _useInputNumber.handleBlur,
      handleChange = _useInputNumber.handleChange,
      handleKeyDown = _useInputNumber.handleKeyDown,
      inputValue = _useInputNumber.inputValue,
      arrowDisabled = _useInputNumber.arrowDisabled,
      entered = _useInputNumber.entered,
      handleMouseEnter = _useInputNumber.handleMouseEnter,
      handleMouseLeave = _useInputNumber.handleMouseLeave,
      changeStep = _useInputNumber.changeStep,
      handleFocus = _useInputNumber.handleFocus,
      handleDoubleClick = _useInputNumber.handleDoubleClick,
      handleStepMouseEnter = _useInputNumber.handleStepMouseEnter,
      handleStepMouseLeave = _useInputNumber.handleStepMouseLeave;

  var arrowSuffixNode = useMemo(function () {
    return !disabled && !readOnly ? /*#__PURE__*/React.createElement(ArrowWrapper, {
      changeStep: changeStep,
      arrowDisabled: arrowDisabled,
      size: size,
      iconSize: iconSize,
      showRightBorder: showRightBorder,
      showArrowBorder: showArrowBorder,
      onMouseEnter: handleStepMouseEnter,
      onMouseLeave: handleStepMouseLeave
    }) : suffixNode;
  }, [arrowDisabled, changeStep, disabled, handleStepMouseEnter, handleStepMouseLeave, iconSize, readOnly, showArrowBorder, showRightBorder, size, suffixNode]);
  var AddNode = theme.components.InputNumber.plusIcon || _AddIcon;
  var MinusNode = theme.components.InputNumber.minusIcon || _MinusIcon;
  var handleStepChange = useCallback(function (direction) {
    if (disabled || arrowDisabled[direction] || readOnly) {
      return;
    }

    changeStep(direction);
  }, [arrowDisabled, changeStep, disabled, readOnly]);
  var minusPrefixNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledNodeWrapper, {
      theme: theme,
      $type: "prefix",
      size: size,
      $iconSize: iconSize,
      onClick: function onClick() {
        return handleStepChange('down');
      },
      onMouseEnter: handleStepMouseEnter,
      onMouseLeave: handleStepMouseLeave
    }, /*#__PURE__*/React.createElement(IconButton, {
      disabled: disabled || readOnly || arrowDisabled.down,
      tabIndex: -1
    }, /*#__PURE__*/React.createElement(MinusNode, null)));
  }, [arrowDisabled.down, disabled, handleStepChange, handleStepMouseEnter, handleStepMouseLeave, iconSize, readOnly, size, theme]);
  var addSuffixNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledNodeWrapper, {
      theme: theme,
      $type: "suffix",
      size: size,
      $iconSize: iconSize,
      onClick: function onClick() {
        return handleStepChange('up');
      },
      onMouseEnter: handleStepMouseEnter,
      onMouseLeave: handleStepMouseLeave
    }, /*#__PURE__*/React.createElement(IconButton, {
      disabled: disabled || readOnly || arrowDisabled.up,
      tabIndex: -1
    }, /*#__PURE__*/React.createElement(AddNode, null)));
  }, [arrowDisabled.up, disabled, handleStepChange, handleStepMouseEnter, handleStepMouseLeave, iconSize, readOnly, size, theme]);
  var inputPrefixNode = mode === 'plus-minus' ? minusPrefixNode : prefixNode;
  var inputSuffixNode = mode === 'plus-minus' ? addSuffixNode : entered && !hiddenArrow ? arrowSuffixNode : suffixNode;
  var finalHasFeedback = mode !== 'plus-minus' && hasFeedback;
  return /*#__PURE__*/React.createElement(InputNumberWrapper, _extends({
    theme: theme,
    $entered: entered,
    $hiddenArrow: hiddenArrow,
    $mode: mode,
    value: inputValue,
    ref: ref,
    size: size,
    disabled: disabled,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave,
    onBlur: handleBlur,
    prefixNode: inputPrefixNode,
    suffixNode: inputSuffixNode,
    hasFeedback: finalHasFeedback,
    onChange: handleChange,
    onFocus: handleFocus,
    onKeyDown: handleKeyDown,
    inputRef: handleInputRef,
    width: finalWidth,
    readOnly: readOnly,
    bordered: bordered,
    onDoubleClick: handleDoubleClick
  }, other));
});
InputNumber.__MUYA_INPUTNUMBER = true;
export default InputNumber;