import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["size", "disabled", "readOnly", "status", "middleNode", "width", "height", "onMouseEnter", "onMouseLeave", "className", "style", "styles", "value", "defaultValue", "placeholder", "leftLimit", "rightLimit", "onChange", "onPressEnter", "onFocus", "onBlur", "inputProps", "bordered", "innerRef"];
import React, { useMemo, useRef } from 'react';
import addPx from '../utils/addPx';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { StyledRangeInputNumberWrapper, StyledMiddleNode } from './styled';
import useRangeInputNumber from './useRangeInputNumber';
import InputNumber from './InputNumber';
var defaultStyles = {
  inputWrapper: '',
  headInput: '',
  tailInput: '',
  nodeDivider: ''
};
var RangeInputNumber = memoForwardRef(function (props, ref) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      readOnly = props.readOnly,
      status = props.status,
      _props$middleNode = props.middleNode,
      middleNode = _props$middleNode === void 0 ? '~' : _props$middleNode,
      width = props.width,
      height = props.height,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      className = props.className,
      style = props.style,
      styles = props.styles,
      value = props.value,
      defaultValue = props.defaultValue,
      _props$placeholder = props.placeholder,
      placeholder = _props$placeholder === void 0 ? [] : _props$placeholder,
      _props$leftLimit = props.leftLimit,
      leftLimit = _props$leftLimit === void 0 ? [-Infinity, Infinity] : _props$leftLimit,
      _props$rightLimit = props.rightLimit,
      rightLimit = _props$rightLimit === void 0 ? [-Infinity, Infinity] : _props$rightLimit,
      onChange = props.onChange,
      onPressEnter = props.onPressEnter,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      inputProps = props.inputProps,
      _props$bordered = props.bordered,
      bordered = _props$bordered === void 0 ? true : _props$bordered,
      innerRef = props.innerRef,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('range-input-number', defaultStyles, styles);
  var inputWrapperStyleItem = useMemo(function () {
    return mergeStyleItem({
      className: className,
      style: style
    }, innerStyles.inputWrapper);
  }, [className, innerStyles.inputWrapper, style]);

  var _useRangeInputNumber = useRangeInputNumber(props),
      focus = _useRangeInputNumber.focus,
      finalValue = _useRangeInputNumber.finalValue,
      leftInputRef = _useRangeInputNumber.leftInputRef,
      rightInputRef = _useRangeInputNumber.rightInputRef,
      handleFocus = _useRangeInputNumber.handleFocus,
      handleBlur = _useRangeInputNumber.handleBlur,
      handleLeftChange = _useRangeInputNumber.handleLeftChange,
      handleRightChange = _useRangeInputNumber.handleRightChange,
      handleKeyDown = _useRangeInputNumber.handleKeyDown;

  var more = useRef({
    focus: function focus() {
      var position = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var option = arguments.length > 1 ? arguments[1] : undefined;

      if (position === 1) {
        rightInputRef.current.focus(option);
      } else {
        leftInputRef.current.focus(option);
      }
    },
    blur: function blur() {
      var position = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

      if (position === 1) {
        rightInputRef.current.blur();
      } else {
        leftInputRef.current.blur();
      }
    }
  });

  var handleRef = _useForkRef(ref, innerRef, more.current);

  var _finalValue = _slicedToArray(finalValue, 2),
      leftValue = _finalValue[0],
      rightValue = _finalValue[1];

  var inputWidth = width !== undefined ? width : theme.components.InputNumber.rangeInputNumberWidth[size];
  var inputHeight = height !== undefined ? height : theme.components.Input.inputHeight[size];
  return /*#__PURE__*/React.createElement(StyledRangeInputNumberWrapper, _extends({
    theme: theme,
    disabled: disabled,
    focus: focus,
    $bordered: bordered,
    $width: addPx(inputWidth),
    $height: addPx(inputHeight),
    size: size,
    hasError: status === 'error',
    ref: handleRef
  }, inputWrapperStyleItem, other), /*#__PURE__*/React.createElement(InputNumber, _extends({
    width: "45%",
    height: "100%"
  }, inputProps, innerStyles.headInput, {
    value: leftValue,
    disabled: disabled,
    onKeyDown: handleKeyDown,
    placeholder: placeholder[0],
    size: size,
    inputRef: leftInputRef,
    readOnly: readOnly,
    bordered: false,
    showRightBorder: true,
    arrowBordered: bordered,
    onChange: handleLeftChange,
    onFocus: handleFocus,
    onBlur: handleBlur,
    min: leftLimit[0],
    max: leftLimit[1]
  })), /*#__PURE__*/React.createElement(StyledMiddleNode, _extends({}, innerStyles.nodeDivider, {
    theme: theme,
    $size: size,
    $disabled: disabled
  }), middleNode), /*#__PURE__*/React.createElement(InputNumber, _extends({
    width: "45%",
    height: "100%"
  }, inputProps, innerStyles.tailInput, {
    value: rightValue,
    disabled: disabled,
    onKeyDown: handleKeyDown,
    size: size,
    placeholder: placeholder[1],
    inputRef: rightInputRef,
    readOnly: readOnly,
    bordered: false,
    arrowBordered: bordered,
    onFocus: handleFocus,
    onBlur: handleBlur,
    onChange: handleRightChange,
    min: rightLimit[0],
    max: rightLimit[1]
  })));
});
RangeInputNumber.__MUYA_RANGEINPUTNUMBER = true;
export default RangeInputNumber;