import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useCallback, useMemo, useRef, useState } from 'react';
import forkHandler from '../utils/forkHandler';
import { getRangeDefaultValue } from './utils';
export default function useRangeInputNumber(props) {
  var disabled = props.disabled,
      value = props.value,
      defaultValue = props.defaultValue,
      onChange = props.onChange,
      _props$inputProps = props.inputProps,
      inputProps = _props$inputProps === void 0 ? {} : _props$inputProps,
      onKeyDown = props.onKeyDown,
      onPressEnter = props.onPressEnter,
      onFocus = props.onFocus,
      onBlur = props.onBlur;
  var autoFocus = inputProps.autoFocus;
  var leftInputRef = useRef(null);
  var rightInputRef = useRef(null);
  var valueFromProps = useMemo(function () {
    if (value) {
      return getRangeDefaultValue(value);
    }

    if (defaultValue) {
      return getRangeDefaultValue(defaultValue);
    }

    return ['', ''];
  }, [defaultValue, value]);

  var _useState = useState(valueFromProps),
      _useState2 = _slicedToArray(_useState, 2),
      innerValue = _useState2[0],
      setInnerValue = _useState2[1];

  var finalValue = 'value' in props ? valueFromProps : innerValue;
  var updateValue = useCallback(function (newValue) {
    setInnerValue(newValue);

    if (onChange) {
      var formatterValue = [parseFloat(newValue[0]), parseFloat(newValue[1])];
      onChange(newValue, formatterValue);
    }
  }, [onChange]);
  var setInputValue = useCallback(function (v, inputPos) {
    var _finalValue = _slicedToArray(finalValue, 2),
        oldLeft = _finalValue[0],
        oldRight = _finalValue[1];

    var newValue = [oldLeft, oldRight];
    newValue[inputPos] = v;
    updateValue(newValue);
  }, [finalValue, updateValue]);
  var handleLeftChange = useCallback(function (displayedValue, formatterValue) {
    setInputValue(displayedValue, 0);
  }, [setInputValue]);
  var handleRightChange = useCallback(function (displayedValue, formatterValue) {
    setInputValue(displayedValue, 1);
  }, [setInputValue]);

  if (disabled) {
    leftInputRef.current && leftInputRef.current.blur();
    rightInputRef.current && rightInputRef.current.blur();
  }

  var handleKeyDown = useMemo(function () {
    return forkHandler(function (e) {
      if (e.keyCode === 13 && onPressEnter) {
        var formatterValue = [parseFloat(finalValue[0]), parseFloat(finalValue[1])];
        onPressEnter(finalValue, formatterValue);
      }
    }, onKeyDown);
  }, [finalValue, onKeyDown, onPressEnter]);

  var _useState3 = useState(autoFocus),
      _useState4 = _slicedToArray(_useState3, 2),
      focus = _useState4[0],
      setFocus = _useState4[1];

  var handleFocus = useMemo(function () {
    return forkHandler(function () {
      setFocus(true);
    }, onFocus);
  }, [onFocus]);
  var handleBlur = useMemo(function () {
    return forkHandler(function () {
      setFocus(false);
    }, onBlur);
  }, [onBlur]);
  return {
    focus: focus,
    finalValue: finalValue,
    leftInputRef: leftInputRef,
    rightInputRef: rightInputRef,
    handleFocus: handleFocus,
    handleBlur: handleBlur,
    handleLeftChange: handleLeftChange,
    handleRightChange: handleRightChange,
    handleKeyDown: handleKeyDown
  };
}