import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import JSBI from 'jsbi';
export var isValidProps = function isValidProps(value) {
  return value !== undefined && value !== null;
};
export var isNotCompleteNumber = function isNotCompleteNumber(num) {
  return isNaN(num) || num === '' || num === null || num && num.toString().indexOf('.') === num.toString().length - 1;
};
export var handleSymbol = function handleSymbol(str) {
  return str === '-' || str === '.' || str === '-.';
};
export var addNum = function addNum(num1, num2) {
  var sq1;
  var sq2;
  var m;

  try {
    sq1 = num1.toString().split('.')[1].length;
  } catch (e) {
    sq1 = 0;
  }

  try {
    sq2 = num2.toString().split('.')[1].length;
  } catch (e) {
    sq2 = 0;
  }

  m = Math.pow(10, Math.max(sq1, sq2));
  return (Math.round(num1 * m) + Math.round(num2 * m)) / m;
};
export var fullCharToHalfChar = function fullCharToHalfChar(str) {
  var result = '';

  for (var i = 0; i < str.length; i++) {
    var code = str.charCodeAt(i); // 处理全角空格

    if (code === 12288) {
      result += String.fromCharCode(code - 12256);
    } else if (code > 65280 && code < 65375) {
      result += String.fromCharCode(code - 65248);
    } else {
      result += String.fromCharCode(code);
    }
  }

  return result;
};
export function getValueFromInputNumberEvent(s, n) {
  return isNotCompleteNumber(s) || isNaN(n) ? s : n;
}
export function getRangeDefaultValue(defaultValue) {
  var value = defaultValue || [];

  var _value = _slicedToArray(value, 2),
      _value$ = _value[0],
      left = _value$ === void 0 ? '' : _value$,
      _value$2 = _value[1],
      right = _value$2 === void 0 ? '' : _value$2;

  left = isNaN(Number(left)) ? '' : left;
  right = isNaN(Number(right)) ? '' : right;
  return [left.toString(), right.toString()];
}
export function supportBigInt() {
  return typeof BigInt === 'function';
}
export function trimNumber(numStr) {
  var str = numStr.trim();
  var negative = str.startsWith('-');

  if (negative) {
    str = str.slice(1);
  }

  str = str // 移除末尾 0. `1.000` => `1.`, `1.100` => `1.1`
  .replace(/(\.\d*[^0])0*$/, '$1') // 移除. `1.` => `1`
  .replace(/\.0*$/, '') // 移除整数前面 0. `0001` => `1`, 000.1' => `.1`
  .replace(/^0+/, '');

  if (str.startsWith('.')) {
    str = "0".concat(str);
  }

  var trimStr = str || '0';
  var splitNumber = trimStr.split('.');
  var integerStr = splitNumber[0] || '0';
  var decimalStr = splitNumber[1] || '0';

  if (integerStr === '0' && decimalStr === '0') {
    negative = false;
  }

  var negativeStr = negative ? '-' : '';
  return {
    negative: negative,
    negativeStr: negativeStr,
    trimStr: trimStr,
    integerStr: integerStr,
    decimalStr: decimalStr,
    fullStr: "".concat(negativeStr).concat(trimStr)
  };
}
export function isE(number) {
  var str = String(number);
  return !Number.isNaN(Number(str)) && str.includes('e');
}
/**
 * [Legacy] Convert 1e-9 to 0.000000001.
 * This may lose some precision if user really want 1e-9.
 */

export function getNumberPrecision(number) {
  var numStr = String(number);

  if (isE(number)) {
    var precision = Number(numStr.slice(numStr.indexOf('e-') + 2));
    var decimalMatch = numStr.match(/\.(\d+)/);

    if (decimalMatch && decimalMatch[1]) {
      precision += decimalMatch[1].length;
    }

    return precision;
  }

  return numStr.includes('.') && validateNumber(numStr) ? numStr.length - numStr.indexOf('.') - 1 : 0;
}
/**
 * Convert number (includes scientific notation) to -xxx.yyy format
 */

export function num2str(number) {
  var numStr = String(number);

  if (isE(number)) {
    if (number > Number.MAX_SAFE_INTEGER) {
      // eslint-disable-next-line no-undef
      return String(supportBigInt() ? JSBI.BigInt(number).toString() : Number.MAX_SAFE_INTEGER);
    }

    if (number < Number.MIN_SAFE_INTEGER) {
      // eslint-disable-next-line no-undef
      return String(supportBigInt() ? JSBI.BigInt(number).toString() : Number.MIN_SAFE_INTEGER);
    }

    numStr = number.toFixed(getNumberPrecision(numStr));
  }

  return trimNumber(numStr).fullStr;
}
export function validateNumber(num) {
  if (typeof num === 'number') {
    return !Number.isNaN(num);
  } // Empty


  if (!num) {
    return false;
  }

  return (// Normal type: 11.28
    /^\s*-?\d+(\.\d+)?\s*$/.test(num) || // Pre-number: 1.
    /^\s*-?\d+\.\s*$/.test(num) || // Post-number: .1
    /^\s*-?\.\d+\s*$/.test(num)
  );
}