import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _extends from "@babel/runtime/helpers/extends";
var _excluded = ["children", "direction", "draggable", "miniSizes", "initialSizes", "gutterColor", "styles", "renderGutterDragger", "onGutterMouseDown", "onGutterMouseUp"];
// fork from https://gitlab.qunhequnhe.com/tool-frontend/kaf-group/tools-ui-pro
import React, { useRef } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import { StyledLayoutSplitterContainer, StyledLayoutSplitterContent, StyledGutterContainer, StyledGutterDragger } from './styled';
import useLayoutSplitter from './useLayoutSplitter';
import { useTheme } from '../utils/useTheme';
var Gutter = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var styles = props.styles,
      direction = props.direction,
      onMouseDown = props.onMouseDown,
      draggable = props.draggable,
      gutterDragger = props.gutterDragger,
      gutterColor = props.gutterColor;
  var theme = useTheme();
  return /*#__PURE__*/React.createElement(StyledGutterContainer, _extends({
    ref: ref,
    theme: theme,
    $direction: direction,
    $draggable: draggable,
    $gutterColor: gutterColor,
    onMouseDown: onMouseDown
  }, styles.gutter), gutterDragger || /*#__PURE__*/React.createElement(StyledGutterDragger, styles.dragger));
});
var defaultStyles = {
  content: '',
  gutter: '',
  dragger: ''
};
var LayoutSplitter = memoForwardRef(function (props, ref) {
  var children = props.children,
      _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'horizontal' : _props$direction,
      _props$draggable = props.draggable,
      draggable = _props$draggable === void 0 ? true : _props$draggable,
      miniSizes = props.miniSizes,
      initialSizes = props.initialSizes,
      gutterColor = props.gutterColor,
      styles = props.styles,
      renderGutterDragger = props.renderGutterDragger,
      onGutterMouseDown = props.onGutterMouseDown,
      onGutterMouseUp = props.onGutterMouseUp,
      restProps = _objectWithoutProperties(props, _excluded);

  var containerRef = useRef(null);

  var handleRef = _useForkRef(containerRef, ref);

  var _useLayoutSplitter = useLayoutSplitter(props),
      childRefs = _useLayoutSplitter.childRefs,
      gutterRefs = _useLayoutSplitter.gutterRefs,
      addRef = _useLayoutSplitter.addRef,
      setSizes = _useLayoutSplitter.setSizes,
      handleMouseDown = _useLayoutSplitter.handleMouseDown;

  React.useImperativeHandle(ref, function () {
    if (containerRef.current) {
      containerRef.current.setSizes = setSizes;
    }

    return containerRef.current;
  });
  var innerStyles = useStyles('layout-splitter', defaultStyles, styles);
  return /*#__PURE__*/React.createElement(StyledLayoutSplitterContainer, _extends({
    ref: handleRef,
    $direction: direction
  }, restProps), children && children.map(function (c, idx) {
    var isSplitter = c.type.__MUYA_LAYOUT_SPLITTER;
    var gutterDragger = renderGutterDragger && renderGutterDragger({
      index: idx,
      setSizes: setSizes
    });
    return /*#__PURE__*/React.createElement(React.Fragment, {
      key: idx
    }, /*#__PURE__*/React.createElement(StyledLayoutSplitterContent, _extends({
      ref: function ref(el) {
        addRef(childRefs, el);
      }
    }, innerStyles.content), isSplitter ? /*#__PURE__*/React.cloneElement(c, {
      draggable: draggable
    }) : c), idx < children.length - 1 && /*#__PURE__*/React.createElement(Gutter, {
      ref: function ref(el) {
        addRef(gutterRefs, el);
      },
      draggable: draggable,
      styles: innerStyles,
      direction: direction,
      gutterColor: gutterColor,
      gutterDragger: gutterDragger,
      onMouseDown: function onMouseDown(e) {
        return handleMouseDown(idx, e);
      }
    }));
  }));
});
LayoutSplitter.__MUYA_LAYOUT_SPLITTER = true;
export default LayoutSplitter;