import { useThrottle as _useThrottle } from "@muya-ui/utils/es/hooks/useThrottle";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _extends from "@babel/runtime/helpers/extends";
import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useEffect, useRef, useState } from 'react';
import addPx from '../utils/addPx';
import { createPairs, getCursorIcon, getInnerSize, getMousePosition } from './utils';
var DefaultMinSize = 0;
var dragThrottleTime = 50;
export default function useLayoutSplitter(props) {
  var _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'horizontal' : _props$direction,
      _props$initialSizes = props.initialSizes,
      initialSizes = _props$initialSizes === void 0 ? [] : _props$initialSizes,
      miniSizes = props.miniSizes,
      _props$draggable = props.draggable,
      draggable = _props$draggable === void 0 ? true : _props$draggable,
      children = props.children,
      onGutterMouseDown = props.onGutterMouseDown,
      onGutterMouseUp = props.onGutterMouseUp;

  var _useState = useState([]),
      _useState2 = _slicedToArray(_useState, 2),
      pairs = _useState2[0],
      setPairs = _useState2[1];

  var _useState3 = useState({
    isDragging: false,
    draggingIdx: undefined
  }),
      _useState4 = _slicedToArray(_useState3, 2),
      dragState = _useState4[0],
      setDragState = _useState4[1];

  var childRefs = useRef([]);
  var gutterRefs = useRef([]);

  var setInitialSizes = function setInitialSizes(sizes) {
    // All children must have common parent.
    var parent = childRefs.current[0].parentNode;

    if (!parent) {
      _devWarning("[LayoutSplitter]: \u7236\u8282\u70B9\u4E0D\u5B58\u5728\u3002");

      return;
    }

    var parentSize = getInnerSize(direction, parent);

    if (parentSize === undefined) {
      _devWarning("[LayoutSplitter]: \u7236\u8282\u70B9\u5C3A\u5BF8\u4E3A ".concat(parentSize, " \u4E0D\u5408\u6CD5\u3002"));

      return;
    }

    childRefs.current.forEach(function (c, idx) {
      var isFirst = idx === 0;
      var isLast = idx === childRefs.current.length - 1;
      var gutter = gutterRefs.current[isLast ? idx - 1 : idx];
      var gutterSize = gutter.getBoundingClientRect()[direction === 'horizontal' ? 'width' : 'height'];
      gutterSize = isFirst || isLast ? gutterSize / 2 : gutterSize;
      var calc = '';

      if (sizes.length === 0) {
        calc = "calc(".concat(100 / childRefs.current.length, "% - ").concat(gutterSize, "px)");
      } else {
        var size = addPx(sizes[idx]);

        if (size) {
          calc = "calc(".concat(size, " - ").concat(gutterSize, "px)");
        } else {
          c.style.flex = '1';
        }
      }

      if (direction === 'horizontal') {
        if (calc) {
          c.style.width = calc;
        } // Reset the child wrapper's height because the direction could have changed.


        c.style.height = '100%';
      } else {
        if (calc) {
          c.style.height = calc;
        } // Reset the child wrapper's width because the direction could have changed.


        c.style.width = '100%';
      }
    });
  };

  var updatePairsState = function updatePairsState() {
    setPairs(createPairs(direction, childRefs.current, gutterRefs.current, miniSizes, DefaultMinSize));
  };

  var setSizes = function setSizes(sizes) {
    if (children === undefined) {
      _devWarning('[LayoutSplitter]: children 不能为空。');

      return;
    }

    if (!Array.isArray(children)) {
      _devWarning('[LayoutSplitter]: children 必须为数组。');

      return;
    }

    if (children.length <= 1) {
      _devWarning('[LayoutSplitter]: children 数组长度必须大于 1。');

      return;
    }

    if (!childRefs.current.length || !gutterRefs.current.length) {
      console.error('[LayoutSplitter]: ref 设置出错。');
      return;
    }

    setInitialSizes(sizes);
    updatePairsState();
  };

  var addRef = function addRef(refs, el) {
    if (el && !refs.current.includes(el)) {
      refs.current.push(el);
    }
  };

  var calculateSizes = function calculateSizes(direction, gutterIdx) {
    var pair = pairs[gutterIdx];
    var parentSize = getInnerSize(direction, pair.parent);

    if (!parentSize) {
      _devWarning('[LayoutSplitter]: "pair.parent" 尺寸不合法。');

      return;
    }

    var gutterSize = pair.gutter[direction === 'horizontal' ? 'clientWidth' : 'clientHeight'];
    var start;
    var end;
    var size;
    var aSizePct;
    var bSizePct;

    if (direction === 'horizontal') {
      start = pair.a.getBoundingClientRect().left;
      end = pair.b.getBoundingClientRect().right;
      aSizePct = (pair.a.getBoundingClientRect().width + gutterSize) / parentSize * 100;
      bSizePct = (pair.b.getBoundingClientRect().width + gutterSize) / parentSize * 100;
      size = pair.a.getBoundingClientRect().width + gutterSize * 2 + pair.b.getBoundingClientRect().width;
    } else {
      start = pair.a.getBoundingClientRect().top;
      end = pair.b.getBoundingClientRect().bottom;
      aSizePct = (pair.a.getBoundingClientRect().height + gutterSize) / parentSize * 100;
      bSizePct = (pair.b.getBoundingClientRect().height + gutterSize) / parentSize * 100;
      size = pair.a.getBoundingClientRect().height + gutterSize * 2 + pair.b.getBoundingClientRect().height;
    }

    pairs[gutterIdx] = _extends({}, pair, {
      start: start,
      end: end,
      size: size,
      aSizePct: aSizePct,
      bSizePct: bSizePct,
      gutterSize: gutterSize
    });
    setPairs(pairs);
  };

  var updatePairCloseTo = function updatePairCloseTo(pairIdx, position) {
    var pair = pairs[pairIdx];
    var prevPair = pairs[pairIdx - 1];
    var lastPair = pairs[pairIdx + 1];
    var DEFAULT_MINSIZE = 42;
    var gutterSize = pair.gutterSize,
        aMini = pair.aMini,
        bMini = pair.bMini,
        size = pair.size;

    if (position < gutterSize + Math.max(aMini, DEFAULT_MINSIZE)) {
      pair.closeToA = true; // tslint:disable-next-line: no-unused-expression

      prevPair && (prevPair.closeToB = true);
    } else {
      pair.closeToA = false; // tslint:disable-next-line: no-unused-expression

      prevPair && (prevPair.closeToB = false);
    }

    if (position >= size - (gutterSize + Math.max(bMini, DEFAULT_MINSIZE))) {
      pair.closeToB = true; // tslint:disable-next-line: no-unused-expression

      lastPair && (lastPair.closeToA = true);
    } else {
      pair.closeToB = false; // tslint:disable-next-line: no-unused-expression

      lastPair && (lastPair.closeToA = false);
    }

    setPairs(pairs);
  };

  var adjustSize = function adjustSize(pairIndex, direction, position) {
    var pair = pairs[pairIndex];
    var percentage = pair.aSizePct + pair.bSizePct;
    var aSizePct = position / pair.size * percentage;
    var bSizePct = percentage - position / pair.size * percentage;
    var aCalc = "calc(".concat(aSizePct, "% - ").concat(pair.gutterSize, "px)");
    var bCalc = "calc(".concat(bSizePct, "% - ").concat(pair.gutterSize, "px)");

    if (direction === 'horizontal') {
      pair.a.style.width = aCalc;
      pair.b.style.width = bCalc;
    } else {
      pair.a.style.height = aCalc;
      pair.b.style.height = bCalc;
    }
  };

  var drag = _useEventCallback(function (e, direction) {
    if (!dragState.isDragging) {
      return;
    }

    if (dragState.draggingIdx === undefined) {
      _devWarning('[LayoutSplitter]: draggingIdx 为 undefined。');

      return;
    }

    var pair = pairs[dragState.draggingIdx]; // 'position' is the width of the 'a' element in a pair.

    var position = getMousePosition(direction, e) - pair.start; // Limit the maximum and minimum size of resized children.

    var gutterSize = pair.gutterSize,
        aMini = pair.aMini,
        bMini = pair.bMini,
        size = pair.size;

    if (position < gutterSize + aMini) {
      position = gutterSize + aMini;
    }

    if (position >= size - (gutterSize + bMini)) {
      position = size - (pair.gutterSize + bMini);
    }

    updatePairCloseTo(dragState.draggingIdx, position);
    adjustSize(dragState.draggingIdx, direction, position);
  });

  var handleMouseDown = function handleMouseDown(gutterIdx, e) {
    calculateSizes(direction, gutterIdx);

    if (!draggable || dragState.isDragging) {
      return;
    }

    startDragging(direction, gutterIdx);
    e.preventDefault();

    if (onGutterMouseDown) {
      onGutterMouseDown(e, pairs);
    }
  };

  var handleMouseMove = _useThrottle(function (e) {
    if (!dragState.isDragging) {
      return;
    }

    drag(e, direction);
  }, dragThrottleTime);

  var startDragging = function startDragging(direction, gutterIdx) {
    setDragState({
      isDragging: true,
      draggingIdx: gutterIdx
    });
    var pair = pairs[gutterIdx]; // Disable selection.

    pair.a.style.userSelect = 'none';
    pair.b.style.userSelect = 'none'; // Set the mouse cursor.
    // Must be done at multiple levels, nut just for a gutter.
    // The mouse cursor might move outside of the gutter element.

    pair.gutter.style.cursor = getCursorIcon(direction);
    pair.parent.style.cursor = getCursorIcon(direction);
    document.body.style.cursor = getCursorIcon(direction);
  };

  var handleMouseUp = _useEventCallback(function (e) {
    if (!dragState.isDragging) {
      return;
    }

    setDragState(_extends({}, dragState, {
      isDragging: false
    }));

    if (dragState.draggingIdx === undefined) {
      _devWarning('[LayoutSplitter]: draggingIdx 为 undefined。');

      return;
    } // 重新计算下尺寸信息


    calculateSizes(direction, dragState.draggingIdx);
    var pair = pairs[dragState.draggingIdx]; // Disable selection.

    pair.a.style.userSelect = '';
    pair.b.style.userSelect = ''; // Set the mouse cursor.
    // Must be done at multiple levels, not just for a gutter.
    // The mouse cursor might move outside of the gutter element.

    pair.gutter.style.cursor = '';
    pair.parent.style.cursor = '';
    document.body.style.cursor = '';

    if (onGutterMouseUp) {
      onGutterMouseUp(e, pairs);
    }
  }); // eslint-disable-next-line react-hooks/exhaustive-deps


  useEffect(function () {
    return setSizes(initialSizes);
  }, []);
  useEffect(function () {
    if (dragState.isDragging) {
      window.addEventListener('mousemove', handleMouseMove);
      window.addEventListener('mouseup', handleMouseUp);
    } else {
      window.removeEventListener('mousemove', handleMouseMove);
      window.removeEventListener('mouseup', handleMouseUp);
    }
  }, [dragState.isDragging, handleMouseMove, handleMouseUp]);
  return {
    childRefs: childRefs,
    gutterRefs: gutterRefs,
    addRef: addRef,
    handleMouseDown: handleMouseDown,
    setSizes: setSizes
  };
}