import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
export function getMousePosition(dir, e) {
  if (dir === 'horizontal') {
    return e.clientX;
  }

  return e.clientY;
}
export function getCursorIcon(dir) {
  if (dir === 'horizontal') {
    return 'col-resize';
  }

  return 'row-resize';
}
export function createPairs(direction, children, gutters, miniSizes, defaultMiniSize) {
  // All children must have common parent.
  var parent = children[0].parentNode;

  if (!parent) {
    _devWarning("[LayoutSplitter]: \u7236\u8282\u70B9\u4E0D\u5B58\u5728\u3002");
  }

  var parentSize = getInnerSize(direction, parent);

  if (parentSize === undefined) {
    _devWarning("[LayoutSplitter]: \u7236\u8282\u70B9\u5C3A\u5BF8\u4E3A ".concat(parentSize, " \u4E0D\u5408\u6CD5\u3002"));
  }

  if (!miniSizes || typeof miniSizes === 'number') {
    miniSizes = children.map(function (_) {
      return miniSizes || defaultMiniSize || 0;
    });
  } else {
    miniSizes.forEach(function (item) {
      if (!item || typeof item !== 'number') {
        item = 0;
      }
    });
  }

  var pairs = [];
  children.forEach(function (_, idx) {
    if (idx > 0) {
      var a = children[idx - 1];
      var b = children[idx];
      var aMini = miniSizes[idx - 1];
      var bMini = miniSizes[idx];
      var gutter = gutters[idx - 1];
      var start = direction === 'horizontal' ? a.getBoundingClientRect().left : a.getBoundingClientRect().top;
      var end = direction === 'horizontal' ? b.getBoundingClientRect().right : b.getBoundingClientRect().bottom;
      var size = direction === 'horizontal' ? a.getBoundingClientRect().width + gutter.getBoundingClientRect().width + b.getBoundingClientRect().width : a.getBoundingClientRect().height + gutter.getBoundingClientRect().height + b.getBoundingClientRect().height;
      var gutterSize = direction === 'horizontal' ? gutter.getBoundingClientRect().width : gutter.getBoundingClientRect().height;
      var pair = {
        idx: idx - 1,
        // TODO: Do we need to have a reference to the whole elements? Aren't indexes enough?
        a: a,
        b: b,
        aMini: aMini,
        bMini: bMini,
        closeToA: false,
        closeToB: false,
        gutter: gutter,
        parent: parent,
        start: start,
        end: end,
        size: size,
        gutterSize: gutterSize,
        // At the start, all elements has the same width.
        aSizePct: 100 / children.length,
        bSizePct: 100 / children.length
      };
      pairs.push(pair);
    }
  });
  return pairs;
}
export function getInnerSize(direction, element) {
  var computedStyle = getComputedStyle(element);

  if (!computedStyle) {
    return;
  }

  var size = direction === 'horizontal' ? element.clientWidth : element.clientHeight;

  if (size === 0) {
    return;
  }

  if (direction === 'horizontal') {
    size -= parseFloat(computedStyle.paddingLeft) + parseFloat(computedStyle.paddingRight);
  } else {
    size -= parseFloat(computedStyle.paddingTop) + parseFloat(computedStyle.paddingBottom);
  }

  return size;
}