import _extends from "@babel/runtime/helpers/extends";
import { MoreIcon as _MoreIcon } from "@muya-ui/theme-light/es/icons/MoreIcon";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _omit from "lodash/omit";
import React, { useMemo, useRef } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { allMenuProp, rootMenuKey } from './const';
import MenuContext from './MenuContext';
import MenuDivider from './MenuDivider';
import MenuItem from './MenuItem';
import MenuItemGroup from './MenuItemGroup';
import { StyledMenu, StyledMenuScrollWrapper, StyledMenuWrapper } from './styled';
import SubMenu from './SubMenu';
import { useMenu } from './useMenu';
import { useMenuBase } from './useMenuBase';
var defaultStyles = {
  wrapper: '',
  menu: '',
  menuScrollWrapper: ''
};
var Menu = memoForwardRef(function (props, ref) {
  var width = props.width,
      height = props.height,
      children = props.children,
      onScroll = props.onScroll,
      styles = props.styles,
      style = props.style,
      className = props.className,
      menuItemAlign = props.menuItemAlign,
      noSpacing = props.noSpacing,
      onClick = props.onClick,
      _props$collapseOverfl = props.collapseOverflowMenu,
      collapseOverflowMenu = _props$collapseOverfl === void 0 ? true : _props$collapseOverfl;
  var theme = useTheme();
  var rootMenuRef = useRef(null);

  var handleRef = _useForkRef(rootMenuRef, ref);

  var contextValue = useMenu(props, rootMenuRef);
  var inlineMode = contextValue.inlineMode,
      verticalMode = contextValue.verticalMode,
      horizontalMode = contextValue.horizontalMode,
      size = contextValue.size,
      mode = contextValue.mode,
      inlineCollapsed = contextValue.inlineCollapsed,
      rootMenuScrollable = contextValue.rootMenuScrollable,
      hideRootMenuSpacing = contextValue.hideRootMenuSpacing,
      maxItemCountPerPage = contextValue.maxItemCountPerPage,
      autoItemCountPerPage = contextValue.autoItemCountPerPage,
      handleItemSelect = contextValue.handleItemSelect,
      handleItemDeselect = contextValue.handleItemDeselect,
      handleMenuScrollRef = contextValue.handleMenuScrollRef;

  var finalHandleRef = _useForkRef(handleMenuScrollRef, handleRef);

  var _useMenuBase = useMenuBase({
    mode: mode,
    level: 1,
    eventKey: rootMenuKey,
    parentIsRootMenu: true,
    parentMenuOpen: true,
    hasScrollBar: rootMenuScrollable,
    menuHasIcon: false,
    children: children,
    onItemSelect: handleItemSelect,
    onItemDeselect: handleItemDeselect,
    onClick: onClick
  }),
      elements = _useMenuBase.elements,
      renderMenuWidget = _useMenuBase.renderMenuWidget;

  var menuStyles = useMemo(function () {
    if (styles) {
      var wrapper = styles.wrapper,
          menu = styles.menu,
          menuScrollWrapper = styles.menuScrollWrapper;
      return {
        wrapper: wrapper,
        menu: menu,
        menuScrollWrapper: menuScrollWrapper
      };
    }
  }, [styles]);
  var innerStyles = useStyles('menu', defaultStyles, menuStyles);

  var domProps = _omit(props, allMenuProp);

  var menuItems = useMemo(function () {
    // 如果当前是收起状态，但是元素超过了高度。
    if (collapseOverflowMenu && inlineCollapsed && rootMenuRef.current) {
      var containerHeight = rootMenuRef.current.offsetHeight;
      var showCount = 0;
      var contentHeight = 0;
      var _theme$components$Men = theme.components.Menu,
          _height = _theme$components$Men.height,
          marginVertical = _theme$components$Men.item.marginVertical,
          paddingVertical = _theme$components$Men.divider.paddingVertical;
      elements.forEach(function (el) {
        if (el.type && el.type.displayName === 'MenuItemGroup') {
          contentHeight += el.props.children.length * (_height + 2 * marginVertical);
        } else if (el.type && el.type.displayName === 'MenuDivider') {
          contentHeight += 2 * paddingVertical + 1;
        } else {
          contentHeight += _height + 2 * marginVertical;
        }

        if (contentHeight <= containerHeight) {
          showCount++;
        }
      });

      if (containerHeight < contentHeight) {
        // 将超出范围的元素收起来
        return elements.slice(0, showCount - 1).concat([/*#__PURE__*/React.createElement(SubMenu, {
          icon: /*#__PURE__*/React.createElement(_MoreIcon, null),
          key: "more"
        }, elements.slice(showCount - 1))]);
      }
    }

    return elements; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [collapseOverflowMenu, elements, inlineCollapsed, theme.components.Menu, rootMenuRef.current]);
  return /*#__PURE__*/React.createElement(MenuContext.Provider, {
    value: contextValue
  }, /*#__PURE__*/React.createElement(StyledMenuWrapper, _extends({}, domProps, {
    ref: finalHandleRef,
    theme: theme,
    $width: width,
    $height: height,
    $inlineMode: inlineMode,
    $verticalMode: verticalMode,
    $inlineCollapsed: inlineCollapsed,
    className: [innerStyles.wrapper.className, className].join(' ').trim(),
    style: _extends({}, innerStyles.wrapper.style, style),
    onScroll: onScroll
  }), /*#__PURE__*/React.createElement(StyledMenu, _extends({
    theme: theme,
    $size: size,
    $inlineMode: inlineMode,
    $horizontalMode: horizontalMode,
    $inlineCollapsed: inlineCollapsed,
    $sub: false,
    $hideRootMenuSpacing: hideRootMenuSpacing,
    $menuItemAlign: menuItemAlign,
    $noSpacing: noSpacing,
    $hasScrollBar: rootMenuScrollable,
    $maxItemCountPerPage: maxItemCountPerPage,
    $autoItemCountPerPage: autoItemCountPerPage
  }, innerStyles.menu), /*#__PURE__*/React.createElement(StyledMenuScrollWrapper, innerStyles.menuScrollWrapper, menuItems.map(renderMenuWidget)))));
});
Menu.Divider = MenuDivider;
Menu.Item = MenuItem;
Menu.Group = MenuItemGroup;
Menu.SubMenu = SubMenu;
export default Menu;