import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _omit from "lodash/omit";
import React, { useMemo, useRef } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { allMenuItemProp } from './const';
import { useMenuContext } from './MenuContext';
import { StyledMenuItem, StyledMenuItemContent, StyledMenuItemText, StyledMenuItemTooltip } from './styled';
import { useMenuItem } from './useMenuItem';
var defaultStyles = {
  item: '',
  itemContent: '',
  itemText: ''
};
var MenuItem = memoForwardRef(function (props, ref) {
  var context = useMenuContext();
  var itemRef = useRef(null);

  var handleRef = _useForkRef(itemRef, ref);

  var disabled = props.disabled,
      menuHasIcon = props.menuHasIcon,
      level = props.level,
      className = props.className,
      style = props.style,
      icon = props.icon,
      selectedIcon = props.selectedIcon,
      children = props.children,
      styles = props.styles;
  var theme = useTheme();
  var tooltip = theme.components.Menu.tooltip;

  var _useMenuItem = useMenuItem(props, context, itemRef),
      isSelected = _useMenuItem.isSelected,
      showChildren = _useMenuItem.showChildren,
      attrs = _useMenuItem.attrs,
      handleClick = _useMenuItem.handleClick;

  var menuStyles = context.styles,
      inlineMode = context.inlineMode,
      horizontalMode = context.horizontalMode,
      itemIcon = context.itemIcon,
      inlineCollapsed = context.inlineCollapsed,
      renderMenuItemChild = context.renderMenuItemChild;
  var itemStyles = useMemo(function () {
    var itemStyles = _extends({}, menuStyles, styles);

    var item = itemStyles.item,
        itemContent = itemStyles.itemContent,
        itemText = itemStyles.itemText;
    return {
      item: item,
      itemContent: itemContent,
      itemText: itemText
    };
  }, [menuStyles, styles]);
  var innerStyles = useStyles('menu', defaultStyles, itemStyles);
  var finalStyle = useMemo(function () {
    return _extends({}, innerStyles.item.style, style);
  }, [innerStyles.item.style, style]);

  var domProps = _omit(props, allMenuItemProp);

  var iconNode = useMemo(function () {
    var finalIcon = icon || itemIcon;

    if (isSelected && selectedIcon) {
      finalIcon = selectedIcon;
    }

    if (typeof finalIcon === 'function') {
      return finalIcon(props);
    } else {
      return finalIcon;
    }
  }, [icon, isSelected, itemIcon, props, selectedIcon]);
  var childrenNode = useMemo(function () {
    if (renderMenuItemChild) {
      return renderMenuItemChild(_extends({}, props, {
        selected: isSelected
      }));
    } else {
      return children;
    }
  }, [children, isSelected, props, renderMenuItemChild]);
  var itemNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledMenuItem, _extends({
      ref: handleRef
    }, domProps, attrs, {
      theme: theme,
      $menuHasIcon: menuHasIcon,
      $isSelected: isSelected,
      $disabled: disabled,
      $level: level,
      $inlineMode: inlineMode,
      $horizontalMode: horizontalMode,
      $inlineCollapsed: inlineCollapsed,
      className: [innerStyles.item.className, className].join(' ').trim(),
      style: finalStyle,
      onClick: handleClick
    }), /*#__PURE__*/React.createElement(StyledMenuItemContent, innerStyles.itemContent, iconNode, /*#__PURE__*/React.createElement(StyledMenuItemText, innerStyles.itemText, childrenNode)));
  }, [attrs, childrenNode, className, disabled, domProps, finalStyle, handleClick, handleRef, horizontalMode, iconNode, inlineCollapsed, inlineMode, innerStyles.item.className, innerStyles.itemContent, innerStyles.itemText, isSelected, level, menuHasIcon, theme]);

  if (showChildren) {
    return itemNode;
  } else {
    return /*#__PURE__*/React.createElement(StyledMenuItemTooltip, {
      title: children,
      offset: tooltip.offset,
      size: tooltip.size,
      placement: "right"
    }, itemNode);
  }
});
MenuItem.displayName = 'MenuItem';
export default MenuItem;