import _extends from "@babel/runtime/helpers/extends";
import _omit from "lodash/omit";
import React, { useMemo } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { allMenuGroupProp } from './const';
import { useMenuContext } from './MenuContext';
import { StyledMenuItemGroup, StyledMenuItemGroupLabel } from './styled';
import { useMenuBase } from './useMenuBase';
var defaultStyles = {
  group: '',
  groupLabel: ''
};
var MenuItemGroup = memoForwardRef(function (props, ref) {
  var _props$title = props.title,
      title = _props$title === void 0 ? '' : _props$title,
      children = props.children,
      eventKey = props.eventKey,
      parentIsRootMenu = props.parentIsRootMenu,
      parentMenuOpen = props.parentMenuOpen,
      parentMenuScrollable = props.parentMenuScrollable,
      menuHasIcon = props.menuHasIcon,
      level = props.level,
      style = props.style,
      styles = props.styles,
      className = props.className,
      onClick = props.onClick,
      onItemSelect = props.onItemSelect,
      onItemDeselect = props.onItemDeselect;
  var theme = useTheme();

  var _ref = useMenuContext(),
      menuStyles = _ref.styles,
      inlineMode = _ref.inlineMode,
      inlineCollapsed = _ref.inlineCollapsed,
      mode = _ref.mode,
      autoScrollToSelection = _ref.autoScrollToSelection;

  var _useMenuBase = useMenuBase({
    mode: mode,
    level: level,
    eventKey: "".concat(eventKey, "_menu_group"),
    parentIsRootMenu: parentIsRootMenu,
    parentMenuOpen: parentMenuOpen,
    hasScrollBar: parentMenuScrollable,
    menuHasIcon: false,
    children: children,
    onClick: onClick,
    onItemSelect: onItemSelect,
    onItemDeselect: onItemDeselect,
    autoScrollToSelection: autoScrollToSelection
  }),
      elements = _useMenuBase.elements,
      renderMenuWidget = _useMenuBase.renderMenuWidget;

  var groupStyles = useMemo(function () {
    var groupStyles = _extends({}, menuStyles, styles);

    var group = groupStyles.group,
        groupLabel = groupStyles.groupLabel;
    return {
      group: group,
      groupLabel: groupLabel
    };
  }, [menuStyles, styles]);
  var innerStyles = useStyles('menu', defaultStyles, groupStyles);
  var finalStyle = useMemo(function () {
    return _extends({}, innerStyles.group.style, style);
  }, [innerStyles.group.style, style]);
  var showLabel = !inlineCollapsed || inlineCollapsed && level > 1;

  var domProps = _omit(props, allMenuGroupProp);

  return /*#__PURE__*/React.createElement(StyledMenuItemGroup, _extends({
    ref: ref,
    theme: theme,
    $level: level,
    $menuHasIcon: menuHasIcon,
    $inlineMode: inlineMode,
    $inlineCollapsed: inlineCollapsed,
    className: [innerStyles.group.className, className].join(' ').trim(),
    style: finalStyle
  }, domProps), showLabel && /*#__PURE__*/React.createElement(StyledMenuItemGroupLabel, innerStyles.groupLabel, title), elements.map(renderMenuWidget));
});
MenuItemGroup.displayName = 'MenuItemGroup';
export default MenuItemGroup;