import { FoldIcon as _FoldIcon } from "@muya-ui/theme-light/es/icons/FoldIcon";
import _extends from "@babel/runtime/helpers/extends";
import _omit from "lodash/omit";
import React, { useMemo, useRef } from 'react';
import Animation from '../Animation';
import ExpandIcon from '../ExpandIcon';
import ExpandWrapper from '../styled/components/ExpandWrapper';
import Trigger from '../Trigger';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { allSubMenuProp } from './const';
import { useMenuContext } from './MenuContext';
import { StyledMenu, StyledMenuScrollWrapper, StyledSubMenu, StyledSubMenuTitle, StyledSubMenuTitleAndIconWrapper, StyledSubMenuTitleWrapper } from './styled';
import { useMenuBase } from './useMenuBase';
import { useSubMenu } from './useSubMenu';
import { applyStyle } from './utils';
var defaultStyles = {
  menu: '',
  subMenu: '',
  subMenuTitle: '',
  menuScrollWrapper: ''
};
var SubMenu = memoForwardRef(function (props, ref) {
  var eventKey = props.eventKey,
      title = props.title,
      disabled = props.disabled,
      level = props.level,
      icon = props.icon,
      selectedIcon = props.selectedIcon,
      menuHasIcon = props.menuHasIcon,
      children = props.children,
      className = props.className,
      style = props.style,
      _props$popperProps = props.popperProps,
      popperProps = _props$popperProps === void 0 ? {} : _props$popperProps,
      _props$flip = props.flip,
      flip = _props$flip === void 0 ? true : _props$flip,
      arrowPointAtCenter = props.arrowPointAtCenter,
      parentIsRootMenu = props.parentIsRootMenu,
      parentMenuScrollable = props.parentMenuScrollable,
      triggerId = props.triggerId,
      styles = props.styles;
  var titleRef = useRef(null);
  var theme = useTheme();
  var context = useMenuContext();
  var getPopupContainer = context.getPopupContainer,
      inlineMode = context.inlineMode,
      expandIcon = context.expandIcon,
      subMenuIcon = context.subMenuIcon,
      triggerSubMenuAction = context.triggerSubMenuAction,
      triggerDisableOpenOnExiting = context.triggerDisableOpenOnExiting,
      TransitionComponent = context.TransitionComponent,
      menuStyles = context.styles,
      forceSubMenuRender = context.forceSubMenuRender,
      inlineCollapsed = context.inlineCollapsed,
      horizontalMode = context.horizontalMode,
      verticalMode = context.verticalMode,
      maxItemCountPerPage = context.maxItemCountPerPage,
      autoItemCountPerPage = context.autoItemCountPerPage,
      size = context.size,
      hideRootMenuSpacing = context.hideRootMenuSpacing,
      autoScrollToSelection = context.autoScrollToSelection,
      mode = context.mode;

  var _useSubMenu = useSubMenu(props, context, titleRef),
      isOpen = _useSubMenu.isOpen,
      _menuId = _useSubMenu._menuId,
      placement = _useSubMenu.placement,
      appear = _useSubMenu.appear,
      hasScrollBar = _useSubMenu.hasScrollBar,
      handleMenuScrollRef = _useSubMenu.handleMenuScrollRef,
      handleTitleMouseEnter = _useSubMenu.handleTitleMouseEnter,
      handleTitleMouseLeave = _useSubMenu.handleTitleMouseLeave,
      handleTitleClick = _useSubMenu.handleTitleClick,
      handleVisibleChange = _useSubMenu.handleVisibleChange,
      handleSubMenuClick = _useSubMenu.handleSubMenuClick,
      handleItemSelect = _useSubMenu.handleItemSelect,
      handleItemDeselect = _useSubMenu.handleItemDeselect,
      isSelected = _useSubMenu.isSelected,
      menuWidth = _useSubMenu.menuWidth;

  var finalIcon = useMemo(function () {
    if (isSelected && selectedIcon) {
      return selectedIcon;
    }

    return icon || subMenuIcon;
  }, [icon, isSelected, selectedIcon, subMenuIcon]);

  var _useMenuBase = useMenuBase({
    mode: mode,
    level: level + 1,
    eventKey: "".concat(eventKey, "_submenu"),
    parentIsRootMenu: false,
    parentMenuOpen: isOpen,
    menuHasIcon: !!finalIcon,
    children: children,
    hasScrollBar: hasScrollBar,
    autoScrollToSelection: autoScrollToSelection,
    onClick: handleSubMenuClick,
    onItemSelect: handleItemSelect,
    onItemDeselect: handleItemDeselect
  }),
      elements = _useMenuBase.elements,
      renderMenuWidget = _useMenuBase.renderMenuWidget;

  var subMenuStyles = useMemo(function () {
    var subMenuStyles = _extends({}, menuStyles, styles);

    var menu = subMenuStyles.menu,
        subMenu = subMenuStyles.subMenu,
        subMenuTitle = subMenuStyles.subMenuTitle,
        menuScrollWrapper = subMenuStyles.menuScrollWrapper;
    return {
      menu: menu,
      subMenu: subMenu,
      subMenuTitle: subMenuTitle,
      menuScrollWrapper: menuScrollWrapper
    };
  }, [menuStyles, styles]);
  var offset = useMemo(function () {
    if (parentMenuScrollable && mode !== 'vertical-right') {
      return theme.components.Menu.subMenu.offsetHasScrollBar;
    } else {
      return theme.components.Menu.subMenu.offset;
    }
  }, [mode, parentMenuScrollable, theme.components.Menu.subMenu.offset, theme.components.Menu.subMenu.offsetHasScrollBar]);
  var innerStyles = useStyles('menu', defaultStyles, subMenuStyles);
  var finalStyle = useMemo(function () {
    return _extends({}, innerStyles.subMenu.style, style);
  }, [innerStyles.subMenu.style, style]);
  var container = parentIsRootMenu && !hideRootMenuSpacing || !titleRef.current ? getPopupContainer : titleRef.current.parentNode;

  var domProps = _omit(props, allSubMenuProp);

  var showTitle = !inlineCollapsed || inlineCollapsed && level > 1;
  var ariaOwns = useMemo(function () {
    var ariaOwns = {};

    if (isOpen) {
      ariaOwns = {
        '': _menuId
      };
    }

    return ariaOwns;
  }, [_menuId, isOpen]);
  var finalPopperProps = useMemo(function () {
    var finalPopperProps = _extends({}, popperProps, {
      lazyMount: !forceSubMenuRender
    });

    if (hasScrollBar) {
      finalPopperProps.modifiers = _extends({}, finalPopperProps.modifiers, {
        preventOverflow: {
          boundariesElement: 'viewport',
          padding: theme.components.Menu.subMenu.preventOverflowPadding / 2
        }
      });
    }

    if (parentMenuScrollable) {
      finalPopperProps.modifiers = _extends({}, finalPopperProps.modifiers, {
        applyStyle: {
          fn: applyStyle
        }
      });
    }

    return finalPopperProps;
  }, [forceSubMenuRender, hasScrollBar, parentMenuScrollable, popperProps, theme.components.Menu.subMenu.preventOverflowPadding]); // 渲染 children

  var childrenNode = useMemo(function () {
    var style;

    if (horizontalMode) {
      style = {
        width: menuWidth
      };
    }

    if (inlineMode && !inlineCollapsed) {
      return /*#__PURE__*/React.createElement(Animation.Collapse, {
        appear: appear,
        easing: "linear",
        timeout: theme.transition.pattern.duration.status,
        "in": isOpen
      }, /*#__PURE__*/React.createElement(StyledMenu, _extends({
        theme: theme,
        $size: size,
        $inlineMode: inlineMode,
        $horizontalMode: horizontalMode,
        $inlineCollapsed: inlineCollapsed,
        $sub: true,
        $hideRootMenuSpacing: hideRootMenuSpacing
      }, innerStyles.menu), /*#__PURE__*/React.createElement(StyledMenuScrollWrapper, _extends({
        ref: handleMenuScrollRef
      }, innerStyles.menuScrollWrapper), elements.map(renderMenuWidget))));
    }

    return /*#__PURE__*/React.createElement(StyledMenu, {
      theme: theme,
      $size: size,
      $inlineMode: inlineMode,
      $horizontalMode: horizontalMode,
      $inlineCollapsed: inlineCollapsed,
      $sub: true,
      $hasScrollBar: hasScrollBar,
      $hideRootMenuSpacing: hideRootMenuSpacing,
      $maxItemCountPerPage: maxItemCountPerPage,
      $autoItemCountPerPage: autoItemCountPerPage,
      className: innerStyles.menu.className,
      style: _extends({}, innerStyles.menu.style, style)
    }, /*#__PURE__*/React.createElement(StyledMenuScrollWrapper, _extends({
      ref: handleMenuScrollRef
    }, innerStyles.menuScrollWrapper), elements.map(renderMenuWidget)));
  }, [appear, autoItemCountPerPage, elements, handleMenuScrollRef, hasScrollBar, hideRootMenuSpacing, horizontalMode, inlineCollapsed, inlineMode, innerStyles.menu, innerStyles.menuScrollWrapper, isOpen, maxItemCountPerPage, menuWidth, renderMenuWidget, size, theme]);
  var iconNode = useMemo(function () {
    if (typeof finalIcon === 'function') {
      return finalIcon(props);
    } else {
      return finalIcon;
    }
  }, [finalIcon, props]);
  var expandIconNode = useMemo(function () {
    if (React.Children.count(children) > 0 && showTitle) {
      var DefaultExpandIcon = theme.components.Menu.expandIcon || _FoldIcon;
      var token = theme.components.Menu,
          lineHeight = theme.typography.spec.lineHeight;
      var rotateProps; // 垂直菜单箭头方向调整

      if (verticalMode || inlineCollapsed && level > 1) {
        rotateProps = {
          $rotate: 270,
          $expandRotate: 270
        };
      }

      return /*#__PURE__*/React.createElement(ExpandWrapper, _extends({
        theme: theme,
        $expanded: isOpen,
        $lineHeight: horizontalMode ? lineHeight[token.fontLevel] : 0
      }, rotateProps), /*#__PURE__*/React.createElement(ExpandIcon, {
        icon: expandIcon,
        DefaultIcon: DefaultExpandIcon,
        disabled: disabled
      }));
    }
  }, [children, disabled, expandIcon, horizontalMode, inlineCollapsed, isOpen, level, showTitle, theme, verticalMode]);
  var titleNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledSubMenuTitleWrapper, _extends({
      ref: titleRef,
      "aria-expanded": isOpen,
      "aria-haspopup": "true",
      onClick: handleTitleClick,
      onMouseEnter: handleTitleMouseEnter,
      onMouseLeave: handleTitleMouseLeave
    }, ariaOwns, {
      title: typeof title === 'string' ? title : undefined
    }), /*#__PURE__*/React.createElement(StyledSubMenuTitle, _extends({}, innerStyles.subMenuTitle, {
      theme: theme,
      $selected: isSelected
    }), /*#__PURE__*/React.createElement(StyledSubMenuTitleAndIconWrapper, null, iconNode, showTitle && title), expandIconNode));
  }, [ariaOwns, expandIconNode, handleTitleClick, handleTitleMouseEnter, handleTitleMouseLeave, iconNode, innerStyles.subMenuTitle, isOpen, isSelected, showTitle, theme, title]);
  var inlineMenu = useMemo(function () {
    if (inlineMode && !inlineCollapsed) {
      return /*#__PURE__*/React.createElement(React.Fragment, null, titleNode, childrenNode);
    }
  }, [childrenNode, inlineCollapsed, inlineMode, titleNode]);
  var verticalOrInlineCollapseMenu = useMemo(function () {
    if (!inlineMode || inlineCollapsed) {
      return /*#__PURE__*/React.createElement(Trigger, {
        hideArrow: true,
        disabled: disabled,
        triggerAction: triggerSubMenuAction,
        disableOpenOnExiting: triggerDisableOpenOnExiting,
        TransitionComponent: TransitionComponent,
        placement: placement,
        open: isOpen,
        offset: offset,
        popup: childrenNode,
        flip: flip,
        triggerId: triggerId,
        arrowPointAtCenter: arrowPointAtCenter,
        popperProps: finalPopperProps,
        container: container,
        onVisibleChange: handleVisibleChange
      }, titleNode);
    }
  }, [TransitionComponent, arrowPointAtCenter, childrenNode, container, disabled, finalPopperProps, flip, handleVisibleChange, inlineCollapsed, inlineMode, isOpen, offset, placement, titleNode, triggerDisableOpenOnExiting, triggerId, triggerSubMenuAction]);
  return /*#__PURE__*/React.createElement(StyledSubMenu, _extends({
    ref: ref,
    role: "menuitem"
  }, domProps, {
    className: [innerStyles.subMenu.className, className].join(' ').trim(),
    style: finalStyle,
    theme: theme,
    $disabled: disabled,
    $level: level,
    $menuHasIcon: menuHasIcon,
    $inlineMode: inlineMode,
    $inlineCollapsed: inlineCollapsed,
    $selected: isSelected,
    $horizontalMode: horizontalMode,
    $isOpen: isOpen
  }), inlineMenu, verticalOrInlineCollapseMenu);
});
SubMenu.displayName = 'SubMenu';
export default SubMenu;