import _extends from "@babel/runtime/helpers/extends";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import LayoutSiderContext from '../Layout/LayoutSiderContext';
import { useCallback, useEffect, useMemo, useRef, useState, useContext } from 'react';
import { shallowEqualArrays } from 'shallow-equal';
// eslint-disable-next-line complexity
export var useMenu = function useMenu(props, rootMenuRef) {
  var _props$defaultSelecte = props.defaultSelectedKeys,
      defaultSelectedKeys = _props$defaultSelecte === void 0 ? [] : _props$defaultSelecte,
      selectedKeys = props.selectedKeys,
      _props$defaultOpenKey = props.defaultOpenKeys,
      defaultOpenKeys = _props$defaultOpenKey === void 0 ? [] : _props$defaultOpenKey,
      openKeys = props.openKeys,
      _props$selectable = props.selectable,
      selectable = _props$selectable === void 0 ? true : _props$selectable,
      _props$inlineCollapse = props.inlineCollapsed,
      inlineCollapsed = _props$inlineCollapse === void 0 ? false : _props$inlineCollapse,
      _props$accordion = props.accordion,
      accordion = _props$accordion === void 0 ? false : _props$accordion,
      _props$mode = props.mode,
      mode = _props$mode === void 0 ? 'vertical' : _props$mode,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      TransitionComponent = props.TransitionComponent,
      _props$forceSubMenuRe = props.forceSubMenuRender,
      forceSubMenuRender = _props$forceSubMenuRe === void 0 ? false : _props$forceSubMenuRe,
      _props$multiple = props.multiple,
      multiple = _props$multiple === void 0 ? false : _props$multiple,
      _props$triggerSubMenu = props.triggerSubMenuAction,
      triggerSubMenuAction = _props$triggerSubMenu === void 0 ? 'hover' : _props$triggerSubMenu,
      _props$triggerDisable = props.triggerDisableOpenOnExiting,
      triggerDisableOpenOnExiting = _props$triggerDisable === void 0 ? false : _props$triggerDisable,
      expandIcon = props.expandIcon,
      itemIcon = props.itemIcon,
      subMenuIcon = props.subMenuIcon,
      autoScrollToSelection = props.autoScrollToSelection,
      getPopupContainer = props.getPopupContainer,
      _props$hideRootMenuSp = props.hideRootMenuSpacing,
      hideRootMenuSpacing = _props$hideRootMenuSp === void 0 ? false : _props$hideRootMenuSp,
      _props$maxItemCountPe = props.maxItemCountPerPage,
      maxItemCountPerPage = _props$maxItemCountPe === void 0 ? 6.5 : _props$maxItemCountPe,
      autoItemCountPerPage = props.autoItemCountPerPage,
      renderMenuItemChild = props.renderMenuItemChild,
      onOpenChange = props.onOpenChange,
      onSelectChange = props.onSelectChange,
      onItemSelect = props.onItemSelect,
      onItemDeselect = props.onItemDeselect,
      styles = props.styles;

  var _useContext = useContext(LayoutSiderContext),
      siderCollapsed = _useContext.siderCollapsed;

  var isSelectControlled = ('selectedKeys' in props);
  var isOpenControlled = ('openKeys' in props);
  var isInlineCollapsedControlled = 'inlineCollapsed' in props || typeof siderCollapsed !== 'undefined';

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      hasScrollBar = _useState2[0],
      setHasScrollBar = _useState2[1];

  var prevInlineCollapsedOpenKeys = useRef((isOpenControlled ? openKeys : defaultOpenKeys) || []);

  var _useState3 = useState((isSelectControlled ? selectedKeys : defaultSelectedKeys) || []),
      _useState4 = _slicedToArray(_useState3, 2),
      selectedKeysState = _useState4[0],
      setSelectedKeysState = _useState4[1];

  var _useState5 = useState((isOpenControlled ? openKeys : defaultOpenKeys) || []),
      _useState6 = _slicedToArray(_useState5, 2),
      openKeysState = _useState6[0],
      setOpenKeysState = _useState6[1];

  var inlineMode = mode === 'inline';
  var horizontalMode = mode === 'horizontal';
  var verticalMode = !inlineMode && !horizontalMode;
  var finalInlineCollapsed = inlineMode && (siderCollapsed || inlineCollapsed);
  var finalTriggerSubMenuAction = inlineMode && !finalInlineCollapsed ? 'click' : triggerSubMenuAction;
  var handleMenuScrollRef = useCallback(function (el) {
    if (el) {
      setHasScrollBar(el.scrollHeight > el.offsetHeight);
    }
  }, []);

  var handleOpenChange = _useEventCallback(function (openInfo) {
    var changed = false;

    var newOpenKeys = _toConsumableArray(openKeysState);
    /**
     * 手风琴菜单
     */


    if (openInfo.parentIsRootMenu && openInfo.open && accordion) {
      newOpenKeys = [];
      changed = true;
    }

    if (openInfo.open) {
      // 当前打开的 keys 中不存在那么添加
      changed = newOpenKeys.indexOf(openInfo.key) === -1;

      if (changed) {
        newOpenKeys.push(openInfo.key);
      }
    } else {
      // 当前打开的 keys 中存在那么移除
      var index = newOpenKeys.indexOf(openInfo.key);
      changed = index !== -1;

      if (changed) {
        newOpenKeys.splice(index, 1);
      }
    }

    if (changed) {
      if (onOpenChange) {
        onOpenChange(newOpenKeys);
      }

      if (!isOpenControlled) {
        setOpenKeysState(newOpenKeys);
      }
    }
  });

  var handleItemSelect = useCallback(function (selectInfo) {
    if (selectable) {
      var newSelectedKeys;
      var selectedKey = selectInfo.key;

      if (multiple) {
        newSelectedKeys = [].concat(_toConsumableArray(selectedKeysState), [selectedKey]);
      } else {
        newSelectedKeys = [selectedKey];
      }

      if (!isSelectControlled) {
        setSelectedKeysState(newSelectedKeys);
      }

      if (onSelectChange) {
        onSelectChange(_extends({}, selectInfo, {
          selectedKeys: selectedKeysState,
          newSelectedKeys: newSelectedKeys
        }));
      }

      if (onItemSelect) {
        onItemSelect(_extends({}, selectInfo, {
          selectedKeys: selectedKeysState,
          newSelectedKeys: newSelectedKeys
        }));
      } // 单选，弹出菜单时自动关闭


      if (!multiple && (!inlineMode || finalInlineCollapsed)) {
        if (onOpenChange) {
          onOpenChange([]);
        }

        if (!isOpenControlled) {
          setOpenKeysState([]);
        }
      }
    }
  }, [finalInlineCollapsed, inlineMode, isOpenControlled, isSelectControlled, multiple, onItemSelect, onOpenChange, onSelectChange, selectable, selectedKeysState]);
  var handleItemDeselect = useCallback(function (selectInfo) {
    if (selectable) {
      var newSelectedKeys = _toConsumableArray(selectedKeysState);

      var selectedKey = selectInfo.key;
      var index = newSelectedKeys.indexOf(selectedKey);

      if (index !== -1) {
        newSelectedKeys.splice(index, 1);
      }

      if (!isSelectControlled) {
        setSelectedKeysState(newSelectedKeys);
      }

      if (onSelectChange) {
        onSelectChange(_extends({}, selectInfo, {
          selectedKeys: selectedKeysState,
          newSelectedKeys: newSelectedKeys
        }));
      }

      if (onItemDeselect) {
        onItemDeselect(_extends({}, selectInfo, {
          selectedKeys: selectedKeysState,
          newSelectedKeys: newSelectedKeys
        }));
      }
    }
  }, [isSelectControlled, onItemDeselect, onSelectChange, selectable, selectedKeysState]); // 受控状态更新

  if (isSelectControlled && !shallowEqualArrays(selectedKeysState, selectedKeys || [])) {
    setSelectedKeysState(selectedKeys || []);
  }

  if (isOpenControlled && !shallowEqualArrays(openKeysState, openKeys || [])) {
    setOpenKeysState(openKeys || []);
  }

  useEffect(function () {
    if (isInlineCollapsedControlled) {
      // 关闭的时候保存开关状态
      if (finalInlineCollapsed) {
        prevInlineCollapsedOpenKeys.current = openKeysState;
      }

      var newOpenKeys = finalInlineCollapsed ? [] : prevInlineCollapsedOpenKeys.current;

      if (onOpenChange) {
        onOpenChange(newOpenKeys);
      }

      if (!isOpenControlled) {
        setOpenKeysState(newOpenKeys);
      } // 打开的时候清空保存的开关状态


      if (!finalInlineCollapsed) {
        prevInlineCollapsedOpenKeys.current = [];
      }
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [finalInlineCollapsed, isInlineCollapsedControlled]);
  var contextValue = {
    rootMenuScrollable: hasScrollBar,
    size: size,
    mode: mode,
    inlineMode: inlineMode,
    horizontalMode: horizontalMode,
    verticalMode: verticalMode,
    autoScrollToSelection: autoScrollToSelection,
    triggerSubMenuAction: finalTriggerSubMenuAction,
    triggerDisableOpenOnExiting: triggerDisableOpenOnExiting,
    TransitionComponent: TransitionComponent,
    forceSubMenuRender: forceSubMenuRender,
    inlineCollapsed: finalInlineCollapsed,
    multiple: multiple,
    expandIcon: expandIcon,
    itemIcon: itemIcon,
    subMenuIcon: subMenuIcon,
    getPopupContainer: getPopupContainer,
    hideRootMenuSpacing: hideRootMenuSpacing,
    maxItemCountPerPage: maxItemCountPerPage,
    autoItemCountPerPage: autoItemCountPerPage,
    openKeys: openKeysState,
    selectedKeys: selectedKeysState,
    styles: styles,
    rootMenuRef: rootMenuRef,
    renderMenuItemChild: renderMenuItemChild,
    handleOpenChange: handleOpenChange,
    handleItemSelect: handleItemSelect,
    handleItemDeselect: handleItemDeselect,
    handleMenuScrollRef: handleMenuScrollRef
  }; // eslint-disable-next-line react-hooks/exhaustive-deps

  return useMemo(function () {
    return contextValue;
  }, [contextValue.rootMenuScrollable, contextValue.openKeys, contextValue.selectedKeys, contextValue.size, contextValue.mode, contextValue.inlineMode, contextValue.verticalMode, contextValue.horizontalMode, contextValue.inlineCollapsed, contextValue.multiple, contextValue.expandIcon, contextValue.itemIcon, contextValue.subMenuIcon, contextValue.hideRootMenuSpacing, contextValue.maxItemCountPerPage, contextValue.autoItemCountPerPage, contextValue.autoScrollToSelection]);
};