import { cloneElement, useMemo } from 'react';
import { getKeyFromChildrenIndex, getValidChildElements } from './utils';
export var useMenuBase = function useMenuBase(props) {
  var level = props.level,
      eventKey = props.eventKey,
      hasScrollBar = props.hasScrollBar,
      parentIsRootMenu = props.parentIsRootMenu,
      parentMenuOpen = props.parentMenuOpen,
      menuHasIcon = props.menuHasIcon,
      children = props.children,
      onClick = props.onClick,
      onItemSelect = props.onItemSelect,
      onItemDeselect = props.onItemDeselect,
      _props$autoScrollToSe = props.autoScrollToSelection,
      autoScrollToSelection = _props$autoScrollToSe === void 0 ? true : _props$autoScrollToSe; // 获取子节点

  var elements = useMemo(function () {
    return getValidChildElements(children);
  }, [children]); // 渲染子节点

  var renderMenuWidget = function renderMenuWidget(child, i) {
    if (!child) {
      return null;
    }

    if (child.type && child.type.displayName === 'MenuDivider') {
      return child;
    }

    var key = getKeyFromChildrenIndex(child, eventKey, i);
    var childProps = child.props;

    if (!childProps || typeof child.type === 'string') {
      return child;
    }

    return /*#__PURE__*/cloneElement(child, {
      level: level,
      index: i,
      eventKey: key,
      parentEventKey: eventKey,
      parentMenuOpen: parentMenuOpen,
      parentIsRootMenu: parentIsRootMenu,
      parentMenuScrollable: hasScrollBar && autoScrollToSelection,
      menuHasIcon: menuHasIcon,
      originOnClick: childProps.onClick,
      onClick: onClick,
      onItemSelect: onItemSelect,
      onItemDeselect: onItemDeselect
    });
  };

  return {
    elements: elements,
    renderMenuWidget: renderMenuWidget
  };
};