import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _extends from "@babel/runtime/helpers/extends";
import { useEffect, useMemo } from 'react';
import scrollIntoView from '../utils/scrollIntoView';

/* istanbul ignore next */
export var useMenuItem = function useMenuItem(props, context, itemRef) {
  var selectedKeys = context.selectedKeys,
      multiple = context.multiple,
      inlineCollapsed = context.inlineCollapsed,
      inlineMode = context.inlineMode,
      verticalMode = context.verticalMode,
      horizontalMode = context.horizontalMode,
      hideRootMenuSpacing = context.hideRootMenuSpacing,
      rootMenuRef = context.rootMenuRef,
      rootMenuScrollable = context.rootMenuScrollable;
  var disabled = props.disabled,
      eventKey = props.eventKey,
      onClick = props.onClick,
      originOnClick = props.originOnClick,
      level = props.level,
      role = props.role,
      title = props.title,
      parentMenuOpen = props.parentMenuOpen,
      parentMenuScrollable = props.parentMenuScrollable,
      onItemSelect = props.onItemSelect,
      onItemDeselect = props.onItemDeselect; // item 的选中状态

  var isSelected = selectedKeys.indexOf(eventKey) !== -1;
  var showChildren = !inlineCollapsed || inlineCollapsed && level > 1;
  var attrs = useMemo(function () {
    var attrs = {
      title: title,
      role: role || 'menuitem',
      'aria-disabled': disabled
    };

    if (role === 'option') {
      attrs = _extends({}, attrs, {
        role: 'option',
        'aria-selected': isSelected
      });
    } else if (role === null || role === 'none') {
      attrs.role = 'none';
    }
  }, [disabled, isSelected, role, title]); // 激活时滚动到该 item

  useEffect(function () {
    // 父菜单打开，且选中的情况下需要进行滚动
    if (parentMenuOpen && isSelected && itemRef.current) {
      // 弹出菜单
      if (parentMenuScrollable && ((inlineMode && inlineCollapsed || verticalMode || horizontalMode) && level > 0 || verticalMode && hideRootMenuSpacing && level >= 0)) {
        scrollIntoView(itemRef.current, {
          onlyFirstScrollableParent: true,
          behavior: 'auto'
        }); // 非弹出菜单按根节点进行滚动
      } else if (rootMenuRef.current && rootMenuScrollable) {
        scrollIntoView(itemRef.current, {
          parentNode: rootMenuRef.current,
          behavior: 'auto'
        });
      }
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [parentMenuOpen, parentMenuScrollable, rootMenuScrollable, isSelected, itemRef]); // 点击时调用外部的点击函数 + 处理选中状态

  var handleClick = _useEventCallback(function (e) {
    e.stopPropagation();

    if (disabled) {
      return;
    }

    var info = {
      key: eventKey,
      keyPath: [eventKey],
      item: itemRef,
      nativeEvent: e
    };

    if (multiple) {
      if (isSelected) {
        onItemDeselect(info);
      } else {
        onItemSelect(info);
      }
    } else if (!isSelected) {
      onItemSelect(info);
    }

    if (onClick) {
      onClick(info);
    }

    if (originOnClick) {
      originOnClick(info);
    }
  }, []);

  return {
    attrs: attrs,
    showChildren: showChildren,
    isSelected: isSelected,
    handleClick: handleClick
  };
};