import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { placementMap } from './const';
import { loopMenuItemRecursively } from './utils';
/* istanbul ignore next */

export var useSubMenu = function useSubMenu(props, context, titleRef) {
  var openKeys = context.openKeys,
      selectedKeys = context.selectedKeys,
      mode = context.mode,
      multiple = context.multiple,
      inlineMode = context.inlineMode,
      horizontalMode = context.horizontalMode,
      inlineCollapsed = context.inlineCollapsed,
      triggerSubMenuAction = context.triggerSubMenuAction,
      handleOpenChange = context.handleOpenChange;
  var eventKey = props.eventKey,
      onClick = props.onClick,
      originOnClick = props.originOnClick,
      onTitleMouseEnter = props.onTitleMouseEnter,
      onTitleMouseLeave = props.onTitleMouseLeave,
      onTitleClick = props.onTitleClick,
      parentIsRootMenu = props.parentIsRootMenu,
      disabled = props.disabled,
      children = props.children,
      onItemSelect = props.onItemSelect,
      onItemDeselect = props.onItemDeselect;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      hasScrollBar = _useState2[0],
      setHasScrollBar = _useState2[1];

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      appear = _useState4[0],
      setAppear = _useState4[1];

  var menuWidthRef = useRef(0);
  var mouseenterTimeoutRef = useRef();
  var isOpen = useMemo(function () {
    return openKeys.indexOf(eventKey) > -1;
  }, [eventKey, openKeys]);

  var _menuId = "".concat(eventKey, "$Menu");

  var placement = placementMap[mode]; // item 的选中状态

  var isSelected = useMemo(function () {
    var ret = {
      find: false
    };
    loopMenuItemRecursively(children, selectedKeys, ret);
    return selectedKeys.indexOf(eventKey) !== -1 || ret.find;
  }, [children, eventKey, selectedKeys]);
  var handleMenuScrollRef = useCallback(function (el) {
    if (el) {
      setHasScrollBar(el.scrollHeight > el.offsetHeight);
    }
  }, []);
  /**
   * 触发 open 事件，点击 title 或者 popup 展现
   * 如果没有 children 那么还会触发选中行为
   */

  var triggerOpenChange = useCallback(function (open, type) {
    if (open === isOpen) return; // 第一次打开是将 appear 设为 true，使得后面的动画正常

    if (inlineMode && !inlineCollapsed && !appear) {
      setAppear(true);
    }

    var changeInfo = {
      key: eventKey,
      item: titleRef,
      trigger: type,
      open: open,
      parentIsRootMenu: parentIsRootMenu
    };

    if (type === 'mouseenter') {
      // make sure mouseenter happen after other menu item's mouseleave
      mouseenterTimeoutRef.current = setTimeout(function () {
        handleOpenChange(changeInfo);
      }, 0);
    } else {
      handleOpenChange(changeInfo);
    }
  }, [appear, eventKey, handleOpenChange, inlineCollapsed, inlineMode, isOpen, parentIsRootMenu, titleRef]);
  var addKeyPath = useCallback(function (info) {
    return _extends({}, info, {
      keyPath: (info.keyPath || []).concat([eventKey])
    });
  }, [eventKey]); // title 的事件

  var handleTitleMouseEnter = useCallback(function (e) {
    if (disabled) return;

    if (onTitleMouseEnter) {
      onTitleMouseEnter({
        key: eventKey,
        nativeEvent: e
      });
    }
  }, [disabled, eventKey, onTitleMouseEnter]);
  var handleTitleMouseLeave = useCallback(function (e) {
    if (disabled) return;

    if (onTitleMouseLeave) {
      onTitleMouseLeave({
        key: eventKey,
        nativeEvent: e
      });
    }
  }, [disabled, eventKey, onTitleMouseLeave]);
  var handleItemSelect = useCallback(function (info) {
    if (onItemSelect) {
      onItemSelect(addKeyPath(info));
    }
  }, [addKeyPath, onItemSelect]);
  var handleItemDeselect = useCallback(function (info) {
    if (onItemDeselect) {
      onItemDeselect(addKeyPath(info));
    }
  }, [addKeyPath, onItemDeselect]);
  var handleTitleClick = useCallback(function (e) {
    e.stopPropagation();
    if (disabled) return;

    if (onTitleClick) {
      onTitleClick({
        key: eventKey,
        nativeEvent: e
      });
    }

    if (horizontalMode && titleRef.current) {
      menuWidthRef.current = parseFloat(getComputedStyle(titleRef.current).width || '0');
    }

    if (inlineMode && !inlineCollapsed) {
      triggerOpenChange(!isOpen, 'click');
    } // 没有 children 那么触发选中


    if (React.Children.count(children) === 0) {
      var info = {
        key: eventKey,
        keyPath: [eventKey],
        item: titleRef,
        nativeEvent: e
      };

      if (multiple) {
        if (isSelected) {
          handleItemDeselect(info);
        } else {
          handleItemSelect(info);
        }
      } else if (!isSelected) {
        handleItemSelect(info);
      }
    }
  }, [children, disabled, eventKey, handleItemDeselect, handleItemSelect, horizontalMode, inlineCollapsed, inlineMode, isOpen, isSelected, multiple, onTitleClick, titleRef, triggerOpenChange]);
  var handleVisibleChange = useCallback(function (visible) {
    if (triggerSubMenuAction === 'hover') {
      triggerOpenChange(visible, visible ? 'mouseenter' : 'mouseleave');
    } else {
      triggerOpenChange(visible, 'click');
    }
  }, [triggerOpenChange, triggerSubMenuAction]);
  var handleSubMenuClick = useCallback(function (info) {
    if (onClick) {
      onClick(addKeyPath(info));
    }

    if (originOnClick) {
      originOnClick(addKeyPath(info));
    }
  }, [addKeyPath, onClick, originOnClick]);
  useEffect(function () {
    return function () {
      if (mouseenterTimeoutRef.current) {
        clearTimeout(mouseenterTimeoutRef.current);
      }
    };
  });
  return {
    isOpen: isOpen,
    _menuId: _menuId,
    placement: placement,
    appear: appear,
    hasScrollBar: hasScrollBar,
    handleMenuScrollRef: handleMenuScrollRef,
    handleTitleMouseEnter: handleTitleMouseEnter,
    handleTitleMouseLeave: handleTitleMouseLeave,
    handleTitleClick: handleTitleClick,
    handleVisibleChange: handleVisibleChange,
    handleSubMenuClick: handleSubMenuClick,
    handleItemSelect: handleItemSelect,
    handleItemDeselect: handleItemDeselect,
    isSelected: isSelected,
    menuWidth: menuWidthRef.current
  };
};