import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _extends from "@babel/runtime/helpers/extends";
import _classCallCheck from "@babel/runtime/helpers/classCallCheck";
import _createClass from "@babel/runtime/helpers/createClass";
import _inherits from "@babel/runtime/helpers/inherits";
import _possibleConstructorReturn from "@babel/runtime/helpers/possibleConstructorReturn";
import _getPrototypeOf from "@babel/runtime/helpers/getPrototypeOf";

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

import { EventEmitter } from 'events';

var ExpireQueue = /*#__PURE__*/function (_EventEmitter) {
  _inherits(ExpireQueue, _EventEmitter);

  var _super = _createSuper(ExpireQueue);

  function ExpireQueue() {
    var _this;

    _classCallCheck(this, ExpireQueue);

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _super.call.apply(_super, [this].concat(args));
    _this._items = [];
    _this._timer = 0;
    _this._setting = {
      interval: 3000,
      timeout: 300,
      max: 3
    };
    _this._id = 0;
    return _this;
  }

  _createClass(ExpireQueue, [{
    key: "refresh",
    value: function refresh() {
      var now = Date.now();

      var items = this._items.filter(function (item) {
        return item.fixed || item.expiration && item.expiration > now;
      });

      if (items.length !== this._items.length) {
        this._items = items;
        this.emit('update');
      }
    }
  }, {
    key: "stop",
    value: function stop() {
      if (this._timer) {
        clearTimeout(this._timer);
      }
    }
  }, {
    key: "tick",
    value: function tick() {
      var _this2 = this;

      this.stop();
      var items = this._items;

      if (items.length < 1) {
        return;
      }

      var _this$_getMinExpTime = this._getMinExpTime(),
          intervalTime = _this$_getMinExpTime.time; // 当所有都是 undefined 的时候，表示都是 fixed 不需要心跳处理了


      if (intervalTime === undefined) {
        return;
      }

      if (intervalTime <= 0) {
        this.refresh();
      } else {
        this._timer = setTimeout(function () {
          _this2.refresh();
        }, intervalTime);
      }
    }
  }, {
    key: "reset",
    value: function reset(setting) {
      var oldSetting = this._setting;

      var newSetting = _extends({}, oldSetting, setting);

      this._setting = newSetting;
    }
  }, {
    key: "remove",
    value: function remove(id) {
      var items = this._items.filter(function (item) {
        return item.id !== id;
      });

      this._items = items;
      this.emit('update');
    }
  }, {
    key: "fixedItem",
    value: function fixedItem(id) {
      this._setItemFixed(id, true);
    }
  }, {
    key: "unFixedItem",
    value: function unFixedItem(id) {
      this._setItemFixed(id, false);
    }
  }, {
    key: "pop",
    value: function pop() {
      var item = this._items.pop();

      if (item) {
        this.emit('update');
      }

      return item;
    }
  }, {
    key: "shift",
    value: function shift() {
      var item = this._items.shift();

      if (item) {
        this.emit('update');
      }

      return item;
    }
  }, {
    key: "unshift",
    value: function unshift(item) {
      var max = this._setting.max;

      var newItem = this._formatItem(item);

      this._items.unshift(newItem);

      if (this._items.length > max) {
        this._items.pop();
      } // this.execTime = performance.now();


      this.emit('update');
      return newItem.id;
    }
  }, {
    key: "push",
    value: function push(item) {
      var max = this._setting.max;

      var newItem = this._formatItem(item);

      var index = this._items.findIndex(function (it) {
        return it.id === item.id;
      }); // 如果item的id在队列中已存在，替换


      if (index !== -1) {
        this._items.splice(index, 1, newItem);
      } else {
        this._items.push(newItem);

        if (this._items.length > max) {
          this._items.shift();
        }
      } // this.execTime = performance.now();


      this.emit('update');
      return newItem.id;
    }
  }, {
    key: "_formatItem",
    value: function _formatItem(item) {
      var _this$_setting = this._setting,
          defaultInterval = _this$_setting.interval,
          timeout = _this$_setting.timeout;

      if (!item.expiration) {
        var interval = item.interval || defaultInterval;
        var delay = interval + timeout;
        item.expiration = Date.now() + delay;
      } // 内部记录的自增id在外面不传入id时正常递增，传入id时，将自增id置为外部传入的值


      if (!item.id && item.id !== 0) {
        item.id = this._id++;
      } else {
        this._id = item.id;
      }

      item.fixed = item.fixed || false;
      return item;
    }
  }, {
    key: "_setItemFixed",
    value: function _setItemFixed(id, fixed) {
      this._items.forEach(function (item) {
        if (item.id === id) {
          item.fixed = fixed;
        }
      });
    }
  }, {
    key: "_getMinExpTime",
    value: function _getMinExpTime() {
      var now = Date.now();
      var items = this._items;
      var time;

      for (var i = 0, l = items.length; i < l; i++) {
        var item = items[i];
        var newTime = item.expiration - now;

        if (!item.fixed && (!time || newTime < time)) {
          time = newTime;
        }
      }

      return {
        time: time
      };
    }
  }, {
    key: "setting",
    get: function get() {
      return this._setting;
    }
  }, {
    key: "length",
    get: function get() {
      return this._items.length;
    }
  }, {
    key: "items",
    get: function get() {
      return _toConsumableArray(this._items);
    }
  }]);

  return ExpireQueue;
}(EventEmitter);

export { ExpireQueue as default };