import { useEffectOnce as _useEffectOnce } from "@muya-ui/utils/es/hooks/useEffectOnce";
import { CloseIcon as _CloseIcon } from "@muya-ui/theme-light/es/icons/CloseIcon";
import { ReloadIcon as _ReloadIcon } from "@muya-ui/theme-light/es/icons/ReloadIcon";
import { ReminderIcon as _ReminderIcon } from "@muya-ui/theme-light/es/icons/ReminderIcon";
import { SuccessIcon as _SuccessIcon } from "@muya-ui/theme-light/es/icons/SuccessIcon";
import { ClearIcon as _ClearIcon } from "@muya-ui/theme-light/es/icons/ClearIcon";
import { InformIcon as _InformIcon } from "@muya-ui/theme-light/es/icons/InformIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["item", "state", "timeout", "onClose", "enterFrom", "expireQueue", "hoverStop", "onMouseEnter", "onMouseLeave", "className", "style"];
import React, { useRef } from 'react';
import styled, { css } from 'styled-components';
import IconButton from '../IconButton';
import Spin from '../Spin';
import Typography from '../Typography';
import { StyledTitle } from '../Typography/Title';
import forkHandler from '../utils/forkHandler';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
var StyledItem = styled.div.withConfig({
  displayName: "StyledItem"
})(["", ""], function (props) {
  var theme = props.theme;
  var colors = theme.colors;
  var token = theme.components.Notification.notification;
  var itemMarginBottom = token.itemMarginBottom,
      borderRadius = token.borderRadius,
      width = token.width,
      minHeight = token.minHeight,
      shadow = token.shadow;
  return css(["display:flex;flex-direction:column;background:", ";box-shadow:", ";border-radius:", ";margin-bottom:", "px;width:", "px;min-height:", "px;pointer-events:all;"], colors.pattern.background.higher, shadow, borderRadius, itemMarginBottom, width, minHeight);
});
var BaseNode = styled.div.withConfig({
  displayName: "BaseNode"
})([""]);
var StyledContent = styled.div.withConfig({
  displayName: "StyledContent"
})(["", ""], function (props) {
  var theme = props.theme;
  var token = theme.components.Notification.notification;
  var contentStyle = token.contentStyle;
  return css(["width:100%;position:relative;box-sizing:border-box;padding:", ";"], contentStyle.plainPadding);
});
var StyledActions = styled.div.withConfig({
  displayName: "StyledActions"
})(["", ""], function (props) {
  var theme = props.theme;
  var token = theme.components.Notification.notification.actionsStyle;
  return css(["padding:", ";"], token.padding);
});
var StyledIcon = styled.div.withConfig({
  displayName: "StyledIcon"
})(["", ""], function (props) {
  var theme = props.theme;
  var token = theme.components.Notification.notification;
  var iconStyle = token.iconStyle;
  return css(["position:absolute;font-size:", "px;line-height:1;top:", "px;left:", "px;"], iconStyle.size, iconStyle.top, iconStyle.left);
});
var StyledClose = styled(IconButton).withConfig({
  displayName: "StyledClose"
})([""]);
var defaultStyles = {
  self: '',
  close: '',
  container: '',
  icon: '',
  title: '',
  content: '',
  contentWrapper: ''
};
var NotificationItemWithoutStyle = memoForwardRef(function (props, ref) {
  var item = props.item,
      state = props.state,
      timeout = props.timeout,
      onClose = props.onClose,
      enterFrom = props.enterFrom,
      expireQueue = props.expireQueue,
      hoverStop = props.hoverStop,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      className = props.className,
      _props$style = props.style,
      style = _props$style === void 0 ? {} : _props$style,
      otherProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('notification-item', defaultStyles, item.styles);
  var token = theme.components.Notification;
  var iconNode;

  if (item.icon) {
    iconNode = /*#__PURE__*/React.createElement(StyledIcon, _extends({}, innerStyles.icon, {
      theme: theme
    }), item.icon);
  } else if (item.type && item.type !== 'loading') {
    var tokenIcon = _extends({
      info: _InformIcon,
      error: _ClearIcon,
      success: _SuccessIcon,
      warning: _ReminderIcon,
      loading: _ReloadIcon
    }, token.icon);

    var Icon = tokenIcon[item.type];
    var bgColor = token.iconBgColor[item.type];
    iconNode = /*#__PURE__*/React.createElement(StyledIcon, _extends({}, innerStyles.icon, {
      theme: theme
    }), /*#__PURE__*/React.createElement(Icon, {
      bgColor: bgColor
    }));
  } else if (item.type && item.type === 'loading') {
    iconNode = /*#__PURE__*/React.createElement(StyledIcon, _extends({}, innerStyles.icon, {
      theme: theme
    }), /*#__PURE__*/React.createElement(Spin, null));
  }

  var titleNode;

  if (typeof item.title === 'string') {
    titleNode = /*#__PURE__*/React.createElement(Typography.Title, _extends({}, innerStyles.title, {
      level: token.notification.titleLevel,
      ellipsis: true
    }), item.title);
  } else {
    titleNode = item.title;
  }

  var contentNode;

  if (typeof item.content === 'string') {
    contentNode = /*#__PURE__*/React.createElement(Typography.Text, innerStyles.content, item.content);
  } else {
    contentNode = item.content;
  }

  var CloseIcon = token.notification.closeBtn || _CloseIcon;
  var notFixed = useRef(!item.fixed);
  var handleMouseEnter = forkHandler(function () {
    if (hoverStop && notFixed.current) {
      expireQueue.fixedItem(item.id);
    }
  }, onMouseEnter);
  var handleMouseLeave = forkHandler(function () {
    if (hoverStop && notFixed.current) {
      expireQueue.unFixedItem(item.id);
      expireQueue.tick();
    }
  }, onMouseLeave);

  var containerProps = _extends({
    style: _extends({}, innerStyles.self.style, style),
    className: [className, innerStyles.self.className].join(' ')
  }, otherProps);

  _useEffectOnce(function () {
    return function () {
      if (item.onClose) {
        item.onClose();
      }
    };
  });

  return /*#__PURE__*/React.createElement(BaseNode, _extends({
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave,
    ref: ref
  }, containerProps), /*#__PURE__*/React.createElement(StyledItem, _extends({}, innerStyles.container, {
    theme: theme
  }), /*#__PURE__*/React.createElement(StyledContent, _extends({
    theme: theme
  }, innerStyles.contentWrapper), iconNode, titleNode, contentNode, /*#__PURE__*/React.createElement(StyledClose, _extends({
    theme: theme
  }, innerStyles.close, {
    onClick: onClose
  }), /*#__PURE__*/React.createElement(CloseIcon, null))), item.actions && /*#__PURE__*/React.createElement(StyledActions, null, item.actions)));
});
var NotificationItem = styled(NotificationItemWithoutStyle).withConfig({
  displayName: "NotificationItem"
})(["", ""], function (props) {
  var theme = props.theme,
      item = props.item,
      state = props.state,
      timeout = props.timeout,
      _props$enterFrom = props.enterFrom,
      enterFrom = _props$enterFrom === void 0 ? 'right' : _props$enterFrom;
  var colors = theme.colors,
      transition = theme.transition;
  var containerTop = theme.components.Notification.containerTop;
  var token = theme.components.Notification.notification;
  var contentStyle = token.contentStyle,
      lineMaxChar = token.lineMaxChar;
  var maxHeight = item.maxHeight || 120;
  var hasIcon = item.type || item.icon;

  if (typeof item.content === 'string' && item.content.length > 40) {
    var lineNum = Math.ceil(item.content.length / lineMaxChar);
    maxHeight = token.minHeight + lineNum * contentStyle.lineHeight;
  }

  var left = containerTop + token.width;
  left = enterFrom === 'right' ? left : -left;
  var transitionMap = {
    entering: '',
    entered: css(["max-height:", "px;transform:translate(0,0);transition-timing-function:", ";opacity:1;"], maxHeight, transition.pattern.easing.enter),
    exiting: css(["max-height:0;opacity:0;transform:translate(0,0);transition-timing-function:", ";"], transition.pattern.easing.leave),
    exited: '',
    unmounted: ''
  };
  var transformCSS = transitionMap[state] || css(["max-height:0;opacity:0;transform:translate(0,0);"]);
  return css(["transition:all ", "ms ", ";pointer-events:all;display:flex;align-items:center;justify-content:center;", " ", " & ", "{margin-bottom:", "px;}& ", "{color:", ";}& ", "{position:absolute;font-size:", "px;top:", "px;right:", "px;}"], timeout, token.easing, transformCSS, hasIcon && css(["& ", "{padding:", ";}"], StyledContent, contentStyle.iconPadding), StyledTitle, token.titleMarginBottom, StyledIcon, colors.pattern.feature[item.type || 'success'], StyledClose, token.closeStyle.size, token.closeStyle.top, token.closeStyle.right);
});
export default withThemeForStyled(NotificationItem);