import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["onMouseEnter", "onMouseLeave", "position", "expireQueue", "hoverStop", "fullScreen", "children", "container"];
import React, { useRef } from 'react';
import { Transition, TransitionGroup } from 'react-transition-group';
import styled, { css } from 'styled-components';
import Portal from '../Portal';
import { withThemeForStyled } from '../utils/withTheme';
import useExpireQueue from './useNotificationList';
var BaseNode = styled.div.withConfig({
  displayName: "BaseNode"
})([""]);
var NotificationListWithoutStyle = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      position = props.position,
      expireQueue = props.expireQueue,
      hoverStop = props.hoverStop,
      fullScreen = props.fullScreen,
      children = props.children,
      container = props.container,
      otherProps = _objectWithoutProperties(props, _excluded);

  var containerRef = useRef(null);

  var handleRef = _useForkRef(containerRef, ref);

  var _useExpireQueue = useExpireQueue({
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    position: position,
    expireQueue: expireQueue,
    hoverStop: hoverStop,
    fullScreen: fullScreen
  }, containerRef),
      items = _useExpireQueue.items,
      handleMouseEnter = _useExpireQueue.handleMouseEnter,
      handleMouseLeave = _useExpireQueue.handleMouseLeave;

  var poolTimeout = expireQueue.setting.timeout;
  return /*#__PURE__*/React.createElement(Portal, {
    container: container
  }, /*#__PURE__*/React.createElement(BaseNode, _extends({
    ref: handleRef,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave
  }, otherProps), /*#__PURE__*/React.createElement(TransitionGroup, null, items.map(function (item) {
    return /*#__PURE__*/React.createElement(Transition, {
      key: item.id,
      timeout: {
        enter: 10,
        exit: poolTimeout
      }
    }, function (state) {
      return children(state, item);
    });
  }))));
});
var NotificationList = styled(NotificationListWithoutStyle).withConfig({
  displayName: "NotificationList"
})(["", ""], function (props) {
  var theme = props.theme,
      position = props.position;
  var _theme$components$Not = theme.components.Notification,
      containerCenterTop = _theme$components$Not.containerCenterTop,
      containerTop = _theme$components$Not.containerTop,
      containerBottom = _theme$components$Not.containerBottom;
  var positionCss;

  if (position === 'top-center') {
    positionCss = css(["top:", "px;left:0;right:0;width:100%;flex-direction:column;"], containerCenterTop);
  } else if (position === 'top-left') {
    positionCss = css(["top:", "px;left:", "px;flex-direction:column;"], containerTop, containerTop);
  } else if (position === 'top-right') {
    positionCss = css(["top:", "px;right:", "px;flex-direction:column;"], containerTop, containerTop);
  } else if (position === 'bottom-left') {
    positionCss = css(["bottom:", "px;left:", "px;flex-direction:column-reverse;"], containerBottom, containerTop);
  } else if (position === 'bottom-right') {
    positionCss = css(["bottom:", "px;right:", "px;flex-direction:column-reverse;"], containerBottom, containerTop);
  }

  return css(["position:fixed;pointer-events:none;z-index:", ";", ""], theme.zIndex.pattern.toast, positionCss);
});
export default withThemeForStyled(NotificationList);