import { useAfterEffect as _useAfterEffect } from "@muya-ui/utils/es/hooks/useAfterEffect";
import { useEffectOnce as _useEffectOnce } from "@muya-ui/utils/es/hooks/useEffectOnce";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _includes from "lodash/includes";
import { useRef, useState } from 'react';
import forkHandler from '../utils/forkHandler';
export default function useNotificationList(props, containerRef) {
  var onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      position = props.position,
      expireQueue = props.expireQueue,
      hoverStop = props.hoverStop,
      fullScreen = props.fullScreen;
  var poolTimeout = expireQueue.setting.timeout;

  var _useState = useState(expireQueue.items),
      _useState2 = _slicedToArray(_useState, 2),
      items = _useState2[0],
      setItems = _useState2[1];

  var update = function update() {
    var newItems = expireQueue.items;
    setItems(newItems);

    if (newItems.length) {
      expireQueue.tick();
    }
  };

  _useEffectOnce(function () {
    expireQueue.on('update', update);
    update();
    return function () {
      expireQueue.removeListener('update', update);
    };
  });

  var lastMax = useRef(expireQueue.setting.max);

  _useAfterEffect(function () {
    if (containerRef.current && fullScreen) {
      var _containerRef$current = containerRef.current.getBoundingClientRect(),
          top = _containerRef$current.top,
          bottom = _containerRef$current.bottom;

      if (top < 0 || bottom > window.innerHeight) {
        if (_includes(position, 'top')) {
          expireQueue.shift();
        } else {
          expireQueue.pop();
        }

        lastMax.current = expireQueue.setting.max;
        expireQueue.reset({
          max: expireQueue.length
        });
      } else {
        expireQueue.reset({
          max: lastMax.current
        });
      }
    }
  }, poolTimeout);

  var handleMouseEnter = forkHandler(function () {
    if (hoverStop) {
      expireQueue.stop();
    }
  }, onMouseEnter);
  var handleMouseLeave = forkHandler(function () {
    if (hoverStop) {
      expireQueue.tick();
    }
  }, onMouseLeave);
  return {
    items: items,
    handleMouseEnter: handleMouseEnter,
    handleMouseLeave: handleMouseLeave,
    setItems: setItems
  };
}