import _extends from "@babel/runtime/helpers/extends";
import React from 'react';
import useLocale from '../Locale/useLocale';
import Select, { Option } from '../Select';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import PaginationPagerButton from './PaginationPagerButton';
import PaginationPagerEllipsis from './PaginationPagerEllipsis';
import { PaginationInput, StyledPageNumber, StyledPaginationJump, StyledPaginationJumpText, StyledPaginationText, StyledPaginationWrap, StyledSimpleActivePage, StyledSimplePageNum, StyledSimplePageSplit, StyledSimplePageText, StyledPaginationTotalText } from './styled';
import usePagination from './usePagination';
var defaultStyles = {
  wrapper: '',
  input: '',
  numberItem: '',
  simpleNumberItem: '',
  simpleArrow: '',
  jumpText: ''
};
var Pagination = memoForwardRef(function (props, ref) {
  var locale = useLocale();
  var theme = useTheme();
  var _theme$components$Pag = theme.components.Pagination,
      inputWidth = _theme$components$Pag.inputWidth,
      simpleSizeMap = _theme$components$Pag.simpleSizeMap;
  var itemRender = props.itemRender,
      _props$showTotal = props.showTotal,
      showTotal = _props$showTotal === void 0 ? false : _props$showTotal,
      _props$totalRecords = props.totalRecords,
      totalRecords = _props$totalRecords === void 0 ? 0 : _props$totalRecords,
      align = props.align,
      hideOnSinglePage = props.hideOnSinglePage;

  var _usePagination = usePagination(props),
      finalCurrent = _usePagination.finalCurrent,
      simpleQuickJumperValue = _usePagination.simpleQuickJumperValue,
      quickJumperValue = _usePagination.quickJumperValue,
      pageAmount = _usePagination.pageAmount,
      visiblePieces = _usePagination.visiblePieces,
      prevIsDisabled = _usePagination.prevIsDisabled,
      nextIsDisabled = _usePagination.nextIsDisabled,
      finalPageSize = _usePagination.finalPageSize,
      handleQuickJumperInputPressEnter = _usePagination.handleQuickJumperInputPressEnter,
      handleQuickJumperInputBlur = _usePagination.handleQuickJumperInputBlur,
      handleQuickJumperInputChange = _usePagination.handleQuickJumperInputChange,
      handleSimpleQuickJumperInputBlur = _usePagination.handleSimpleQuickJumperInputBlur,
      handleSimpleQuickJumperInputChange = _usePagination.handleSimpleQuickJumperInputChange,
      handleSimpleQuickJumperInputFocus = _usePagination.handleSimpleQuickJumperInputFocus,
      handlePrev = _usePagination.handlePrev,
      handleNext = _usePagination.handleNext,
      handlePageSizeChange = _usePagination.handlePageSizeChange,
      restProps = _usePagination.restProps,
      size = _usePagination.size,
      showQuickJumper = _usePagination.showQuickJumper,
      showPageSizeChanger = _usePagination.showPageSizeChanger,
      pageSizeOptions = _usePagination.pageSizeOptions,
      isDarkBackground = _usePagination.isDarkBackground,
      simple = _usePagination.simple,
      styles = _usePagination.styles;

  var innerStyles = useStyles('pagination', defaultStyles, styles);
  var simpleNode = React.useMemo(function () {
    if (!simple) {
      return;
    }

    var currentPageNode;

    if (showQuickJumper) {
      currentPageNode = /*#__PURE__*/React.createElement(PaginationInput, _extends({
        theme: theme,
        width: inputWidth[simpleSizeMap[size]],
        size: simpleSizeMap[size],
        value: simpleQuickJumperValue,
        onChange: handleSimpleQuickJumperInputChange,
        onPressEnter: handleQuickJumperInputPressEnter,
        onBlur: handleSimpleQuickJumperInputBlur,
        onFocus: handleSimpleQuickJumperInputFocus
      }, innerStyles.input));
    } else {
      currentPageNode = /*#__PURE__*/React.createElement(StyledSimpleActivePage, {
        theme: theme
      }, finalCurrent);
    }

    return /*#__PURE__*/React.createElement(StyledSimplePageNum, _extends({
      size: size,
      theme: theme
    }, innerStyles.simpleNumberItem), currentPageNode, /*#__PURE__*/React.createElement(StyledSimplePageSplit, null, "/"), /*#__PURE__*/React.createElement(StyledSimplePageText, null, pageAmount));
  }, [finalCurrent, handleQuickJumperInputPressEnter, handleSimpleQuickJumperInputBlur, handleSimpleQuickJumperInputChange, handleSimpleQuickJumperInputFocus, innerStyles.input, innerStyles.simpleNumberItem, inputWidth, pageAmount, showQuickJumper, simple, simpleQuickJumperValue, simpleSizeMap, size, theme]);
  var normalNode = React.useMemo(function () {
    if (simple) {
      return;
    }

    return visiblePieces.map(function (visiblePiece, index) {
      var key = "".concat(visiblePiece.type, "-").concat(index);

      if (visiblePiece.type === 'ellipsis') {
        return /*#__PURE__*/React.createElement(PaginationPagerEllipsis, _extends({
          onClick: visiblePiece.onClick,
          size: size,
          isDarkBackground: isDarkBackground
        }, innerStyles.numberItem, {
          type: visiblePiece["goto"],
          key: key,
          itemRender: itemRender,
          page: visiblePiece.page
        }));
      }

      var node = /*#__PURE__*/React.createElement(StyledPaginationText, null, visiblePiece.pageNumber);
      var children = itemRender ? itemRender(Number(visiblePiece.pageNumber), 'page', node) : node;
      return /*#__PURE__*/React.createElement(StyledPageNumber, _extends({
        key: key,
        onClick: visiblePiece.onClick,
        isActive: visiblePiece.pageNumber === finalCurrent,
        theme: theme,
        size: size,
        isDarkBackground: isDarkBackground
      }, innerStyles.numberItem), children);
    });
  }, [finalCurrent, innerStyles.numberItem, isDarkBackground, itemRender, simple, size, theme, visiblePieces]);
  var pagerStyleItem = simple ? innerStyles.simpleArrow : innerStyles.numberItem;

  var basePagerProps = _extends({
    simple: simple,
    size: size,
    isDarkBackground: isDarkBackground,
    itemRender: itemRender,
    finalCurrent: finalCurrent
  }, pagerStyleItem);

  var normalQuickJumperNode = React.useMemo(function () {
    if (simple || !showQuickJumper) {
      return;
    }

    return /*#__PURE__*/React.createElement(StyledPaginationJump, {
      theme: theme,
      size: size
    }, /*#__PURE__*/React.createElement(StyledPaginationJumpText, _extends({
      theme: theme,
      size: size
    }, innerStyles.jumpText), locale['Pagination.jumpTo']), /*#__PURE__*/React.createElement(PaginationInput, _extends({
      theme: theme,
      width: inputWidth[size],
      size: size,
      value: quickJumperValue,
      onChange: handleQuickJumperInputChange,
      onPressEnter: handleQuickJumperInputPressEnter,
      onBlur: handleQuickJumperInputBlur
    }, innerStyles.input)), /*#__PURE__*/React.createElement(StyledPaginationJumpText, _extends({
      theme: theme,
      size: size
    }, innerStyles.jumpText), locale['Pagination.page']));
  }, [handleQuickJumperInputBlur, handleQuickJumperInputChange, handleQuickJumperInputPressEnter, innerStyles.input, innerStyles.jumpText, inputWidth, locale, quickJumperValue, showQuickJumper, simple, size, theme]);
  var pageSizeNode = React.useMemo(function () {
    if (simple || !showPageSizeChanger) {
      return;
    }

    return /*#__PURE__*/React.createElement(StyledPaginationJump, {
      theme: theme,
      size: size
    }, /*#__PURE__*/React.createElement(Select, {
      value: finalPageSize,
      size: size,
      onChange: handlePageSizeChange,
      autoResize: true
    }, pageSizeOptions.map(function (item) {
      return /*#__PURE__*/React.createElement(Option, {
        value: item,
        key: item.toString()
      }, "".concat(item, " ").concat(locale['Pagination.pageSizeChangerSuffix']));
    })));
  }, [finalPageSize, handlePageSizeChange, locale, pageSizeOptions, showPageSizeChanger, simple, size, theme]);
  var paginationTotalNode = React.useMemo(function () {
    if (!showTotal) return null;
    var totalNode = typeof showTotal === 'function' ? showTotal(totalRecords) : locale['Pagination.totalText'].replace(/{{total}}/, String(totalRecords));
    return /*#__PURE__*/React.createElement(StyledPaginationTotalText, {
      theme: theme,
      size: size
    }, totalNode);
  }, [locale, showTotal, size, theme, totalRecords]); // 如果只有一页，且没有开启pageSize切换器

  if (pageAmount === 1 && hideOnSinglePage && !pageSizeNode) return null;
  return /*#__PURE__*/React.createElement(StyledPaginationWrap, _extends({
    ref: ref,
    theme: theme,
    $align: align
  }, innerStyles.wrapper, restProps), paginationTotalNode, /*#__PURE__*/React.createElement(PaginationPagerButton, _extends({
    onClick: handlePrev,
    type: "prev",
    disabled: prevIsDisabled
  }, basePagerProps)), simpleNode, normalNode, /*#__PURE__*/React.createElement(PaginationPagerButton, _extends({
    onClick: handleNext,
    type: "next",
    disabled: nextIsDisabled
  }, basePagerProps)), pageSizeNode, normalQuickJumperNode);
});
export default Pagination;