import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { ClearIcon as _ClearIcon } from "@muya-ui/theme-light/es/icons/ClearIcon";
import { ReminderIcon as _ReminderIcon } from "@muya-ui/theme-light/es/icons/ReminderIcon";
import { InformIcon as _InformIcon } from "@muya-ui/theme-light/es/icons/InformIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["defaultOpen", "open", "onVisibleChange", "title", "content", "actions", "style", "className", "onCancel", "onConfirm", "cancelText", "confirmText", "cancelButtonType", "confirmButtonType", "cancelButtonProps", "confirmButtonProps", "useInlineButton", "type", "icon"];
import React, { useCallback, useMemo, useState } from 'react';
import Button from '../Button';
import PopoverCard from '../PopoverCard';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import { StyledIconWrapper, StyledInlineButton, StyledTitleWrapper } from './styled';
export default memoForwardRef(function (props, ref) {
  var defaultOpen = props.defaultOpen,
      openProp = props.open,
      onVisibleChange = props.onVisibleChange,
      title = props.title,
      content = props.content,
      actions = props.actions,
      style = props.style,
      className = props.className,
      onCancel = props.onCancel,
      onConfirm = props.onConfirm,
      cancelText = props.cancelText,
      confirmText = props.confirmText,
      cancelButtonType = props.cancelButtonType,
      _props$confirmButtonT = props.confirmButtonType,
      confirmButtonTypeProp = _props$confirmButtonT === void 0 ? 'primary' : _props$confirmButtonT,
      cancelButtonProps = props.cancelButtonProps,
      confirmButtonProps = props.confirmButtonProps,
      useInlineButton = props.useInlineButton,
      type = props.type,
      icon = props.icon,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();

  var icons = _extends({
    info: _InformIcon,
    warning: _ReminderIcon,
    error: _ClearIcon
  }, theme.components.Popconfirm.icons);

  var _theme$components$Pop = theme.components.Popconfirm,
      paddingLeftWithType = _theme$components$Pop.paddingLeftWithType,
      width = _theme$components$Pop.width;

  var _useState = useState(defaultOpen),
      _useState2 = _slicedToArray(_useState, 2),
      innerOpen = _useState2[0],
      setInnerOpen = _useState2[1];

  var isControlled = ('open' in props);
  var open = isControlled ? openProp : innerOpen;
  var handleVisibleChange = useCallback(function (visible) {
    if (!isControlled) {
      setInnerOpen(visible);
    }

    if (onVisibleChange) {
      onVisibleChange(visible);
    }
  }, [isControlled, onVisibleChange]);
  var handleConfirm = useCallback(function (e) {
    // 非受控模式，点击确认关闭确认框
    if (!isControlled) {
      setInnerOpen(false);
    }

    if (onConfirm) {
      onConfirm(e);
    }
  }, [isControlled, onConfirm]);
  var handleCancel = useCallback(function (e) {
    // 非受控模式，点击确认关闭确认框
    if (!isControlled) {
      setInnerOpen(false);
    }

    if (onCancel) {
      onCancel(e);
    }
  }, [isControlled, onCancel]); // 确认按钮类型

  var confirmButtonType = useMemo(function () {
    if (type === 'error') {
      return 'danger';
    }

    return confirmButtonTypeProp;
  }, [confirmButtonTypeProp, type]); // 确认图标

  var innerIcon = useMemo(function () {
    if (icon) {
      return icon;
    }

    if (type) {
      var IconComp = icons[type];
      return /*#__PURE__*/React.createElement(IconComp, null);
    }
  }, [icon, icons, type]);
  var cancelButton = useMemo(function () {
    return useInlineButton ? /*#__PURE__*/React.createElement(StyledInlineButton, _extends({
      key: "cancel",
      size: "s",
      type: cancelButtonType,
      onClick: handleCancel
    }, cancelButtonProps), cancelText) : /*#__PURE__*/React.createElement(Button, _extends({
      key: "cancel",
      size: "s",
      plain: true,
      type: cancelButtonType,
      onClick: handleCancel
    }, cancelButtonProps), cancelText);
  }, [cancelButtonProps, cancelButtonType, cancelText, handleCancel, useInlineButton]);
  var confirmButton = useMemo(function () {
    return useInlineButton ? /*#__PURE__*/React.createElement(StyledInlineButton, _extends({
      key: "confirm",
      size: "s",
      onClick: handleConfirm,
      type: confirmButtonType
    }, confirmButtonProps), confirmText) : /*#__PURE__*/React.createElement(Button, _extends({
      key: "confirm",
      size: "s",
      onClick: handleConfirm,
      type: confirmButtonType
    }, confirmButtonProps), confirmText);
  }, [confirmButtonProps, confirmButtonType, confirmText, handleConfirm, useInlineButton]);
  var innerActions = useMemo(function () {
    return [cancelText ? cancelButton : null, confirmText ? confirmButton : null];
  }, [cancelButton, cancelText, confirmButton, confirmText]);
  var innerTitle = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledTitleWrapper, null, /*#__PURE__*/React.createElement(StyledIconWrapper, {
      theme: theme,
      $type: type
    }, innerIcon), title);
  }, [innerIcon, theme, title, type]);
  var innerStyle = useMemo(function () {
    return _extends({
      width: width,
      paddingLeft: type ? "".concat(paddingLeftWithType, "px") : undefined
    }, style);
  }, [paddingLeftWithType, style, type, width]);
  return /*#__PURE__*/React.createElement(PopoverCard, _extends({
    ref: ref,
    open: open,
    title: type ? innerTitle : title,
    content: content,
    actions: actions ? actions : innerActions,
    style: innerStyle,
    className: className,
    onVisibleChange: handleVisibleChange
  }, other));
});