import { setRef as _setRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["anchorEl", "children", "container", "disablePortal", "keepMounted", "lazyMount", "modifiers", "open", "placement", "popperOptions", "popperRef", "transition", "transitionProps", "zIndex"];
import PopperJS from 'popper.js';
import React, { useCallback, useEffect, useRef, useState } from 'react';
import styled, { css } from 'styled-components';
import Portal from '../Portal';
import useTheme from '../utils/useTheme';
export function flipPlacement(placement) {
  var direction = typeof window !== 'undefined' && document.body.getAttribute('dir') || 'ltr';

  if (direction !== 'rtl') {
    return placement;
  }

  switch (placement) {
    case 'bottom-end':
      return 'bottom-start';

    case 'bottom-start':
      return 'bottom-end';

    case 'top-end':
      return 'top-start';

    case 'top-start':
      return 'top-end';

    default:
      return placement;
  }
}

function getAnchorEl(anchorEl) {
  return typeof anchorEl === 'function' ? anchorEl() : anchorEl;
}

var useEnhancedEffect = typeof window !== 'undefined' ? React.useLayoutEffect : React.useEffect;
var defaultPopperOptions = {};
var StyledPopper = styled.div.withConfig({
  displayName: "StyledPopper"
})(["position:absolute;left:0;top:0;z-index:", ";", ""], function (props) {
  return props.$zIndex;
}, function (props) {
  return props.$shouldHide && css(["width:0px;height:0px;overflow:hidden;visibility:hidden;"]);
});
/**
 * Poppers rely on the 3rd party library [Popper.js](https://github.com/FezVrasta/popper.js) for positioning.
 */

var Popper = /*#__PURE__*/React.forwardRef(function Popper(props, ref) {
  var anchorEl = props.anchorEl,
      children = props.children,
      container = props.container,
      _props$disablePortal = props.disablePortal,
      disablePortal = _props$disablePortal === void 0 ? false : _props$disablePortal,
      _props$keepMounted = props.keepMounted,
      keepMounted = _props$keepMounted === void 0 ? true : _props$keepMounted,
      _props$lazyMount = props.lazyMount,
      lazyMount = _props$lazyMount === void 0 ? true : _props$lazyMount,
      modifiers = props.modifiers,
      open = props.open,
      _props$placement = props.placement,
      placementProps = _props$placement === void 0 ? 'bottom' : _props$placement,
      _props$popperOptions = props.popperOptions,
      popperOptions = _props$popperOptions === void 0 ? defaultPopperOptions : _props$popperOptions,
      popperRefProp = props.popperRef,
      _props$transition = props.transition,
      transition = _props$transition === void 0 ? false : _props$transition,
      transitionProps = props.transitionProps,
      zIndex = props.zIndex,
      other = _objectWithoutProperties(props, _excluded);

  var tooltipRef = useRef(null);

  var ownRef = _useForkRef(tooltipRef, ref);

  var popperRef = useRef(null);

  var handlePopperRef = _useForkRef(popperRef, popperRefProp);

  var handlePopperRefRef = useRef(null);
  var firstMountRef = useRef(false);
  var theme = useTheme();
  useEnhancedEffect(function () {
    handlePopperRefRef.current = handlePopperRef;
  }, [handlePopperRef]);
  React.useImperativeHandle(popperRefProp, function () {
    return popperRef.current;
  }, []);

  var _useState = useState(!open),
      _useState2 = _slicedToArray(_useState, 2),
      exited = _useState2[0],
      setExited = _useState2[1];

  var _useState3 = useState(),
      _useState4 = _slicedToArray(_useState3, 2),
      placement = _useState4[0],
      setPlacement = _useState4[1];

  var handlePopperUpdate = useCallback(function (data) {
    if (data.placement !== placement) {
      setPlacement(data.placement);
    }
  }, [placement]);
  var handleOpen = useCallback(function () {
    var popperNode = tooltipRef.current;

    if (!popperNode || !anchorEl || !open) {
      return;
    }

    if (popperRef.current) {
      popperRef.current.destroy();

      if (handlePopperRefRef.current) {
        handlePopperRefRef.current(null);
      }
    }

    var popperReferenceElement = getAnchorEl(anchorEl);

    if (popperReferenceElement) {
      var popper = new PopperJS(popperReferenceElement, popperNode, _extends({
        placement: flipPlacement(placementProps),
        positionFixed: true
      }, popperOptions, {
        modifiers: _extends({
          preventOverflow: {
            // boundariesElement: disablePortal ? 'scrollParent' : 'window',
            enabled: false
          },
          hide: {
            enabled: false
          }
        }, modifiers, popperOptions.modifiers),
        // We could have been using a custom modifier like react-popper is doing.
        // But it seems this is the best public API for this use case.
        onCreate: function onCreate(data) {
          handlePopperUpdate(data);
          popperOptions.onCreate && popperOptions.onCreate(data);
        },
        onUpdate: function onUpdate(data) {
          handlePopperUpdate(data);
          popperOptions.onUpdate && popperOptions.onUpdate(data);
        }
      }));

      if (handlePopperRefRef.current) {
        handlePopperRefRef.current(popper);
      }
    }
  }, [anchorEl, handlePopperUpdate, modifiers, open, placementProps, popperOptions]);
  var handleRef = useCallback(function (node) {
    _setRef(ownRef, node);

    handleOpen();
  }, [ownRef, handleOpen]);

  var handleEnter = function handleEnter(node, isAppearing) {
    setExited(false);

    if (transitionProps && transitionProps.onEnter) {
      transitionProps.onEnter(node, isAppearing);
    }
  };

  var handleClose = function handleClose() {
    if (!popperRef.current) {
      return;
    }

    popperRef.current.destroy();

    if (handlePopperRefRef.current) {
      handlePopperRefRef.current(null);
    }
  };

  var handleExited = function handleExited(node) {
    setExited(true);
    handleClose();

    if (transitionProps && transitionProps.onExited) {
      transitionProps.onExited(node);
    }
  };

  useEffect(function () {
    if (popperRef.current) {
      popperRef.current.update();
    }
  });
  useEffect(function () {
    // Let's update the popper position.
    handleOpen();
  }, [handleOpen]);
  useEffect(function () {
    return function () {
      handleClose();
    }; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  useEffect(function () {
    if (!open && !transition) {
      // Otherwise handleExited will call this.
      handleClose();
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [open, transition]);
  /**
   * Popper 关闭的情况下有两种情况是不需要渲染的：
   * 1. 第一次渲染 Popper 且开启了 lazyMount 的情况下，用于预防提前渲染无用的节点
   * 2. keepMounted 为 false，且未开启动画或者已经退出的情况下
   * 需要保持内部组件状态的话需要将 keepMounted 设为 true
   */

  if (!open && (lazyMount && !firstMountRef.current || !keepMounted && (!transition || exited))) {
    return null;
  }

  var childProps = {
    placement: placement || flipPlacement(placementProps)
  };

  if (transition) {
    childProps.transitionProps = _extends({}, transitionProps, {
      "in": open,
      onEnter: handleEnter,
      onExited: handleExited
    });
  } // 如果是第一次渲染菜单，那么将 ref 设为 true


  if (!firstMountRef.current) {
    firstMountRef.current = true;
  }

  var shouldHide = transition ? exited : !open;
  var finalZIndex = zIndex || zIndex === 0 ? zIndex : theme.zIndex.pattern.popper;
  return /*#__PURE__*/React.createElement(Portal, {
    disablePortal: disablePortal,
    container: container
  }, /*#__PURE__*/React.createElement(StyledPopper, _extends({
    ref: handleRef,
    role: "tooltip",
    $zIndex: finalZIndex,
    $shouldHide: shouldHide
  }, other), typeof children === 'function' ? children(childProps) : children));
});
export default Popper;