import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import React from 'react';
import ReactDOM from 'react-dom';
import memoForwardRef from '../utils/memoForwardRef';
import { getPortalContainer } from './utils';
var useEnhancedEffect = typeof window !== 'undefined' ? React.useLayoutEffect : React.useEffect;
/**
 * Portals provide a first-class way to render children into a DOM node
 * that exists outside the DOM hierarchy of the parent component.
 */

var Portal = memoForwardRef(function Portal(props, ref) {
  var children = props.children,
      container = props.container,
      _props$disablePortal = props.disablePortal,
      disablePortal = _props$disablePortal === void 0 ? false : _props$disablePortal,
      onRendered = props.onRendered;

  var _React$useState = React.useState(null),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      mountNode = _React$useState2[0],
      setMountNode = _React$useState2[1];

  var childRef = React.useRef(null);

  var handleRef = _useForkRef(children.ref, childRef);

  useEnhancedEffect(function () {
    if (!disablePortal) {
      setMountNode(getPortalContainer(container));
    }
  }, [container, disablePortal]);
  React.useImperativeHandle(ref, function () {
    return mountNode || childRef.current;
  }, [mountNode]);
  useEnhancedEffect(function () {
    if (onRendered && mountNode) {
      onRendered();
    }
  }, [mountNode, onRendered]);

  if (disablePortal) {
    React.Children.only(children);
    return /*#__PURE__*/React.cloneElement(children, {
      ref: handleRef
    });
  }

  return mountNode ? /*#__PURE__*/ReactDOM.createPortal(children, mountNode) : mountNode;
});
export default Portal;