import { SvgIcon as _SvgIcon } from "@muya-ui/icon-base/es/SvgIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["textStyle", "textColor", "progressColor", "bgColor", "percentSpacing", "showPercentage", "showPercentageSymbol", "animate", "animationDuration", "fontSize", "style", "className", "type", "circleLineWidth", "roundedStroke"];
import React, { useMemo } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import { StyledProgressBg, StyledProgressLine } from './styled';
export default memoForwardRef(function (props, ref) {
  var _props$status = props.status,
      status = _props$status === void 0 ? 'normal' : _props$status,
      _props$progress = props.progress,
      progress = _props$progress === void 0 ? 0 : _props$progress;
  var theme = useTheme();
  var transitionPattern = theme.transition.pattern,
      colorsPattern = theme.colors.pattern,
      token = theme.components.Progress;
  var defaultProgressColor = colorsPattern.feature.info;
  var defaultTextColor = colorsPattern.text.assistant;

  if (status === 'error') {
    defaultProgressColor = colorsPattern.feature.error;
  } else if (status === 'success' || progress >= 100) {
    defaultTextColor = colorsPattern.feature.success;
    defaultProgressColor = colorsPattern.feature.success;
  }

  var textStyle = props.textStyle,
      _props$textColor = props.textColor,
      textColor = _props$textColor === void 0 ? defaultTextColor : _props$textColor,
      _props$progressColor = props.progressColor,
      progressColor = _props$progressColor === void 0 ? defaultProgressColor : _props$progressColor,
      _props$bgColor = props.bgColor,
      bgColor = _props$bgColor === void 0 ? token.defaultBgColor : _props$bgColor,
      _props$percentSpacing = props.percentSpacing,
      percentSpacing = _props$percentSpacing === void 0 ? 0 : _props$percentSpacing,
      _props$showPercentage = props.showPercentage,
      showPercentage = _props$showPercentage === void 0 ? true : _props$showPercentage,
      _props$showPercentage2 = props.showPercentageSymbol,
      showPercentageSymbol = _props$showPercentage2 === void 0 ? true : _props$showPercentage2,
      _props$animate = props.animate,
      animate = _props$animate === void 0 ? true : _props$animate,
      _props$animationDurat = props.animationDuration,
      animationDuration = _props$animationDurat === void 0 ? transitionPattern.duration.status : _props$animationDurat,
      _props$fontSize = props.fontSize,
      fontSize = _props$fontSize === void 0 ? token.defaultFontSize : _props$fontSize,
      style = props.style,
      className = props.className,
      _props$type = props.type,
      type = _props$type === void 0 ? 'line' : _props$type,
      _props$circleLineWidt = props.circleLineWidth,
      circleWidthPercent = _props$circleLineWidt === void 0 ? 12.5 : _props$circleLineWidt,
      _props$roundedStroke = props.roundedStroke,
      roundedStroke = _props$roundedStroke === void 0 ? true : _props$roundedStroke,
      other = _objectWithoutProperties(props, _excluded);

  var outerRadius = 200;
  var circleLineWidth = circleWidthPercent * outerRadius / 100;
  var radius = outerRadius - circleLineWidth;
  var diameter = Math.round(Math.PI * radius * 2); // 内部圆直径

  var strokeDashoffset = Math.round((100 - Math.min(progress, 100)) / 100 * diameter);
  var transition = animate ? "".concat(type === 'line' ? 'width' : 'stroke-dashoffset', " ").concat(animationDuration, "ms ").concat(transitionPattern.easing.status) : undefined;
  var viewBox = "-".concat(circleLineWidth, " -").concat(circleLineWidth, " ").concat(outerRadius * 2, " ").concat(outerRadius * 2);
  var textElement = useMemo(function () {
    return /*#__PURE__*/React.createElement("text", {
      style: textStyle,
      fill: textColor,
      x: radius,
      y: radius,
      fontSize: radius / 2,
      textAnchor: "middle",
      dominantBaseline: "central"
    }, /*#__PURE__*/React.createElement("tspan", {
      textAnchor: "middle",
      dominantBaseline: "central"
    }, progress), showPercentageSymbol && /*#__PURE__*/React.createElement("tspan", {
      textAnchor: "middle",
      dominantBaseline: "central",
      dx: percentSpacing
    }, "%"));
  }, [percentSpacing, progress, radius, showPercentageSymbol, textColor, textStyle]);

  if (type === 'line') {
    return /*#__PURE__*/React.createElement(StyledProgressLine, {
      theme: theme,
      $bgColor: bgColor,
      style: style,
      className: className
    }, /*#__PURE__*/React.createElement(StyledProgressBg, {
      style: {
        width: "".concat(progress, "%"),
        background: progressColor,
        transition: transition
      }
    }));
  }

  return /*#__PURE__*/React.createElement(_SvgIcon, _extends({
    ref: ref,
    style: style,
    className: className,
    fontSize: fontSize,
    viewBox: viewBox
  }, other), /*#__PURE__*/React.createElement("circle", {
    stroke: bgColor,
    cx: radius,
    cy: radius,
    r: radius,
    strokeWidth: circleLineWidth,
    fill: "none"
  }), /*#__PURE__*/React.createElement("circle", {
    stroke: progressColor,
    transform: "rotate(-90 ".concat(radius, " ").concat(radius, ")"),
    cx: radius,
    cy: radius,
    r: radius,
    strokeDasharray: diameter,
    strokeDashoffset: diameter,
    strokeWidth: circleLineWidth,
    strokeLinecap: roundedStroke ? 'round' : 'butt',
    fill: "none",
    style: {
      strokeDashoffset: strokeDashoffset,
      transition: transition
    }
  }), showPercentage ? textElement : /*#__PURE__*/React.createElement(React.Fragment, null));
});