import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["value", "disabled", "onChange", "children", "ellipsis", "width"];
import React, { useMemo } from 'react';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import Radio from './Radio';
import { StyledRadioGroup } from './styled';
import useRadioGroup from './useRadioGroup';
var RadioGroup = memoForwardRef(function (props, ref) {
  var _useRadioGroup = useRadioGroup(props),
      opts = _useRadioGroup.opts,
      valueState = _useRadioGroup.valueState,
      handleChange = _useRadioGroup.handleChange,
      handleClick = _useRadioGroup.handleClick,
      children = _useRadioGroup.children,
      renderRadio = _useRadioGroup.renderRadio,
      radioEllipsis = _useRadioGroup.radioEllipsis,
      radioWidth = _useRadioGroup.radioWidth,
      restProps = _useRadioGroup.restProps;

  var theme = useTheme();
  var radioList = useMemo(function () {
    if (opts && opts.length) {
      return opts.map(function (option, optIndex) {
        var label = option.label,
            value = option.value,
            disabled = option.disabled,
            ellipsis = option.ellipsis,
            width = option.width;
        var radioNode = /*#__PURE__*/React.createElement(Radio, {
          key: String(value),
          value: value,
          size: props.size,
          styles: props.styles,
          ellipsis: ellipsis === undefined ? radioEllipsis : ellipsis,
          width: width || radioWidth,
          disabled: props.disabled || disabled,
          checked: valueState === value,
          onChange: handleChange.bind(null, option),
          onClick: handleClick.bind(null, option)
        }, label);
        return renderRadio(radioNode, option, optIndex);
      });
    }

    return React.Children.map(children, function (child, childIndex) {
      if (! /*#__PURE__*/React.isValidElement(child)) {
        return null;
      }

      var _child$props = child.props,
          value = _child$props.value,
          disabled = _child$props.disabled,
          onChange = _child$props.onChange,
          label = _child$props.children,
          ellipsis = _child$props.ellipsis,
          width = _child$props.width,
          otherChildProps = _objectWithoutProperties(_child$props, _excluded);

      var option = {
        value: value,
        disabled: disabled,
        onChange: onChange,
        label: label
      };
      var radioNode = /*#__PURE__*/React.cloneElement(child, _extends({}, otherChildProps, {
        children: label,
        value: value,
        ellipsis: ellipsis === undefined ? radioEllipsis : ellipsis,
        width: width || radioWidth,
        checked: valueState === value,
        disabled: props.disabled || disabled,
        styles: props.styles,
        size: props.size,
        onChange: handleChange.bind(null, option),
        onClick: handleClick.bind(null, option)
      }));
      return renderRadio(radioNode, option, childIndex);
    });
  }, [opts, children, props.size, props.styles, props.disabled, radioEllipsis, radioWidth, valueState, handleChange, handleClick, renderRadio]);
  return /*#__PURE__*/React.createElement(StyledRadioGroup, _extends({
    ref: ref,
    theme: theme
  }, restProps), radioList);
});
RadioGroup.useRadioGroup = useRadioGroup;
export default RadioGroup;