import styled, { css } from 'styled-components';
import InlineButton from '../InlineButton';
import { ellipsisWithWidth } from '../styled/mixins/ellipsis';
export var StyledRadioInput = styled.input.withConfig({
  displayName: "StyledRadioInput"
})(["box-sizing:border-box;touch-action:manipulation;position:absolute;z-index:1;cursor:pointer;opacity:0;left:0;top:0;bottom:0;right:0;"]);
export var StyledRadioIcon = styled.span.withConfig({
  displayName: "StyledRadioIcon"
})(["position:relative;display:inline-block;line-height:1;white-space:nowrap;vertical-align:middle;outline:none;cursor:pointer;"]);

var radioInnerCSS = function radioInnerCSS(props) {
  var _props$theme = props.theme,
      token = _props$theme.components.Radio,
      _props$theme$transiti = _props$theme.transition.spec,
      duration = _props$theme$transiti.duration,
      easing = _props$theme$transiti.easing,
      $size = props.$size,
      $checked = props.$checked;
  var iconSize = token.iconSize[$size];
  var checkedCenterSize = token.checkedCenterSize[$size];
  return css(["transition:all ", "ms ", ";width:", "px;height:", "px;border-width:1px;border-color:", ";background-color:", ";&::after{content:'';width:", "px;height:", "px;background:", ";transform:", ";position:absolute;border-radius:50%;top:", "px;left:", "px;transition:all ", "ms ", ";}"], duration.normal, easing.easeInOut, iconSize, iconSize, $checked ? token.borderColor.checked : token.borderColor.unChecked, $checked ? token.iconColor.checked : token.iconColor.unChecked, checkedCenterSize, checkedCenterSize, token.checkedCenterBg, $checked ? 'scale(1)' : 'scale(0)', (iconSize - checkedCenterSize) / 2, (iconSize - checkedCenterSize) / 2, duration.normal, easing.easeInOut);
};

export var StyledRadioInner = styled.span.withConfig({
  displayName: "StyledRadioInner"
})(["display:block;box-sizing:border-box;border-radius:50%;border-style:solid;", ""], radioInnerCSS);

var textSizeCSS = function textSizeCSS(props) {
  var _props$theme2 = props.theme,
      colors = _props$theme2.colors,
      _props$theme2$typogra = _props$theme2.typography.spec,
      fontSize = _props$theme2$typogra.fontSize,
      lineHeight = _props$theme2$typogra.lineHeight,
      textIcon = _props$theme2.spacing.pattern.textIcon,
      token = _props$theme2.components.Radio,
      $size = props.$size;
  var fontLevel = token.fontLevel[$size];
  return css(["color:", ";font-size:", "px;line-height:", "px;padding-left:", "px;padding-right:", "px;"], colors.pattern.text.title, fontSize[fontLevel], lineHeight[fontLevel], textIcon[fontLevel], textIcon[fontLevel]);
};

export var StyledRadioText = styled.span.withConfig({
  displayName: "StyledRadioText"
})(["vertical-align:middle;", ""], textSizeCSS); // 状态的样式

var radioStatusCSS = function radioStatusCSS(props) {
  var _props$theme3 = props.theme,
      opacity = _props$theme3.opacity,
      token = _props$theme3.components.Radio,
      $disabled = props.$disabled,
      $checked = props.$checked,
      $width = props.$width,
      $ellipsis = props.$ellipsis;
  var interactive = !$disabled;
  var interactiveCSS;

  if (!interactive) {
    interactiveCSS = css(["opacity:", ";cursor:not-allowed;", ",", "{cursor:not-allowed;}"], opacity.pattern.disabled, StyledRadioInput, StyledRadioIcon);
  } else {
    interactiveCSS = css(["&:focus,&:hover{", "{border-color:", ";background-color:", ";}}&:active{", "{border-color:", ";background-color:", ";}}"], StyledRadioInner, $checked ? token.borderColor.checkedHover : token.borderColor.hover, $checked ? token.iconColor.checkedHover : token.iconColor.hover, StyledRadioInner, $checked ? token.borderColor.checkedClick : token.borderColor.click, $checked ? token.borderColor.checkedClick : token.iconColor.click);
  }

  return css(["", " ", ""], ellipsisWithWidth($ellipsis, $width), interactiveCSS);
};

export var StyledRadioLabel = styled.label.withConfig({
  displayName: "StyledRadioLabel"
})(["box-sizing:border-box;user-select:none;display:inline-block;white-space:nowrap;cursor:pointer;", ""], radioStatusCSS);
export var StyledRadioGroup = styled.div.withConfig({
  displayName: "StyledRadioGroup"
})(["display:inline-block;", "{margin-right:", "px;}"], StyledRadioLabel, function (props) {
  return props.theme.components.Radio.marginRightInGroup;
});
export var StyledInlineButton = styled(InlineButton).withConfig({
  displayName: "StyledInlineButton"
})(["", ""], function (props) {
  var _props$theme4 = props.theme,
      opacity = _props$theme4.opacity,
      token = _props$theme4.components.Radio,
      disabled = props.disabled,
      $checked = props.$checked;
  var interactive = !disabled;
  var interactiveCSS;

  if (!interactive) {
    interactiveCSS = css(["opacity:", ";cursor:not-allowed;", ",", "{cursor:not-allowed;}"], opacity.pattern.disabled, StyledRadioInput, StyledRadioIcon);
  } else {
    interactiveCSS = css(["color:", ";&:focus,&:hover{svg{color:", ";}}&:active{svg{color:", ";}}"], $checked ? token.iconColor.checked : token.borderColor.unChecked, $checked ? token.iconColor.checkedHover : token.borderColor.hover, $checked ? token.iconColor.checkedClick : token.borderColor.click);
  }

  return css(["", ""], interactiveCSS);
});