import _extends from "@babel/runtime/helpers/extends";
import { ClockIcon as _ClockIcon } from "@muya-ui/theme-light/es/icons/ClockIcon";
import React, { useMemo } from 'react';
import { StyledPopPanel } from '../styled/components/PopPanel';
import Trigger from '../Trigger';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import RangeDateTimeCalendar from './RangeDateTimeCalendar';
import { StyledInput, StyledMiddleNode, StyledRangeInput } from './styled';
import useRangeDateTimePicker from './useRangeDateTimePicker';
import { StyledSuffixIcon } from '../styled/components/SuffixIcon';
import { useLocale } from '../Locale';
var defaultStyles = {
  popPanel: ''
};
var RangeDateTimePicker = memoForwardRef(function (props, ref) {
  var theme = useTheme();
  var locale = useLocale();

  var _useRangeDateTimePick = useRangeDateTimePicker(props),
      focusState = _useRangeDateTimePick.focusState,
      finalUIState = _useRangeDateTimePick.finalUIState,
      calendarDate = _useRangeDateTimePick.calendarDate,
      finalValue = _useRangeDateTimePick.finalValue,
      otherProps = _useRangeDateTimePick.otherProps,
      finalTimeFormat = _useRangeDateTimePick.finalTimeFormat,
      handleStartFocus = _useRangeDateTimePick.handleStartFocus,
      handleEndFocus = _useRangeDateTimePick.handleEndFocus,
      handleCalendarChange = _useRangeDateTimePick.handleCalendarChange,
      handleClickAway = _useRangeDateTimePick.handleClickAway,
      handleStartClear = _useRangeDateTimePick.handleStartClear,
      handleEndClear = _useRangeDateTimePick.handleEndClear,
      handleConfirm = _useRangeDateTimePick.handleConfirm,
      handleCalendarUIChange = _useRangeDateTimePick.handleCalendarUIChange,
      styles = _useRangeDateTimePick.styles,
      disabled = _useRangeDateTimePick.disabled,
      readOnly = _useRangeDateTimePick.readOnly,
      allowClear = _useRangeDateTimePick.allowClear,
      calendarStyles = _useRangeDateTimePick.calendarStyles,
      disableTimeAfter = _useRangeDateTimePick.disableTimeAfter,
      disableTimeBefore = _useRangeDateTimePick.disableTimeBefore,
      placeholder = _useRangeDateTimePick.placeholder,
      defaultTime = _useRangeDateTimePick.defaultTime,
      container = _useRangeDateTimePick.container,
      inputReadOnly = _useRangeDateTimePick.inputReadOnly,
      todayDate = _useRangeDateTimePick.todayDate,
      disableDate = _useRangeDateTimePick.disableDate,
      options = _useRangeDateTimePick.options,
      hideItemOutside = _useRangeDateTimePick.hideItemOutside,
      renderDecadeItem = _useRangeDateTimePick.renderDecadeItem,
      renderMonthItem = _useRangeDateTimePick.renderMonthItem,
      renderYearItem = _useRangeDateTimePick.renderYearItem,
      renderMonthLabel = _useRangeDateTimePick.renderMonthLabel,
      timeRowNum = _useRangeDateTimePick.timeRowNum,
      hourStep = _useRangeDateTimePick.hourStep,
      minuteStep = _useRangeDateTimePick.minuteStep,
      secondStep = _useRangeDateTimePick.secondStep,
      disabledHours = _useRangeDateTimePick.disabledHours,
      disabledMinutes = _useRangeDateTimePick.disabledMinutes,
      disabledSeconds = _useRangeDateTimePick.disabledSeconds,
      hideDisabledOptions = _useRangeDateTimePick.hideDisabledOptions,
      placement = _useRangeDateTimePick.placement,
      popperProps = _useRangeDateTimePick.popperProps,
      arrowPointAtCenter = _useRangeDateTimePick.arrowPointAtCenter,
      triggerId = _useRangeDateTimePick.triggerId,
      flip = _useRangeDateTimePick.flip; // input 的 styles 直接传个 input


  var inputStyles = useMemo(function () {
    if (!styles) {
      return;
    }

    var inputWrapper = styles.inputWrapper,
        input = styles.input,
        nodeDivider = styles.nodeDivider,
        nodeWrapper = styles.nodeWrapper,
        clearWrapper = styles.clearWrapper,
        statusWrapper = styles.statusWrapper;
    return {
      inputWrapper: inputWrapper,
      input: input,
      nodeDivider: nodeDivider,
      nodeWrapper: nodeWrapper,
      clearWrapper: clearWrapper,
      statusWrapper: statusWrapper
    };
  }, [styles]);
  var currentStyles = useMemo(function () {
    if (!styles) {
      return;
    }

    var popPanel = styles.popPanel;
    return {
      popPanel: popPanel
    };
  }, [styles]);
  var innerStyles = useStyles('datetime-picker', defaultStyles, currentStyles);
  var InputIcon = theme.components.TimePicker.inputIcon || _ClockIcon;
  var innerSuffixNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledSuffixIcon, {
      theme: theme
    }, /*#__PURE__*/React.createElement(InputIcon, null));
  }, [theme]);
  var realNode = /*#__PURE__*/React.createElement(StyledRangeInput, {
    ref: ref
  }, /*#__PURE__*/React.createElement(StyledInput, _extends({
    $focus: focusState === 'start',
    value: finalValue[0],
    suffixNode: innerSuffixNode,
    clearReplace: true,
    focusWhenClear: false,
    theme: theme,
    styles: inputStyles,
    disabled: disabled,
    allowClear: allowClear,
    onFocus: handleStartFocus,
    onClear: handleStartClear,
    readOnly: readOnly || inputReadOnly,
    placeholder: placeholder[0]
  }, otherProps, {
    hasFeedback: false
  })), /*#__PURE__*/React.createElement(StyledMiddleNode, {
    theme: theme,
    $size: otherProps.size
  }, locale['RangeDateTimePicker.middleText']), /*#__PURE__*/React.createElement(StyledInput, _extends({
    $focus: focusState === 'end',
    value: finalValue[1],
    suffixNode: innerSuffixNode,
    clearReplace: true,
    focusWhenClear: false,
    theme: theme,
    styles: inputStyles,
    disabled: disabled,
    allowClear: allowClear,
    onFocus: handleEndFocus,
    onClear: handleEndClear,
    readOnly: readOnly || inputReadOnly,
    placeholder: placeholder[1]
  }, otherProps, {
    hasFeedback: false
  })));

  if (disabled || readOnly) {
    return realNode;
  }

  var calendarProps = {
    timeRowNum: timeRowNum,
    styles: calendarStyles,
    options: options,
    todayDate: todayDate,
    allowClear: allowClear,
    disableDate: disableDate,
    timeFormat: finalTimeFormat,
    defaultTime: defaultTime,
    disableTimeAfter: disableTimeAfter,
    disableTimeBefore: disableTimeBefore,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // ---- timepicker panel props
    hourStep: hourStep,
    minuteStep: minuteStep,
    secondStep: secondStep,
    disabledHours: disabledHours,
    disabledMinutes: disabledMinutes,
    disabledSeconds: disabledSeconds,
    hideDisabledOptions: hideDisabledOptions
  };
  var calendarNode = /*#__PURE__*/React.createElement(StyledPopPanel, _extends({}, innerStyles.popPanel, {
    theme: theme
  }), /*#__PURE__*/React.createElement(RangeDateTimeCalendar, _extends({
    viewDate: finalUIState.viewDate,
    viewType: finalUIState.viewType,
    range: calendarDate.range,
    start: calendarDate.start,
    end: calendarDate.end,
    onChange: handleCalendarChange,
    onConfirm: handleConfirm,
    onUIChange: handleCalendarUIChange,
    status: focusState
  }, calendarProps)));
  var triggerProps = {
    placement: placement,
    popperProps: popperProps,
    arrowPointAtCenter: arrowPointAtCenter,
    triggerId: triggerId,
    flip: flip,
    hideArrow: true,
    container: container
  };
  return /*#__PURE__*/React.createElement(Trigger, _extends({
    popup: calendarNode,
    triggerAction: "click",
    open: finalUIState.popupOpen,
    onClickAway: handleClickAway
  }, triggerProps), realNode);
});
RangeDateTimePicker.__MUYA_RANGEDATETIMEPICKER = true;
export default RangeDateTimePicker;