import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import _defineProperty from "@babel/runtime/helpers/defineProperty";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["allowClear", "todayDate", "viewType", "viewDate", "disableRange", "disableDate", "options", "onUIChange", "defaultViewDate", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "timeRowNum", "hourStep", "minuteStep", "secondStep", "disabledHours", "disabledMinutes", "disabledSeconds", "hideDisabledOptions", "range", "start", "end", "disableTimeAfter", "disableTimeBefore", "defaultDate", "timeFormat", "defaultTime", "onChange", "onClear", "onSelect", "onConfirm", "styles", "status"];
import dayjs from 'dayjs';
import { useEffect, useMemo, useState } from 'react';
import useCalendar from '../Calendar/useCalendar';
import { formatCalendarOptions } from '../Calendar/utils';
import { transformDateByTime } from '../DateTimePicker/useDateTimeCalendar';
import mergeBooleanFns from '../utils/mergeBooleanFns';

/**
 * 根据禁用之前之后的时间来自定义时间的禁用
 * @param disableTimeBefore
 * @param disableTimeAfter
 */
export function getDisableDateFn(disableTimeBefore, disableTimeAfter) {
  if (!disableTimeAfter && !disableTimeBefore) {
    return;
  }

  var disableTimeAfterFn;
  var disableTimeBeforeFn;

  if (disableTimeAfter) {
    var time = dayjs(disableTimeAfter); // 注意：一定是开区间的

    disableTimeAfterFn = function disableTimeAfterFn(date) {
      return date.isAfter(time) && !date.isSame(time, 'd');
    };
  }

  if (disableTimeBefore) {
    var _time = dayjs(disableTimeBefore); // 注意：一定是开区间的


    disableTimeBeforeFn = function disableTimeBeforeFn(date) {
      return date.isBefore(_time) && !date.isSame(_time, 'd');
    };
  }

  var disableDateByTime = function disableDateByTime(date) {
    if (disableTimeAfterFn && disableTimeAfterFn(date)) {
      return true;
    }

    if (disableTimeBeforeFn && disableTimeBeforeFn(date)) {
      return true;
    }

    return false;
  };

  return disableDateByTime;
}
export default function useDateTimeCalendar(props) {
  var _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      todayDate = props.todayDate,
      viewType = props.viewType,
      viewDate = props.viewDate,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      onUIChange = props.onUIChange,
      defaultViewDate = props.defaultViewDate,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      timeRowNum = props.timeRowNum,
      hourStep = props.hourStep,
      minuteStep = props.minuteStep,
      secondStep = props.secondStep,
      disabledHours = props.disabledHours,
      disabledMinutes = props.disabledMinutes,
      disabledSeconds = props.disabledSeconds,
      hideDisabledOptions = props.hideDisabledOptions,
      rangeProps = props.range,
      startProps = props.start,
      endProps = props.end,
      disableTimeAfterProps = props.disableTimeAfter,
      disableTimeBeforeProps = props.disableTimeBefore,
      defaultDate = props.defaultDate,
      _props$timeFormat = props.timeFormat,
      timeFormat = _props$timeFormat === void 0 ? 'HH:mm:ss' : _props$timeFormat,
      _props$defaultTime = props.defaultTime,
      defaultTime = _props$defaultTime === void 0 ? 'now' : _props$defaultTime,
      onChange = props.onChange,
      onClear = props.onClear,
      onSelect = props.onSelect,
      onConfirm = props.onConfirm,
      styles = props.styles,
      _props$status = props.status,
      status = _props$status === void 0 ? 'start' : _props$status,
      otherProps = _objectWithoutProperties(props, _excluded);

  var isRangeControlled = ('range' in props);
  var isStartControlled = ('start' in props);
  var isEndControlled = ('end' in props);
  var isControlled = isRangeControlled || isStartControlled || isEndControlled;
  var valueFromProps = useMemo(function () {
    var range;
    var start;
    var end;

    if (defaultDate) {
      var innerDate = defaultDate.map(function (date) {
        return dayjs(date);
      });
      start = innerDate[0];
      end = innerDate[1];

      if (defaultDate[0] && defaultDate[1]) {
        range = innerDate;
      }
    }

    range = isRangeControlled ? rangeProps : range;
    start = isStartControlled ? startProps : start;
    end = isEndControlled ? endProps : end;
    return {
      range: range,
      start: start,
      end: end
    };
  }, [defaultDate, endProps, isEndControlled, isRangeControlled, isStartControlled, rangeProps, startProps]);

  var _useState = useState(valueFromProps),
      _useState2 = _slicedToArray(_useState, 2),
      stateValue = _useState2[0],
      setStateValue = _useState2[1];

  var _useState3 = useState(disableTimeBeforeProps),
      _useState4 = _slicedToArray(_useState3, 2),
      disableTimeBefore = _useState4[0],
      setDisableTimeBefore = _useState4[1];

  var _useState5 = useState(disableTimeAfterProps),
      _useState6 = _slicedToArray(_useState5, 2),
      disableTimeAfter = _useState6[0],
      setDisableTimeAfter = _useState6[1];

  var finalValue = isControlled ? valueFromProps : stateValue;
  var memoDisableTimeBefore = useMemo(function () {
    if (disableTimeBefore) {
      var innerDate = dayjs(disableTimeBefore);
      return innerDate.isValid() ? innerDate : undefined;
    }
  }, [disableTimeBefore]);
  var memoDisableTimeAfter = useMemo(function () {
    if (disableTimeAfter) {
      var innerDate = dayjs(disableTimeAfter);
      return innerDate.isValid() ? innerDate : undefined;
    }
  }, [disableTimeAfter]);
  var disableTime = useMemo(function () {
    return getDisableDateFn(memoDisableTimeBefore, memoDisableTimeAfter);
  }, [memoDisableTimeAfter, memoDisableTimeBefore]);

  var _useState7 = useState(status),
      _useState8 = _slicedToArray(_useState7, 2),
      selectStatus = _useState8[0],
      setSelectStatus = _useState8[1];

  var finalStatus = 'status' in props ? status : selectStatus; // disableTime 和 disableDate 同时存在的话要合并

  var finalDisableDate = useMemo(function () {
    return mergeBooleanFns(disableDate, disableTime);
  }, [disableDate, disableTime]);

  var updateValue = function updateValue(value) {
    var start = value.start,
        end = value.end,
        range = value.range;
    var finalRange = range || stateValue.range;
    var finalEnd = end || stateValue.end;
    var finalStart = start || stateValue.start;

    if (memoDisableTimeAfter && end && end.isAfter(memoDisableTimeAfter)) {
      finalEnd = memoDisableTimeAfter;
      finalRange = finalStart && [finalStart, finalEnd];
    }

    if (memoDisableTimeBefore && start && start.isBefore(memoDisableTimeBefore)) {
      finalStart = memoDisableTimeBefore;
      finalRange = finalEnd && [finalStart, finalEnd];
    } // 如果range中已有值，但是本次更新没有传入range，要将range传入的值对应更新


    if (!range && finalRange) {
      finalRange = [finalStart, finalEnd];
    }

    setStateValue({
      range: finalRange,
      start: finalStart,
      end: finalEnd
    });

    if (onChange) {
      onChange(value);
    }
  };

  var handleClear = _useEventCallback(function () {
    if (!isControlled) {
      setSelectStatus('start');
      setStateValue({});
    }

    setDisableTimeBefore(disableTimeBeforeProps);
    setDisableTimeAfter(disableTimeAfterProps);

    if (onClear) {
      onClear();
    }

    if (onChange) {
      onChange();
    }
  });

  var handleConfirm = _useEventCallback(function () {
    if (finalStatus === 'start') {
      setSelectStatus('end');
    }

    if (onConfirm && finalValue) {
      onConfirm(finalValue.range || []);
    }
  });

  var calendarProps = {
    allowClear: allowClear,
    todayDate: todayDate,
    disableRange: disableRange,
    disableDate: finalDisableDate,
    onUIChange: onUIChange,
    defaultViewDate: defaultViewDate
  };

  if ('viewType' in props) {
    calendarProps.viewType = viewType;
  }

  if ('viewDate' in props) {
    calendarProps.viewDate = viewDate;
  }

  var calendarState = useCalendar(calendarProps);

  var handleTimeChange = _useEventCallback(function (time) {
    if (finalStatus === 'start') {
      updateValue({
        start: time
      });
    } else {
      updateValue({
        end: time
      });
    }
  });

  var handleOptionClick = _useEventCallback(function (time) {
    calendarState.updateUIBySelectedDate(time);
    updateValue(_defineProperty({}, finalStatus, time));
  });

  var handleDateClick = _useEventCallback(function (date) {
    calendarState.handleItemClick(date);
    var start = finalValue.start;
    var end = finalValue.end;

    if (finalStatus === 'start') {
      start = start ? date : transformDateByTime(date, Array.isArray(defaultTime) ? defaultTime[0] : defaultTime);
      updateValue({
        start: start
      });
    } else {
      end = end ? date : transformDateByTime(date, Array.isArray(defaultTime) ? defaultTime[1] : defaultTime);
      updateValue({
        end: end
      });
    }

    if (onSelect) {
      onSelect([start, end]);
    }
  });

  var isValid = function isValid(date) {
    if (!date) return false;
    return dayjs(date).isValid();
  };

  var handleItemHover = _useEventCallback(function (date) {
    var start = finalValue.start,
        end = finalValue.end;

    if (isValid(start) && isValid(end) || !start && !end) {
      return;
    }

    var range = finalStatus === 'start' ? end && [date, end] : start && [start, date];
    updateValue({
      range: range
    });
  });

  var timePickerPanelProps = {
    hourStep: hourStep,
    minuteStep: minuteStep,
    secondStep: secondStep,
    disabledHours: disabledHours,
    disabledMinutes: disabledMinutes,
    disabledSeconds: disabledSeconds,
    disableTimeAfter: memoDisableTimeAfter,
    disableTimeBefore: memoDisableTimeBefore,
    hideDisabledOptions: hideDisabledOptions,
    value: finalStatus === 'start' ? finalValue.start : finalValue.end,
    onChange: handleTimeChange,
    format: timeFormat
  };
  var finalOptions = useMemo(function () {
    return formatCalendarOptions(options, handleOptionClick, function (opt) {
      if (typeof opt.date === 'function') {
        _devWarning('[RangeDateTimeCalendar]: option.date() return invalid date.');
      } else {
        _devWarning('[RangeDateTimeCalendar]: options passed invalid date.');
      }
    });
  }, [handleOptionClick, options]);
  useEffect(function () {
    setDisableTimeAfter(disableTimeAfterProps);
    setDisableTimeBefore(disableTimeBeforeProps);
  }, [disableTimeAfterProps, disableTimeBeforeProps]);
  return {
    calendarState: calendarState,
    finalUIState: calendarState.finalUIState,
    otherProps: otherProps,
    finalValue: finalValue,
    timePanelTitle: '',
    timePickerPanelProps: timePickerPanelProps,
    finalOptions: finalOptions,
    finalStatus: finalStatus,
    timeRowNum: timeRowNum,
    // 直接透传
    styles: styles,
    allowClear: allowClear,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // 事件处理
    handleTimeChange: handleTimeChange,
    handleClear: handleClear,
    handleConfirm: handleConfirm,
    handleDateClick: handleDateClick,
    // 给其他 Hook 使用
    updateValue: updateValue,
    // for test
    handleOptionClick: handleOptionClick,
    handleItemHover: handleItemHover
  };
}