import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _omit from "lodash/omit";
import _get from "lodash/get";
import React, { useCallback, useMemo, useRef } from 'react';
import BaseMenu from '../BaseMenu';
import useLocale from '../Locale/useLocale';
import Result from '../Result';
import Spin from '../Spin';
import Trigger from '../Trigger';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { allSelectProps } from './const';
import Option from './Option';
import OptionDivider from './OptionDivider';
import OptionGroup from './OptionGroup';
import SelectInput from './SelectInput';
import { StyledMenuItemWithoutContent, StyledNotFoundContent } from './styled';
import useSelect from './useSelect';
import { getChildType, getOptionPropValue, isOption, isOptionDivider, isOptionGroup } from './utils';
var defaultStyles = {
  menu: '',
  menuGroup: '',
  menuItem: '',
  menuDivider: ''
};
var Select = memoForwardRef(function (props, ref) {
  var locale = useLocale();
  var _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      _props$placeholder = props.placeholder,
      placeholder = _props$placeholder === void 0 ? locale['Select.placeholder'] : _props$placeholder,
      _props$maxVerticalTag = props.maxVerticalTagCount,
      maxVerticalTagCount = _props$maxVerticalTag === void 0 ? 2.5 : _props$maxVerticalTag,
      notFoundContent = props.notFoundContent,
      _props$loading = props.loading,
      loading = _props$loading === void 0 ? false : _props$loading,
      loadingIcon = props.loadingIcon,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      popperProps = props.popperProps,
      flip = props.flip,
      arrowPointAtCenter = props.arrowPointAtCenter,
      getPopupContainer = props.getPopupContainer,
      onPopupScroll = props.onPopupScroll,
      _props$leaveDelay = props.leaveDelay,
      leaveDelay = _props$leaveDelay === void 0 ? 100 : _props$leaveDelay,
      selectedIcon = props.selectedIcon,
      width = props.width,
      popupWidth = props.popupWidth,
      _props$maxItemCountPe = props.maxItemCountPerPage,
      maxItemCountPerPage = _props$maxItemCountPe === void 0 ? 6.5 : _props$maxItemCountPe,
      _props$mode = props.mode,
      mode = _props$mode === void 0 ? 'default' : _props$mode,
      _props$checkable = props.checkable,
      checkable = _props$checkable === void 0 ? false : _props$checkable,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      children = props.children,
      styles = props.styles,
      _props$labelInValue = props.labelInValue,
      labelInValue = _props$labelInValue === void 0 ? false : _props$labelInValue,
      _props$backfill = props.backfill,
      backfill = _props$backfill === void 0 ? false : _props$backfill,
      expandIcon = props.expandIcon,
      _props$hideExpandIcon = props.hideExpandIcon,
      hideExpandIcon = _props$hideExpandIcon === void 0 ? false : _props$hideExpandIcon,
      _props$loadingAll = props.loadingAll,
      loadingAll = _props$loadingAll === void 0 ? true : _props$loadingAll,
      inputRef = props.inputRef,
      menuRef = props.menuRef,
      _props$autoResize = props.autoResize,
      autoResize = _props$autoResize === void 0 ? false : _props$autoResize,
      _props$bordered = props.bordered,
      bordered = _props$bordered === void 0 ? true : _props$bordered,
      _props$readOnly = props.readOnly,
      readOnly = _props$readOnly === void 0 ? false : _props$readOnly,
      _props$showNoDataWhen = props.showNoDataWhenSearch,
      showNoDataWhenSearch = _props$showNoDataWhen === void 0 ? false : _props$showNoDataWhen,
      renderTag = props.renderTag,
      dropdownRender = props.dropdownRender,
      tokenSeparators = props.tokenSeparators;
  var theme = useTheme();
  var token = theme.components.Select;
  var innerMenuRef = useRef(null);
  var firstSelectedItemRef = useRef(null);
  var innerInputRef = useRef(null);

  var handleInputRef = _useForkRef(innerInputRef, inputRef);

  var nodeRef = useRef(null);

  var handleRef = _useForkRef(ref, nodeRef);

  var handleMenuRef = _useForkRef(menuRef, innerMenuRef);

  var _useSelect = useSelect(props, nodeRef, innerInputRef, innerMenuRef, firstSelectedItemRef),
      inputValue = _useSelect.inputValue,
      popupVisibleState = _useSelect.popupVisibleState,
      valueState = _useSelect.valueState,
      activeKey = _useSelect.activeKey,
      mergedOptions = _useSelect.mergedOptions,
      filteredOptions = _useSelect.filteredOptions,
      allOptions = _useSelect.allOptions,
      isMultiple = _useSelect.isMultiple,
      hasValue = _useSelect.hasValue,
      showSearch = _useSelect.showSearch,
      tagsExtraOptions = _useSelect.tagsExtraOptions,
      searchReservedOptions = _useSelect.searchReservedOptions,
      firstSelectedOption = _useSelect.firstSelectedOption,
      combobox = _useSelect.combobox,
      optionSelectedIndex = _useSelect.optionSelectedIndex,
      handleSelect = _useSelect.handleSelect,
      handleDeselect = _useSelect.handleDeselect,
      handleOptionClick = _useSelect.handleOptionClick,
      handleOptionMouseEnter = _useSelect.handleOptionMouseEnter,
      handlePopupEnter = _useSelect.handlePopupEnter,
      handlePopupExited = _useSelect.handlePopupExited,
      handlePopupVisibleChange = _useSelect.handlePopupVisibleChange,
      handleClear = _useSelect.handleClear,
      handleInputChange = _useSelect.handleInputChange,
      handleInputKeyDown = _useSelect.handleInputKeyDown,
      handleAddTag = _useSelect.handleAddTag,
      handleSearch = _useSelect.handleSearch,
      menuWidth = _useSelect.menuWidth;

  var selectMenuStyles = useMemo(function () {
    if (styles) {
      var menu = styles.menu,
          menuGroup = styles.menuGroup,
          menuItem = styles.menuItem,
          menuDivider = styles.menuDivider;
      return {
        menu: menu,
        menuGroup: menuGroup,
        menuItem: menuItem,
        menuDivider: menuDivider
      };
    }
  }, [styles]);
  var innerStyles = useStyles('select', defaultStyles, selectMenuStyles);
  var loadingContentNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledMenuItemWithoutContent, {
      key: "loading",
      theme: theme,
      size: size
    }, loadingIcon || /*#__PURE__*/React.createElement(Spin, null));
  }, [loadingIcon, size, theme]);
  var notFoundContentNode = useMemo(function () {
    if (notFoundContent) {
      return notFoundContent;
    } else {
      var subTitle = locale['Select.noDataText'];

      if (mode === 'tags') {
        subTitle = locale['Select.pressEnterText'];
      } else if (showSearch && inputValue) {
        subTitle = locale['Select.noResultText'];
      }

      return /*#__PURE__*/React.createElement(StyledNotFoundContent, {
        theme: theme
      }, /*#__PURE__*/React.createElement(Result, {
        subTitle: subTitle,
        type: "emptySmall"
      }));
    }
  }, [inputValue, locale, mode, notFoundContent, showSearch, theme]);
  var getMenuItemByOption = useCallback(function (option) {
    var selectedIndex = optionSelectedIndex(option);
    var refProps = {};

    if (firstSelectedOption && firstSelectedOption.value === option.value) {
      refProps.ref = firstSelectedItemRef;
    }

    return /*#__PURE__*/React.createElement(BaseMenu.Item, _extends({}, refProps, innerStyles.menuItem, {
      key: option.value,
      size: size,
      disabled: !!option.disabled,
      selected: selectedIndex !== -1,
      checkable: checkable,
      selectedIcon: selectedIcon,
      active: activeKey === "".concat(option.value),
      onClick: function onClick(e) {
        return handleOptionClick(option, e);
      },
      onMouseEnter: handleOptionMouseEnter
    }), option.label || option.value);
  }, [activeKey, checkable, firstSelectedOption, handleOptionClick, handleOptionMouseEnter, innerStyles.menuItem, optionSelectedIndex, selectedIcon, size]);
  var getSearchReservedMenuItem = useCallback(function () {
    var menus = [];

    if (showSearch) {
      for (var _i = 0, _Object$keys = Object.keys(searchReservedOptions); _i < _Object$keys.length; _i++) {
        var key = _Object$keys[_i];
        var option = searchReservedOptions[key];

        if (!filteredOptions[option.value] && !tagsExtraOptions[option.value]) {
          menus.push(getMenuItemByOption(option));
        }
      }
    }

    return menus;
  }, [filteredOptions, getMenuItemByOption, searchReservedOptions, showSearch, tagsExtraOptions]);
  var getTagsExtraMenuItem = useCallback(function () {
    var menus = [];

    if (mode === 'tags') {
      for (var _i2 = 0, _Object$keys2 = Object.keys(tagsExtraOptions); _i2 < _Object$keys2.length; _i2++) {
        var key = _Object$keys2[_i2];
        menus.push(getMenuItemByOption(tagsExtraOptions[key]));
      }
    }

    return menus;
  }, [getMenuItemByOption, mode, tagsExtraOptions]); // 渲染逻辑

  var getMenuContentFromChildren = useCallback(function (children) {
    var menus = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    // eslint-disable-next-line complexity
    React.Children.forEach(children, function (child, index) {
      if (!child) return;
      var type = getChildType(child);

      if (isOptionGroup(type)) {
        var groupChildren = getMenuContentFromChildren(child.props.children);

        if (groupChildren.length > 0) {
          var style = innerStyles.menuGroup.style;

          var childStyle = _get(child, 'props.style');

          if (childStyle) {
            style = _extends({}, style, childStyle);
          }

          menus.push( /*#__PURE__*/React.createElement(BaseMenu.Group, _extends({}, child.props, {
            style: style,
            className: [innerStyles.menuGroup.className, _get(child, 'props.className', '')].join(' ').trim(),
            key: child.key || child.props.label
          }), groupChildren));
        }
      } else if (isOption(type)) {
        var option;
        var value = getOptionPropValue(child, 'value');

        if (showSearch && filteredOptions[value]) {
          option = filteredOptions[value];
        } else if (!showSearch && mergedOptions[value]) {
          option = mergedOptions[value];
        }

        if (option) {
          var selectedIndex = optionSelectedIndex(option);
          var refProps = {};

          if (firstSelectedOption && firstSelectedOption.value === value) {
            refProps.ref = firstSelectedItemRef;
          }

          var _children = getOptionPropValue(child, 'children'); // 没有 child.style 时保证是一个引用


          var _style = innerStyles.menuItem.style;

          var _childStyle = _get(child, 'props.style');

          if (_childStyle) {
            _style = _extends({}, _style, _childStyle);
          }

          menus.push( /*#__PURE__*/React.createElement(BaseMenu.Item, _extends({}, child.props, refProps, {
            style: _style,
            className: [innerStyles.menuItem.className, _get(child, 'props.className', '')].join(' ').trim(),
            key: child.key || option.value,
            size: size,
            selected: selectedIndex !== -1,
            checkable: checkable,
            active: activeKey === "".concat(option.value),
            selectedIcon: selectedIcon,
            onClick: function onClick(e) {
              return handleOptionClick(option, e);
            },
            onMouseEnter: function onMouseEnter(e) {
              return handleOptionMouseEnter(e, _get(child, 'props.onMouseEnter'));
            }
          }), _children || option.label || option.value));
        }
      } else if (isOptionDivider(type)) {
        menus.push( /*#__PURE__*/React.createElement(BaseMenu.Divider, _extends({}, child.props, innerStyles.menuDivider, {
          key: child.key || "divider_".concat(index)
        })));
      } else if (child.props && Array.isArray(child.props.children)) {
        var _children2 = getMenuContentFromChildren(child.props.children);

        if (_children2 && _children2.length > 0) {
          menus.push.apply(menus, _toConsumableArray(_children2));
        }
      }

      return;
    });
    return menus;
  }, [activeKey, checkable, filteredOptions, firstSelectedOption, handleOptionClick, handleOptionMouseEnter, innerStyles.menuDivider, innerStyles.menuGroup.className, innerStyles.menuGroup.style, innerStyles.menuItem.className, innerStyles.menuItem.style, mergedOptions, optionSelectedIndex, selectedIcon, showSearch, size]);
  var menuContent = useMemo(function () {
    var menuContent;

    if (loading && loadingAll) {
      menuContent = loadingContentNode;
    } else {
      menuContent = [].concat(_toConsumableArray(getSearchReservedMenuItem()), _toConsumableArray(getMenuContentFromChildren(children)), _toConsumableArray(getTagsExtraMenuItem()));

      if (loading) {
        menuContent.push(loadingContentNode);
      }
    }

    if ((showNoDataWhenSearch || showSearch && inputValue || !showSearch) && !combobox && menuContent.length === 0) {
      menuContent = notFoundContentNode;
    }

    return menuContent;
  }, [children, combobox, getMenuContentFromChildren, getSearchReservedMenuItem, getTagsExtraMenuItem, inputValue, loading, loadingAll, loadingContentNode, notFoundContentNode, showNoDataWhenSearch, showSearch]);
  var menus = useMemo(function () {
    var menus = /*#__PURE__*/React.createElement(BaseMenu.Menu, _extends({
      ref: handleMenuRef,
      maxItemCountPerPage: maxItemCountPerPage,
      width: popupWidth || menuWidth,
      size: size,
      onScroll: onPopupScroll,
      noShadow: !!dropdownRender,
      tabIndex: -1
    }, innerStyles.menu), menuContent);

    if (dropdownRender) {
      return dropdownRender(menus);
    } else {
      return menus;
    }
  }, [dropdownRender, handleMenuRef, innerStyles.menu, maxItemCountPerPage, menuContent, menuWidth, onPopupScroll, popupWidth, size]);
  var showMenu = useMemo(function () {
    var showMenu = true;

    if (showSearch && (!inputValue || combobox && inputValue) && menuContent.length === 0) {
      showMenu = false;
    }

    return showMenu;
  }, [combobox, inputValue, menuContent, showSearch]);
  var transitionProps = useMemo(function () {
    return {
      onEnter: handlePopupEnter,
      onExited: handlePopupExited
    };
  }, [handlePopupEnter, handlePopupExited]);

  var restProps = _omit(props, allSelectProps);

  return /*#__PURE__*/React.createElement(Trigger, _extends({
    container: getPopupContainer,
    placement: placement,
    leaveDelay: leaveDelay,
    triggerAction: "click",
    open: disabled || readOnly ? false : popupVisibleState && showMenu,
    offset: token.offset[size],
    onVisibleChange: handlePopupVisibleChange,
    popperProps: popperProps,
    flip: flip,
    arrowPointAtCenter: arrowPointAtCenter,
    popup: menus,
    transitionProps: transitionProps,
    disabled: disabled || readOnly,
    hideArrow: true
  }, restProps), /*#__PURE__*/React.createElement(SelectInput, {
    ref: handleRef,
    inputRef: handleInputRef,
    menuRef: innerMenuRef,
    disabled: disabled,
    placeholder: placeholder,
    inputValue: inputValue,
    popupVisible: popupVisibleState,
    showMenu: showMenu,
    maxVerticalTagCount: maxVerticalTagCount,
    showSearch: showSearch,
    labelInValue: labelInValue,
    backfill: backfill,
    expandIcon: expandIcon,
    hideExpandIcon: hideExpandIcon,
    value: valueState,
    allOptions: allOptions,
    isMultiple: isMultiple,
    hasValue: hasValue,
    onClear: handleClear,
    onKeyDown: handleInputKeyDown,
    onInputChange: handleInputChange,
    onAddTag: handleAddTag,
    onSelect: handleSelect,
    onDeselect: handleDeselect,
    size: size,
    mode: mode,
    width: autoResize ? 'auto' : width,
    styles: styles,
    autoResize: autoResize,
    onSearch: handleSearch,
    bordered: bordered,
    readOnly: readOnly,
    combobox: combobox,
    renderTag: renderTag,
    tokenSeparators: tokenSeparators
  }));
});
Select.__MUYA_SELECT = true;
Select.Option = Option;
Select.OptionDivider = OptionDivider;
Select.OptionGroup = OptionGroup;
export default Select;