import _extends from "@babel/runtime/helpers/extends";
import { FoldIcon as _FoldIcon } from "@muya-ui/theme-light/es/icons/FoldIcon";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _omit from "lodash/omit";
import _endsWith from "lodash/endsWith";
import React, { cloneElement, useMemo, useRef } from 'react';
import ExpandIcon from '../ExpandIcon';
import Input, { TagsInput } from '../Input';
import { StyledPopperInputWrapper } from '../Input/styled';
import ExpandWrapper from '../styled/components/ExpandWrapper';
import addPx from '../utils/addPx';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { allSelectInputProps } from './const';
import { StyledSelectInputSizer, StyledLabelWrapper } from './styled';
import useSelectInput from './useSelectInput';
var defaultStyles = {
  inputWrapper: '',
  input: ''
};
var SelectInput = memoForwardRef(function (props, ref) {
  var id = props.id,
      disabled = props.disabled,
      allowClear = props.allowClear,
      readOnly = props.readOnly,
      placeholder = props.placeholder,
      status = props.status,
      maxVerticalTagCount = props.maxVerticalTagCount,
      maxTagWidth = props.maxTagWidth,
      maxTagTextLength = props.maxTagTextLength,
      showSearch = props.showSearch,
      expandIcon = props.expandIcon,
      inputValue = props.inputValue,
      popupVisible = props.popupVisible,
      showMenu = props.showMenu,
      value = props.value,
      isMultiple = props.isMultiple,
      hasValue = props.hasValue,
      mode = props.mode,
      width = props.width,
      hideExpandIcon = props.hideExpandIcon,
      getInputElement = props.getInputElement,
      styles = props.styles,
      style = props.style,
      inputRef = props.inputRef,
      className = props.className,
      onClear = props.onClear,
      prefixNode = props.prefixNode,
      suffixNode = props.suffixNode,
      _props$clearReplace = props.clearReplace,
      clearReplace = _props$clearReplace === void 0 ? true : _props$clearReplace,
      autoResize = props.autoResize,
      bordered = props.bordered,
      size = props.size,
      combobox = props.combobox,
      renderTag = props.renderTag;
  var theme = useTheme();
  var token = theme.components.Select;
  var innerInputRef = useRef(null);
  var sizerRef = useRef(null);

  var handleRef = _useForkRef(innerInputRef, inputRef);

  var _useSelectInput = useSelectInput(props, innerInputRef, sizerRef),
      disabledValue = _useSelectInput.disabledValue,
      readOnlyValue = _useSelectInput.readOnlyValue,
      getDefaultModeLabel = _useSelectInput.getDefaultModeLabel,
      labelByValue = _useSelectInput.labelByValue,
      handleAddTag = _useSelectInput.handleAddTag,
      handleRemoveTag = _useSelectInput.handleRemoveTag,
      handleInput = _useSelectInput.handleInput,
      handleInputFocus = _useSelectInput.handleInputFocus,
      handleInputBlur = _useSelectInput.handleInputBlur,
      handleInputKeyDown = _useSelectInput.handleInputKeyDown,
      handleInputPaste = _useSelectInput.handleInputPaste;

  var realVisible = popupVisible && showMenu;
  var finalAllowClear = allowClear && !disabled && !readOnly;
  var selectInputStyles = useMemo(function () {
    var selectInputStyles = {};

    if (styles) {
      var inputWrapper = styles.inputWrapper,
          input = styles.input;
      selectInputStyles = {
        inputWrapper: inputWrapper,
        input: input
      };
    }

    return selectInputStyles;
  }, [styles]);
  var innerStyles = useStyles('select', defaultStyles, selectInputStyles);
  var wrapperStyleItem = useMemo(function () {
    return mergeStyleItem({
      className: className,
      style: style
    }, innerStyles.inputWrapper);
  }, [className, innerStyles.inputWrapper, style]);

  var inputRestProps = _omit(props, allSelectInputProps);

  var finalSuffixNode = useMemo(function () {
    var finalSuffixNode = [];

    if (suffixNode) {
      finalSuffixNode.push(suffixNode);
    }

    if (!hideExpandIcon) {
      var DefaultExpandIcon = token.expandIcon || _FoldIcon;
      finalSuffixNode.push( /*#__PURE__*/React.createElement(ExpandWrapper, {
        key: "expand-icon",
        theme: theme,
        $expanded: realVisible
      }, /*#__PURE__*/React.createElement(ExpandIcon, {
        icon: expandIcon,
        DefaultIcon: DefaultExpandIcon,
        disabled: disabled
      })));
    }

    return finalSuffixNode;
  }, [disabled, expandIcon, hideExpandIcon, realVisible, suffixNode, theme, token.expandIcon]);

  var renderSingleModeNode = function renderSingleModeNode() {
    var inputProps = _extends({}, inputRestProps, innerStyles.input, {
      allowClear: finalAllowClear,
      disabled: disabled,
      readOnly: readOnly,
      width: typeof width === 'string' && _endsWith(width, '%') ? '100%' : width,
      placeholder: placeholder,
      value: inputValue,
      inputRef: handleRef,
      prefixNode: prefixNode,
      suffixNode: finalSuffixNode,
      clearReplace: clearReplace,
      onKeyDown: handleInputKeyDown,
      onInput: handleInput,
      onFocus: handleInputFocus,
      onBlur: handleInputBlur,
      onClear: onClear
    });

    var label = getDefaultModeLabel(combobox); // 非搜索模式那么不可输入

    if (!showSearch || !popupVisible && !combobox) {
      inputProps.readOnly = true;
    }

    if (typeof label === 'string' || typeof label === 'number') {
      if (!realVisible && !combobox || combobox && !popupVisible && label) {
        inputProps.value = String(label);
      }

      if (!inputValue && label) {
        inputProps.placeholder = String(label);
      }
    } else {
      inputProps.replacedInputNode = /*#__PURE__*/React.createElement(StyledLabelWrapper, null, label);
    }

    if (inputProps.value) {
      inputProps.title = inputProps.value;
    }

    var inputElement = /*#__PURE__*/React.createElement(Input, {
      type: "text"
    });

    if (getInputElement) {
      inputElement = getInputElement();
    }

    var input = /*#__PURE__*/cloneElement(inputElement, _extends({
      ref: ref
    }, inputProps));

    if (!autoResize) {
      return input;
    }

    return /*#__PURE__*/React.createElement(React.Fragment, null, input, /*#__PURE__*/React.createElement(StyledSelectInputSizer, {
      ref: sizerRef,
      theme: theme,
      $size: size
    }, label || placeholder));
  }; // 渲染 input 和 tag


  var renderMultipleModeNode = function renderMultipleModeNode() {
    return /*#__PURE__*/React.createElement(TagsInput, _extends({}, inputRestProps, innerStyles.input, {
      ref: ref,
      inputRef: handleRef,
      suffixNode: finalSuffixNode,
      width: typeof width === 'string' && _endsWith(width, '%') ? '100%' : width,
      disabled: disabled,
      readOnly: readOnly,
      inputReadOnly: !showSearch,
      value: value,
      styles: selectInputStyles,
      inputValue: inputValue,
      disabledValue: disabledValue,
      readOnlyValue: readOnlyValue,
      placeholder: hasValue ? '' : placeholder,
      allowAdd: mode === 'tags',
      allowClear: finalAllowClear,
      getTagText: labelByValue,
      maxTagTextLength: maxTagTextLength,
      maxVerticalTagCount: maxVerticalTagCount,
      maxTagWidth: maxTagWidth,
      scrollTopAfterEffect: !popupVisible,
      onAddTag: handleAddTag,
      onRemoveTag: handleRemoveTag,
      onInputKeyDown: handleInputKeyDown,
      onInput: handleInput,
      onFocus: handleInputFocus,
      onBlur: handleInputBlur,
      onClear: onClear,
      renderTag: renderTag,
      onPaste: handleInputPaste
    }));
  };

  var renderInput = function renderInput() {
    if (isMultiple) {
      return renderMultipleModeNode();
    } else {
      return renderSingleModeNode();
    }
  };

  return /*#__PURE__*/React.createElement(StyledPopperInputWrapper, _extends({
    id: id,
    role: "combobox",
    key: "selection",
    $pointer: !isMultiple && !showSearch && !disabled || readOnly,
    $popupVisible: realVisible,
    $width: addPx(width),
    $status: status,
    $bordered: bordered,
    theme: theme
  }, wrapperStyleItem), renderInput());
});
export default SelectInput;