import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _typeof from "@babel/runtime/helpers/typeof";
import _extends from "@babel/runtime/helpers/extends";
import { usePrevious as _usePrevious } from "@muya-ui/utils/es/hooks/usePrevious";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _uniq from "lodash/uniq";
import _isEmpty from "lodash/isEmpty";
import { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { shallowEqualArrays, shallowEqualObjects } from 'shallow-equal';
import { KeyCode } from '../utils/keyCode';
import { getActiveKeyFromValue, getOptionsFromChildren, getValueFromProps } from './utils';
export default function useSelect(props, nodeRef, inputRef, menuRef, firstSelectedItemRef) {
  var _props$mode = props.mode,
      mode = _props$mode === void 0 ? 'default' : _props$mode;
  /**
   * 是否多选
   */

  var isMultiple = mode === 'multiple' || mode === 'tags';
  var popupVisible = props.popupVisible,
      _props$labelInValue = props.labelInValue,
      labelInValue = _props$labelInValue === void 0 ? false : _props$labelInValue,
      maxTagCount = props.maxTagCount,
      children = props.children,
      _props$filterOption = props.filterOption,
      filterOption = _props$filterOption === void 0 ? true : _props$filterOption,
      _props$showSearch = props.showSearch,
      showSearch = _props$showSearch === void 0 ? isMultiple : _props$showSearch,
      _props$backfill = props.backfill,
      backfill = _props$backfill === void 0 ? false : _props$backfill,
      _props$reservedSearch = props.reservedSearchResult,
      reservedSearchResult = _props$reservedSearch === void 0 ? true : _props$reservedSearch,
      _props$defaultActiveF = props.defaultActiveFirstOption,
      defaultActiveFirstOption = _props$defaultActiveF === void 0 ? false : _props$defaultActiveF,
      comboboxProp = props.combobox,
      popupWidth = props.popupWidth,
      onClear = props.onClear,
      onSelect = props.onSelect,
      onDeselect = props.onDeselect,
      onChange = props.onChange,
      onPopupVisibleChange = props.onPopupVisibleChange,
      onSearch = props.onSearch;
  var menuWidthRef = useRef(0);
  /**
   * 是否受控
   */

  var isControlled = ('value' in props);
  var isPopupVisibleControlled = ('popupVisible' in props);
  /**
   * 搜索框的的内容：
   * 1. mode 为 default 且 showSearch 为 true 时，显示搜索框
   * 2. mode 为 multiple 时，显示搜索框
   * 3. mode 为 tags 时，显示搜索框，此搜索框还支持回车将输入的文字作为新的 option，且选中
   */

  var _useState = useState(''),
      _useState2 = _slicedToArray(_useState, 2),
      inputValue = _useState2[0],
      setInputValue = _useState2[1];
  /**
   * 弹出状态，可传入 popupVisible 作为初始值并进行控制
   */


  var _useState3 = useState(popupVisible || false),
      _useState4 = _slicedToArray(_useState3, 2),
      popupVisibleState = _useState4[0],
      setPopupVisibleState = _useState4[1];
  /**
   * 选中的值
   */


  var _useState5 = useState(function () {
    return getValueFromProps(props);
  }),
      _useState6 = _slicedToArray(_useState5, 2),
      valueState = _useState6[0],
      setValueState = _useState6[1];
  /**
   * 键盘激活的 index
   */


  var _useState7 = useState(function () {
    return getActiveKeyFromValue(valueState);
  }),
      _useState8 = _slicedToArray(_useState7, 2),
      activeKey = _useState8[0],
      setActiveKey = _useState8[1];
  /**
   * tags 模式下，通过 input 添加的 option
   */


  var _useState9 = useState({}),
      _useState10 = _slicedToArray(_useState9, 2),
      tagsExtraOptions = _useState10[0],
      setTagsExtraOptions = _useState10[1];
  /**
   * 上一次的 inputValue
   */


  var preInputValue = _usePrevious(inputValue);
  /**
   * combobox 在多选下不生效
   */


  var combobox = comboboxProp && !isMultiple;
  /**
   * 当前值是否存在
   */

  var hasValue = isMultiple && valueState.length > 0 || !isMultiple && !!valueState[0];
  /**
   * 从 children 中解析出的 options
   */

  var _useMemo = useMemo(function () {
    return getOptionsFromChildren(children);
  }, [children]),
      options = _useMemo.options,
      optionKeysOrder = _useMemo.optionKeysOrder;
  /**
   * 合并了 tag 生成的 options 和从 children 解析出来的 options
   */


  var mergedOptions = useMemo(function () {
    return _extends({}, options, tagsExtraOptions);
  }, [options, tagsExtraOptions]);
  /**
   * search 模式下，保留选项，防止二次搜索被清空
   */

  var _useState11 = useState(function () {
    if (showSearch && reservedSearchResult) {
      return valueState.reduce(function (options, value) {
        var optionKey;

        if (value && _typeof(value) === 'object') {
          optionKey = value.value;
        } else {
          optionKey = value;
        }

        if (optionKey && mergedOptions[optionKey]) {
          options[optionKey] = mergedOptions[optionKey];
        }

        return options;
      }, {});
    } else {
      return {};
    }
  }),
      _useState12 = _slicedToArray(_useState11, 2),
      searchReservedOptions = _useState12[0],
      setSearchReservedOptions = _useState12[1];
  /**
   * 所有的 options
   */


  var allOptions = useMemo(function () {
    return _extends({}, searchReservedOptions, mergedOptions);
  }, [mergedOptions, searchReservedOptions]);
  /**
   * 搜索词过滤后的 options
   */

  var filteredOptions = useMemo(function () {
    if (!filterOption) {
      return mergedOptions;
    } else {
      var _filteredOptions = {};

      for (var _i = 0, _Object$keys = Object.keys(mergedOptions); _i < _Object$keys.length; _i++) {
        var key = _Object$keys[_i];
        var option = mergedOptions[key];

        if (filterOption === true) {
          var _option$label = option.label,
              label = _option$label === void 0 ? '' : _option$label,
              _value = option.value;

          if ("".concat(label).indexOf(inputValue) >= 0 || "".concat(_value).indexOf(inputValue) >= 0) {
            _filteredOptions[key] = option;
          }
        } else {
          if (filterOption(inputValue, option)) {
            _filteredOptions[key] = option;
          }
        }
      }

      return _filteredOptions;
    }
  }, [filterOption, mergedOptions, inputValue]);
  /**
   * 上一次的 filteredOptions
   */

  var preFilteredOptions = _usePrevious(filteredOptions);
  /**
   * 选中的第一个 option
   */


  var firstSelectedOption = useMemo(function () {
    if (valueState.length > 0) {
      if (labelInValue) {
        return allOptions[valueState[0].value];
      } else {
        return allOptions[valueState[0]];
      }
    }
  }, [labelInValue, allOptions, valueState]);
  /**
   * 计算 option 在当前选中值中的下标，下标为 -1 表示未选中
   */

  var optionSelectedIndex = useCallback(function (option) {
    var selectedIndex = -1;

    for (var i = 0; i < valueState.length; i++) {
      var temp = valueState[i];

      if (labelInValue && temp.value === option.value || !labelInValue && temp === option.value) {
        selectedIndex = i;
        break;
      }
    }

    return selectedIndex;
  }, [labelInValue, valueState]);
  var focusInput = useCallback(function () {
    if (inputRef && inputRef.current) {
      inputRef.current.focus();
    }
  }, [inputRef]);
  /**
   * 修改 popup 状态，内部使用
   */

  var handlePopupVisibleChange = useCallback(function (popupVisible) {
    if (!isPopupVisibleControlled) {
      setPopupVisibleState(popupVisible);
    }

    if (onPopupVisibleChange) {
      onPopupVisibleChange(popupVisible);
    } // 关闭弹窗后，输入框强制失焦


    if (popupVisible === false) {
      // 保证失焦动作在弹窗状态变更完成后执行，修复表单校验错误的问题
      setTimeout(function () {
        if (inputRef.current) {
          inputRef.current.blur();
        }
      }, 0);
    } // 动态调整下拉菜单的宽度
    // 若外部指定了下拉框宽度，不动态调整


    if (!popupWidth && nodeRef.current && nodeRef.current.offsetWidth !== menuWidthRef.current && popupVisible) {
      menuWidthRef.current = nodeRef.current.offsetWidth;
    }
  }, [inputRef, isPopupVisibleControlled, nodeRef, onPopupVisibleChange, popupWidth]);
  /**
   * value 变化回调
   */

  var handleChange = useCallback(function (value, visible) {
    if (onChange) {
      onChange(value);
    } // 单选需要关闭


    handlePopupVisibleChange(visible);
  }, [handlePopupVisibleChange, onChange]);
  /**
   * 选择一个 option：
   * 1. 点击一个未选中的 option
   * 2. tags 模式下回车
   */
  // eslint-disable-next-line complexity

  var handleSelect = _useEventCallback(function (option) {
    var popupVisible = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    var e = arguments.length > 2 ? arguments[2] : undefined;
    focusInput();

    if (option.disabled && e) {
      e.stopPropagation();
      return;
    }

    var newValueState;

    if (labelInValue) {
      if (isMultiple) {
        if (typeof maxTagCount === 'undefined' || Object.keys(valueState).length < maxTagCount) {
          newValueState = [].concat(_toConsumableArray(valueState), [{
            label: option.label,
            value: option.value
          }]);
          handleChange(newValueState, isMultiple);
          inputValue && setInputValue('');
        }
      } else {
        newValueState = [{
          label: option.label,
          value: option.value
        }];
        handleChange(newValueState[0], popupVisible);
      }
    } else {
      if (isMultiple) {
        if (typeof maxTagCount === 'undefined' || Object.keys(valueState).length < maxTagCount) {
          newValueState = [].concat(_toConsumableArray(valueState), [option.value]);
          handleChange(newValueState, isMultiple);
          inputValue && setInputValue('');
        }
      } else {
        newValueState = [option.value];
        handleChange(newValueState[0], popupVisible);
      }
    } // 保存搜索选项


    if (showSearch && reservedSearchResult) {
      if (isMultiple) {
        setSearchReservedOptions(_extends({}, searchReservedOptions, _defineProperty({}, option.value, option)));
      } else {
        setSearchReservedOptions(_defineProperty({}, option.value, option));
      }
    }

    if (!isControlled && newValueState) {
      setValueState(newValueState);
    }

    if (onSelect) {
      onSelect(option);
    }
  });

  var handleMultiSelect = _useEventCallback(function (options) {
    focusInput();
    var newValueState;
    var selectOptions = options;
    var len = Object.keys(valueState).length;

    if (typeof maxTagCount !== 'undefined' && len + options.length > maxTagCount) {
      selectOptions = options.slice(0, maxTagCount - len);
    }

    if (selectOptions.length === 0) return;

    if (labelInValue) {
      newValueState = [].concat(_toConsumableArray(valueState), _toConsumableArray(selectOptions.map(function (option) {
        return {
          label: option.value,
          value: option.value
        };
      })));
    } else {
      newValueState = [].concat(_toConsumableArray(valueState), _toConsumableArray(selectOptions.map(function (option) {
        return option.value;
      })));
    }

    handleChange(newValueState, false);
    inputValue && setInputValue(''); // 保存搜索选项

    if (showSearch && reservedSearchResult) {
      setSearchReservedOptions(_extends({}, searchReservedOptions, selectOptions.reduce(function (prev, option) {
        prev[option.value] = option;
        return prev;
      }, {})));
    }

    if (!isControlled && newValueState) {
      setValueState(newValueState);
    }

    if (onSelect) {
      onSelect(selectOptions);
    }
  });
  /**
   * 取消选择一个 option：
   * 1. 点击一个已选中的 option
   * 2. tags 模式下点击 tag 的 closeIcon
   */


  var handleDeselect = _useEventCallback(function (option, selectedIndex, e) {
    focusInput();

    if (option.disabled && e) {
      e.stopPropagation();
      return;
    }

    var newValueState;
    newValueState = _toConsumableArray(valueState);
    newValueState.splice(selectedIndex, 1);
    handleChange(newValueState, true);
    /**
     * tags 模式下取消选择需要删除 tags 回车增加的 option 选项
     */

    if (mode === 'tags' && tagsExtraOptions[option.value]) {
      var newTagsExtraOptions = _extends({}, tagsExtraOptions);

      delete newTagsExtraOptions[option.value];
      setTagsExtraOptions(_extends({}, newTagsExtraOptions));
    }

    if (isMultiple) {
      inputValue && setInputValue('');
    } // 去除搜索的选项


    if (showSearch && reservedSearchResult) {
      var newSearchReservedOptions = _extends({}, searchReservedOptions);

      delete newSearchReservedOptions[option.value];
      setSearchReservedOptions(newSearchReservedOptions);
    }

    if (!isControlled) {
      setValueState(newValueState);
    }

    if (onDeselect) {
      onDeselect(option, selectedIndex);
    }
  });
  /**
   * 点击 option
   */


  var handleOptionClick = useCallback(function (option, e) {
    e.stopPropagation();
    var selectedIndex = optionSelectedIndex(option);

    if (selectedIndex === -1) {
      handleSelect(option, false, e);
    } else {
      if (isMultiple) {
        handleDeselect(option, selectedIndex, e);
      } else {
        focusInput(); // 单选模式下点击 item 只关闭

        handlePopupVisibleChange(false);
      }
    }
  }, [focusInput, handleDeselect, handlePopupVisibleChange, handleSelect, isMultiple, optionSelectedIndex]);

  var handleOptionMouseEnter = _useEventCallback(function (e, onMouseEnter) {
    if (activeKey) {
      setActiveKey('');
    }

    if (onMouseEnter) {
      onMouseEnter(e);
    }
  });

  var handlePopupEnter = useCallback(function () {
    if (showSearch && combobox && firstSelectedOption && !inputValue) {
      setInputValue("".concat(firstSelectedOption.label || firstSelectedOption.value));
    }
  }, [combobox, firstSelectedOption, inputValue, showSearch]);
  /**
   * popup 动画退出时，清空 input
   */

  var handlePopupExited = useCallback(function () {
    if (showSearch && !combobox) {
      inputValue && setInputValue('');
    } else if (combobox && firstSelectedOption && !_isEmpty(filteredOptions)) {
      setInputValue("".concat(firstSelectedOption.label || firstSelectedOption.value));
    }

    if (firstSelectedOption && firstSelectedOption.value && firstSelectedOption.value !== activeKey) {
      setActiveKey(firstSelectedOption.value);
    }

    if (menuRef && menuRef.current) {
      if (firstSelectedItemRef && firstSelectedItemRef.current) {
        menuRef.current.firstElementChild.scrollTop = firstSelectedItemRef.current.offsetTop;
      } else {
        menuRef.current.firstElementChild.scrollTop = 0;
      }
    }
  }, [activeKey, combobox, filteredOptions, firstSelectedItemRef, firstSelectedOption, inputValue, menuRef, showSearch]);
  /**
   * 清空输入框
   */

  var handleClear = _useEventCallback(function (e) {
    inputValue && setInputValue('');

    if (isMultiple) {
      handleChange([], false);
    } else {
      if (labelInValue) {
        handleChange(undefined, false);
      } else {
        handleChange('', false);
      }
    }

    if (showSearch && reservedSearchResult) {
      setSearchReservedOptions({});
    }

    if (mode === 'tags') {
      setTagsExtraOptions({});
    }

    if (!isControlled) {
      setValueState([]);
    }

    if (onClear) {
      onClear(e);
    }
  }, [handleChange, isControlled, isMultiple, showSearch, labelInValue, mode, onClear, reservedSearchResult]);
  /**
   * 输入框输入文本
   */


  var handleInputChange = _useEventCallback(function (value) {
    setInputValue(value);

    if (mode === 'default' && showSearch && combobox) {
      var hasOption = !filteredOptions[value] && !searchReservedOptions[value];
      handleChange(value, true);

      if (hasOption) {
        setActiveKey('');
      }
    }
  });

  var handleSearch = _useEventCallback(function (value) {
    if (activeKey) {
      setActiveKey('');
    }

    onSearch && onSearch(value);
  });

  var sortKeys = useCallback(function (keys) {
    return keys.sort(function (a, b) {
      if (_typeof(optionKeysOrder[b]) === undefined) {
        return -1;
      }

      if (_typeof(optionKeysOrder[a]) === undefined) {
        return 1;
      }

      return optionKeysOrder[a] - optionKeysOrder[b];
    });
  }, [optionKeysOrder]);

  var getNextAvailableKey = _useEventCallback(function (keyCode) {
    var options = showSearch ? _extends({}, filteredOptions, searchReservedOptions) : allOptions;
    var keys = sortKeys(Object.keys(options));
    var len = keys.length;
    var flag = false;

    if (keyCode === KeyCode.UP) {
      for (var i = len - 1; i >= 0; i--) {
        var key = keys[i];

        if (key === "".concat(activeKey) || flag) {
          flag = true;

          if (i === 0 && !options[keys[len - 1]].disabled) {
            return keys[len - 1];
          } else if (i !== 0 && !options[keys[i - 1]].disabled) {
            return keys[i - 1];
          }
        }
      }
    } else {
      for (var _i2 = 0; _i2 < len; _i2++) {
        var _key = keys[_i2];

        if (_key === "".concat(activeKey) || flag) {
          flag = true;

          if (_i2 === len - 1 && !options[keys[0]].disabled) {
            return keys[0];
          } else if (_i2 !== len - 1 && !options[keys[_i2 + 1]].disabled) {
            return keys[_i2 + 1];
          }
        }
      }
    }
  });

  var handleInputKeyDown = _useEventCallback( // eslint-disable-next-line complexity
  function (e) {
    e.stopPropagation();

    if (e.keyCode !== KeyCode.DOWN && e.keyCode !== KeyCode.UP && e.keyCode !== KeyCode.ENTER && e.keyCode !== KeyCode.ESC) {
      return;
    }

    if (e.keyCode === KeyCode.ENTER) {
      if (popupVisibleState && activeKey) {
        var _options = _extends({}, filteredOptions, searchReservedOptions);

        var option = _options[activeKey];

        if (option) {
          var selectedIndex = optionSelectedIndex(option);

          if (selectedIndex === -1) {
            handleSelect(option, false, e);
          } else {
            if (isMultiple) {
              handleDeselect(option, selectedIndex, e);
            } else {
              handlePopupVisibleChange(false);
            }
          }
        } else {
          handlePopupVisibleChange(!popupVisibleState);
        }
      } else {
        if (isMultiple && inputValue) return;
        handlePopupVisibleChange(!popupVisibleState);
      }

      return;
    }

    if (e.keyCode === KeyCode.ESC && popupVisibleState) {
      handlePopupVisibleChange(false);
      return;
    }

    if (e.keyCode === KeyCode.DOWN || e.keyCode === KeyCode.UP) {
      if (activeKey) {
        var key = getNextAvailableKey(e.keyCode) || getActiveKeyFromValue(valueState);

        if (key) {
          setActiveKey(key);

          if (backfill) {
            var activeOption = allOptions[key];
            setInputValue("".concat(activeOption.label || activeOption.value));
          }

          return;
        }
      }

      var keys = Object.keys(_extends({}, filteredOptions, searchReservedOptions));

      if (!_isEmpty(keys)) {
        if (e.keyCode === KeyCode.UP) {
          setActiveKey(keys[keys.length - 1]);
        } else {
          setActiveKey(keys[0]);
        }
      }
    }
  }, [activeKey, allOptions, backfill, filteredOptions, getNextAvailableKey, handleDeselect, handlePopupVisibleChange, handleSelect, inputValue, isMultiple, optionSelectedIndex, popupVisibleState, searchReservedOptions, valueState]);
  /**
   * tags 模式下输入文本并回车
   */


  var handleAddTag = useCallback(function (value, e) {
    // 自动分词下，一次性添加多个 tag
    if (Array.isArray(value)) {
      var tagOptions = _uniq(value).map(function (v) {
        return {
          value: v
        };
      }).filter(function (option) {
        return optionSelectedIndex(option) === -1;
      });

      if (tagOptions.length > 0) {
        setInputValue('');
        setTagsExtraOptions(_extends({}, tagsExtraOptions, tagOptions.reduce(function (prev, option) {
          prev[option.value] = option;
          return prev;
        }, {})));
        handleMultiSelect(tagOptions);
      }
    } else if (value) {
      var tagOption = {
        value: value
      };

      if (optionSelectedIndex(tagOption) === -1) {
        setInputValue('');
        setTagsExtraOptions(_extends({}, tagsExtraOptions, _defineProperty({}, value, tagOption)));
        handleSelect(tagOption, true, e);
      }
    } else {
      handlePopupVisibleChange(false);
    }
  }, [handleMultiSelect, handlePopupVisibleChange, handleSelect, optionSelectedIndex, tagsExtraOptions]);
  useEffect(function () {
    // 单选搜索模式下输入值改变或者搜索项改变后(远程搜索的情况)选择第一项
    if (mode === 'default' && showSearch && defaultActiveFirstOption && popupVisibleState && (inputValue !== preInputValue || !shallowEqualObjects(preFilteredOptions, filteredOptions)) && inputValue) {
      var keys = sortKeys(Object.keys(filteredOptions));

      if (keys.length > 0 && activeKey === '') {
        var newActiveKey = filteredOptions[keys[0]];

        if (newActiveKey !== null && _typeof(newActiveKey) === 'object') {
          setActiveKey(newActiveKey.value);
        } else {
          setActiveKey(newActiveKey);
        }
      }
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [filteredOptions, inputValue]);
  useEffect(function () {
    // 多选的时候如果输入框输入了内容，且 popup 是关闭的，那么打开输入框
    if (mode !== 'default' && !popupVisibleState && inputValue !== preInputValue && inputValue) {
      handlePopupVisibleChange(true);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [inputValue, popupVisibleState]); // options 变的时候，自动缓存未缓存的搜索结果

  useEffect(function () {
    if (showSearch && reservedSearchResult) {
      var newSearchReservedOptions = _extends({}, searchReservedOptions);

      var flag = false;
      valueState.forEach(function (value) {
        if (value && _typeof(value) === 'object') {
          if (!searchReservedOptions[value.value] && allOptions[value.value]) {
            newSearchReservedOptions[value.value] = allOptions[value.value];
            flag = true;
          }
        } else {
          if (!searchReservedOptions[value] && allOptions[value]) {
            newSearchReservedOptions[value] = allOptions[value];
            flag = true;
          }
        }
      });

      if (flag) {
        setSearchReservedOptions(newSearchReservedOptions);
      }
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [options]); // popupVisible 受控的情况

  useEffect(function () {
    if (!popupWidth && nodeRef.current && nodeRef.current.offsetWidth !== menuWidthRef.current) {
      menuWidthRef.current = nodeRef.current.offsetWidth;
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [popupVisibleState]); // props 改变，更新状态

  if (isControlled) {
    var newValueState = getValueFromProps(props);

    if (!shallowEqualArrays(valueState, newValueState)) {
      setValueState(newValueState); // combobox 情况下，关闭的时候如果 value 变了需要同步 inputValue 的清空情况

      if (!popupVisibleState && combobox && (_isEmpty(newValueState) || newValueState.length === 1 && !newValueState[0]) && inputValue) {
        setInputValue('');
      }
    }
  }

  if (isPopupVisibleControlled && popupVisible !== popupVisibleState) {
    setPopupVisibleState(popupVisible);
  }

  return {
    // 状态
    inputValue: inputValue,
    popupVisibleState: popupVisibleState,
    valueState: valueState,
    activeKey: activeKey,
    // 计算值
    options: options,
    filteredOptions: filteredOptions,
    mergedOptions: mergedOptions,
    allOptions: allOptions,
    isMultiple: isMultiple,
    hasValue: hasValue,
    showSearch: showSearch,
    tagsExtraOptions: tagsExtraOptions,
    searchReservedOptions: searchReservedOptions,
    firstSelectedOption: firstSelectedOption,
    combobox: combobox,
    // utils
    optionSelectedIndex: optionSelectedIndex,
    // 弹出
    handlePopupVisibleChange: handlePopupVisibleChange,
    // 选择面板
    handleSelect: handleSelect,
    handleMultiSelect: handleMultiSelect,
    handleDeselect: handleDeselect,
    handleOptionClick: handleOptionClick,
    handlePopupEnter: handlePopupEnter,
    handlePopupExited: handlePopupExited,
    handleOptionMouseEnter: handleOptionMouseEnter,
    // 选择器
    handleClear: handleClear,
    handleInputChange: handleInputChange,
    handleInputKeyDown: handleInputKeyDown,
    handleAddTag: handleAddTag,
    handleSearch: handleSearch,
    menuWidth: menuWidthRef.current
  };
}