import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _toArray from "@babel/runtime/helpers/toArray";
import _typeof from "@babel/runtime/helpers/typeof";
import _get from "lodash/get";
import React from 'react';
export var noop = function noop() {};
/**
 * value 在 select 内部都通过数组保存，因此提供转换函数
 */

export var toArray = function toArray(value) {
  var ret = [];

  if (typeof value === 'undefined') {
    ret = [];
  } else if (!Array.isArray(value)) {
    ret = [value];
  } else {
    ret = value;
  }

  return ret;
};
/**
 * 获取 child 的组件类型，对于函数组件目前通过 displayName 进行判断
 */

export var getChildType = function getChildType(child) {
  if (child.type && child.type.displayName) {
    return child.type.displayName;
  }

  return '';
};
export var isOption = function isOption(type) {
  return type === 'Option';
};
export var isOptionGroup = function isOptionGroup(type) {
  return type === 'OptionGroup';
};
export var isOptionDivider = function isOptionDivider(type) {
  return type === 'OptionDivider';
};
export function getOptionPropValue(child, prop) {
  return child.props[prop];
}
export function isSelectChildNode(child) {
  var type = getChildType(child);
  return isOption(type) || isOptionGroup(type) || isOptionDivider(type);
}
/**
 * 从 props 获取 value
 */

export var getValueFromProps = function getValueFromProps(props) {
  var value = [];

  if ('value' in props) {
    value = toArray(props.value);
  } else if ('defaultValue' in props) {
    value = toArray(props.defaultValue);
  }

  return value;
};
export var getActiveKeyFromValue = function getActiveKeyFromValue(valueState) {
  var value = _get(valueState, '0', '');

  if (value && _typeof(value) === 'object') {
    return value.value;
  } else {
    return value;
  }
};
/**
 * 从 children 中读取 option 节点
 */

export var getOptionsFromChildren = function getOptionsFromChildren(children) {
  var optionInfos = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var keys = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
  React.Children.forEach(children, function (child) {
    if (!child) {
      return;
    }

    var type = getChildType(child);

    if (isOptionGroup(type) || !isOption(type) && child.props && Array.isArray(child.props.children)) {
      getOptionsFromChildren(child.props.children, optionInfos, keys);
    } else if (isOption(type)) {
      var value = getOptionPropValue(child, 'value');
      optionInfos[value] = {
        value: value,
        label: getOptionPropValue(child, 'label') || getOptionPropValue(child, 'children'),
        disabled: child.props.disabled,
        readOnly: child.props.readOnly
      };
      keys.push(value);
    }
  });
  return {
    options: optionInfos,
    optionKeysOrder: keys.reduce(function (ans, cur, index) {
      ans[cur] = index;
      return ans;
    }, {})
  };
};
export function getSeparatedContent(text, tokens) {
  if (!tokens || !tokens.length) {
    return null;
  }

  var match = false;

  function separate(str, _ref) {
    var _ref2 = _toArray(_ref),
        token = _ref2[0],
        restTokens = _ref2.slice(1);

    if (!token) {
      return [str];
    }

    var list = str.split(token);
    match = match || list.length > 1;
    return list.reduce(function (prevList, unitStr) {
      return [].concat(_toConsumableArray(prevList), _toConsumableArray(separate(unitStr, restTokens)));
    }, []).filter(function (unit) {
      return unit;
    });
  }

  var list = separate(text, tokens);
  return match ? list : null;
}