import styled, { css } from 'styled-components';
import { skeletonActiveCss, skeletonNormalCss } from '../styled/mixins/skeleton';
export var StyledBaseSkeletonItem = styled.div.withConfig({
  displayName: "StyledBaseSkeletonItem"
})(["", ";"], function (props) {
  var theme = props.theme;
  var activeStyle = props.active ? skeletonActiveCss({
    theme: theme
  }) : skeletonNormalCss({
    theme: theme
  });
  return css(["", ""], activeStyle);
}); // block

export var StyledBlock = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledBlock"
})(["width:100%;height:100%;"]); // card

export var StyledCardWrapper = styled.div.withConfig({
  displayName: "StyledCardWrapper"
})(["display:inline-flex;flex-flow:column nowrap;align-items:flex-start;"]);
export var StyledCardTitle = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledCardTitle"
})(["width:", ";height:", "px;margin-top:", "px;"], function (props) {
  return props.theme.components.Skeleton.card.titleWidth;
}, function (props) {
  return props.theme.components.Skeleton.card.titleHeight;
}, function (props) {
  return props.theme.spacing.spec.s5;
});
export var StyledCardDesc = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledCardDesc"
})(["width:", ";height:", "px;margin-top:", "px;"], function (props) {
  return props.theme.components.Skeleton.card.descWidth;
}, function (props) {
  return props.theme.components.Skeleton.card.descHeight;
}, function (props) {
  return props.theme.spacing.spec.s5;
});
export var StyledCardContent = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledCardContent"
})(["flex:1;align-self:stretch;"]); // paragraph

export var StyledParagraphWrapper = styled.div.withConfig({
  displayName: "StyledParagraphWrapper"
})(["display:flex;flex-flow:column nowrap;align-items:flex-start;"]);
export var StyledParagraphTitle = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledParagraphTitle"
})(["height:", "px;margin-bottom:", "px;width:", "px;"], function (props) {
  return props.theme.components.Skeleton.paragraph.height;
}, function (props) {
  return props.theme.components.Skeleton.paragraph.titleMarginBottom;
}, function (props) {
  return props.theme.components.Skeleton.paragraph.titleWidth;
});
export var StyledParagraph = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledParagraph"
})(["height:", "px;margin-top:", "px;width:100%;"], function (props) {
  return props.theme.components.Skeleton.paragraph.height;
}, function (props) {
  return props.theme.components.Skeleton.paragraph.marginTop;
}); // tree

export var StyledTreeSquare = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledTreeSquare"
})(["height:", "px;width:", "px;margin-right:", "px;"], function (props) {
  return props.theme.components.Skeleton.tree.squareSize;
}, function (props) {
  return props.theme.components.Skeleton.tree.squareSize;
}, function (props) {
  return props.theme.components.Skeleton.tree.squareMarginRight;
});
export var StyledTreeContent = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledTreeContent"
})(["flex:1;height:", "px;"], function (props) {
  return props.theme.components.Skeleton.tree.contentHeight;
});
export var StyledTreeNode = styled.div.withConfig({
  displayName: "StyledTreeNode"
})(["display:flex;align-items:center;"]);
export var StyledTreeHead = styled(StyledTreeNode).withConfig({
  displayName: "StyledTreeHead"
})(["margin-right:", ";"], function (props) {
  var _props$theme$componen = props.theme.components.Skeleton.tree,
      squareSize = _props$theme$componen.squareSize,
      squareMarginRight = _props$theme$componen.squareMarginRight;
  return "".concat(squareMarginRight + squareSize, "px");
});
export var StyledTreeChild = styled(StyledTreeNode).withConfig({
  displayName: "StyledTreeChild"
})(["margin-left:", ";margin-top:", "px;&:nth-child(2n){", "{transform:scaleX(0.5);transform-origin:left center;}}"], function (props) {
  var _props$theme$componen2 = props.theme.components.Skeleton.tree,
      squareSize = _props$theme$componen2.squareSize,
      squareMarginRight = _props$theme$componen2.squareMarginRight;
  return "".concat(squareMarginRight + squareSize, "px");
}, function (props) {
  return props.theme.components.Skeleton.tree.treeNodeMarginTop;
}, StyledTreeContent);
export var StyledTreeWrapper = styled.div.withConfig({
  displayName: "StyledTreeWrapper"
})(["display:flex;flex-flow:column nowrap;& + &{margin-top:", "px;}"], function (props) {
  return props.theme.components.Skeleton.navigation.navigationNodeMarginTop;
}); // navigation

export var StyledNavigationSquare = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledNavigationSquare"
})(["height:", "px;width:", "px;margin-right:", "px;"], function (props) {
  return props.theme.components.Skeleton.navigation.squareSize;
}, function (props) {
  return props.theme.components.Skeleton.navigation.squareSize;
}, function (props) {
  return props.theme.components.Skeleton.navigation.squareMarginRight;
});
export var StyledNavigationNode = styled(StyledBaseSkeletonItem).withConfig({
  displayName: "StyledNavigationNode"
})(["height:", "px;margin-top:", "px;transform-origin:left center;"], function (props) {
  return props.theme.components.Skeleton.navigation.contentHeight;
}, function (props) {
  return props.theme.components.Skeleton.navigation.navigationNodeMarginTop;
});
export var StyledNavigationWrapper = styled.div.withConfig({
  displayName: "StyledNavigationWrapper"
})(["display:flex;flex-flow:column nowrap;", ""], function (props) {
  var navigationNodeScaleList = props.theme.components.Skeleton.navigation.navigationNodeScaleList;
  return navigationNodeScaleList.map(function (v, i) {
    return css(["& > ", ":nth-of-type(", "n+", "){transform:", ";}"], StyledNavigationNode, navigationNodeScaleList.length, i + 2, "scaleX(".concat(v, ")"));
  });
});
export var StyledNavigationHead = styled.div.withConfig({
  displayName: "StyledNavigationHead"
})(["display:flex;align-items:center;margin-bottom:", "px;", "{margin:0;transform:", ";flex:1;}"], function (props) {
  return props.theme.components.Skeleton.navigation.navigationHeadMarginBottom;
}, StyledNavigationNode, function (props) {
  return "scaleX(".concat(props.theme.components.Skeleton.navigation.navigationHeadScale, ")");
});