import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo } from 'react';
import Tooltip from '../Tooltip';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { StyledBaseRootTrack, StyledCircle, StyledLabelNode, StyledMarkLabel, StyledMarkPoint, StyledMarksRow, StyledRange, StyledRoot } from './styled';
import useBaseSlider from './useBaseSlider';

function defaultTipFormatter(str) {
  return str.toString();
}

var defaultStyles = {
  startCircle: '',
  endCircle: '',
  markPoint: '',
  markPointInclude: '',
  marksRow: '',
  markLabel: '',
  track: '',
  range: ''
};
var BaseSlider = memoForwardRef(function (props, ref) {
  var _useBaseSlider = useBaseSlider(props),
      startCircleStyle = _useBaseSlider.startCircleStyle,
      endCircleStyle = _useBaseSlider.endCircleStyle,
      rangeStyle = _useBaseSlider.rangeStyle,
      start = _useBaseSlider.start,
      end = _useBaseSlider.end,
      innerMarksOnly = _useBaseSlider.innerMarksOnly,
      finalMarks = _useBaseSlider.finalMarks,
      markPropsMap = _useBaseSlider.markPropsMap,
      markNums = _useBaseSlider.markNums,
      movingType = _useBaseSlider.movingType,
      trackRef = _useBaseSlider.trackRef,
      handleStartMoveStartCircle = _useBaseSlider.handleStartMoveStartCircle,
      handleStartMoveEndCircle = _useBaseSlider.handleStartMoveEndCircle,
      handleTrackClick = _useBaseSlider.handleTrackClick,
      handleEnterStart = _useBaseSlider.handleEnterStart,
      handleLeaveStart = _useBaseSlider.handleLeaveStart,
      handleEnterEnd = _useBaseSlider.handleEnterEnd,
      handleLeaveEnd = _useBaseSlider.handleLeaveEnd,
      endTooltipOpen = _useBaseSlider.endTooltipOpen,
      startTooltipOpen = _useBaseSlider.startTooltipOpen,
      hideStart = _useBaseSlider.hideStart,
      vertical = _useBaseSlider.vertical,
      tooltipVisible = _useBaseSlider.tooltipVisible,
      otherProps = _useBaseSlider.otherProps,
      styles = _useBaseSlider.styles,
      disabled = _useBaseSlider.disabled,
      tooltipPlacement = _useBaseSlider.tooltipPlacement,
      markLabelDisabled = _useBaseSlider.markLabelDisabled,
      _useBaseSlider$tipFor = _useBaseSlider.tipFormatter,
      tipFormatter = _useBaseSlider$tipFor === void 0 ? defaultTipFormatter : _useBaseSlider$tipFor;

  var innerStyles = useStyles('slider', defaultStyles, styles);
  var theme = useTheme();
  var memoStartCircleStyle = useMemo(function () {
    if (!innerStyles.startCircle.style) {
      return startCircleStyle;
    }

    return _extends({}, innerStyles.startCircle.style, startCircleStyle);
  }, [innerStyles.startCircle.style, startCircleStyle]);
  var startCircleNode = useMemo(function () {
    return hideStart ? null : /*#__PURE__*/React.createElement(StyledCircle, {
      className: innerStyles.startCircle.className,
      $moving: movingType === 'start',
      theme: theme,
      $disabled: disabled,
      $vertical: vertical,
      style: memoStartCircleStyle,
      onMouseDown: handleStartMoveStartCircle,
      onMouseEnter: handleEnterStart,
      onMouseLeave: handleLeaveStart
    });
  }, [disabled, handleEnterStart, handleLeaveStart, handleStartMoveStartCircle, hideStart, innerStyles.startCircle.className, memoStartCircleStyle, movingType, theme, vertical]);
  var memoEndCircleStyle = useMemo(function () {
    if (!innerStyles.endCircle.style) {
      return endCircleStyle;
    }

    return _extends({}, innerStyles.endCircle.style, endCircleStyle);
  }, [endCircleStyle, innerStyles.endCircle.style]); // 第二个控制点的渲染

  var endCircleNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledCircle, {
      className: innerStyles.endCircle.className,
      $moving: movingType === 'end',
      theme: theme,
      $disabled: disabled,
      $vertical: vertical,
      style: memoEndCircleStyle,
      onMouseDown: handleStartMoveEndCircle,
      onMouseEnter: handleEnterEnd,
      onMouseLeave: handleLeaveEnd
    });
  }, [disabled, handleEnterEnd, handleLeaveEnd, handleStartMoveEndCircle, innerStyles.endCircle.className, memoEndCircleStyle, movingType, theme, vertical]);
  var startNode = startCircleNode;
  var endNode = endCircleNode;

  if (tooltipVisible) {
    startNode = startCircleNode && /*#__PURE__*/React.createElement(Tooltip, {
      title: tipFormatter(start),
      placement: tooltipPlacement,
      open: startTooltipOpen
    }, startCircleNode);
    endNode = /*#__PURE__*/React.createElement(Tooltip, {
      title: tipFormatter(end),
      placement: tooltipPlacement,
      open: endTooltipOpen
    }, endCircleNode);
  }

  var markPointsNode;
  var marksRowNode;

  if (finalMarks) {
    markPointsNode = markNums.map(function (keyNum) {
      var $include = keyNum > start && keyNum < end;
      var markProps = markPropsMap[keyNum];
      var innerStyle = markProps.style;

      if ($include && innerStyles.markPointInclude.style) {
        innerStyle = _extends({}, innerStyles.markPointInclude.style, innerStyle);
      } else if (!$include && innerStyles.markPoint.style) {
        innerStyle = _extends({}, innerStyles.markPoint.style, innerStyle);
      }

      var pointClassName = $include ? innerStyles.markPointInclude.className : innerStyles.markPoint.className;
      return /*#__PURE__*/React.createElement(StyledMarkPoint, {
        className: pointClassName,
        theme: theme,
        $include: $include,
        $disabled: disabled,
        key: keyNum,
        $vertical: vertical,
        style: innerStyle,
        onClick: markProps.onClick
      });
    });
  }

  var markLabelPropsMap = useMemo(function () {
    var map = {};

    if (!finalMarks) {
      return map;
    }

    markNums.forEach(function (keyNum) {
      var label = finalMarks[keyNum];
      var markItem = label;
      var markProps = markPropsMap[keyNum];
      var markStyle = markProps.style;
      var markLabel = label;

      if (markItem.label) {
        markLabel = markItem.label;
      }

      if (markItem.style) {
        markStyle = _extends({}, markItem.style, markStyle);
      }

      if (innerStyles.markLabel.style) {
        markStyle = _extends({}, innerStyles.markLabel.style, markStyle);
      }

      map[keyNum] = {
        markProps: {
          className: innerStyles.markLabel.className,
          style: markStyle,
          onClick: markProps.onClick
        },
        markLabel: markLabel
      };
    });
    return map;
  }, [finalMarks, innerStyles.markLabel.className, innerStyles.markLabel.style, markNums, markPropsMap]);

  if (finalMarks && !markLabelDisabled) {
    marksRowNode = /*#__PURE__*/React.createElement(StyledMarksRow, {
      theme: theme,
      $vertical: vertical
    }, markNums.map(function (keyNum) {
      var _markLabelPropsMap$ke = markLabelPropsMap[keyNum],
          markProps = _markLabelPropsMap$ke.markProps,
          markLabel = _markLabelPropsMap$ke.markLabel;
      var labelNode;

      if ( /*#__PURE__*/React.isValidElement(markLabel)) {
        labelNode = markLabel;
      } else {
        labelNode = /*#__PURE__*/React.createElement(StyledLabelNode, {
          theme: theme,
          size: "s",
          fontWeight: "lighter"
        }, markLabel);
      }

      return /*#__PURE__*/React.createElement(StyledMarkLabel, _extends({
        theme: theme,
        $disabled: disabled,
        key: keyNum,
        $vertical: vertical
      }, markProps), labelNode);
    }));
  }

  var rangeStyleItem = useMemo(function () {
    return mergeStyleItem({
      style: rangeStyle
    }, innerStyles.range);
  }, [innerStyles.range, rangeStyle]);
  return /*#__PURE__*/React.createElement(StyledRoot, _extends({
    theme: theme,
    $disabled: disabled,
    $vertical: vertical,
    $moving: movingType !== 'none',
    $disableClick: innerMarksOnly
  }, otherProps, {
    ref: ref
  }), /*#__PURE__*/React.createElement(StyledBaseRootTrack, _extends({}, innerStyles.track, {
    theme: theme,
    $disabled: disabled,
    $disableClick: innerMarksOnly,
    $moving: movingType !== 'none',
    $vertical: vertical,
    ref: trackRef,
    onClick: handleTrackClick
  }), startNode, endNode, markPointsNode, /*#__PURE__*/React.createElement(StyledRange, _extends({
    $moving: movingType !== 'none',
    theme: theme,
    $disabled: disabled,
    $vertical: vertical,
    $disableClick: innerMarksOnly
  }, rangeStyleItem))), marksRowNode);
});
export default BaseSlider;