import React from 'react';
import styled, { css } from 'styled-components';
import InlineButton from '../InlineButton';

function cursorCss(props) {
  if (props.$disabled) {
    return css(["cursor:not-allowed;"]);
  }

  if (props.$moving) {
    return css(["cursor:grabbing;"]);
  }

  if (props.$disableClick) {
    return;
  }

  return css(["cursor:pointer;"]);
}

export var StyledRoot = styled.div.withConfig({
  displayName: "StyledRoot"
})(["position:relative;box-sizing:border-box;", " ", ""], cursorCss, function (props) {
  var _props$theme = props.theme,
      token = _props$theme.components.Slider,
      opacity = _props$theme.opacity;
  var hoverCss;

  if (!props.$disabled) {
    hoverCss = css(["&:hover ", "{background-color:", ";}"], StyledBaseRootTrackInner, token.trackHoverBgColor);
  }

  if (props.$vertical) {
    return css(["display:flex;padding:0 ", "px;", ";"], token.rootPadding, hoverCss);
  }

  return css(["padding:", "px 0;", ";", ";"], token.rootPadding, props.$disabled && "opacity: ".concat(opacity.pattern.disabled), hoverCss);
});
export var StyledRange = styled.div.withConfig({
  displayName: "StyledRange"
})(["position:absolute;", " ", ""], cursorCss, function (props) {
  var _props$theme2 = props.theme,
      token = _props$theme2.components.Slider,
      borderRadius = _props$theme2.size.spec.borderRadius;
  var directionCss;

  if (props.$vertical) {
    directionCss = css(["width:100%;"]);
  } else {
    directionCss = css(["height:100%;"]);
  }

  return css(["border-radius:", ";background-color:", ";z-index:1;", ""], borderRadius.s5, token.trackActiveBgColor, directionCss);
}); // 放 marks

var StyledMarkPointInner = styled.div.withConfig({
  displayName: "StyledMarkPointInner"
})(["position:absolute;box-sizing:border-box;", " ", ""], cursorCss, function (props) {
  var _props$theme3 = props.theme,
      token = _props$theme3.components.Slider,
      borderRadius = _props$theme3.size.spec.borderRadius;
  var directionCss;

  if (props.$vertical) {
    directionCss = css(["left:50%;"]);
  } else {
    directionCss = css(["top:50%;"]);
  }

  var bgColor = token.markPointColor;
  var borderColor = token.markPointBorderColor;

  if (props.$include) {
    borderColor = token.includeMarkPointBorderColor;
    bgColor = token.includeMarkPointColor;
  }

  return css(["z-index:2;height:", "px;width:", "px;display:flex;align-items:center;justify-content:center;transform:translate(-50%,-50%);", " &::before{content:'';display:block;box-sizing:border-box;height:", "px;width:", "px;border:", "px solid ", ";border-radius:", ";background-color:", ";}&:hover{&::before{border-color:", ";}}"], token.circleSize, token.circleSize, directionCss, token.markPointSize, token.markPointSize, token.markPointBorderSize, borderColor, borderRadius.s5, bgColor, token.includeMarkPointColor);
});
export var StyledMarkPoint = /*#__PURE__*/React.memo(StyledMarkPointInner);
var StyledMarksRowInner = styled.div.withConfig({
  displayName: "StyledMarksRowInner"
})(["position:relative;", ""], function (props) {
  var token = props.theme.components.Slider;

  if (props.$vertical) {
    return css(["height:100%;flex:1;margin-left:", "px;"], token.markRowGutter);
  }

  return css(["width:100%;margin-top:", "px;height:20px;"], token.markRowGutter);
});
export var StyledMarksRow = /*#__PURE__*/React.memo(StyledMarksRowInner);
export var StyledLabelNode = styled(InlineButton).withConfig({
  displayName: "StyledLabelNode"
})(["", ""], function (props) {
  return css(["font-size:", "px;line-height:", "px;"], props.theme.components.Slider.labelNodeFontSize, props.theme.components.Slider.labelNodeLineHeight);
});
var StyledMarkLabelInner = styled.div.withConfig({
  displayName: "StyledMarkLabelInner"
})(["position:absolute;transform:translate(-50%,0);", " ", ""], cursorCss, function (props) {
  if (props.$vertical) {
    return css(["left:0;transform:translate(0,-50%);"]);
  }

  return css(["transform:translate(-50%,0);top:0;"]);
});
export var StyledMarkLabel = /*#__PURE__*/React.memo(StyledMarkLabelInner);
var StyledCircleInner = styled.div.withConfig({
  displayName: "StyledCircleInner"
})(["position:absolute;z-index:3;box-sizing:content-box;", " ", ""], cursorCss, function (props) {
  var token = props.theme.components.Slider;
  var borderColor = token.circleBorderColor;
  var hoverCss;
  var activeCss = css(["width:", "px;height:", "px;box-shadow:", ";&::before{border-color:", ";}"], token.circleActiveSize, token.circleActiveSize, token.circleActiveBoxShadow, token.circleBorderHoverColor);

  if (!props.$disabled && !props.$moving) {
    hoverCss = css(["&:hover{cursor:grab;", "}"], activeCss);
  } else if (!props.$disabled && props.$moving) {
    hoverCss = css(["cursor:grabbing;padding:", "px;background-color:", ";", ""], token.circleActiveBorderSize, token.circleActiveBorderColor, activeCss);
  }

  var directionCss;

  if (props.$vertical) {
    directionCss = css(["left:50%;transform:translate(-50%,-50%);"]);
  } else {
    directionCss = css(["transform:translate(-50%,-50%);top:50%;"]);
  }

  return css(["width:", "px;height:", "px;border-radius:50%;box-shadow:", ";&::before{content:'';height:100%;width:100%;display:block;box-sizing:border-box;border-radius:50%;background-color:", ";border:", "px solid ", ";}", " ", ""], token.circleSize, token.circleSize, token.circleBoxShadow, token.circleColor, token.circleBorderWidth, borderColor, hoverCss, directionCss);
});
export var StyledCircle = /*#__PURE__*/React.memo(StyledCircleInner); // 基础轨道

var StyledBaseRootTrackInner = styled.div.withConfig({
  displayName: "StyledBaseRootTrackInner"
})(["position:relative;box-sizing:border-box;user-select:none;", " ", ""], cursorCss, function (props) {
  var _props$theme4 = props.theme,
      token = _props$theme4.components.Slider,
      borderRadius = _props$theme4.size.spec.borderRadius;
  var directionCss;

  if (props.$vertical) {
    directionCss = css(["width:", "px;height:100%;"], token.trackHeight);
  } else {
    directionCss = css(["width:100%;height:", "px;"], token.trackHeight);
  }

  return css(["background-color:", ";border-radius:", ";", ""], token.trackBgColor, borderRadius.s5, directionCss);
});
export var StyledBaseRootTrack = /*#__PURE__*/React.memo(StyledBaseRootTrackInner);