import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["direction", "spacing", "block", "styles", "children", "style", "className"];
import React, { Children, forwardRef, useMemo } from 'react';
import styled, { css } from 'styled-components';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
var StyledWrapper = styled.div.withConfig({
  displayName: "StyledWrapper"
})(["", ""], function (props) {
  var $direction = props.$direction,
      $block = props.$block;
  var directionStyle = $direction === 'horizontal' ? css(["align-items:center;"]) : css(["flex-direction:column;"]);
  var displayStyle = $block ? css(["display:flex;"]) : css(["display:inline-flex;"]);
  return css(["", " ", ""], displayStyle, directionStyle);
});
var StyledItem = styled.div.withConfig({
  displayName: "StyledItem"
})(["", ""], function (props) {
  var $direction = props.$direction,
      $spacing = props.$spacing;
  if (!$spacing) return;
  var marginStyle = $direction === 'horizontal' ? css(["margin-right:", "px;"], $spacing) : css(["margin-bottom:", "px;"], $spacing);
  return css(["", ""], marginStyle);
});
var defaultStyles = {
  wrapper: '',
  item: ''
};
export default /*#__PURE__*/React.memo( /*#__PURE__*/forwardRef(function (props, ref) {
  var _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'horizontal' : _props$direction,
      _props$spacing = props.spacing,
      spacing = _props$spacing === void 0 ? 's4' : _props$spacing,
      _props$block = props.block,
      block = _props$block === void 0 ? false : _props$block,
      styles = props.styles,
      children = props.children,
      style = props.style,
      className = props.className,
      other = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('space', defaultStyles, styles);
  var wrapperStyleItem = mergeStyleItem({
    style: style,
    className: className
  }, innerStyles.wrapper);

  var isValidNode = function isValidNode(node) {
    return node !== null && node !== undefined && typeof node !== 'boolean';
  };

  var validChildren = React.Children.toArray(children).filter(function (child) {
    return isValidNode(child);
  });
  var count = validChildren.length;
  var items = useMemo(function () {
    return Children.map(validChildren, function (child, index) {
      var spacingNumber = typeof spacing === 'number' ? spacing : theme.spacing.spec[spacing];
      return /*#__PURE__*/React.createElement(StyledItem, _extends({
        $direction: direction,
        $spacing: index === count - 1 ? undefined : spacingNumber
      }, innerStyles.item), child);
    });
  }, [count, direction, innerStyles.item, spacing, theme.spacing.spec, validChildren]);

  if (count === 0) {
    return null;
  }

  return /*#__PURE__*/React.createElement(StyledWrapper, _extends({
    ref: ref,
    $direction: direction,
    $block: block
  }, wrapperStyleItem, other), items);
}));