import { useEffectOnce as _useEffectOnce } from "@muya-ui/utils/es/hooks/useEffectOnce";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useCallback, useMemo, useState } from 'react';
import { getElementTop, getScroller, getScrollTop, isHidden } from './utils';
export default function useSticky(props, divEl) {
  var offsetTop = props.offsetTop,
      target = props.target,
      onStatusChange = props.onStatusChange;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      fixed = _useState2[0],
      setFixed = _useState2[1];

  var _useState3 = useState(0),
      _useState4 = _slicedToArray(_useState3, 2),
      transform = _useState4[0],
      setTransform = _useState4[1];

  var _useState5 = useState(false),
      _useState6 = _slicedToArray(_useState5, 2),
      scrollTick = _useState6[0],
      setScrollTick = _useState6[1];

  var _useState7 = useState(false),
      _useState8 = _slicedToArray(_useState7, 2),
      targetScrollTick = _useState8[0],
      setTargetScrollTick = _useState8[1];

  var handleChange = useCallback(function (status) {
    if (onStatusChange && fixed !== status) {
      onStatusChange(status);
    }
  }, [fixed, onStatusChange]);
  var calculate = useCallback(function (top, target, current) {
    var transform = target.getBoundingClientRect().top;
    var distance = transform - current.getBoundingClientRect().top;
    var originalPosition = Math.abs(distance - getScrollTop(target));
    var scrollTop = getScrollTop(target);

    if (scrollTop === 0) {
      setFixed(false);
      handleChange(false);
      return;
    }

    if (originalPosition - scrollTop <= top) {
      setTransform(transform);
    } else {
      setTransform(transform + originalPosition - scrollTop - top);
    }

    setFixed(true);
    handleChange(true);
  }, [handleChange]);

  var onScroll = _useEventCallback(function () {
    setScrollTick(false);
    var current = divEl.current;
    if (!current) return;

    if (isHidden(current)) {
      return;
    }

    var top = Number(offsetTop);
    var scrollTop = getScrollTop(window);
    var topToPageTop = getElementTop(current);

    if (target) {
      if (!fixed) {
        return;
      }

      calculate(top, target(), current);
      return;
    }

    if (scrollTop + top > topToPageTop) {
      setFixed(true);
      handleChange(true);
      setTransform(0);
    } else {
      setFixed(false);
      handleChange(false);
    }
  }, [divEl, target, fixed, offsetTop]);

  var handleScroll = useCallback(function () {
    !scrollTick && requestAnimationFrame(onScroll);
    setScrollTick(true);
  }, [onScroll, scrollTick]);
  var onTargetScroll = useCallback(function () {
    setTargetScrollTick(false);
    var current = divEl.current;
    if (!current || !target) return;

    if (isHidden(current)) {
      return;
    }

    var top = Number(offsetTop);
    calculate(top, target(), current);
  }, [divEl, target, offsetTop, calculate]);
  var handleTargetScroll = useCallback(function () {
    !targetScrollTick && requestAnimationFrame(onTargetScroll);
    setTargetScrollTick(true);
  }, [onTargetScroll, targetScrollTick]);

  _useEffectOnce(function () {
    var current = divEl.current;
    var scroller = window;

    if (current) {
      scroller = getScroller(current);

      if (target) {
        window.addEventListener('scroll', handleScroll);
        target().addEventListener('scroll', handleTargetScroll, true);
      } else {
        scroller.addEventListener('scroll', handleScroll);
      }
    }

    return function () {
      if (target) {
        window.removeEventListener('scroll', handleScroll);
        target().removeEventListener('scroll', handleTargetScroll, true);
      }

      scroller.removeEventListener('scroll', handleScroll);
    };
  });

  var style = useMemo(function () {
    if (!fixed) {
      return;
    }

    var style = {};

    if (Number(offsetTop)) {
      style.top = "".concat(offsetTop, "px");
    }

    if (transform) {
      style.transform = "translate3d(0, ".concat(transform, "px, 0)");
      style.WebkitTransform = "translate3d(0, ".concat(transform, "px, 0)");
      style.willChange = 'transform';
    }

    return style;
  }, [fixed, offsetTop, transform]);
  return {
    style: style,
    fixed: fixed,
    divEl: divEl
  };
}