import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
export function getItemOffsets(items, prevKey, panelStartPoint) {
  return items.map(function (item) {
    var itemPrev = item[prevKey];
    var offset = itemPrev - panelStartPoint;
    return offset;
  });
}
/**
 * 根据offset找到数组中的 index
 * @param offset
 * @param offsets 一定是正数数组，并是从小到大排列的，由 calcStepsByPageOneInCloneItems 产生
 */

export function findIndexByOffset(offset, offsets) {
  for (var l = offsets.length, i = l - 1; i >= 0; i--) {
    var currentOffset = offsets[i];

    if (currentOffset <= offset) {
      return i;
    }
  }

  return 0;
}
export function findFirstItemIndexInPageTwo(itemOffsets, containerSize) {
  for (var i = 0, l = itemOffsets.length; i < l; i++) {
    var offset = itemOffsets[i];

    if (offset > containerSize) {
      return i;
    }
  }

  return 0;
}
export function generateDisabledLoopSwipeManager() {
  return {
    get index() {
      return 0;
    },

    get itemIndex() {
      return 0;
    },

    get realItemIndex() {
      return 0;
    },

    get size() {
      return 0;
    },

    get itemSize() {
      return 0;
    },

    get realItemSize() {
      return 0;
    },

    get offset() {
      return 0;
    },

    get direction() {
      return 'horizontal';
    },

    get locked() {
      return true;
    },

    lock: function lock() {},
    unlock: function unlock() {},
    prevItem: function prevItem() {},
    nextItem: function nextItem() {},
    goToItem: function goToItem() {},
    next: function next() {},
    prev: function prev() {},
    goTo: function goTo() {},
    hasStep: function hasStep() {
      return false;
    },
    hasItem: function hasItem() {
      return false;
    },
    getStepOffset: function getStepOffset() {
      return 0;
    },
    getItemOffset: function getItemOffset() {
      return 0;
    },
    updateOffset: function updateOffset() {}
  };
}
export default function generateLoopSwipeManager(args) {
  var items = args.items,
      cloneItemRects = args.cloneItemRects,
      containerSize = args.containerSize,
      direction = args.direction,
      innerGutter = args.gutter,
      stepIndex = args.stepIndex,
      itemIndex = args.itemIndex,
      _args$defaultIndex = args.defaultIndex,
      defaultIndex = _args$defaultIndex === void 0 ? 0 : _args$defaultIndex;
  var isH = direction === 'horizontal';
  var prevKey = isH ? 'left' : 'top';
  var nextKey = isH ? 'right' : 'bottom';
  var panelStartPoint = items[0][prevKey];
  var itemOffsets = getItemOffsets(items, prevKey, panelStartPoint);
  var cloneItemOffsets = getItemOffsets(cloneItemRects, prevKey, panelStartPoint); // 公共的状态

  var currentOffset = 0;
  var locked = false;
  var stepContainerSize = containerSize + innerGutter; // item 的状态

  var itemSize = items.length;
  var currentItemIndex = itemIndex > 0 ? itemIndex : defaultIndex;
  var realItemIndex = currentItemIndex;
  var prevRealItemIndex = currentItemIndex;
  var contentSize = cloneItemOffsets[0];
  var totalOffset = cloneItemRects[cloneItemRects.length - 1][nextKey] - panelStartPoint;
  var realItemOffsets = [].concat(_toConsumableArray(itemOffsets), _toConsumableArray(cloneItemOffsets));

  var getCurrentItemIndexByRealItemIndex = function getCurrentItemIndexByRealItemIndex(inputIndex) {
    if (inputIndex >= itemOffsets.length) {
      return inputIndex - itemOffsets.length;
    }

    return inputIndex;
  };

  var getRealItemIndexByCurrentItemIndex = function getRealItemIndexByCurrentItemIndex(inputIndex) {
    if (inputIndex > 1) {
      return inputIndex;
    }

    var indexA = inputIndex;
    var indexB = inputIndex + itemSize;
    var prevOffset = realItemOffsets[prevRealItemIndex];
    var offsetA = realItemOffsets[indexA];
    var offsetB = realItemOffsets[indexB];

    if (Math.abs(offsetA - prevOffset) < Math.abs(offsetB - prevOffset)) {
      return indexA;
    }

    return indexB;
  };

  if (currentItemIndex && currentItemIndex >= 1) {
    realItemIndex = currentItemIndex;
  } else if (currentItemIndex === 0) {
    realItemIndex = itemSize;
  }

  currentItemIndex = getCurrentItemIndexByRealItemIndex(realItemIndex);
  prevRealItemIndex = realItemIndex;

  var checkLockedByCurrentItemIndex = function checkLockedByCurrentItemIndex(inputIndex) {
    var innerRealItemIndex = getRealItemIndexByCurrentItemIndex(inputIndex);

    if (innerRealItemIndex <= 0) {
      locked = true;
    } else if (innerRealItemIndex > itemSize) {
      locked = true;
    }
  }; // 分页数量


  var stepSize = Math.ceil(contentSize / stepContainerSize); // 当前的页码

  var currentIndex = stepIndex && stepIndex > 0 ? stepIndex : defaultIndex; // 因为循环的原因，第一页所在的位置实际是会变更的

  if (itemOffsets[currentIndex] < stepContainerSize) {
    currentOffset = cloneItemOffsets[currentIndex];
  } else {
    currentOffset = itemOffsets[currentIndex];
  } // 当前基准偏移


  var deltaOffset = currentOffset;

  while (deltaOffset >= contentSize && deltaOffset > 0) {
    deltaOffset -= contentSize;
  }

  if (deltaOffset > 0) {
    var ceilNum = Math.ceil(deltaOffset / stepContainerSize);
    var floorNum = Math.floor(deltaOffset / stepContainerSize);
    currentIndex = ceilNum === floorNum ? ceilNum : ceilNum - 1;
  }

  var getValidDeltaOffset = function getValidDeltaOffset() {
    var offset = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : deltaOffset;

    if (offset < stepContainerSize) {
      return contentSize + offset;
    } else {
      return offset;
    }
  };

  var checkBoundary = function checkBoundary() {
    return currentOffset - stepContainerSize < 0 || currentOffset + stepContainerSize * 2 > totalOffset;
  };

  var checkLocked = function checkLocked() {
    var outOfBound = checkBoundary();

    if (outOfBound) {
      locked = true;
    }
  };

  return {
    get index() {
      return currentIndex;
    },

    get itemIndex() {
      return currentItemIndex;
    },

    get realItemIndex() {
      return currentItemIndex;
    },

    get size() {
      return stepSize;
    },

    get itemSize() {
      return itemSize;
    },

    get realItemSize() {
      return itemSize;
    },

    get offset() {
      return currentOffset;
    },

    get direction() {
      return direction;
    },

    get locked() {
      return locked;
    },

    get itemOffsets() {
      return realItemOffsets;
    },

    getCurrentItemIndexByRealItemIndex: getCurrentItemIndexByRealItemIndex,
    getRealItemIndexByCurrentItemIndex: getRealItemIndexByCurrentItemIndex,
    lock: function lock() {
      locked = true;
    },
    unlock: function unlock() {
      locked = false;
      var outOfBound = checkBoundary();

      if (outOfBound) {
        var newDeltaOffset = currentOffset;

        while (newDeltaOffset >= contentSize) {
          newDeltaOffset -= contentSize;
        }

        deltaOffset = newDeltaOffset;
        currentOffset = getValidDeltaOffset();
      }

      var innerRealItemIndex = getRealItemIndexByCurrentItemIndex(currentItemIndex);

      if (innerRealItemIndex === 0) {
        prevRealItemIndex = itemSize;
      } else if (innerRealItemIndex >= itemSize) {
        prevRealItemIndex = innerRealItemIndex - itemSize;
      }
    },
    prevItem: function prevItem() {
      if (locked) return;
      prevRealItemIndex = getRealItemIndexByCurrentItemIndex(currentItemIndex);
      currentItemIndex = currentItemIndex - 1;

      if (currentItemIndex < 0) {
        currentItemIndex = itemOffsets.length - 1;
      }

      checkLockedByCurrentItemIndex(currentItemIndex);
    },
    nextItem: function nextItem() {
      if (locked) return;
      prevRealItemIndex = getRealItemIndexByCurrentItemIndex(currentItemIndex);
      currentItemIndex = currentItemIndex + 1;

      if (currentItemIndex >= itemOffsets.length) {
        currentItemIndex = 0;
      }

      checkLockedByCurrentItemIndex(currentItemIndex);
    },
    goToItem: function goToItem(itemIndex) {
      if (locked) return;

      if (this.hasItem(itemIndex)) {
        prevRealItemIndex = getRealItemIndexByCurrentItemIndex(currentItemIndex);
        currentItemIndex = itemIndex;
        checkLockedByCurrentItemIndex(itemIndex);
      }
    },
    next: function next() {
      if (locked) return;
      currentIndex = (currentIndex + 1) % stepSize;
      currentOffset = currentOffset + stepContainerSize;
      checkLocked();
    },
    prev: function prev() {
      if (locked) return;

      if (currentIndex === 0) {
        currentIndex = stepSize - 1;
      } else {
        currentIndex = (currentIndex - 1) % stepSize;
      }

      currentOffset = currentOffset - stepContainerSize;
      checkLocked();
    },
    goTo: function goTo(stepIndex) {
      if (locked) return;

      if (this.hasStep(stepIndex)) {
        currentOffset = this.getStepOffset(stepIndex);
        currentIndex = stepIndex;
        checkLocked();
      }
    },
    hasStep: function hasStep(stepIndex) {
      return stepIndex >= 0 && stepIndex < stepSize;
    },
    hasItem: function hasItem(itemIndex) {
      return itemIndex >= 0 && itemIndex < itemSize;
    },
    getStepOffset: function getStepOffset(stepIndex) {
      if (stepIndex !== currentIndex) {
        // FIXME: 两条循环的时候 0 -> 1 的方向无法判断
        var dir = (stepIndex >= currentIndex || currentIndex === stepSize - 1 && stepIndex === 0) && !(currentIndex === 0 && stepIndex === stepSize - 1) ? 'asc' : 'desc';

        while (currentIndex !== stepIndex) {
          if (dir === 'asc') {
            currentOffset += containerSize;

            if (currentIndex === stepSize - 1) {
              currentIndex = 0;
            } else {
              currentIndex++;
            }
          } else {
            currentOffset -= containerSize;

            if (currentIndex === 0) {
              currentIndex = stepSize - 1;
            } else {
              currentIndex--;
            }
          }
        }

        checkLocked();
      }

      return currentOffset;
    },
    getItemOffset: function getItemOffset(inputItemIndex) {
      var newRealItemIndex = getRealItemIndexByCurrentItemIndex(inputItemIndex);
      currentItemIndex = inputItemIndex;
      prevRealItemIndex = newRealItemIndex;
      checkLockedByCurrentItemIndex(currentItemIndex);
      currentOffset = realItemOffsets[newRealItemIndex];
      return currentOffset;
    },
    updateOffset: function updateOffset(offset) {
      currentOffset = offset;
    }
  };
}