import { useDebounce as _useDebounce3 } from "@muya-ui/utils/es/hooks/useDebounce";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useForceUpdate as _useForceUpdate } from "@muya-ui/utils/es/hooks/useForceUpdate";
import { useEffect, useMemo, useRef } from 'react';
import generateLoopSwipeManager, { generateDisabledLoopSwipeManager } from './generateLoopSwipeManager';
import useBaseSwipe from './useBaseSwipe';
export default function useInnerLoopSwipe(props, containerRef) {
  var _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'horizontal' : _props$direction,
      _props$stepIndex = props.stepIndex,
      stepIndex = _props$stepIndex === void 0 ? -1 : _props$stepIndex,
      _props$itemIndex = props.itemIndex,
      itemIndex = _props$itemIndex === void 0 ? -1 : _props$itemIndex,
      equalNum = props.equalNum,
      onStepsChange = props.onStepsChange,
      gutter = props.gutter,
      _props$defaultIndex = props.defaultIndex,
      defaultIndex = _props$defaultIndex === void 0 ? 0 : _props$defaultIndex,
      children = props.children,
      transitionStatus = props.transitionStatus,
      enableDiffChildren = props.enableDiffChildren;
  var swipeManager = useRef();
  var isH = direction === 'horizontal';
  var innerGutter = gutter || 0;

  var forceUpdate = _useForceUpdate();

  var isForceUpdate = useRef(false);

  var forceRefreshSwipe = function forceRefreshSwipe() {
    isForceUpdate.current = true;
    forceUpdate();
  };

  var endRectChange = function endRectChange() {
    if (onStepsChange && swipeManager.current) {
      onStepsChange(swipeManager.current);
    }

    forceRefreshSwipe();
  };

  var handleRectChange = function handleRectChange(itemRects, containerRect, cloneItemRects) {
    // 如果没有 clone 的项，表面当前的面板是无法进行循环的
    // 不需要做任何处理
    if (cloneItemRects.length <= 0) {
      swipeManager.current = generateDisabledLoopSwipeManager();
      endRectChange();
      return;
    }

    var containerSize = isH ? containerRect.width : containerRect.height;
    swipeManager.current = generateLoopSwipeManager({
      items: itemRects,
      cloneItemRects: cloneItemRects,
      containerSize: containerSize,
      direction: direction,
      gutter: innerGutter,
      stepIndex: stepIndex,
      itemIndex: itemIndex,
      defaultIndex: defaultIndex
    });
    endRectChange();
  };

  var _useBaseSwipe = useBaseSwipe({
    loop: true,
    children: children,
    direction: direction,
    equalNum: equalNum,
    gutter: gutter,
    containerRef: containerRef,
    onRectChange: handleRectChange,
    enableDiffChildren: enableDiffChildren
  }),
      nodes = _useBaseSwipe.nodes,
      finalDuration = _useBaseSwipe.finalDuration;

  var innerOffset = 0;
  var innerTransitionStatus = transitionStatus || 'on';

  if (swipeManager.current && itemIndex >= 0) {
    innerOffset = swipeManager.current.getItemOffset(itemIndex);
  } else if (swipeManager.current && stepIndex >= 0) {
    innerOffset = swipeManager.current.getStepOffset(stepIndex);
  } // 如果移到了临界点，需要解除锁定，并forceUpdate


  var resetPosition = function resetPosition() {
    if (!swipeManager.current) {
      return;
    }

    if (swipeManager.current.locked) {
      swipeManager.current.unlock();
      forceRefreshSwipe();
    }
  };

  if (isForceUpdate.current) {
    innerTransitionStatus = 'off';
  }

  var _useDebounce = _useDebounce3(resetPosition, finalDuration),
      _useDebounce2 = _slicedToArray(_useDebounce, 1),
      debounceResetPosition = _useDebounce2[0];

  useEffect(function () {
    debounceResetPosition();
  });
  isForceUpdate.current = false;
  var position = isH ? "".concat(-innerOffset, "px, 0") : "0, ".concat(-innerOffset, "px");
  var panelStyle = useMemo(function () {
    return {
      transform: "translate(".concat(position, ")")
    };
  }, [position]);
  return {
    finalDuration: finalDuration,
    manager: swipeManager,
    panelStyle: panelStyle,
    nodes: nodes,
    transitionStatus: innerTransitionStatus,
    // 只是为了测试而返回的一些操作
    handleRectChange: handleRectChange,
    swipeManager: swipeManager,
    resetPosition: resetPosition
  };
}