import { useForceUpdate as _useForceUpdate } from "@muya-ui/utils/es/hooks/useForceUpdate";
import { useMemo, useRef } from 'react';
import generateSwipeManager, { generateDisabledSwipeManager } from './generateSwipeManager';
import useBaseSwipe from './useBaseSwipe';
import { calculateSize } from './utils';
export default function useInnerSwipe(props, containerRef) {
  var defaultIndex = props.defaultIndex,
      _props$itemIndex = props.itemIndex,
      itemIndex = _props$itemIndex === void 0 ? -1 : _props$itemIndex,
      _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'horizontal' : _props$direction,
      _props$stepIndex = props.stepIndex,
      stepIndex = _props$stepIndex === void 0 ? -1 : _props$stepIndex,
      _props$offset = props.offset,
      offset = _props$offset === void 0 ? -1 : _props$offset,
      equalNum = props.equalNum,
      gutter = props.gutter,
      onStepsChange = props.onStepsChange,
      children = props.children,
      transitionStatus = props.transitionStatus,
      enableDiffChildren = props.enableDiffChildren;
  var swipeManager = useRef();
  var isH = direction === 'horizontal';

  var forceUpdate = _useForceUpdate();

  var isForceUpdate = useRef(false);

  var endRectChange = function endRectChange() {
    if (swipeManager.current && onStepsChange) {
      onStepsChange(swipeManager.current);
    }

    isForceUpdate.current = true;
    forceUpdate();
  };

  var handleRectChange = function handleRectChange(itemRects, containerRect) {
    if (equalNum && equalNum >= itemRects.length || // 当单个超出容器是有问题的，这种情况暂不处理
    itemRects.length <= 1) {
      swipeManager.current = generateDisabledSwipeManager();
      endRectChange();
      return;
    }

    var panelSize = calculateSize(itemRects, isH);
    var containerSize = isH ? containerRect.width : containerRect.height;

    if (panelSize <= containerSize) {
      swipeManager.current = generateDisabledSwipeManager();
      endRectChange();
      return;
    }

    swipeManager.current = generateSwipeManager({
      items: itemRects,
      containerSize: containerSize,
      direction: direction,
      stepIndex: stepIndex,
      itemIndex: itemIndex,
      defaultIndex: defaultIndex
    });
    endRectChange();
  };

  var _useBaseSwipe = useBaseSwipe({
    children: children,
    direction: direction,
    equalNum: equalNum,
    gutter: gutter,
    onRectChange: handleRectChange,
    containerRef: containerRef,
    enableDiffChildren: enableDiffChildren
  }),
      nodes = _useBaseSwipe.nodes,
      finalDuration = _useBaseSwipe.finalDuration;

  var finalOffset = 0;
  var innerTransitionStatus = transitionStatus || 'on';

  if (offset >= 0) {
    finalOffset = offset;
    innerTransitionStatus = transitionStatus || 'off';
  } else if (swipeManager.current && itemIndex >= 0) {
    finalOffset = swipeManager.current.getItemOffset(itemIndex);
  } else if (swipeManager.current && stepIndex >= 0) {
    finalOffset = swipeManager.current.getStepOffset(stepIndex);
  }

  if (isForceUpdate.current) {
    innerTransitionStatus = 'off';
  }

  var position = isH ? "".concat(-finalOffset, "px, 0") : "0, ".concat(-finalOffset, "px");
  var translateCSS = useMemo(function () {
    return {
      transform: "translate(".concat(position, ")")
    };
  }, [position]);
  isForceUpdate.current = false;
  return {
    finalDuration: finalDuration,
    panelStyle: translateCSS,
    nodes: nodes,
    transitionStatus: innerTransitionStatus,
    // 只是为了测试而返回的一些操作
    handleRectChange: handleRectChange,
    finalOffset: finalOffset
  };
}