import _extends from "@babel/runtime/helpers/extends";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useRef, useState } from 'react';
import { shallowEqualObjects } from 'shallow-equal';
export default function useLoopSwipe() {
  var _useState = useState({
    itemIndex: 0,
    stepIndex: 0,
    stepSize: 0,
    itemSize: 0,
    loop: false,
    transitionStatus: 'off'
  }),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var swipeManager = useRef();

  var updateState = function updateState() {
    var manager = swipeManager.current;
    var index = manager.index,
        size = manager.size,
        realItemIndex = manager.realItemIndex,
        itemSize = manager.itemSize,
        realItemSize = manager.realItemSize;
    setState({
      loop: itemSize > realItemSize,
      stepIndex: index,
      stepSize: size,
      itemIndex: realItemIndex,
      itemSize: realItemSize
    });
  };

  var firstStepsChange = useRef(true);

  var onStepsChange = _useEventCallback(function (manager) {
    swipeManager.current = manager;
    var index = manager.index,
        size = manager.size,
        realItemIndex = manager.realItemIndex,
        itemSize = manager.itemSize,
        realItemSize = manager.realItemSize;
    var newState = {
      stepIndex: firstStepsChange.current ? index : state.stepIndex,
      stepSize: size,
      itemIndex: firstStepsChange.current ? realItemIndex : state.itemIndex,
      itemSize: realItemSize,
      loop: itemSize > realItemSize,
      transitionStatus: 'off'
    };
    firstStepsChange.current = false; // 步长发生了变化，极限情况处理一下

    if (newState.stepIndex !== undefined && newState.stepIndex >= manager.size) {
      newState.stepIndex = manager.size - 1;
    }

    if (!shallowEqualObjects(state, newState)) {
      setState(newState);
    }
  });

  var onPrev = _useEventCallback(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.prev();
      updateState();
    }
  });

  var onNext = _useEventCallback(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.next();
      updateState();
    }
  });

  var onGoTo = _useEventCallback(function (stepIndex) {
    var manager = swipeManager.current;

    if (manager && manager.hasStep(stepIndex)) {
      manager.goTo(stepIndex);
      updateState();
    }
  });

  var onItemPrev = _useEventCallback(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.prevItem();
      updateState();
    }
  });

  var onItemNext = _useEventCallback(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.nextItem();
      updateState();
    }
  });

  var onItemGoTo = _useEventCallback(function (itemIndex) {
    var manager = swipeManager.current;

    if (manager && manager.hasItem(itemIndex)) {
      manager.goToItem(itemIndex);
      updateState();
    }
  });

  return _extends({}, state, {
    manager: swipeManager.current,
    onStepsChange: onStepsChange,
    onNext: onNext,
    onPrev: onPrev,
    onGoTo: onGoTo,
    onItemNext: onItemNext,
    onItemPrev: onItemPrev,
    onItemGoTo: onItemGoTo
  });
}