import _extends from "@babel/runtime/helpers/extends";
import { useLockScroll as _useLockScroll } from "@muya-ui/utils/es/hooks/useLockScroll";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { useCallback, useRef, useState } from 'react';
import { shallowEqualObjects } from 'shallow-equal';
export default function useSwipe() {
  var _useState = useState({
    itemIndex: 0,
    stepIndex: 0,
    offset: 0,
    hasNext: false,
    hasPrev: false,
    stepSize: 0,
    hasNextItem: false,
    hasPrevItem: false,
    itemSize: 0,
    transitionStatus: 'off'
  }),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      active = _useState4[0],
      setActive = _useState4[1];

  var swipeManager = useRef();

  var updateState = function updateState(newOffset) {
    var manager = swipeManager.current;
    var index = manager.index,
        hasNext = manager.hasNext,
        hasPrev = manager.hasPrev,
        size = manager.size,
        itemIndex = manager.itemIndex,
        hasNextItem = manager.hasNextItem,
        hasPrevItem = manager.hasPrevItem,
        itemSize = manager.itemSize;
    setState({
      hasNext: hasNext,
      hasPrev: hasPrev,
      stepIndex: index,
      stepSize: size,
      offset: newOffset,
      itemIndex: itemIndex,
      hasNextItem: hasNextItem,
      hasPrevItem: hasPrevItem,
      itemSize: itemSize,
      transitionStatus: 'on'
    });
  };

  var firstStepsChange = useRef(true);

  var onStepsChange = _useEventCallback(function (manager) {
    swipeManager.current = manager;
    var newState = {
      transitionStatus: 'off',
      stepIndex: firstStepsChange.current ? manager.index : state.stepIndex,
      stepSize: manager.size,
      hasNext: manager.hasNext,
      hasPrev: manager.hasPrev,
      itemIndex: firstStepsChange.current ? manager.itemIndex : state.itemIndex,
      hasNextItem: manager.hasNextItem,
      hasPrevItem: manager.hasPrevItem,
      itemSize: manager.itemSize,
      offset: manager.offset
    };
    firstStepsChange.current = false; // 步长发生了变化，极限情况处理一下

    if (newState.stepIndex !== undefined && newState.stepIndex >= manager.size) {
      newState.stepIndex = manager.size - 1;
    }

    if (!shallowEqualObjects(state, newState)) {
      setState(newState);
    }
  }); // 获取指定 item 所在的 step


  var getItemStep = function getItemStep(itemIndex) {
    var manager = swipeManager.current;

    if (manager) {
      return manager.getItemStep(itemIndex);
    }

    return 0;
  };

  var onPrev = _useEventCallback(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.prev();
      updateState();
    }
  });

  var onNext = _useEventCallback(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.next();
      updateState();
    }
  });

  var onGoTo = _useEventCallback(function (stepIndex) {
    var manager = swipeManager.current;

    if (manager && manager.hasStep(stepIndex)) {
      manager.goTo(stepIndex);
      updateState();
    }
  });

  var onItemPrev = _useEventCallback(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.prevItem();
      updateState();
    }
  });

  var onItemNext = _useEventCallback(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.nextItem();
      updateState();
    }
  });

  var onItemGoTo = _useEventCallback(function (itemIndex) {
    var manager = swipeManager.current;

    if (manager && manager.hasItem(itemIndex)) {
      manager.goToItem(itemIndex);
      updateState();
    }
  });

  var onWheel = _useEventCallback(function (e) {
    var manager = swipeManager.current;

    if (!manager || !active) {
      return;
    }

    var diff = manager.direction === 'horizontal' ? e.deltaX : e.deltaY;
    var currentOffset = state.offset || manager.offset;
    var newOffset = currentOffset + diff;

    if (newOffset < 0 || newOffset > manager.maxOffset) {
      return;
    }

    manager.updateOffset(newOffset);
    updateState(newOffset);
  });

  var onWheelActive = useCallback(function () {
    setActive(true);
  }, []);
  var onWheelDisable = useCallback(function () {
    setActive(false);
  }, []);

  _useLockScroll(active);

  return _extends({}, state, {
    manager: swipeManager.current,
    getItemStep: getItemStep,
    onStepsChange: onStepsChange,
    onNext: onNext,
    onPrev: onPrev,
    onGoTo: onGoTo,
    onItemNext: onItemNext,
    onItemPrev: onItemPrev,
    onItemGoTo: onItemGoTo,
    onWheel: onWheel,
    onWheelActive: onWheelActive,
    onWheelDisable: onWheelDisable
  });
}