import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _extends from "@babel/runtime/helpers/extends";
import { forkRef as _forkRef } from "@muya-ui/utils/es/hooks/useForkRef";
var _excluded = ["style"];
import { Children, cloneElement, createRef } from 'react';
export function rectSizeEqual(rectA, rectB) {
  if (!rectA || !rectB) {
    return false;
  }

  return rectA.width === rectB.width && rectA.height === rectB.height;
}
export function transformItems(items) {
  return items.map(function (item) {
    return item.current.getBoundingClientRect();
  });
}

function getStepPoints(items, isH) {
  var firstPoint = isH ? items[0].left : items[0].top;
  return items.map(function (item) {
    return isH ? item.right - firstPoint : item.bottom - firstPoint;
  });
}
/**
 * 根据一个个 bounding rect 来计算，要覆盖掉 size 需要多少个 rect
 * @param rects 输入的矩形数组
 * @param isH 水平还是垂直
 * @param size 要覆盖到的区间
 */


export function calculateCloneNum(rects, isH, size) {
  var items = getStepPoints(rects, isH);

  for (var i = 0, l = items.length; i < l; i++) {
    var item = items[i];

    if (item >= size) {
      return i + 1;
    }
  }
}
export function calculateSize(items, isH) {
  var firstItem = items[0];
  var lastItem = items[items.length - 1];

  if (isH) {
    return lastItem.right - firstItem.left;
  }

  return lastItem.bottom - firstItem.top;
}

function notNull(value) {
  return value !== null;
}

function cloneMore(arr, num, refs, cloneItems) {
  var i = 0;

  while (i < num) {
    var child = arr[i];
    var innerChildRef = /*#__PURE__*/createRef();

    var handleChildRef = _forkRef(refs[i], innerChildRef);

    cloneItems.push(innerChildRef);
    arr.push( /*#__PURE__*/cloneElement(child, _extends({}, child.props, {
      key: arr.length,
      ref: handleChildRef
    })));
    i++;
  }
}

export function cloneChildren(children) {
  var containerStyle = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var cloneNum = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var innerItems = [];
  var cloneItems = [];
  var newChildren = [];
  var formerRefs = [];
  Children.forEach(children, function (child) {
    var _child$props = child.props,
        style = _child$props.style,
        otherChildProps = _objectWithoutProperties(_child$props, _excluded);

    var innerChildRef = /*#__PURE__*/createRef();

    var handleChildRef = _forkRef(child.ref, innerChildRef);

    var childStyle = _extends({}, style, containerStyle);

    innerItems.push(innerChildRef);
    formerRefs.push(child.ref);
    newChildren.push( /*#__PURE__*/cloneElement(child, _extends({}, otherChildProps, {
      key: notNull(child.key) ? child.key : newChildren.length,
      style: childStyle,
      ref: handleChildRef
    })));
  });
  var num = cloneNum;
  var realItemNum = newChildren.length;

  if (num > realItemNum && realItemNum > 0) {
    while (num > realItemNum) {
      num -= realItemNum;
      cloneMore(newChildren, realItemNum, formerRefs, cloneItems);
    }
  }

  if (num > 0) {
    cloneMore(newChildren, num, formerRefs, cloneItems);
  }

  return {
    refs: innerItems,
    cloneRefs: cloneItems,
    nodes: newChildren
  };
}
/**
 * 根据容器节点来设置子容器的样式，只有设置的了等分的时候需要
 * @param isH
 * @param containerRect
 * @param equalNum
 * @param gutter
 */

export function getChildStyle(isH, containerRect, equalNum) {
  var gutter = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
  var childStyle = {};

  if (equalNum && containerRect && isH) {
    var containerSize = containerRect.width;
    childStyle.width = (containerSize - gutter * (equalNum - 1)) / equalNum;
    gutter && (childStyle.marginRight = gutter);
  } else if (equalNum && containerRect) {
    var _containerSize = containerRect.height;
    childStyle.height = (_containerSize - gutter * (equalNum - 1)) / equalNum;
    gutter && (childStyle.marginBottom = gutter);
  }

  return childStyle;
}