import _extends from "@babel/runtime/helpers/extends";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { FilterIcon as _FilterIcon } from "@muya-ui/theme-light/es/icons/FilterIcon";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _get from "lodash/get";
var _excluded = ["column", "activeColumnKey", "size", "filterColumnData", "onVisibleChange", "onClickAway", "onFilterColumnKeyChange", "onFilterColumnDataChange", "onFilterColumnListChange"];
import React, { forwardRef, useCallback, useMemo, useState } from 'react';
import Button from '../Button';
import { CheckboxGroup } from '../Checkbox';
import useLocale from '../Locale/useLocale';
import PopoverCard from '../PopoverCard';
import { RadioGroup } from '../Radio';
import useTheme from '../utils/useTheme';
import { StyledFilter, StyledFilterItem, StyledFilterWrapper } from './styled';
export default /*#__PURE__*/React.memo( /*#__PURE__*/forwardRef(function (props, ref) {
  var column = props.column,
      activeColumnKey = props.activeColumnKey,
      _props$size = props.size,
      size = _props$size === void 0 ? 'l' : _props$size,
      filterColumnData = props.filterColumnData,
      onVisibleChange = props.onVisibleChange,
      onClickAway = props.onClickAway,
      onFilterColumnKeyChange = props.onFilterColumnKeyChange,
      onFilterColumnDataChange = props.onFilterColumnDataChange,
      onFilterColumnListChange = props.onFilterColumnListChange,
      other = _objectWithoutProperties(props, _excluded);

  var locale = useLocale();
  var theme = useTheme();
  var token = theme.components.Table;
  var _token$filterIcon = token.filterIcon,
      FilterIconToken = _token$filterIcon === void 0 ? _FilterIcon : _token$filterIcon,
      filterData = token.filterData;
  var _column$filterMultipl = column.filterMultiple,
      filterMultiple = _column$filterMultipl === void 0 ? true : _column$filterMultipl,
      filters = column.filters,
      _column$filterIconHig = column.filterIconHighlight,
      filterIconHighlight = _column$filterIconHig === void 0 ? false : _column$filterIconHig,
      filterVisible = column.filterVisible,
      renderFilterActions = column.renderFilterActions,
      renderFilterContent = column.renderFilterContent,
      onFilterVisibleChange = column.onFilterVisibleChange,
      onFilterEntered = column.onFilterEntered,
      onFilterExited = column.onFilterExited,
      filterPopperProps = column.filterPopperProps;
  var hasSorter = !!column.sorter;
  var buttonSize = filterData[size].buttonSize;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      open = _useState2[0],
      setOpen = _useState2[1];

  var isControlled = ('filterVisible' in column);
  var finalOpen = useMemo(function () {
    if (column.key !== activeColumnKey) {
      return false;
    }

    if (isControlled) {
      return filterVisible;
    }

    return open;
  }, [activeColumnKey, column.key, filterVisible, isControlled, open]); // 只有当前列激活时，才同步

  var _useState3 = useState(filterColumnData.selectedFilterValues),
      _useState4 = _slicedToArray(_useState3, 2),
      selectedFilterValues = _useState4[0],
      setSelectedFilterValues = _useState4[1];

  var handleVisibleChange = useCallback(function (v) {
    setOpen(v);

    if (onVisibleChange) {
      onVisibleChange(v);
    }

    if (onFilterVisibleChange) {
      onFilterVisibleChange(v);
    }
  }, [onFilterVisibleChange, onVisibleChange]);

  var handleConfirm = _useEventCallback(function () {
    setOpen(false);
    onFilterColumnDataChange(_extends({}, filterColumnData, {
      selectedFilterValues: selectedFilterValues
    }));
    onFilterVisibleChange && onFilterVisibleChange(false);
  }, [onFilterColumnDataChange, onFilterVisibleChange]);

  var handleReset = _useEventCallback(function () {
    setOpen(false);
    setSelectedFilterValues([]);
    onFilterColumnDataChange(_extends({}, filterColumnData, {
      selectedFilterValues: []
    }));
    onFilterVisibleChange && onFilterVisibleChange(false);
  }, [onFilterColumnDataChange, onFilterVisibleChange]); // 点击其他区域关闭，相当于点击保存


  var handleClickAway = _useEventCallback(function (e) {
    if (finalOpen) {
      onFilterColumnDataChange(_extends({}, filterColumnData, {
        selectedFilterValues: selectedFilterValues
      }));
    }

    onClickAway && onClickAway(e);
  }, [finalOpen, onClickAway, onFilterColumnDataChange]);

  var handleFilterChange = useCallback(function (value) {
    if (Array.isArray(value)) {
      setSelectedFilterValues(value);
    } else {
      setSelectedFilterValues([value]);
    }
  }, []);
  var handleFilterColumnListChange = useCallback(function (outerFilterColumnList) {
    if (!outerFilterColumnList) {
      onFilterColumnDataChange(_extends({}, filterColumnData, {
        selectedFilterValues: selectedFilterValues
      }));
    } else {
      onFilterColumnListChange(outerFilterColumnList);
    }
  }, [filterColumnData, onFilterColumnDataChange, onFilterColumnListChange, selectedFilterValues]);
  var handleFilterIconClick = useCallback(function (e) {
    e.stopPropagation();
    setSelectedFilterValues(filterColumnData.selectedFilterValues);
    onFilterColumnKeyChange(column);
  }, [column, filterColumnData.selectedFilterValues, onFilterColumnKeyChange]);
  var handleEntered = useCallback(function () {
    if (onFilterEntered) {
      onFilterEntered();
    }
  }, [onFilterEntered]);
  var handleExited = useCallback(function () {
    if (onFilterExited) {
      onFilterExited();
    }
  }, [onFilterExited]);
  var transitionProps = useMemo(function () {
    return {
      onEntered: handleEntered,
      onExited: handleExited
    };
  }, [handleEntered, handleExited]);
  var highlightStyle = useMemo(function () {
    if (_get(filterColumnData, 'selectedFilterValues', []).length || filterIconHighlight) {
      return {
        color: theme.colors.spec.brand
      };
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [filterColumnData.selectedFilterValues, filterIconHighlight, theme.colors.spec.brand]);
  var filterIcon = useMemo(function () {
    if (hasSorter) {
      return /*#__PURE__*/React.createElement(StyledFilterWrapper, {
        onClick: handleFilterIconClick
      }, /*#__PURE__*/React.createElement(StyledFilter, {
        style: highlightStyle
      }, column.filterIcon || /*#__PURE__*/React.createElement(FilterIconToken, null)));
    }

    return /*#__PURE__*/React.createElement(StyledFilter, {
      style: highlightStyle,
      onClick: handleFilterIconClick
    }, column.filterIcon || /*#__PURE__*/React.createElement(FilterIconToken, null));
  }, [column.filterIcon, handleFilterIconClick, hasSorter, highlightStyle]);
  var filterActions = useMemo(function () {
    if (renderFilterActions) {
      return renderFilterActions({
        confirm: handleConfirm,
        reset: handleReset,
        selectedFilterValues: selectedFilterValues,
        setSelectedFilterValues: setSelectedFilterValues,
        setFilterColumnKey: onFilterColumnKeyChange,
        syncFilterColumnsList: handleFilterColumnListChange
      });
    }

    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Button, {
      size: buttonSize,
      plain: true,
      onClick: handleReset,
      key: "reset"
    }, locale['Table.filterResetText']), /*#__PURE__*/React.createElement(Button, {
      size: buttonSize,
      onClick: handleConfirm,
      key: "confirm",
      type: "primary"
    }, locale['Table.filterConfirmText']));
  }, [renderFilterActions, buttonSize, handleReset, locale, handleConfirm, selectedFilterValues, onFilterColumnKeyChange, handleFilterColumnListChange]);
  var filterContent = useMemo(function () {
    if (renderFilterContent) {
      return renderFilterContent({
        confirm: handleConfirm,
        reset: handleReset,
        selectedFilterValues: selectedFilterValues,
        setSelectedFilterValues: setSelectedFilterValues,
        setFilterColumnKey: onFilterColumnKeyChange,
        syncFilterColumnsList: handleFilterColumnListChange
      });
    }

    if (!filters) return null;
    if (!filters.length) return null;
    var options = filters.map(function (f) {
      return {
        value: f.value,
        label: f.text
      };
    });

    if (filterMultiple) {
      return /*#__PURE__*/React.createElement(CheckboxGroup, {
        size: buttonSize,
        options: options,
        value: selectedFilterValues,
        onChange: handleFilterChange,
        renderCheckbox: function renderCheckbox(node, option) {
          return /*#__PURE__*/React.createElement(StyledFilterItem, {
            theme: theme,
            key: option.value
          }, node);
        }
      });
    }

    return /*#__PURE__*/React.createElement(RadioGroup, {
      size: buttonSize,
      options: options,
      value: selectedFilterValues[0],
      onChange: handleFilterChange,
      renderRadio: function renderRadio(node, option) {
        return /*#__PURE__*/React.createElement(StyledFilterItem, {
          theme: theme,
          key: option.value
        }, node);
      }
    });
  }, [renderFilterContent, filters, filterMultiple, buttonSize, selectedFilterValues, handleFilterChange, handleConfirm, handleReset, onFilterColumnKeyChange, handleFilterColumnListChange, theme]);
  return /*#__PURE__*/React.createElement(PopoverCard, _extends({
    placement: "bottom",
    triggerAction: "click"
  }, other, {
    transitionProps: transitionProps,
    onVisibleChange: handleVisibleChange,
    onClickAway: handleClickAway,
    content: filterContent,
    actions: filterActions,
    open: finalOpen,
    ref: ref,
    popperProps: filterPopperProps
  }), filterIcon);
}));