import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _typeof from "@babel/runtime/helpers/typeof";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import { FoldIcon as _FoldIcon } from "@muya-ui/theme-light/es/icons/FoldIcon";
import { UnfoldIcon as _UnfoldIcon } from "@muya-ui/theme-light/es/icons/UnfoldIcon";
import _includes from "lodash/includes";
import _find from "lodash/find";
var _excluded = ["position"];
import React, { useMemo, useRef } from 'react';
import EllipsisContent from '../EllipsisContent';
import useLocale from '../Locale/useLocale';
import Result from '../Result';
import Spin from '../Spin';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import ColGroup from './ColGroup';
import FilterPopoverCard from './FilterPopoverCard';
import { StyledBodyRowTr, StyledEmptyTr, StyledExpandButton, StyledExpandCell, StyledExpandedTr, StyledHeader, StyledHeaderContent, StyledHeaderIconWrapper, StyledSortBottomArrow, StyledSortUpArrow, StyledTable, StyledTableWrapper, StyledPagination, StyledDoubleTableWrapper, StyledDoubleTableContainer } from './styled';
import useTable from './useTable';
import { getHasFilter, getHasSort } from './utils';
var defaultStyles = {
  spinWrapper: '',
  wrapper: '',
  paginationWrapper: ''
};
export default function Table(props) {
  var locale = useLocale();
  var theme = useTheme();
  var headerRef = useRef(null);
  var onRow = props.onRow,
      onCell = props.onCell,
      onHeaderRow = props.onHeaderRow,
      onHeaderCell = props.onHeaderCell,
      _props$bordered = props.bordered,
      bordered = _props$bordered === void 0 ? false : _props$bordered,
      _props$stripe = props.stripe,
      stripe = _props$stripe === void 0 ? false : _props$stripe,
      _props$size = props.size,
      size = _props$size === void 0 ? 'l' : _props$size,
      emptyText = props.emptyText,
      _props$defaultSortDir = props.defaultSortDirections,
      defaultSortDirections = _props$defaultSortDir === void 0 ? ['ascend', 'descend'] : _props$defaultSortDir,
      _props$ellipsis = props.ellipsis,
      ellipsisProp = _props$ellipsis === void 0 ? false : _props$ellipsis,
      rowSelection = props.rowSelection,
      _props$components = props.components,
      components = _props$components === void 0 ? {} : _props$components,
      _props$expandable = props.expandable,
      expandable = _props$expandable === void 0 ? {} : _props$expandable,
      _props$scrollBarSize = props.scrollBarSize,
      scrollBarSize = _props$scrollBarSize === void 0 ? 'l' : _props$scrollBarSize,
      fixedHeader = props.fixedHeader,
      fixedFooter = props.fixedFooter,
      style = props.style,
      className = props.className,
      styles = props.styles,
      _props$emptyHeight = props.emptyHeight,
      emptyHeight = _props$emptyHeight === void 0 ? 'auto' : _props$emptyHeight,
      wrapRef = props.wrapRef,
      paginationProps = props.pagination,
      _props$renderMode = props.renderMode,
      renderMode = _props$renderMode === void 0 ? 'single' : _props$renderMode;
  var _expandable$style = expandable.style,
      expandableStyle = _expandable$style === void 0 ? {} : _expandable$style,
      expandedRowRender = expandable.expandedRowRender,
      _expandable$expandedR = expandable.expandedRowBackgroundColor,
      expandedRowBackgroundColor = _expandable$expandedR === void 0 ? theme.components.Table.trBackground.stripe : _expandable$expandedR,
      _expandable$indentSiz = expandable.indentSize,
      indentSize = _expandable$indentSiz === void 0 ? theme.components.Table.expandIndent : _expandable$indentSiz,
      _expandable$childrenC = expandable.childrenColumnName,
      childrenColumnName = _expandable$childrenC === void 0 ? 'children' : _expandable$childrenC;
  var token = theme.components.Table;
  var _token$upArrowIcon = token.upArrowIcon,
      UpArrowIconToken = _token$upArrowIcon === void 0 ? _UnfoldIcon : _token$upArrowIcon,
      _token$bottomArrowIco = token.bottomArrowIcon,
      BottomArrowIconToken = _token$bottomArrowIco === void 0 ? _FoldIcon : _token$bottomArrowIco;

  var _ref = _typeof(paginationProps) === 'object' ? paginationProps : {},
      _ref$position = _ref.position,
      position = _ref$position === void 0 ? 'bottomRight' : _ref$position,
      restPaginationProps = _objectWithoutProperties(_ref, _excluded);

  var innerStyles = useStyles('table', defaultStyles, styles);
  var ComponentTable = components.Table;
  var ComponentHeader = components.Header; // 组件形式引入自定义 Body

  var ComponentBody;
  var componentBodyRef = React.useRef(null);
  var componentBodyProps = {}; // 函数形式自定义 Body

  var bodyHandle;

  if (components.Body) {
    if ( /*#__PURE__*/React.isValidElement(components.Body)) {
      ComponentBody = components.Body;
    } else if (typeof components.Body === 'function') {
      bodyHandle = components.Body;
    } else if (_typeof(components.Body) === 'object' && components.Body.component) {
      ComponentBody = components.Body.component;

      if (components.Body.ref) {
        componentBodyRef = components.Body.ref;
      }

      if (components.Body.props) {
        componentBodyProps = components.Body.props;
      }
    }
  }

  var TableEl = ComponentTable || 'table';
  var HeaderEl = ComponentHeader || 'thead';
  var BodyEl = ComponentBody || 'tbody';
  var HeaderRowEl = components.HeaderRow || 'tr';
  var HeaderCellEl = components.HeaderCell || 'th';
  var tableMode = React.useMemo(function () {
    return components.Header || components.Body || renderMode === 'double' ? 'double' : 'single';
  }, [components.Body, components.Header, renderMode]);

  var _useTable = useTable(props, headerRef, componentBodyRef, tableMode),
      showLeftColumnsShadow = _useTable.showLeftColumnsShadow,
      showRightColumnsShadow = _useTable.showRightColumnsShadow,
      tableState = _useTable.tableState,
      expandedRowKeys = _useTable.expandedRowKeys,
      paginationState = _useTable.paginationState,
      container = _useTable.container,
      getRowKey = _useTable.getRowKey,
      columns = _useTable.columns,
      dataSource = _useTable.dataSource,
      allExpandableRowKeys = _useTable.allExpandableRowKeys,
      tableLayout = _useTable.tableLayout,
      tableProps = _useTable.tableProps,
      spinProps = _useTable.spinProps,
      handleSort = _useTable.handleSort,
      handleFilterColumnKeyChange = _useTable.handleFilterColumnKeyChange,
      handleFilterColumnDataChange = _useTable.handleFilterColumnDataChange,
      handleFilterColumnListChange = _useTable.handleFilterColumnListChange,
      handleExpandRow = _useTable.handleExpandRow,
      handleTableSpinWrapperRef = _useTable.handleTableSpinWrapperRef,
      handleTableWrapperScroll = _useTable.handleTableWrapperScroll,
      handlePageChange = _useTable.handlePageChange,
      handlePageSizeChange = _useTable.handlePageSizeChange,
      tableWrapperRef = _useTable.tableWrapperRef,
      headerArr = _useTable.headerArr,
      renderColumns = _useTable.renderColumns,
      handleDoubleTableWrapperScroll = _useTable.handleDoubleTableWrapperScroll,
      doubleTableHeaderWrapperRef = _useTable.doubleTableHeaderWrapperRef;

  var forkedWrapRef = _useForkRef(tableWrapperRef, wrapRef); // Render headers


  var headers = useMemo(function () {
    var headerRowProps = onHeaderRow && onHeaderRow();
    var trList = headerArr.map(function (_columns) {
      var thList = _columns.map(function (c, cIndex) {
        var _c$sortDirections = c.sortDirections,
            sortDirections = _c$sortDirections === void 0 ? defaultSortDirections : _c$sortDirections,
            renderFilterContent = c.renderFilterContent,
            sorter = c.sorter,
            _c$filters = c.filters,
            filters = _c$filters === void 0 ? [] : _c$filters,
            key = c.key,
            title = c.title,
            children = c.children,
            _c$align = c.align,
            align = _c$align === void 0 ? 'left' : _c$align;
        var hasSorter = getHasSort(sorter, children);
        var hasFilter = getHasFilter(filters, renderFilterContent, children);
        var hasAction = hasSorter || hasFilter;
        var isSortColumn = key === tableState.sortColumnKey;
        var filterColumnData = _find(tableState.filterColumnList || [], function (fc) {
          return fc.key === c.key;
        }) || {
          key: key,
          filterValues: filters.map(function (f) {
            return f.value;
          }),
          selectedFilterValues: []
        };
        var outerHeaderCellProps = onHeaderCell ? onHeaderCell(c, cIndex) : {};

        var headerCellProps = _extends({}, outerHeaderCellProps, {
          style: _extends({
            textAlign: align
          }, outerHeaderCellProps.style)
        });

        if (key === 'selection-column') {
          headerCellProps.style.lineHeight = 0;
        } // 无操作，直接返回th元素


        if (!hasAction) {
          return /*#__PURE__*/React.createElement(HeaderCellEl, _extends({
            rowSpan: c.rowspan || 1,
            colSpan: c.colspan || 1,
            key: key
          }, headerCellProps), title);
        } else {
          // 有排序 or 筛选的情况
          var sortIcons = /*#__PURE__*/React.createElement(React.Fragment, null, _includes(sortDirections, 'ascend') && /*#__PURE__*/React.createElement(StyledSortUpArrow, null, /*#__PURE__*/React.createElement(UpArrowIconToken, null)), _includes(sortDirections, 'descend') && /*#__PURE__*/React.createElement(StyledSortBottomArrow, null, /*#__PURE__*/React.createElement(BottomArrowIconToken, null)));
          var filterPopoverCard = /*#__PURE__*/React.createElement(FilterPopoverCard, {
            column: c,
            activeColumnKey: tableState.filterColumnKey,
            filterColumnData: filterColumnData,
            onFilterColumnKeyChange: handleFilterColumnKeyChange,
            onFilterColumnDataChange: handleFilterColumnDataChange,
            onFilterColumnListChange: handleFilterColumnListChange
          });
          return /*#__PURE__*/React.createElement(StyledHeader, _extends({
            key: key,
            as: HeaderCellEl,
            theme: theme,
            $sortOrder: tableState.sortOrder,
            $isSortColumn: isSortColumn,
            $fixedHeader: fixedHeader,
            rowSpan: c.rowspan || 1,
            colSpan: c.colspan || 1,
            onClick: function onClick() {
              return handleSort(c);
            }
          }, headerCellProps), /*#__PURE__*/React.createElement(StyledHeaderContent, {
            $align: align
          }, title, /*#__PURE__*/React.createElement(StyledHeaderIconWrapper, null, hasSorter ? sortIcons : hasFilter ? filterPopoverCard : null)), hasSorter && hasFilter ? filterPopoverCard : null);
        }
      });

      return /*#__PURE__*/React.createElement(HeaderRowEl, _extends({}, headerRowProps, {
        key: _columns[0].key
      }), thList);
    });
    return trList;
  }, [onHeaderRow, headerArr, defaultSortDirections, tableState.sortColumnKey, tableState.filterColumnList, tableState.filterColumnKey, tableState.sortOrder, onHeaderCell, handleFilterColumnKeyChange, handleFilterColumnDataChange, handleFilterColumnListChange, HeaderCellEl, theme, fixedHeader, handleSort]);
  var rows = useMemo(function () {
    // 空状态
    if (!dataSource.length) {
      var emptyResult = emptyText;

      if (!emptyText) {
        emptyResult = /*#__PURE__*/React.createElement(Result, {
          subTitle: locale['Table.emptyText'],
          type: "emptySmall"
        });
      } else if (typeof emptyText === 'string') {
        emptyResult = /*#__PURE__*/React.createElement(Result, {
          subTitle: emptyText,
          type: "emptySmall"
        });
      }

      return /*#__PURE__*/React.createElement(StyledEmptyTr, null, /*#__PURE__*/React.createElement("td", {
        colSpan: renderColumns.length,
        style: {
          textAlign: 'center',
          height: emptyHeight
        }
      }, emptyResult));
    }

    var renderRows = function renderRows(dataSource) {
      var indent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      return dataSource.map(function (d, rIndex) {
        var rowProps = onRow && onRow(d, rIndex);
        var bodyRowKey = getRowKey(d, rIndex);
        var BodyRow = components.BodyRow,
            BodyCell = components.BodyCell;
        var bodyRowProps = {
          rowKey: bodyRowKey,
          rowIndex: rIndex,
          data: d,
          rowProps: rowProps
        };

        var currentRowExpanded = _includes(expandedRowKeys, bodyRowKey); // 当前行展开状态


        var currentRowExpandable = _includes(allExpandableRowKeys, bodyRowKey); // 当前行是否可展开


        var rowChildren = d[childrenColumnName];
        var cells = renderColumns.map(function (c, cIndex) {
          var render = c.render,
              dataIndex = c.dataIndex,
              key = c.key,
              _c$ellipsis = c.ellipsis,
              ellipsis = _c$ellipsis === void 0 ? ellipsisProp : _c$ellipsis,
              _c$ellipsisTooltip = c.ellipsisTooltip,
              ellipsisTooltip = _c$ellipsisTooltip === void 0 ? true : _c$ellipsisTooltip,
              ellipsisTitle = c.ellipsisTitle,
              renderEllipsisContent = c.renderEllipsisContent,
              _c$align2 = c.align,
              align = _c$align2 === void 0 ? 'left' : _c$align2;
          var outerCellProps = onCell ? onCell(d, rIndex, c, cIndex) : {};
          var text = dataIndex ? d[dataIndex] : '';
          var rawContent = render ? render(d, cIndex, rIndex) : text; // 如果render返回的是对象形式，覆盖列合并等内容

          if (rawContent && rawContent.children) {
            var _ref2 = rawContent,
                children = _ref2.children,
                _props = _ref2.props;
            rawContent = children; // 如果render中制定了Cell相关的属性和onCell的属性合并一下子，render中的值优先级更高

            outerCellProps = _extends({}, outerCellProps, _props);
          }

          var cellProps = _extends({}, outerCellProps, {
            style: _extends({
              textAlign: align
            }, outerCellProps.style)
          });

          if (key === 'selection-column') {
            cellProps.style.lineHeight = 0;
          }

          var bodyCellProps = _extends({}, bodyRowProps, {
            columnKey: key,
            columnIndex: cIndex,
            column: c,
            cellProps: cellProps
          });

          var ellipsisContent = /*#__PURE__*/React.createElement(EllipsisContent, {
            ellipsis: ellipsis,
            enableTooltip: ellipsisTooltip,
            enableTitle: ellipsisTitle,
            customContent: renderEllipsisContent ? renderEllipsisContent(d, cIndex, rIndex) : undefined
          }, rawContent);
          var cellContent = ellipsis ? ellipsisContent : rawContent;

          if (cIndex === (rowSelection ? 1 : 0) && // 第一列
          !!allExpandableRowKeys.length // 存在可展开的行
          ) {
            cellContent = /*#__PURE__*/React.createElement(StyledExpandCell, null, indent ? /*#__PURE__*/React.createElement("div", {
              style: {
                width: indent,
                height: 1,
                display: 'inline-block'
              }
            }) : null, /*#__PURE__*/React.createElement(StyledExpandButton, {
              type: "normal",
              size: size,
              plain: true,
              theme: theme,
              $expanded: currentRowExpanded,
              $expandable: currentRowExpandable,
              onClick: function onClick() {
                return handleExpandRow(d, rIndex);
              }
            }), cellContent);
          } // 当 colSpan 或 rowSpan 为 0 则表示该单元格隐藏，方便调用单元格合并


          var showCell = bodyCellProps.cellProps.colSpan !== 0 && bodyCellProps.cellProps.rowSpan !== 0;

          if (BodyCell) {
            return showCell ? /*#__PURE__*/React.createElement(BodyCell, _extends({
              key: key
            }, bodyCellProps), cellContent) : null;
          }

          return showCell ? /*#__PURE__*/React.createElement("td", _extends({
            key: key
          }, cellProps), cellContent) : null;
        });
        var expandedRowNode = null;

        if (rowChildren && currentRowExpanded && currentRowExpandable) {
          expandedRowNode = renderRows(rowChildren, indent + indentSize);
        }

        if (expandedRowRender && currentRowExpandable) {
          expandedRowNode = /*#__PURE__*/React.createElement(StyledExpandedTr, {
            $expanded: currentRowExpanded
          }, /*#__PURE__*/React.createElement("td", {
            colSpan: renderColumns.length,
            style: expandableStyle
          }, expandedRowRender(d, rIndex, currentRowExpanded)));
        }

        if (BodyRow) {
          return /*#__PURE__*/React.createElement(React.Fragment, {
            key: bodyRowKey
          }, /*#__PURE__*/React.createElement(BodyRow, bodyRowProps, cells), expandedRowNode);
        }

        return /*#__PURE__*/React.createElement(React.Fragment, {
          key: bodyRowKey
        }, /*#__PURE__*/React.createElement(StyledBodyRowTr, _extends({}, rowProps, {
          $rowIndex: rIndex,
          $stripe: stripe,
          theme: theme
        }), cells), expandedRowNode);
      });
    };

    return renderRows(dataSource);
  }, [dataSource, emptyText, renderColumns, emptyHeight, locale, onRow, getRowKey, components, expandedRowKeys, allExpandableRowKeys, childrenColumnName, expandedRowRender, stripe, theme, ellipsisProp, onCell, rowSelection, size, handleExpandRow, indentSize, expandableStyle]);
  var footer = useMemo(function () {
    // 有数据并且Column定义了footer的情况下，渲染表尾
    if (renderColumns.some(function (c) {
      return c.footer;
    }) && dataSource.length) {
      return /*#__PURE__*/React.createElement("tfoot", null, /*#__PURE__*/React.createElement("tr", null, renderColumns.map(function (c) {
        return /*#__PURE__*/React.createElement("td", {
          key: c.key
        }, c.footer);
      })));
    }

    return null;
  }, [dataSource.length, renderColumns]);

  var completeTableEl = function completeTableEl(children) {
    return /*#__PURE__*/React.createElement(StyledTable, {
      as: TableEl,
      theme: theme,
      bordered: bordered,
      stripe: stripe,
      size: size,
      tableLayout: tableLayout,
      $expandedRowBackgroundColor: expandedRowBackgroundColor,
      $columns: columns
    }, /*#__PURE__*/React.createElement(ColGroup, {
      renderColumns: renderColumns
    }), children);
  };

  var singleTable = /*#__PURE__*/React.createElement(StyledTableWrapper, _extends({
    $showLeftColumnsShadow: showLeftColumnsShadow,
    $showRightColumnsShadow: showRightColumnsShadow,
    $scrollBarSize: scrollBarSize,
    $expandedRowBackgroundColor: expandedRowBackgroundColor,
    $columns: columns,
    $fixedHeader: fixedHeader,
    $fixedFooter: fixedFooter,
    $bordered: bordered,
    $stripe: stripe,
    theme: theme,
    ref: forkedWrapRef,
    onScroll: handleTableWrapperScroll,
    className: [innerStyles.wrapper.className, className].join(' ').trim(),
    style: _extends({}, innerStyles.wrapper.style, style)
  }, tableProps), completeTableEl( /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(HeaderEl, {
    ref: headerRef
  }, headers), /*#__PURE__*/React.createElement(BodyEl, null, rows), footer)));
  var doubleTable = /*#__PURE__*/React.createElement(StyledDoubleTableContainer, {
    style: _extends({
      display: 'flex',
      flexDirection: 'column'
    }, innerStyles.wrapper.style, style),
    className: [innerStyles.wrapper.className, className].join(' ').trim()
  }, /*#__PURE__*/React.createElement(StyledDoubleTableWrapper, _extends({
    $showLeftColumnsShadow: showLeftColumnsShadow,
    $showRightColumnsShadow: showRightColumnsShadow,
    $scrollBarSize: scrollBarSize,
    $expandedRowBackgroundColor: expandedRowBackgroundColor,
    $columns: columns,
    $fixedHeader: fixedHeader,
    $bordered: bordered,
    $stripe: stripe,
    $hiddenScrollBar: true,
    theme: theme,
    onScroll: ComponentHeader ? function () {} : handleDoubleTableWrapperScroll,
    ref: doubleTableHeaderWrapperRef,
    style: {
      borderBottom: 'none'
    }
  }, tableProps), ComponentHeader ? /*#__PURE__*/React.createElement(ComponentHeader, {
    onScroll: handleDoubleTableWrapperScroll
  }, /*#__PURE__*/React.createElement(ColGroup, {
    renderColumns: renderColumns
  })) : completeTableEl( /*#__PURE__*/React.createElement("thead", {
    ref: headerRef
  }, headers))), /*#__PURE__*/React.createElement(StyledDoubleTableWrapper, _extends({
    $showLeftColumnsShadow: showLeftColumnsShadow,
    $showRightColumnsShadow: showRightColumnsShadow,
    $scrollBarSize: scrollBarSize,
    $expandedRowBackgroundColor: expandedRowBackgroundColor,
    $columns: columns,
    $fixedFooter: fixedFooter,
    $bordered: bordered,
    $stripe: stripe,
    theme: theme // 当body被外部handle时，onScroll也传递到外部手动调用
    ,
    onScroll: !ComponentBody && !bodyHandle ? handleDoubleTableWrapperScroll : function () {},
    style: {
      flex: 1
    },
    ref: forkedWrapRef
  }, tableProps), ComponentBody && /*#__PURE__*/React.createElement(ComponentBody, _extends({
    ref: componentBodyRef,
    onScroll: handleDoubleTableWrapperScroll,
    dataSource: dataSource,
    columns: columns
  }, componentBodyProps), function (customChildren) {
    return completeTableEl( /*#__PURE__*/React.createElement(React.Fragment, null, customChildren, footer));
  }), bodyHandle && bodyHandle({
    setChildren: function setChildren(customChildren) {
      return completeTableEl( /*#__PURE__*/React.createElement(React.Fragment, null, customChildren, footer));
    },
    ref: componentBodyRef,
    onScroll: handleDoubleTableWrapperScroll
  }), !ComponentBody && !bodyHandle && completeTableEl( /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("tbody", null, rows), footer))));
  var pagination = useMemo(function () {
    return paginationProps ? /*#__PURE__*/React.createElement(StyledPagination, _extends({}, innerStyles.paginationWrapper, restPaginationProps, paginationState, {
      theme: theme,
      $position: position,
      onChange: handlePageChange,
      onPageSizeChange: handlePageSizeChange
    })) : null;
  }, [handlePageChange, handlePageSizeChange, innerStyles.paginationWrapper, paginationProps, paginationState, position, restPaginationProps, theme]);
  var spinNode = useMemo(function () {
    return container && /*#__PURE__*/React.createElement(Spin, spinProps);
  }, [container, spinProps]);
  return /*#__PURE__*/React.createElement("div", _extends({
    ref: handleTableSpinWrapperRef
  }, innerStyles.spinWrapper), _includes(position, 'top') && pagination, tableMode === 'double' ? doubleTable : singleTable, spinNode, _includes(position, 'bottom') && pagination);
}