import _extends from "@babel/runtime/helpers/extends";
import _flatten from "lodash/flatten";
import _includes from "lodash/includes";
import styled, { css } from 'styled-components';
import Button from '../Button';
import Pagination from '../Pagination';
import { scrollBarStyle } from '../ScrollView/styled';
import addPx from '../utils/addPx';
import { setParentAndGetDepth, getChildIndexes } from './utils';
var alignMap = {
  left: 'flex-start',
  center: 'center',
  right: 'flex-end'
}; // 内容区域容器，包含图标和文字

export var StyledHeaderContent = styled.div.withConfig({
  displayName: "StyledHeaderContent"
})(["", ""], function (_ref) {
  var $align = _ref.$align;
  return css(["justify-content:", ";"], alignMap[$align]);
}); // 内容右侧，图标的容器

export var StyledHeaderIconWrapper = styled.div.withConfig({
  displayName: "StyledHeaderIconWrapper"
})([""]); // 排序功能的上下箭头图标

export var StyledSortUpArrow = styled.i.withConfig({
  displayName: "StyledSortUpArrow"
})([""]);
export var StyledSortBottomArrow = styled.i.withConfig({
  displayName: "StyledSortBottomArrow"
})([""]); // 排序面板内容

export var StyledFilterItem = styled.div.withConfig({
  displayName: "StyledFilterItem"
})(["display:flex;align-items:center;margin-bottom:", "px;&:last-child{margin-bottom:0;}"], function (props) {
  return props.theme.spacing.spec.s6;
}); // 筛选功能的图标

export var StyledFilter = styled.i.withConfig({
  displayName: "StyledFilter"
})([""]); // 筛选功能最外层容器

export var StyledFilterWrapper = styled.div.withConfig({
  displayName: "StyledFilterWrapper"
})([""]); // tbody row

export var StyledBodyRowTr = styled.tr.withConfig({
  displayName: "StyledBodyRowTr"
})(["", ""], function (props) {
  var theme = props.theme,
      $rowIndex = props.$rowIndex,
      $stripe = props.$stripe;

  if ($rowIndex % 2 !== 0 && $stripe) {
    return css(["& td:after{background:", ";}"], theme.components.Table.trBackground.stripe);
  }
}); // 空状态tr容器

export var StyledEmptyTr = styled.tr.withConfig({
  displayName: "StyledEmptyTr"
})([""]); // 行展开的tr

export var StyledExpandedTr = styled.tr.withConfig({
  displayName: "StyledExpandedTr"
})(["", ""], function (props) {
  if (!props.$expanded) {
    return css(["display:none;"]);
  }
});
export var StyledExpandCell = styled.div.withConfig({
  displayName: "StyledExpandCell"
})(["display:flex;justify-content:flex-start;align-items:center;"]); // 行展开的按钮

export var StyledExpandButton = styled(Button).withConfig({
  displayName: "StyledExpandButton"
})(["", ""], function (props) {
  var $expanded = props.$expanded,
      _props$$expandable = props.$expandable,
      $expandable = _props$$expandable === void 0 ? true : _props$$expandable,
      theme = props.theme,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size;
  var expandStyle = !$expanded && css(["&::before{transform:translateY(-50%) rotate(-180deg);}&::after{transform:translateX(-50%) rotate(0deg);}"]);
  var expandableStyle = !$expandable && css(["opacity:0;"]); // 必须为偶数，否则内部加号图标会乱

  var fontSize = theme.components.Button.fontSize[size];

  if (fontSize % 2 !== 0) {
    fontSize += 1;
  }

  return css(["width:", "px;height:", "px;padding:0;outline:none;transition:all 0.3s;user-select:none;margin-right:", "px;flex-shrink:0;&::before,&::after{position:absolute;background:currentColor;transition:transform 0.3s ease-out;content:'';}&::before{top:50%;right:3px;left:3px;height:1px;transform:translateY(-50%);}&::after{top:3px;bottom:3px;left:50%;width:1px;transform:translateX(-50%) rotate(90deg);}", " ", ""], fontSize, fontSize, theme.components.Table.expandIndent, expandStyle, expandableStyle);
}); // 带样式的表头th

export var StyledHeader = styled.th.withConfig({
  displayName: "StyledHeader"
})(["", ""], function (props) {
  var $isSortColumn = props.$isSortColumn,
      $sortOrder = props.$sortOrder,
      $fixedHeader = props.$fixedHeader,
      theme = props.theme;
  var token = theme.components.Table,
      colors = theme.colors,
      typography = theme.typography,
      spacing = theme.spacing,
      zIndex = theme.zIndex,
      _theme$transition$pat = theme.transition.pattern,
      easing = _theme$transition$pat.easing,
      duration = _theme$transition$pat.duration;
  return css(["position:", ";transition:background-color ", " ", "ms;user-select:none;cursor:pointer;background:", ";&:hover{", ",", "{", ",", "{color:", ";}}background:", ";}", ",", "{", ",", "{font-size:", "px;color:", ";}}", "{display:flex;align-items:center;width:100%;height:100%;box-sizing:border-box;", "{display:flex;flex-flow:column nowrap;margin-left:", "px;}", ",", "{&,svg{display:block;width:", "px;height:", "px;}", "}", "{margin-bottom:-1px;", "}", "{margin-top:-1px;", "}}", "{position:absolute;right:0;top:0;display:flex;align-items:center;justify-content:center;height:100%;padding:0 ", "px;margin-left:", "px;z-index:", ";}"], $fixedHeader ? 'sticky' : 'relative', easing.status, duration.status, token.headOrFooterBackground, StyledHeaderContent, StyledFilterWrapper, StyledHeaderIconWrapper, StyledFilter, colors.pattern.text.assistant, token.headOrFooterHoverBackground, StyledHeaderContent, StyledFilterWrapper, StyledHeaderIconWrapper, StyledFilter, typography.spec.fontSize.s1, colors.pattern.icon.normal, StyledHeaderContent, StyledHeaderIconWrapper, spacing.spec.s1, StyledSortUpArrow, StyledSortBottomArrow, token.arrowSize, token.arrowSize, !!$sortOrder && $isSortColumn && css(["color:", ";"], colors.pattern.text.assistant), StyledSortUpArrow, $sortOrder === 'ascend' && $isSortColumn && css(["color:", ";"], colors.spec.brand), StyledSortBottomArrow, function (props) {
    return $sortOrder === 'descend' && $isSortColumn && css(["color:", ";"], colors.spec.brand);
  }, StyledFilterWrapper, spacing.spec.s4, spacing.spec.s4, zIndex.spec.s1);
}); // 单元格样式

function makeCellStyle(props) {
  var size = props.size,
      theme = props.theme;
  var _theme$components$Tab = theme.components.Table.sizeData[size],
      headerPaddingLevels = _theme$components$Tab.headerPaddingLevels,
      cellPaddingLevels = _theme$components$Tab.cellPaddingLevels,
      fontLevel = _theme$components$Tab.fontLevel;
  var headerPadding = headerPaddingLevels.map(function (l) {
    return "".concat(theme.spacing.spec[l], "px ");
  });
  var cellPadding = cellPaddingLevels.map(function (l) {
    return "".concat(theme.spacing.spec[l], "px ");
  });
  return css(["th,td{margin:0;box-sizing:border-box;}line-height:", "px;font-size:", "px;thead{th{padding:", ";}", "{padding:0;}", "{padding:", ";}}tfoot td{padding:", ";}tbody td{padding:", ";}"], theme.typography.spec.lineHeight[fontLevel], theme.typography.spec.fontSize[fontLevel], headerPadding, StyledHeader, StyledHeaderContent, headerPadding, headerPadding, cellPadding);
}

export var StyledTableWrapper = styled.div.withConfig({
  displayName: "StyledTableWrapper"
})(["position:relative;overflow:auto;", ""], function (props) {
  var _props$theme = props.theme,
      border = _props$theme.colors.pattern.border,
      token = _props$theme.components.Table;
  var fixedColumns = props.$columns.map(function (c, index) {
    return _extends({}, c, {
      index: index
    });
  }).filter(function (c) {
    return !!c.fixed && 'width' in c;
  });
  var leftFixedColumns = fixedColumns.filter(function (c) {
    return c.fixed === 'left';
  });
  var rightFixedColumns = fixedColumns.filter(function (c) {
    return c.fixed === 'right';
  });

  var getFixedColumnStyles = function getFixedColumnStyles(columns, direction) {
    var styleList = [];
    var depth = setParentAndGetDepth(columns);
    columns.reduce( // acc 为 [ 前面节点宽度, 当前的 childIndex 列表 ]
    function (acc, c, i) {
      var lastList = acc[1];
      var childList = getChildIndexes(lastList, c, direction, depth); // childList 的各个 length 的集合

      var childIndexes = childList.map(function (v) {
        return v.length;
      });
      var lastChildList = childList.map(function (v) {
        return v.slice(-1)[0];
      });
      var width = acc[0];
      var boxShadowStyle;
      var showLeftShadow = props.$showLeftColumnsShadow;
      var showRightShadow = props.$showRightColumnsShadow;

      if (direction === 'left' && showLeftShadow && lastChildList.includes(c) && i === columns.length - 1) {
        boxShadowStyle = css(["border-right:none;&:before{content:'';right:0;left:unset;top:0;bottom:0;position:absolute;width:6px;z-index:-1;box-shadow:", ";}"], token.leftFixedColumnBoxShadow);
      }

      if (direction === 'right' && showRightShadow && lastChildList.includes(c) && i === columns.length - 1) {
        boxShadowStyle = css(["&:before{content:'';left:0;right:unset;top:0;bottom:0;position:absolute;width:6px;box-shadow:", ";}"], token.rightFixedColumnBoxShadow);
      }

      styleList.push(css(["", ""], childIndexes.map(function (child, idx) {
        var headerFootStyles = [];

        for (var _i = 0, j = 0; _i < child; _i++) {
          // 数组中之前已经出现的节点，认为这个节点的 DOM 结构在另一个 tr 中，所以要改变 nth-child 的量
          // 例如 getChildIndexes 注释案例中第二，第三节点中的 “1”
          if (_flatten(childList.slice(0, idx)).includes(childList[idx][_i])) {
            continue;
          }

          headerFootStyles.push(css(["& thead tr:nth-child(", ") th:", "(", "),& tfoot tr:nth-child(", ") td:", "(", "){position:sticky;", ":", ";z-index:", ";", ";}"], idx + 1, direction === 'left' ? 'nth-child' : 'nth-last-child', j + 1, idx + 1, direction === 'left' ? 'nth-child' : 'nth-last-child', j + 1, direction, addPx(childList[idx].slice(0, _i).map(function (v) {
            return Number(v.width);
          }).reduce(function (b, p) {
            return b + p;
          }, 0)), token.fixedZIndex + 1, _i === child - 1 ? boxShadowStyle : ''));
          j++;
        }

        return headerFootStyles;
      })));
      var bodyLen = childIndexes.slice(-1)[0];

      for (var _i2 = 0; _i2 < bodyLen; _i2++) {
        styleList.push(css(["& tbody tr td:", "(", "){position:sticky;", ":", ";z-index:", ";", ";}"], direction === 'left' ? 'nth-child' : 'nth-last-child', _i2 + 1, direction, addPx(childList.slice(-1)[0].slice(0, _i2).map(function (v) {
          return Number(v.width);
        }).reduce(function (b, p) {
          return b + p;
        }, 0)), token.fixedZIndex, _i2 === bodyLen - 1 ? boxShadowStyle : ''));
      }

      return [width + (typeof c.width === 'number' ? c.width : parseInt(c.width, 10)), childList];
    }, [0, []]);
    return styleList;
  };

  return css(["", " ", " &,& *{", "}", "}", " ", "}", "}"], getFixedColumnStyles(leftFixedColumns, 'left'), getFixedColumnStyles(rightFixedColumns.reverse(), 'right'), scrollBarStyle({
    size: props.$scrollBarSize,
    theme: props.theme
  }), props.$bordered && css(["border-left:1px solid ", ";& th,td{border:1px solid ", ";border-top:none;border-left:none;position:relative;&:before{content:'';position:absolute;left:0;width:100%;}&:before{top:-1px;}&::after{content:'';position:absolute;bottom:0;}}& thead > tr:nth-child(1) > th{border-top:1px solid ", ";}& tr:last-child td{border-bottom:none;}"], border.normal, border.normal, border.normal),
  /* 开启斑马纹且 bordered: false 时不存在底线 */
  (!props.$stripe || props.$bordered) && css(["border-bottom:1px solid ", ";"], border.normal), props.$fixedHeader && css(["& thead th{position:sticky;top:0;z-index:", ";}"], token.fixedZIndex), props.$fixedFooter && css(["& tfoot td{position:sticky;bottom:0;z-index:", ";}"], token.fixedZIndex));
});
export var StyledTable = styled.table.withConfig({
  displayName: "StyledTable"
})(["", ""], function (props) {
  var $expandedRowBackgroundColor = props.$expandedRowBackgroundColor,
      theme = props.theme;
  var border = theme.colors.pattern.border,
      token = theme.components.Table;
  return css(["text-align:left;width:100%;background-color:", ";color:", ";border-radius:", " ", " 0 0;border-spacing:0;table-layout:", ";", " thead{tr:first-child{& th:first-child{border-top-left-radius:", ";}& th:last-child{border-top-right-radius:", ";}}tr th{background:", ";font-weight:", ";color:", ";overflow-wrap:break-word;}}tbody{tr{& > td{position:relative;z-index:1;background:", ";overflow-wrap:break-word;&:after{content:' ';position:absolute;z-index:-1;left:0;right:0;top:0;bottom:0;}}&:hover > td:after{background:", ";}", " ", "}", "{background:", ";&:hover{background:", ";}}", "{& > td{background:", ";&:hover{background:", ";}&:hover:after{background:", ";}}}}tfoot{tr td{background:", ";font-weight:", ";overflow-wrap:break-word;}}"], token.tableBackground, token.color, token.borderRadius, token.borderRadius, props.tableLayout, makeCellStyle, token.borderRadius, token.borderRadius, token.headOrFooterBackground, token.headOrFooterFontWeight, token.headOrFooterColor, token.trBackground.normal, token.trBackground.hover, !props.stripe && !props.bordered && css(["& > td{border-bottom:1px solid ", ";}&:last-child > td{border-bottom:none;}"], border.normal), props.stripe && !props.bordered && css(["& > td{border-top:1px solid transparent;}&:last-child > td{border-bottom:1px solid transparent;}&:hover{& > td,& + tr > td{border-top-color:", ";}&:last-child > td{border-bottom-color:", ";}}"], border.normal, border.normal), StyledEmptyTr, token.trBackground.normal, token.trBackground.normal, StyledExpandedTr, $expandedRowBackgroundColor, $expandedRowBackgroundColor, $expandedRowBackgroundColor, token.headOrFooterBackground, token.headOrFooterFontWeight);
});
export var StyledPagination = styled(Pagination).withConfig({
  displayName: "StyledPagination"
})(["", ""], function (props) {
  var _props$$position = props.$position,
      $position = _props$$position === void 0 ? 'bottomRight' : _props$$position,
      _props$size2 = props.size,
      size = _props$size2 === void 0 ? 'm' : _props$size2,
      Table = props.theme.components.Table;

  var getJustifyContent = function getJustifyContent() {
    if (_includes($position, 'Left')) {
      return css(["justify-content:flex-start;"]);
    }

    if (_includes($position, 'Center')) {
      return css(["justify-content:center;"]);
    }

    if (_includes($position, 'Right')) {
      return css(["justify-content:flex-end;"]);
    }
  };

  var marginStyle = _includes($position, 'top') ? css(["margin-bottom:", "px;"], Table.paginationMargin[size]) : css(["margin-top:", "px;"], Table.paginationMargin[size]);
  return css(["display:flex;flex-wrap:wrap;", ";", ";"], getJustifyContent(), marginStyle);
});
export var StyledDoubleTableWrapper = styled(StyledTableWrapper).withConfig({
  displayName: "StyledDoubleTableWrapper"
})(["", ""], function (props) {
  var theme = props.theme,
      _props$$scrollBarSize = props.$scrollBarSize,
      $scrollBarSize = _props$$scrollBarSize === void 0 ? 'm' : _props$$scrollBarSize;
  var scrollBarToken = theme.components.ScrollView.scrollBar;
  var hiddenScrollBarStyle;

  if (props.$hiddenScrollBar) {
    hiddenScrollBarStyle = css(["padding-right:", "px;&::-webkit-scrollbar{width:0;height:0;}"], scrollBarToken.size[$scrollBarSize]);
  }

  return css(["", ""], hiddenScrollBarStyle);
});
export var StyledDoubleTableContainer = styled.div.withConfig({
  displayName: "StyledDoubleTableContainer"
})(["display:flex;flex-direction:column;"]);