import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _extends from "@babel/runtime/helpers/extends";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _uniq from "lodash/uniq";
import _omit from "lodash/omit";
import _includes from "lodash/includes";
import _get from "lodash/get";
import _findIndex from "lodash/findIndex";
import _find from "lodash/find";
import _compact from "lodash/compact";
var _excluded = ["onChange", "onDataSourceChange", "columns", "dataSource", "rowKey", "size", "loading", "defaultSortDirections", "defaultTableState", "ellipsis", "deepSort", "tableLayout", "rowSelection", "expandable", "onScroll", "pagination"];
import React, { useCallback, useEffect, useMemo, useRef, useState, useLayoutEffect } from 'react';
import { shallowEqualArrays } from 'shallow-equal';
import Checkbox from '../Checkbox';
import { useLocale } from '../Locale';
import Radio from '../Radio';
import { conductCheck, convertDataToEntities } from '../Tree/utils';
import { arrayAdd, arrayDel } from '../utils/array';
import forkHandler from '../utils/forkHandler';
import { genHeaderArr, setParentAndGetDepth, sort } from './utils';
var omitPropKeys = ['onRow', 'onCell', 'onHeaderRow', 'onHeaderCell', 'bordered', 'stripe', 'emptyText', 'components', 'scrollBarSize', 'style', 'className', 'styles', 'fixedHeader', 'fixedFooter'];
/** Lock frame, when frame pass reset the lock. */

export function useTimeoutLock(defaultState) {
  var frameRef = useRef(defaultState || null);
  var timeoutRef = useRef();

  function cleanUp() {
    window.clearTimeout(timeoutRef.current);
  }

  function setState(newState) {
    frameRef.current = newState;
    cleanUp();
    timeoutRef.current = window.setTimeout(function () {
      frameRef.current = null;
      timeoutRef.current = undefined;
    }, 100);
  }

  function getState() {
    return frameRef.current;
  }

  useEffect(function () {
    return cleanUp;
  }, []);
  return [setState, getState];
}
export default function useTable(props, ref, componentBodyRef, tableMode) {
  var locale = useLocale();

  var onChange = props.onChange,
      onDataSourceChange = props.onDataSourceChange,
      columnsProp = props.columns,
      dataSource = props.dataSource,
      _props$rowKey = props.rowKey,
      rowKey = _props$rowKey === void 0 ? 'key' : _props$rowKey,
      _props$size = props.size,
      size = _props$size === void 0 ? 'l' : _props$size,
      _props$loading = props.loading,
      loading = _props$loading === void 0 ? false : _props$loading,
      _props$defaultSortDir = props.defaultSortDirections,
      defaultSortDirections = _props$defaultSortDir === void 0 ? ['ascend', 'descend'] : _props$defaultSortDir,
      _props$defaultTableSt = props.defaultTableState,
      defaultTableState = _props$defaultTableSt === void 0 ? {} : _props$defaultTableSt,
      _props$ellipsis = props.ellipsis,
      ellipsisProp = _props$ellipsis === void 0 ? false : _props$ellipsis,
      _props$deepSort = props.deepSort,
      deepSortProp = _props$deepSort === void 0 ? false : _props$deepSort,
      tableLayoutProp = props.tableLayout,
      rowSelection = props.rowSelection,
      _props$expandable = props.expandable,
      expandable = _props$expandable === void 0 ? {} : _props$expandable,
      onScroll = props.onScroll,
      _props$pagination = props.pagination,
      paginationProps = _props$pagination === void 0 ? {} : _props$pagination,
      restProps = _objectWithoutProperties(props, _excluded);

  var expandedRowKeys = expandable.expandedRowKeys,
      _expandable$defaultEx = expandable.defaultExpandedRowKeys,
      defaultExpandedRowKeys = _expandable$defaultEx === void 0 ? [] : _expandable$defaultEx,
      _expandable$defaultEx2 = expandable.defaultExpandAllRows,
      defaultExpandAllRows = _expandable$defaultEx2 === void 0 ? false : _expandable$defaultEx2,
      onExpandedRowsChange = expandable.onExpandedRowsChange,
      expandedRowRender = expandable.expandedRowRender,
      onExpand = expandable.onExpand,
      rowExpandable = expandable.rowExpandable,
      _expandable$childrenC = expandable.childrenColumnName,
      childrenColumnName = _expandable$childrenC === void 0 ? 'children' : _expandable$childrenC;

  var rowSelectionType = _get(rowSelection, 'type', 'checkbox'); // common


  var getRowKey = useCallback(function (d, rIndex) {
    return typeof rowKey === 'function' ? rowKey(d, rIndex) : d[rowKey];
  }, [rowKey]);

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      showLeftColumnsShadow = _useState2[0],
      setShowLeftColumnsShadow = _useState2[1];

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      showRightColumnsShadow = _useState4[0],
      setShowRightColumnsShadow = _useState4[1];

  var _useState5 = useState(null),
      _useState6 = _slicedToArray(_useState5, 2),
      container = _useState6[0],
      setContainer = _useState6[1];

  var tableWrapperRef = useRef(null);
  var doubleTableHeaderWrapperRef = useRef(null);
  var handleTableSpinWrapperRef = useCallback(function (e) {
    if (!e || e === container) return; // 减少不必要的状态更新

    setContainer(e);
  }, [container]); // 筛选、排序逻辑

  var isControlled = ('tableState' in props);

  var _useState7 = useState(defaultTableState),
      _useState8 = _slicedToArray(_useState7, 2),
      tableState = _useState8[0],
      setTableState = _useState8[1];

  var finalTableState = isControlled ? props.tableState || {} : tableState;
  var updateTableState = useCallback(function (stateAction) {
    if (!isControlled) {
      setTableState(stateAction);
    } else {
      if (typeof stateAction === 'function') {
        stateAction(finalTableState);
      }
    }
  }, [finalTableState, isControlled]); // 点击排序回调

  var handleSort = _useEventCallback(function (column) {
    if (!column.sorter) return;
    updateTableState(function (prevState) {
      var _column$sortDirection = column.sortDirections,
          sortDirections = _column$sortDirection === void 0 ? defaultSortDirections : _column$sortDirection;
      var sortOrder = prevState.sortOrder,
          sortColumnKey = prevState.sortColumnKey;
      var newSortOrder; // 切换新的列时，直接使用sortDirection的第一个值来初始化

      if (column.key === sortColumnKey && sortOrder !== undefined) {
        // 按照sortDirections的内容依次切换排序状态
        var methodIndex = sortDirections.indexOf(sortOrder) + 1;
        newSortOrder = methodIndex === sortDirections.length ? undefined : sortDirections[methodIndex];
      } else {
        newSortOrder = sortDirections[0];
      }

      var newTableState = _extends({}, prevState, {
        sortOrder: newSortOrder,
        sortColumnKey: newSortOrder ? column.key : ''
      });

      onChange && onChange(newTableState);
      return newTableState;
    });
  }); // 点击确定或者重置时更新 filter 状态


  var handleFilterColumnDataChange = _useEventCallback(function (newFilterColumn) {
    updateTableState(function (prev) {
      var _finalTableState$filt = finalTableState.filterColumnList,
          filterColumnList = _finalTableState$filt === void 0 ? [] : _finalTableState$filt;

      var newFilterColumnList = _toConsumableArray(filterColumnList);

      var index = _findIndex(filterColumnList, function (f) {
        return f.key === newFilterColumn.key;
      });

      if (index >= 0) {
        newFilterColumnList.splice(index, 1, newFilterColumn);
      } else {
        newFilterColumnList.push(newFilterColumn);
      }

      var filteredData = getFilteredDataSource(dataSource, newFilterColumnList); // 获取筛选后的数据长度

      var newTableState = _extends({}, prev, {
        filterColumnList: newFilterColumnList,
        pagination: {
          current: 1,
          pageSize: paginationProps.pageSize || 10,
          totalRecords: filteredData.length
        }
      });

      onChange && onChange(newTableState);
      return newTableState;
    });
  });

  var handleFilterColumnListChange = _useEventCallback(function (outerFilterColumnList) {
    updateTableState(function (prev) {
      var newTableState = _extends({}, prev, {
        filterColumnList: outerFilterColumnList
      });

      onChange && onChange(newTableState);
      return newTableState;
    });
  }); // 点击 filter 修改当前的筛选列


  var handleFilterColumnKeyChange = _useEventCallback(function (c) {
    updateTableState(function (prev) {
      var filterColumnKeyChanged = prev.filterColumnKey !== c.key;

      if (filterColumnKeyChanged) {
        var newTableState = _extends({}, prev, {
          filterColumnKey: c.key
        });

        onChange && onChange(newTableState);
        return newTableState;
      }

      return prev;
    });
  }); // 存在 fixed: left 的列


  var hasFixedLeftColumn = useMemo(function () {
    return columnsProp.some(function (c) {
      return c.fixed === 'left';
    });
  }, [columnsProp]); // 最终的 columns

  var columns = useMemo(function () {
    var leftFixedColumns = columnsProp.filter(function (c) {
      return c.fixed === 'left';
    });
    var rightFixedColumns = columnsProp.filter(function (c) {
      return c.fixed === 'right';
    });
    var normalColumns = columnsProp.filter(function (c) {
      return !c.fixed;
    });
    return [].concat(_toConsumableArray(leftFixedColumns), _toConsumableArray(normalColumns), _toConsumableArray(rightFixedColumns));
  }, [columnsProp]);
  var headerDepth = useMemo(function () {
    return setParentAndGetDepth(columns);
  }, [columns]);

  var _useMemo = useMemo(function () {
    return genHeaderArr(columns, headerDepth);
  }, [columns, headerDepth]),
      _useMemo2 = _slicedToArray(_useMemo, 2),
      baseRenderColumns = _useMemo2[1];

  var getFilteredDataSource = useCallback(function (dataSource) {
    var filterColumnList = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    var newDataSource = _toConsumableArray(dataSource);

    if (filterColumnList && filterColumnList.length) {
      filterColumnList.forEach(function (filterColumn) {
        var column = _find(baseRenderColumns, function (c) {
          return c.key === filterColumn.key;
        });

        if (!column) return;
        var onFilter = column.onFilter;
        if (!onFilter) return;
        if (!filterColumn.selectedFilterValues.length) return; // 执行到这里，onFilter and selectedFilterValues肯定有值
        // 同一列存在多个筛选条件，满足其中任意一个条件即展示
        // 筛选完毕，将筛选结果赋值给dataSource，继续筛选下一列的数据

        newDataSource = newDataSource.filter(function (d) {
          return filterColumn.selectedFilterValues.some(function (v) {
            return onFilter(v, d);
          });
        });
      });
    }

    return newDataSource;
  }, [baseRenderColumns]);
  var filteredData = useMemo(function () {
    return getFilteredDataSource(dataSource, tableState.filterColumnList);
  }, [dataSource, getFilteredDataSource, tableState.filterColumnList]); // 分页逻辑

  var _useState9 = useState(paginationProps.totalRecords || filteredData.length),
      _useState10 = _slicedToArray(_useState9, 2),
      totalRecords = _useState10[0],
      setTotal = _useState10[1];

  var _useState11 = useState(paginationProps.current || 1),
      _useState12 = _slicedToArray(_useState11, 2),
      current = _useState12[0],
      setCurrent = _useState12[1];

  var _useState13 = useState(paginationProps.pageSize || 10),
      _useState14 = _slicedToArray(_useState13, 2),
      pageSize = _useState14[0],
      setPageSize = _useState14[1];

  useEffect(function () {
    if (paginationProps.current && current !== paginationProps.current) {
      setCurrent(paginationProps.current);
    }

    if (paginationProps.pageSize && pageSize !== paginationProps.pageSize) {
      setPageSize(paginationProps.pageSize);
    } // eslint-disable-next-line

  }, [paginationProps.current, paginationProps.pageSize]); // 总数据量改变时，如果外部传入的current值超出当前页码的边界，通知外部去改变

  useEffect(function () {
    var total = totalRecords;

    if (paginationProps.totalRecords !== undefined) {
      totalRecords !== paginationProps.totalRecords && (total = paginationProps.totalRecords);
    } else {
      total = filteredData.length;
    }

    setTotal(total); // 计算总页数

    var totalPages = Math.ceil(total / pageSize); // 只有当current超出页码边界时，向外传递正确的数据

    if (totalPages > 0 && (current < 1 || current > totalPages)) {
      // 计算真实的当前页应该是什么
      var page = current <= 1 ? 1 : current > totalPages ? totalPages : current;
      updateTableState(function (prev) {
        var pageChanged = current !== page;

        if (pageChanged) {
          if (paginationProps.current === undefined) {
            setCurrent(page);
          }

          var newTableState = _extends({}, prev, {
            pagination: {
              pageSize: pageSize,
              totalRecords: total,
              current: page
            }
          });

          onChange && onChange(newTableState);
          return newTableState;
        }

        return prev;
      });
    } // eslint-disable-next-line

  }, [filteredData.length, paginationProps.totalRecords]);

  var handlePageChange = _useEventCallback(function (page) {
    updateTableState(function (prev) {
      var pageChanged = current !== page;

      if (pageChanged) {
        if (paginationProps.current === undefined) {
          setCurrent(page);
        }

        var newTableState = _extends({}, prev, {
          pagination: {
            pageSize: pageSize,
            totalRecords: totalRecords,
            current: page
          }
        });

        onChange && onChange(newTableState);
        return newTableState;
      }

      return prev;
    });
  }, []);

  var handlePageSizeChange = _useEventCallback(function (pageSiz) {
    updateTableState(function (prev) {
      var pageChanged = pageSize !== pageSiz;

      if (pageChanged) {
        if (paginationProps.pageSize === undefined) {
          setPageSize(pageSiz);
        } // 每页大小改变后，重新计算总页数


        var totalPages = Math.ceil(totalRecords / pageSiz); // 计算真实的当前页应该是什么

        var page = current <= 1 ? 1 : current > totalPages ? totalPages : current; // 如果得到的页码和当前不一致，设置一下

        if (page !== current) {
          setCurrent(page);
        }

        var newTableState = _extends({}, prev, {
          pagination: {
            current: page,
            totalRecords: totalRecords,
            pageSize: pageSiz
          }
        });

        onChange && onChange(newTableState);
        return newTableState;
      }

      return prev;
    });
  }, []); // 最终的 dataSource


  var isPageControlled = ('pagination' in props);
  var finalDataSource = useMemo(function () {
    var sortColumnKey = finalTableState.sortColumnKey,
        sortOrder = finalTableState.sortOrder,
        filterColumnList = finalTableState.filterColumnList;
    var newDataSource = getFilteredDataSource(dataSource, filterColumnList);

    var sortColumn = _find(baseRenderColumns, function (c) {
      return !!(c.key && c.key === sortColumnKey);
    });

    if (sortColumn && sortColumn.sorter && sortOrder && sortColumnKey) {
      var _sorter = sortColumn.sorter,
          _sortColumn$deepSort = sortColumn.deepSort,
          deepSort = _sortColumn$deepSort === void 0 ? deepSortProp : _sortColumn$deepSort;
      newDataSource = sort({
        deepSort: deepSort,
        data: newDataSource,
        sorter: function sorter(a, b) {
          var result = _sorter(a, b, sortOrder);

          if (result !== 0) {
            return sortOrder === 'descend' ? -result : result;
          }

          return 0;
        },
        childrenColumnName: childrenColumnName
      });
    } // FIXED: 如果totalRecords不受控,而且经过上面的计算，数组长度变化了，重新计算一次total


    if (isPageControlled && !paginationProps.totalRecords && newDataSource.length !== totalRecords) {
      setTotal(newDataSource.length);
    } // 分页选项生效时，如果newDataSource数据量超过单页条数需要进行分割


    if (isPageControlled && pageSize < newDataSource.length) {
      var startIndex = (current - 1) * pageSize;
      newDataSource = newDataSource.slice(startIndex, startIndex + pageSize);
    }

    onDataSourceChange && onDataSourceChange(newDataSource);
    return newDataSource;
  }, [baseRenderColumns, childrenColumnName, current, dataSource, deepSortProp, finalTableState, getFilteredDataSource, isPageControlled, onDataSourceChange, pageSize, paginationProps.totalRecords, totalRecords]); // 展开逻辑

  var getExpandableRowKeys = useCallback(function (dataSource) {
    return dataSource.reduce(function (acc, d, rIndex) {
      var key = rowExpandable && !rowExpandable(d) ? '' : getRowKey(d, rIndex);
      var rowChildren = d[childrenColumnName];
      var rowHasChildren = rowChildren && Array.isArray(rowChildren);

      if (expandedRowRender) {
        return acc.concat([key]);
      }

      if (rowHasChildren) {
        return acc.concat([key]).concat(getExpandableRowKeys(rowChildren));
      }

      return acc;
    }, []).filter(function (k) {
      return !!k;
    });
  }, [childrenColumnName, expandedRowRender, getRowKey, rowExpandable]);
  var allExpandableRowKeys = useMemo(function () {
    return getExpandableRowKeys(finalDataSource);
  }, [finalDataSource, getExpandableRowKeys]); // 展开状态

  var isExpandableControlled = ('expandedRowKeys' in expandable);

  var _useState15 = useState(defaultExpandAllRows ? allExpandableRowKeys : defaultExpandedRowKeys),
      _useState16 = _slicedToArray(_useState15, 2),
      expandedRowKeysState = _useState16[0],
      setExpandedRowKeysState = _useState16[1];

  var finalExpandedRowKeys = isExpandableControlled ? expandedRowKeys || [] : expandedRowKeysState; // 点击展开回调

  var handleExpandRow = useCallback(function (record, rIndex) {
    var rowKey = getRowKey(record, rIndex);

    var isExpanded = _includes(finalExpandedRowKeys, rowKey);

    var nextExpanded = !isExpanded;
    onExpand && onExpand(nextExpanded, record);

    var newRowKeys = _toConsumableArray(finalExpandedRowKeys);

    if (nextExpanded) {
      // 展开
      newRowKeys = _uniq([].concat(_toConsumableArray(newRowKeys), [rowKey]));
    } else {
      // 收起
      newRowKeys = newRowKeys.filter(function (k) {
        return k !== rowKey;
      });
    }

    setExpandedRowKeysState(newRowKeys);
    onExpandedRowsChange && onExpandedRowsChange(newRowKeys);
  }, [finalExpandedRowKeys, getRowKey, onExpand, onExpandedRowsChange]); // 选择逻辑

  var _useMemo3 = useMemo(function () {
    if (rowSelection) {
      return convertDataToEntities(dataSource, {
        key: 'key',
        title: 'title',
        isLeaf: 'isLeaf',
        children: childrenColumnName
      }, false, getRowKey);
    } else {
      // eslint-disable-next-line @typescript-eslint/no-object-literal-type-assertion
      return {
        keyEntities: {}
      };
    }
  }, [childrenColumnName, dataSource, getRowKey, rowSelection]),
      keyEntities = _useMemo3.keyEntities;

  var isControlledRowSelection = rowSelection && 'selectedRowKeys' in rowSelection;

  var _useState17 = useState(function () {
    if (rowSelection && rowSelection.checkStrictly === false && rowSelectionType === 'checkbox') {
      return conductCheck(rowSelection.selectedRowKeys || [], true, keyEntities);
    } else {
      return {
        checked: isControlledRowSelection ? rowSelection.selectedRowKeys || [] : [],
        halfChecked: []
      };
    }
  }),
      _useState18 = _slicedToArray(_useState17, 2),
      selectedRowKeys = _useState18[0],
      setSelectedRowKeys = _useState18[1];

  var isDisabledCheck = useCallback(function (d) {
    if (rowSelection) {
      var getPropsMethod = rowSelectionType === 'checkbox' ? rowSelection.getCheckboxProps : rowSelection.getRadioProps;
      var disabled = getPropsMethod ? getPropsMethod(d).disabled : false;
      return disabled;
    }
  }, [rowSelection, rowSelectionType]);

  var handleCheckboxChange = _useEventCallback(function (selected, rowKey, row) {
    if (rowSelection) {
      var oriCheckedKeys = selectedRowKeys.checked,
          oriHalfCheckedKeys = selectedRowKeys.halfChecked;
      var checkedObj;

      if (rowSelection.checkStrictly === false) {
        var _conductCheck = conductCheck([rowKey], selected, keyEntities, {
          checkedKeys: oriCheckedKeys,
          halfCheckedKeys: oriHalfCheckedKeys
        }),
            newCheckedKeys = _conductCheck.checked,
            newHalfCheckedKeys = _conductCheck.halfChecked;

        checkedObj = {
          checked: newCheckedKeys,
          halfChecked: newHalfCheckedKeys
        };
      } else {
        var _newCheckedKeys = selected ? arrayAdd(oriCheckedKeys, rowKey) : arrayDel(oriCheckedKeys, rowKey);

        var _newHalfCheckedKeys = arrayDel(oriHalfCheckedKeys, rowKey);

        checkedObj = {
          checked: _newCheckedKeys,
          halfChecked: _newHalfCheckedKeys
        };
      }

      if (!isControlledRowSelection) {
        setSelectedRowKeys(checkedObj);
      }

      var selectedRows = _compact(checkedObj.checked.map(function (key) {
        return _get(keyEntities[key], 'node');
      }));

      rowSelection.onChange && rowSelection.onChange(checkedObj.checked, selectedRows);
      rowSelection.onSelect && rowSelection.onSelect(selected, row);
    }
  });

  var handleRadioChange = _useEventCallback(function (selected, rowKey, row) {
    if (rowSelection) {
      var oriCheckedKeys = selectedRowKeys.checked;
      var checkedObj = {
        checked: [rowKey],
        halfChecked: []
      };

      if (oriCheckedKeys[0] === rowKey) {
        // 如果已选中不触发事件
        return;
      } else if (!isControlledRowSelection) {
        setSelectedRowKeys(checkedObj);
      }

      var selectedRows = [_get(keyEntities[rowKey], 'node')];
      rowSelection.onChange && rowSelection.onChange(checkedObj.checked, selectedRows);
      rowSelection.onSelect && rowSelection.onSelect(selected, row);
    }
  });

  var handleAllCheckboxChange = useCallback(function (selected) {
    if (rowSelection) {
      var targetRowKeys = selected ? Object.keys(keyEntities).filter(function (key) {
        return !isDisabledCheck(keyEntities[key].node);
      }).map(function (key) {
        return keyEntities[key].key;
      }) : [];
      var checkedObj = {
        checked: targetRowKeys,
        halfChecked: []
      };

      if (!isControlledRowSelection) {
        setSelectedRowKeys(checkedObj);
      }

      var selectedRows = _compact(checkedObj.checked.map(function (key) {
        return _get(keyEntities[key], 'node');
      }));

      rowSelection.onChange && rowSelection.onChange(checkedObj.checked, selectedRows);
      rowSelection.onSelectAll && rowSelection.onSelectAll(selected, selectedRows);
    }
  }, [isControlledRowSelection, isDisabledCheck, keyEntities, rowSelection]);
  var selectedColumn = useMemo(function () {
    if (rowSelection) {
      var allKeys = Object.keys(keyEntities).filter(function (key) {
        return !isDisabledCheck(keyEntities[key].node);
      }).map(function (key) {
        return keyEntities[key].key;
      });
      var checkedCurrentAll = allKeys.every(function (key) {
        return _includes(selectedRowKeys.checked, key);
      });
      var checkedCurrentSome = allKeys.some(function (key) {
        return _includes(selectedRowKeys.checked, key);
      }); // 为了保证不产生额外影响，仅将 getCheckboxProps 返回的 size 传递给 columnTitle

      var columnTitleCheckboxSize = size;

      try {
        if (rowSelection.getCheckboxProps) {
          // eslint-disable-next-line @typescript-eslint/no-object-literal-type-assertion
          var checkboxProps = rowSelection.getCheckboxProps({});

          if (checkboxProps.size) {
            columnTitleCheckboxSize = checkboxProps.size;
          }
        }
      } catch (_unused) {} // 注入 selection-column


      var defaultSelectionColumnTitle = rowSelectionType === 'radio' ? null : /*#__PURE__*/React.createElement(Checkbox, {
        onChange: function onChange(e) {
          return handleAllCheckboxChange(e.target.checked);
        },
        checked: !!allKeys.length && checkedCurrentAll,
        indeterminate: !checkedCurrentAll && checkedCurrentSome,
        size: columnTitleCheckboxSize,
        disabled: !allKeys.length
      }); // TODO: 改为计算值，需要调整 token 结构

      var defaultSelectionColumnWidth = 68;
      var selectionColumn = {
        key: 'selection-column',
        width: rowSelection.columnWidth || defaultSelectionColumnWidth,
        title: rowSelection.columnTitle || defaultSelectionColumnTitle,
        render: function render(data, _cIndex, rIndex) {
          var finalRowKey = getRowKey(data, rIndex);
          var checkboxProps = rowSelection.getCheckboxProps ? rowSelection.getCheckboxProps(data) : {};
          var radioProps = rowSelection.getRadioProps ? rowSelection.getRadioProps(data) : {};
          var selectNode;

          if (rowSelectionType === 'radio') {
            selectNode = /*#__PURE__*/React.createElement(Radio, _extends({
              onChange: function onChange(e) {
                return handleRadioChange(e.target.checked, finalRowKey, data);
              },
              checked: _includes(selectedRowKeys.checked, finalRowKey),
              size: size
            }, radioProps));
          } else {
            selectNode = /*#__PURE__*/React.createElement(Checkbox, _extends({
              onChange: function onChange(e) {
                return handleCheckboxChange(e.target.checked, finalRowKey, data);
              },
              checked: _includes(selectedRowKeys.checked, finalRowKey),
              indeterminate: _includes(selectedRowKeys.halfChecked, finalRowKey),
              size: size
            }, checkboxProps));
          }

          return rowSelection.renderCell ? rowSelection.renderCell(_includes(selectedRowKeys.checked, finalRowKey), data, rIndex, selectNode) : selectNode;
        }
      };

      if (hasFixedLeftColumn) {
        selectionColumn.fixed = 'left';
      }

      return selectionColumn;
    }
  }, [getRowKey, handleAllCheckboxChange, handleCheckboxChange, handleRadioChange, hasFixedLeftColumn, isDisabledCheck, keyEntities, rowSelection, rowSelectionType, selectedRowKeys.checked, selectedRowKeys.halfChecked, size]);
  var finalColumns = useMemo(function () {
    if (selectedColumn) {
      return [selectedColumn].concat(_toConsumableArray(columns));
    } else {
      return columns;
    }
  }, [columns, selectedColumn]);

  var _useMemo4 = useMemo(function () {
    return genHeaderArr(finalColumns, headerDepth);
  }, [finalColumns, headerDepth]),
      _useMemo5 = _slicedToArray(_useMemo4, 2),
      headerArr = _useMemo5[0],
      renderColumns = _useMemo5[1]; // tableLayout: https://css-tricks.com/almanac/properties/t/table-layout/


  var tableLayout = useMemo(function () {
    if (tableLayoutProp) {
      return tableLayoutProp;
    } else {
      return columns.some(function (c) {
        return 'width' in c || ('ellipsis' in c ? c.ellipsis : ellipsisProp);
      }) ? 'fixed' : 'auto';
    }
  }, [columns, ellipsisProp, tableLayoutProp]); // Table固定列设置左右阴影逻辑

  var handleFixedColumnShadow = useCallback(function () {
    // 双table模式下加了阴影性能有问题，待解决
    if (!tableWrapperRef.current || tableMode === 'double') return;
    var _tableWrapperRef$curr = tableWrapperRef.current,
        scrollLeft = _tableWrapperRef$curr.scrollLeft,
        clientWidth = _tableWrapperRef$curr.clientWidth,
        scrollWidth = _tableWrapperRef$curr.scrollWidth; // 如果 body 的滚动节点在自定义 Body 节点内的情况

    if (scrollWidth <= clientWidth && componentBodyRef.current) {
      scrollLeft = componentBodyRef.current.scrollLeft;
      clientWidth = componentBodyRef.current.clientWidth;
      scrollWidth = componentBodyRef.current.scrollWidth;
    } // 当前Table不可滚动，所有阴影不展示


    if (scrollWidth <= clientWidth) {
      setShowLeftColumnsShadow(false);
      setShowRightColumnsShadow(false);
      return;
    } // 根据滚动的距离来展示阴影


    if (scrollLeft <= 0) {
      setShowLeftColumnsShadow(false);
    } else {
      setShowLeftColumnsShadow(true);
    }

    if (scrollLeft + clientWidth === scrollWidth) {
      setShowRightColumnsShadow(false);
    } else {
      setShowRightColumnsShadow(true);
    }
  }, [componentBodyRef, tableMode]);
  var handleTableWrapperScroll = useMemo(function () {
    return forkHandler(handleFixedColumnShadow, onScroll);
  }, [handleFixedColumnShadow, onScroll]);

  var _useTimeoutLock = useTimeoutLock(null),
      _useTimeoutLock2 = _slicedToArray(_useTimeoutLock, 2),
      setScrollTarget = _useTimeoutLock2[0],
      getScrollTarget = _useTimeoutLock2[1];

  var handleWheel = React.useCallback(function (e) {
    var _ref = e,
        currentTarget = _ref.currentTarget,
        scrollLeft = _ref.scrollLeft,
        target = _ref.target;
    var offset = scrollLeft || currentTarget && currentTarget.scrollLeft || target && target.scrollLeft;
    var compareTarget = currentTarget || target || {};

    if (!getScrollTarget() || getScrollTarget() === compareTarget) {
      setScrollTarget(compareTarget);
      doubleTableHeaderWrapperRef.current && (doubleTableHeaderWrapperRef.current.scrollLeft = offset);
      tableWrapperRef.current && (tableWrapperRef.current.scrollLeft = offset);
      componentBodyRef.current && (componentBodyRef.current.scrollLeft = offset);
    }
  }, [componentBodyRef, getScrollTarget, setScrollTarget]);
  var handleDoubleTableWrapperScroll = useMemo(function () {
    return forkHandler(handleTableWrapperScroll, handleWheel);
  }, [handleTableWrapperScroll, handleWheel]);
  React.useEffect(function () {
    function onWheel(e) {
      var _ref2 = e,
          currentTarget = _ref2.currentTarget,
          deltaX = _ref2.deltaX,
          deltaY = _ref2.deltaY; // 触摸屏存在双相同时滚动的情况，过滤掉双向滚动

      if (deltaX && Math.abs(deltaX) > Math.abs(deltaY)) {
        handleWheel({
          currentTarget: currentTarget,
          scrollLeft: currentTarget.scrollLeft + deltaX
        });
        e.preventDefault();
      }
    }

    if (tableMode === 'double') {
      tableWrapperRef.current && tableWrapperRef.current.addEventListener('wheel', onWheel);
      doubleTableHeaderWrapperRef.current && doubleTableHeaderWrapperRef.current.addEventListener('wheel', onWheel);
      componentBodyRef.current && componentBodyRef.current.addEventListener('wheel', onWheel);
      return function () {
        tableWrapperRef.current && tableWrapperRef.current.removeEventListener('wheel', onWheel);
        doubleTableHeaderWrapperRef.current && doubleTableHeaderWrapperRef.current.removeEventListener('wheel', onWheel);
        componentBodyRef.current && componentBodyRef.current.removeEventListener('wheel', onWheel);
      };
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [tableWrapperRef.current, tableMode]);
  /**
   * 以下为计算的 props
   * spinProps：Table 加载的 Spin 组件的 props
   */

  var spinProps = useMemo(function () {
    return _extends({
      container: container,
      desc: locale['Table.loadingText']
    }, typeof loading === 'boolean' ? {
      spinning: loading
    } : loading);
  }, [container, loading, locale]);

  var tableProps = _omit(restProps, omitPropKeys);

  useEffect(function () {
    if (isControlledRowSelection && !shallowEqualArrays(rowSelection.selectedRowKeys || [], selectedRowKeys.checked)) {
      var newSelectedRowKeys = {
        checked: rowSelection.selectedRowKeys || [],
        halfChecked: []
      };

      if (rowSelection.checkStrictly === false && rowSelectionType === 'checkbox') {
        newSelectedRowKeys = conductCheck(rowSelection.selectedRowKeys || [], true, keyEntities);
      }

      setSelectedRowKeys(newSelectedRowKeys);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [isControlledRowSelection, keyEntities, rowSelection]); // Table挂载时、浏览器窗口大小发生变化时，需要重新计算当前阴影状态

  useEffect(function () {
    handleFixedColumnShadow();
    window.addEventListener('resize', handleFixedColumnShadow);
    return function () {
      window.removeEventListener('resize', handleFixedColumnShadow);
    };
  }, []); // eslint-disable-line
  // Column 变化时重新计算阴影状态

  useEffect(function () {
    handleFixedColumnShadow(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [columns]); // 多级表头的 fixedheader 情况修复

  /** @todo: 通过两层表格的形式替换js计算偏移值 */

  useLayoutEffect(function () {
    if (props.fixedHeader) {
      var domHeader = ref.current;

      if (domHeader) {
        var topVal = 0;
        var domTrList = domHeader.querySelectorAll('tr');
        domTrList.forEach(function (domTr) {
          var iteratorTh = domTr.querySelectorAll('th');
          iteratorTh.forEach(function (domTh) {
            domTh.style.top = topVal + 'px';
          });
          topVal += domTr.offsetHeight;
        });
      }
    }
  }, [props.fixedHeader, ref, props.columns]);
  return {
    // state
    showLeftColumnsShadow: showLeftColumnsShadow,
    showRightColumnsShadow: showRightColumnsShadow,
    container: container,
    tableState: finalTableState,
    expandedRowKeys: finalExpandedRowKeys,
    paginationState: {
      totalRecords: totalRecords,
      current: current,
      pageSize: pageSize
    },
    // computed
    hasFixedLeftColumn: hasFixedLeftColumn,
    columns: finalColumns,
    dataSource: finalDataSource,
    tableLayout: tableLayout,
    allExpandableRowKeys: allExpandableRowKeys,
    tableProps: tableProps,
    spinProps: spinProps,
    // utils
    getRowKey: getRowKey,
    // callback
    handleSort: handleSort,
    handleFilterColumnKeyChange: handleFilterColumnKeyChange,
    handleFilterColumnDataChange: handleFilterColumnDataChange,
    handleFilterColumnListChange: handleFilterColumnListChange,
    handleExpandRow: handleExpandRow,
    handleTableSpinWrapperRef: handleTableSpinWrapperRef,
    handleTableWrapperScroll: handleTableWrapperScroll,
    handlePageSizeChange: handlePageSizeChange,
    handlePageChange: handlePageChange,
    tableWrapperRef: tableWrapperRef,
    headerArr: headerArr,
    renderColumns: renderColumns,
    handleDoubleTableWrapperScroll: handleDoubleTableWrapperScroll,
    doubleTableHeaderWrapperRef: doubleTableHeaderWrapperRef
  };
}