import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import _isArray from "lodash/isArray";
import _concat from "lodash/concat";

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * 获取columns的最深children层数
 *
 * @export
 * @template T
 * @param {ITableColumn<T>[]} columns
 * @returns
 */
export function setParentAndGetDepth(columns) {
  var depth = 0;

  function loop(data, parent, loopDepth) {
    var _loopDepth = loopDepth;

    if (depth >= 500) {
      _devWarning('[Table]: 表头遍历次数超过500次，可能存在循环引用');

      return;
    }

    if (++_loopDepth > depth) {
      depth = _loopDepth;
    }

    var _iterator = _createForOfIteratorHelper(data),
        _step;

    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var item = _step.value;

        if (parent) {
          item.parent = parent;
        }

        if (item && item.children) {
          loop(item.children, item, _loopDepth);
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  }

  loop(columns, undefined, 0);
  return depth;
}
/**
 * 将columns处理为用于渲染header使用和用于渲染body使用的两个数组
 * 如：
 * 1      3      5
 * ├── 2  ├── 4
 *        ├── 6
 *            ├── 7
 * 返回的第一个数组为[[1, 3, 5], [2, 4, 6], [7]]
 * 返回的第二个数组为[2, 4, 7, 5]
 *
 * @export
 * @template T
 * @param {IWrapTableColumn<T>[]} columns
 * @param {number} maxDepth children的最大深度，可以从 setParentAndGetDepth 获得
 * @returns {[IWrapTableColumn<T>[][], IWrapTableColumn<T>[]]}
 */

export function genHeaderArr(columns, maxDepth) {
  var headerArr = [];
  var renderArr = [];

  function loop(data, loopDepth) {
    var arr = [];

    var _iterator2 = _createForOfIteratorHelper(data),
        _step2;

    try {
      for (_iterator2.s(); !(_step2 = _iterator2.n()).done;) {
        var item = _step2.value;

        if (item.children && _isArray(item.children)) {
          item.colspan = 0;
          item.rowspan = 1;
          loop(item.children, loopDepth + 1);
        } else {
          item.rowspan = loopDepth < maxDepth ? 1 + maxDepth - loopDepth : 1;
          item.colspan = 1;
          renderArr.push(item);
          var forkItem = item;

          while (forkItem.parent) {
            forkItem.parent.colspan = forkItem.parent.colspan + 1;
            forkItem = forkItem.parent;
          }
        }

        arr.push(item);
      }
    } catch (err) {
      _iterator2.e(err);
    } finally {
      _iterator2.f();
    }

    headerArr[loopDepth - 1] = _concat([], headerArr[loopDepth - 1] || [], arr);
  }

  loop(columns, 1);
  return [headerArr, renderArr];
}
/** 获得 column 列表，[[columnA, columnB], [columnA, columnC]]
 * 如：
 * 1      3      5
 *        ├── 4
 *        ├── 6
 *            ├── 7
 * 返回为[[1,3,5], [1,4,6,5], [1,4,7,5]]
 *
 * 用于计算宽度
 *
 */

export function getChildIndexes(lastIndexes, column, direction, depth) {
  var currentIdxArr = lastIndexes.map(function (v) {
    return v.slice();
  }).slice();

  function deep(item, idx) {
    if (!currentIdxArr[idx]) {
      currentIdxArr[idx] = [item];
    } else {
      currentIdxArr[idx].push(item);
    }

    if (item.children) {
      (direction === 'left' ? item.children : item.children.slice().reverse()).forEach(function (v) {
        return deep(v, idx + 1);
      });
    } else if (idx < depth - 1) {
      for (var i = idx + 1; i < depth; i++) {
        currentIdxArr[i] ? currentIdxArr[i].push(item) : currentIdxArr[i] = [item];
      }
    }
  }

  deep(column, 0);
  return currentIdxArr;
}
export function getHasSort(sorter, children) {
  return !!sorter && !_isArray(children);
}
export function getHasFilter(filters, renderFilterContent, children) {
  return (!!filters && !!filters.length || !!renderFilterContent) && !_isArray(children);
}
export function sort(props) {
  var data = props.data,
      sorter = props.sorter,
      childrenColumnName = props.childrenColumnName,
      deepSort = props.deepSort;
  var target = data.sort(sorter);

  if (deepSort) {
    var _iterator3 = _createForOfIteratorHelper(target),
        _step3;

    try {
      for (_iterator3.s(); !(_step3 = _iterator3.n()).done;) {
        var item = _step3.value;
        var children = item[childrenColumnName];

        if (children) {
          item[childrenColumnName] = sort({
            data: children,
            sorter: sorter,
            childrenColumnName: childrenColumnName,
            deepSort: deepSort
          });
        }
      }
    } catch (err) {
      _iterator3.e(err);
    } finally {
      _iterator3.f();
    }
  }

  return target;
}