import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["index", "selected", "type", "divider", "buttonType", "size", "htmlType", "disabled", "busy", "component", "prefixNode", "suffixNode", "onClick", "loading", "textFine", "fontWeight", "width", "constant", "href", "target", "rel", "showIndicator", "styles", "children", "tooltipProps", "badgeProps"];
import React, { useMemo } from 'react';
import styled, { css } from 'styled-components';
import { InlineButton } from '../Button';
import addPx from '../utils/addPx';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import { withThemeForStyled } from '../utils/withTheme';
import Tooltip from '../Tooltip';
import Badge from '../Badge';
var InnerButton = styled(InlineButton).withConfig({
  displayName: "InnerButton"
})([""]);
InnerButton.__MUYA_BUTTON = true;
var BaseNode = styled.div.withConfig({
  displayName: "BaseNode"
})([""]);
var defaultStyles = {
  item: ''
};
var TabPure = memoForwardRef(function (props, ref) {
  var index = props.index,
      selected = props.selected,
      type = props.type,
      divider = props.divider,
      buttonTypeProp = props.buttonType,
      size = props.size,
      htmlType = props.htmlType,
      disabled = props.disabled,
      busy = props.busy,
      asProp = props.component,
      prefixNode = props.prefixNode,
      suffixNode = props.suffixNode,
      onClick = props.onClick,
      loading = props.loading,
      textFine = props.textFine,
      fontWeight = props.fontWeight,
      width = props.width,
      constant = props.constant,
      href = props.href,
      target = props.target,
      rel = props.rel,
      showIndicator = props.showIndicator,
      styles = props.styles,
      children = props.children,
      tooltipProps = props.tooltipProps,
      badgeProps = props.badgeProps,
      otherProps = _objectWithoutProperties(props, _excluded);

  var innerStyles = useStyles('tab', defaultStyles, styles);
  var buttonType = useMemo(function () {
    if (selected) {
      return 'primary';
    }

    if (buttonTypeProp) {
      return buttonTypeProp;
    }

    if (type === 'card') {
      return 'normal';
    }

    return 'strong';
  }, [buttonTypeProp, selected, type]);

  var buttonProps = _extends({
    type: buttonType,
    size: size,
    htmlType: htmlType,
    disabled: disabled,
    onClick: onClick,
    busy: busy,
    fontWeight: selected ? 'normal' : 'lighter',
    component: asProp,
    prefixNode: prefixNode,
    suffixNode: suffixNode,
    loading: loading,
    width: width,
    constant: constant,
    href: href,
    target: target,
    rel: rel,
    styles: styles
  }, innerStyles.item);

  var innerButtonNode = /*#__PURE__*/React.createElement(InnerButton, buttonProps, children);

  if (badgeProps) {
    innerButtonNode = /*#__PURE__*/React.createElement(Badge, badgeProps, innerButtonNode);
  }

  if (tooltipProps) {
    if (disabled) {
      buttonProps.component = 'span';
    }

    innerButtonNode = /*#__PURE__*/React.createElement(Tooltip, tooltipProps, innerButtonNode);
  }

  return /*#__PURE__*/React.createElement(BaseNode, _extends({}, otherProps, {
    ref: ref
  }), innerButtonNode);
});

function lineTabCss(props) {
  var theme = props.theme,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      selected = props.selected,
      _props$showIndicator = props.showIndicator,
      showIndicator = _props$showIndicator === void 0 ? false : _props$showIndicator,
      tabPosition = props.tabPosition;
  var token = theme.components.Tabs;
  var height = token.height[size];
  var padding = token.padding[size];
  var paddingCss;

  if (padding) {
    paddingCss = css(["padding:", ";"], padding);
  }

  var selectedCss;

  if (selected && showIndicator) {
    selectedCss = css(["&::before{content:'';display:block;position:absolute;bottom:0;height:", "px;background:", ";width:100%;}"], token.indicator.activeHeight, token.indicator.activeColor);
  }

  return css(["& ", "{min-height:", ";", "}position:relative;", ":", "px;", ""], InnerButton, addPx(height), paddingCss, tabPosition === 'right' ? 'margin-left' : 'margin-right', token.marginRight[size], selectedCss);
}

function cardTabCss(props) {
  var theme = props.theme,
      _props$size2 = props.size,
      size = _props$size2 === void 0 ? 'm' : _props$size2,
      selected = props.selected,
      disabled = props.disabled,
      busy = props.busy,
      _props$divider = props.divider,
      divider = _props$divider === void 0 ? 'on' : _props$divider;
  var colors = theme.colors;
  var card = theme.components.Tabs.card;
  var height = card.height[size];
  var hasMargin = card.marginRight > 0;
  var interactive = !(busy || disabled);
  var statusCss;

  if (selected) {
    statusCss = css(["border-color:", ";border-bottom:1px solid transparent;border-top-left-radius:", ";border-top-right-radius:", ";background-color:", ";z-index:1;"], colors.pattern.border.normal, card.borderRadius, card.borderRadius, card.bgSelectedColor);
  } else {
    statusCss = css(["background-color:", ";border-bottom-color:", ";", " ", ""], card.bgColor, colors.pattern.border.normal, hasMargin && css(["border-color:", ";border-bottom:1px solid transparent;border-top-left-radius:", ";border-top-right-radius:", ";"], colors.pattern.border.normal, card.borderRadius, card.borderRadius), interactive && css(["&:hover{background-color:", ";", "{color:", ";}}"], card.bgHoverColor, InnerButton, card.hoverColor));
  }

  var orderCss;
  var padding = card.padding[size];

  if (divider === 'on') {
    var dividerSize = card.dividerSize[size];
    orderCss = css(["&::before{content:'';display:block;position:absolute;top:", "px;right:-1px;height:", "px;width:1px;background:", ";}"], (height - dividerSize) / 2, dividerSize, colors.pattern.border.normal);
  }

  return css(["position:relative;border:1px solid transparent;margin-right:", "px;& ", "{height:", "px;padding:0 ", "px;line-height:1;}", " ", " &:first-child{border-top-left-radius:", ";}&:last-child{border-top-right-radius:", ";margin-right:0;}"], card.marginRight, InnerButton, height, padding, statusCss, orderCss, card.borderRadius, card.borderRadius);
}

function capsuleTabCss(props) {
  var theme = props.theme,
      _props$size3 = props.size,
      size = _props$size3 === void 0 ? 'm' : _props$size3,
      selected = props.selected,
      disabled = props.disabled,
      busy = props.busy,
      tabPosition = props.tabPosition;
  var token = theme.components.Tabs;
  var interactive = !(busy || disabled);
  var height = token.height[size];
  var borderRadius = token.capsule.inlineBtnRadius || token.capsule.radius;
  var clickColor = token.capsule.clickColor;
  var inlineBtnCss = css(["background-color:", ";min-height:", ";border-radius:", ";padding:0 ", ";", ""], token.capsule.btnBgColor.normal, addPx(height), borderRadius[size], token.capsule.btnPadding[size], function () {
    if (!selected) {
      return css(["background:", ";", ""], token.capsule.btnBgColor.normal, interactive && css(["color:", ";&:hover{background:", ";}&:active{", " background:", ";}"], token.capsule.color, token.capsule.btnBgColor.hover, !!clickColor && css(["color:", ";"], token.capsule.clickColor), token.capsule.btnBgColor.click));
    } else {
      return css(["", " background:", ";"], !!clickColor && css(["color:", ";"], token.capsule.clickColor), token.capsule.btnBgColor.click);
    }
  });
  return css(["& ", "{", "}position:relative;&:not(:last-child){", ":", ";}"], InnerButton, inlineBtnCss, tabPosition === 'top' || tabPosition === 'bottom' ? 'margin-right' : 'margin-bottom', token.capsule.bgPadding[size]);
}

var Tab = styled(TabPure).withConfig({
  displayName: "Tab"
})(["box-sizing:border-box;", ""], function (props) {
  if (props.type === 'card') {
    return cardTabCss(props);
  } else if (props.type === 'capsule') {
    return capsuleTabCss(props);
  } else {
    return lineTabCss(props);
  }
});
export default withThemeForStyled(Tab);