import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _uniq from "lodash/uniq";
import _pick from "lodash/pick";
import _get from "lodash/get";
import _findIndex from "lodash/findIndex";
var _excluded = ["visible", "children", "keepMounted", "theme"],
    _excluded2 = ["type", "panelAnimation", "defaultIndex", "panelTriggerResize", "panelKeepMounted", "tabPosition", "style"];
import React, { useContext, useEffect, useMemo } from 'react';
import styled, { css } from 'styled-components';
import Animation from '../Animation';
import memoForwardRef from '../utils/memoForwardRef';
import { withProperties } from '../utils/type-helper';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
import CardTabs from './CardTabs';
import { defaultStyles } from './const';
import LineTabs from './LineTabs';
import { BaseNode, ExtraContentWrapper, StyledPanelDefaultContainer, StyledPanelSwipeContainer, StyledTabsWrapper } from './styled';
import Tab from './Tab';
import TabPanel from './TabPanel';
import { ImgPoolContext } from '../Img';
import CapsuleTabs from './CapsuleTabs';

var PanelItem = function PanelItem(props) {
  var visible = props.visible,
      children = props.children,
      type = props.type,
      theme = props.theme,
      _props$keepMounted = props.keepMounted,
      keepMounted = _props$keepMounted === void 0 ? false : _props$keepMounted;
  var hasVisibleRef = React.useRef(false);
  var mountedEl = /*#__PURE__*/React.createElement(BaseNode, {
    style: visible ? {} : {
      display: 'none'
    }
  }, type === 'fade' ? /*#__PURE__*/React.createElement(Animation.Fade, {
    "in": visible,
    timeout: theme.components.Tabs.panel.fade
  }, children) : /*#__PURE__*/React.createElement(React.Fragment, null, children));

  if (!keepMounted) {
    if (!visible) {
      return null;
    }

    if (type === 'fade') {
      return /*#__PURE__*/React.createElement(Animation.Fade, {
        "in": true,
        timeout: theme.components.Tabs.panel.fade
      }, children);
    } else {
      return /*#__PURE__*/React.createElement(React.Fragment, null, children);
    }
  } else if (keepMounted === 'lazy') {
    if (hasVisibleRef.current) {
      return mountedEl;
    } else if (visible) {
      hasVisibleRef.current = true;
      return mountedEl;
    } else {
      return null;
    }
  } else {
    return mountedEl;
  }
};

var SwipePanelItem = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var visible = props.visible,
      children = props.children,
      _props$keepMounted2 = props.keepMounted,
      keepMounted = _props$keepMounted2 === void 0 ? false : _props$keepMounted2,
      theme = props.theme,
      otherProps = _objectWithoutProperties(props, _excluded);

  var hasVisibleRef = React.useRef(false);
  var childNode = null;

  if (!keepMounted) {
    if (visible) {
      childNode = children;
    } else {
      childNode = null;
    }
  } else if (keepMounted === 'lazy') {
    if (hasVisibleRef.current) {
      childNode = children;
    } else if (visible) {
      hasVisibleRef.current = true;
      childNode = children;
    }
  } else {
    childNode = children;
  }

  return /*#__PURE__*/React.createElement("div", _extends({}, otherProps, {
    ref: ref
  }), childNode);
});
var TabsPure = memoForwardRef(function (props, ref) {
  var _props$type = props.type,
      type = _props$type === void 0 ? 'line' : _props$type,
      _props$panelAnimation = props.panelAnimation,
      panelAnimation = _props$panelAnimation === void 0 ? 'static' : _props$panelAnimation,
      defaultIndex = props.defaultIndex,
      _props$panelTriggerRe = props.panelTriggerResize,
      panelTriggerResize = _props$panelTriggerRe === void 0 ? false : _props$panelTriggerRe,
      panelKeepMountedProp = props.panelKeepMounted,
      tabPositionProp = props.tabPosition,
      style = props.style,
      otherProps = _objectWithoutProperties(props, _excluded2);

  var tabPosition = useMemo(function () {
    if (typeof tabPositionProp === 'undefined' || type === 'card') {
      return 'top';
    }

    return tabPositionProp;
  }, [tabPositionProp, type]);
  var theme = useTheme();
  var panelKeepMounted = useMemo(function () {
    // panelKeepMounted 计算逻辑，swipe 时默认值为 true，其他为 false
    if (typeof panelKeepMountedProp === 'undefined') {
      if (panelAnimation === 'swipe') {
        return true;
      } else {
        return false;
      }
    }

    return panelKeepMountedProp;
  }, [panelAnimation, panelKeepMountedProp]);
  var vertical = useMemo(function () {
    return tabPosition === 'left' || tabPosition === 'right';
  }, [tabPosition]);
  var tabPanelChildrenArr = React.useMemo(function () {
    return React.Children.toArray(props.children).filter(function (child) {
      return /*#__PURE__*/React.isValidElement(child) && child.type.__MUYA_TABPANEL;
    });
  }, [props.children]);

  var _React$useState = React.useState(tabPanelChildrenArr.length ? 'tabPanel' : 'tab'),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      renderMode = _React$useState2[0],
      setRenderMode = _React$useState2[1];

  var _React$useState3 = React.useState(defaultIndex || _get(tabPanelChildrenArr, '0.props.index', undefined)),
      _React$useState4 = _slicedToArray(_React$useState3, 2),
      _currentIdxKey = _React$useState4[0],
      setCurrentIdxKey = _React$useState4[1];

  var _React$useState5 = React.useState([_currentIdxKey]),
      _React$useState6 = _slicedToArray(_React$useState5, 2),
      initialIdxKeys = _React$useState6[0],
      setInitialIdxKeys = _React$useState6[1]; // 缓存上次选中的 tab，用于 swipe 动画


  var _React$useState7 = React.useState(undefined),
      _React$useState8 = _slicedToArray(_React$useState7, 2),
      preIdxKey = _React$useState8[0],
      setPreIdxKey = _React$useState8[1];

  var clearPreIdxKey = React.useCallback(function () {
    setPreIdxKey(undefined);
  }, [setPreIdxKey]); // TabPanel.props中的index的值

  var currentIdxKey = 'index' in props ? props['index'] : _currentIdxKey; // 当前选中currenIdxKey在tabPanelChildrenArr中的索引值，用于swipe定位

  var currentIdx = React.useMemo(function () {
    var _currentIdx = _findIndex(tabPanelChildrenArr, function (child) {
      return child.props.index === currentIdxKey;
    });

    return _currentIdx;
  }, [currentIdxKey, tabPanelChildrenArr]);
  var enableBadge = React.useMemo(function () {
    var badgeTabLen = 0;

    if (renderMode === 'tabPanel') {
      badgeTabLen = tabPanelChildrenArr.filter(function (child) {
        return !!_get(child, 'props.tabProps.badgeProps', false);
      }).length;
    } else {
      badgeTabLen = React.Children.toArray(props.children).filter(function (child) {
        return /*#__PURE__*/React.isValidElement(child) && child.props.badgeProps;
      }).length;
    }

    return !!badgeTabLen;
  }, [props.children, renderMode, tabPanelChildrenArr]); // 为了兼容只对开启了 badge 的 Tab 设置 overflow

  var innerStyle = useMemo(function () {
    if (!enableBadge) {
      return _extends({
        overflow: 'hidden'
      }, style);
    }

    return style;
  }, [enableBadge, style]);
  var innerStyles = useStyles('tabs', defaultStyles, props.styles);
  var pool = useContext(ImgPoolContext);
  /** 用于切换 tab 时触发 resize 重新计算 */

  useEffect(function () {
    if (panelTriggerResize) {
      pool.throttleCheck({
        type: 'resize'
      });
    }
  }, [currentIdxKey, panelTriggerResize, pool]);
  useEffect(function () {
    // children中任一节点为TabPanel则认为开启了tabPanel模式
    if (tabPanelChildrenArr.length && renderMode === 'tab') {
      setRenderMode('tabPanel');
    } else if (!tabPanelChildrenArr.length && renderMode === 'tabPanel') {
      setRenderMode('tab');
    }

    var tabPanelKeyUniq = _uniq(tabPanelChildrenArr.map(function (child) {
      return child.props.index;
    }));

    if (tabPanelKeyUniq.length !== tabPanelChildrenArr.length) {
      _devWarning('[TabPanel]: duplicate indexes');
    }
  }, [props.children, renderMode, tabPanelChildrenArr]);
  var wrapChange = React.useCallback(function (index) {
    setPreIdxKey(currentIdxKey);
    setCurrentIdxKey(index);
    setInitialIdxKeys(_uniq([].concat(_toConsumableArray(initialIdxKeys), [index])));
    props.onChange && props.onChange(index);
  }, [currentIdxKey, initialIdxKeys, props]);

  if (renderMode === 'tab') {
    if (type === 'card') {
      return /*#__PURE__*/React.createElement(CardTabs, _extends({
        ref: ref,
        type: "card",
        defaultIndex: defaultIndex,
        enableBadge: enableBadge,
        vertical: vertical,
        tabPosition: tabPosition,
        style: innerStyle
      }, otherProps));
    } else if (type === 'capsule') {
      return /*#__PURE__*/React.createElement(CapsuleTabs, _extends({
        ref: ref,
        type: "capsule",
        defaultIndex: defaultIndex,
        enableBadge: enableBadge,
        vertical: vertical,
        tabPosition: tabPosition,
        style: innerStyle
      }, otherProps));
    } else {
      return /*#__PURE__*/React.createElement(LineTabs, _extends({
        ref: ref,
        type: "line",
        defaultIndex: defaultIndex,
        enableBadge: enableBadge,
        vertical: vertical,
        tabPosition: tabPosition,
        style: innerStyle
      }, otherProps));
    }
  } else {
    var TypedTabs = LineTabs;

    switch (type) {
      case 'card':
        TypedTabs = CardTabs;
        break;

      case 'line':
        TypedTabs = LineTabs;
        break;

      case 'capsule':
        TypedTabs = CapsuleTabs;
        break;

      default:
        break;
    }

    var tabsNode = /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(TypedTabs, _extends({
      ref: ref,
      type: type
    }, otherProps, {
      onChange: wrapChange,
      index: currentIdxKey,
      enableBadge: enableBadge,
      vertical: vertical,
      tabPosition: tabPosition,
      style: innerStyle
    }), tabPanelChildrenArr.map(function (child) {
      var _child$props = child.props,
          tab = _child$props.tab,
          index = _child$props.index,
          tabProps = _child$props.tabProps;
      return /*#__PURE__*/React.createElement(Tab, _extends({
        key: index,
        index: index
      }, tabProps), tab);
    })), panelAnimation === 'swipe' ? /*#__PURE__*/React.createElement(StyledPanelSwipeContainer, _extends({
      theme: theme,
      equalNum: 1,
      stepIndex: currentIdx // TODO: 去掉 swipePanel，把styles.panel提升到这层节点
      ,
      styles: _pick(props.styles, 'panel'),
      onTransitionEnd: clearPreIdxKey
    }, innerStyles.swipePanel), tabPanelChildrenArr.map(function (child) {
      return /*#__PURE__*/React.createElement(SwipePanelItem, {
        key: child.props.index,
        keepMounted: panelKeepMounted,
        visible: panelKeepMounted === 'lazy' ? initialIdxKeys.includes(child.props.index) : child.props.index === currentIdxKey || child.props.index === preIdxKey,
        theme: theme
      }, child);
    })) : /*#__PURE__*/React.createElement(StyledPanelDefaultContainer, _extends({}, innerStyles.panel, {
      theme: theme
    }), tabPanelChildrenArr.map(function (child) {
      return /*#__PURE__*/React.createElement(PanelItem, {
        type: panelAnimation,
        visible: child.props.index === currentIdxKey,
        key: child.props.index,
        theme: theme,
        keepMounted: panelKeepMounted
      }, child);
    })));

    if (tabPosition !== 'top') {
      return /*#__PURE__*/React.createElement(StyledTabsWrapper, _extends({}, innerStyles.wrapper, {
        $tabPosition: tabPosition
      }), tabsNode);
    } else {
      return tabsNode;
    }
  }
});
var TabsWithNoTheme = styled(TabsPure).withConfig({
  displayName: "TabsWithNoTheme"
})(["position:relative;display:flex;", ""], function (props) {
  var theme = props.theme,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$type2 = props.type,
      type = _props$type2 === void 0 ? 'line' : _props$type2,
      _props$tabPosition = props.tabPosition,
      tabPosition = _props$tabPosition === void 0 ? 'top' : _props$tabPosition;
  var token = theme.components.Tabs;
  var padding = token.padding[size];
  var paddingCss = css(["padding-bottom:", "px;"], token.indicator.bgHeight); // line indicator 需要为extraContent增加paddingBottom，确保内容对齐

  if (type === 'line' && padding) {
    paddingCss = css(["padding:", ";"], padding);
  }

  return css(["", " ", " ", " & ", "{", ";}"], (tabPosition === 'right' || tabPosition === 'bottom') && 'order: 1;', (tabPosition === 'left' || tabPosition === 'right') && 'width: fit-content;', (tabPosition === 'top' || tabPosition === 'bottom') && 'align-items: center;', ExtraContentWrapper, paddingCss);
});
var Tabs = withThemeForStyled(TabsWithNoTheme);
export default withProperties(Tabs, {
  Tab: Tab,
  TabPanel: TabPanel
});