import { RightIcon as _RightIcon } from "@muya-ui/theme-light/es/icons/RightIcon";
import { LeftIcon as _LeftIcon } from "@muya-ui/theme-light/es/icons/LeftIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["type", "size", "htmlType", "disabled", "busy", "component", "children", "prefixNode", "suffixNode", "onClick"],
    _excluded2 = ["children", "onChange", "height", "size", "type", "swipe", "selectedIndex", "enableDiffChildren", "equalNum", "gutter", "duration"];
import React, { useContext, useEffect } from 'react';
import styled, { css } from 'styled-components';
import IconButton from '../IconButton';
import ImgPoolContext from '../Img/ImgPoolContext';
import Swipe from '../Swipe';
import addPx from '../utils/addPx';
import memoForwardRef from '../utils/memoForwardRef';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
import useTabsContainer from './useTabsContainer';
var InnerButton = styled(IconButton).withConfig({
  displayName: "InnerButton"
})([""]);
var StyledSwipe = styled(Swipe).withConfig({
  displayName: "StyledSwipe"
})(["z-index:1;"]);
var BaseNode = styled.div.withConfig({
  displayName: "BaseNode"
})([""]);
var ButtonContainerPure = memoForwardRef(function (props, ref) {
  var type = props.type,
      size = props.size,
      htmlType = props.htmlType,
      disabled = props.disabled,
      busy = props.busy,
      asProp = props.component,
      children = props.children,
      prefixNode = props.prefixNode,
      suffixNode = props.suffixNode,
      onClick = props.onClick,
      otherProps = _objectWithoutProperties(props, _excluded);

  return /*#__PURE__*/React.createElement(BaseNode, _extends({}, otherProps, {
    ref: ref
  }), /*#__PURE__*/React.createElement(InnerButton, {
    onClick: onClick,
    busy: busy
  }, children));
});
var ButtonContainer = styled(ButtonContainerPure).withConfig({
  displayName: "ButtonContainer"
})(["", ""], function (props) {
  var theme = props.theme,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      type = props.type;
  var token = theme.components.Tabs;
  var height = type === 'line' ? token.height[size] : token.card.height[size];
  var padding = token.padding[size];
  var paddingCss;

  if (type === 'line' && padding) {
    paddingCss = css(["padding:", ";"], padding);
  }

  return css(["& ", "{width:", "px;min-height:", ";align-items:center;justify-content:center;", "}"], InnerButton, height, addPx(height), paddingCss);
});
var TabsContainerPure = memoForwardRef(function (props, ref) {
  var children = props.children,
      onChange = props.onChange,
      height = props.height,
      _props$size2 = props.size,
      size = _props$size2 === void 0 ? 'm' : _props$size2,
      type = props.type,
      swipe = props.swipe,
      selectedIndex = props.selectedIndex,
      enableDiffChildren = props.enableDiffChildren,
      equalNum = props.equalNum,
      gutter = props.gutter,
      duration = props.duration,
      otherProps = _objectWithoutProperties(props, _excluded2);

  var _useTabsContainer = useTabsContainer({
    swipe: !!swipe,
    selectedIndex: selectedIndex,
    onChange: onChange
  }),
      onPrev = _useTabsContainer.onPrev,
      onNext = _useTabsContainer.onNext,
      onStepsChange = _useTabsContainer.onStepsChange,
      hasNext = _useTabsContainer.hasNext,
      hasPrev = _useTabsContainer.hasPrev,
      stepIndex = _useTabsContainer.stepIndex,
      onTransitionEnd = _useTabsContainer.onTransitionEnd;

  var theme = useTheme();
  var imgPool = useContext(ImgPoolContext);
  var swipable = hasNext || hasPrev;
  var token = theme.components.Tabs;

  var tokenIcon = _extends({
    prev: _LeftIcon,
    next: _RightIcon
  }, token.icon);

  var PrevIcon = tokenIcon.prev;
  var NextIcon = tokenIcon.next;
  var iconSize = token.iconSize[size];
  var iconStyle = {
    width: iconSize,
    height: iconSize
  };
  var offSwipe = !swipe;
  var prevBtn = swipable && !offSwipe ? /*#__PURE__*/React.createElement(ButtonContainer, {
    size: size,
    theme: theme,
    type: type,
    onClick: onPrev,
    busy: !hasPrev
  }, /*#__PURE__*/React.createElement(PrevIcon, {
    style: iconStyle
  })) : null;
  var nextBtn = swipable && !offSwipe ? /*#__PURE__*/React.createElement(ButtonContainer, {
    size: size,
    theme: theme,
    type: type,
    onClick: onNext,
    busy: !hasNext
  }, /*#__PURE__*/React.createElement(NextIcon, {
    style: iconStyle
  })) : null;
  var innerChildren = children;

  if (!offSwipe) {
    var swipeProps = {
      enableDiffChildren: enableDiffChildren,
      equalNum: equalNum,
      gutter: gutter,
      duration: duration
    };
    innerChildren = /*#__PURE__*/React.createElement(StyledSwipe, _extends({}, swipeProps, {
      onTransitionEnd: onTransitionEnd,
      onStepsChange: onStepsChange,
      stepIndex: stepIndex
    }), children);
  }
  /**
   * 左右箭头是计算得到的，因此容器尺寸可能会出现变化，如果 swipable 变为了 true，那么需要出发一次 resize，使得 swipe 重新计算尺寸
   */


  useEffect(function () {
    if (swipe && swipable) {
      imgPool.throttleCheck({
        type: 'resize'
      });
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [swipable]);
  return /*#__PURE__*/React.createElement(BaseNode, _extends({}, otherProps, {
    ref: ref
  }), prevBtn, innerChildren, nextBtn);
}); // margin 和 padding 使 badge 不会被 overflow 裁掉

var TabsContainer = styled(TabsContainerPure).withConfig({
  displayName: "TabsContainer"
})(["display:flex;flex-wrap:nowrap;white-space:nowrap;align-items:center;flex:1;overflow:hidden;z-index:1;", ""], function (props) {
  var height = props.height,
      _props$type = props.type,
      type = _props$type === void 0 ? 'line' : _props$type,
      theme = props.theme,
      _props$size3 = props.size,
      size = _props$size3 === void 0 ? 'm' : _props$size3,
      enableBadge = props.enableBadge,
      vertical = props.vertical;
  var components = theme.components;
  var token = components.Tabs;
  var calcCss;
  var barCss;
  var badgePatchCss;
  var verticalCss;

  if (height) {
    calcCss = css(["height:", "px;align-items:center;"], height);
  }

  if (type === 'capsule') {
    barCss = css(["background:", ";padding:", ";border-radius:", ";"], token.capsule.bgColor, token.capsule.bgPadding[size], token.capsule.radius[size]);
  }

  if (enableBadge) {
    badgePatchCss = css(["padding-top:20px;margin-top:-20px;"]);
  }

  if (vertical) {
    verticalCss = css(["flex-direction:column;"]);
  }

  return css(["", " ", " ", " ", ""], verticalCss, badgePatchCss, calcCss, barCss);
});
export default withThemeForStyled(TabsContainer);