import _extends from "@babel/runtime/helpers/extends";
import { forkRef as _forkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
var _excluded = ["disabled", "busy", "size", "index", "onClick", "href", "showIndicator", "className", "style"];
import React from 'react';
import useStyles from '../utils/useStyles';
import { defaultStyles } from './const';
import mergeStyleItem from '../utils/mergeStyleItem';
export function getItemDividerStatus(configs, currentIndex) {
  var currentItem = configs[currentIndex];

  if (currentItem.selected) {
    return 'off';
  }

  var lastItemIndex = configs.length - 1;

  if (currentIndex === lastItemIndex) {
    return 'off';
  }

  var nextItem = configs[currentIndex + 1];

  if (!currentItem.selected && nextItem && nextItem.selected) {
    return 'off';
  }
}
export default function useTabsBase(props) {
  var showIndicator = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  var type = props.type,
      index = props.index,
      defaultIndex = props.defaultIndex,
      size = props.size,
      disabled = props.disabled,
      busy = props.busy,
      onChange = props.onChange,
      styles = props.styles,
      tabPosition = props.tabPosition,
      children = props.children;
  var indexFromProps = React.useMemo(function () {
    if (index !== undefined) {
      return index;
    }

    if (defaultIndex !== undefined) {
      return defaultIndex;
    }

    return 0;
  }, [defaultIndex, index]);

  var _React$useState = React.useState(indexFromProps),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      stateIndex = _React$useState2[0],
      setStateIndex = _React$useState2[1];

  var finalIndex = 'index' in props ? indexFromProps : stateIndex;
  var innerStyles = useStyles('tabs', defaultStyles, styles);
  var updateIndex = React.useCallback(function (newIndex) {
    setStateIndex(newIndex);

    if (onChange) {
      onChange(newIndex);
    }
  }, [onChange]);
  var elementsFromChildren = React.useMemo(function () {
    var elements = [];
    React.Children.forEach(children, function (child) {
      if (! /*#__PURE__*/React.isValidElement(child)) {
        return;
      }

      elements.push(child);
    });
    return elements;
  }, [children]); // type === 'card' 时，要计算是否有 divider

  var _React$useMemo = React.useMemo(function () {
    var selectedIndex = -1;
    var configs = elementsFromChildren.map(function (child, childIndex) {
      var childValue = child.props.index;
      var currentValue = childValue !== undefined ? childValue : childIndex;
      var selected = finalIndex === currentValue;

      if (selected) {
        selectedIndex = childIndex;
      }

      var config = {
        selected: selected,
        index: currentValue
      };
      return config;
    });
    return {
      itemConfigs: configs,
      selectedIndex: selectedIndex
    };
  }, [elementsFromChildren, finalIndex]),
      itemConfigs = _React$useMemo.itemConfigs,
      selectedIndex = _React$useMemo.selectedIndex;

  var _React$useMemo2 = React.useMemo(function () {
    var innerItems = [];
    var newChildren = elementsFromChildren.map(function (child, childIndex) {
      var _child$props = child.props,
          childDisabled = _child$props.disabled,
          childBusy = _child$props.busy,
          childSize = _child$props.size,
          cIndex = _child$props.index,
          _onClick = _child$props.onClick,
          childHref = _child$props.href,
          childShowIndicator = _child$props.showIndicator,
          className = _child$props.className,
          style = _child$props.style,
          otherChildProps = _objectWithoutProperties(_child$props, _excluded);

      var _itemConfigs$childInd = itemConfigs[childIndex],
          currentValue = _itemConfigs$childInd.index,
          selected = _itemConfigs$childInd.selected;
      var newChildKey = child.key || currentValue;
      var currentDisabled = childDisabled || disabled;
      var innerChildRef = /*#__PURE__*/React.createRef();

      var handleChildRef = _forkRef(child.ref, innerChildRef);

      innerItems.push({
        ref: innerChildRef,
        selected: selected
      });
      var divider; // type === 'card' 时，选中状态、最后一个、下一个选中都没有风格符

      if (type === 'card') {
        divider = getItemDividerStatus(itemConfigs, childIndex);
      }

      var tabStyleItem = mergeStyleItem({
        className: className,
        style: style
      }, innerStyles.tab);
      var finalTabStyleItem = selected ? mergeStyleItem(tabStyleItem, innerStyles.activeTab) : tabStyleItem;

      var newProps = _extends({}, otherChildProps, {
        type: type,
        disabled: currentDisabled,
        busy: childBusy || busy,
        selected: selected,
        divider: divider,
        index: currentValue,
        href: childHref,
        onClick: function onClick(e) {
          if (!selected && !childHref) {
            updateIndex(currentValue);
          }

          _onClick && _onClick(e);
        },
        size: size,
        tabPosition: tabPosition
      }, finalTabStyleItem);

      newProps.ref = handleChildRef;
      newProps.key = newChildKey;

      if (type !== 'card') {
        newProps.showIndicator = childShowIndicator !== undefined ? childShowIndicator : showIndicator;
      }

      return /*#__PURE__*/React.cloneElement(child, newProps);
    });
    return {
      resultChildren: newChildren,
      innerItems: innerItems
    };
  }, [busy, disabled, elementsFromChildren, innerStyles.activeTab, innerStyles.tab, itemConfigs, showIndicator, size, tabPosition, type, updateIndex]),
      resultChildren = _React$useMemo2.resultChildren,
      innerItems = _React$useMemo2.innerItems;

  return {
    items: innerItems,
    children: resultChildren,
    selectedIndex: selectedIndex,
    finalIndex: finalIndex,
    updateIndex: updateIndex
  };
}