import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import { useEffect, useRef } from 'react';
import { useSwipe } from '../Swipe';
export default function useTabsContainer(props) {
  var swipe = props.swipe,
      selectedIndex = props.selectedIndex,
      onChange = props.onChange;

  var _useSwipe = useSwipe(),
      onPrev = _useSwipe.onPrev,
      onNext = _useSwipe.onNext,
      onGoTo = _useSwipe.onGoTo,
      onStepsChange = _useSwipe.onStepsChange,
      getItemStep = _useSwipe.getItemStep,
      hasNext = _useSwipe.hasNext,
      hasPrev = _useSwipe.hasPrev,
      stepIndex = _useSwipe.stepIndex,
      manager = _useSwipe.manager;

  var containerOffset = useRef(0);

  var updateContainerOffset = function updateContainerOffset() {
    if (manager) {
      containerOffset.current = manager.offset;
    }
  };

  var btnPrev = _useEventCallback(function () {
    updateContainerOffset();
    onPrev();
  });

  var btnNext = _useEventCallback(function () {
    updateContainerOffset();
    onNext();
  });

  var innerOnStepsChange = function innerOnStepsChange(manager) {
    onStepsChange(manager);
    updateContainerOffset();
    onChange && onChange();
  };

  var onTransitionEnd = function onTransitionEnd() {
    updateContainerOffset();
    onChange && onChange();
  };

  useEffect(function () {
    if (swipe) {
      setTimeout(function () {
        var newStepIndex = getItemStep(selectedIndex);

        if (newStepIndex !== stepIndex) {
          updateContainerOffset();
          onGoTo(newStepIndex);
        }
      }, 500);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [selectedIndex]);
  useEffect(function () {
    if (manager) {
      var newOffset = manager.offset;

      if (newOffset !== containerOffset.current) {
        onChange && onChange(newOffset - containerOffset.current);
        containerOffset.current = newOffset;
      }
    }
  });
  return {
    onStepsChange: innerOnStepsChange,
    onPrev: btnPrev,
    onNext: btnNext,
    hasNext: hasNext,
    hasPrev: hasPrev,
    stepIndex: stepIndex,
    onTransitionEnd: onTransitionEnd
  };
}