import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { CloseIcon as _CloseIcon } from "@muya-ui/theme-light/es/icons/CloseIcon";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["visible", "closable", "children", "color", "shape", "size", "bordered", "colorInverse", "disabled", "borderColor", "maxWidth", "styles", "onClick", "onClose"];
import React, { useCallback, useMemo, useRef, useState } from 'react';
import styled from 'styled-components';
import Animation from '../Animation';
import memoForwardRef from '../utils/memoForwardRef';
import useAdjustMiniFont from '../utils/useAdjustMiniFont';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
import { StyledCloseIcon, StyledTagText, tagCss } from './styled';
var BaseNode = styled.span.withConfig({
  displayName: "BaseNode"
})([""]);
var defaultStyles = {
  childrenWrapper: '',
  closeIcon: ''
};
var PureTag = memoForwardRef(function (props, ref) {
  var _props$visible = props.visible,
      visible = _props$visible === void 0 ? true : _props$visible,
      closable = props.closable,
      children = props.children,
      color = props.color,
      shape = props.shape,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      bordered = props.bordered,
      colorInverse = props.colorInverse,
      disabled = props.disabled,
      borderColor = props.borderColor,
      maxWidth = props.maxWidth,
      styles = props.styles,
      onClick = props.onClick,
      onClose = props.onClose,
      restProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var token = theme.components.Tag;
  var isVisibleControlled = ('visible' in props);
  var fontSize = token.fontSize[size];
  var textRef = useRef(null);
  var _theme$components$Tag = theme.components.Tag.closeIcon,
      CloseIcon = _theme$components$Tag === void 0 ? _CloseIcon : _theme$components$Tag;

  var _useState = useState(visible),
      _useState2 = _slicedToArray(_useState, 2),
      visibleState = _useState2[0],
      setVisibleState = _useState2[1];

  useAdjustMiniFont(textRef, fontSize);
  var innerStyles = useStyles('tag', defaultStyles, styles);
  var handleClick = useCallback(function (e) {
    if (!disabled) {
      if (onClick) {
        onClick(e);
      }
    } else {
      e.preventDefault();
    }
  }, [disabled, onClick]);
  var handleCloseIconClick = useCallback(function (e) {
    e.stopPropagation();

    if (!disabled) {
      /* istanbul ignore else */
      if (onClose) {
        onClose(e);
      }

      if (e.defaultPrevented) {
        return;
      }
      /* istanbul ignore else */


      if (!isVisibleControlled) {
        setVisibleState(false);
      }
    } else {
      e.preventDefault();
    }
  }, [disabled, isVisibleControlled, onClose]);
  var handleEntered = useCallback(function (node) {
    /* istanbul ignore next */
    if (node && node.style) {
      node.style.display = '';
    }
  }, []);
  var handleExited = useCallback(function (node) {
    /* istanbul ignore next */
    if (node && node.style) {
      node.style.display = 'none';
    }
  }, []);

  if (isVisibleControlled && visible !== visibleState) {
    setVisibleState(visible);
  }

  var TagNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(BaseNode, _extends({
      ref: ref,
      onClick: handleClick
    }, restProps), /*#__PURE__*/React.createElement(StyledTagText, _extends({
      ref: textRef
    }, innerStyles.childrenWrapper), children), closable && /*#__PURE__*/React.createElement(StyledCloseIcon, _extends({}, innerStyles.closeIcon, {
      theme: theme,
      $size: size,
      $color: color,
      $colorInverse: colorInverse
    }), /*#__PURE__*/React.createElement(CloseIcon, {
      onClick: handleCloseIconClick
    })));
  }, [children, closable, color, colorInverse, handleClick, handleCloseIconClick, innerStyles.childrenWrapper, innerStyles.closeIcon, ref, restProps, size, theme]);

  if (disabled) {
    return TagNode;
  }

  return /*#__PURE__*/React.createElement(Animation.Fade, {
    "in": visibleState,
    onEntered: handleEntered,
    onExited: handleExited
  }, TagNode);
});
var Tag = styled(PureTag).withConfig({
  displayName: "Tag"
})(["", ";"], tagCss);
export default withThemeForStyled(Tag);