import _extends from "@babel/runtime/helpers/extends";
import _padStart from "lodash/padStart";
import React, { useMemo } from 'react';
import styled, { css } from 'styled-components';
import { scrollBarStyle } from '../ScrollView/styled';
import memoForwardRef from '../utils/memoForwardRef';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import useTimePickerNumberList from './useTimePickerNumberList';
var StyledRow = styled.div.withConfig({
  displayName: "StyledRow"
})([""]);
var StyledLabel = styled.div.withConfig({
  displayName: "StyledLabel"
})([""]);
var StyledItem = styled.div.withConfig({
  displayName: "StyledItem"
})(["", ""], function (_ref) {
  var theme = _ref.theme,
      $disabled = _ref.$disabled,
      _ref$$selected = _ref.$selected,
      $selected = _ref$$selected === void 0 ? false : _ref$$selected;
  var token = theme.components.TimePicker;
  var itemCss;

  if ($disabled) {
    itemCss = css(["color:", ";cursor:not-allowed;"], token.itemColor.disabled);
  } else if ($selected) {
    itemCss = css(["background-color:", ";color:", ";border-radius:", ";"], token.itemBg.selected, token.itemColor.selected, theme.size.spec.borderRadius.s1);
  } else {
    itemCss = css(["color:", ";cursor:pointer;border-radius:", ";&:hover{background-color:", ";}&:active{color:", ";background-color:", ";}"], token.itemColor.normal, theme.size.spec.borderRadius.s1, token.itemBg.hover, token.itemColor.clicked, token.itemBg.clicked);
  }

  return css(["height:", "px;width:", "px;line-height:", "px;", ""], token.itemHeight, token.itemWidth, token.itemHeight, itemCss);
});
var StyledScrollContainer = styled.div.withConfig({
  displayName: "StyledScrollContainer"
})([""]);
var BaseNode = styled.div.withConfig({
  displayName: "BaseNode"
})(["", ""], function (_ref2) {
  var theme = _ref2.theme,
      $number = _ref2.$number;
  var token = theme.components.TimePicker;
  var pattern = theme.colors.pattern;
  var scrollHeight = token.rowHeight * $number;
  var scrollPaddingBottom = token.rowHeight * ($number - 1);
  return css(["text-align:center;user-select:none;flex:1;font-size:", "px;", "{height:", "px;line-height:", "px;color:", ";}", "{height:", "px;display:flex;justify-content:center;align-items:center;}", "{box-sizing:border-box;height:", "px;overflow-y:auto;padding:0 4px ", "px;", "}"], theme.typography.spec.fontSize.s1, StyledLabel, token.rowHeight, token.rowHeight, pattern.text.secondary, StyledRow, token.rowHeight, StyledScrollContainer, scrollHeight, scrollPaddingBottom, scrollBarStyle({
    theme: theme
  }));
});
var defaultStyles = {
  colItem: '',
  colLabel: '',
  colItemBlock: '',
  colItemSelected: '',
  colItemDisabled: ''
};
var TimePickerNumberList = memoForwardRef(function (props, ref) {
  var theme = useTheme();

  var _useTimePickerNumberL = useTimePickerNumberList(props),
      otherProps = _useTimePickerNumberL.otherProps,
      items = _useTimePickerNumberL.items,
      scrollContainerRef = _useTimePickerNumberL.scrollContainerRef,
      rowNum = _useTimePickerNumberL.rowNum;

  var pickStyles = useMemo(function () {
    if (!props.styles) {
      return;
    }

    var _props$styles = props.styles,
        colLabel = _props$styles.colLabel,
        colItem = _props$styles.colItem,
        colItemBlock = _props$styles.colItemBlock,
        colItemSelected = _props$styles.colItemSelected,
        colItemDisabled = _props$styles.colItemDisabled;
    return {
      colLabel: colLabel,
      colItem: colItem,
      colItemBlock: colItemBlock,
      colItemSelected: colItemSelected,
      colItemDisabled: colItemDisabled
    };
  }, [props.styles]);
  var innerStyles = useStyles('time-picker', defaultStyles, pickStyles);
  var nodes = useMemo(function () {
    return items.map(function (item) {
      var itemText = _padStart(item.num.toString(), 2, '0');

      var innerStyleItem = innerStyles.colItemBlock;

      if (item.isSelected) {
        innerStyleItem = mergeStyleItem(innerStyles.colItemBlock, innerStyles.colItemSelected);
      } else if (item.disabled) {
        innerStyleItem = mergeStyleItem(innerStyles.colItemBlock, innerStyles.colItemDisabled);
      }

      return /*#__PURE__*/React.createElement(StyledRow, _extends({}, innerStyles.colItem, {
        key: item.num,
        ref: item.itemRef
      }), /*#__PURE__*/React.createElement(StyledItem, _extends({}, innerStyleItem, {
        theme: theme,
        $selected: item.isSelected,
        $disabled: item.disabled,
        onClick: item.handleClick
      }), itemText));
    });
  }, [innerStyles.colItem, innerStyles.colItemBlock, innerStyles.colItemDisabled, innerStyles.colItemSelected, items, theme]);
  return /*#__PURE__*/React.createElement(BaseNode, _extends({
    ref: ref,
    theme: theme,
    $number: rowNum
  }, otherProps), /*#__PURE__*/React.createElement(StyledLabel, innerStyles.colLabel, props.label), /*#__PURE__*/React.createElement(StyledScrollContainer, {
    ref: scrollContainerRef
  }, nodes));
});
export default TimePickerNumberList;