import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo } from 'react';
import styled, { css } from 'styled-components';
import useLocale from '../Locale/useLocale';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import TimePickerNumberList from './TimePickerNumberList';
import useTimePickerPanel from './useTimePickerPanel';
var StyledPanel = styled.div.withConfig({
  displayName: "StyledPanel"
})(["", ""], function (props) {
  var token = props.theme.components.TimePicker;
  var pattern = props.theme.colors.pattern;
  return css(["width:", "px;display:flex;justify-content:center;background-color:", ";padding:", ";"], token.width, pattern.background.higher, token.panelPadding);
});
var defaultStyles = {
  hourCol: '',
  minuteCol: '',
  secondCol: ''
};
var TimePickerPanel = memoForwardRef(function (props, ref) {
  var theme = useTheme();

  var _useTimePickerPanel = useTimePickerPanel(props),
      otherProps = _useTimePickerPanel.otherProps,
      selectedHour = _useTimePickerPanel.selectedHour,
      selectedMinute = _useTimePickerPanel.selectedMinute,
      selectedSecond = _useTimePickerPanel.selectedSecond,
      disabledHours = _useTimePickerPanel.disabledHours,
      disabledMinutes = _useTimePickerPanel.disabledMinutes,
      disabledSeconds = _useTimePickerPanel.disabledSeconds,
      hideDisabledOptions = _useTimePickerPanel.hideDisabledOptions,
      showMinute = _useTimePickerPanel.showMinute,
      showSecond = _useTimePickerPanel.showSecond,
      hourStep = _useTimePickerPanel.hourStep,
      minuteStep = _useTimePickerPanel.minuteStep,
      secondStep = _useTimePickerPanel.secondStep,
      rowNum = _useTimePickerPanel.rowNum,
      styles = _useTimePickerPanel.styles,
      defaultScrollBehavior = _useTimePickerPanel.defaultScrollBehavior,
      handleHourChange = _useTimePickerPanel.handleHourChange,
      handleMinuteChange = _useTimePickerPanel.handleMinuteChange,
      handleSecondChange = _useTimePickerPanel.handleSecondChange;

  var pickStyles = useMemo(function () {
    if (!styles) {
      return;
    }

    var hourCol = styles.hourCol,
        minuteCol = styles.minuteCol,
        secondCol = styles.secondCol;
    return {
      hourCol: hourCol,
      minuteCol: minuteCol,
      secondCol: secondCol
    };
  }, [styles]);
  var innerStyles = useStyles('time-picker', defaultStyles, pickStyles);

  var _useLocale = useLocale(),
      hourLabel = _useLocale['TimePicker.hour'],
      minuteLabel = _useLocale['TimePicker.minute'],
      secondLabel = _useLocale['TimePicker.second'];

  return /*#__PURE__*/React.createElement(StyledPanel, _extends({
    ref: ref,
    theme: theme
  }, otherProps), /*#__PURE__*/React.createElement(TimePickerNumberList, _extends({}, innerStyles.hourCol, {
    label: hourLabel,
    max: 23,
    rowNum: rowNum,
    step: hourStep,
    selected: selectedHour,
    disableNum: disabledHours,
    hideDisabledNum: hideDisabledOptions,
    onChange: handleHourChange,
    styles: styles,
    defaultScrollBehavior: defaultScrollBehavior
  })), showMinute && /*#__PURE__*/React.createElement(TimePickerNumberList, _extends({}, innerStyles.minuteCol, {
    label: minuteLabel,
    rowNum: rowNum,
    step: minuteStep,
    selected: selectedMinute,
    disableNum: disabledMinutes,
    hideDisabledNum: hideDisabledOptions,
    onChange: handleMinuteChange,
    styles: styles,
    defaultScrollBehavior: defaultScrollBehavior
  })), showSecond && /*#__PURE__*/React.createElement(TimePickerNumberList, _extends({}, innerStyles.secondCol, {
    label: secondLabel,
    rowNum: rowNum,
    step: secondStep,
    selected: selectedSecond,
    disableNum: disabledSeconds,
    hideDisabledNum: hideDisabledOptions,
    onChange: handleSecondChange,
    styles: styles,
    defaultScrollBehavior: defaultScrollBehavior
  })));
});
export default TimePickerPanel;