import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import { useDebounce as _useDebounce3 } from "@muya-ui/utils/es/hooks/useDebounce";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import dayjs from 'dayjs';
import { useMemo, useRef, useState } from 'react';
import { formatCalendarOptions } from '../Calendar/utils';
import forkHandler from '../utils/forkHandler';
import { transformConfigType2Time, transformStr2Date } from './utils';
var timeFormat = 'HH:mm:ss';
export default function useTimePicker(props, inputRef) {
  var value = props.value,
      defaultValue = props.defaultValue,
      onChange = props.onChange,
      onUIChange = props.onUIChange,
      popupOpen = props.popupOpen,
      _props$format = props.format,
      format = _props$format === void 0 ? timeFormat : _props$format,
      onFocus = props.onFocus,
      onClear = props.onClear,
      onPressEnter = props.onPressEnter,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      _props$inputDelay = props.inputDelay,
      inputDelay = _props$inputDelay === void 0 ? 200 : _props$inputDelay,
      options = props.options;
  var propsDate = useMemo(function () {
    var v = value || defaultValue || '';

    var _transformConfigType = transformConfigType2Time(v),
        _transformConfigType2 = _slicedToArray(_transformConfigType, 1),
        pDate = _transformConfigType2[0];

    return pDate;
  }, [defaultValue, value]);
  var propsValue = propsDate.isValid() ? propsDate.format(format) : '';

  var _useState = useState(propsDate),
      _useState2 = _slicedToArray(_useState, 2),
      stateDate = _useState2[0],
      setStateDate = _useState2[1];

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      statePopupOpen = _useState4[0],
      setPopupOpen = _useState4[1];

  var _useState5 = useState(propsValue),
      _useState6 = _slicedToArray(_useState5, 2),
      stateValue = _useState6[0],
      setStateValue = _useState6[1];

  var propsDateIsValid = propsDate.isValid();
  var stateDateIsValid = stateDate.isValid();

  if ('value' in props && (propsDateIsValid && !stateDateIsValid || !propsDateIsValid && stateDateIsValid || propsDateIsValid && stateDateIsValid && !propsDate.isSame(stateDate, 's'))) {
    setStateDate(propsDate);
  }

  var finalPopupOpen = popupOpen !== undefined ? popupOpen : statePopupOpen;
  var finalValue = stateValue;

  if ('value' in props && !finalPopupOpen) {
    finalValue = propsValue;
  }

  var updateTime = function updateTime(newTime) {
    setStateDate(newTime);

    if (onChange) {
      onChange(newTime);
    }
  }; // 切换日历面板的显示隐藏


  var updateUI = function updateUI(innerPopupOpen) {
    var newState = {
      popupOpen: innerPopupOpen
    };
    setPopupOpen(innerPopupOpen);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var lastValidDate = useRef(propsDate.isValid() ? propsDate : undefined);

  var selectByValue = function selectByValue(newValue) {
    var valueDate = transformStr2Date(newValue, stateDate);
    var valid = valueDate.isValid();

    if (valid) {
      lastValidDate.current = valueDate;
    }

    if (newValue === '' && allowClear) {
      updateTime(dayjs(''));
      return '';
    }

    if (lastValidDate.current) {
      updateTime(lastValidDate.current);
      return lastValidDate.current.format(format);
    }

    return '';
  };

  var _useDebounce = _useDebounce3(selectByValue, inputDelay),
      _useDebounce2 = _slicedToArray(_useDebounce, 2),
      debounceUpdateInputSelected = _useDebounce2[0],
      clearUpdateInputSelected = _useDebounce2[1];
  /**
   * 关闭日历面板
   */


  var closePopup = function closePopup() {
    if (!finalPopupOpen) {
      return;
    }

    updateUI(false);
    clearUpdateInputSelected();
  };

  var handleInputChange = _useEventCallback(function (e) {
    debounceUpdateInputSelected(e.target.value);
    setStateValue(e.target.value);
  });

  var handlePanelChange = _useEventCallback(function (date) {
    updateTime(date);
    setStateValue(date.format(format));
  });

  var handleFocus = _useEventCallback(forkHandler(function () {
    updateUI(true);
  }, onFocus));

  var handleClear = _useEventCallback(forkHandler(function () {
    clearUpdateInputSelected();
    setStateValue('');
    updateTime(dayjs(''));
  }, onClear));

  var handleClickAway = _useEventCallback(closePopup);

  var handlePressEnter = _useEventCallback(forkHandler(function () {
    if (inputRef.current) {
      inputRef.current.blur();
    }

    var newValue = selectByValue(finalValue);
    closePopup();
    setStateValue(newValue);
  }, onPressEnter));

  var handleOptionSelect = _useEventCallback(function (newDate) {
    if (!stateDate.isSame(newDate, 's')) {
      updateTime(newDate);
      setStateValue(newDate.format(format));
    }
  });

  var finalOptions = useMemo(function () {
    return formatCalendarOptions(options, handleOptionSelect, function (opt) {
      if (typeof opt.date === 'function') {
        _devWarning('[TimePicker]: option.date() return invalid date.');
      } else {
        _devWarning('[TimePicker]: options passed invalid date.');
      }
    });
  }, [handleOptionSelect, options]);
  return {
    finalPopupOpen: finalPopupOpen,
    finalValue: finalValue,
    stateDate: stateDate,
    finalOptions: finalOptions,
    handleFocus: handleFocus,
    handleInputChange: handleInputChange,
    handlePressEnter: handlePressEnter,
    handleClickAway: handleClickAway,
    handleClear: handleClear,
    handlePanelChange: handlePanelChange
  };
}