import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["children", "label", "max", "min", "step", "rowNum", "selected", "onChange", "disableNum", "hideDisabledNum", "defaultScrollBehavior", "styles"];
import { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import scrollIntoView from '../utils/scrollIntoView';
import useTheme from '../utils/useTheme';
export default function useTimePickerNumberList(props) {
  var children = props.children,
      label = props.label,
      _props$max = props.max,
      max = _props$max === void 0 ? 59 : _props$max,
      _props$min = props.min,
      min = _props$min === void 0 ? 0 : _props$min,
      _props$step = props.step,
      step = _props$step === void 0 ? 1 : _props$step,
      _props$rowNum = props.rowNum,
      rowNum = _props$rowNum === void 0 ? 6 : _props$rowNum,
      propSelected = props.selected,
      onChange = props.onChange,
      disableNum = props.disableNum,
      hideDisabledNum = props.hideDisabledNum,
      _props$defaultScrollB = props.defaultScrollBehavior,
      defaultScrollBehavior = _props$defaultScrollB === void 0 ? 'auto' : _props$defaultScrollB,
      styles = props.styles,
      otherProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStep = step >= 1 ? step : 1;
  var innerDefaultSelected = min - 1;

  var _useState = useState(innerDefaultSelected),
      _useState2 = _slicedToArray(_useState, 2),
      selected = _useState2[0],
      setSelected = _useState2[1];

  var finalSelected = selected;

  if (propSelected !== undefined) {
    finalSelected = propSelected;
  }

  var updateSelected = useCallback(function (num) {
    setSelected(num);
    onChange && onChange(num);
  }, [onChange]);
  var selectedRef = useRef(null);
  var scrollContainerRef = useRef(null);
  var scrollBehavior = useRef(defaultScrollBehavior);
  var items = useMemo(function () {
    var nums = [];
    var start = min;

    while (start <= max) {
      nums.push(start);
      start += innerStep;
    }

    if (hideDisabledNum && disableNum) {
      nums = nums.filter(function (num) {
        return !disableNum(num);
      });
    }

    return nums.map(function (num) {
      var isSelected = num === finalSelected;
      var itemRef = isSelected ? selectedRef : undefined;
      var disabled = false;

      if (!hideDisabledNum && disableNum) {
        disabled = disableNum(num);
      }

      return {
        num: num,
        isSelected: isSelected,
        itemRef: itemRef,
        disabled: disabled,
        handleClick: function handleClick() {
          if (isSelected || disabled) {
            return;
          }

          scrollBehavior.current = 'smooth';
          updateSelected(num);
        }
      };
    });
  }, [disableNum, finalSelected, hideDisabledNum, innerStep, max, min, updateSelected]);
  useEffect(function () {
    if (finalSelected > innerDefaultSelected && selectedRef.current && scrollContainerRef.current) {
      scrollIntoView(selectedRef.current, {
        parentNode: scrollContainerRef.current,
        behavior: scrollBehavior.current,
        time: theme.transition.spec.duration.normal,
        align: {
          top: 0
        }
      });
    }
  }, [finalSelected, innerDefaultSelected, min, selected, theme.transition.spec.duration.normal]);
  return {
    rowNum: rowNum,
    selectedRef: selectedRef,
    scrollContainerRef: scrollContainerRef,
    otherProps: otherProps,
    items: items,
    finalSelected: finalSelected
  };
}