import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["color", "dot", "children", "date", "styles", "className", "style", "position"];
import memoForwardRef from '../utils/memoForwardRef';
import React, { useMemo } from 'react';
import { StyledTimelineItem, StyledDot, StyledContentContainer, StyledLine, StyledDate, StyledContent, StyledNodeContainer } from './styled';
import useTheme from '../utils/useTheme';
import { useTimelineContext } from './TimelineContext';
import useStyles from '../utils/useStyles';
import mergeStyleItem from '../utils/mergeStyleItem';
import Background from '../Background';
var defaultStyles = {
  item: '',
  content: '',
  date: ''
};
var TimelineItem = memoForwardRef(function (props, ref) {
  var color = props.color,
      dot = props.dot,
      children = props.children,
      date = props.date,
      styles = props.styles,
      className = props.className,
      style = props.style,
      _props$position = props.position,
      position = _props$position === void 0 ? 'left' : _props$position,
      otherProps = _objectWithoutProperties(props, _excluded);

  var context = useTimelineContext();
  var mode = context.mode;
  var timelineStyles = useMemo(function () {
    if (styles) {
      var item = styles.item,
          content = styles.content,
          _date = styles.date;
      return {
        item: item,
        content: content,
        date: _date
      };
    }
  }, [styles]);
  var innerStyles = useStyles('timeline', defaultStyles, timelineStyles);
  var finalStyles = mergeStyleItem({
    className: className,
    style: style
  }, innerStyles.item);
  var theme = useTheme();
  var token = theme.components.Timeline;
  var dotColor = color || token.dotColor;
  var dotNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledNodeContainer, {
      theme: theme
    }, dot ? /*#__PURE__*/React.createElement(Background, {
      display: "flex",
      height: "fit-content",
      style: {
        zIndex: 1
      }
    }, dot) : /*#__PURE__*/React.createElement(StyledDot, {
      theme: theme,
      $color: dotColor,
      $size: token.dotSize
    }), /*#__PURE__*/React.createElement(StyledLine, {
      theme: theme
    }));
  }, [dot, dotColor, theme, token.dotSize]);
  var contentNode = useMemo(function () {
    if (mode === 'right') {
      return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(StyledContentContainer, {
        theme: theme,
        $position: position,
        $mode: mode
      }, /*#__PURE__*/React.createElement(StyledDate, _extends({
        theme: theme
      }, innerStyles.date, {
        $position: position
      }), date), /*#__PURE__*/React.createElement(StyledContent, _extends({
        theme: theme
      }, innerStyles.content, {
        $position: position
      }), children)), dotNode);
    }

    if (mode === 'alternate-left') {
      return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(StyledDate, _extends({
        theme: theme
      }, innerStyles.date, {
        $mode: mode
      }), date), dotNode, /*#__PURE__*/React.createElement(StyledContent, _extends({
        theme: theme
      }, innerStyles.content, {
        $mode: mode
      }), children));
    }

    if (mode === 'alternate-right') {
      return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(StyledContent, _extends({
        theme: theme
      }, innerStyles.content, {
        $mode: mode
      }), children), dotNode, /*#__PURE__*/React.createElement(StyledDate, _extends({
        theme: theme
      }, innerStyles.date, {
        $mode: mode
      }), date));
    }

    return /*#__PURE__*/React.createElement(React.Fragment, null, dotNode, /*#__PURE__*/React.createElement(StyledContentContainer, {
      theme: theme,
      $position: position,
      $mode: mode
    }, /*#__PURE__*/React.createElement(StyledDate, _extends({
      theme: theme
    }, innerStyles.date, {
      $position: position
    }), date), /*#__PURE__*/React.createElement(StyledContent, _extends({
      theme: theme
    }, innerStyles.content, {
      $position: position
    }), children)));
  }, [children, date, dotNode, innerStyles.content, innerStyles.date, mode, position, theme]);
  return /*#__PURE__*/React.createElement(StyledTimelineItem, _extends({
    $mode: mode,
    theme: theme
  }, finalStyles, otherProps), contentNode);
});
export default TimelineItem;