import styled, { css } from 'styled-components';
var PositionDirectionEnum;

(function (PositionDirectionEnum) {
  PositionDirectionEnum["left"] = "row";
  PositionDirectionEnum["right"] = "row-reverse";
  PositionDirectionEnum["top"] = "column";
  PositionDirectionEnum["bottom"] = "column-reverse";
})(PositionDirectionEnum || (PositionDirectionEnum = {}));

var ModeAlignEnum;

(function (ModeAlignEnum) {
  ModeAlignEnum["left"] = "flex-start";
  ModeAlignEnum["right"] = "flex-end";
  ModeAlignEnum["alternate"] = "baseline";
  ModeAlignEnum["alternate-left"] = "stretch";
  ModeAlignEnum["alternate-right"] = "stretch";
})(ModeAlignEnum || (ModeAlignEnum = {}));

export var StyledTimeline = styled.div.withConfig({
  displayName: "StyledTimeline"
})(["", ""], function (props) {
  var _props$$mode = props.$mode,
      $mode = _props$$mode === void 0 ? 'left' : _props$$mode,
      _props$theme = props.theme,
      colors = _props$theme.colors,
      typography = _props$theme.typography;
  var alternateStyle = $mode === 'alternate' && css(["margin-left:50%;"]);
  return css(["display:flex;flex-direction:column;color:", ";font-size:", "px;line-height:", "px;align-items:", ";", ";& a{font-size:", "px;line-height:", "px;}"], colors.spec.neutral1.normal, typography.spec.fontSize.s1, typography.spec.lineHeight.s1, ModeAlignEnum[$mode], alternateStyle, typography.spec.fontSize.s1, typography.spec.lineHeight.s1);
});
export var StyledNodeContainer = styled.span.withConfig({
  displayName: "StyledNodeContainer"
})(["", ""], function (props) {
  var token = props.theme.components.Timeline;
  return css(["position:relative;display:flex;justify-content:center;width:", "px;flex-shrink:0;"], token.dotSize + token.contentLeftMargin * 2);
});
export var StyledDot = styled.i.withConfig({
  displayName: "StyledDot"
})(["", ""], function (props) {
  var $color = props.$color,
      $size = props.$size,
      typography = props.theme.typography;
  var dotMargin = Math.floor((typography.spec.lineHeight.s1 - $size) / 2);
  return css(["display:block;width:", "px;height:", "px;border-radius:50%;background-color:", ";margin-top:", "px;"], $size, $size, $color, dotMargin);
});
export var StyledLine = styled.span.withConfig({
  displayName: "StyledLine"
})(["", ""], function (props) {
  var _props$theme2 = props.theme,
      colors = _props$theme2.colors,
      typography = _props$theme2.typography,
      token = _props$theme2.components.Timeline;
  var dotMargin = Math.floor((typography.spec.lineHeight.s1 - token.dotSize) / 2);
  return css(["position:absolute;top:", "px;display:block;width:", "px;height:calc(100% + ", "px);background-color:", ";"], token.dotSize + dotMargin, token.lineWidth, token.dotSize + dotMargin, colors.pattern.background.divider);
});
export var StyledContentContainer = styled.div.withConfig({
  displayName: "StyledContentContainer"
})(["", ""], function (props) {
  var _props$$position = props.$position,
      $position = _props$$position === void 0 ? 'left' : _props$$position,
      $mode = props.$mode;
  return css(["display:flex;flex-direction:", ";text-align:", ";"], PositionDirectionEnum[$position], $mode);
});

function makeContentStyle(props) {
  var $position = props.$position,
      _props$$mode2 = props.$mode,
      $mode = _props$$mode2 === void 0 ? 'left' : _props$$mode2,
      token = props.theme.components.Timeline;
  var marginStyle = !$mode.includes('alternate') && ($position === 'left' ? css(["&:not(:last-child){margin-right:", "px;}"], token.contentLeftMargin) : $position === 'right' ? css(["&:not(:first-child){margin-right:", "px;}"], token.contentLeftMargin) : $position === 'top' ? css(["&:not(:last-child){margin-bottom:", "px;}"], token.contentTopMargin) : css(["&:not(:first-child){margin-bottom:", "px;}"], token.contentTopMargin));
  var widthStyle = ($mode === 'alternate-left' || $mode === 'alternate-right') && css(["width:calc((100% - ", "px) / 2);"], token.dotSize + token.contentLeftMargin * 2);
  return css(["", ";", ";"], marginStyle, widthStyle);
}

export var StyledContent = styled.div.withConfig({
  displayName: "StyledContent"
})(["", ""], function (props) {
  var $mode = props.$mode;
  var alignStyle = $mode === 'alternate-right' && css(["text-align:right;"]);
  return css(["", ";", ";"], makeContentStyle(props), alignStyle);
});
export var StyledDate = styled.div.withConfig({
  displayName: "StyledDate"
})(["", ""], function (props) {
  var $mode = props.$mode;
  var alignStyle = $mode === 'alternate-left' && css(["text-align:right;"]);
  return css(["", ";", ";flex-shrink:0;"], makeContentStyle(props), alignStyle);
});
export var StyledTimelineItem = styled.div.withConfig({
  displayName: "StyledTimelineItem"
})(["", ""], function (props) {
  var $mode = props.$mode,
      token = props.theme.components.Timeline;
  var alternateStyle = $mode === 'alternate' && css(["&:nth-child(2n + 1) ", "{text-align:left;}&:nth-child(2n) ", "{transform:translateX(calc(-100% - ", "px));text-align:right;}"], StyledContentContainer, StyledContentContainer, token.dotSize + token.contentLeftMargin * 2);
  return css(["position:relative;display:flex;padding:", "px 0;", ";&:last-child{", "{display:none;}}"], token.itemPadding, alternateStyle, StyledLine);
});