import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import _isArray from "lodash/isArray";
import React, { useCallback } from 'react';
import TransferPanel from './TransferPanel';
import { StyledTransfer, StyledTransferIconContainer } from './styled';
import useTheme from '../utils/useTheme';
import TransferIcon from './TransferIcon';
import useTransfer from './useTransfer';
import useTransferTree from './useTransferTree';
import { useStyles } from '../utils/useStyles';
import { useLocale } from '../Locale';
import memoForwardRef from '../utils/memoForwardRef';
var defaultStyles = {
  container: '',
  leftPanel: '',
  rightPanel: '',
  leftHeader: '',
  rightHeader: '',
  leftFoot: '',
  rightFoot: ''
};

var TransferPure = function TransferPure(props, ref) {
  var theme = useTheme();
  var children = props.children,
      _props$targetSource = props.targetSource,
      targetSource = _props$targetSource === void 0 ? [] : _props$targetSource,
      defaultTargetSource = props.defaultTargetSource,
      dataSource = props.dataSource,
      transferIcon = props.transferIcon,
      _titles = props.titles,
      _mode = props.mode,
      _props$showSearch = props.showSearch,
      showSearch = _props$showSearch === void 0 ? false : _props$showSearch,
      onSearch = props.onSearch,
      _searchPlaceholder = props.searchPlaceholder,
      filterOption = props.filterOption,
      renderItem = props.renderItem,
      _props$footer = props.footer,
      footer = _props$footer === void 0 ? [] : _props$footer,
      _props$showCount = props.showCount,
      showCount = _props$showCount === void 0 ? true : _props$showCount,
      _props$showSelectAll = props.showSelectAll,
      showSelectAll = _props$showSelectAll === void 0 ? true : _props$showSelectAll,
      onChange = props.onChange,
      styles = props.styles,
      rightItemExtra = props.rightItemExtra,
      searchProps = props.searchProps,
      emptyNode = props.emptyNode,
      onCopyAll = props.onCopyAll,
      _props$headerExtraNod = props.headerExtraNode,
      headerExtraNode = _props$headerExtraNod === void 0 ? [] : _props$headerExtraNod,
      _props$order = props.order,
      order = _props$order === void 0 ? 'DESC' : _props$order,
      _props$panelRef = props.panelRef,
      panelRef = _props$panelRef === void 0 ? [] : _props$panelRef,
      _props$panelOnScroll = props.panelOnScroll,
      panelOnScroll = _props$panelOnScroll === void 0 ? [] : _props$panelOnScroll;
  var locale = useLocale(); // 计算 mode
  // 如果自定义了children默认给copy,否则默认给move

  var mode = React.useMemo(function () {
    if (_mode) {
      return _mode;
    } else if (children && typeof children === 'function') {
      return 'copy';
    } else {
      return 'move';
    }
  }, [_mode, children]); // 计算titles

  var titles = React.useMemo(function () {
    if (_titles) {
      if (_isArray(_titles) && _titles.length === 2) {
        return _titles;
      } else {
        _devWarning('[Transer]titles 参数必须传入一个长度为2的数组');

        return [locale['Transer.list1'], locale['Transer.list2']];
      }
    } else {
      return [locale['Transer.list1'], locale['Transer.list2']];
    }
  }, [_titles, locale]); // 计算 searchPlaceholder

  var searchPlaceholder = React.useMemo(function () {
    var defaultSearchPlaceholder = locale['Transer.search'];

    if (_searchPlaceholder) {
      if (_isArray(_searchPlaceholder)) {
        var placeholderArr = _searchPlaceholder.map(function (item) {
          return item || defaultSearchPlaceholder;
        });

        return placeholderArr;
      } else if (typeof _searchPlaceholder === 'string') {
        return [_searchPlaceholder, _searchPlaceholder];
      } else {
        _devWarning('[Transer]searchPlaceholder 参数必须传入一个数组');

        return [defaultSearchPlaceholder, defaultSearchPlaceholder];
      }
    } else {
      return [defaultSearchPlaceholder, defaultSearchPlaceholder];
    }
  }, [_searchPlaceholder, locale]); // 计算左右emptyNode

  var _React$useMemo = React.useMemo(function () {
    var leftEmptyNode;
    var rightEmptyNode;

    if (emptyNode && Array.isArray(emptyNode)) {
      leftEmptyNode = emptyNode[0];
      rightEmptyNode = emptyNode[1];
    } else if ('emptyNode' in props) {
      leftEmptyNode = rightEmptyNode = emptyNode;
    }

    return [leftEmptyNode, rightEmptyNode];
  }, [emptyNode, props]),
      _React$useMemo2 = _slicedToArray(_React$useMemo, 2),
      leftEmptyNode = _React$useMemo2[0],
      rightEmptyNode = _React$useMemo2[1];

  var isControlled = !!('targetSource' in props);

  var _useTransfer = useTransfer(dataSource, isControlled ? targetSource : defaultTargetSource || [], isControlled, mode, order, onChange),
      leftData = _useTransfer.leftData,
      rightData = _useTransfer.rightData,
      leftAction = _useTransfer.leftAction,
      rightAction = _useTransfer.rightAction,
      _leftActionCopyAll = _useTransfer.leftActionCopyAll;

  var leftActionCopyAll = typeof onCopyAll === 'function' ? onCopyAll : _leftActionCopyAll;
  var getPanelShowSearch = useCallback(function (dir) {
    return showSearch === true || showSearch === dir;
  }, [showSearch]);
  var leftShowSearch = getPanelShowSearch('left');
  var rightShowSearch = getPanelShowSearch('right');
  var innerStyles = useStyles('transfer', defaultStyles, styles);
  var container = innerStyles.container,
      leftPanel = innerStyles.leftPanel,
      leftHeader = innerStyles.leftHeader,
      leftFoot = innerStyles.leftFoot,
      rightPanel = innerStyles.rightPanel,
      rightHeader = innerStyles.rightHeader,
      rightFoot = innerStyles.rightFoot;
  return /*#__PURE__*/React.createElement(StyledTransfer, _extends({
    theme: theme,
    ref: ref
  }, container), /*#__PURE__*/React.createElement(TransferPanel, {
    title: titles[0],
    direction: "left",
    items: leftData,
    handleClick: leftAction,
    showSearch: leftShowSearch,
    onSearch: onSearch,
    searchPlaceholder: searchPlaceholder[0],
    filterOption: filterOption,
    renderItem: renderItem,
    footer: footer[0],
    rightItems: rightData,
    mode: mode,
    showCount: showCount,
    showSelectAll: showSelectAll,
    leftActionCopyAll: leftActionCopyAll,
    containerStyles: leftPanel,
    headerStyles: leftHeader,
    footStyles: leftFoot,
    searchProps: searchProps,
    emptyNode: leftEmptyNode,
    headerExtraNode: headerExtraNode[0],
    scrollContainerRef: panelRef[0],
    onScroll: panelOnScroll[0]
  }, children), /*#__PURE__*/React.createElement(StyledTransferIconContainer, {
    theme: theme
  }, transferIcon || /*#__PURE__*/React.createElement(TransferIcon, {
    mode: mode
  })), /*#__PURE__*/React.createElement(TransferPanel, {
    title: titles[1],
    direction: "right",
    items: rightData,
    handleClick: rightAction,
    showSearch: rightShowSearch,
    onSearch: onSearch,
    searchPlaceholder: searchPlaceholder[1],
    filterOption: filterOption,
    renderItem: renderItem,
    footer: footer[1],
    mode: mode,
    showCount: showCount,
    containerStyles: rightPanel,
    headerStyles: rightHeader,
    footStyles: rightFoot,
    rightItemExtra: rightItemExtra,
    searchProps: searchProps,
    emptyNode: rightEmptyNode,
    headerExtraNode: headerExtraNode[1],
    scrollContainerRef: panelRef[1],
    onScroll: panelOnScroll[1]
  }, children));
};

var Transfer = memoForwardRef(TransferPure);
Transfer.useTransferTree = useTransferTree;
Transfer.__MUYA_TRANSFER = true;
export default Transfer;