import _extends from "@babel/runtime/helpers/extends";
import { SearchIcon as _SearchIcon } from "@muya-ui/theme-light/es/icons/SearchIcon";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _map from "lodash/map";
import _includes from "lodash/includes";
import React, { useCallback, useMemo, useState } from 'react';
import EllipsisContent from '../EllipsisContent';
import Input from '../Input';
import { useLocale } from '../Locale';
import { Result } from '../Result';
import useTheme from '../utils/useTheme';
import { StyledEmptyResultContainer, StyledTransferCheckbox, StyledTransferItemExtraIcon, StyledTransferPanel, StyledTransferPanelCount, StyledTransferPanelFooter, StyledTransferPanelHeader, StyledTransferPanelItem, StyledTransferPanelItemText, StyledTransferPanelList, StyledTransferPanelSearch, StyledTransferPanelTitle } from './styled';

var TransferPanel = function TransferPanel(props) {
  var theme = useTheme();
  var children = props.children,
      direction = props.direction,
      mode = props.mode,
      title = props.title,
      _props$items = props.items,
      items = _props$items === void 0 ? [] : _props$items,
      renderItem = props.renderItem,
      handleClick = props.handleClick,
      onSearch = props.onSearch,
      searchPlaceholder = props.searchPlaceholder,
      showSearch = props.showSearch,
      showCount = props.showCount,
      filterOption = props.filterOption,
      rightItems = props.rightItems,
      showSelectAll = props.showSelectAll,
      leftActionCopyAll = props.leftActionCopyAll,
      footer = props.footer,
      containerStyles = props.containerStyles,
      headerStyles = props.headerStyles,
      footStyles = props.footStyles,
      rightItemExtra = props.rightItemExtra,
      searchProps = props.searchProps,
      emptyNode = props.emptyNode,
      headerExtraNode = props.headerExtraNode,
      scrollContainerRef = props.scrollContainerRef,
      onScroll = props.onScroll;

  var _useState = useState(''),
      _useState2 = _slicedToArray(_useState, 2),
      searchVal = _useState2[0],
      setSearchVal = _useState2[1];

  var locale = useLocale();

  var _handleClick = useCallback(function (key, disabled) {
    return disabled ? function () {} : function () {
      return handleClick(key);
    };
  }, [handleClick]);

  var handleSearch = useCallback(function (e) {
    var val = e.target.value;
    setSearchVal(val);
    typeof onSearch === 'function' && onSearch(direction, val);
  }, [direction, onSearch]);
  var showItems = useMemo(function () {
    var nextShowItems = items.filter(function (item) {
      return filterOption ? filterOption(searchVal, item) : _includes(item.title || '', searchVal);
    });
    return nextShowItems;
  }, [filterOption, items, searchVal]);
  var handleCopyAll = useCallback(function () {
    typeof leftActionCopyAll === 'function' && leftActionCopyAll(showItems);
  }, [leftActionCopyAll, showItems]);
  var itemRenderer = useCallback(function (item) {
    return renderItem ? renderItem(item) : item.title;
  }, [renderItem]);
  var itemChecked = useCallback(function (item) {
    return _includes(_map(rightItems, 'key'), item.key);
  }, [rightItems]);
  var prefixRenderer = useCallback(function (item) {
    return direction === 'left' && mode === 'copy' ? /*#__PURE__*/React.createElement(StyledTransferCheckbox, {
      theme: theme,
      size: "s",
      checked: itemChecked(item),
      onClick: _handleClick(item.key, !!item.disabled)
    }) : null;
  }, [_handleClick, direction, itemChecked, mode, theme]);
  var suffixRender = useMemo(function () {
    if (direction === 'right' && mode === 'copy') {
      return rightItemExtra || /*#__PURE__*/React.createElement(StyledTransferItemExtraIcon, {
        theme: theme
      });
    } else {
      return null;
    }
  }, [direction, mode, rightItemExtra, theme]);
  var itemsRenderer = useMemo(function () {
    var renderChildren;

    if (children && typeof children === 'function') {
      renderChildren = children({
        direction: direction,
        filteredItems: showItems,
        filteredItemKeys: _map(showItems, 'key'),
        onItemSelect: handleClick
      });
    }

    if (renderChildren) {
      return renderChildren;
    } else if (showItems.length) {
      return showItems.map(function (item) {
        return /*#__PURE__*/React.createElement(StyledTransferPanelItem, {
          $disabled: !!item.disabled,
          theme: theme,
          key: item.key,
          onClick: _handleClick(item.key, !!item.disabled)
        }, prefixRenderer(item), /*#__PURE__*/React.createElement(StyledTransferPanelItemText, {
          theme: theme,
          enableTitle: true
        }, itemRenderer(item)), suffixRender);
      });
    } else if (emptyNode !== undefined) {
      return emptyNode;
    } else {
      var subTitle = locale['Select.noDataText'];
      return /*#__PURE__*/React.createElement(StyledEmptyResultContainer, null, /*#__PURE__*/React.createElement(Result, {
        subTitle: subTitle,
        type: "emptySmall"
      }));
    }
  }, [_handleClick, children, direction, emptyNode, handleClick, itemRenderer, locale, prefixRenderer, showItems, suffixRender, theme]);
  var searchRenderer = useMemo(function () {
    return showSearch ? /*#__PURE__*/React.createElement(StyledTransferPanelSearch, {
      theme: theme
    }, /*#__PURE__*/React.createElement(Input, _extends({
      size: "s",
      placeholder: searchPlaceholder,
      suffixNode: /*#__PURE__*/React.createElement(_SearchIcon, null),
      allowClear: true,
      clearReplace: true
    }, searchProps, {
      onChange: handleSearch
    }))) : null;
  }, [handleSearch, searchPlaceholder, searchProps, showSearch, theme]);
  var headerRender = useMemo(function () {
    var countData;

    if (direction === 'left' && mode === 'copy') {
      var rightKeys = _map(rightItems, 'key');

      countData = "".concat(showItems.filter(function (item) {
        return _includes(rightKeys, item.key);
      }).length, "/").concat(showItems.length);
    } else {
      countData = "".concat(showItems.length);
    }

    return /*#__PURE__*/React.createElement(StyledTransferPanelHeader, _extends({
      theme: theme
    }, headerStyles), direction === 'left' && showSelectAll && mode === 'copy' ? /*#__PURE__*/React.createElement(StyledTransferCheckbox, {
      size: "s",
      theme: theme,
      checked: rightItems.length === items.length,
      indeterminate: !!(rightItems.length && rightItems.length !== items.length),
      onClick: handleCopyAll
    }) : null, /*#__PURE__*/React.createElement(StyledTransferPanelTitle, {
      theme: theme
    }, /*#__PURE__*/React.createElement(EllipsisContent, {
      enableTitle: true
    }, title)), showCount ? /*#__PURE__*/React.createElement(StyledTransferPanelCount, {
      theme: theme
    }, countData) : null, headerExtraNode || null);
  }, [direction, handleCopyAll, headerExtraNode, headerStyles, items.length, mode, rightItems, showCount, showItems, showSelectAll, theme, title]);
  var footerRender = useMemo(function () {
    return footer ? /*#__PURE__*/React.createElement(StyledTransferPanelFooter, _extends({
      theme: theme
    }, footStyles), footer) : null;
  }, [footStyles, footer, theme]);
  return /*#__PURE__*/React.createElement(StyledTransferPanel, _extends({
    theme: theme
  }, containerStyles), headerRender, searchRenderer, /*#__PURE__*/React.createElement(StyledTransferPanelList, {
    theme: theme,
    contentRef: scrollContainerRef,
    onScroll: onScroll
  }, itemsRenderer), footerRender);
};

export default TransferPanel;