import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _xor from "lodash/xor";
import _map from "lodash/map";
import _includes from "lodash/includes";
import _find from "lodash/find";
import { useCallback, useMemo, useState } from 'react';

function useTransfer(dataSource, targetSource, isController, mode, order, handleChange) {
  var initLeftData = useMemo(function () {
    return mode === 'move' ? dataSource.filter(function (item) {
      return !_includes(targetSource, item.key);
    }) : dataSource;
  }, [dataSource, mode, targetSource]);
  var initRightData = useMemo(function () {
    return targetSource.map(function (key) {
      return _find(dataSource, function (item) {
        return item.key === key;
      });
    });
  }, [dataSource, targetSource]);

  var _useState = useState(initLeftData),
      _useState2 = _slicedToArray(_useState, 2),
      _leftData = _useState2[0],
      setLeftData = _useState2[1];

  var _useState3 = useState(initRightData),
      _useState4 = _slicedToArray(_useState3, 2),
      _rightData = _useState4[0],
      setRightData = _useState4[1];

  var leftData = isController ? initLeftData : _leftData;
  var rightData = isController ? initRightData : _rightData;
  var emitClick = useCallback(function (transferKey, actionDirection, nextTargetKeys, nextSourceKeys) {
    typeof handleChange === 'function' && handleChange(transferKey, actionDirection, nextTargetKeys, nextSourceKeys);
  }, [handleChange]);
  var getKeys = useCallback(function (data) {
    return _map(data, 'key');
  }, []);
  var dataSourceKeys = useMemo(function () {
    return getKeys(dataSource);
  }, [dataSource, getKeys]);
  /**
   * 左边点击，左边不变，右边新增
   */

  var leftActionCopy = useCallback(function (key) {
    var newRightData;

    if (_includes(getKeys(rightData), key)) {
      newRightData = rightData.filter(function (item) {
        return item.key !== key;
      });
    } else {
      newRightData = order === 'DESC' ? [_find(leftData, function (item) {
        return item.key === key;
      })].concat(_toConsumableArray(rightData)) : [].concat(_toConsumableArray(rightData), [_find(leftData, function (item) {
        return item.key === key;
      })]);
    }

    setRightData(newRightData);
    emitClick(getKeys(newRightData), dataSourceKeys, 'left', [key]);
  }, [dataSourceKeys, emitClick, getKeys, leftData, order, rightData]);
  /**
   * 左边点击，左边减少，右边新增
   */

  var leftActionMove = useCallback(function (key) {
    var newRightData = order === 'DESC' ? [_find(leftData, function (item) {
      return item.key === key;
    })].concat(_toConsumableArray(rightData)) : [].concat(_toConsumableArray(rightData), [_find(leftData, function (item) {
      return item.key === key;
    })]);
    var newLeftData = leftData.filter(function (leftItem) {
      return leftItem.key !== key;
    });
    setLeftData(newLeftData);
    setRightData(newRightData);
    emitClick(getKeys(newRightData), getKeys(newLeftData), 'left', [key]);
  }, [emitClick, getKeys, leftData, order, rightData]);
  /**
   * 右边点击，左边不变，右边减少
   */

  var rightActionCopy = useCallback(function (key) {
    var newRightData = rightData.filter(function (item) {
      return item.key !== key;
    });
    setRightData(newRightData);
    emitClick(getKeys(newRightData), dataSourceKeys, 'right', [key]);
  }, [dataSourceKeys, emitClick, getKeys, rightData]);
  /**
   * 右边点击，左边增加，右边减少
   */

  var rightActionMove = useCallback(function (key) {
    var newRightData = rightData.filter(function (item) {
      return item.key !== key;
    });
    var newRightKeys = getKeys(newRightData);
    var newLeftData = dataSource.filter(function (item) {
      return !_includes(newRightKeys, item.key);
    });
    setRightData(newRightData);
    setLeftData(newLeftData);
    emitClick(getKeys(newRightData), getKeys(newLeftData), 'right', [key]);
  }, [dataSource, emitClick, getKeys, rightData]);
  /**
   * 左边全选，所有不disabled的item都被移到右边
   */

  var leftActionCopyAll = useCallback(function (filteredItems) {
    // 过滤 disabled 为 true 的数据
    var enabledFilteredItems = filteredItems.filter(function (item) {
      return !item.disabled;
    });
    var rightKeys = getKeys(rightData);
    var filteredKeys = getKeys(enabledFilteredItems);
    var filteredItemsUnChecked = enabledFilteredItems.filter(function (item) {
      return !_includes(rightKeys, item.key);
    });
    var newRightData; // 当前已全选

    if (filteredItemsUnChecked.length === 0) {
      newRightData = rightData.filter(function (item) {
        return !_includes(filteredKeys, item.key);
      });
    } // 当前未全选
    else {
      newRightData = [].concat(_toConsumableArray(filteredItemsUnChecked), _toConsumableArray(rightData));
    }

    setRightData(newRightData);
    emitClick(getKeys(newRightData), dataSourceKeys, 'left', _xor(getKeys(newRightData), rightKeys));
  }, [dataSourceKeys, emitClick, getKeys, rightData]);
  return {
    leftData: leftData,
    rightData: rightData,
    leftAction: mode === 'move' ? leftActionMove : leftActionCopy,
    rightAction: mode === 'move' ? rightActionMove : rightActionCopy,
    leftActionCopyAll: leftActionCopyAll
  };
}

export default useTransfer;