import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _isEmpty from "lodash/isEmpty";
var _excluded = ["transitionType", "transitionNodes", "onTransitionEnd", "styles", "lastChildFlags"];
import React from 'react';
import Animation from '../Animation';
import memoForwardRef from '../utils/memoForwardRef';
import TreeNode from './TreeNode';
var TransitionTreeNode = memoForwardRef(function (props, ref) {
  var transitionType = props.transitionType,
      transitionNodes = props.transitionNodes,
      onTransitionEnd = props.onTransitionEnd,
      styles = props.styles,
      lastChildFlags = props.lastChildFlags,
      restProps = _objectWithoutProperties(props, _excluded);

  var _React$useState = React.useState(true),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      visible = _React$useState2[0],
      setVisible = _React$useState2[1];

  var transitionedRef = React.useRef(false);

  var handleTransitionEnd = function handleTransitionEnd() {
    if (!transitionedRef.current) {
      onTransitionEnd();
    }

    transitionedRef.current = true;
  };

  React.useEffect(function () {
    if (transitionNodes && transitionType === 'hide' && visible) {
      setVisible(false);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [visible]);
  React.useEffect(function () {
    return function () {
      if (transitionNodes) {
        handleTransitionEnd();
      }
    };
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  []);

  if (!_isEmpty(transitionNodes)) {
    return /*#__PURE__*/React.createElement(Animation.Collapse, {
      ref: ref,
      "in": visible,
      appear: transitionType === 'show',
      onEntered: handleTransitionEnd,
      onExited: handleTransitionEnd
    }, /*#__PURE__*/React.createElement(React.Fragment, null, transitionNodes.map(function (node) {
      var data = node.data;
      var key = data.key;
      return /*#__PURE__*/React.createElement(TreeNode, _extends({
        styles: styles,
        data: data,
        nodeKey: key,
        key: key,
        lastChildFlags: lastChildFlags
      }, data));
    })));
  }

  return /*#__PURE__*/React.createElement(TreeNode, _extends({}, restProps, {
    styles: styles,
    lastChildFlags: lastChildFlags
  }));
});
export default TransitionTreeNode;