import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _omit from "lodash/omit";
var _excluded = ["className", "style", "styles", "treeRef", "scrollable"],
    _excluded2 = ["transitionType", "transitionNodes", "onNodeTransitionEnd", "list", "containerProps", "wrapperProps", "handleScroll"];
import React, { useMemo, useRef, useLayoutEffect } from 'react';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { allTreeProps, TRANSITION_KEY } from './const';
import { StyledTree, StyledTreeScrollWrapper } from './styled';
import TransitionTreeNode from './TransitionTreeNode';
import TreeContext from './TreeContext';
import TreeNode from './TreeNode';
import useTree from './useTree';
var defaultStyles = {
  scrollWrapper: '',
  wrapper: ''
};

var Tree = function Tree(props) {
  var className = props.className,
      style = props.style,
      styles = props.styles,
      treeRef = props.treeRef,
      _props$scrollable = props.scrollable,
      scrollable = _props$scrollable === void 0 ? false : _props$scrollable,
      restProps = _objectWithoutProperties(props, _excluded);

  var treeScrollWrapperRef = useRef(null);
  var innerTreeRef = useRef(null);
  var theme = useTheme();

  var _useTree = useTree(restProps, theme, innerTreeRef),
      transitionType = _useTree.transitionType,
      transitionNodes = _useTree.transitionNodes,
      onNodeTransitionEnd = _useTree.onNodeTransitionEnd,
      list = _useTree.list,
      containerProps = _useTree.containerProps,
      wrapperProps = _useTree.wrapperProps,
      handleScroll = _useTree.handleScroll,
      contextValue = _objectWithoutProperties(_useTree, _excluded2);

  var domProps = _omit(props, allTreeProps);

  var virtual = contextValue.virtual,
      size = contextValue.size,
      treeData = contextValue.treeData;

  var handleChildrenRef = _useForkRef(containerProps.ref, innerTreeRef);

  var handleRef = _useForkRef(handleChildrenRef, treeRef);

  var treeStyles = useMemo(function () {
    var treeStyles;

    if (styles) {
      var wrapper = styles.wrapper,
          scrollWrapper = styles.scrollWrapper;
      treeStyles = {
        wrapper: wrapper,
        scrollWrapper: scrollWrapper
      };
    }

    return treeStyles;
  }, [styles]);
  var innerStyles = useStyles('tree', defaultStyles, treeStyles);
  var innerStyle = useMemo(function () {
    var innerStyle = _extends({}, innerStyles.wrapper.style, style);

    if (virtual) {
      return _extends({}, innerStyle, containerProps.style);
    }

    return innerStyle;
  }, [containerProps.style, innerStyles.wrapper.style, style, virtual]);
  var children = useMemo(function () {
    return treeData.map(function (node) {
      return /*#__PURE__*/React.createElement(TreeNode, _extends({
        styles: styles,
        key: node.key,
        nodeKey: node.key,
        data: node
      }, node));
    });
  }, [styles, treeData]);
  var virtualChildren = useMemo(function () {
    return /*#__PURE__*/React.createElement("div", wrapperProps, list.map(function (_ref) {
      var item = _ref.data;
      var data = item.data;
      var key = data.key;
      return /*#__PURE__*/React.createElement(TransitionTreeNode, _extends({
        styles: styles,
        key: key,
        nodeKey: key,
        data: data,
        lastChildFlags: item.lastChildFlags || [],
        transitionType: transitionType,
        transitionNodes: key === TRANSITION_KEY ? transitionNodes : null,
        onTransitionEnd: onNodeTransitionEnd
      }, data));
    }));
  }, [list, onNodeTransitionEnd, styles, transitionNodes, transitionType, wrapperProps]);
  var treeNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledTree, _extends({
      ref: handleRef,
      role: "tree",
      $size: size,
      $scrollable: scrollable,
      theme: theme
    }, domProps, {
      onScroll: handleScroll,
      className: [innerStyles.wrapper.className, className].join(' ').trim(),
      style: innerStyle
    }), virtual ? virtualChildren : children);
  }, [children, className, domProps, handleRef, handleScroll, innerStyle, innerStyles.wrapper.className, scrollable, size, theme, virtual, virtualChildren]);
  useLayoutEffect(function () {
    if (scrollable && treeScrollWrapperRef.current && innerTreeRef.current) {
      if (treeScrollWrapperRef.current.scrollWidth <= treeScrollWrapperRef.current.offsetWidth) {
        innerTreeRef.current.style.width = '';
      } else {
        innerTreeRef.current.style.width = 'fit-content';
      }
    }
  });
  return /*#__PURE__*/React.createElement(TreeContext.Provider, {
    value: contextValue
  }, scrollable ? /*#__PURE__*/React.createElement(StyledTreeScrollWrapper, _extends({
    ref: treeScrollWrapperRef
  }, innerStyles.scrollWrapper, {
    theme: theme
  }), treeNode) : treeNode);
};

export default Tree;