import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _extends from "@babel/runtime/helpers/extends";
import { RightIcon as _RightIcon } from "@muya-ui/theme-light/es/icons/RightIcon";
import _range from "lodash/range";
import _pick from "lodash/pick";
import _isEmpty from "lodash/isEmpty";
import _get from "lodash/get";
var _excluded = ["data", "children", "title", "nodeKey", "isLeaf", "disabled", "checkable", "selectable", "draggable", "disableCheckbox", "icon", "expandIcon", "nodeRef", "styles", "lastChildFlags", "content"];
import React, { useMemo, useRef } from 'react';
import Animation from '../Animation';
import Checkbox from '../Checkbox';
import ExpandIcon from '../ExpandIcon';
import Spin from '../Spin';
import ExpandWrapper from '../styled/components/ExpandWrapper';
import mergeStyleItem from '../utils/mergeStyleItem';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { StyledChildTree, StyledExpandHotZoneLine, StyledLine, StyledTitle, StyledTreeIconWrapper, StyledTreeNode, StyledTreeNodeContent, StyledTreeNodeSelector, StyledTreeNodeSelectorWrapper, StyleExpandHotZone, StyledIndicator } from './styled';
import { useTreeContext } from './TreeContext';
import { useTreeNode } from './useTreeNode';
var defaultStyles = {
  node: '',
  selectedNode: '',
  checkedNode: '',
  nodeContent: '',
  nodeSelector: '',
  nodeIconWrapper: '',
  nodeTitle: '',
  childTree: ''
};

var ExpandHotZone = function ExpandHotZone(props) {
  var loading = props.loading,
      selected = props.selected,
      noChild = props.noChild,
      onExpand = props.onExpand,
      showLine = props.showLine,
      theme = props.theme,
      isLeaf = props.isLeaf,
      expanded = props.expanded,
      customIcon = props.customIcon;
  var token = theme.components.Tree;

  if (loading && noChild) {
    return /*#__PURE__*/React.createElement(StyleExpandHotZone, null, /*#__PURE__*/React.createElement(Spin, null));
  }

  if (!isLeaf) {
    return /*#__PURE__*/React.createElement(StyleExpandHotZone, {
      onClick: onExpand
    }, /*#__PURE__*/React.createElement(ExpandWrapper, {
      theme: theme,
      $expanded: expanded,
      $expandRotate: 90
    }, /*#__PURE__*/React.createElement(ExpandIcon, {
      icon: customIcon,
      iconSize: token.expandIconSize,
      DefaultIcon: token.expandIcon || _RightIcon,
      buttonType: selected ? 'primary' : 'weak'
    })));
  } else {
    return /*#__PURE__*/React.createElement(StyleExpandHotZone, null, showLine && /*#__PURE__*/React.createElement(StyledExpandHotZoneLine, null));
  }
};

var NodeSelector = function NodeSelector(props) {
  var title = props.title,
      disableNativeTitle = props.disableNativeTitle,
      styles = props.styles,
      customIcon = props.customIcon,
      contentNode = props.contentNode;
  return /*#__PURE__*/React.createElement(StyledTreeNodeSelectorWrapper, _extends({
    title: typeof title === 'string' && !disableNativeTitle ? title : ''
  }, styles.nodeSelector), customIcon && /*#__PURE__*/React.createElement(StyledTreeIconWrapper, styles.nodeIconWrapper, customIcon), contentNode || /*#__PURE__*/React.createElement(StyledTitle, styles.nodeTitle, title));
};

var Line = function Line(props) {
  var level = props.level,
      size = props.size,
      lastChildFlags = props.lastChildFlags,
      contentHeight = props.contentHeight,
      theme = props.theme;

  var levels = _range(level);

  return /*#__PURE__*/React.createElement(React.Fragment, null, levels.map(function (l) {
    var isLastLevel = l === level - 1;
    return (!lastChildFlags[l + 1] || isLastLevel) && /*#__PURE__*/React.createElement(StyledLine, {
      key: l,
      theme: theme,
      $level: l,
      $size: size,
      $isLastChild: lastChildFlags[l + 1],
      $contentHeight: contentHeight
    });
  }));
};

var IndicatorLine = function IndicatorLine(props) {
  var theme = props.theme;
  return /*#__PURE__*/React.createElement(StyledIndicator, {
    theme: theme
  });
};

var TreeNode = function TreeNode(props) {
  var data = props.data,
      children = props.children,
      title = props.title,
      nodeKey = props.nodeKey,
      isLeafProp = props.isLeaf,
      nodeDisabled = props.disabled,
      nodeCheckable = props.checkable,
      nodeSelectable = props.selectable,
      nodeDraggable = props.draggable,
      disableCheckbox = props.disableCheckbox,
      nodeIcon = props.icon,
      nodeExpandIcon = props.expandIcon,
      nodeRef = props.nodeRef,
      styles = props.styles,
      lastChildFlags = props.lastChildFlags,
      content = props.content,
      domProps = _objectWithoutProperties(props, _excluded);

  var contentRef = useRef(null);
  var lastChildRef = useRef(null);
  var childTreeRenderedRef = useRef(false);
  var theme = useTheme();
  var context = useTreeContext();
  var size = context.size,
      keyEntities = context.keyEntities,
      treeDraggable = context.draggable,
      treeIcon = context.icon,
      treeExpandIcon = context.expandIcon,
      renderNodeContent = context.renderNodeContent,
      renderNodeRightArea = context.renderNodeRightArea,
      renderCheckbox = context.renderCheckbox,
      showLine = context.showLine,
      disableNativeTitle = context.disableNativeTitle,
      renderAfterExpand = context.renderAfterExpand,
      virtual = context.virtual,
      dropLevelOffset = context.dropLevelOffset;
  var draggable = treeDraggable && nodeDraggable !== false; // 通过 tree 状态计算得到的节点状态

  var _useTreeNode = useTreeNode(props, context, contentRef),
      dragNodeHighlight = _useTreeNode.dragNodeHighlight,
      nodeState = _useTreeNode.nodeState,
      expandAppear = _useTreeNode.expandAppear,
      isLeaf = _useTreeNode.isLeaf,
      disabled = _useTreeNode.disabled,
      checkable = _useTreeNode.checkable,
      onDragEnter = _useTreeNode.onDragEnter,
      onDragOver = _useTreeNode.onDragOver,
      onDragLeave = _useTreeNode.onDragLeave,
      onDrop = _useTreeNode.onDrop,
      onDragEnd = _useTreeNode.onDragEnd,
      onCheck = _useTreeNode.onCheck,
      onExpand = _useTreeNode.onExpand,
      onMouseEnter = _useTreeNode.onMouseEnter,
      onMouseLeave = _useTreeNode.onMouseLeave,
      onContextMenu = _useTreeNode.onContextMenu,
      onClick = _useTreeNode.onClick,
      onDoubleClick = _useTreeNode.onDoubleClick,
      onDragStart = _useTreeNode.onDragStart;

  var treeNodeStyles = useMemo(function () {
    if (styles) {
      return _pick(styles, ['node', 'selectedNode', 'checkedNode', 'nodeContent', 'nodeSelector', 'nodeTitle', 'childTree']);
    }
  }, [styles]); // 连线，需要动态计算连线距底部的距离

  var lineBottom;

  if (showLine && !virtual) {
    var lastChildHeight = _get(lastChildRef, 'current.offsetHeight', 0);

    var lastChildContentHeight = _get(lastChildRef, 'current.firstChild.offsetHeight', 0);

    lineBottom = lastChildHeight / 2;

    if (lastChildHeight > lastChildContentHeight + 4) {
      lineBottom = lastChildHeight - (lastChildContentHeight + 4) / 2;
    }
  }

  var innerStyles = useStyles('tree', defaultStyles, treeNodeStyles);
  var selected = nodeState.selected,
      checked = nodeState.checked,
      halfChecked = nodeState.halfChecked,
      expanded = nodeState.expanded,
      loading = nodeState.loading,
      dragOver = nodeState.dragOver,
      dragOverGapTop = nodeState.dragOverGapTop,
      dragOverGapBottom = nodeState.dragOverGapBottom;

  var _ref = keyEntities[nodeKey] || {},
      _ref$level = _ref.level,
      level = _ref$level === void 0 ? 0 : _ref$level;

  var finalNodeStyleItem = innerStyles.node;

  if (selected) {
    finalNodeStyleItem = mergeStyleItem(finalNodeStyleItem, innerStyles.selectedNode);
  }

  if (checked) {
    finalNodeStyleItem = mergeStyleItem(finalNodeStyleItem, innerStyles.checkedNode);
  }

  var checkboxNode = useMemo(function () {
    if (checkable) {
      if (renderCheckbox) {
        return renderCheckbox(_extends({}, data, {
          checked: checked,
          halfChecked: halfChecked,
          disabled: disabled,
          disableCheckbox: disableCheckbox,
          onCheck: onCheck
        }));
      }

      return /*#__PURE__*/React.createElement(Checkbox, {
        size: "s",
        indeterminate: !checked && halfChecked,
        checked: checked,
        onChange: onCheck,
        disabled: disabled || disableCheckbox
      });
    }
  }, [checkable, checked, data, disableCheckbox, disabled, halfChecked, onCheck, renderCheckbox]);
  var customContentNode = useMemo(function () {
    var rightAreaNode;

    if (renderNodeRightArea) {
      rightAreaNode = renderNodeRightArea(_extends({}, props, nodeState, {
        isLeaf: isLeaf
      }));
    } // 自定义所有内容


    if (renderNodeContent) {
      return renderNodeContent(_extends({}, props, nodeState, {
        isLeaf: isLeaf
      })); // 自定义右侧操作区
    } else if (rightAreaNode) {
      return /*#__PURE__*/React.createElement(StyledTreeNodeSelector, null, /*#__PURE__*/React.createElement(StyledTitle, innerStyles.nodeTitle, title), rightAreaNode);
    }
  }, [innerStyles.nodeTitle, isLeaf, nodeState, props, renderNodeContent, renderNodeRightArea, title]);
  var customExpandIcon = useMemo(function () {
    var expandIcon = nodeExpandIcon || treeExpandIcon;

    if (expandIcon) {
      return /*#__PURE__*/React.createElement("span", {
        onClick: onExpand
      }, typeof expandIcon === 'function' ? expandIcon(_extends({}, props, nodeState, {
        isLeaf: isLeaf
      })) : expandIcon);
    }
  }, [isLeaf, nodeExpandIcon, nodeState, onExpand, props, treeExpandIcon]);
  var customIcon = useMemo(function () {
    var icon = nodeIcon || treeIcon;

    if (icon) {
      return typeof icon === 'function' ? icon(_extends({}, props, nodeState, {
        isLeaf: isLeaf
      })) : icon;
    }
  }, [isLeaf, nodeIcon, nodeState, props, treeIcon]);
  var contentNode = useMemo(function () {
    return /*#__PURE__*/React.createElement(StyledTreeNodeContent, _extends({
      ref: contentRef
    }, innerStyles.nodeContent, {
      draggable: !disabled && draggable || undefined,
      "aria-grabbed": !disabled && draggable || undefined,
      onDragStart: draggable ? onDragStart : undefined,
      onMouseEnter: onMouseEnter,
      onMouseLeave: onMouseLeave,
      onContextMenu: onContextMenu,
      onClick: onClick,
      onDoubleClick: onDoubleClick
    }), (dragOverGapTop || dragOverGapBottom) && /*#__PURE__*/React.createElement(IndicatorLine, {
      theme: theme
    }), /*#__PURE__*/React.createElement(ExpandHotZone, {
      loading: loading,
      selected: selected,
      noChild: _isEmpty(children),
      onExpand: onExpand,
      showLine: showLine,
      expanded: expanded,
      theme: theme,
      isLeaf: isLeaf,
      customIcon: customExpandIcon
    }), checkboxNode, /*#__PURE__*/React.createElement(NodeSelector, {
      customIcon: customIcon,
      contentNode: customContentNode,
      title: typeof customContentNode === 'string' ? customContentNode : title,
      disableNativeTitle: disableNativeTitle,
      styles: innerStyles
    }));
  }, [checkboxNode, children, customContentNode, customExpandIcon, customIcon, disableNativeTitle, disabled, dragOverGapBottom, dragOverGapTop, draggable, expanded, innerStyles, isLeaf, loading, onClick, onContextMenu, onDoubleClick, onDragStart, onExpand, onMouseEnter, onMouseLeave, selected, showLine, theme, title]);
  var childTreeNode = useMemo(function () {
    var childTree;

    if (virtual || renderAfterExpand && !expanded && !childTreeRenderedRef.current) {
      childTree = null;
    } else if (children && children.length > 0) {
      childTreeRenderedRef.current = true;
      childTree = /*#__PURE__*/React.createElement(Animation.Collapse, {
        appear: expandAppear,
        "in": expanded
      }, /*#__PURE__*/React.createElement(React.Fragment, null, children.map(function (node, index) {
        if (index === children.length - 1 && showLine) {
          return /*#__PURE__*/React.createElement(TreeNode, _extends({
            key: node.key,
            nodeKey: node.key,
            parentKey: nodeKey,
            data: node,
            styles: styles,
            nodeRef: lastChildRef
          }, node));
        } else {
          return /*#__PURE__*/React.createElement(TreeNode, _extends({
            key: node.key,
            nodeKey: node.key,
            parentKey: nodeKey,
            data: node,
            styles: styles
          }, node));
        }
      })));
    }

    return /*#__PURE__*/React.createElement(StyledChildTree, innerStyles.childTree, childTree);
  }, [children, expandAppear, expanded, innerStyles.childTree, nodeKey, renderAfterExpand, showLine, styles, virtual]);
  return /*#__PURE__*/React.createElement(StyledTreeNode, _extends({}, domProps, finalNodeStyleItem, {
    role: "treeitem",
    theme: theme,
    $size: size,
    $isLeaf: isLeaf,
    $level: level,
    $selected: selected,
    $disabled: disabled,
    $dragOver: dragOver,
    $dragOverGapTop: dragOverGapTop,
    $dragOverGapBottom: dragOverGapBottom,
    $dragNodeHighlight: dragNodeHighlight,
    $dropLevelOffset: dropLevelOffset,
    $showLine: showLine,
    $virtual: virtual,
    $lineBottom: lineBottom,
    ref: nodeRef,
    onDragEnter: draggable ? onDragEnter : undefined,
    onDragOver: draggable ? onDragOver : undefined,
    onDragLeave: draggable ? onDragLeave : undefined,
    onDrop: draggable ? onDrop : undefined,
    onDragEnd: draggable ? onDragEnd : undefined
  }), showLine && virtual && level > 0 && /*#__PURE__*/React.createElement(Line, {
    theme: theme,
    level: level,
    size: size,
    lastChildFlags: lastChildFlags,
    contentHeight: _get(contentRef, 'current.offsetHeight', 0)
  }), contentNode, childTreeNode);
};

export default TreeNode;