import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _isUndefined from "lodash/isUndefined";
import { useEffect, useMemo, useState } from 'react';
import { CheckEventTrigger, ExpandEventTrigger } from './types';
import { getTreeNodeState } from './utils';
/* istanbul ignore next */

export var useTreeNode = function useTreeNode(props, context, contentRef) {
  var nodeKey = props.nodeKey,
      isLeafProp = props.isLeaf,
      nodeDisabled = props.disabled,
      nodeCheckable = props.checkable,
      nodeSelectable = props.selectable,
      disableCheckbox = props.disableCheckbox;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      hovering = _useState2[0],
      setHovering = _useState2[1];

  var state = useMemo(function () {
    var state = getTreeNodeState(nodeKey, context);
    state.hovering = hovering;
    return state;
  }, [context, hovering, nodeKey]);
  var loaded = state.loaded,
      expanded = state.expanded,
      loading = state.loading;
  var expandOnClick = context.expandOnClick,
      checkOnClick = context.checkOnClick,
      keyEntities = context.keyEntities,
      loadData = context.loadData,
      treeDisabled = context.disabled,
      treeCheckable = context.checkable,
      notLeafCheckable = context.notLeafCheckable,
      treeSelectable = context.selectable,
      onNodeLoad = context.onNodeLoad,
      onNodeClick = context.onNodeClick,
      onNodeCheck = context.onNodeCheck,
      onNodeSelect = context.onNodeSelect,
      onNodeDoubleClick = context.onNodeDoubleClick,
      onNodeMouseEnter = context.onNodeMouseEnter,
      onNodeMouseLeave = context.onNodeMouseLeave,
      onNodeContextMenu = context.onNodeContextMenu,
      onNodeDragStart = context.onNodeDragStart,
      onNodeDragEnter = context.onNodeDragEnter,
      onNodeDragOver = context.onNodeDragOver,
      onNodeDragLeave = context.onNodeDragLeave,
      onNodeDragEnd = context.onNodeDragEnd,
      onNodeDrop = context.onNodeDrop,
      onNodeExpand = context.onNodeExpand; // 首次渲染不需要展开动画

  var _useState3 = useState(false),
      _useState4 = _slicedToArray(_useState3, 2),
      expandAppear = _useState4[0],
      setExpandAppear = _useState4[1];

  var _useState5 = useState(false),
      _useState6 = _slicedToArray(_useState5, 2),
      dragNodeHighlight = _useState6[0],
      setDragNodeHighlight = _useState6[1];

  var disabled = !!(treeDisabled || nodeDisabled);
  var hasChildren = useMemo(function () {
    var _ref = keyEntities[nodeKey] || {},
        _ref$children = _ref.children,
        children = _ref$children === void 0 ? [] : _ref$children;

    return !!children.length;
  }, [keyEntities, nodeKey]);
  var isLeaf = useMemo(function () {
    if (isLeafProp === false) {
      return false;
    }

    return isLeafProp || !loadData && !hasChildren || loadData && loaded && !hasChildren;
  }, [hasChildren, isLeafProp, loadData, loaded]);
  var checkable = useMemo(function () {
    /**
     * 1. 树或者树节点 checkable 为 false，那么当前节点不能选中
     * 2. 树可以选中且当前为非叶子节点，且 notLeafCheckable 为 false，那么当前节点不能选中
     */
    if (!treeCheckable || nodeCheckable === false || treeCheckable && (nodeCheckable || _isUndefined(nodeCheckable)) && !isLeaf && !notLeafCheckable) {
      return false;
    }

    return treeCheckable;
  }, [treeCheckable, nodeCheckable, isLeaf, notLeafCheckable]);
  var selectable = useMemo(function () {
    if (typeof nodeSelectable === 'boolean') {
      return nodeSelectable;
    }

    return treeSelectable;
  }, [nodeSelectable, treeSelectable]);
  var eventData = useMemo(function () {
    return _extends({}, state, props, {
      nodeRef: contentRef
    });
  }, [contentRef, props, state]);

  var onExpand = _useEventCallback(function (e) {
    var trigger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ExpandEventTrigger.ExpandIcon;
    e.stopPropagation(); // 事件触发的第一次展开需要动画

    if (!expandAppear) {
      setExpandAppear(true);
    }

    onNodeExpand(e, eventData, trigger);
  });

  var onSelect = _useEventCallback(function (e) {
    if (disabled) return;
    onNodeSelect(e, eventData);
  }, [disabled, eventData, onNodeSelect]);

  var onCheck = _useEventCallback(function (e) {
    var trigger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : CheckEventTrigger.Checkbox;
    if (disabled) return;
    if (!checkable || disableCheckbox) return;
    onNodeCheck(e, eventData, trigger);
  });

  var onClick = _useEventCallback(function (e) {
    onNodeClick(e, eventData);

    if (selectable) {
      onSelect(e);
    }

    if (checkOnClick) {
      onCheck(e, CheckEventTrigger.TreeNode);
    }

    if (expandOnClick) {
      onExpand(e, ExpandEventTrigger.TreeNode);
    }
  });

  var onDoubleClick = _useEventCallback(function (e) {
    onNodeDoubleClick(e, eventData);
  });

  var onMouseEnter = _useEventCallback(function (e) {
    if (!hovering) {
      setHovering(true);
    }

    onNodeMouseEnter(e, eventData);
  });

  var onMouseLeave = _useEventCallback(function (e) {
    if (hovering) {
      setHovering(false);
    }

    onNodeMouseLeave(e, eventData);
  });

  var onContextMenu = _useEventCallback(function (e) {
    onNodeContextMenu(e, eventData);
  });

  var onDragStart = _useEventCallback(function (e) {
    e.stopPropagation();
    setDragNodeHighlight(true);
    onNodeDragStart(e, eventData);

    try {
      e.dataTransfer.setData('text/plain', '');
    } catch (error) {}
  });

  var onDragEnter = _useEventCallback(function (e) {
    e.preventDefault();
    e.stopPropagation();
    onNodeDragEnter(e, eventData);
  });

  var onDragOver = _useEventCallback(function (e) {
    e.preventDefault();
    e.stopPropagation();
    onNodeDragOver(e, eventData);
  });

  var onDragLeave = _useEventCallback(function (e) {
    e.stopPropagation();
    onNodeDragLeave(e, eventData);
  });

  var onDragEnd = _useEventCallback(function (e) {
    e.stopPropagation();
    setDragNodeHighlight(false);
    onNodeDragEnd(e, eventData);
  });

  var onDrop = _useEventCallback(function (e) {
    e.preventDefault();
    e.stopPropagation();
    setDragNodeHighlight(false);
    onNodeDrop(e, eventData);
  });

  useEffect(function () {
    if (loading) return;

    if (loadData && expanded && !isLeaf && !loaded && !hasChildren) {
      onNodeLoad(eventData);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [expanded, hasChildren, isLeaf, loadData, loaded]);
  return {
    dragNodeHighlight: dragNodeHighlight,
    nodeState: state,
    expandAppear: expandAppear,
    disabled: disabled,
    checkable: checkable,
    selectable: selectable,
    isLeaf: isLeaf,
    onSelect: onSelect,
    onCheck: onCheck,
    onClick: onClick,
    onDoubleClick: onDoubleClick,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    onContextMenu: onContextMenu,
    onDragStart: onDragStart,
    onDragEnter: onDragEnter,
    onDragOver: onDragOver,
    onDragLeave: onDragLeave,
    onDragEnd: onDragEnd,
    onDrop: onDrop,
    onExpand: onExpand
  };
};