import { SelectIcon as _SelectIcon } from "@muya-ui/theme-light/es/icons/SelectIcon";
import _typeof from "@babel/runtime/helpers/typeof";
import _extends from "@babel/runtime/helpers/extends";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import React, { useRef } from 'react';
import Result from '../Result';
import Tree from '../Tree';
import { StyledTreeNodeLabel } from '../Tree/styled';
import Trigger from '../Trigger';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { StyledTreeNodeLabelWithIcon, StyledTreePopup } from './styled';
import TreeSelectInput from './TreeSelectInput';
import useTreeSelect from './useTreeSelect';
var defaultStyles = {
  inputWrapper: '',
  popup: '',
  scrollContainer: ''
};
var TreeSelect = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      treeCheckable = props.treeCheckable,
      selectedIcon = props.selectedIcon,
      treeRenderNodeContent = props.treeRenderNodeContent,
      notFoundContent = props.notFoundContent,
      styles = props.styles,
      inputRef = props.inputRef,
      popupWidth = props.popupWidth;
  var theme = useTheme();
  var token = theme.components.TreeSelect;
  var innerInputRef = useRef(null);

  var handleInputRef = _useForkRef(innerInputRef, inputRef);

  var nodeRef = useRef(null);

  var handleRef = _useForkRef(ref, nodeRef);

  var _useTreeSelect = useTreeSelect(props, nodeRef, innerInputRef),
      getTriggerProps = _useTreeSelect.getTriggerProps,
      getInputProps = _useTreeSelect.getInputProps,
      getTreeProps = _useTreeSelect.getTreeProps,
      treeWidthRef = _useTreeSelect.treeWidthRef,
      showEmptyView = _useTreeSelect.showEmptyView,
      noResultText = _useTreeSelect.noResultText,
      showSearch = _useTreeSelect.showSearch,
      inputValue = _useTreeSelect.inputValue;

  var innerStyles = useStyles('tree-select', defaultStyles, styles);

  var renderNodeContent = function renderNodeContent(props) {
    if (typeof treeRenderNodeContent === 'function') {
      return treeRenderNodeContent(_extends({}, props, {
        inputValue: inputValue
      }));
    } else {
      var titleNode = props.title; // 搜索模式下文本高亮，title 为对象类型时无法匹配文本，因此忽略高亮

      if (showSearch && _typeof(props.title) !== 'object') {
        var title = "".concat(props.title) || '';
        var index = title.indexOf(inputValue);
        var beforeStr = title.substr(0, index);
        var afterStr = title.substr(index + inputValue.length);
        titleNode = index > -1 ? /*#__PURE__*/React.createElement("span", null, beforeStr, /*#__PURE__*/React.createElement("span", {
          style: {
            color: theme.colors.pattern.text.highlight
          }
        }, inputValue), afterStr) : /*#__PURE__*/React.createElement("span", null, title);
      }

      if (treeCheckable) {
        return /*#__PURE__*/React.createElement(StyledTreeNodeLabel, null, titleNode);
      } else {
        var SelectedIcon = token.selectedIcon || _SelectIcon;
        var selectedIconNode = selectedIcon || /*#__PURE__*/React.createElement(SelectedIcon, {
          style: {
            flexShrink: 0,
            marginLeft: token.selectedIconMarginLeft,
            color: token.selectedIconColor
          }
        });
        return /*#__PURE__*/React.createElement(StyledTreeNodeLabelWithIcon, null, /*#__PURE__*/React.createElement(StyledTreeNodeLabel, null, titleNode), props.selected && selectedIconNode);
      }
    }
  };

  var tree = /*#__PURE__*/React.createElement(StyledTreePopup, _extends({
    theme: theme,
    $width: popupWidth || treeWidthRef.current,
    $showEmptyView: showEmptyView
  }, innerStyles.popup), showEmptyView ? notFoundContent || /*#__PURE__*/React.createElement(Result, {
    subTitle: noResultText,
    type: "emptySmall"
  }) : /*#__PURE__*/React.createElement(Tree, _extends({}, getTreeProps(), {
    renderNodeContent: renderNodeContent
  })));
  return /*#__PURE__*/React.createElement(Trigger, _extends({}, getTriggerProps(), {
    popup: tree,
    offset: token.offset[size]
  }), /*#__PURE__*/React.createElement(TreeSelectInput, _extends({
    ref: handleRef,
    inputRef: handleInputRef,
    wrapperStyleItem: innerStyles.inputWrapper
  }, getInputProps())));
});
TreeSelect.__MUYA_TREESELECT = true;
export default TreeSelect;