import _extends from "@babel/runtime/helpers/extends";
import { FoldIcon as _FoldIcon } from "@muya-ui/theme-light/es/icons/FoldIcon";
import _omit from "lodash/omit";
import _endsWith from "lodash/endsWith";
import React, { cloneElement, useMemo } from 'react';
import ExpandIcon from '../ExpandIcon';
import Input, { TagsInput } from '../Input';
import { StyledPopperInputWrapper } from '../Input/styled';
import ExpandWrapper from '../styled/components/ExpandWrapper';
import addPx from '../utils/addPx';
import useTheme from '../utils/useTheme';
import { allTreeSelectInputProps } from './const';
import useTreeSelectInput from './useTreeSelectInput';
var TreeSelectInput = /*#__PURE__*/React.forwardRef(function (props, ref) {
  var id = props.id,
      disabled = props.disabled,
      readOnly = props.readOnly,
      placeholder = props.placeholder,
      showSearch = props.showSearch,
      expandIcon = props.expandIcon,
      hideExpandIcon = props.hideExpandIcon,
      inputValue = props.inputValue,
      popupVisible = props.popupVisible,
      value = props.value,
      disabledValue = props.disabledValue,
      multiple = props.multiple,
      hasValue = props.hasValue,
      width = props.width,
      backfill = props.backfill,
      inputRef = props.inputRef,
      onClear = props.onClear,
      prefixNode = props.prefixNode,
      suffixNode = props.suffixNode,
      maxVerticalTagCount = props.maxVerticalTagCount,
      maxTagWidth = props.maxTagWidth,
      maxTagTextLength = props.maxTagTextLength,
      wrapperStyleItem = props.wrapperStyleItem,
      status = props.status,
      renderTag = props.renderTag,
      filterTag = props.filterTag,
      _props$clearReplace = props.clearReplace,
      clearReplace = _props$clearReplace === void 0 ? true : _props$clearReplace;
  var theme = useTheme();
  var token = theme.components.Select;

  var _useTreeSelectInput = useTreeSelectInput(props),
      getInputLabel = _useTreeSelectInput.getInputLabel,
      getTagText = _useTreeSelectInput.getTagText,
      handleRemoveTag = _useTreeSelectInput.handleRemoveTag,
      handleInput = _useTreeSelectInput.handleInput;

  var inputRestProps = _omit(props, allTreeSelectInputProps);

  var finalSuffixNode = useMemo(function () {
    var finalSuffixNode = [];

    if (suffixNode) {
      finalSuffixNode.push(suffixNode);
    }

    if (!hideExpandIcon) {
      var DefaultExpandIcon = token.expandIcon || _FoldIcon;
      finalSuffixNode.push( /*#__PURE__*/React.createElement(ExpandWrapper, {
        key: "expand-icon",
        theme: theme,
        $expanded: popupVisible
      }, /*#__PURE__*/React.createElement(ExpandIcon, {
        icon: expandIcon,
        DefaultIcon: DefaultExpandIcon,
        disabled: disabled
      })));
    }

    return finalSuffixNode;
  }, [disabled, expandIcon, hideExpandIcon, popupVisible, suffixNode, theme, token.expandIcon]);

  var renderSingleModeNode = function renderSingleModeNode() {
    var inputProps = _extends({}, inputRestProps, {
      disabled: disabled,
      readOnly: readOnly,
      width: typeof width === 'string' && _endsWith(width, '%') ? '100%' : width,
      placeholder: placeholder,
      prefixNode: prefixNode,
      value: inputValue,
      inputRef: inputRef,
      suffixNode: finalSuffixNode,
      clearReplace: clearReplace,
      onInput: handleInput,
      onClear: onClear
    });

    var label = getInputLabel();

    if (showSearch && popupVisible) {
      if (!backfill) {
        inputProps.placeholder = "".concat(label) || placeholder;
      }
    } else {
      inputProps.value = "".concat(label);
      inputProps.readOnly = true;
    }

    var inputElement = /*#__PURE__*/React.createElement(Input, {
      type: "text"
    });
    return /*#__PURE__*/cloneElement(inputElement, _extends({
      ref: ref
    }, inputProps));
  }; // 渲染 input 和 tag


  var renderMultipleModeNode = function renderMultipleModeNode() {
    return /*#__PURE__*/React.createElement(TagsInput, _extends({}, inputRestProps, {
      ref: ref,
      inputRef: inputRef,
      suffixNode: finalSuffixNode,
      width: typeof width === 'string' && _endsWith(width, '%') ? '100%' : width,
      disabled: disabled,
      readOnly: readOnly,
      inputReadOnly: !showSearch,
      value: filterTag ? value.filter(filterTag) : value,
      disabledValue: disabledValue,
      inputValue: inputValue,
      placeholder: hasValue ? '' : placeholder,
      getTagText: getTagText,
      scrollTopAfterEffect: !popupVisible,
      maxTagTextLength: maxTagTextLength,
      maxVerticalTagCount: maxVerticalTagCount,
      maxTagWidth: maxTagWidth,
      onRemoveTag: handleRemoveTag,
      onInput: handleInput,
      onClear: onClear,
      renderTag: renderTag
    }));
  };

  var renderInput = function renderInput() {
    if (multiple) {
      return renderMultipleModeNode();
    } else {
      return renderSingleModeNode();
    }
  };

  return /*#__PURE__*/React.createElement(StyledPopperInputWrapper, _extends({
    id: id,
    role: "combobox",
    key: "selection",
    $pointer: !multiple && !showSearch && !disabled && !readOnly,
    $popupVisible: popupVisible,
    $width: addPx(width),
    $status: status,
    theme: theme
  }, wrapperStyleItem), renderInput());
});
export default TreeSelectInput;