import _extends from "@babel/runtime/helpers/extends";
import { usePrevious as _usePrevious } from "@muya-ui/utils/es/hooks/usePrevious";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import _get from "lodash/get";
var _excluded = ["value", "defaultValue", "disabled", "readOnly", "allowClear", "onChange", "size", "placeholder", "showSearch", "backfill", "maxVerticalTagCount", "labelInValue", "onClear", "onSelect", "onDeselect", "maxTagCount", "expandIcon", "hideExpandIcon", "renderTag", "filterTag", "clearInputWhenSelect", "treeData", "treeIcon", "treeExpandIcon", "treeExpandOnClick", "treeRenderAfterExpand", "treeRenderNodeContent", "filterTreeNode", "treeExpandedKeys", "treeAutoExpandParent", "treeDefaultExpandParent", "treeDefaultExpandedKeys", "treeDefaultExpandAll", "showAllChecked", "onTreeExpand", "treeCheckStrictly", "treeScrollable", "treeVirtual", "treeDynamicPropsForNode", "loadData", "onLoad", "customKeyName", "customTitleName", "customChildrenName", "fieldNames", "popupVisible", "popupWidth", "getPopupContainer", "onPopupVisibleChange", "placement", "leaveDelay", "styles"];
import { useEffect, useMemo, useRef, useState } from 'react';
import { shallowEqualArrays } from 'shallow-equal';
import useLocale from '../Locale/useLocale';
import { conductCheck, convertDataToEntities, transformTreeData, warningWithoutKey } from '../Tree/utils';
import { arrayAdd, arrayDel } from '../utils/array';
import { defaultFilterFunc, getCheckedKeysFromValue, getFilterTree, getValueFromProps } from './utils';
export default function useTreeSelect(props, nodeRef, inputRef) {
  _devWarning('[Tree]: prop `customKeyName`、`customTitleName`、`customChildrenName` is deprecated，use `fieldNames` instead.', 'customKeyName' in props || 'customTitleName' in props || 'customChildrenName' in props);

  var locale = useLocale();
  var _props$multiple = props.multiple,
      multiple = _props$multiple === void 0 ? false : _props$multiple,
      treeCheckable = props.treeCheckable;
  /**
   * 是否多选，checkable 时自动开启
   */

  var isMultiple = !!(multiple || treeCheckable);

  var value = props.value,
      _props$defaultValue = props.defaultValue,
      defaultValue = _props$defaultValue === void 0 ? [] : _props$defaultValue,
      disabled = props.disabled,
      readOnly = props.readOnly,
      allowClear = props.allowClear,
      onChange = props.onChange,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$placeholder = props.placeholder,
      placeholder = _props$placeholder === void 0 ? locale['TreeSelect.placeholder'] : _props$placeholder,
      _props$showSearch = props.showSearch,
      showSearch = _props$showSearch === void 0 ? isMultiple : _props$showSearch,
      _props$backfill = props.backfill,
      backfill = _props$backfill === void 0 ? false : _props$backfill,
      _props$maxVerticalTag = props.maxVerticalTagCount,
      maxVerticalTagCount = _props$maxVerticalTag === void 0 ? 2.5 : _props$maxVerticalTag,
      labelInValue = props.labelInValue,
      onClear = props.onClear,
      onSelect = props.onSelect,
      onDeselect = props.onDeselect,
      maxTagCount = props.maxTagCount,
      expandIcon = props.expandIcon,
      hideExpandIcon = props.hideExpandIcon,
      renderTag = props.renderTag,
      filterTag = props.filterTag,
      _props$clearInputWhen = props.clearInputWhenSelect,
      clearInputWhenSelect = _props$clearInputWhen === void 0 ? true : _props$clearInputWhen,
      treeData = props.treeData,
      treeIcon = props.treeIcon,
      treeExpandIcon = props.treeExpandIcon,
      treeExpandOnClick = props.treeExpandOnClick,
      treeRenderAfterExpand = props.treeRenderAfterExpand,
      treeRenderNodeContent = props.treeRenderNodeContent,
      _props$filterTreeNode = props.filterTreeNode,
      filterTreeNode = _props$filterTreeNode === void 0 ? defaultFilterFunc : _props$filterTreeNode,
      treeExpandedKeys = props.treeExpandedKeys,
      treeAutoExpandParent = props.treeAutoExpandParent,
      _props$treeDefaultExp = props.treeDefaultExpandParent,
      treeDefaultExpandParent = _props$treeDefaultExp === void 0 ? true : _props$treeDefaultExp,
      _props$treeDefaultExp2 = props.treeDefaultExpandedKeys,
      treeDefaultExpandedKeys = _props$treeDefaultExp2 === void 0 ? [] : _props$treeDefaultExp2,
      treeDefaultExpandAll = props.treeDefaultExpandAll,
      _props$showAllChecked = props.showAllChecked,
      showAllChecked = _props$showAllChecked === void 0 ? true : _props$showAllChecked,
      onTreeExpand = props.onTreeExpand,
      treeCheckStrictly = props.treeCheckStrictly,
      treeScrollable = props.treeScrollable,
      treeVirtual = props.treeVirtual,
      treeDynamicPropsForNode = props.treeDynamicPropsForNode,
      loadData = props.loadData,
      onLoad = props.onLoad,
      _props$customKeyName = props.customKeyName,
      customKeyName = _props$customKeyName === void 0 ? 'key' : _props$customKeyName,
      _props$customTitleNam = props.customTitleName,
      customTitleName = _props$customTitleNam === void 0 ? 'title' : _props$customTitleNam,
      _props$customChildren = props.customChildrenName,
      customChildrenName = _props$customChildren === void 0 ? 'children' : _props$customChildren,
      fieldNamesProp = props.fieldNames,
      popupVisible = props.popupVisible,
      popupWidth = props.popupWidth,
      getPopupContainer = props.getPopupContainer,
      onPopupVisibleChange = props.onPopupVisibleChange,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      _props$leaveDelay = props.leaveDelay,
      leaveDelay = _props$leaveDelay === void 0 ? 100 : _props$leaveDelay,
      styles = props.styles,
      restProps = _objectWithoutProperties(props, _excluded);

  var treeWidthRef = useRef('auto');
  var fieldNames = useMemo(function () {
    var fieldNames = fieldNamesProp || {};
    var names = {
      children: fieldNames.children || customChildrenName,
      key: fieldNames.key || customKeyName,
      title: fieldNames.title || customTitleName,
      isLeaf: fieldNames.isLeaf || 'isLeaf'
    };
    return names;
  }, [customChildrenName, customKeyName, customTitleName, fieldNamesProp]);
  var finalAllowClear = allowClear && !disabled && !readOnly;
  /**
   * 是否受控
   */

  var isControlled = ('value' in props);
  var isPopupVisibleControlled = ('popupVisible' in props);
  /**
   * Tree 数据
   */

  var _useState = useState(transformTreeData(treeData, fieldNames)),
      _useState2 = _slicedToArray(_useState, 2),
      treeDataState = _useState2[0],
      setTreeDataState = _useState2[1];
  /**
   * 直接用 key 索引的 treeData 数据
   */


  var _useState3 = useState(function () {
    return convertDataToEntities(treeDataState, fieldNames).keyEntities;
  }),
      _useState4 = _slicedToArray(_useState3, 2),
      keyEntities = _useState4[0],
      setKeyEntities = _useState4[1];
  /**
   * 搜索框的的内容：
   * 1. 单选且 showSearch 为 true 时，显示搜索框
   * 2. 多选时，显示搜索框
   */


  var _useState5 = useState(''),
      _useState6 = _slicedToArray(_useState5, 2),
      inputValue = _useState6[0],
      setInputValue = _useState6[1];
  /**
   * 弹出状态，可传入 popupVisible 作为初始值并进行控制
   */


  var _useState7 = useState(popupVisible || false),
      _useState8 = _slicedToArray(_useState7, 2),
      popupVisibleState = _useState8[0],
      setPopupVisibleState = _useState8[1];
  /**
   * 选中的值
   */


  var _useState9 = useState(function () {
    return getValueFromProps(props);
  }),
      _useState10 = _slicedToArray(_useState9, 2),
      valueState = _useState10[0],
      setValueState = _useState10[1];

  var _useState11 = useState(function () {
    return getCheckedKeysFromValue(props);
  }),
      _useState12 = _slicedToArray(_useState11, 2),
      checkedKeysState = _useState12[0],
      setCheckedKeysState = _useState12[1];
  /**
   * 上一次的 inputValue
   */


  var prevInputValue = _usePrevious(inputValue);

  var prevTreeData = _usePrevious(treeData);
  /**
   * 当前值是否存在
   */


  var hasValue = isMultiple && valueState.length > 0 || !isMultiple && !!valueState[0];
  /**
   * 第一个选中节点的 entity
   */

  var firstSelectedEntity = useMemo(function () {
    if (valueState && valueState[0]) {
      if (labelInValue) {
        return keyEntities[valueState[0].value];
      } else {
        return keyEntities[valueState[0]];
      }
    }
  }, [keyEntities, labelInValue, valueState]);
  var valueKeys = useMemo(function () {
    if (labelInValue) {
      return valueState.map(function (item) {
        return item.value;
      });
    } else {
      return valueState;
    }
  }, [labelInValue, valueState]);
  var filterTreeData = useMemo(function () {
    return getFilterTree(treeDataState, inputValue, filterTreeNode);
  }, [filterTreeNode, inputValue, treeDataState]);
  var disabledValue = useMemo(function () {
    return valueKeys.filter(function (key) {
      return _get(keyEntities[key], 'node.disabled', false);
    });
  }, [keyEntities, valueKeys]);
  var showEmptyView = filterTreeData.length === 0;
  /**
   * 单选
   * 非搜索模式：统一清空
   * 搜索模式：
   * - backfill 为 true，如果有选中的则设置为选中的，否则清空
   * - backfill 为 false，清空
   */

  var resetInputValue = function resetInputValue() {
    if (!multiple && showSearch && backfill) {
      if (firstSelectedEntity) {
        var title = firstSelectedEntity.node.title;
        setInputValue("".concat(title));
      } else {
        inputValue && setInputValue('');
      }
    } else {
      inputValue && setInputValue('');
    }
  };

  var focusInput = function focusInput() {
    if (inputRef && inputRef.current) {
      inputRef.current.focus();
    }
  };
  /**
   * 修改 popup 状态，内部使用
   */


  var handlePopupVisibleChange = function handlePopupVisibleChange(popupVisible) {
    if (!isPopupVisibleControlled) {
      setPopupVisibleState(popupVisible);
    }

    if (onPopupVisibleChange) {
      onPopupVisibleChange(popupVisible);
    } // 动态调整下拉菜单的宽度


    if (!popupWidth && nodeRef.current && nodeRef.current.offsetWidth !== treeWidthRef.current && popupVisible) {
      treeWidthRef.current = nodeRef.current.offsetWidth;
    }
  };
  /**
   * popup 动画退出时，清空 input
   */


  var handlePopupExited = function handlePopupExited() {
    resetInputValue();
  };

  var getLabeledValue = function getLabeledValue(value) {
    return value.map(function (key) {
      if (keyEntities[key]) {
        return {
          label: keyEntities[key].node.title,
          value: key
        };
      } else {
        return {
          label: key,
          value: key
        };
      }
    });
  };
  /**
   * value 变化回调
   */


  var handleChange = function handleChange(value, visible, info) {
    if (onChange) {
      onChange(value, info);
    } // 单选需要关闭


    handlePopupVisibleChange(visible);
  };

  var triggerChange = function triggerChange(selectedKeys, info) {
    if (isMultiple) {
      if (typeof maxTagCount === 'undefined' || Object.keys(valueState).length < maxTagCount) {
        if (labelInValue) {
          handleChange(getLabeledValue(selectedKeys), true, info);
        } else {
          handleChange(selectedKeys, true, info);
        }

        if (clearInputWhenSelect) {
          inputValue && setInputValue('');
        }
      }
    } else {
      if (labelInValue) {
        handleChange({
          label: info.data.title,
          value: selectedKeys[0]
        }, false, info);
      } else {
        handleChange(selectedKeys[0], false, info);
      }
    }
  };
  /**
   * 清空输入框
   */


  var handleClear = function handleClear(e) {
    inputValue && setInputValue('');

    if (isMultiple) {
      handleChange([], false);
    } else {
      handleChange(undefined, false);
    }

    if (treeCheckable) {
      setCheckedKeysState([]);
    }

    if (!isControlled) {
      setValueState([]);
    }

    if (onClear) {
      onClear(e);
    }
  };
  /**
   * 输入框输入文本
   */


  var handleInputChange = function handleInputChange(value) {
    setInputValue(value);
  };
  /**
   * 选择
   */


  var handleSelect = function handleSelect(selectedKeys, info) {
    focusInput();

    if (info.data.disabled && info.nativeEvent) {
      info.nativeEvent.stopPropagation();
      return;
    }

    triggerChange(selectedKeys, info);

    if (!isControlled && selectedKeys) {
      if (labelInValue) {
        setValueState(getLabeledValue(selectedKeys));
      } else {
        setValueState(selectedKeys);
      }
    }

    if (info.selected && onSelect) {
      onSelect(selectedKeys, info);
    } else if (!info.selected && onDeselect) {
      onDeselect(selectedKeys, info);
    }
  };

  var removeCheckKey = function removeCheckKey(nodeKey) {
    var oriCheckedKeys = [];
    var oriHalfCheckedKeys = [];

    if (Array.isArray(checkedKeysState)) {
      oriCheckedKeys = checkedKeysState;
    } else {
      oriCheckedKeys = checkedKeysState.checked;
      oriHalfCheckedKeys = checkedKeysState.halfChecked;
    }

    var targetChecked = false;
    var checkedObj;

    if (treeCheckStrictly) {
      var newCheckedKeys = targetChecked ? arrayAdd(oriCheckedKeys, nodeKey) : arrayDel(oriCheckedKeys, nodeKey);
      var newHalfCheckedKeys = arrayDel(oriHalfCheckedKeys, nodeKey);
      checkedObj = {
        checked: newCheckedKeys,
        halfChecked: newHalfCheckedKeys
      };
    } else {
      var _conductCheck = conductCheck([nodeKey], targetChecked, keyEntities, {
        checkedKeys: oriCheckedKeys,
        halfCheckedKeys: oriHalfCheckedKeys
      }),
          _newCheckedKeys = _conductCheck.checked,
          _newHalfCheckedKeys = _conductCheck.halfChecked;

      checkedObj = {
        checked: _newCheckedKeys,
        halfChecked: _newHalfCheckedKeys
      };

      _newCheckedKeys.forEach(function (checkedKey) {
        var entity = keyEntities[checkedKey];
        if (!entity) return;
      });
    }

    setCheckedKeysState(checkedObj);
    return checkedObj.checked;
  };

  var handleDeselect = function handleDeselect(nodeKey, e) {
    e.stopPropagation();
    focusInput();
    var newKeys = valueKeys.filter(function (key) {
      return key !== nodeKey;
    });

    if (treeCheckable) {
      newKeys = removeCheckKey(nodeKey);
    }

    if (isMultiple) {
      if (labelInValue) {
        handleChange(getLabeledValue(newKeys), true);
      } else {
        handleChange(newKeys, true);
      }

      if (clearInputWhenSelect) {
        inputValue && setInputValue('');
      }
    } else {
      handleChange(undefined, false);
    }

    if (!isControlled && newKeys) {
      if (labelInValue) {
        setValueState(getLabeledValue(newKeys));
      } else {
        setValueState(newKeys);
      }
    }

    if (onDeselect) {
      onDeselect(newKeys);
    }
  };
  /** 如果父节点选中了，那么过滤掉子节点 */


  var filterChildCheckedKeys = function filterChildCheckedKeys(key, index, keys) {
    var entity = keyEntities[key];

    if (entity && entity.parent) {
      var parentKey = entity.parent.key;

      if (keys.indexOf(parentKey) > -1) {
        return false;
      }
    }

    return true;
  };

  var handleCheck = function handleCheck(checkedKeys, info) {
    setCheckedKeysState(checkedKeys);
    var newKeys = [];

    if (showAllChecked || treeCheckStrictly) {
      newKeys = checkedKeys.checked;
    } else {
      newKeys = checkedKeys.checked.filter(filterChildCheckedKeys);
    }

    if (!isControlled) {
      if (labelInValue) {
        setValueState(getLabeledValue(newKeys));
      } else {
        setValueState(newKeys);
      }
    }

    triggerChange(newKeys, info);

    if (info.checked && onSelect) {
      onSelect(newKeys, info);
    } else if (!info.checked && onDeselect) {
      onDeselect(newKeys, info);
    }
  };

  var getTriggerProps = function getTriggerProps() {
    return _extends({}, restProps, {
      container: getPopupContainer,
      placement: placement,
      triggerAction: 'click',
      open: disabled || readOnly ? false : popupVisibleState,
      onVisibleChange: handlePopupVisibleChange,
      transitionProps: {
        onExited: handlePopupExited
      },
      hideArrow: true,
      leaveDelay: leaveDelay,
      disabled: disabled || readOnly
    });
  };

  var getInputProps = function getInputProps() {
    var inputProps = {
      size: size,
      allowClear: finalAllowClear,
      readOnly: readOnly,
      disabled: disabled,
      placeholder: placeholder,
      showSearch: showSearch,
      inputValue: inputValue,
      popupVisible: popupVisibleState,
      value: valueKeys,
      disabledValue: disabledValue,
      treeKeyEntities: keyEntities,
      multiple: isMultiple,
      hasValue: hasValue,
      backfill: backfill,
      maxVerticalTagCount: maxVerticalTagCount,
      onClear: handleClear,
      onInputChange: handleInputChange,
      onDeselect: handleDeselect,
      labelInValue: labelInValue,
      expandIcon: expandIcon,
      renderTag: renderTag,
      filterTag: filterTag,
      hideExpandIcon: hideExpandIcon
    };
    return inputProps;
  };

  var getTreeProps = function getTreeProps() {
    var treeCheckOnClick = treeCheckable;
    var treeSelectable = !treeCheckable;
    var treeProps = {
      size: size,
      icon: treeIcon,
      expandIcon: treeExpandIcon,
      expandOnClick: treeExpandOnClick,
      checkOnClick: treeCheckOnClick,
      renderAfterExpand: treeRenderAfterExpand,
      multiple: multiple,
      data: treeDataState,
      filterTreeData: filterTreeData,
      defaultExpandParent: treeDefaultExpandParent,
      defaultExpandAll: treeDefaultExpandAll,
      onExpand: onTreeExpand,
      checkable: treeCheckable,
      checkStrictly: treeCheckStrictly,
      selectable: treeSelectable,
      scrollable: treeScrollable,
      virtual: treeVirtual,
      dynamicPropsForNode: treeDynamicPropsForNode,
      fieldNames: fieldNames,
      loadData: loadData,
      onLoad: onLoad
    };

    if (showSearch && inputValue && !loadData && typeof treeDefaultExpandAll === 'undefined') {
      treeProps.defaultExpandAll = true;
    }

    if (treeDefaultExpandedKeys) {
      treeProps.defaultExpandedKeys = treeDefaultExpandedKeys;
    }

    if (treeAutoExpandParent) {
      treeProps.autoExpandParent = treeAutoExpandParent;
    }

    if (treeExpandedKeys) {
      treeProps.expandedKeys = treeExpandedKeys;
    }

    if (treeCheckable) {
      treeProps.checkedKeys = checkedKeysState;
      treeProps.onCheck = handleCheck;
    } else {
      treeProps.selectedKeys = valueKeys;
      treeProps.onSelect = handleSelect;
    }

    return treeProps;
  };

  useEffect(function () {
    var newTreeDataState = null;

    if (prevTreeData !== treeData) {
      newTreeDataState = transformTreeData(treeData, fieldNames);
      setTreeDataState(newTreeDataState);
    }

    if (newTreeDataState) {
      var entitiesMap = convertDataToEntities(newTreeDataState, fieldNames);
      var newKeyEntities = entitiesMap.keyEntities;
      setKeyEntities(newKeyEntities);

      if (process.env.NODE_ENV !== 'production') {
        warningWithoutKey(newTreeDataState, fieldNames);
      }
    }
  }, [treeData, treeDataState, keyEntities, prevTreeData, customKeyName, customTitleName, customChildrenName, fieldNames]);
  useEffect(function () {
    if (multiple && !popupVisibleState && inputValue !== prevInputValue && inputValue) {
      handlePopupVisibleChange(true);
    }
  });
  useEffect(function () {
    if (isControlled) {
      var newValueState = getValueFromProps(props);

      if (!shallowEqualArrays(valueState, newValueState)) {
        // value 改变时同步更新 checked 状态
        if (treeCheckable) {
          var newCheckedKeysState = getCheckedKeysFromValue(props);

          if (Array.isArray(checkedKeysState) && !shallowEqualArrays(checkedKeysState, newCheckedKeysState) || !Array.isArray(checkedKeysState) && !shallowEqualArrays(checkedKeysState.checked, newCheckedKeysState)) {
            setCheckedKeysState(newCheckedKeysState);
          }
        }

        setValueState(newValueState);
      }
    }

    if (isPopupVisibleControlled && popupVisible !== popupVisibleState) {
      setPopupVisibleState(popupVisible);
    }
  }, [popupVisible, popupVisibleState, valueState, props, treeCheckable, checkedKeysState, isControlled, isPopupVisibleControlled]);
  return {
    getTriggerProps: getTriggerProps,
    getInputProps: getInputProps,
    getTreeProps: getTreeProps,
    treeWidthRef: treeWidthRef,
    showSearch: showSearch,
    inputValue: inputValue,
    showEmptyView: showEmptyView,
    valueState: valueState,
    noResultText: locale['TreeSelect.noResultText']
  };
}