import _extends from "@babel/runtime/helpers/extends";
import _typeof from "@babel/runtime/helpers/typeof";
import deepmerge from 'deepmerge';
export var toArray = function toArray(value) {
  var ret = [];

  if (typeof value === 'undefined') {
    ret = [];
  } else if (!Array.isArray(value)) {
    ret = [value];
  } else {
    ret = value;
  }

  return ret;
};
export var getValueFromProps = function getValueFromProps(props) {
  var value = [];

  if ('value' in props) {
    value = toArray(props.value);
  } else if ('defaultValue' in props) {
    value = toArray(props.defaultValue);
  }

  return value;
};
export var getCheckedKeysFromValue = function getCheckedKeysFromValue(props) {
  return getValueFromProps(props).map(function (item) {
    if (item && _typeof(item) === 'object') {
      return item.value;
    } else {
      return item;
    }
  });
};
export function defaultFilterFunc(searchValue, node) {
  var match = false;

  if (searchValue) {
    if ("".concat(node.key).indexOf(searchValue) > -1 || "".concat(node.title).indexOf(searchValue) > -1) {
      match = true;
    }
  }

  return match;
}
export function getFilterTree(treeNodes, searchValue, filterFunc) {
  if (!searchValue) {
    return treeNodes;
  }

  function mapFilteredNodeToData(node) {
    if (!node) return null;
    var match = false;

    if (filterFunc(searchValue, node)) {
      match = true;
    }

    var children = [];

    if (node.children) {
      children = node.children.map(mapFilteredNodeToData).filter(function (n) {
        return n;
      });
    }

    if (children.length || match) {
      return deepmerge({}, _extends({}, node, {
        children: children
      }));
    }

    return null;
  }

  return treeNodes.map(mapFilteredNodeToData).filter(function (node) {
    return node;
  });
}