import _extends from "@babel/runtime/helpers/extends";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import { useForkRef as _useForkRef } from "@muya-ui/utils/es/hooks/useForkRef";
import { useMountedRef as _useMountedRef } from "@muya-ui/utils/es/hooks/useMountedRef";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import _isBoolean from "lodash/isBoolean";
var _excluded = ["triggerId", "children", "triggerActions", "triggerAction", "enterDelay", "leaveDelay", "onVisibleChange", "onClickAway", "open", "defaultOpen", "outsideCloseable", "insideCloseable", "placement", "popperProps", "TransitionComponent", "transitionProps", "container", "popup", "offset", "hideArrow", "arrowStyle", "onEscapeKeyDown", "popperNodeRef", "escapeKeyCloseable", "arrowAdaptive", "arrowPointAtCenter", "flip", "disableOpenOnExiting", "zIndex"];
import { EventEmitter } from 'events';
import React, { useCallback, useMemo, useRef, useState } from 'react';
import ReactDOM from 'react-dom';
import Animation from '../Animation';
import Popper from '../Popper';
import { StyledArrow } from '../styled/components/Arrow';
import getSlideDirection from '../utils/getSlideDirection';
import memoForwardRef from '../utils/memoForwardRef';
import { defaultArrowMargin, defaultArrowWidth, StyledTriggerWrapper } from './styled';
var ctx = new EventEmitter();
ctx.setMaxListeners(30);
document.addEventListener('click', function (e) {
  ctx.emit('click_away', e);
});
var Trigger = memoForwardRef(function (props, ref) {
  var triggerId = props.triggerId,
      children = props.children,
      triggerActions = props.triggerActions,
      triggerAction = props.triggerAction,
      _props$enterDelay = props.enterDelay,
      enterDelay = _props$enterDelay === void 0 ? 0 : _props$enterDelay,
      _props$leaveDelay = props.leaveDelay,
      leaveDelay = _props$leaveDelay === void 0 ? 0 : _props$leaveDelay,
      onVisibleChange = props.onVisibleChange,
      onClickAway = props.onClickAway,
      _props$open = props.open,
      openProp = _props$open === void 0 ? false : _props$open,
      _props$defaultOpen = props.defaultOpen,
      defaultOpen = _props$defaultOpen === void 0 ? false : _props$defaultOpen,
      _props$outsideCloseab = props.outsideCloseable,
      outsideCloseable = _props$outsideCloseab === void 0 ? true : _props$outsideCloseab,
      _props$insideCloseabl = props.insideCloseable,
      insideCloseable = _props$insideCloseabl === void 0 ? true : _props$insideCloseabl,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom' : _props$placement,
      _props$popperProps = props.popperProps,
      popperProps = _props$popperProps === void 0 ? {} : _props$popperProps,
      _props$TransitionComp = props.TransitionComponent,
      TransitionComponent = _props$TransitionComp === void 0 ? Animation.Slide : _props$TransitionComp,
      transitionProps = props.transitionProps,
      container = props.container,
      popup = props.popup,
      offset = props.offset,
      _props$hideArrow = props.hideArrow,
      hideArrow = _props$hideArrow === void 0 ? false : _props$hideArrow,
      _props$arrowStyle = props.arrowStyle,
      arrowStyle = _props$arrowStyle === void 0 ? {} : _props$arrowStyle,
      onEscapeKeyDown = props.onEscapeKeyDown,
      popperNodeRef = props.popperNodeRef,
      _props$escapeKeyClose = props.escapeKeyCloseable,
      escapeKeyCloseable = _props$escapeKeyClose === void 0 ? true : _props$escapeKeyClose,
      _props$arrowAdaptive = props.arrowAdaptive,
      arrowAdaptive = _props$arrowAdaptive === void 0 ? false : _props$arrowAdaptive,
      _props$arrowPointAtCe = props.arrowPointAtCenter,
      arrowPointAtCenter = _props$arrowPointAtCe === void 0 ? false : _props$arrowPointAtCe,
      _props$flip = props.flip,
      flip = _props$flip === void 0 ? true : _props$flip,
      _props$disableOpenOnE = props.disableOpenOnExiting,
      disableOpenOnExiting = _props$disableOpenOnE === void 0 ? false : _props$disableOpenOnE,
      zIndex = props.zIndex,
      rest = _objectWithoutProperties(props, _excluded);

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      childIsFocusVisible = _useState2[0],
      setChildIsFocusVisible = _useState2[1];

  var _useState3 = useState(defaultOpen),
      _useState4 = _slicedToArray(_useState3, 2),
      openState = _useState4[0],
      setOpenState = _useState4[1];

  var _useState5 = useState(),
      _useState6 = _slicedToArray(_useState5, 2),
      childNode = _useState6[0],
      setChildNode = _useState6[1];

  var _React$useState = React.useState(null),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      arrowRef = _React$useState2[0],
      setArrowRef = _React$useState2[1];

  var _useState7 = useState(placement),
      _useState8 = _slicedToArray(_useState7, 2),
      realPlacement = _useState8[0],
      setRealPlacement = _useState8[1];

  var _useState9 = useState(function () {
    return getSlideDirection(placement);
  }),
      _useState10 = _slicedToArray(_useState9, 2),
      direction = _useState10[0],
      setDirection = _useState10[1];

  var _useState11 = useState(false),
      _useState12 = _slicedToArray(_useState11, 2),
      exiting = _useState12[0],
      setExiting = _useState12[1];

  var enterTimer = useRef();
  var leaveTimer = useRef();

  var mountedRef = _useMountedRef(); // 优先级：triggerActions > triggerAction > 默认值：['hover']


  var finalTriggerActions = useMemo(function () {
    if (triggerActions) {
      return triggerActions;
    } else if (triggerAction) {
      return [triggerAction];
    }

    return ['hover'];
  }, [triggerAction, triggerActions]); // 处理子节点的ref

  var handleOwnRef = React.useCallback(function (instance) {
    // eslint-disable-next-line react/no-find-dom-node
    setChildNode(ReactDOM.findDOMNode(instance));
  }, []);

  var handleChildrenRef = _useForkRef(children.ref, handleOwnRef);

  var handleRef = _useForkRef(handleChildrenRef, ref); // 处理popper弹出层节点的ref


  var popperRef = useRef(null);

  var handlePopperRef = _useForkRef(popperRef, popperNodeRef);

  var other = rest; // other会全量透传给children，类型无法确定，因此为any

  var isControlled = ('open' in props);
  var open = isControlled ? openProp : openState;
  var getContainer = useCallback(function () {
    if (typeof container === 'function') {
      return container(childNode);
    } else {
      return container;
    }
  }, [childNode, container]);

  var handleAnimationExit = _useEventCallback(function (node) {
    setExiting(true);

    if (transitionProps && transitionProps.onExit) {
      transitionProps.onExit(node);
    }
  });

  var handleAnimationExited = _useEventCallback(function (node) {
    setExiting(false);

    if (transitionProps && transitionProps.onExited) {
      transitionProps.onExited(node);
    }
  });

  var finalTransitionProps = useMemo(function () {
    return _extends({}, transitionProps, {
      onExit: handleAnimationExit,
      onExited: handleAnimationExited
    });
  }, [handleAnimationExit, handleAnimationExited, transitionProps]);

  var handleOpen = _useEventCallback(function () {
    if (disableOpenOnExiting && exiting) return;

    if (!isControlled) {
      setOpenState(true);
    }

    if (onVisibleChange && !open) {
      onVisibleChange(true);
    }
  });

  var handleClose = _useEventCallback(function () {
    if (!isControlled) {
      setOpenState(false);
    }

    if (onVisibleChange && open) {
      onVisibleChange(false);
    }
  });

  var handleEnter = _useEventCallback(function (e) {
    var childrenProps = children.props;

    if (e.type === 'mouseenter') {
      if (childrenProps.onMouseEnter) {
        childrenProps.onMouseEnter(e);
      }

      if (other.onMouseEnter) {
        other.onMouseEnter(e);
      }
    }

    clearTimeout(enterTimer.current);
    clearTimeout(leaveTimer.current);
    e.persist();
    enterTimer.current = setTimeout(function () {
      handleOpen();
    }, enterDelay);
  }, []);

  var handleBlur = _useEventCallback(function () {
    if (childIsFocusVisible) {
      setChildIsFocusVisible(false);
    }
  }, []);

  var handleFocus = _useEventCallback(function (e) {
    // Workaround for https://github.com/facebook/react/issues/7769
    // The autoFocus of React might trigger the event before the componentDidMount.
    // We need to account for this eventuality.

    /* istanbul ignore if */
    if (!childNode) {
      setChildNode(e.currentTarget);
    }

    setChildIsFocusVisible(true);
    handleEnter(e);
    var childrenProps = children.props;

    if (childrenProps.onFocus) {
      childrenProps.onFocus(e);
    }

    if (other.onFocus) {
      other.onFocus(e);
    }
  }, []);

  var handleLeave = _useEventCallback(function (e) {
    var childrenProps = children.props;

    if (e.type === 'blur') {
      if (childrenProps.onBlur) {
        childrenProps.onBlur(e);
      }

      if (other.onBlur) {
        other.onBlur(e);
      }

      handleBlur();
    }

    if (e.type === 'mouseleave') {
      if (childrenProps.onMouseLeave) {
        childrenProps.onMouseLeave(e);
      }

      if (other.onMouseLeave) {
        other.onMouseLeave(e);
      }
    }

    clearTimeout(enterTimer.current);
    clearTimeout(leaveTimer.current);
    e.persist();
    leaveTimer.current = setTimeout(function () {
      handleClose();
    }, leaveDelay);
  }, []);

  var handleClick = _useEventCallback(function (e) {
    var childrenProps = children.props;

    if (childrenProps.onClick) {
      childrenProps.onClick(e);
    }

    if (other.onClick) {
      other.onClick(e);
    }

    if (open) {
      if (insideCloseable) {
        handleClose();
      }
    } else {
      handleOpen();
    }
  }, []);

  var handleKeyDown = _useEventCallback(function (event) {
    event.stopPropagation(); // 按下ESC键触发的回调

    if (event.key === 'Escape') {
      onEscapeKeyDown && onEscapeKeyDown(event);

      if (escapeKeyCloseable) {
        // childNode should blur
        childNode && childNode.blur(); // hide trigger

        handleClose();
      }
    }

    if (children.props.onKeyDown) {
      children.props.onKeyDown(event);
    }

    if (other.onKeyDown) {
      other.onKeyDown(event);
    }
  }, []);

  var handleContextMenu = _useEventCallback(function (e) {
    e.preventDefault();
  });

  var handleMouseUp = _useEventCallback(function (e) {
    // 右键点击
    if (e.button === 2) {
      var _childrenProps = children.props;

      if (_childrenProps.onClick) {
        _childrenProps.onClick(e);
      }

      if (other.onClick) {
        other.onClick(e);
      }

      if (open) {
        if (insideCloseable) {
          handleClose();
        }
      } else {
        handleOpen();
      }
    }
  });

  var onPopperCreate = useCallback(function (data) {
    if (data.placement !== realPlacement) {
      setRealPlacement(data.placement);
      setDirection(getSlideDirection(data.placement));
    }

    if (popperProps.popperOptions && popperProps.popperOptions.onCreate) {
      popperProps.popperOptions.onCreate(data);
    }
  }, [popperProps.popperOptions, realPlacement]);
  var handleClickAway = React.useCallback(function (event) {
    // Ignore events that have been `event.preventDefault()` marked.

    /* istanbul ignore if */
    if (event.defaultPrevented) {
      return;
    } // IE 11 support, which trigger the handleClickAway even after the unbind

    /* istanbul ignore if */


    if (!mountedRef.current) {
      return;
    }
    /* istanbul ignore if */


    if (!childNode) {
      return;
    }

    var doc = childNode && childNode.ownerDocument || document;

    if (doc.documentElement && doc.documentElement.contains(event.target) && !childNode.contains(event.target) && // 子节点不包含event.target
    !(popperRef.current && popperRef.current.contains(event.target)) // Popper节点不包含event.target
    ) {
      onClickAway && onClickAway(event);

      if (outsideCloseable) {
        handleClose();
      }
    }
  }, [childNode, handleClose, mountedRef, onClickAway, outsideCloseable]);
  React.useEffect(function () {
    if (finalTriggerActions.indexOf('click') > -1 || finalTriggerActions.indexOf('contextMenu') > -1) {
      /**
       * FIXME: 原生的 addEventListener，在 rerender 时表现不正常，暂时用此写法
       */
      ctx.on('click_away', handleClickAway); // document.addEventListener('click', handleClickAway);

      return function () {
        ctx.removeListener('click_away', handleClickAway); // document.removeEventListener('click', handleClickAway);
      };
    }
  }, [finalTriggerActions, handleClickAway]);
  React.useEffect(function () {
    return function () {
      clearTimeout(enterTimer.current);
      clearTimeout(leaveTimer.current);
    };
  }, []);

  var childrenProps = _extends({
    'aria-describedby': open ? triggerId : null
  }, other, children.props);

  var interactiveWrapperListeners = {};

  if (!other.disabled) {
    if (finalTriggerActions.indexOf('hover') > -1) {
      childrenProps.onMouseEnter = handleEnter;
      childrenProps.onMouseLeave = handleLeave;
      interactiveWrapperListeners.onMouseEnter = handleEnter;
      interactiveWrapperListeners.onMouseLeave = handleLeave;
    }

    if (finalTriggerActions.indexOf('focus') > -1) {
      childrenProps.onFocus = handleFocus;
      childrenProps.onBlur = handleLeave;
      childrenProps.onKeyDown = handleKeyDown;
      interactiveWrapperListeners.onFocus = childrenProps.onFocus;
      interactiveWrapperListeners.onBlur = childrenProps.onBlur;
    }

    if (finalTriggerActions.indexOf('click') > -1) {
      childrenProps.onClick = handleClick;
      childrenProps.onKeyDown = handleKeyDown;
    }

    if (finalTriggerActions.indexOf('contextMenu') > -1) {
      childrenProps.onContextMenu = handleContextMenu;
      childrenProps.onMouseUp = handleMouseUp;
      childrenProps.onKeyDown = handleKeyDown;
    }
  } // Fix Tooltip won't hide at disabled button
  // mouse events don't trigger at disabled button in Chrome


  var getDisabledCompatibleChildren = function getDisabledCompatibleChildren(element) {
    // Muya Button用span包裹会导致样式问题，直接使用span类型的Button即可
    if (element.type.__MUYA_BUTTON && element.props.disabled) {
      return /*#__PURE__*/React.cloneElement(element, _extends({
        ref: handleRef,
        component: 'span'
      }, childrenProps));
    }

    if ((element.type === 'button' || element.type.__MUYA_SWITCH === true || element.type.__MUYA_RADIO === true || element.type.__MUYA_CHECKBOX === true) && element.props.disabled) {
      var displayStyle = element.props.style && element.props.style.display ? element.props.style.display : 'inline-block';
      var child = /*#__PURE__*/React.cloneElement(element, {
        style: _extends({}, element.props.style, {
          pointerEvents: 'none'
        })
      });
      var spanStyle = {
        display: displayStyle,
        cursor: 'not-allowed'
      };
      return /*#__PURE__*/React.createElement('span', _extends({
        ref: handleRef
      }, childrenProps, {
        style: spanStyle // children: child,

      }), child);
    }

    return /*#__PURE__*/React.cloneElement(element, _extends({
      ref: handleRef
    }, childrenProps));
  }; // arrowPointAtCenter逻辑


  var offsetEnable = false;
  var flipSetting = useMemo(function () {
    return _isBoolean(flip) ? {
      enabled: flip
    } : flip;
  }, [flip]);
  var offsetValue = 0;
  var mainPlacement = placement.split('-')[0];
  var subPlacement = placement.split('-')[1];

  if (subPlacement && childNode) {
    var arrowMargin = arrowStyle.margin || defaultArrowMargin;
    var arrowWidth = arrowStyle.width || defaultArrowWidth;

    var _childNode$getBoundin = childNode.getBoundingClientRect(),
        width = _childNode$getBoundin.width,
        height = _childNode$getBoundin.height;

    var elementHalf = (mainPlacement === 'top' || mainPlacement === 'bottom' ? width : height) / 2;
    var arrowHalf = arrowWidth / 2;
    var rawOffsetValue = elementHalf - arrowHalf - arrowMargin; // 偏移量

    var autoCenter = rawOffsetValue <= 0; // 元素宽度过小时，箭头自动居中

    if ((arrowPointAtCenter || autoCenter) && !hideArrow) {
      flipSetting.enabled = false; // Popper.js中若要开启offset，需要先关闭flip

      offsetEnable = true;
      offsetValue = subPlacement === 'start' ? rawOffsetValue : -rawOffsetValue;
    }
  }

  var modifiers = useMemo(function () {
    return _extends({
      flip: flipSetting,
      offset: {
        enabled: offsetEnable,
        offset: offsetValue
      },
      arrow: {
        enabled: arrowAdaptive && !hideArrow && Boolean(arrowRef),
        element: arrowRef
      }
    }, popperProps.modifiers);
  }, [arrowAdaptive, arrowRef, flipSetting, hideArrow, offsetEnable, offsetValue, popperProps.modifiers]);
  var popperOptions = useMemo(function () {
    return _extends({}, popperProps.popperOptions, {
      onCreate: onPopperCreate
    });
  }, [onPopperCreate, popperProps.popperOptions]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, getDisabledCompatibleChildren(children), /*#__PURE__*/React.createElement(Popper, _extends({
    ref: handlePopperRef,
    placement: placement,
    anchorEl: childNode,
    open: childNode ? open : false,
    id: childrenProps['aria-describedby'],
    container: getContainer,
    transition: true,
    transitionProps: finalTransitionProps
  }, interactiveWrapperListeners, popperProps, {
    modifiers: modifiers,
    popperOptions: popperOptions,
    zIndex: zIndex
  }), function (_ref) {
    var transitionProps = _ref.transitionProps;
    return /*#__PURE__*/React.createElement(TransitionComponent, _extends({
      direction: direction
    }, transitionProps), /*#__PURE__*/React.createElement(StyledTriggerWrapper, {
      placement: realPlacement,
      offset: offset,
      hideArrow: hideArrow,
      arrowStyle: arrowStyle
    }, popup, !hideArrow && /*#__PURE__*/React.createElement(StyledArrow, {
      ref: setArrowRef,
      placement: realPlacement,
      arrowStyle: arrowStyle
    })));
  }));
});
export default Trigger;