import _extends from "@babel/runtime/helpers/extends";
import React, { useMemo } from 'react';
import styled from 'styled-components';
import CalendarFooter from '../Calendar/CalendarFooter';
import CalendarHead from '../Calendar/CalendarHead';
import CalendarPanel from '../Calendar/CalendarPanel';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import useWeekCalendar from './useWeekCalendar';
var Root = styled.div.withConfig({
  displayName: "Root"
})(["width:", "px;background-color:", ";"], function (props) {
  return props.theme.components.Calendar.defaultWidth;
}, function (props) {
  return props.theme.colors.pattern.background.higher;
});
var defaultStyles = {
  head: '',
  panel: '',
  footer: ''
};
var WeekCalendar = memoForwardRef(function (props, ref) {
  var theme = useTheme();

  var _useWeekCalendar = useWeekCalendar(props),
      finalUIState = _useWeekCalendar.finalUIState,
      finalSelectedWeek = _useWeekCalendar.finalSelectedWeek,
      stateHoveredWeek = _useWeekCalendar.stateHoveredWeek,
      finalOptions = _useWeekCalendar.finalOptions,
      finalTodayDate = _useWeekCalendar.finalTodayDate,
      otherProps = _useWeekCalendar.otherProps,
      handleItemClick = _useWeekCalendar.handleItemClick,
      handlePrevMonth = _useWeekCalendar.handlePrevMonth,
      handleNextMonth = _useWeekCalendar.handleNextMonth,
      handlePrevYear = _useWeekCalendar.handlePrevYear,
      handleNextYear = _useWeekCalendar.handleNextYear,
      handleSwitch = _useWeekCalendar.handleSwitch,
      handleItemHover = _useWeekCalendar.handleItemHover,
      handleLeavePanel = _useWeekCalendar.handleLeavePanel,
      styles = _useWeekCalendar.styles,
      disableWeek = _useWeekCalendar.disableWeek,
      hideItemOutside = _useWeekCalendar.hideItemOutside,
      renderDecadeItem = _useWeekCalendar.renderDecadeItem,
      renderMonthItem = _useWeekCalendar.renderMonthItem,
      renderYearItem = _useWeekCalendar.renderYearItem,
      renderMonthLabel = _useWeekCalendar.renderMonthLabel;

  var pickStyles = useMemo(function () {
    if (styles) {
      var head = styles.head,
          panel = styles.panel,
          footer = styles.footer;
      return {
        head: head,
        panel: panel,
        footer: footer
      };
    }
  }, [styles]);
  var innerStyles = useStyles('calendar', defaultStyles, pickStyles); // CalendarFooter 本身就 memo了，单个节点不需要 memo 了

  var footerNode;

  if (finalOptions && finalOptions.length) {
    footerNode = /*#__PURE__*/React.createElement(CalendarFooter, _extends({}, innerStyles.footer, {
      styles: styles,
      options: finalOptions
    }));
  } // 本就是解构了传给 pure component，所以不需要 memo


  var panelProps = {
    styles: styles,
    todayDate: finalTodayDate,
    isWeek: true,
    disableWeek: disableWeek,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel
  };
  return /*#__PURE__*/React.createElement(Root, _extends({}, otherProps, {
    theme: theme,
    ref: ref
  }), /*#__PURE__*/React.createElement(CalendarHead, _extends({}, innerStyles.head, {
    styles: styles,
    viewType: finalUIState.viewType,
    viewDate: finalUIState.viewDate,
    onPrevMonth: handlePrevMonth,
    onNextMonth: handleNextMonth,
    onPrevYear: handlePrevYear,
    onNextYear: handleNextYear,
    onSwitch: handleSwitch
  })), /*#__PURE__*/React.createElement(CalendarPanel, _extends({}, innerStyles.panel, {
    viewType: finalUIState.viewType,
    viewDate: finalUIState.viewDate,
    hoveredWeek: stateHoveredWeek,
    selectedWeek: finalSelectedWeek,
    onItemClick: handleItemClick,
    onItemHover: handleItemHover,
    panelBlockProps: {
      onMouseLeave: handleLeavePanel
    }
  }, panelProps)), footerNode);
});
export default WeekCalendar;