import { devWarning as _devWarning } from "@muya-ui/utils/es/utils/devWarning";
import { useEventCallback as _useEventCallback } from "@muya-ui/utils/es/hooks/useEventCallback";
import _extends from "@babel/runtime/helpers/extends";
import _slicedToArray from "@babel/runtime/helpers/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["selectedWeek", "defaultWeek", "todayDate", "viewType", "viewDate", "options", "onUIChange", "onChange", "onSelect", "styles", "allowClear", "defaultViewDate", "disableWeek", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel"];
import dayjs from 'dayjs';
import React from 'react';
import { isSame, moveOne } from '../Calendar/utils';
import { useLocale } from '../Locale';
import { formatWeek, getViewDateByWeek, isSameWeek } from './utils';
export default function useCalendar(props) {
  var selectedWeek = props.selectedWeek,
      defaultWeek = props.defaultWeek,
      todayDate = props.todayDate,
      _props$viewType = props.viewType,
      viewType = _props$viewType === void 0 ? 'month' : _props$viewType,
      viewDate = props.viewDate,
      options = props.options,
      onUIChange = props.onUIChange,
      onChange = props.onChange,
      onSelect = props.onSelect,
      styles = props.styles,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      defaultViewDate = props.defaultViewDate,
      disableWeek = props.disableWeek,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      otherProps = _objectWithoutProperties(props, _excluded);

  var isControlled = ('selectedWeek' in props);
  var finalTodayDate = React.useMemo(function () {
    return dayjs(todayDate);
  }, [todayDate]);
  var weekFromProps = React.useMemo(function () {
    return formatWeek(selectedWeek || defaultWeek);
  }, [defaultWeek, selectedWeek]);
  var uiStateFromProps = React.useMemo(function () {
    var innerViewDate;

    if (viewDate) {
      innerViewDate = viewDate;
    } else if (weekFromProps) {
      innerViewDate = getViewDateByWeek(weekFromProps);
    } else if (defaultViewDate) {
      innerViewDate = defaultViewDate;
    } else {
      innerViewDate = finalTodayDate;
    }

    return {
      viewDate: dayjs(innerViewDate),
      viewType: viewType
    };
  }, [defaultViewDate, finalTodayDate, viewDate, viewType, weekFromProps]);

  var _React$useState = React.useState(uiStateFromProps),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      uiState = _React$useState2[0],
      setUIState = _React$useState2[1];

  var _React$useState3 = React.useState(weekFromProps),
      _React$useState4 = _slicedToArray(_React$useState3, 2),
      stateSelectedWeek = _React$useState4[0],
      setSelectedWeek = _React$useState4[1];

  var _React$useState5 = React.useState(),
      _React$useState6 = _slicedToArray(_React$useState5, 2),
      stateHoveredWeek = _React$useState6[0],
      setHoveredWeek = _React$useState6[1];

  var finalUIState = _extends({}, uiState);

  if ('viewDate' in props) {
    finalUIState.viewDate = uiStateFromProps.viewDate;
  }

  if ('viewType' in props) {
    finalUIState.viewType = uiStateFromProps.viewType;
  }

  var updateUI = function updateUI(inputState) {
    var newState = _extends({}, finalUIState, inputState);

    setUIState(newState);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var updateSelectedWeek = function updateSelectedWeek(week) {
    setSelectedWeek(week);

    if (onChange) {
      onChange(week);
    }

    if (onSelect && week) {
      onSelect(week);
    }
  };

  var selectWeek = function selectWeek(week) {
    if (allowClear && isSameWeek(week, stateSelectedWeek)) {
      updateSelectedWeek();
      return;
    }

    updateSelectedWeek(week);
    var newViewDate = getViewDateByWeek(week);

    if (!isSame(newViewDate, finalUIState.viewDate, finalUIState.viewType)) {
      updateUI({
        viewDate: newViewDate
      });
    }
  };

  var handleItemClick = _useEventCallback(function (date) {
    if (finalUIState.viewType === 'year') {
      updateUI({
        viewType: 'month',
        viewDate: finalUIState.viewDate.set('month', date.get('month'))
      });
      return;
    }

    if (finalUIState.viewType === 'decade') {
      updateUI({
        viewType: 'month',
        viewDate: finalUIState.viewDate.set('year', date.get('year'))
      });
      return;
    }

    var selectedWeek = formatWeek(date); // 从面板上点击，这里不可能是 undefined ，看看后续是否要处理

    if (selectedWeek) {
      selectWeek(selectedWeek);
    }
  });

  var handleItemHover = _useEventCallback(function (date, status) {
    if (status === 'normal' || status === 'outside' || status === 'range') {
      setHoveredWeek([date.startOf('w'), date.endOf('w')]);
    }
  });

  var handleLeavePanel = _useEventCallback(function () {
    if (stateHoveredWeek) {
      setHoveredWeek(undefined);
    }
  });

  var handleOption = _useEventCallback(function (week) {
    selectWeek(week);
  });

  var handleMove = function handleMove(step, type) {
    return function () {
      updateUI({
        viewDate: moveOne(finalUIState.viewDate, type, step)
      });
    };
  };

  var handlePrevMonth = _useEventCallback(handleMove(-1, 'month'));

  var handleNextMonth = _useEventCallback(handleMove(1, 'month'));

  var yearBtnType = finalUIState.viewType === 'decade' ? 'decade' : 'year';

  var handlePrevYear = _useEventCallback(handleMove(-1, yearBtnType));

  var handleNextYear = _useEventCallback(handleMove(1, yearBtnType));

  var handleSwitch = _useEventCallback(function (type) {
    updateUI({
      viewType: type
    });
  });

  var locale = useLocale();
  var finalOptions = React.useMemo(function () {
    if (!options || options.length < 1) {
      return;
    }

    var innerOpts = [];
    options.forEach(function (opt) {
      var innerWeek = formatWeek(opt.week);

      if (!innerWeek) {
        _devWarning('[WeekCalendar]: options passed invalid week.');

        return;
      }

      innerOpts.push({
        label: opt.label,
        disabled: opt.disabled,
        onClick: function onClick() {
          handleOption(innerWeek);
        }
      });
    });
    return innerOpts;
  }, [handleOption, options]); // 当国际化变了，得更新内存中 dayjs 对象的国际化

  var updateLocale = function updateLocale() {
    var localeKey = locale.dayjsLocaleKey;
    updateUI({
      viewDate: finalUIState.viewDate.locale(localeKey)
    });

    if (stateSelectedWeek) {
      updateSelectedWeek([stateSelectedWeek[0].locale(localeKey), stateSelectedWeek[1].locale(localeKey)]);
    }

    if (stateHoveredWeek) {
      setHoveredWeek([stateHoveredWeek[0].locale(localeKey), stateHoveredWeek[1].locale(localeKey)]);
    }
  };

  React.useEffect(function () {
    var localeKey = locale.dayjsLocaleKey; // 任一变了就可以了

    var localeChange = localeKey !== finalUIState.viewDate.locale();

    if (!localeChange) {
      return;
    }

    updateLocale();
  });
  React.useEffect(function () {
    if (isControlled) {
      setSelectedWeek(weekFromProps);
    }
  }, [isControlled, weekFromProps]);
  return {
    // 处理过的数据
    finalUIState: finalUIState,
    finalSelectedWeek: stateSelectedWeek,
    stateHoveredWeek: stateHoveredWeek,
    finalOptions: finalOptions,
    finalTodayDate: finalTodayDate,
    otherProps: otherProps,
    // 事件处理
    handleItemClick: handleItemClick,
    handlePrevMonth: handlePrevMonth,
    handleNextMonth: handleNextMonth,
    handlePrevYear: handlePrevYear,
    handleNextYear: handleNextYear,
    handleSwitch: handleSwitch,
    handleItemHover: handleItemHover,
    handleLeavePanel: handleLeavePanel,
    // 透传
    styles: styles,
    disableWeek: disableWeek,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // 另外一个 hooks 中使用
    updateSelectedWeek: updateSelectedWeek,
    // for test
    updateUI: updateUI,
    updateLocale: updateLocale,
    selectWeek: selectWeek
  };
}