import { useEffectOnce as _useEffectOnce } from "@muya-ui/utils/es/hooks/useEffectOnce";
import { ReloadIcon as _ReloadIcon } from "@muya-ui/theme-light/es/icons/ReloadIcon";
import { ReminderIcon as _ReminderIcon } from "@muya-ui/theme-light/es/icons/ReminderIcon";
import { SuccessIcon as _SuccessIcon } from "@muya-ui/theme-light/es/icons/SuccessIcon";
import { ClearIcon as _ClearIcon } from "@muya-ui/theme-light/es/icons/ClearIcon";
import { InformIcon as _InformIcon } from "@muya-ui/theme-light/es/icons/InformIcon";
import _extends from "@babel/runtime/helpers/extends";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
var _excluded = ["item", "timeout", "state", "className", "style"];
import React from 'react';
import styled, { css } from 'styled-components';
import Spin from '../Spin';
import memoForwardRef from '../utils/memoForwardRef';
import useStyles from '../utils/useStyles';
import useTheme from '../utils/useTheme';
import { withThemeForStyled } from '../utils/withTheme';
var StyledToastItem = styled.div.withConfig({
  displayName: "StyledToastItem"
})(["", ""], function (props) {
  var theme = props.theme;
  var colors = theme.colors,
      size = theme.size;
  var _theme$components$Not = theme.components.Notification.toast,
      itemMarginBottom = _theme$components$Not.itemMarginBottom,
      shadow = _theme$components$Not.shadow,
      borderRadius = _theme$components$Not.borderRadius;
  return css(["display:flex;background:", ";box-shadow:", ";border-radius:", ";pointer-events:all;margin-bottom:", "px;"], colors.pattern.background.higher, shadow, borderRadius || size.spec.borderRadius.s2, itemMarginBottom);
});
var StyledContent = styled.div.withConfig({
  displayName: "StyledContent"
})(["", ""], function (props) {
  var theme = props.theme;
  var colors = theme.colors;
  var contentStyle = theme.components.Notification.toast.contentStyle;
  return css(["box-sizing:border-box;color:", ";padding:", ";max-width:", "px;min-width:", "px;line-height:", "px;font-size:", "px;"], contentStyle.color || colors.pattern.text.text, contentStyle.padding, contentStyle.maxWidth, contentStyle.minWidth, contentStyle.lineHeight, contentStyle.fontSize);
});
var StyledIcon = styled.div.withConfig({
  displayName: "StyledIcon"
})(["box-sizing:border-box;", ""], function (props) {
  var theme = props.theme;
  var iconStyle = theme.components.Notification.toast.iconStyle;
  return css(["padding:", ";font-size:", "px;line-height:", "px;width:", "px;"], iconStyle.padding, iconStyle.size, iconStyle.size, iconStyle.width);
});
var BaseNode = styled.div.withConfig({
  displayName: "BaseNode"
})([""]);
var ToastItemWithoutStyle = memoForwardRef(function (props, ref) {
  var item = props.item,
      timeout = props.timeout,
      state = props.state,
      className = props.className,
      _props$style = props.style,
      style = _props$style === void 0 ? {} : _props$style,
      otherProps = _objectWithoutProperties(props, _excluded);

  var theme = useTheme();
  var innerStyles = useStyles('toast-item', {
    self: '',
    container: '',
    icon: '',
    content: ''
  }, item.styles);
  var iconNode;

  if (item.icon) {
    iconNode = item.icon;
  } else if (item.type !== 'loading') {
    var token = theme.components.Notification;

    var tokenIcon = _extends({
      info: _InformIcon,
      error: _ClearIcon,
      success: _SuccessIcon,
      warning: _ReminderIcon,
      loading: _ReloadIcon
    }, token.icon);

    var Icon = tokenIcon[item.type];
    var bgColor = token.iconBgColor[item.type];
    iconNode = /*#__PURE__*/React.createElement(StyledIcon, _extends({}, innerStyles.icon, {
      theme: theme
    }), /*#__PURE__*/React.createElement(Icon, {
      bgColor: bgColor
    }));
  } else {
    iconNode = /*#__PURE__*/React.createElement(StyledIcon, _extends({}, innerStyles.icon, {
      theme: theme
    }), /*#__PURE__*/React.createElement(Spin, null));
  }

  var contentNode;

  if (typeof item.content === 'string' || item.enableContentWrapper) {
    contentNode = /*#__PURE__*/React.createElement(StyledContent, _extends({}, innerStyles.content, {
      theme: theme
    }), item.content);
  } else {
    contentNode = item.content;
  }

  var containerProps = _extends({
    style: _extends({}, innerStyles.self.style, style),
    className: [className, innerStyles.self.className].join(' ')
  }, otherProps);

  _useEffectOnce(function () {
    return function () {
      if (item.onClose) {
        item.onClose();
      }
    };
  });

  return /*#__PURE__*/React.createElement(BaseNode, _extends({
    ref: ref
  }, containerProps), /*#__PURE__*/React.createElement(StyledToastItem, _extends({
    theme: theme
  }, innerStyles.container), iconNode, contentNode));
});
var ToastItem = styled(ToastItemWithoutStyle).withConfig({
  displayName: "ToastItem"
})(["", ""], function (props) {
  var theme = props.theme,
      item = props.item,
      state = props.state,
      timeout = props.timeout;
  var colors = theme.colors,
      transition = theme.transition;
  var _theme$components$Not2 = theme.components.Notification.toast,
      lineMaxChar = _theme$components$Not2.lineMaxChar,
      contentStyle = _theme$components$Not2.contentStyle,
      minHeight = _theme$components$Not2.minHeight;
  var maxHeight = item.maxHeight || minHeight;

  if (typeof item.content === 'string' && item.content.length > lineMaxChar) {
    var lineNum = Math.ceil(item.content.length / lineMaxChar);
    maxHeight = minHeight + lineNum * contentStyle.lineHeight;
  }

  var transitionMap = {
    entering: '',
    entered: css(["max-height:", "px;opacity:1;"], maxHeight),
    exiting: css(["max-height:0;opacity:0;transition-timing-function:", ";"], transition.pattern.easing.leave),
    exited: '',
    unmounted: ''
  };
  var transformCSS = transitionMap[state] || css(["max-height:0;opacity:0;"]);
  return css(["transition:all ", "ms ", ";pointer-events:none;display:flex;align-items:center;justify-content:center;", " & ", "{color:", ";}"], timeout, transition.pattern.easing.enter, transformCSS, StyledIcon, colors.pattern.feature[item.type]);
});
export default withThemeForStyled(ToastItem);