import _slicedToArray from "@babel/runtime/helpers/slicedToArray";

/* eslint-disable @typescript-eslint/no-object-literal-type-assertion */
import { useEffect, useState, useMemo, useCallback } from 'react';
import useSize from './useSize';
export default (function (list, options) {
  var _useSize = useSize(),
      _useSize2 = _slicedToArray(_useSize, 2),
      size = _useSize2[0],
      containerRef = _useSize2[1];

  var _useState = useState({
    start: 0,
    end: 10
  }),
      _useState2 = _slicedToArray(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var itemHeight = options.itemHeight,
      _options$overscan = options.overscan,
      overscan = _options$overscan === void 0 ? 5 : _options$overscan;

  if (!itemHeight) {
    console.warn('please enter a valid itemHeight');
  }

  var getViewCapacity = useCallback(function (containerHeight) {
    return Math.ceil(containerHeight / itemHeight);
  }, [itemHeight]);
  var getOffset = useCallback(function (scrollTop) {
    return Math.floor(scrollTop / itemHeight) + 1;
  }, [itemHeight]);
  var calculateRange = useCallback(function () {
    var element = containerRef.current;

    if (element) {
      var offset = getOffset(element.scrollTop);
      var viewCapacity = getViewCapacity(element.clientHeight);
      var from = offset - overscan;
      var to = offset + viewCapacity + overscan;
      setState({
        start: from < 0 ? 0 : from,
        end: to > list.length ? list.length : to
      });
    }
  }, [containerRef, getOffset, getViewCapacity, list.length, overscan]);
  useEffect(function () {
    calculateRange();
  }, [size.width, size.height, calculateRange]);
  var totalHeight = useMemo(function () {
    return list.length * itemHeight;
  }, [itemHeight, list]);
  var getDistanceTop = useCallback(function (index) {
    var height = index * itemHeight;
    return height;
  }, [itemHeight]);
  var scrollTo = useCallback(function (index) {
    if (containerRef.current) {
      containerRef.current.scrollTop = getDistanceTop(index);
      calculateRange();
    }
  }, [calculateRange, containerRef, getDistanceTop]);
  var handleRef = useCallback(function (ele) {
    containerRef.current = ele;
  }, [containerRef]);
  var onScroll = useCallback(function (e) {
    calculateRange();
  }, [calculateRange]);
  var containerProps = useMemo(function () {
    return {
      ref: handleRef,
      onScroll: onScroll,
      style: {
        overflowY: 'auto'
      }
    };
  }, [handleRef, onScroll]);
  var wrapperProps = useMemo(function () {
    return {
      style: {
        boxSizing: 'border-box',
        width: '100%',
        height: totalHeight,
        paddingTop: getDistanceTop(state.start)
      }
    };
  }, [getDistanceTop, state.start, totalHeight]);
  var virtualList = useMemo(function () {
    // 没滚动的时候完整返回
    if (containerRef.current && containerRef.current.clientHeight >= totalHeight) {
      return list.map(function (ele, index) {
        return {
          data: ele,
          index: index + state.start
        };
      });
    }

    return list.slice(state.start, state.end).map(function (ele, index) {
      return {
        data: ele,
        index: index + state.start
      };
    });
  }, [containerRef, list, state.end, state.start, totalHeight]);
  return {
    list: virtualList,
    scrollTo: scrollTo,
    containerProps: containerProps,
    wrapperProps: wrapperProps
  };
});