"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useAnchor;

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = require("react");

var _utils = require("./utils");

var sharpMatcherRegx = /#(\S+)$/;

function useAnchor(props) {
  var onChange = props.onChange,
      _props$offsetTop = props.offsetTop,
      offsetTop = _props$offsetTop === void 0 ? 0 : _props$offsetTop,
      _props$disableHash = props.disableHash,
      disableHash = _props$disableHash === void 0 ? false : _props$disableHash,
      _props$duration = props.duration,
      duration = _props$duration === void 0 ? 300 : _props$duration,
      bounds = props.bounds,
      _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'vertical' : _props$direction;
  var linksRef = (0, _react.useRef)([]);
  var animatingRef = (0, _react.useRef)(false);

  var _useState = (0, _react.useState)(window),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      container = _useState2[0],
      setContainer = _useState2[1];

  var _useState3 = (0, _react.useState)(null),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      activeLink = _useState4[0],
      setActiveLink = _useState4[1];

  var _useState5 = (0, _react.useState)(0),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      indicatorTop = _useState6[0],
      setIndicatorTop = _useState6[1];

  var _useState7 = (0, _react.useState)(0),
      _useState8 = (0, _slicedToArray2["default"])(_useState7, 2),
      indicatorHeight = _useState8[0],
      setIndicatorHeight = _useState8[1];

  var registerLink = (0, _react.useCallback)(function (link) {
    if (linksRef.current.indexOf(link) < 0) {
      linksRef.current.push(link);
    } else {
      (0, _devWarning2.devWarning)("[Anchor]: ".concat(link, " is already registered !"));
    }
  }, []);
  var unregisterLink = (0, _react.useCallback)(function (link) {
    var index = linksRef.current.indexOf(link);

    if (index > 0) {
      linksRef.current.splice(index, 1);
    }
  }, []);
  var getContainer = (0, _react.useCallback)(function (node) {
    if (node && container !== node) {
      setContainer(node);
    }
  }, [container]);
  var setCurrentActiveLink = (0, _react.useCallback)(function (link) {
    if (activeLink !== link) {
      setActiveLink(link);

      if (onChange) {
        onChange(link);
      }
    }
  }, [activeLink, onChange]);
  var getCurrentAnchor = (0, _react.useCallback)(function (offsetTop) {
    var bounds = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;
    var linkSections = [];
    linksRef.current.forEach(function (link) {
      var sharpLinkMatch = sharpMatcherRegx.exec(link.toString());

      if (!sharpLinkMatch) {
        return;
      }

      var target = document.getElementById(sharpLinkMatch[1]);

      if (target) {
        var top = (0, _utils.getOffsetTop)(target, container);

        if (top < offsetTop + bounds) {
          linkSections.push({
            link: link,
            top: top
          });
        }
      }
    });

    if (linkSections.length) {
      var maxSection = linkSections.reduce(function (prev, curr) {
        return curr.top > prev.top ? curr : prev;
      });
      return maxSection.link;
    }

    return '';
  }, [container]);
  var handleScrollTo = (0, _react.useCallback)(function (link) {
    if (animatingRef.current) return;
    setCurrentActiveLink(link);
    var scrollTop = (0, _utils.getScroll)(container, true);
    var sharpLinkMatch = sharpMatcherRegx.exec(link);

    if (!sharpLinkMatch) {
      return;
    }

    var targetElement = document.getElementById(sharpLinkMatch[1]);

    if (!targetElement) {
      return;
    }

    var eleOffsetTop = (0, _utils.getOffsetTop)(targetElement, container);
    var y = scrollTop + eleOffsetTop;
    y -= offsetTop;
    animatingRef.current = true;
    (0, _utils.scrollTo)(y, {
      callback: function callback() {
        animatingRef.current = false;
      },
      duration: duration,
      container: container
    });
  }, [container, duration, offsetTop, setCurrentActiveLink]);
  var handleScroll = (0, _react.useCallback)(function () {
    if (animatingRef.current) {
      return;
    }

    var currentActiveLink = getCurrentAnchor(offsetTop, bounds);
    setCurrentActiveLink(currentActiveLink);
  }, [bounds, getCurrentAnchor, offsetTop, setCurrentActiveLink]);
  var updateIndicator = (0, _react.useCallback)(function (top, height) {
    setIndicatorTop(top);
    setIndicatorHeight(height);
  }, []);
  (0, _react.useEffect)(function () {
    container.addEventListener('scroll', handleScroll);
    return function () {
      container.removeEventListener('scroll', handleScroll);
    };
  }, [container, handleScroll]);
  (0, _react.useEffect)(function () {
    handleScroll();
    var hash = decodeURIComponent(location.hash);

    if (hash && linksRef.current.indexOf(hash) > 0) {
      handleScrollTo(hash);
    }
  }, [container]); // eslint-disable-line

  return {
    activeLink: activeLink,
    disableHash: disableHash,
    direction: direction,
    indicatorHeight: indicatorHeight,
    indicatorTop: indicatorTop,
    registerLink: registerLink,
    unregisterLink: unregisterLink,
    getContainer: getContainer,
    setCurrentActiveLink: setCurrentActiveLink,
    getCurrentAnchor: getCurrentAnchor,
    handleScrollTo: handleScrollTo,
    handleScroll: handleScroll,
    updateIndicator: updateIndicator
  };
}