"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _reflow = _interopRequireDefault(require("../utils/reflow"));

var _withAnimation = require("./withAnimation");

function getScale(value) {
  return "scale(".concat(value, ", ").concat(Math.pow(value, 2), ")");
}

var beforeEnter = function beforeEnter(node, props) {
  var transitionPattern = props.theme.transition.pattern,
      timeout = props.timeout,
      easing = props.easing;
  (0, _reflow["default"])(node);
  node.style.transition = "all ".concat(timeout, "ms ").concat(easing || transitionPattern.easing.enter);
  node.style.webkitTransition = "all ".concat(timeout, "ms ").concat(easing || transitionPattern.easing.enter);
};

var beforeExit = function beforeExit(node, props) {
  var transitionPattern = props.theme.transition.pattern,
      timeout = props.timeout,
      easing = props.easing;
  node.style.opacity = '0';
  node.style.transform = getScale(0.75);
  node.style.webkitTransform = getScale(0.75);
  node.style.transition = "all ".concat(timeout, "ms ").concat(easing || transitionPattern.easing.leave);
  node.style.webkitTransition = "all ".concat(timeout, "ms ").concat(easing || transitionPattern.easing.leave);
};

function makeStyles(state) {
  var defaultStyle = {
    opacity: 0,
    transform: getScale(0.75)
  };
  var transitionStyles = {
    entering: {
      opacity: 1,
      transform: getScale(1)
    },
    entered: {
      opacity: 1,
      transform: 'none'
    },
    exited: {
      visibility: 'hidden'
    }
  };
  return (0, _extends2["default"])({}, defaultStyle, transitionStyles[state]);
}

var _default = (0, _withAnimation.withAnimation)(makeStyles, {
  beforeEnter: beforeEnter,
  beforeExit: beforeExit
});

exports["default"] = _default;