"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.ButtonGroup = void 0;
exports.getOffBorder = getOffBorder;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _react = _interopRequireDefault(require("react"));

var _styledComponents = _interopRequireWildcard(require("styled-components"));

var _useTheme = _interopRequireDefault(require("../utils/useTheme"));

var _withTheme = require("../utils/withTheme");

var _mixins = require("./mixins");

var _excluded = ["type", "size", "htmlType", "disabled", "busy", "shape", "block", "children", "plain", "outline", "lightShadow"],
    _excluded2 = ["disabled", "busy", "shape", "size", "block", "type", "plain", "outline", "lightShadow", "offBorder"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var BaseNode = _styledComponents["default"].div.withConfig({
  displayName: "BaseNode"
})([""]);

/* eslint-disable complexity */

/**
 * 在 Group 中需要根据 z-index 来决定谁的 border 在上面
 * @param childIndex
 * @param configMap
 * @param disabled
 */
function getOffBorder(childIndex, configMap, disabled) {
  var _configMap$childIndex = configMap[childIndex],
      groupType = _configMap$childIndex.groupType,
      zIndex = _configMap$childIndex.zIndex;
  var offBorder;
  var prevItem = configMap[childIndex - 1];
  var nextItem = configMap[childIndex + 1];

  if (disabled && (groupType === 'head' || groupType === 'group')) {
    offBorder = 'right';
  } else if (disabled && groupType === 'tail') {
    offBorder = 'no';
  } // 是第一个的时候，有两种情况
  else if (groupType === 'head' && nextItem && nextItem.zIndex < zIndex) {
    offBorder = 'no';
  } else if (groupType === 'head') {
    offBorder = 'right';
  } // 是最后的一个的时候，也有两种情况
  else if (groupType === 'tail' && prevItem && prevItem.zIndex > zIndex) {
    offBorder = 'left';
  } else if (groupType === 'tail') {
    offBorder = 'no';
  } // 在中间的时候，有四种情况
  else if (groupType === 'group' && nextItem.zIndex > zIndex && prevItem.zIndex > zIndex) {
    offBorder = 'both';
  } else if (groupType === 'group' && nextItem.zIndex < zIndex && prevItem.zIndex < zIndex) {
    offBorder = 'no';
  } else if (groupType === 'group' && nextItem.zIndex < zIndex && prevItem.zIndex > zIndex) {
    offBorder = 'left';
  } else {
    offBorder = 'right';
  }

  return offBorder;
}
/* eslint-enable complexity */


var ButtonGroupWithoutStyle = /*#__PURE__*/_react["default"].memo( /*#__PURE__*/_react["default"].forwardRef(function (props, ref) {
  var type = props.type,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      htmlType = props.htmlType,
      disabled = props.disabled,
      busy = props.busy,
      shape = props.shape,
      block = props.block,
      children = props.children,
      plain = props.plain,
      outline = props.outline,
      lightShadow = props.lightShadow,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var theme = (0, _useTheme["default"])();

  var childrenLen = _react["default"].Children.count(children);

  var notOnlyOne = childrenLen > 1;
  var resultChildren;
  var _theme$components$But = theme.components.Button,
      zIndexOrder = _theme$components$But.zIndexOrder,
      plainIsLight = _theme$components$But.plainIsLight;

  if (children) {
    var configMap = {};

    _react["default"].Children.forEach(children, function (child, childIndex) {
      if (! /*#__PURE__*/_react["default"].isValidElement(child)) {
        return;
      }

      var currentType = child.props.type || type;
      var groupType;

      if (notOnlyOne && !childIndex) {
        groupType = 'head';
      } else if (notOnlyOne && childIndex === childrenLen - 1) {
        groupType = 'tail';
      } else if (notOnlyOne) {
        groupType = 'group';
      }

      configMap[childIndex] = {
        groupType: groupType,
        zIndex: zIndexOrder[currentType] || 0
      };
    });

    resultChildren = _react["default"].Children.map(children, function (child, childIndex) {
      if (! /*#__PURE__*/_react["default"].isValidElement(child)) {
        return null;
      }

      var _child$props = child.props,
          childDisabled = _child$props.disabled,
          childBusy = _child$props.busy,
          childShape = _child$props.shape,
          childSize = _child$props.size,
          childBlock = _child$props.block,
          childType = _child$props.type,
          childPlain = _child$props.plain,
          childOutline = _child$props.outline,
          childLightShadow = _child$props.lightShadow,
          childOffBorder = _child$props.offBorder,
          otherChildProps = (0, _objectWithoutProperties2["default"])(_child$props, _excluded2); // 布尔值需要注意一下

      var currentDisabled = childDisabled !== undefined ? childDisabled : disabled;
      var offBorder = childOffBorder || getOffBorder(childIndex, configMap, currentDisabled);
      var currentType = childType || type;
      var groupType = configMap[childIndex].groupType;
      var finalPlain = typeof childPlain !== 'undefined' ? childPlain : plain;
      var finalOutline = typeof childOutline !== 'undefined' ? childOutline : outline;
      var finalLightShadow = typeof childLightShadow !== 'undefined' ? childLightShadow : lightShadow;

      if (typeof finalOutline === 'undefined' && finalPlain && !plainIsLight) {
        finalOutline = true;
      }

      if (typeof finalLightShadow === 'undefined' && finalPlain && plainIsLight) {
        finalLightShadow = true;
      }

      return /*#__PURE__*/_react["default"].cloneElement(child, (0, _extends2["default"])({}, otherChildProps, {
        disabled: currentDisabled,
        busy: typeof childBusy !== 'undefined' ? childBusy : busy,
        shape: childShape || shape,
        outline: finalOutline,
        lightShadow: finalLightShadow,
        type: currentType,
        offBorder: offBorder,
        groupType: groupType,
        size: size,
        block: block
      }));
    });
  }

  return /*#__PURE__*/_react["default"].createElement(BaseNode, (0, _extends2["default"])({
    ref: ref
  }, otherProps), resultChildren);
}));

var StyledButtonGroup = (0, _styledComponents["default"])(ButtonGroupWithoutStyle).withConfig({
  displayName: "StyledButtonGroup"
})(["", ""], function (props) {
  var _props$size2 = props.size,
      size = _props$size2 === void 0 ? 'm' : _props$size2,
      block = props.block,
      theme = props.theme,
      _props$disableSibling = props.disableSiblingMargin,
      disableSiblingMargin = _props$disableSibling === void 0 ? false : _props$disableSibling;
  return (0, _styledComponents.css)(["position:relative;display:inline-flex;", " ", ""], block && 'width: 100%;', (0, _mixins.siblingCss)({
    size: size,
    theme: theme,
    inGroup: false,
    disableSiblingMargin: disableSiblingMargin
  }));
});
var ButtonGroup = (0, _withTheme.withThemeForStyled)(StyledButtonGroup);
exports.ButtonGroup = ButtonGroup;
var _default = ButtonGroup;
exports["default"] = _default;