"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buttonSizeCSS = void 0;
exports.highlightNeutralColorButtonType = highlightNeutralColorButtonType;
exports.siblingCss = siblingCss;

var _styledComponents = require("styled-components");

var _addPx = _interopRequireDefault(require("../utils/addPx"));

/**
 * 需要高亮 hover 态的中性色按钮
 */
function highlightNeutralColorButtonType(type) {
  return type === 'normal' || type === 'strong' || type === 'secondary';
}

function siblingCss(props) {
  var size = props.size,
      theme = props.theme,
      inGroup = props.inGroup,
      disableSiblingMargin = props.disableSiblingMargin;
  var margin = theme.components.Button.siblingSpacing[size];

  if (margin > 0 && !inGroup && !disableSiblingMargin) {
    return (0, _styledComponents.css)(["& + &{margin-left:", "px;}"], theme.components.Button.siblingSpacing[size]);
  }

  return '';
}

var buttonSizeCSS = function buttonSizeCSS(props) {
  var _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      theme = props.theme,
      propShape = props.shape,
      block = props.block,
      groupType = props.groupType,
      propFontWeight = props.fontWeight,
      width = props.width,
      outline = props.outline;
  var sizeSpec = theme.size.spec,
      components = theme.components;
  var btnToken = components.Button;
  var height = sizeSpec.height[size];
  var padding = btnToken.padding[size];
  var fontWeight = btnToken.fontWeight[propFontWeight || 'normal'];
  var buttonShape = propShape || btnToken.defaultShape;
  var borderRadiusCSS;

  if (groupType === 'group') {
    borderRadiusCSS = (0, _styledComponents.css)(["border-radius:", ";margin-left:-1px;"], btnToken.borderRadius.group);
  } else if (buttonShape === 'circle') {
    borderRadiusCSS = (0, _styledComponents.css)(["border-radius:", ";"], btnToken.borderRadius.circle);
  } else if (buttonShape === 'round') {
    borderRadiusCSS = (0, _styledComponents.css)(["border-radius:", ";"], btnToken.borderRadius.round);
  } else {
    borderRadiusCSS = (0, _styledComponents.css)(["border-radius:", ";"], btnToken.normalBorderRadius ? btnToken.normalBorderRadius[size] : btnToken.borderRadius.normal);
  }

  var innerLeftPadding = padding;
  var innerRightPadding = padding; // 有border的时候为了间距对齐，处理一下 padding

  if (outline || groupType === 'group') {
    innerLeftPadding = padding - 1;
    innerRightPadding = padding - 1;
  } else if (groupType === 'head') {
    innerRightPadding = padding - 1;
  } else if (groupType === 'tail') {
    innerLeftPadding = padding - 1;
  }

  var innerWidth;

  if (buttonShape === 'square' || buttonShape === 'circle') {
    innerWidth = "".concat(height, "px");
  } else if (block) {
    innerWidth = '100%';
  } else if (width) {
    innerWidth = (0, _addPx["default"])(width);
  }

  return (0, _styledComponents.css)(["height:", "px;padding:0 ", "px 0 ", "px;font-weight:", ";font-size:", "px;line-height:", "px;", " ", ""], height, innerRightPadding, innerLeftPadding, fontWeight, btnToken.fontSize[size], btnToken.lineHeight[size], innerWidth && (0, _styledComponents.css)(["width:", ";"], innerWidth), borderRadiusCSS);
};

exports.buttonSizeCSS = buttonSizeCSS;