"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _react = _interopRequireWildcard(require("react"));

var _styledComponents = _interopRequireWildcard(require("styled-components"));

var _memoForwardRef = _interopRequireDefault(require("../utils/memoForwardRef"));

var _useStyles = _interopRequireDefault(require("../utils/useStyles"));

var _useTheme = _interopRequireDefault(require("../utils/useTheme"));

var _utils = require("./utils");

var _excluded = ["status", "date", "viewType", "rowType", "children", "isCurrent", "disabled", "readOnly", "isLabel", "styles", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "onItemClick", "onItemHover", "onClick", "onMouseEnter"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var ItemBlock = _styledComponents["default"].div.withConfig({
  displayName: "ItemBlock"
})(["position:relative;height:100%;"]);

var InnerItem = _styledComponents["default"].div.withConfig({
  displayName: "InnerItem"
})(["display:flex;align-items:center;justify-content:center;height:100%;width:100%;"]);

// 根据在一行中的位置和是否是区间选中来判断左右使用的是 padding 还是 margin
function getGutterCss(rowType, status, innerGutter) {
  var cssObj = {};

  if (rowType === 'head' || status === 'range-start') {
    cssObj.marginLeft = innerGutter;
  } else {
    cssObj.paddingLeft = innerGutter;
  }

  if (rowType === 'tail' || status === 'range-end') {
    cssObj.marginRight = innerGutter;
  } else {
    cssObj.paddingRight = innerGutter;
  }

  return cssObj;
}
/**
 * 获取 item 的 color
 * @param status 状态
 * @param isCurrent 是否是今天
 * @param $disabled 是否禁用
 * @param theme 主题
 */


function getItemColor(status, isCurrent, $disabled, theme) {
  var itemToken = theme.components.Calendar.item;

  if (status === 'selected' || status === 'range-end' || status === 'range-start') {
    return itemToken.color.selected;
  }

  if (status === 'range' || status === 'range-hovered') {
    return itemToken.color.current;
  }

  if ($disabled) {
    return itemToken.color.disabled;
  }

  if (status === 'outside') {
    return itemToken.color.outside;
  }

  if (status === 'normal' && isCurrent) {
    return itemToken.color.current;
  }

  return itemToken.color.normal;
}

function getItemCss(props) {
  var _props$$viewType = props.$viewType,
      $viewType = _props$$viewType === void 0 ? 'month' : _props$$viewType,
      $disabled = props.$disabled,
      theme = props.theme,
      _props$$status = props.$status,
      $status = _props$$status === void 0 ? 'normal' : _props$$status,
      $label = props.$label,
      $rowType = props.$rowType,
      $readOnly = props.$readOnly;
  var itemToken = theme.components.Calendar.item;
  var innerGutter = $viewType === 'month' ? itemToken.gutterInMonth : itemToken.gutterOther;
  var isInRange = $status === 'range-end' || $status === 'range-start' || $status === 'range';
  var blockCssObj = {};
  var innerItemCssObj = {};
  var innerItemActiveCssObj = {};
  var innerItemHoverCssObj = {};
  var valid = !$disabled && !$readOnly;

  if ($status === 'normal' && valid && !$label) {
    innerItemHoverCssObj.backgroundColor = itemToken.background.hover;
    innerItemActiveCssObj.backgroundColor = itemToken.background.click;
    innerItemActiveCssObj.color = itemToken.color.current;
  } else if ($status === 'selected' || $status === 'range-end' || $status === 'range-start') {
    innerItemCssObj.backgroundColor = itemToken.background.selected;

    if (valid) {
      innerItemActiveCssObj.backgroundColor = itemToken.background.selectedClick;
    }
  } else if ($status === 'hovered') {
    blockCssObj.backgroundColor = itemToken.background.hover;
  } else if ($status === 'range-hovered') {
    blockCssObj.backgroundColor = itemToken.background.rangeClick;
  } else if ($status === 'range' && valid) {
    innerItemHoverCssObj.backgroundColor = itemToken.background.range;
    innerItemActiveCssObj.backgroundColor = itemToken.background.rangeClick;
  }

  if (isInRange) {
    blockCssObj.backgroundColor = itemToken.background.range;
  }

  if (valid && isInRange) {
    innerItemCssObj.opacity = theme.opacity.pattern.disabled;
  }

  return (0, _styledComponents.css)(["& ", "{", " ", "}& ", "{border-radius:", ";", " &:hover{", "}&:active{", "}}"], ItemBlock, getGutterCss($rowType, $status, innerGutter), blockCssObj, InnerItem, itemToken.borderRadius, innerItemCssObj, innerItemHoverCssObj, innerItemActiveCssObj);
}

var Root = _styledComponents["default"].div.withConfig({
  displayName: "Root"
})(["", ""], function (props) {
  var theme = props.theme,
      _props$$status2 = props.$status,
      $status = _props$$status2 === void 0 ? 'normal' : _props$$status2,
      $current = props.$current,
      $label = props.$label,
      $hide = props.$hide,
      $disabled = props.$disabled,
      $readOnly = props.$readOnly;
  var cssObj = {
    color: getItemColor($status, $current, $disabled, theme)
  };
  var itemToken = theme.components.Calendar.item;

  if ($disabled) {
    cssObj.cursor = 'not-allowed';
  } else if ($readOnly) {
    cssObj.cursor = 'default';
  } else if (!$label) {
    cssObj.cursor = 'pointer';
  }

  if ($hide) {
    cssObj.visibility = 'hidden';
  }

  if ($current && ($status === 'normal' || $status === 'range') && !$disabled && !$readOnly) {
    cssObj.fontWeight = theme.typography.spec.fontWeight.semibold;
  }

  return (0, _styledComponents.css)(["user-select:none;box-sizing:border-box;position:relative;height:", "px;font-size:", "px;line-height:", "px;", " ", ""], itemToken.height, itemToken.fontSize, itemToken.fontSize, cssObj, getItemCss(props));
});

var defaultRenderMonthLabel = function defaultRenderMonthLabel(date) {
  return date.format('dd');
};

var defaultRenderMonthItem = function defaultRenderMonthItem(date) {
  return date.format('D');
};

var defaultRenderYearItem = function defaultRenderYearItem(date) {
  return date.format('MMM');
};

var defaultRenderDecadeItem = function defaultRenderDecadeItem(date) {
  return date.format('YYYY');
};

var defaultStyles = {
  panelItemWrapper: '',
  panelItemContent: ''
};
var CalendarItem = (0, _memoForwardRef["default"])(function (props, ref) {
  var _props$status = props.status,
      status = _props$status === void 0 ? 'normal' : _props$status,
      date = props.date,
      viewType = props.viewType,
      rowType = props.rowType,
      children = props.children,
      isCurrent = props.isCurrent,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      _props$readOnly = props.readOnly,
      readOnly = _props$readOnly === void 0 ? false : _props$readOnly,
      isLabel = props.isLabel,
      styles = props.styles,
      hideItemOutside = props.hideItemOutside,
      _props$renderDecadeIt = props.renderDecadeItem,
      renderDecadeItem = _props$renderDecadeIt === void 0 ? defaultRenderDecadeItem : _props$renderDecadeIt,
      _props$renderMonthIte = props.renderMonthItem,
      renderMonthItem = _props$renderMonthIte === void 0 ? defaultRenderMonthItem : _props$renderMonthIte,
      _props$renderYearItem = props.renderYearItem,
      renderYearItem = _props$renderYearItem === void 0 ? defaultRenderYearItem : _props$renderYearItem,
      _props$renderMonthLab = props.renderMonthLabel,
      renderMonthLabel = _props$renderMonthLab === void 0 ? defaultRenderMonthLabel : _props$renderMonthLab,
      onItemClick = props.onItemClick,
      onItemHover = props.onItemHover,
      onClick = props.onClick,
      onMouseEnter = props.onMouseEnter,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var theme = (0, _useTheme["default"])();
  var pickStyles = (0, _react.useMemo)(function () {
    if (styles) {
      var panelItemWrapper = styles.panelItemWrapper,
          panelItemContent = styles.panelItemContent;
      return {
        panelItemWrapper: panelItemWrapper,
        panelItemContent: panelItemContent
      };
    }
  }, [styles]);
  var innerStyles = (0, _useStyles["default"])('calendar', defaultStyles, pickStyles);
  var hideItem = status === 'outside' && viewType === 'month' && hideItemOutside === 'month';
  var innerStatus = (0, _utils.transformItemStatus)(status, disabled);

  var handleClick = function handleClick() {
    if (disabled || readOnly) {
      return;
    }

    onItemClick && onItemClick(date, innerStatus);
  };

  var handleEnter = function handleEnter() {
    if (disabled || readOnly) {
      return;
    }

    onItemHover && onItemHover(date, innerStatus);
  };

  var innerNode;

  if (isLabel) {
    innerNode = renderMonthLabel(date);
  } else if (viewType === 'month') {
    innerNode = renderMonthItem(date, innerStatus);
  } else if (viewType === 'year') {
    innerNode = renderYearItem(date, innerStatus);
  } else if (viewType === 'decade') {
    innerNode = renderDecadeItem(date, innerStatus);
  }

  return /*#__PURE__*/_react["default"].createElement(Root, (0, _extends2["default"])({}, otherProps, {
    $disabled: disabled,
    $readOnly: readOnly,
    theme: theme,
    $viewType: viewType,
    $rowType: rowType,
    $status: status,
    $current: !!isCurrent,
    $label: !!isLabel,
    $hide: hideItem,
    onClick: handleClick,
    onMouseEnter: handleEnter,
    ref: ref
  }), /*#__PURE__*/_react["default"].createElement(ItemBlock, innerStyles.panelItemWrapper, /*#__PURE__*/_react["default"].createElement(InnerItem, innerStyles.panelItemContent, innerNode)));
});
var _default = CalendarItem;
exports["default"] = _default;