"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _react = _interopRequireWildcard(require("react"));

var _styledComponents = _interopRequireWildcard(require("styled-components"));

var _memoForwardRef = _interopRequireDefault(require("../utils/memoForwardRef"));

var _useStyles = _interopRequireDefault(require("../utils/useStyles"));

var _useTheme = _interopRequireDefault(require("../utils/useTheme"));

var _CalendarItem = _interopRequireDefault(require("./CalendarItem"));

var _calendarPanelUtils = require("./calendarPanelUtils");

var _utils = require("./utils");

var _excluded = ["viewType", "viewDate", "selected", "range", "start", "end", "isRange", "disableRange", "disableDate", "onItemClick", "onItemHover", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "todayDate", "styles", "min", "max", "fixedStart", "fixedEnd", "readOnly", "panelBlockProps", "isWeek", "hoveredWeek", "selectedWeek", "disableWeek"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var Container = _styledComponents["default"].div.withConfig({
  displayName: "Container"
})(["display:flex;flex-wrap:wrap;position:relative;"]);

var StyledItem = (0, _styledComponents["default"])(_CalendarItem["default"]).withConfig({
  displayName: "StyledItem"
})([""]);

var StyledItemM = /*#__PURE__*/_react["default"].memo(StyledItem);

var defaultStyles = {
  panelMonthBlock: '',
  panelYearBlock: '',
  panelDecadeBlock: '',
  panelItem: '',
  panelLabelItem: '',
  panelLabelBlock: ''
};

var Root = _styledComponents["default"].div.withConfig({
  displayName: "Root"
})(["width:100%;", ""], function (props) {
  var viewType = props.$type,
      theme = props.theme;
  var panelToken = theme.components.Calendar.panel;
  var typeCss;

  if (viewType === 'month') {
    typeCss = (0, _styledComponents.css)(["padding:", ";& ", "{padding:", ";}& ", "{width:", "%;margin-bottom:", "px;}"], panelToken.containerMonthPadding, Container, panelToken.monthPadding, StyledItem, 1 / 7 * 100, panelToken.monthItemMarginBottom);
  } else {
    typeCss = (0, _styledComponents.css)(["padding:", ";& ", "{margin:", ";}& ", "{width:", "%;margin-bottom:", "px;}"], panelToken.containerOtherPadding, Container, panelToken.otherMargin, StyledItem, 1 / 3 * 100, panelToken.otherItemMarginBottom);
  }

  return (0, _styledComponents.css)(["", ""], typeCss);
});

var CalendarPanel = (0, _memoForwardRef["default"])(function (props, ref) {
  var viewType = props.viewType,
      viewDate = props.viewDate,
      selected = props.selected,
      range = props.range,
      start = props.start,
      end = props.end,
      isRange = props.isRange,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      onItemClick = props.onItemClick,
      onItemHover = props.onItemHover,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      todayDate = props.todayDate,
      styles = props.styles,
      min = props.min,
      max = props.max,
      fixedStart = props.fixedStart,
      fixedEnd = props.fixedEnd,
      readOnly = props.readOnly,
      panelBlockProps = props.panelBlockProps,
      _props$isWeek = props.isWeek,
      isWeek = _props$isWeek === void 0 ? false : _props$isWeek,
      hoveredWeek = props.hoveredWeek,
      selectedWeek = props.selectedWeek,
      disableWeek = props.disableWeek,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var theme = (0, _useTheme["default"])();
  var pickStyles = (0, _react.useMemo)(function () {
    if (styles) {
      var panelMonthBlock = styles.panelMonthBlock,
          panelYearBlock = styles.panelYearBlock,
          panelDecadeBlock = styles.panelDecadeBlock,
          panelItem = styles.panelItem,
          panelLabelItem = styles.panelLabelItem,
          panelLabelBlock = styles.panelLabelBlock;
      return {
        panelMonthBlock: panelMonthBlock,
        panelYearBlock: panelYearBlock,
        panelDecadeBlock: panelDecadeBlock,
        panelItem: panelItem,
        panelLabelItem: panelLabelItem,
        panelLabelBlock: panelLabelBlock
      };
    }
  }, [styles]);
  var innerStyles = (0, _useStyles["default"])('calendar', defaultStyles, pickStyles);
  var dates = (0, _react.useMemo)(function () {
    if (viewType === 'month') {
      return (0, _utils.getDaysOfMonth)(viewDate);
    }

    if (viewType === 'year') {
      return (0, _utils.getMonthsOfYear)(viewDate);
    }

    return (0, _utils.getDecadeYears)(viewDate);
  }, [viewType, viewDate]); // 标签，只有月视图有

  var labelBlock = (0, _react.useMemo)(function () {
    if (viewType !== 'month') {
      return;
    }

    var weekLabels = dates.slice(0, 7);
    return /*#__PURE__*/_react["default"].createElement(Container, innerStyles.panelLabelBlock, weekLabels.map(function (labelDate) {
      return /*#__PURE__*/_react["default"].createElement(StyledItemM, (0, _extends2["default"])({}, innerStyles.panelLabelItem, {
        styles: styles,
        renderMonthLabel: renderMonthLabel,
        date: labelDate,
        isLabel: true,
        viewType: "month",
        key: labelDate.unix()
      }));
    }));
  }, [dates, innerStyles.panelLabelBlock, innerStyles.panelLabelItem, renderMonthLabel, styles, viewType]);
  var blockStyleItem = innerStyles.panelMonthBlock;

  if (viewType === 'year') {
    blockStyleItem = innerStyles.panelYearBlock;
  } else if (viewType === 'decade') {
    blockStyleItem = innerStyles.panelYearBlock;
  }

  var dateItems = (0, _react.useMemo)(function () {
    var getStatus = (0, _calendarPanelUtils.createGetStatus)({
      start: start,
      end: end,
      range: range,
      selected: selected,
      viewType: viewType,
      isRange: isRange,
      viewDate: viewDate,
      // week
      isWeek: isWeek,
      hoveredWeek: hoveredWeek,
      selectedWeek: selectedWeek
    });
    var checkDisabled = (0, _calendarPanelUtils.createCheckDisabled)({
      disableRange: disableRange,
      disableDate: disableDate,
      viewType: viewType,
      min: min,
      max: max,
      start: start,
      end: end,
      fixedStart: fixedStart,
      fixedEnd: fixedEnd,
      isRange: isRange,
      // week
      isWeek: isWeek,
      disableWeek: disableWeek
    });
    var rowNum = viewType === 'month' ? 7 : 3;
    var dateType = (0, _utils.getDateType)(viewType);
    var itemPropsList = dates.map(function (date, dateIndex) {
      var status = getStatus(date, dateIndex);
      var disabled = checkDisabled(date);
      var rowType = 'normal';
      var isCurrent = date.isSame(todayDate, dateType);
      var remain = dateIndex % rowNum;

      if (remain === 0) {
        rowType = 'head';
      } else if (remain === rowNum - 1) {
        rowType = 'tail';
      }

      return {
        hideItemOutside: hideItemOutside,
        renderDecadeItem: renderDecadeItem,
        renderMonthItem: renderMonthItem,
        renderYearItem: renderYearItem,
        onItemClick: onItemClick,
        onItemHover: onItemHover,
        status: status,
        disabled: disabled,
        readOnly: viewType === 'month' && readOnly,
        rowType: rowType,
        date: date,
        isCurrent: isCurrent,
        styles: styles
      };
    });

    if (isRange) {
      (0, _calendarPanelUtils.setOnlyStart)(itemPropsList);
    }

    return itemPropsList.map(function (dateItem) {
      return /*#__PURE__*/_react["default"].createElement(StyledItemM, (0, _extends2["default"])({}, innerStyles.panelItem, dateItem, {
        viewType: viewType,
        key: dateItem.date.unix()
      }));
    });
  }, [dates, disableDate, disableRange, disableWeek, end, fixedEnd, fixedStart, hideItemOutside, hoveredWeek, innerStyles.panelItem, isRange, isWeek, max, min, onItemClick, onItemHover, range, readOnly, renderDecadeItem, renderMonthItem, renderYearItem, selected, selectedWeek, start, styles, todayDate, viewDate, viewType]);
  return /*#__PURE__*/_react["default"].createElement(Root, (0, _extends2["default"])({
    theme: theme,
    $type: viewType
  }, otherProps, {
    ref: ref
  }), labelBlock, /*#__PURE__*/_react["default"].createElement(Container, (0, _extends2["default"])({}, blockStyleItem, panelBlockProps), dateItems));
});
var _default = CalendarPanel;
exports["default"] = _default;