"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useMultiSelectCalendar;

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _isEqual2 = _interopRequireDefault(require("lodash/isEqual"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var _Locale = require("../Locale");

var _useCalendarBase = _interopRequireDefault(require("./useCalendarBase"));

var _utils = require("./utils");

var _excluded = ["selectType", "allowClear", "selectedDate", "todayDate", "viewType", "viewDate", "defaultDate", "disableRange", "disableDate", "options", "onUIChange", "onChange", "onSelect", "styles", "defaultViewDate", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "readOnly"];

function useMultiSelectCalendar(props) {
  var baseData = (0, _useCalendarBase["default"])(props);
  var _props$selectType = props.selectType,
      selectType = _props$selectType === void 0 ? 'date' : _props$selectType,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      selectedDate = props.selectedDate,
      todayDate = props.todayDate,
      viewType = props.viewType,
      viewDate = props.viewDate,
      defaultDate = props.defaultDate,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      onUIChange = props.onUIChange,
      onChange = props.onChange,
      onSelect = props.onSelect,
      styles = props.styles,
      defaultViewDate = props.defaultViewDate,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      readOnly = props.readOnly,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var isControlled = ('selectedDate' in props);
  var defaultSelectedDate;

  if (selectedDate) {
    defaultSelectedDate = selectedDate.map(function (d) {
      return (0, _dayjs["default"])(d);
    });
  } else if (defaultDate) {
    defaultSelectedDate = defaultDate.map(function (d) {
      return (0, _dayjs["default"])(d);
    });
  }

  var selectTypeRef = (0, _react.useRef)(selectType);
  var defaultTransformViewType = (0, _utils.transformType)(selectType);
  var viewDateFromProps = (0, _react.useMemo)(function () {
    var inputViewDate = viewDate || selectedDate && selectedDate[0] || defaultDate && defaultDate[0] || defaultViewDate;

    if (inputViewDate) {
      return (0, _dayjs["default"])(inputViewDate);
    }

    return baseData.finalTodayDate;
  }, [baseData.finalTodayDate, defaultDate, defaultViewDate, selectedDate, viewDate]);
  var uiStateFromProps = {
    viewDate: viewDateFromProps,
    viewType: viewType || defaultTransformViewType
  };

  var _useState = (0, _react.useState)(uiStateFromProps),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      uiState = _useState2[0],
      setUIState = _useState2[1];

  var _useState3 = (0, _react.useState)(defaultSelectedDate || []),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      stateSelectedDate = _useState4[0],
      setSelectedDate = _useState4[1];

  var finalUIState = (0, _extends2["default"])({}, uiState);

  if ('viewDate' in props) {
    finalUIState.viewDate = uiStateFromProps.viewDate;
  }

  if ('viewType' in props) {
    finalUIState.viewType = uiStateFromProps.viewType;
  }

  if (selectTypeRef.current !== selectType) {
    selectTypeRef.current = selectType;
    setUIState(uiStateFromProps);
    setSelectedDate([]);
  }

  var updateUI = function updateUI(inputState) {
    if (!inputState.viewDate && !inputState.viewType) {
      return;
    }

    var newState = (0, _extends2["default"])({}, finalUIState, inputState);
    setUIState(newState);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var updateUIBySelectedDate = function updateUIBySelectedDate(date) {
    var backToDefaultViewType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    var newUIState = {
      viewDate: date,
      viewType: finalUIState.viewType
    };

    if (backToDefaultViewType && finalUIState.viewType !== defaultTransformViewType) {
      newUIState.viewType = defaultTransformViewType;
    }

    updateUI(newUIState);
  };

  var updateSelectedDate = function updateSelectedDate(date, changedDate) {
    var syncView = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var backToDefaultViewType = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    setSelectedDate(date);

    if (onChange) {
      onChange(date);
    }

    if (onSelect && changedDate) {
      onSelect(changedDate);
    }

    if (syncView && changedDate) {
      updateUIBySelectedDate(changedDate, backToDefaultViewType);
    }
  };

  var handleItemClick = (0, _useEventCallback2.useEventCallback)(function (date, status) {
    if (finalUIState.viewType === 'year' && selectType !== 'month') {
      updateUI({
        viewType: 'month',
        viewDate: finalUIState.viewDate.set('month', date.get('month'))
      });
      return;
    }

    if (finalUIState.viewType === 'decade' && selectType !== 'year') {
      updateUI({
        viewType: selectType === 'date' ? 'month' : 'year',
        viewDate: finalUIState.viewDate.set('year', date.get('year'))
      });
      return;
    }

    if (stateSelectedDate && stateSelectedDate.some(function (d) {
      return d.isSame(date, selectType);
    }) && allowClear) {
      updateSelectedDate(stateSelectedDate.filter(function (d) {
        return !d.isSame(date, selectType);
      }));
    } else {
      updateSelectedDate([].concat((0, _toConsumableArray2["default"])(stateSelectedDate.filter(function (d) {
        return !d.isSame(date, selectType);
      })), [date]), date, status === 'outside');
    }
  });
  var handleOptionClick = (0, _useEventCallback2.useEventCallback)(function (date) {
    // 如果选中的值中已有option的值则不继续添加
    if (stateSelectedDate.some(function (d) {
      return d.isSame(date, selectType);
    })) {
      updateSelectedDate(stateSelectedDate, date, true, true);
      return;
    }

    updateSelectedDate([].concat((0, _toConsumableArray2["default"])(stateSelectedDate), [date]), date, true, true);
  });

  var handleMove = function handleMove(step, type) {
    return function () {
      updateUI({
        viewDate: (0, _utils.moveOne)(finalUIState.viewDate, type, step)
      });
    };
  };

  var handlePrevMonth = (0, _useEventCallback2.useEventCallback)(handleMove(-1, 'month'));
  var handleNextMonth = (0, _useEventCallback2.useEventCallback)(handleMove(1, 'month'));
  var yearBtnType = finalUIState.viewType === 'decade' ? 'decade' : 'year';
  var handlePrevYear = (0, _useEventCallback2.useEventCallback)(handleMove(-1, yearBtnType));
  var handleNextYear = (0, _useEventCallback2.useEventCallback)(handleMove(1, yearBtnType));
  var handleSwitch = (0, _useEventCallback2.useEventCallback)(function (type) {
    updateUI({
      viewType: type
    });
  });
  var locale = (0, _Locale.useLocale)();
  var finalOptions = (0, _react.useMemo)(function () {
    return (0, _utils.formatCalendarOptions)(options, handleOptionClick, function (opt) {
      if (typeof opt.date === 'function') {
        (0, _devWarning2.devWarning)('[Calendar]: option.date() return invalid date.');
      } else {
        (0, _devWarning2.devWarning)('[Calendar]: options passed invalid date.');
      }
    });
  }, [handleOptionClick, options]); // 当国际化变了，得更新内存中 dayjs 对象的国际化

  var updateLocale = function updateLocale() {
    var localeKey = locale.dayjsLocaleKey;
    updateUI({
      viewDate: finalUIState.viewDate.locale(localeKey)
    });

    if (stateSelectedDate.length > 0) {
      updateSelectedDate(stateSelectedDate.map(function (d) {
        return d.locale(localeKey);
      }));
    }
  };

  (0, _react.useEffect)(function () {
    var localeKey = locale.dayjsLocaleKey; // 任一变了就可以了

    var localeChange = localeKey !== finalUIState.viewDate.locale();

    if (!localeChange) {
      return;
    }

    updateLocale();
  });
  (0, _react.useEffect)(function () {
    // 判断传入值是否和当前值相同
    var isDifferent = selectedDate && !(0, _isEqual2["default"])(selectedDate.map(function (d) {
      return (0, _dayjs["default"])(d).format('YYYY-MM-DD');
    }), stateSelectedDate.map(function (d) {
      return (0, _dayjs["default"])(d).format('YYYY-MM-DD');
    }));

    if (isControlled && isDifferent) {
      setSelectedDate(selectedDate ? selectedDate.map(function (d) {
        return (0, _dayjs["default"])(d);
      }) : []);
    }
  }, [isControlled, selectedDate, stateSelectedDate]);
  return (0, _extends2["default"])({}, baseData, {
    otherProps: otherProps,
    handleItemClick: handleItemClick,
    handlePrevMonth: handlePrevMonth,
    handleNextMonth: handleNextMonth,
    handlePrevYear: handlePrevYear,
    handleNextYear: handleNextYear,
    handleSwitch: handleSwitch,
    finalUIState: finalUIState,
    finalSelectedDate: stateSelectedDate,
    finalOptions: finalOptions,
    updateUI: updateUI,
    updateSelectedDate: updateSelectedDate,
    updateUIBySelectedDate: updateUIBySelectedDate,
    readOnly: readOnly,
    // 透传回去的属性
    styles: styles,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    disableDate: disableDate
  });
}