"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useRangeCalendar;
exports.formatRangeState = formatRangeState;
exports.getFixedState = getFixedState;
exports.getStateFromProps = getStateFromProps;
exports.getUIStateFromProps = getUIStateFromProps;
exports.updateRangeViewDate = updateRangeViewDate;
exports.updateRangeViewType = updateRangeViewType;

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var _Locale = require("../Locale");

var _useCalendarBase = _interopRequireDefault(require("./useCalendarBase"));

var _utils = require("./utils");

var _excluded = ["selectType", "options", "defaultRange", "disableRange", "disableDate", "fixedEndDate", "fixedStartDate", "minRangeLength", "maxRangeLength", "allowClear", "defaultViewDate", "editKey", "viewDate", "viewType", "range", "start", "end", "todayDate", "onUIChange", "onChange", "onSelect", "styles", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "readOnly"];

// 从 props 中获取 UI 状态
function getUIStateFromProps(props, fixedStart, fixedEnd) {
  var _props$selectType = props.selectType,
      selectType = _props$selectType === void 0 ? 'date' : _props$selectType,
      defaultRange = props.defaultRange,
      defaultViewDate = props.defaultViewDate,
      viewDate = props.viewDate,
      viewType = props.viewType,
      range = props.range,
      start = props.start,
      end = props.end;
  var innerViewType = (0, _utils.getRangeViewType)(selectType, viewType);
  var argViewDate;
  var argNum = 1;

  if (viewDate) {
    argViewDate = viewDate;
  } else if (start && end) {
    argViewDate = [start, end];
  } else if (start && !end) {
    argViewDate = [start, start];
  } else if (!start && end) {
    argViewDate = [end, end];
    argNum = -1;
  } else if (range && !start && !end) {
    argViewDate = range;
  } else if (defaultRange) {
    argViewDate = defaultRange;
  } else if (fixedStart) {
    argViewDate = [fixedStart, fixedStart];
  } else if (fixedEnd) {
    argViewDate = [fixedEnd, fixedEnd];
    argNum = -1;
  } else if (defaultViewDate) {
    argViewDate = defaultViewDate;
  }

  var innerViewDate = (0, _utils.getRangeViewDate)(innerViewType, argViewDate, argNum);
  return {
    viewType: innerViewType,
    viewDate: innerViewDate
  };
}

function getStateFromProps(props) {
  var start = props.start,
      end = props.end,
      range = props.range,
      defaultRange = props.defaultRange;
  var notControlled = !start && !range && !end;

  if (notControlled && !defaultRange) {
    return {};
  }

  if (notControlled && defaultRange) {
    var newRange = (0, _utils.getRangeInOrder)(defaultRange);

    if (!newRange) {
      return {};
    }

    var _newRange = (0, _slicedToArray2["default"])(newRange, 2),
        newStart = _newRange[0],
        newEnd = _newRange[1];

    return {
      start: newStart,
      end: newEnd,
      range: newRange
    };
  }

  var innerRange = (0, _utils.getRangeInOrder)(range);
  var innerStart;
  var innerEnd;

  if (start) {
    innerStart = (0, _dayjs["default"])(start);
  }

  if (end) {
    innerEnd = (0, _dayjs["default"])(end);
  }

  return {
    range: innerRange,
    start: innerStart,
    end: innerEnd
  };
}

function updateRangeViewType(key, viewType, type) {
  if (key === 1) {
    return [viewType[0], type];
  }

  return [type, viewType[1]];
}

function updateRangeViewDate(key, viewDate, date) {
  if (key === 1) {
    return [viewDate[0], date];
  }

  return [date, viewDate[1]];
}

function getFixedState(state, fixedStart, fixedEnd) {
  var noStartEnd = !state.start && !state.end;

  if (fixedStart && noStartEnd) {
    state.start = fixedStart;
    var innerRangeEnd = state.range && state.range[1] ? state.range[1] : fixedStart;
    state.range = [fixedStart, innerRangeEnd];
    return state;
  }

  if (fixedEnd && noStartEnd) {
    state.end = fixedEnd;
    var innerRangeStart = state.range && state.range[0] ? state.range[0] : fixedEnd;
    state.range = [innerRangeStart, fixedEnd];
    return state;
  }

  return state;
}

function formatRangeState(newState, selectType) {
  var start = newState.start,
      end = newState.end,
      range = newState.range;
  var innerStart = start;
  var innerEnd = end;
  var innerRange = range;

  if (innerStart) {
    var newStart = (0, _utils.formatStartEnd)(innerStart, false, selectType);

    if (!newStart.isSame(innerStart)) {
      innerStart = newStart;

      if (innerRange && innerRange[0]) {
        innerRange[0] = newStart;
      }
    }
  }

  if (innerEnd) {
    var newEnd = (0, _utils.formatStartEnd)(innerEnd, true, selectType);

    if (!newEnd.isSame(innerEnd)) {
      innerEnd = newEnd;

      if (innerRange && innerRange[1]) {
        innerRange[1] = newEnd;
      }
    }
  }

  return {
    start: innerStart,
    end: innerEnd,
    range: innerRange
  };
}

function useRangeCalendar(props) {
  var baseData = (0, _useCalendarBase["default"])(props);
  var _props$selectType2 = props.selectType,
      selectType = _props$selectType2 === void 0 ? 'date' : _props$selectType2,
      options = props.options,
      defaultRange = props.defaultRange,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      fixedEndDate = props.fixedEndDate,
      fixedStartDate = props.fixedStartDate,
      minRangeLength = props.minRangeLength,
      maxRangeLength = props.maxRangeLength,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      defaultViewDate = props.defaultViewDate,
      editKey = props.editKey,
      viewDate = props.viewDate,
      viewType = props.viewType,
      range = props.range,
      start = props.start,
      end = props.end,
      todayDate = props.todayDate,
      onUIChange = props.onUIChange,
      onChange = props.onChange,
      onSelect = props.onSelect,
      styles = props.styles,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      readOnly = props.readOnly,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var isStartControlled = ('start' in props);
  var isEndControlled = ('end' in props);
  var isRangeControlled = ('range' in props);
  var selectTypeRef = (0, _react.useRef)(selectType);
  var defaultTransformViewType = (0, _utils.transformType)(selectType);
  var finalFixedStart = (0, _react.useMemo)(function () {
    return fixedStartDate ? (0, _dayjs["default"])(fixedStartDate) : undefined;
  }, [fixedStartDate]);
  var finalFixedEnd = (0, _react.useMemo)(function () {
    return fixedEndDate ? (0, _dayjs["default"])(fixedEndDate) : undefined;
  }, [fixedEndDate]);
  var uiStateFromProps = (0, _react.useMemo)(function () {
    return getUIStateFromProps({
      selectType: selectType,
      defaultRange: defaultRange,
      defaultViewDate: defaultViewDate,
      viewDate: viewDate,
      viewType: viewType,
      start: start,
      end: end,
      range: range
    }, finalFixedStart, finalFixedEnd);
  }, [defaultRange, defaultViewDate, end, finalFixedEnd, finalFixedStart, range, selectType, start, viewDate, viewType]);
  var stateFromProps = (0, _react.useMemo)(function () {
    return getStateFromProps({
      start: start,
      end: end,
      range: range,
      defaultRange: defaultRange
    });
  }, [defaultRange, end, range, start]);

  var _useState = (0, _react.useState)(stateFromProps),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var _useState3 = (0, _react.useState)(uiStateFromProps),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      uiState = _useState4[0],
      setUIState = _useState4[1];

  var finalUIState = (0, _extends2["default"])({}, uiState);

  if ('viewType' in props) {
    finalUIState.viewType = uiStateFromProps.viewType;
  }

  if ('viewDate' in props) {
    finalUIState.viewDate = uiStateFromProps.viewDate;
  } // 固定某一个选中的时间


  getFixedState(state, finalFixedStart, finalFixedEnd);

  if (selectTypeRef.current !== selectType) {
    selectTypeRef.current = selectType;
    setUIState(uiStateFromProps);
    setState(stateFromProps);
  }

  var updateUI = function updateUI(inputState) {
    if (!inputState.viewDate && !inputState.viewType) {
      return;
    }

    var newState = (0, _extends2["default"])({}, finalUIState, inputState);

    var _newState$viewDate = (0, _slicedToArray2["default"])(newState.viewDate, 2),
        viewLeftDate = _newState$viewDate[0],
        viewRightDate = _newState$viewDate[1];

    var _finalUIState$viewDat = (0, _slicedToArray2["default"])(finalUIState.viewDate, 1),
        oldViewLeftDate = _finalUIState$viewDat[0]; // 目标状态下，两个日历的日期一样了


    var rightIsEarily = !viewLeftDate.isBefore(viewRightDate);
    var leftNotChange = (0, _utils.isSame)(viewLeftDate, oldViewLeftDate, defaultTransformViewType); // 右边更早，同时做成没变，那么移动左边的日历

    if (rightIsEarily && leftNotChange) {
      var newViewLeftDate = (0, _utils.moveOne)(viewRightDate, defaultTransformViewType, -1);
      newState.viewDate = [newViewLeftDate, viewRightDate];
    } else if (rightIsEarily) {
      var newViewRightDate = (0, _utils.moveOne)(viewLeftDate, defaultTransformViewType);
      newState.viewDate = [viewLeftDate, newViewRightDate];
    }

    setUIState(newState);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var updateSelected = function updateSelected(inputState) {
    var mergeState = (0, _extends2["default"])({}, state, inputState);
    var newState = formatRangeState(mergeState, selectType);
    setState(newState);

    if (onChange) {
      onChange(newState);
    }

    if (onSelect && newState.start && newState.end) {
      onSelect([newState.start, newState.end]);
    }

    return newState;
  }; // 在 range datepicker 里用
  // 注意传进来的 range 必须是 valid 也就是合法的 Dayjs 对象


  var selectRange = function selectRange(range) {
    var backToDefaultViewType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    var newRange = (0, _utils.getRangeInOrder)(range);
    updateSelected({
      start: newRange[0],
      range: newRange,
      end: newRange[1]
    });

    var _finalUIState$viewDat2 = (0, _slicedToArray2["default"])(finalUIState.viewDate, 2),
        viewLeftDate = _finalUIState$viewDat2[0],
        viewRightDate = _finalUIState$viewDat2[1];

    var _finalUIState$viewTyp = (0, _slicedToArray2["default"])(finalUIState.viewType, 2),
        leftViewType = _finalUIState$viewTyp[0],
        rightViewType = _finalUIState$viewTyp[1];

    var startIsSame = (0, _utils.isSame)(newRange[0], viewLeftDate, leftViewType);
    var endIsSame = (0, _utils.isSame)(newRange[1], viewRightDate, rightViewType);
    var newUIState = {};

    if (backToDefaultViewType && (leftViewType !== defaultTransformViewType || rightViewType !== defaultTransformViewType)) {
      newUIState.viewType = [defaultTransformViewType, defaultTransformViewType];
    }

    if (!startIsSame || !endIsSame) {
      newUIState.viewDate = (0, _utils.getRangeViewDate)([defaultTransformViewType, defaultTransformViewType], newRange);
    }

    updateUI(newUIState);
  };
  /**
   * 选择区间第一个点
   * @param date 选择的日期
   * @param key 0 | 1 0 表示左边的面板，1 表示右边的面板
   * @param syncView 是否同步面板
   */


  var selectFirstDate = function selectFirstDate(date, key) {
    var syncView = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    var _finalUIState$viewDat3 = (0, _slicedToArray2["default"])(finalUIState.viewDate, 2),
        viewLeftDate = _finalUIState$viewDat3[0],
        viewRightDate = _finalUIState$viewDat3[1];

    var _finalUIState$viewTyp2 = (0, _slicedToArray2["default"])(finalUIState.viewType, 2),
        viewLeftType = _finalUIState$viewTyp2[0],
        viewRightType = _finalUIState$viewTyp2[1];

    var innerStart;
    var innerEnd;
    var innerRange;
    var newViewLeftDate;
    var newViewRightDate;

    if (key === 1) {
      innerEnd = date;
      var rangeStart = minRangeLength ? date.subtract(minRangeLength, (0, _utils.getSelectDateType)(selectType)) : date;
      innerRange = [rangeStart, innerEnd];

      if (rangeStart.isBefore(viewLeftDate)) {
        newViewLeftDate = rangeStart;
      }
    } else {
      innerStart = date;
      var rangeEnd = minRangeLength ? date.add(minRangeLength, (0, _utils.getSelectDateType)(selectType)) : date;
      innerRange = [innerStart, rangeEnd];

      if (rangeEnd.isAfter(viewRightDate)) {
        newViewRightDate = rangeEnd;
      }
    }

    updateSelected({
      start: innerStart,
      range: innerRange,
      end: innerEnd
    });

    if (syncView && key === 1 && !(0, _utils.isSame)(viewRightDate, date, viewRightType)) {
      newViewRightDate = date;
    } else if (syncView && !(0, _utils.isSame)(viewLeftDate, date, viewLeftType)) {
      newViewLeftDate = date;
    }

    if (newViewLeftDate || newViewRightDate) {
      var innerViewLeftDate = newViewLeftDate || viewLeftDate;
      var innerViewRightDate = newViewRightDate || viewRightDate;
      updateUI({
        viewDate: [innerViewLeftDate, innerViewRightDate]
      });
    }
  };

  var clearSelected = function clearSelected() {
    updateSelected({
      start: undefined,
      range: undefined,
      end: undefined
    });
  }; // eslint-disable-next-line complexity


  var clickSelectDate = function clickSelectDate(date, key) {
    var noSelectedDate = !state.end && !state.start;

    if (noSelectedDate) {
      selectFirstDate(date, key);
      return;
    }

    var hasSelected = state.end && state.start;

    if (hasSelected && (0, _utils.inRange)(date, [state.start, state.end]) && allowClear) {
      clearSelected();
      return;
    } // 已有选中的情况下，不再进入初始化选择的程序
    // if (hasSelected && !(finalFixedStart || finalFixedEnd)) {
    //   selectFirstDate(date, key);
    //   return;
    // }


    if (hasSelected && finalFixedStart) {
      updateSelected({
        start: finalFixedStart,
        end: date,
        range: [finalFixedStart, date]
      });
      return;
    }

    if (hasSelected && finalFixedEnd) {
      updateSelected({
        start: date,
        end: finalFixedEnd,
        range: [date, finalFixedEnd]
      });
      return;
    }

    var compareDate = state.start || state.end; // 已有选中的情况下，当前聚焦在哪个框，则选择哪个值

    if (state.start && state.end) {
      if (editKey === 0) {
        // 编辑开始时间， 比结束时间晚，要清空结束时间
        // FIX: minRangeLength 和 maxRangeLength 存在的时候需要做相关判断
        var diffDays = Math.abs(date.diff(state.end, 'day'));
        var needReset = false;

        if (minRangeLength && diffDays < minRangeLength) {
          needReset = true;
        }

        if (maxRangeLength && diffDays > maxRangeLength) {
          needReset = true;
        }

        if (date.isAfter(state.end) || needReset) {
          selectFirstDate(date, editKey);
          return;
        }

        compareDate = state.end;
      } else if (editKey === 1) {
        var _diffDays = Math.abs(date.diff(state.start, 'day'));

        var _needReset = false;

        if (minRangeLength && _diffDays < minRangeLength) {
          _needReset = true;
        }

        if (maxRangeLength && _diffDays > maxRangeLength) {
          _needReset = true;
        } // 编辑结束时间， 比开始时间早，清空开始时间


        if (date.isBefore(state.start) || _needReset) {
          selectFirstDate(date, editKey);
          return;
        }

        compareDate = state.start;
      }
    }

    var range = (0, _utils.getRangeInOrder)([compareDate, date]);
    var start = range[0];
    var end = range[1];
    updateSelected({
      start: start,
      end: end,
      range: range
    });
  };

  var hoverSelectDate = function hoverSelectDate(date) {
    if (state.start && state.end || !state.start && !state.end) {
      return;
    }

    var compareDate = state.start || state.end;
    var range = (0, _utils.getRangeInOrder)([compareDate, date]);
    var start;
    var end; // 这里的逻辑是跟当前 hover 到做比较
    // 如果时间比较早的是数组的第一个，且跟当前 hover 到的是同一个
    // 那么的 end 应该是上次点击选中的

    if (range[0].isSame(date, 'day')) {
      end = range[1];
    } else {
      start = range[0];
    }

    updateSelected({
      range: range,
      start: start,
      end: end
    });
  };

  var handleSwitch = function handleSwitch(key) {
    return function (type) {
      updateUI({
        viewType: updateRangeViewType(key, finalUIState.viewType, type)
      });
    };
  };

  var handleMove = function handleMove(step, cType, key) {
    return function () {
      var newDate = (0, _utils.moveOne)(finalUIState.viewDate[key], cType, step);
      updateUI({
        viewDate: updateRangeViewDate(key, finalUIState.viewDate, newDate)
      });
    };
  };

  var handleItemClick = function handleItemClick(key) {
    return function (date) {
      if (finalUIState.viewType[key] === 'year' && selectType !== 'month') {
        updateUI({
          viewType: updateRangeViewType(key, finalUIState.viewType, 'month'),
          viewDate: updateRangeViewDate(key, finalUIState.viewDate, finalUIState.viewDate[key].set('month', date.get('month')))
        });
        return;
      }

      if (finalUIState.viewType[key] === 'decade' && selectType !== 'year') {
        var calendarType = selectType === 'date' ? 'month' : 'year';
        updateUI({
          viewType: updateRangeViewType(key, finalUIState.viewType, calendarType),
          viewDate: updateRangeViewDate(key, finalUIState.viewDate, finalUIState.viewDate[key].set('year', date.get('year')))
        });
        return;
      }

      clickSelectDate(date, key);
    };
  };

  var _finalUIState$viewTyp3 = (0, _slicedToArray2["default"])(finalUIState.viewType, 2),
      viewLeftType = _finalUIState$viewTyp3[0],
      viewRightType = _finalUIState$viewTyp3[1];

  var leftYearType = viewLeftType === 'decade' ? 'decade' : 'year';
  var rightYearType = viewRightType === 'decade' ? 'decade' : 'year';
  var handleLeftSwitch = (0, _useEventCallback2.useEventCallback)(handleSwitch(0));
  var handleLeftClick = (0, _useEventCallback2.useEventCallback)(handleItemClick(0));
  var handleLeftPrevMonth = (0, _useEventCallback2.useEventCallback)(handleMove(-1, 'month', 0));
  var handleLeftPrevYear = (0, _useEventCallback2.useEventCallback)(handleMove(-1, leftYearType, 0));
  var handleLeftNextMonth = (0, _useEventCallback2.useEventCallback)(handleMove(1, 'month', 0));
  var handleLeftNextYear = (0, _useEventCallback2.useEventCallback)(handleMove(1, leftYearType, 0));
  var handleRightSwitch = (0, _useEventCallback2.useEventCallback)(handleSwitch(1));
  var handleRightClick = (0, _useEventCallback2.useEventCallback)(handleItemClick(1));
  var handleRightPrevMonth = (0, _useEventCallback2.useEventCallback)(handleMove(-1, 'month', 1));
  var handleRightPrevYear = (0, _useEventCallback2.useEventCallback)(handleMove(-1, rightYearType, 1));
  var handleRightNextMonth = (0, _useEventCallback2.useEventCallback)(handleMove(1, 'month', 1));
  var handleRightNextYear = (0, _useEventCallback2.useEventCallback)(handleMove(1, rightYearType, 1));
  var handleItemHover = (0, _useEventCallback2.useEventCallback)(hoverSelectDate);
  var locale = (0, _Locale.useLocale)();
  var handleOptionSelect = (0, _useEventCallback2.useEventCallback)(function (inputRange) {
    selectRange(inputRange, true);
  });
  var finalOptions = (0, _react.useMemo)(function () {
    if (!options || options.length < 1) {
      return;
    }

    var innerOpts = [];
    options.forEach(function (opt) {
      var rangeFn;

      if (typeof opt.range === 'function') {
        var newRange = opt.range();
        var innerRange = (0, _utils.getRangeInOrder)(newRange);

        if (!innerRange) {
          (0, _devWarning2.devWarning)('[RangeCalendar]: option.range() return invalid range.');
          return;
        } // 处理了一下，opt.range 执行的时候应该是合法的，但是这里类型判断有问题
        // 所以 as any 处理了一下


        rangeFn = function rangeFn() {
          return (0, _utils.getRangeInOrder)(opt.range());
        };
      } else {
        var _innerRange = (0, _utils.getRangeInOrder)(opt.range);

        if (!_innerRange) {
          (0, _devWarning2.devWarning)('[RangeCalendar]: rangeOptions passed invalid range.');
          return;
        }

        rangeFn = function rangeFn() {
          return _innerRange;
        };
      }

      innerOpts.push({
        label: opt.label,
        disabled: opt.disabled,
        onClick: function onClick() {
          handleOptionSelect(rangeFn());
        }
      });
    });
    return innerOpts;
  }, [handleOptionSelect, options]); // 当国际化变了，得更新内存中 dayjs 对象的国际化

  var updateLocale = function updateLocale() {
    var localeKey = locale.dayjsLocaleKey;
    var newViewDate = [finalUIState.viewDate[0].locale(localeKey), finalUIState.viewDate[1].locale(localeKey)];
    updateUI({
      viewDate: newViewDate
    });

    if (!state.start && !state.end) {
      return;
    }

    var newRange;

    if (state.range) {
      newRange = [state.range[0].locale(localeKey), state.range[1].locale(localeKey)];
    }

    var newState = {
      start: state.start ? state.start.locale(localeKey) : undefined,
      end: state.end ? state.end.locale(localeKey) : undefined,
      range: newRange
    };
    updateSelected(newState);
  };

  (0, _react.useEffect)(function () {
    var localeKey = locale.dayjsLocaleKey; // 任一变了就可以了

    var localeChange = localeKey !== finalUIState.viewDate[0].locale();

    if (!localeChange) {
      return;
    }

    updateLocale();
  });
  (0, _react.useEffect)(function () {
    if (isStartControlled || isEndControlled || isRangeControlled) {
      var _start = isStartControlled ? stateFromProps.start : state.start;

      var _end = isEndControlled ? stateFromProps.end : state.end; // 如果start和end都为undefined，则range也为undefined


      var _range;

      if (_start || _end) {
        _range = isRangeControlled ? stateFromProps.range : state.range;
      }

      setState({
        start: _start,
        end: _end,
        range: _range
      });
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [isEndControlled, isRangeControlled, isStartControlled, stateFromProps.end, stateFromProps.range, stateFromProps.start]);
  return (0, _extends2["default"])({}, baseData, {
    otherProps: otherProps,
    finalOptions: finalOptions,
    finalState: state,
    finalUIState: finalUIState,
    finalFixedStart: finalFixedStart,
    finalFixedEnd: finalFixedEnd,
    // 事件处理
    handleItemHover: handleItemHover,
    handleLeftSwitch: handleLeftSwitch,
    handleLeftClick: handleLeftClick,
    handleLeftPrevMonth: handleLeftPrevMonth,
    handleLeftPrevYear: handleLeftPrevYear,
    handleLeftNextMonth: handleLeftNextMonth,
    handleLeftNextYear: handleLeftNextYear,
    handleRightSwitch: handleRightSwitch,
    handleRightClick: handleRightClick,
    handleRightPrevMonth: handleRightPrevMonth,
    handleRightPrevYear: handleRightPrevYear,
    handleRightNextMonth: handleRightNextMonth,
    handleRightNextYear: handleRightNextYear,
    handleOptionSelect: handleOptionSelect,
    // 透传
    styles: styles,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    hideItemOutside: hideItemOutside,
    disableDate: disableDate,
    minRangeLength: minRangeLength,
    maxRangeLength: maxRangeLength,
    editKey: editKey,
    readOnly: readOnly,
    // for test 或者在其他 Hooks 里用
    updateUI: updateUI,
    updateSelected: updateSelected,
    selectRange: selectRange,
    selectFirstDate: selectFirstDate,
    clearSelected: clearSelected
  });
}