"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useCascaderMenuItem;

var _react = require("react");

function useCascaderMenuItem(props) {
  var option = props.option,
      fieldNames = props.fieldNames,
      expandTrigger = props.expandTrigger,
      checkedKeys = props.checkedKeys,
      menuIndex = props.menuIndex,
      loading = props.loading,
      loaded = props.loaded,
      active = props.active,
      onLoad = props.onLoad,
      onSelect = props.onSelect,
      onDeselect = props.onDeselect;
  var delayTimer = (0, _react.useRef)();
  var value = option[fieldNames.value];
  var label = option[fieldNames.label] || value;
  var isLeaf = option[fieldNames.isLeaf];
  var children = option[fieldNames.children];
  var pos = option.pos;
  var checked = checkedKeys.checked.findIndex(function (item) {
    return item.key === value && item.pos === pos;
  }) > -1;
  var halfChecked = checkedKeys.halfChecked.findIndex(function (item) {
    return item.key === value && item.pos === pos;
  }) > -1;
  var hasChildren = children && children.length > 0;
  var enableMouseEvent = expandTrigger === 'hover' && (hasChildren || isLeaf === false);
  var handleSelect = (0, _react.useCallback)(function (e) {
    if (onSelect) {
      onSelect(option, menuIndex, e);
    }
  }, [menuIndex, onSelect, option]);
  var handleDeselect = (0, _react.useCallback)(function (e) {
    if (onDeselect) {
      onDeselect([option], menuIndex, e);
    }
  }, [menuIndex, onDeselect, option]);
  var handleCheck = (0, _react.useCallback)(function (e) {
    if (e.target.checked) {
      handleSelect(e);
    } else {
      handleDeselect(e);
    }
  }, [handleDeselect, handleSelect]);
  var handleMouseEvent = (0, _react.useCallback)(function (e) {
    if (!enableMouseEvent) return;

    if (delayTimer.current) {
      clearTimeout(delayTimer.current);
    }

    e.persist();
    delayTimer.current = setTimeout(function () {
      handleSelect(e);
    }, 150);
  }, [enableMouseEvent, handleSelect]);
  (0, _react.useEffect)(function () {
    if (loading || option.loading) return;

    if (active && isLeaf === false && !loaded && !hasChildren) {
      onLoad(value);
    }
  }, [active, hasChildren, isLeaf, loaded, loading, onLoad, option.loading, value]);
  return {
    value: value,
    label: label,
    isLeaf: isLeaf,
    checked: checked,
    halfChecked: halfChecked,
    hasChildren: hasChildren,
    handleSelect: handleSelect,
    handleCheck: handleCheck,
    handleMouseEvent: handleMouseEvent
  };
}