"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useFormatDate;
exports.formatIsChanged = formatIsChanged;

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var defaultFormatMap = {
  date: 'YYYY-MM-DD',
  month: 'YYYY-MM',
  year: 'YYYY'
};

function getDefaultFormat(selectType) {
  return defaultFormatMap[selectType];
}

var dateForCheck = (0, _dayjs["default"])();

function formatIsChanged(formatA, formatB) {
  // 都不存在返回 false
  if (!formatA && !formatB) {
    return false;
  }

  if (!formatA || !formatB) {
    return true;
  }

  var typeA = (0, _typeof2["default"])(formatA);
  var typeB = (0, _typeof2["default"])(formatB);

  if (typeA !== typeB) {
    return true;
  }

  if (typeA === 'function') {
    return formatA(dateForCheck) !== formatB(dateForCheck);
  }

  return formatA !== formatB;
}

function useFormatDate(selectType, format) {
  var innerFormatDate = (0, _react.useMemo)(function () {
    if (typeof format === 'function') {
      return format;
    }

    var formatStr = format || getDefaultFormat(selectType);
    return function (date) {
      return date.format(formatStr);
    };
  }, [format, selectType]);
  var finalFormatDate = (0, _react.useCallback)(function (date) {
    if (date && date.isValid()) {
      return innerFormatDate(date);
    }

    return '';
  }, [innerFormatDate]);
  var lastSelectType = (0, _react.useRef)(selectType);
  var lastFormat = (0, _react.useRef)(format);
  var selectTypeHasChange = false;
  var formatHasChange = false;

  if (selectType !== lastSelectType.current) {
    lastSelectType.current = selectType;
    selectTypeHasChange = true;
  }

  if (formatIsChanged(format, lastFormat.current)) {
    lastFormat.current = format;
    formatHasChange = true;
  }

  return {
    formatDate: finalFormatDate,
    formatHasChanged: selectTypeHasChange || formatHasChange
  };
}