"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useMultiSelectDatePicker;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _useForkRef2 = require("@muya-ui/utils/lib/hooks/useForkRef");

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _pick2 = _interopRequireDefault(require("lodash/pick"));

var _isEqual2 = _interopRequireDefault(require("lodash/isEqual"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var _useMultiSelectCalendar = _interopRequireDefault(require("../Calendar/useMultiSelectCalendar"));

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _useFormatDate2 = _interopRequireDefault(require("../DatePicker/useFormatDate"));

var _excluded = ["selectType", "disableRange", "disableDate", "options", "todayDate", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "onSelect", "viewType", "viewDate", "selectedDate", "defaultViewDate", "disabled", "readOnly", "defaultValue", "value", "onFocus", "onClear", "allowClear", "inputRef", "inputDelay", "format", "popupOpen", "onChange", "onUIChange", "onDateChange", "onCalendarChange", "onInputKeyDown", "onPopupVisibleChange", "suffixNode", "placement", "triggerId", "popperProps", "flip", "arrowPointAtCenter", "calendarStyles", "styles", "getPopupContainer", "inputReadOnly"];

var getDate = function getDate(value, defaultValue) {
  if (value) {
    return value.map(function (v) {
      return (0, _dayjs["default"])(v);
    });
  }

  if (defaultValue) {
    return defaultValue.map(function (v) {
      return (0, _dayjs["default"])(v);
    });
  }

  return [];
};

var getValidDate = function getValidDate(value) {
  return value.filter(function (v) {
    return v.isValid();
  });
};

var getFormatDate = function getFormatDate(value, format) {
  return value.map(function (v) {
    return format(v);
  });
};

function useMultiSelectDatePicker(props) {
  var _props$selectType = props.selectType,
      selectType = _props$selectType === void 0 ? 'date' : _props$selectType,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      todayDate = props.todayDate,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      onSelect = props.onSelect,
      viewType = props.viewType,
      viewDate = props.viewDate,
      selectedDate = props.selectedDate,
      defaultViewDate = props.defaultViewDate,
      disabled = props.disabled,
      readOnly = props.readOnly,
      defaultValue = props.defaultValue,
      value = props.value,
      onFocus = props.onFocus,
      onClear = props.onClear,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      inputRef = props.inputRef,
      _props$inputDelay = props.inputDelay,
      inputDelay = _props$inputDelay === void 0 ? 200 : _props$inputDelay,
      format = props.format,
      popupOpen = props.popupOpen,
      onChange = props.onChange,
      onUIChange = props.onUIChange,
      onDateChange = props.onDateChange,
      onCalendarChange = props.onCalendarChange,
      onInputKeyDown = props.onInputKeyDown,
      onPopupVisibleChange = props.onPopupVisibleChange,
      suffixNode = props.suffixNode,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      triggerId = props.triggerId,
      popperProps = props.popperProps,
      flip = props.flip,
      arrowPointAtCenter = props.arrowPointAtCenter,
      calendarStyles = props.calendarStyles,
      styles = props.styles,
      getPopupContainer = props.getPopupContainer,
      inputReadOnly = props.inputReadOnly,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var innerInputRef = (0, _react.useRef)(null);
  var handleInputRef = (0, _useForkRef2.useForkRef)(innerInputRef, inputRef);

  var _useFormatDate = (0, _useFormatDate2["default"])(selectType, format),
      formatDate = _useFormatDate.formatDate,
      formatHasChanged = _useFormatDate.formatHasChanged;

  var valueControlled = ('value' in props);
  var valuePropDate = (0, _react.useMemo)(function () {
    return getDate(value, defaultValue);
  }, [defaultValue, value]);
  var valuePropStr = (0, _react.useMemo)(function () {
    return getFormatDate(valuePropDate, formatDate);
  }, [formatDate, valuePropDate]); // 从 props 中获得的合法的 dayjs 对象

  var validValueDateFromProps = (0, _react.useMemo)(function () {
    return getValidDate(valuePropDate);
  }, [valuePropDate]);

  var _useState = (0, _react.useState)(valuePropStr),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      stateValue = _useState2[0],
      setStateValue = _useState2[1];

  var _useState3 = (0, _react.useState)(false),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      statePopupOpen = _useState4[0],
      setPopupOpen = _useState4[1];

  var isOpenControlled = ('popupOpen' in props);
  var finalPopupOpen = isOpenControlled ? !!popupOpen : statePopupOpen;
  var valueInStateIsDifferentFromProps = !(0, _isEqual2["default"])(stateValue, valuePropStr); // 情况1：当用户受控使用时，可能外面会处理 value，重新传入的时候，需要判断一下需不需要更新状态里的 value
  // 情况2：字符串格式化发生变化的时候，需要判断一下状态里的 value 需不需要更新

  if ((valueControlled && !finalPopupOpen || formatHasChanged) && valueInStateIsDifferentFromProps) {
    setStateValue(valuePropStr);
  } // pick 相对于解构的优势在于一些受控的 props ，比如这里
  // selectedDate ，如果直接类似 { selectedDate } ，那么 Hooks 中对于受控的判断就不对了。


  var calendarProps = (0, _pick2["default"])(props, ['selectType', 'selectedDate', 'viewType', 'viewDate', 'disableDate', 'disableRange']);

  if (defaultValue) {
    calendarProps.defaultDate = defaultValue;
  }

  if (defaultViewDate) {
    calendarProps.defaultViewDate = defaultViewDate;
  }

  if (valueControlled && !('selectedDate' in props)) {
    calendarProps.selectedDate = validValueDateFromProps;
  }

  calendarProps.onUIChange = (0, _useEventCallback2.useEventCallback)(function (inputState) {
    var newState = (0, _extends2["default"])({
      popupOpen: finalPopupOpen
    }, inputState);

    if (onUIChange) {
      onUIChange(newState);
    }
  });
  calendarProps.onChange = (0, _useEventCallback2.useEventCallback)(function (date) {
    if (!onDateChange && !onChange) {
      return;
    }

    var innerDate = date || [];

    if (onDateChange) {
      var innerValue = getFormatDate(innerDate, formatDate);
      onDateChange(innerValue, innerDate);
    }

    if (onChange) {
      onChange(innerDate);
    }
  });
  /**
   * 这里复用 hooks 的逻辑需要注意的点：
   * * 事件处理一定要在这个 hook 里处理
   */

  var calendarState = (0, _useMultiSelectCalendar["default"])(calendarProps);
  var finalUIState = (0, _extends2["default"])({}, calendarState.finalUIState, {
    popupOpen: finalPopupOpen
  });
  var finalValue = stateValue;
  var finalSelectedDate = calendarState.finalSelectedDate;

  if (valueControlled && !finalPopupOpen) {
    finalValue = valuePropStr;
  } // 切换日历面板的显示隐藏


  var updateUI = function updateUI(innerPopupOpen) {
    var newState = (0, _extends2["default"])({}, calendarState.finalUIState, {
      popupOpen: innerPopupOpen
    });
    handlePopupVisibleChange(innerPopupOpen);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var lastValidDate = (0, _react.useRef)(validValueDateFromProps);

  var selectByValue = function selectByValue(dates, newValue, isAdd) {
    // 如果是新增，需要校验新字符串的正确性
    if (isAdd) {
      var valueDate = (0, _dayjs["default"])(newValue);
      var valid = valueDate.isValid();

      if (valid && !calendarState.isDateDisabled(valueDate)) {
        lastValidDate.current = getDate([].concat((0, _toConsumableArray2["default"])(dates), [newValue]));
      }
    } else {
      lastValidDate.current = getDate(dates.filter(function (d) {
        return d !== newValue;
      }));
    }

    calendarState.updateSelectedDate(lastValidDate.current, (0, _dayjs["default"])(newValue), isAdd);
    var innerDate = getFormatDate(lastValidDate.current, formatDate);

    if (onDateChange) {
      onDateChange(innerDate, lastValidDate.current);
    }

    if (onChange) {
      onChange(lastValidDate.current);
    }

    return innerDate;
  };
  /**
   * 关闭日历面板
   */


  var closePopup = function closePopup() {
    if (!finalUIState.popupOpen) {
      return;
    }

    updateUI(false);
  };

  var handleCalendarUIChange = (0, _useEventCallback2.useEventCallback)(function (inputState) {
    calendarState.updateUI(inputState);
  });
  var handleCalendarChange = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function (date) {
    calendarState.updateSelectedDate(date || []);
    setStateValue(getFormatDate(date || [], formatDate));
  }, onCalendarChange));
  var handleFocus = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    // 设置viewDate为选中的数组中的最后一个
    if (finalSelectedDate.length > 0) {
      calendarState.updateUIBySelectedDate(finalSelectedDate[finalSelectedDate.length - 1]);
    }

    updateUI(true);
  }, onFocus));
  var handleClear = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    calendarState.updateSelectedDate([]);
    setStateValue([]);
  }, onClear));
  var handleClickAway = (0, _useEventCallback2.useEventCallback)(function () {
    if (finalPopupOpen) {
      closePopup();
    }
  });
  var handleAddDate = (0, _useEventCallback2.useEventCallback)(function (value) {
    // 校验输入是否符合规范
    if (value !== (0, _dayjs["default"])(value).format('YYYY-MM-DD') || !(0, _dayjs["default"])(value).isValid()) {
      if (onDateChange) {
        onDateChange(finalValue, lastValidDate.current);
      }

      if (onChange) {
        onChange(lastValidDate.current);
      }

      return;
    }

    var newValue = selectByValue(finalValue, value, true);
    setStateValue(newValue);
  });
  var handleRemoveDate = (0, _useEventCallback2.useEventCallback)(function (value) {
    var newValue = selectByValue(finalValue, value, false);
    setStateValue(newValue);
  });
  var handlePopupVisibleChange = (0, _useEventCallback2.useEventCallback)(function (open) {
    if (!isOpenControlled) {
      setPopupOpen(open);
    }

    if (onPopupVisibleChange && finalPopupOpen !== open) {
      onPopupVisibleChange(open);
    }
  });
  return {
    finalUIState: finalUIState,
    finalValue: finalValue,
    finalSelectedDate: finalSelectedDate,
    otherProps: otherProps,
    // 事件处理
    handleFocus: handleFocus,
    handleCalendarChange: handleCalendarChange,
    handleCalendarUIChange: handleCalendarUIChange,
    handleAddDate: handleAddDate,
    handleRemoveDate: handleRemoveDate,
    handleClickAway: handleClickAway,
    handleClear: handleClear,
    handleInputRef: handleInputRef,
    handleInputChange: function handleInputChange() {},
    handlePressEnter: function handlePressEnter() {},
    // for test
    innerInputRef: innerInputRef,
    // 透传....
    styles: styles,
    disabled: disabled,
    readOnly: readOnly,
    allowClear: allowClear,
    calendarStyles: calendarStyles,
    onSelect: onSelect,
    container: getPopupContainer,
    inputReadOnly: inputReadOnly,
    // props from calendar 直接透传
    selectType: selectType,
    disableRange: disableRange,
    disableDate: disableDate,
    options: options,
    todayDate: todayDate,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // trigger props
    placement: placement,
    popperProps: popperProps,
    arrowPointAtCenter: arrowPointAtCenter,
    triggerId: triggerId,
    flip: flip
  };
}