"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useRangeDatePicker;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _useDebounce4 = require("@muya-ui/utils/lib/hooks/useDebounce");

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _uniq2 = _interopRequireDefault(require("lodash/uniq"));

var _pick2 = _interopRequireDefault(require("lodash/pick"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var _useRangeCalendar = _interopRequireDefault(require("../Calendar/useRangeCalendar"));

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _useFormatDate2 = _interopRequireDefault(require("./useFormatDate"));

var _excluded = ["calendarStyles", "selectType", "disableRange", "disableDate", "options", "todayDate", "minRangeLength", "maxRangeLength", "fixedEndDate", "fixedStartDate", "defaultViewDate", "inputReadOnly", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "defaultRange", "viewDate", "viewType", "range", "start", "end", "onChange", "onSelect", "styles", "getPopupContainer", "value", "defaultValue", "middleNode", "disabled", "readOnly", "onFocus", "onClear", "onPressEnter", "allowClear", "inputDelay", "format", "popupOpen", "placement", "triggerId", "popperProps", "flip", "arrowPointAtCenter", "onDateChange", "onCalendarChange", "onUIChange", "onPopupVisibleChange"];

/* eslint-disable complexity */
function useRangeDatePicker(props, rangeInputRef) {
  var calendarStyles = props.calendarStyles,
      _props$selectType = props.selectType,
      selectType = _props$selectType === void 0 ? 'date' : _props$selectType,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      todayDate = props.todayDate,
      minRangeLength = props.minRangeLength,
      maxRangeLength = props.maxRangeLength,
      fixedEndDate = props.fixedEndDate,
      fixedStartDate = props.fixedStartDate,
      defaultViewDate = props.defaultViewDate,
      inputReadOnly = props.inputReadOnly,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      defaultRange = props.defaultRange,
      viewDate = props.viewDate,
      viewType = props.viewType,
      range = props.range,
      start = props.start,
      end = props.end,
      onChange = props.onChange,
      onSelect = props.onSelect,
      styles = props.styles,
      getPopupContainer = props.getPopupContainer,
      value = props.value,
      defaultValue = props.defaultValue,
      middleNode = props.middleNode,
      disabled = props.disabled,
      readOnly = props.readOnly,
      onFocus = props.onFocus,
      onClear = props.onClear,
      onPressEnter = props.onPressEnter,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      _props$inputDelay = props.inputDelay,
      inputDelay = _props$inputDelay === void 0 ? 200 : _props$inputDelay,
      format = props.format,
      popupOpen = props.popupOpen,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      triggerId = props.triggerId,
      popperProps = props.popperProps,
      flip = props.flip,
      arrowPointAtCenter = props.arrowPointAtCenter,
      onDateChange = props.onDateChange,
      onCalendarChange = props.onCalendarChange,
      onUIChange = props.onUIChange,
      onPopupVisibleChange = props.onPopupVisibleChange,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);

  var _useFormatDate = (0, _useFormatDate2["default"])(selectType, format),
      formatDate = _useFormatDate.formatDate,
      formatHasChanged = _useFormatDate.formatHasChanged;

  var valueControlled = ('value' in props); // 从 props 拿 value 的值

  var valuePropRange = (0, _react.useMemo)(function () {
    var _ref = value || [],
        _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
        _ref2$ = _ref2[0],
        valueStart = _ref2$ === void 0 ? '' : _ref2$,
        _ref2$2 = _ref2[1],
        valueEnd = _ref2$2 === void 0 ? '' : _ref2$2;

    var _ref3 = defaultValue || [],
        _ref4 = (0, _slicedToArray2["default"])(_ref3, 2),
        _ref4$ = _ref4[0],
        defaultValueStart = _ref4$ === void 0 ? '' : _ref4$,
        _ref4$2 = _ref4[1],
        defaultValueEnd = _ref4$2 === void 0 ? '' : _ref4$2;

    defaultValueStart = fixedStartDate || defaultValueStart;
    defaultValueEnd = fixedEndDate || defaultValueEnd;
    valueStart = valueStart || defaultValueStart;
    valueEnd = valueEnd || defaultValueEnd;
    return [(0, _dayjs["default"])(valueStart), (0, _dayjs["default"])(valueEnd)];
  }, [defaultValue, fixedEndDate, fixedStartDate, value]);

  var _valuePropRange = (0, _slicedToArray2["default"])(valuePropRange, 2),
      valueStartFromProps = _valuePropRange[0],
      valueEndFromProps = _valuePropRange[1];

  var validStartFromProps = (0, _react.useMemo)(function () {
    return valueStartFromProps.isValid() ? valueStartFromProps : undefined;
  }, [valueStartFromProps]);
  var validEndFromProps = (0, _react.useMemo)(function () {
    return valueEndFromProps.isValid() ? valueEndFromProps : undefined;
  }, [valueEndFromProps]);
  var valuePropsStrings = (0, _react.useMemo)(function () {
    return [formatDate(validStartFromProps), formatDate(validEndFromProps)];
  }, [formatDate, validEndFromProps, validStartFromProps]);

  var _useState = (0, _react.useState)(false),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      statePopupOpen = _useState2[0],
      setPopupOpen = _useState2[1];

  var _useState3 = (0, _react.useState)(valuePropsStrings),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      stateValue = _useState4[0],
      setStateValue = _useState4[1];

  var _useState5 = (0, _react.useState)(),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      editKey = _useState6[0],
      setEditKey = _useState6[1]; // 记录修改的editKey，只有修改过开始时间和结束时间才自动关闭面板


  var _useState7 = (0, _react.useState)([]),
      _useState8 = (0, _slicedToArray2["default"])(_useState7, 2),
      selectKeys = _useState8[0],
      setSelectKeys = _useState8[1];

  var isOpenControlled = ('popupOpen' in props);
  var finalPopupOpen = isOpenControlled ? !!popupOpen : statePopupOpen;
  var finalValue = stateValue;
  var valueInStateIsDifferentFromProps = stateValue[0] !== valuePropsStrings[0] || stateValue[1] !== valuePropsStrings[1]; // 情况1：当用户受控使用时，可能外面会处理 value，重新传入的时候，需要判断一下需不需要更新状态里的 value
  // 情况2：字符串格式化发生变化的时候，需要判断一下状态里的 value 需不需要更新

  if ((valueControlled && !finalPopupOpen || formatHasChanged) && valueInStateIsDifferentFromProps) {
    setStateValue(valuePropsStrings);
  } // pick 相对于解构的优势在于一些受控的 props ，比如这里
  // viewDate ，如果直接类似 { viewDate } ，那么 Hooks 中对于受控的判断就不对了。


  var rangeCalenderProps = (0, _pick2["default"])(props, ['selectType', 'viewType', 'viewDate', 'start', 'end', 'range', 'minRangeLength', 'fixedEndDate', 'fixedStartDate', 'disableDate', 'disableRange']);

  if (defaultValue) {
    rangeCalenderProps.defaultRange = defaultValue;
  }

  if (defaultRange || defaultViewDate) {
    rangeCalenderProps.defaultViewDate = defaultRange || defaultViewDate;
  }

  if (valueControlled && !('start' in props) && !finalPopupOpen) {
    rangeCalenderProps.start = validStartFromProps;
  }

  if (valueControlled && !('end' in props) && !finalPopupOpen) {
    rangeCalenderProps.end = validEndFromProps;
  }

  if (valueControlled && !('range' in props) && !finalPopupOpen && validStartFromProps && validEndFromProps) {
    rangeCalenderProps.range = valuePropRange;
  }

  rangeCalenderProps.onUIChange = (0, _useEventCallback2.useEventCallback)(function (inputState) {
    var newState = (0, _extends2["default"])({
      popupOpen: finalPopupOpen
    }, inputState);

    if (onUIChange) {
      onUIChange(newState);
    }
  });
  rangeCalenderProps.onChange = (0, _useEventCallback2.useEventCallback)(function (inputState) {
    var start = inputState.start,
        end = inputState.end;

    if (!onChange && !onDateChange) {
      return;
    } // 过滤掉 hover 区间的情况


    if (!start && end || start && !end) {
      return;
    }

    var newRange;

    if (start && end) {
      newRange = [start, end];
    } else {
      newRange = [(0, _dayjs["default"])(''), (0, _dayjs["default"])('')];
    }

    if (onChange) {
      onChange(newRange);
    }

    if (onDateChange) {
      onDateChange([formatDate(start), formatDate(end)], newRange);
    }
  });
  var rangeCalenderState = (0, _useRangeCalendar["default"])(rangeCalenderProps);

  if (valueControlled && !finalPopupOpen) {
    var _valuePropRange2 = (0, _slicedToArray2["default"])(valuePropRange, 2),
        rangeStart = _valuePropRange2[0],
        rangeEnd = _valuePropRange2[1];

    var rangeStartValue = formatDate(rangeStart);
    var rangeEndValue = formatDate(rangeEnd);
    finalValue = [rangeStartValue, rangeEndValue];
  }

  if (rangeCalenderState.finalFixedStart) {
    finalValue[0] = formatDate(rangeCalenderState.finalFixedStart);
  }

  if (rangeCalenderState.finalFixedEnd) {
    finalValue[1] = formatDate(rangeCalenderState.finalFixedEnd);
  }

  var finalUIState = (0, _extends2["default"])({}, rangeCalenderState.finalUIState, {
    popupOpen: finalPopupOpen
  });

  var updateUI = function updateUI(innerPopupOpen) {
    var newState = (0, _extends2["default"])({}, rangeCalenderState.finalUIState, {
      popupOpen: innerPopupOpen
    });
    handlePopupVisibleChange(innerPopupOpen);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var lastLeftValidDate = (0, _react.useRef)(validStartFromProps);
  var lastRightValidDate = (0, _react.useRef)(validEndFromProps);

  var checkLeftValue = function checkLeftValue(leftValue) {
    var leftDate = (0, _dayjs["default"])(leftValue);
    var leftValid = leftDate.isValid();

    if (leftValid && rangeCalenderState.isDateDisabled(leftDate)) {
      leftValid = false;
    } else if (leftValid && rangeCalenderState.finalFixedEnd && leftDate.isAfter(rangeCalenderState.finalFixedEnd)) {
      leftValid = false;
    }

    return [leftValid, leftDate];
  };

  var checkRightValue = function checkRightValue(rightValue) {
    var rightDate = (0, _dayjs["default"])(rightValue);
    var rightValid = rightDate.isValid();

    if (rightValid && rangeCalenderState.isDateDisabled(rightDate)) {
      rightValid = false;
    } else if (rightValid && rangeCalenderState.finalFixedStart && rightDate.isAfter(rangeCalenderState.finalFixedStart)) {
      rightValid = false;
    }

    return [rightValid, rightDate];
  };

  var selectByValue = function selectByValue(newValue) {
    var _newValue = (0, _slicedToArray2["default"])(newValue, 2),
        leftValue = _newValue[0],
        rightValue = _newValue[1];

    var _checkLeftValue = checkLeftValue(leftValue),
        _checkLeftValue2 = (0, _slicedToArray2["default"])(_checkLeftValue, 2),
        leftValid = _checkLeftValue2[0],
        leftDate = _checkLeftValue2[1];

    var _checkRightValue = checkRightValue(rightValue),
        _checkRightValue2 = (0, _slicedToArray2["default"])(_checkRightValue, 2),
        rightValid = _checkRightValue2[0],
        rightDate = _checkRightValue2[1];

    if (leftValid) {
      lastLeftValidDate.current = leftDate;
    }

    if (rightValid) {
      lastRightValidDate.current = rightDate;
    }

    if (!leftValid && !rightValid && allowClear) {
      rangeCalenderState.clearSelected();
      return ['', ''];
    }

    var newLeft = lastLeftValidDate.current;
    var newRight = lastRightValidDate.current;

    if (newLeft && newRight) {
      rangeCalenderState.selectRange([newLeft, newRight]);
      return [formatDate(newLeft), formatDate(newRight)];
    }

    if (newLeft) {
      rangeCalenderState.selectFirstDate(newLeft, 0, true);
      return [formatDate(newLeft), ''];
    }

    if (newRight) {
      rangeCalenderState.selectFirstDate(newRight, 1, true);
      return ['', formatDate(newRight)];
    }

    return ['', ''];
  };

  var _useDebounce = (0, _useDebounce4.useDebounce)(selectByValue, inputDelay),
      _useDebounce2 = (0, _slicedToArray2["default"])(_useDebounce, 2),
      debounceUpdateInputSelected = _useDebounce2[0],
      clearUpdateInputSelected = _useDebounce2[1];

  var closePopup = function closePopup() {
    if (finalUIState.popupOpen) {
      updateUI(false);
      clearUpdateInputSelected();
      setSelectKeys([]);
    }
  };

  var handleCalendarUIChange = (0, _useEventCallback2.useEventCallback)(function (inputState) {
    rangeCalenderState.updateUI(inputState);
  });
  var handleCalendarChange = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function (inputState) {
    var start = inputState.start,
        end = inputState.end;
    rangeCalenderState.updateSelected(inputState); // 当其中一个是空或者同时修改了开始和结束时间

    if (start && end && !(stateValue[0] && stateValue[1]) || selectKeys.length === 2) {
      closePopup();
    }

    setStateValue([formatDate(start), formatDate(end)]);
  }, onCalendarChange));
  var handleInputChange = (0, _useEventCallback2.useEventCallback)(function (e) {
    debounceUpdateInputSelected(e.value);
    setStateValue(e.value);
  });
  var handleFocus = (0, _useEventCallback2.useEventCallback)(function (e, index) {
    if (stateValue[0] && stateValue[1]) {
      setEditKey(index);
      setSelectKeys((0, _uniq2["default"])([].concat((0, _toConsumableArray2["default"])(selectKeys), [index])));
    }

    updateUI(true);
    onFocus && onFocus(e);
  });
  var handleClear = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    rangeCalenderState.clearSelected();
    clearUpdateInputSelected();
    setStateValue(['', '']);
  }, onClear));
  var handleClickAway = (0, _useEventCallback2.useEventCallback)(function () {
    if (finalPopupOpen) {
      closePopup();
    }
  });
  var handlePressEnter = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    if (rangeInputRef.current) {
      rangeInputRef.current.blur(0);
      rangeInputRef.current.blur(1);
    }

    var newValue = selectByValue(finalValue);
    closePopup();
    setStateValue(newValue);
  }, onPressEnter));
  var handlePopupVisibleChange = (0, _useEventCallback2.useEventCallback)(function (open) {
    if (!isOpenControlled) {
      setPopupOpen(open);
    }

    if (onPopupVisibleChange && finalPopupOpen !== open) {
      onPopupVisibleChange(open);
    }
  }); // 监听selectType变化

  (0, _react.useEffect)(function () {
    if (onDateChange) {
      onDateChange(valuePropsStrings, valuePropRange);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [selectType]);
  return {
    finalUIState: finalUIState,
    finalValue: finalValue,
    finalState: rangeCalenderState.finalState,
    otherProps: otherProps,
    editKey: editKey,
    // 事件处理
    handleFocus: handleFocus,
    handleClear: handleClear,
    handleCalendarUIChange: handleCalendarUIChange,
    handleCalendarChange: handleCalendarChange,
    handleInputChange: handleInputChange,
    handleClickAway: handleClickAway,
    handlePressEnter: handlePressEnter,
    // 透传
    styles: styles,
    calendarStyles: calendarStyles,
    allowClear: allowClear,
    disabled: disabled,
    readOnly: readOnly,
    selectType: selectType,
    disableRange: disableRange,
    disableDate: disableDate,
    options: options,
    todayDate: todayDate,
    minRangeLength: minRangeLength,
    maxRangeLength: maxRangeLength,
    fixedEndDate: fixedEndDate,
    fixedStartDate: fixedStartDate,
    container: getPopupContainer,
    inputReadOnly: inputReadOnly,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    onSelect: onSelect,
    // trigger
    placement: placement,
    popperProps: popperProps,
    arrowPointAtCenter: arrowPointAtCenter,
    triggerId: triggerId,
    flip: flip,
    // for test,
    selectByValue: selectByValue,
    checkLeftValue: checkLeftValue,
    checkRightValue: checkRightValue
  };
}