"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useDateTimeCalendar;
exports.getDisableDateFn = getDisableDateFn;
exports.transformDateByDate = transformDateByDate;
exports.transformDateByTime = transformDateByTime;

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var _useCalendar = _interopRequireDefault(require("../Calendar/useCalendar"));

var _utils = require("../Calendar/utils");

var _mergeBooleanFns = _interopRequireDefault(require("../utils/mergeBooleanFns"));

var _excluded = ["allowClear", "todayDate", "viewType", "viewDate", "disableRange", "disableDate", "options", "onUIChange", "defaultViewDate", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "timeRowNum", "hourStep", "minuteStep", "secondStep", "disabledHours", "disabledMinutes", "disabledSeconds", "hideDisabledOptions", "selectedDate", "disableTimeAfter", "disableTimeBefore", "defaultDate", "timeFormat", "defaultTime", "onChange", "onClear", "onSelect", "onConfirm", "styles"];

function transformDateByTime(date, timeStr) {
  var time = timeStr === 'now' ? (0, _dayjs["default"])().format('HH:mm:ss') : timeStr;
  return (0, _dayjs["default"])("".concat(date.format('YYYY-MM-DD'), " ").concat(time));
}

function transformDateByDate(date, otherDate) {
  return (0, _dayjs["default"])("".concat(date.format('YYYY-MM-DD'), " ").concat(otherDate.format('HH:mm:ss')));
}
/**
 * 根据禁用之前之后的时间来自定义时间的禁用
 * @param disableTimeBefore
 * @param disableTimeAfter
 */


function getDisableDateFn(disableTimeBefore, disableTimeAfter) {
  if (!disableTimeAfter && !disableTimeBefore) {
    return;
  }

  var disableTimeAfterFn;
  var disableTimeBeforeFn;

  if (disableTimeAfter) {
    var time = (0, _dayjs["default"])(disableTimeAfter); // 注意：一定是开区间的

    disableTimeAfterFn = function disableTimeAfterFn(date) {
      return date.isAfter(time) && !date.isSame(time, 'd');
    };
  }

  if (disableTimeBefore) {
    var _time = (0, _dayjs["default"])(disableTimeBefore); // 注意：一定是开区间的


    disableTimeBeforeFn = function disableTimeBeforeFn(date) {
      return date.isBefore(_time) && !date.isSame(_time, 'd');
    };
  }

  var disableDateByTime = function disableDateByTime(date) {
    if (disableTimeAfterFn && disableTimeAfterFn(date)) {
      return true;
    }

    if (disableTimeBeforeFn && disableTimeBeforeFn(date)) {
      return true;
    }

    return false;
  };

  return disableDateByTime;
}

function useDateTimeCalendar(props) {
  var _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      todayDate = props.todayDate,
      viewType = props.viewType,
      viewDate = props.viewDate,
      disableRange = props.disableRange,
      disableDate = props.disableDate,
      options = props.options,
      onUIChange = props.onUIChange,
      defaultViewDate = props.defaultViewDate,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      timeRowNum = props.timeRowNum,
      hourStep = props.hourStep,
      minuteStep = props.minuteStep,
      secondStep = props.secondStep,
      disabledHours = props.disabledHours,
      disabledMinutes = props.disabledMinutes,
      disabledSeconds = props.disabledSeconds,
      hideDisabledOptions = props.hideDisabledOptions,
      selectedDate = props.selectedDate,
      disableTimeAfter = props.disableTimeAfter,
      disableTimeBefore = props.disableTimeBefore,
      defaultDate = props.defaultDate,
      _props$timeFormat = props.timeFormat,
      timeFormat = _props$timeFormat === void 0 ? 'HH:mm:ss' : _props$timeFormat,
      _props$defaultTime = props.defaultTime,
      defaultTime = _props$defaultTime === void 0 ? 'now' : _props$defaultTime,
      onChange = props.onChange,
      onClear = props.onClear,
      onSelect = props.onSelect,
      onConfirm = props.onConfirm,
      styles = props.styles,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var isControlled = ('selectedDate' in props);
  var valueFromProps = (0, _react.useMemo)(function () {
    if (isControlled) {
      if (!selectedDate) {
        return;
      }

      var _innerDate = (0, _dayjs["default"])(selectedDate);

      if (_innerDate.isValid()) {
        return _innerDate;
      }

      return;
    }

    if (!defaultDate) {
      return;
    }

    var innerDate = (0, _dayjs["default"])(defaultDate);

    if (innerDate.isValid()) {
      return innerDate;
    }
  }, [defaultDate, isControlled, selectedDate]);

  var _useState = (0, _react.useState)(valueFromProps),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      stateValue = _useState2[0],
      setStateValue = _useState2[1];

  var finalValue = isControlled ? valueFromProps : stateValue;
  var memoDisableTimeBefore = (0, _react.useMemo)(function () {
    if (disableTimeBefore) {
      var innerDate = (0, _dayjs["default"])(disableTimeBefore);
      return innerDate.isValid() ? innerDate : undefined;
    }
  }, [disableTimeBefore]);
  var memoDisableTimeAfter = (0, _react.useMemo)(function () {
    if (disableTimeAfter) {
      var innerDate = (0, _dayjs["default"])(disableTimeAfter);
      return innerDate.isValid() ? innerDate : undefined;
    }
  }, [disableTimeAfter]);
  var disableTime = (0, _react.useMemo)(function () {
    return getDisableDateFn(memoDisableTimeBefore, memoDisableTimeAfter);
  }, [memoDisableTimeAfter, memoDisableTimeBefore]); // disableTime 和 disableDate 同时存在的话要合并

  var finalDisableDate = (0, _react.useMemo)(function () {
    return (0, _mergeBooleanFns["default"])(disableDate, disableTime);
  }, [disableDate, disableTime]);

  var updateValue = function updateValue(date) {
    var innerDate = date;

    if (memoDisableTimeAfter && innerDate && innerDate.isAfter(memoDisableTimeAfter)) {
      innerDate = memoDisableTimeAfter;
    }

    if (memoDisableTimeBefore && innerDate && innerDate.isBefore(memoDisableTimeBefore)) {
      innerDate = memoDisableTimeBefore;
    }

    setStateValue(innerDate);

    if (onChange) {
      onChange(innerDate);
    }

    if (innerDate && onSelect) {
      onSelect(innerDate);
    }

    if (!innerDate && onClear) {
      onClear();
    }
  };

  var handleDateChange = (0, _useEventCallback2.useEventCallback)(function (date) {
    if (!date) {
      updateValue();
      return;
    } // 如果有值，使用之前的值，如果没有使用设置的值


    var transformedDate = finalValue ? transformDateByDate(date, finalValue) : transformDateByTime(date, defaultTime);
    updateValue(transformedDate);
  });
  var handleClear = (0, _useEventCallback2.useEventCallback)(function () {
    updateValue();
  });
  var handleConfirm = (0, _useEventCallback2.useEventCallback)(function () {
    if (onConfirm && finalValue) {
      onConfirm(finalValue);
    }
  });
  var calendarProps = {
    allowClear: allowClear,
    todayDate: todayDate,
    disableRange: disableRange,
    disableDate: finalDisableDate,
    onUIChange: onUIChange,
    defaultViewDate: defaultViewDate,
    selectedDate: finalValue,
    onChange: handleDateChange
  };

  if ('viewType' in props) {
    calendarProps.viewType = viewType;
  }

  if ('viewDate' in props) {
    calendarProps.viewDate = viewDate;
  }

  var calendarState = (0, _useCalendar["default"])(calendarProps);
  var handleTimeChange = (0, _useEventCallback2.useEventCallback)(function (time) {
    updateValue(time);
  });
  var handleOptionClick = (0, _useEventCallback2.useEventCallback)(function (time) {
    calendarState.updateUIBySelectedDate(time);
    updateValue(time);
  });
  var timePickerPanelProps = {
    hourStep: hourStep,
    minuteStep: minuteStep,
    secondStep: secondStep,
    disabledHours: disabledHours,
    disabledMinutes: disabledMinutes,
    disabledSeconds: disabledSeconds,
    disableTimeAfter: memoDisableTimeAfter,
    disableTimeBefore: memoDisableTimeBefore,
    hideDisabledOptions: hideDisabledOptions,
    value: finalValue,
    onChange: handleTimeChange,
    format: timeFormat
  };
  var finalOptions = (0, _react.useMemo)(function () {
    return (0, _utils.formatCalendarOptions)(options, handleOptionClick, function (opt) {
      if (typeof opt.date === 'function') {
        (0, _devWarning2.devWarning)('[DateTimeCalendar]: option.date() return invalid date.');
      } else {
        (0, _devWarning2.devWarning)('[DateTimeCalendar]: options passed invalid date.');
      }
    });
  }, [handleOptionClick, options]);
  return {
    calendarState: calendarState,
    otherProps: otherProps,
    finalValue: finalValue,
    timePanelTitle: finalValue ? finalValue.format(timeFormat) : '',
    timePickerPanelProps: timePickerPanelProps,
    finalOptions: finalOptions,
    timeRowNum: timeRowNum,
    // 直接透传
    styles: styles,
    allowClear: allowClear,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // 事件处理
    handleTimeChange: handleTimeChange,
    handleDateChange: handleDateChange,
    handleClear: handleClear,
    handleConfirm: handleConfirm,
    // 给其他 Hook 使用
    updateValue: updateValue,
    // for test
    handleOptionClick: handleOptionClick
  };
}