"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useForm;

var _useEffectOnce2 = require("@muya-ui/utils/lib/hooks/useEffectOnce");

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _extends3 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _isFunction2 = _interopRequireDefault(require("lodash/isFunction"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _isBoolean2 = _interopRequireDefault(require("lodash/isBoolean"));

var _hasIn2 = _interopRequireDefault(require("lodash/hasIn"));

var _asyncValidator = _interopRequireDefault(require("async-validator"));

var _deepmerge = _interopRequireDefault(require("deepmerge"));

var _react = require("react");

var _scrollIntoView = _interopRequireDefault(require("../utils/scrollIntoView"));

var _utils = require("./utils");

// State reducer
function formReducer(state, msg) {
  switch (msg.type) {
    case 'SET_VALUES':
      return (0, _extends3["default"])({}, state, {
        values: msg.payload
      });

    case 'SET_TOUCHED':
      return (0, _extends3["default"])({}, state, {
        touched: msg.payload
      });

    case 'SET_ERRORS':
      return (0, _extends3["default"])({}, state, {
        errors: msg.payload
      });

    case 'SET_STATUS':
      return (0, _extends3["default"])({}, state, {
        status: msg.payload
      });

    case 'SET_ISSUBMITTING':
      return (0, _extends3["default"])({}, state, {
        isSubmitting: msg.payload
      });

    case 'SET_ISVALIDATING':
      return (0, _extends3["default"])({}, state, {
        isValidating: msg.payload
      });

    case 'SET_FIELD_VALUE':
      return (0, _extends3["default"])({}, state, {
        values: (0, _utils.setIn)(state.values, msg.payload.field, msg.payload.value)
      });

    case 'SET_FIELD_TOUCHED':
      return (0, _extends3["default"])({}, state, {
        touched: (0, _utils.setIn)(state.touched, msg.payload.field, msg.payload.value)
      });

    case 'SET_FIELD_ERROR':
      {
        var newErrors = msg.payload.enableNestedErrors ? (0, _utils.setIn)(state.errors, msg.payload.field, msg.payload.value) : (0, _extends3["default"])({}, state.errors, (0, _defineProperty2["default"])({}, msg.payload.field, msg.payload.value));
        return (0, _extends3["default"])({}, state, {
          errors: newErrors
        });
      }

    case 'RESET_FORM':
    case 'SET_FORM_STATE':
      return (0, _extends3["default"])({}, state, msg.payload);

    case 'SUBMIT_ATTEMPT':
      return (0, _extends3["default"])({}, state, {
        touched: (0, _utils.setNestedObjectValues)(state.values, true),
        isSubmitting: true,
        submitCount: state.submitCount + 1
      });

    case 'SUBMIT_FAILURE':
      return (0, _extends3["default"])({}, state, {
        isSubmitting: false
      });

    case 'SUBMIT_SUCCESS':
      return (0, _extends3["default"])({}, state, {
        isSubmitting: false
      });

    default:
      return state;
  }
} // Initial empty states // objects


var emptyErrors = {};
var emptyTouched = {};
var emptyValues = {};

function useForm(props) {
  var onChange = props.onChange,
      valuesProp = props.values,
      _props$defaultValues = props.defaultValues,
      defaultValues = _props$defaultValues === void 0 ? emptyValues : _props$defaultValues,
      _props$defaultErrors = props.defaultErrors,
      defaultErrors = _props$defaultErrors === void 0 ? emptyErrors : _props$defaultErrors,
      _props$defaultTouched = props.defaultTouched,
      defaultTouched = _props$defaultTouched === void 0 ? emptyTouched : _props$defaultTouched,
      defaultStatus = props.defaultStatus,
      _props$validateOnBlur = props.validateOnBlur,
      validateOnBlur = _props$validateOnBlur === void 0 ? true : _props$validateOnBlur,
      _props$validateOnChan = props.validateOnChange,
      validateOnChange = _props$validateOnChan === void 0 ? true : _props$validateOnChan,
      _props$enableNestedEr = props.enableNestedErrors,
      enableNestedErrors = _props$enableNestedEr === void 0 ? false : _props$enableNestedEr,
      _props$scrollToFirstE = props.scrollToFirstError,
      scrollToFirstError = _props$scrollToFirstE === void 0 ? false : _props$scrollToFirstE,
      scrollSettings = props.scrollSettings,
      onSubmit = props.onSubmit,
      onReset = props.onReset,
      children = props.children;
  var defaultValuesRef = (0, _react.useRef)(defaultValues);
  var defaultErrorsRef = (0, _react.useRef)(defaultErrors);
  var defaultTouchedRef = (0, _react.useRef)(defaultTouched);
  var defaultStatusRef = (0, _react.useRef)(defaultStatus);
  var formItemsRef = (0, _react.useRef)({});
  var isControlled = ('values' in props);

  var _useReducer = (0, _react.useReducer)(formReducer, {
    values: defaultValues,
    errors: defaultErrors,
    touched: defaultTouched,
    isSubmitting: false,
    isValidating: false,
    submitCount: 0,
    status: defaultStatus
  }),
      _useReducer2 = (0, _slicedToArray2["default"])(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var fieldRules = (0, _react.useRef)({});
  var fieldProps = (0, _react.useRef)({}); // 受控模式情况使用props中的values，非受控使用state.values

  var currentValues = isControlled && valuesProp ? valuesProp : state.values;
  var handleFormItemRef = (0, _react.useCallback)(function (name, ref) {
    formItemsRef.current[name] = ref;
  }, []);
  var resetForm = (0, _useEventCallback2.useEventCallback)(function (nextState) {
    var values = nextState && nextState.values ? nextState.values : defaultValuesRef.current;
    var errors = nextState && nextState.errors ? nextState.errors : defaultErrorsRef.current ? defaultErrorsRef.current : defaultErrors || {};
    var touched = nextState && nextState.touched ? nextState.touched : defaultTouchedRef.current ? defaultTouchedRef.current : defaultTouched || {};
    var status = nextState && nextState.status ? nextState.status : defaultStatusRef.current ? defaultStatusRef.current : defaultStatus;
    defaultValuesRef.current = values;
    defaultErrorsRef.current = errors;
    defaultTouchedRef.current = touched;
    defaultStatusRef.current = status;
    dispatch({
      type: 'RESET_FORM',
      payload: {
        isSubmitting: !!nextState && !!nextState.isSubmitting,
        errors: errors,
        touched: touched,
        status: status,
        values: values,
        isValidating: !!nextState && !!nextState.isValidating,
        submitCount: !!nextState && !!nextState.submitCount && typeof nextState.submitCount === 'number' ? nextState.submitCount : 0
      }
    });
    onChange && onChange(values, helpers);
  }, [defaultErrors, defaultTouched, defaultStatus, onChange]);
  var registerFieldRules = (0, _react.useCallback)(function (name, rule) {
    fieldRules.current[name] = rule;
  }, []);
  var unregisterFieldRules = (0, _react.useCallback)(function (name) {
    delete fieldRules.current[name];
  }, []);
  var registerFieldProps = (0, _react.useCallback)(function (name, props) {
    fieldProps.current[name] = props;
  }, []);
  var unregisterFieldProps = (0, _react.useCallback)(function (name) {
    delete fieldProps.current[name];
  }, []);
  var setStatus = (0, _react.useCallback)(function (status) {
    dispatch({
      type: 'SET_STATUS',
      payload: status
    });
  }, []);
  var setSubmitting = (0, _react.useCallback)(function (isSubmitting) {
    dispatch({
      type: 'SET_ISSUBMITTING',
      payload: isSubmitting
    });
  }, []);
  var setErrors = (0, _react.useCallback)(function (errors) {
    dispatch({
      type: 'SET_ERRORS',
      payload: errors
    });
  }, []);
  var setFieldError = (0, _react.useCallback)(function (field, value) {
    dispatch({
      type: 'SET_FIELD_ERROR',
      payload: {
        field: field,
        value: value,
        enableNestedErrors: enableNestedErrors
      }
    });
  }, [enableNestedErrors]); // 表单整体校验

  var validateForm = (0, _useEventCallback2.useEventCallback)(function (newValues) {
    var values = newValues || currentValues;
    dispatch({
      type: 'SET_ISVALIDATING',
      payload: true
    }); // 校验所有已注册过Rule的Field

    var allValues = {};
    Object.keys(fieldRules.current).forEach(function (field) {
      var value = (0, _utils.getIn)(values, field);
      allValues[field] = value;
    });
    var validator = new _asyncValidator["default"](fieldRules.current);
    return validator.validate(allValues, {}, function (_errors, fieldErrors) {
      dispatch({
        type: 'SET_ISVALIDATING',
        payload: false
      });

      if (fieldErrors) {
        var newErrors = {};

        if (enableNestedErrors) {
          Object.keys(fieldErrors).forEach(function (field) {
            newErrors = (0, _utils.setIn)(newErrors, field, fieldErrors[field]);
          });
          setErrors(newErrors);
        } else {
          setErrors(fieldErrors);
        }
      } else {
        setErrors({});
      }
    });
  }, [currentValues, setErrors]);
  var validatePartialForm = (0, _useEventCallback2.useEventCallback)(function (fields, partialValues) {
    var values = partialValues || currentValues;
    dispatch({
      type: 'SET_ISVALIDATING',
      payload: true
    }); // 校验所有已注册过Rule的Field

    var partialRules = {};
    fields.forEach(function (field) {
      var rule = fieldRules.current[field];

      if (rule) {
        partialRules[field] = rule;
      }
    });
    var handledPartialValues = {};
    Object.keys(partialRules).forEach(function (field) {
      var value = (0, _utils.getIn)(values, field);
      handledPartialValues[field] = value;
    });
    var validator = new _asyncValidator["default"](partialRules);
    return validator.validate(handledPartialValues, {}, function (_errors, fieldErrors) {
      dispatch({
        type: 'SET_ISVALIDATING',
        payload: false
      });

      if (fieldErrors) {
        var newErrors = {};

        if (enableNestedErrors) {
          Object.keys(fieldErrors).forEach(function (field) {
            newErrors = (0, _utils.setIn)(newErrors, field, fieldErrors[field]);
          });
          setErrors(newErrors);
        } else {
          setErrors(fieldErrors);
        }
      } else {
        setErrors({});
      }
    });
  }, [currentValues, setErrors]); // 校验单个表单项

  var validateField = (0, _useEventCallback2.useEventCallback)(function (field, outerValue) {
    var rule = fieldRules.current[field];
    var currentFieldProps = fieldProps.current[field];
    var validateFirst = currentFieldProps ? currentFieldProps.validateFirst : false;
    var value = outerValue === undefined ? (0, _utils.getIn)(currentValues, field) : outerValue;

    if (rule) {
      dispatch({
        type: 'SET_ISVALIDATING',
        payload: true
      });
      var validator = new _asyncValidator["default"]((0, _defineProperty2["default"])({}, field, rule));
      return validator.validate((0, _defineProperty2["default"])({}, field, value), {
        first: validateFirst
      }, function (errors) {
        dispatch({
          type: 'SET_ISVALIDATING',
          payload: false
        });
        setFieldError(field, errors);
      });
    }

    return Promise.resolve();
  }, [currentValues]);
  var setTouched = (0, _react.useCallback)(function (touched) {
    dispatch({
      type: 'SET_TOUCHED',
      payload: touched
    });

    if (validateOnBlur) {
      validateForm();
    }
  }, [validateForm, validateOnBlur]);
  var setValues = (0, _useEventCallback2.useEventCallback)(function (values) {
    var shouldValidate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    dispatch({
      type: 'SET_VALUES',
      payload: values
    });
    onChange && onChange(values, helpers);

    if (validateOnChange && shouldValidate) {
      validateForm(values);
    }
  });
  var setPartialValues = (0, _useEventCallback2.useEventCallback)(function (values) {
    var shouldValidate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var deep = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var newValues;

    if (deep) {
      newValues = (0, _deepmerge["default"])((0, _extends3["default"])({}, currentValues), values, (0, _isBoolean2["default"])(deep) ? {
        arrayMerge: _utils.overwriteArrayMerge
      } : deep);
    } else {
      newValues = (0, _extends3["default"])({}, currentValues, values);
    }

    dispatch({
      type: 'SET_VALUES',
      payload: newValues
    });
    onChange && onChange(newValues, helpers);

    if (validateOnChange && shouldValidate) {
      var validateFields = [];
      Object.keys(fieldRules.current).forEach(function (field) {
        if ((0, _hasIn2["default"])(values, field)) {
          validateFields.push(field);
        }
      });
      validatePartialForm(validateFields, values);
    }
  });
  var setFieldValue = (0, _useEventCallback2.useEventCallback)(function (field, value) {
    var shouldValidate = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    dispatch({
      type: 'SET_FIELD_VALUE',
      payload: {
        field: field,
        value: value
      }
    });
    onChange && onChange((0, _utils.setIn)(currentValues, field, value), helpers, field, value);
    return validateOnChange && shouldValidate ? validateField(field, value) : Promise.resolve();
  }, [currentValues, onChange, validateField, validateOnChange]);
  var setFieldTouched = (0, _react.useCallback)(function (field) {
    var touched = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    var shouldValidate = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    dispatch({
      type: 'SET_FIELD_TOUCHED',
      payload: {
        field: field,
        value: touched
      }
    });
    return validateOnBlur && shouldValidate ? validateField(field) : Promise.resolve();
  }, [validateField, validateOnBlur]);
  var getFieldError = (0, _react.useCallback)(function (field) {
    var error = enableNestedErrors ? (0, _utils.getIn)(state.errors, field) : state.errors[field] || [];

    if (error && error.length) {
      return error.map(function (item) {
        return item.message;
      });
    }
  }, [enableNestedErrors, state]);
  var getFieldsError = (0, _react.useCallback)(function (fields) {
    var finalFields = fields || Object.keys(fieldRules.current);
    var allErrors = {};
    finalFields.forEach(function (field) {
      var error = getFieldError(field);

      if (error) {
        allErrors[field] = error;
      }
    });
    return allErrors;
  }, [getFieldError]);
  var scrollToField = (0, _react.useCallback)(function (name) {
    var itemRef = formItemsRef.current[name];

    if (itemRef) {
      try {
        (0, _scrollIntoView["default"])(itemRef, (0, _extends3["default"])({
          onlyFirstScrollableParent: true,
          align: {
            top: 0
          }
        }, scrollSettings));
      } catch (_unused) {
        (0, _devWarning2.devWarning)('[Form]: form scrollToFieldError occur error');
      }
    } else {
      (0, _devWarning2.devWarning)("[Form]: form name: ".concat(name, " field not exist"));
    }
  }, [scrollSettings]);
  var scrollToFirstErrorField = (0, _react.useCallback)(function (errors) {
    if (!(0, _isEmpty2["default"])(errors)) {
      var firstErrorField = errors[0].field;
      scrollToField(firstErrorField);
    }
  }, [scrollToField]);
  var helpers = {
    resetForm: resetForm,
    validateForm: validateForm,
    validatePartialForm: validatePartialForm,
    validateField: validateField,
    setErrors: setErrors,
    setFieldError: setFieldError,
    setFieldTouched: setFieldTouched,
    setFieldValue: setFieldValue,
    setStatus: setStatus,
    setSubmitting: setSubmitting,
    setTouched: setTouched,
    setValues: setValues,
    setPartialValues: setPartialValues,
    getFieldError: getFieldError,
    getFieldsError: getFieldsError,
    registerFieldRules: registerFieldRules,
    unregisterFieldRules: unregisterFieldRules,
    registerFieldProps: registerFieldProps,
    unregisterFieldProps: unregisterFieldProps,
    scrollToField: scrollToField,
    scrollToFirstErrorField: scrollToFirstErrorField
  };
  var submitForm = (0, _react.useCallback)(function () {
    dispatch({
      type: 'SUBMIT_ATTEMPT'
    });
    return validateForm().then(function () {
      dispatch({
        type: 'SUBMIT_SUCCESS'
      });
      onSubmit && onSubmit(currentValues, helpers);
    })["catch"](function (data) {
      dispatch({
        type: 'SUBMIT_FAILURE'
      });

      if (scrollToFirstError) {
        scrollToFirstErrorField(data.errors);
      }
    });
  }, [currentValues, helpers, onSubmit, scrollToFirstError, scrollToFirstErrorField, validateForm]);
  var handleSubmit = (0, _react.useCallback)(function (e) {
    if (e && e.preventDefault && (0, _isFunction2["default"])(e.preventDefault)) {
      e.preventDefault();
    }

    if (e && e.stopPropagation && (0, _isFunction2["default"])(e.stopPropagation)) {
      e.stopPropagation();
    }

    return submitForm();
  }, [submitForm]);
  var handleReset = (0, _react.useCallback)(function (e) {
    if (e && e.preventDefault && (0, _isFunction2["default"])(e.preventDefault)) {
      e.preventDefault();
    }

    if (e && e.stopPropagation && (0, _isFunction2["default"])(e.stopPropagation)) {
      e.stopPropagation();
    }

    if (onReset) {
      onReset(currentValues, helpers);
    }

    resetForm();
  }, [currentValues, helpers, onReset, resetForm]);
  (0, _useEffectOnce2.useEffectOnce)(function () {
    if ('scrollToFirstErrorSettings' in props) {
      (0, _devWarning2.devWarning)('[Form]: prop `scrollToFirstErrorSettings` is deprecated, please use `scrollSettings` instead.');
    }
  });
  return (0, _extends3["default"])({}, state, helpers, props, {
    handleFormItemRef: handleFormItemRef,
    handleReset: handleReset,
    handleSubmit: handleSubmit,
    children: children,
    defaultStatus: defaultStatusRef.current,
    defaultErrors: defaultErrorsRef.current,
    defaultTouched: defaultTouchedRef.current,
    defaultValues: defaultValuesRef.current,
    validateOnBlur: validateOnBlur,
    validateOnChange: validateOnChange,
    onSubmit: onSubmit,
    onReset: onReset
  });
}