"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getIn = getIn;
exports.overwriteArrayMerge = exports.isInteger = void 0;
exports.setIn = setIn;
exports.setNestedObjectValues = setNestedObjectValues;

var _toPath2 = _interopRequireDefault(require("lodash/toPath"));

var _isObject2 = _interopRequireDefault(require("lodash/isObject"));

var _clone2 = _interopRequireDefault(require("lodash/clone"));

var isInteger = function isInteger(obj) {
  return String(Math.floor(Number(obj))) === obj;
};

exports.isInteger = isInteger;

var overwriteArrayMerge = function overwriteArrayMerge(destinationArray, sourceArray) {
  return sourceArray;
};
/**
 * Deeply get a value from an object via its path.
 */


exports.overwriteArrayMerge = overwriteArrayMerge;

function getIn(obj, key, def) {
  var p = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
  var path = (0, _toPath2["default"])(key);

  while (obj && p < path.length) {
    obj = obj[path[p++]]; // eslint-disable-line
  }

  return obj === undefined ? def : obj;
}
/**
 * Deeply set a value from in object via it's path. If the value at `path`
 * has changed, return a shallow copy of obj with `value` set at `path`.
 * If `value` has not changed, return the original `obj`.
 *
 * Existing objects / arrays along `path` are also shallow copied. Sibling
 * objects along path retain the same internal js reference. Since new
 * objects / arrays are only created along `path`, we can test if anything
 * changed in a nested structure by comparing the object's reference in
 * the old and new object, similar to how russian doll cache invalidation
 * works.
 *
 * In earlier versions of this function, which used cloneDeep, there were
 * issues whereby settings a nested value would mutate the parent
 * instead of creating a new object. `clone` avoids that bug making a
 * shallow copy of the objects along the update path
 * so no object is mutated in place.
 *
 * Before changing this function, please read through the following
 * discussions.
 *
 * @see https://github.com/developit/linkstate
 * @see https://github.com/jaredpalmer/formik/pull/123
 */


function setIn(obj, path, value) {
  var res = (0, _clone2["default"])(obj); // this keeps inheritance when obj is a class

  var resVal = res;
  var i = 0;
  var pathArray = (0, _toPath2["default"])(path);

  for (; i < pathArray.length - 1; i++) {
    var currentPath = pathArray[i];
    var currentObj = getIn(obj, pathArray.slice(0, i + 1));

    if (currentObj && ((0, _isObject2["default"])(currentObj) || Array.isArray(currentObj))) {
      resVal = resVal[currentPath] = (0, _clone2["default"])(currentObj);
    } else {
      var nextPath = pathArray[i + 1];
      resVal = resVal[currentPath] = isInteger(nextPath) && Number(nextPath) >= 0 ? [] : {};
    }
  } // Return original object if new value is the same as current


  if ((i === 0 ? obj : resVal)[pathArray[i]] === value) {
    return obj;
  }

  if (value === undefined) {
    delete resVal[pathArray[i]];
  } else {
    resVal[pathArray[i]] = value;
  } // If the path array has a single element, the loop did not run.
  // Deleting on `resVal` had no effect in this scenario, so we delete on the result instead.


  if (i === 0 && value === undefined) {
    delete res[pathArray[i]];
  }

  return res;
}
/**
 * Recursively a set the same value for all keys and arrays nested object, cloning
 * @param object
 * @param value
 * @param visited
 * @param response
 */


function setNestedObjectValues(object, value) {
  var visited = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new WeakMap();
  var response = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  for (var _i = 0, _Object$keys = Object.keys(object); _i < _Object$keys.length; _i++) {
    var k = _Object$keys[_i];
    var val = object[k];

    if ((0, _isObject2["default"])(val)) {
      if (!visited.get(val)) {
        visited.set(val, true); // In order to keep array values consistent for both dot path  and
        // bracket syntax, we need to check if this is an array so that
        // this will output  { friends: [true] } and not { friends: { "0": true } }

        response[k] = Array.isArray(val) ? [] : {};
        setNestedObjectValues(val, value, visited, response[k]);
      }
    } else {
      response[k] = value;
    }
  }

  return response;
}