"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.StyledSkeleton = exports.BaseSpan = exports.BaseImg = exports.BaseDiv = void 0;

var _styledComponents = _interopRequireWildcard(require("styled-components"));

var _Skeleton = _interopRequireDefault(require("../Skeleton"));

var _skeleton = require("../styled/mixins/skeleton");

var _addPx = _interopRequireDefault(require("../utils/addPx"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function rectCss(props) {
  var $width = props.$width,
      $height = props.$height;
  var cssObj = {};

  if ($width) {
    cssObj.width = (0, _addPx["default"])($width);
  }

  if ($height) {
    cssObj.height = (0, _addPx["default"])($height);
  }

  return cssObj;
}

var AspectRatioMap = {
  '1:1': 1,
  '4:3': 3 / 4,
  '3:2': 2 / 3,
  '16:9': 9 / 16,
  '3:4': 4 / 3,
  '9:16': 16 / 9
};

function blockCss(props) {
  var $width = props.$width,
      $height = props.$height,
      $aspectRatio = props.$aspectRatio,
      $status = props.$status,
      $loadingType = props.$loadingType,
      $useDefaultImgAndIsFromTheme = props.$useDefaultImgAndIsFromTheme,
      theme = props.theme;
  var aspectCss;

  if ($aspectRatio) {
    var cssObj = {
      position: 'relative',
      height: 0
    };
    var heightRatio = AspectRatioMap[$aspectRatio];

    if (typeof $width === 'number' && $width > 0) {
      cssObj.height = Math.floor($width * heightRatio);
    } else {
      cssObj.paddingBottom = "".concat(heightRatio * 100, "%");
    }

    if ($width) {
      cssObj.width = (0, _addPx["default"])($width);
    }

    aspectCss = cssObj;
  } else if ($width !== undefined || $height !== undefined) {
    aspectCss = rectCss(props);
  }

  var loadingSkeletonCss;

  if ($status === 'none' && $loadingType === 'skeleton') {
    loadingSkeletonCss = (0, _styledComponents.css)(["position:relative;"]);
  }

  var baseCss;

  if ($useDefaultImgAndIsFromTheme) {
    var token = theme.components.Img;
    baseCss = (0, _styledComponents.css)(["background-repeat:", ";background-size:", ";"], token.errorBgRepeat, token.errorBgSize);
  } else {
    baseCss = (0, _styledComponents.css)(["background-repeat:no-repeat;background-size:cover;"]);
  }

  return (0, _styledComponents.css)(["", " ", " ", " background-position:center;transition:background-image ", "ms ", ";"], aspectCss, loadingSkeletonCss, baseCss, theme.transition.spec.duration.normal, theme.transition.pattern.easing.enter);
}

function divCss(props) {
  var $status = props.$status,
      $loadingType = props.$loadingType,
      theme = props.theme;

  if ($status === 'none' && $loadingType === 'spin') {
    return (0, _styledComponents.css)(["display:flex;align-items:center;justify-content:center;color:", ";", ""], theme.colors.spec.brand, _skeleton.skeletonNormalCss);
  }
}

function spanCss(props) {
  var $status = props.$status,
      $loadingType = props.$loadingType,
      theme = props.theme;

  if ($status === 'none' && $loadingType === 'spin') {
    return (0, _styledComponents.css)(["display:inline-flex;align-items:center;justify-content:center;color:", ";", ""], theme.colors.spec.brand, _skeleton.skeletonNormalCss);
  }

  return (0, _styledComponents.css)(["display:inline-block;"]);
}

var BaseDiv = _styledComponents["default"].div.withConfig({
  displayName: "BaseDiv"
})(["", " ", ""], blockCss, divCss);

exports.BaseDiv = BaseDiv;

var BaseImg = _styledComponents["default"].img.withConfig({
  displayName: "BaseImg"
})(["", ""], rectCss);

exports.BaseImg = BaseImg;

var BaseSpan = _styledComponents["default"].span.withConfig({
  displayName: "BaseSpan"
})(["", " ", ""], blockCss, spanCss);

exports.BaseSpan = BaseSpan;
var StyledSkeleton = (0, _styledComponents["default"])(_Skeleton["default"]).withConfig({
  displayName: "StyledSkeleton"
})(["width:100%;height:100%;"]);
exports.StyledSkeleton = StyledSkeleton;