"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createSourceImg = createSourceImg;
exports.cropImg = cropImg;
exports.getImgStyle = getImgStyle;
exports.getNewTranslate = getNewTranslate;
exports.getRangeOffset = getRangeOffset;
exports.rotateIsOdd = rotateIsOdd;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

// 旋转是 90 的奇数倍
function rotateIsOdd(rotate) {
  return Math.floor(rotate / 90) % 2 !== 0;
}

function getImgStyle(size, rotate, img) {
  var style = {};

  if (!img) {
    return style;
  }

  var switchWidthHeight = rotateIsOdd(rotate);
  var width = switchWidthHeight ? size[1] : size[0];
  var height = switchWidthHeight ? size[0] : size[1];
  var imgWidth = img.naturalWidth;
  var imgHeight = img.naturalHeight;
  var boxRatio = width / height;
  var imgRatio = imgWidth / imgHeight;

  if (imgRatio > boxRatio) {
    style.height = height;
    var boxWidth = imgRatio * height;
    style.width = boxWidth;
  } else {
    style.width = width;
    var boxHeight = width / imgRatio;
    style.height = boxHeight;
  }

  style.top = (size[1] - style.height) / 2;
  style.left = (size[0] - style.width) / 2;
  return style;
}
/**
 * 判断两个区间是否有重叠，有重叠返回 A -> B 的偏移量
 * @param rangeA
 * @param rangeB
 */


function getRangeOffset(rangeA, rangeB) {
  var _rangeA = (0, _slicedToArray2["default"])(rangeA, 2),
      startA = _rangeA[0],
      endA = _rangeA[1];

  var _rangeB = (0, _slicedToArray2["default"])(rangeB, 2),
      startB = _rangeB[0],
      endB = _rangeB[1];

  if (startA > startB) {
    return startA - startB;
  }

  if (endA < endB) {
    return endA - endB;
  }

  return 0;
}

function getNewTranslate(maskRect, imgRect, translate) {
  var topOffset = getRangeOffset([imgRect.top, imgRect.bottom], [maskRect.top, maskRect.bottom]);
  var leftOffset = getRangeOffset([imgRect.left, imgRect.right], [maskRect.left, maskRect.right]);

  if (topOffset === 0 && leftOffset === 0) {
    return;
  }

  var newTranslate = [translate[0] - leftOffset, translate[1] - topOffset];
  return newTranslate;
} // 临时使用


// 后面有 canvas 的不好再单测里面跑

/* istanbul ignore next */

/**
 * 创建原图旋转后的 canvas
 * @param img 图片实例
 * @param imgRect 图片的位置信息
 * @param rotate 旋转
 * @param ratio 缩放比例
 */
function createSourceImg(img, imgRect, rotate) {
  var ratio = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;
  var canvas = document.createElement('canvas');
  var canvasWidth = imgRect.width * ratio;
  var canvasHeight = imgRect.height * ratio;
  canvas.width = canvasWidth;
  canvas.height = canvasHeight;
  var x = canvasWidth / 2;
  var y = canvasHeight / 2;
  var ctx = canvas.getContext('2d');
  var drawWidth = canvasWidth;
  var drawHeight = canvasHeight;

  if (rotateIsOdd(rotate)) {
    drawWidth = canvasHeight;
    drawHeight = canvasWidth;
  }

  ctx.translate(x, y);
  ctx.rotate(rotate * Math.PI / 180);
  ctx.drawImage(img, -drawWidth / 2, -drawHeight / 2, drawWidth, drawHeight);
  return canvas;
}
/**
 * 生成剪裁后的 canvas
 * @param img 图片实例
 * @param imgRect 图片的位置信息
 * @param maskRect 遮罩的位置
 * @param rotate 旋转
 * @param ratio 缩放比例
 */


function cropImg(img, imgRect, maskRect, rotate) {
  var ratio = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 1;
  // 先把原图的 canvas 画出来
  var canvas = document.createElement('canvas');
  var canvasWidth = maskRect.width * ratio;
  var canvasHeight = maskRect.height * ratio;
  canvas.width = canvasWidth;
  canvas.height = canvasHeight;
  var dx = (maskRect.left - imgRect.left) * ratio;
  var dy = (maskRect.top - imgRect.top) * ratio;
  var sourceImg = createSourceImg(img, imgRect, rotate, ratio);
  canvas.getContext('2d').drawImage(sourceImg, dx, dy, canvasWidth, canvasHeight, 0, 0, canvasWidth, canvasHeight);
  return canvas;
}