"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ExtraNodes = ExtraNodes;
exports.StyledTextareaWrapper = exports.StyledTextareaLimitWrapper = exports.StyledTextarea = exports.StyledTagsWrapper = exports.StyledTagsInputWrapper = exports.StyledTagsInputSuffixNodeWrapper = exports.StyledTagsInputContentWrapper = exports.StyledTagsInput = exports.StyledStatusWrapper = exports.StyledPopperInputWrapper = exports.StyledNodeWrapper = exports.StyledInputWrapper = exports.StyledInputSizer = exports.StyledInput = exports.StyledDivider = void 0;
exports.makeInputStyle = makeInputStyle;
exports.makeInputWrapperStyle = makeInputWrapperStyle;
exports.makeStatusWrapperStyle = makeStatusWrapperStyle;
exports.makeTextareaLimitWrapper = makeTextareaLimitWrapper;
exports.makeTextareaStyle = makeTextareaStyle;
exports.makeWrapperStyle = makeWrapperStyle;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _isInteger2 = _interopRequireDefault(require("lodash/isInteger"));

var _react = _interopRequireWildcard(require("react"));

var _styledComponents = _interopRequireWildcard(require("styled-components"));

var _styled = require("../ScrollView/styled");

var _ellipsis = _interopRequireDefault(require("../styled/mixins/ellipsis"));

var _addPx = _interopRequireDefault(require("../utils/addPx"));

var _InputTag = _interopRequireDefault(require("./InputTag"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function makeWrapperStyle(props) {
  var theme = props.theme,
      size = props.size,
      disabled = props.disabled,
      focus = props.focus,
      $width = props.$width,
      $height = props.$height,
      $bordered = props.$bordered,
      hasError = props.hasError;
  var colorsPattern = theme.colors.pattern,
      _theme$typography$spe = theme.typography.spec,
      fontSize = _theme$typography$spe.fontSize,
      lineHeight = _theme$typography$spe.lineHeight,
      _theme$transition$pat = theme.transition.pattern,
      easing = _theme$transition$pat.easing,
      duration = _theme$transition$pat.duration,
      token = theme.components.Input;
  var fontLevel = token.fontLevel[size]; // 错误样式

  var errorStyle = (0, _styledComponents.css)(["border-color:", ";box-shadow:", ";"], colorsPattern.feature.error, token.errorFocusShadow); // 聚焦样式

  var focusStyle = (0, _styledComponents.css)(["border-color:", ";box-shadow:", ";"], token.borderColor.focused, token.focusShadow); // 禁用样式

  var disabledStyle = (0, _styledComponents.css)(["background:", ";cursor:not-allowed;input,svg{cursor:not-allowed;}"], colorsPattern.background.disabled);
  var widthStyle = $width && (0, _styledComponents.css)(["width:", ";"], $width);
  var heightStyle = $height && (0, _styledComponents.css)(["height:", ";"], $height);
  var borderStyle = $bordered && (0, _styledComponents.css)(["border:1px solid ", ";"], token.borderColor.normal);
  return (0, _styledComponents.css)(["display:inline-flex;position:relative;box-sizing:border-box;align-items:center;font-size:", "px;line-height:", "px;border-radius:", ";", " transition:border ", "ms ", ",width ", "ms ", ";vertical-align:middle;background-color:", ";&:hover{border-color:", ";}", " ", " ", " ", " ", ""], fontSize[fontLevel], lineHeight[fontLevel], token.borderRadius[size], borderStyle, duration.status, easing.status, duration.status, easing.status, token.bgColor, !(focus || disabled || hasError) && token.borderColor.hover, widthStyle, heightStyle, focus && focusStyle, hasError && errorStyle, disabled && disabledStyle);
}

function makeInputWrapperStyle(props) {
  var theme = props.theme,
      size = props.size;
  var _theme$components$Inp = theme.components.Input,
      inputPadding = _theme$components$Inp.inputPadding,
      inputWidth = _theme$components$Inp.inputWidth,
      inputHeight = _theme$components$Inp.inputHeight;
  return (0, _styledComponents.css)(["width:", "px;height:", "px;padding:0 ", "px;"], inputWidth[size], inputHeight[size], inputPadding[size]);
}

function makeTextareaStyle(props) {
  var theme = props.theme,
      size = props.size,
      limit = props.limit,
      _props$resize = props.resize,
      resize = _props$resize === void 0 ? 'vertical' : _props$resize,
      $height = props.$height,
      autosize = props.autosize,
      $width = props.$width;
  var _theme$typography$spe2 = theme.typography.spec,
      fontSize = _theme$typography$spe2.fontSize,
      lineHeight = _theme$typography$spe2.lineHeight,
      token = theme.components.Input;
  var heightStyle = $height && (0, _styledComponents.css)(["height:", ";"], $height);
  var fontLevel = token.fontLevel[size];
  var padding = autosize ? token.autosizeTextareaPadding : token.textareaPadding;
  var currentPadding = padding[size];
  var space = currentPadding[0];

  switch (currentPadding.length) {
    case 1:
    case 2:
      space *= 2;
      break;

    default:
      space += currentPadding[2];
      break;
  }

  var width = $width ? $width : "".concat(token.textareaWidth[size], "px");
  var limitStyle = limit && limit > 0 && (0, _styledComponents.css)(["::-webkit-scrollbar{width:0;}border-bottom:", "px solid transparent;padding-bottom:0;-ms-overflow-style:none;resize:none;"], token.textareaLimitMarginBottom[size]);
  return (0, _styledComponents.css)(["resize:", ";width:", ";height:", "px;font-size:", "px;line-height:", "px;min-height:", "px;padding:", ";box-sizing:border-box;", " ", ""], resize, width, token.textareaHeight[size], fontSize[fontLevel], lineHeight[fontLevel], lineHeight[fontLevel] + space, currentPadding.map(function (p) {
    return "".concat(p, "px ");
  }), heightStyle, limitStyle);
}

function makeInputStyle(props) {
  var _props$theme = props.theme,
      colors = _props$theme.colors,
      _props$theme$typograp = _props$theme.typography.spec,
      fontSize = _props$theme$typograp.fontSize,
      lineHeight = _props$theme$typograp.lineHeight,
      token = _props$theme.components.Input,
      disabled = props.disabled,
      size = props.size;
  var fontLevel = token.fontLevel[size];
  return (0, _styledComponents.css)(["display:inline-block;outline:none;border:0;padding:0;margin:0;flex:1;min-width:0;background:none;font-size:", "px;line-height:", "px;caret-color:", ";color:", ";", " &::selection{background:", ";}&::placeholder{color:", ";}"], fontSize[fontLevel], lineHeight[fontLevel], colors.spec.brand, disabled ? colors.pattern.text.disabled : token.color, disabled && (0, _styledComponents.css)(["-webkit-text-fill-color:currentColor;opacity:1;"]), token.selectionBackground, colors.pattern.text.placeholder);
}

function makeStatusWrapperStyle(props) {
  var theme = props.theme,
      status = props.status;

  if (status) {
    return (0, _styledComponents.css)(["line-height:0;cursor:default;color:", ";"], theme.colors.pattern.feature[status]);
  }
}

function makeTextareaLimitWrapper(props) {
  var theme = props.theme,
      size = props.size;
  var _theme$components$Inp2 = theme.components.Input.textareaLimitPosition[size],
      right = _theme$components$Inp2.right,
      bottom = _theme$components$Inp2.bottom;
  return (0, _styledComponents.css)(["position:absolute;right:", "px;bottom:", "px;"], right, bottom);
}

var StyledInputWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledInputWrapper"
})(["", " ", ""], makeInputWrapperStyle, makeWrapperStyle);

exports.StyledInputWrapper = StyledInputWrapper;

var StyledTextareaWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledTextareaWrapper"
})(["", ""], makeWrapperStyle);

exports.StyledTextareaWrapper = StyledTextareaWrapper;

var StyledInput = _styledComponents["default"].input.withConfig({
  displayName: "StyledInput"
})(["", ""], makeInputStyle);

exports.StyledInput = StyledInput;

var StyledTextarea = _styledComponents["default"].textarea.withConfig({
  displayName: "StyledTextarea"
})(["", " ", ""], makeInputStyle, makeTextareaStyle);

exports.StyledTextarea = StyledTextarea;

function makeAddonNodeMarginStyle(props) {
  var theme = props.theme,
      size = props.size,
      $type = props.$type,
      $small = props.$small;
  var spacing = theme.components.Input.inputPadding[size];
  var left = theme.components.Input.inputAddonNodeMargin[size];
  var marginRight = spacing;
  var marginLeft = spacing;

  if ($small === 'prefix') {
    marginRight = left;
  } else if ($small === 'suffix') {
    marginLeft = left;
  }

  return $type === 'prefix' ? (0, _styledComponents.css)(["margin-right:", "px;"], marginRight) : (0, _styledComponents.css)(["margin-left:", "px;"], marginLeft);
}

var StyledNodeWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledNodeWrapper"
})(["", ""], function (props) {
  var theme = props.theme,
      $disabled = props.$disabled,
      children = props.children;
  var token = theme.components.Input;
  var _theme$colors$pattern = theme.colors.pattern.text,
      darktip = _theme$colors$pattern.darktip,
      secondary = _theme$colors$pattern.secondary,
      assistant = _theme$colors$pattern.assistant; // 字符串节点固定颜色。React节点有特殊颜色

  var elementStyle = typeof children !== 'string' && (0, _styledComponents.css)(["color:", ";&:hover{color:", ";}&:active{color:", ";}cursor:", ";"], darktip, secondary, assistant, $disabled ? 'not-allowed' : 'pointer');
  return (0, _styledComponents.css)(["", " line-height:0;color:", ";flex-shrink:0;outline:none;cursor:default;user-select:none;display:flex;height:100%;align-items:center;", ""], makeAddonNodeMarginStyle(props), token.color, elementStyle);
});

exports.StyledNodeWrapper = StyledNodeWrapper;

var StyledDivider = _styledComponents["default"].div.withConfig({
  displayName: "StyledDivider"
})(["", ""], function (props) {
  var theme = props.theme,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size;
  var fontLevel = theme.components.Input.fontLevel[size];
  return (0, _styledComponents.css)(["", " width:1px;height:", "px;flex-shrink:0;cursor:default;user-select:none;background:", ";"], makeAddonNodeMarginStyle(props), theme.typography.spec.fontSize[fontLevel], theme.colors.pattern.background.divider);
});

exports.StyledDivider = StyledDivider;

function ExtraNodes(props) {
  var nodes = props.nodes,
      $type = props.$type,
      size = props.size,
      theme = props.theme,
      full = props.full,
      $nodeStyleItem = props.$nodeStyleItem,
      $dividerStyleItem = props.$dividerStyleItem,
      $disabled = props.$disabled;
  return (0, _react.useMemo)(function () {
    if (!nodes) {
      return null;
    }

    var childNum = _react.Children.count(nodes);

    return /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, _react.Children.map(nodes, function (child, index) {
      if (!child) return null;
      var $small;

      if (!full && index === childNum - 1 && $type === 'prefix') {
        $small = 'prefix';
      }

      return /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, {
        key: index
      }, index > 0 && /*#__PURE__*/_react["default"].createElement(StyledDivider, (0, _extends2["default"])({
        $type: $type,
        $disabled: $disabled,
        theme: theme,
        size: size
      }, $dividerStyleItem)), /*#__PURE__*/_react["default"].createElement(StyledNodeWrapper, (0, _extends2["default"])({
        $type: $type,
        theme: theme,
        size: size
      }, $nodeStyleItem, {
        $small: $small,
        $disabled: $disabled
      }), child));
    }), full && /*#__PURE__*/_react["default"].createElement(StyledNodeWrapper, (0, _extends2["default"])({
      $type: $type,
      $disabled: $disabled,
      theme: theme,
      size: size
    }, $nodeStyleItem)));
  }, [$disabled, $dividerStyleItem, $nodeStyleItem, $type, full, nodes, size, theme]);
}

var StyledStatusWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledStatusWrapper"
})(["", ""], makeStatusWrapperStyle);

exports.StyledStatusWrapper = StyledStatusWrapper;

var StyledTextareaLimitWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledTextareaLimitWrapper"
})(["", ""], makeTextareaLimitWrapper); // tags input


exports.StyledTextareaLimitWrapper = StyledTextareaLimitWrapper;

var StyledTagsInputContentWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledTagsInputContentWrapper"
})(["position:relative;box-sizing:border-box;overflow-x:hidden;overflow-y:auto;outline:none;"]);

exports.StyledTagsInputContentWrapper = StyledTagsInputContentWrapper;

var StyledTagsWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledTagsWrapper"
})(["align-self:flex-start;display:flex;flex-wrap:wrap;align-items:center;"]);

exports.StyledTagsWrapper = StyledTagsWrapper;

var StyledTagsInput = _styledComponents["default"].input.withConfig({
  displayName: "StyledTagsInput"
})(["display:inline-block;overflow:hidden;background:transparent;border-width:0;margin:0;padding:0;outline:0;margin-bottom:4px;flex-shrink:0;"]);

exports.StyledTagsInput = StyledTagsInput;

var StyledInputSizer = _styledComponents["default"].div.withConfig({
  displayName: "StyledInputSizer"
})(["position:absolute;top:0;left:0;visibility:hidden;height:0;overflow:scroll;white-space:pre;line-height:1;"]);

exports.StyledInputSizer = StyledInputSizer;

var StyledTagsInputSuffixNodeWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledTagsInputSuffixNodeWrapper"
})([""]); // TODO: 样式重构
// eslint-disable-next-line complexity


exports.StyledTagsInputSuffixNodeWrapper = StyledTagsInputSuffixNodeWrapper;

var tagsInputCss = function tagsInputCss(props) {
  var _props$theme2 = props.theme,
      colorsPattern = _props$theme2.colors.pattern,
      fontSize = _props$theme2.typography.spec.fontSize,
      sizeSpec = _props$theme2.size.spec,
      _props$theme2$transit = _props$theme2.transition.pattern,
      duration = _props$theme2$transit.duration,
      easing = _props$theme2$transit.easing,
      token = _props$theme2.components.Input,
      $size = props.$size,
      $width = props.$width,
      $focus = props.$focus,
      $disabled = props.$disabled,
      $hasValue = props.$hasValue,
      $maxVerticalTagCount = props.$maxVerticalTagCount,
      $maxTagWidth = props.$maxTagWidth,
      $hasError = props.$hasError,
      $horizontalScroll = props.$horizontalScroll,
      $bordered = props.$bordered;
  var hasStatus = $focus || $disabled || $hasError;
  var finalWidth = $width || sizeSpec.width[$size];
  var computedMaxTagWidth = $maxTagWidth;

  if (!$maxTagWidth && typeof finalWidth === 'number') {
    computedMaxTagWidth = finalWidth - token.tagsInput.contentPaddingRight[$size] - token.inputTag.outerPadding * 2;
  }

  var fontLevel = token.fontLevel[$size];
  var disabledCss = (0, _styledComponents.css)(["background:", ";border:1px solid ", ";cursor:not-allowed;"], colorsPattern.background.disabled, token.borderColor.normal); // 错误样式

  var errorStyle = (0, _styledComponents.css)(["border-color:", ";box-shadow:", ";"], colorsPattern.feature.error, $focus ? token.errorFocusShadow : 'none');
  var borderStyle = $bordered && (0, _styledComponents.css)(["border:1px solid ", ";"], $focus ? token.borderColor.focused : token.borderColor.normal);
  return (0, _styledComponents.css)(["position:relative;display:inline-flex;overflow:hidden;outline:none;box-sizing:border-box;width:", ";", " box-shadow:", ";border-radius:", ";background:", ";line-height:1;font-size:", "px;transition:border ", "ms ", ";cursor:text;&:hover{border-color:", ";}", ";", ";", " ", " ", "{padding-left:", "px;padding-right:", "px;", ";", ";", ";}", "{", ";}", "{width:1px;color:", ";font-size:", "px;}input{&[readonly]{cursor:", ";}&::placeholder{color:", ";}}", "{font-size:", "px;}", "{", " ", ";}", "{max-width:", "px;}"], (0, _addPx["default"])(finalWidth), borderStyle, $focus ? token.focusShadow : 'none', token.borderRadius[$size], token.bgColor, fontSize[fontLevel], duration.status, easing.status, !hasStatus && token.borderColor.hover, $hasValue && (0, _styledComponents.css)(["padding:", ";"], token.tagsInput.wrapperPadding), $horizontalScroll && (0, _styledComponents.css)(["justify-content:space-between;"]), $disabled && disabledCss, $hasError && errorStyle, StyledTagsInputContentWrapper, token.inputPadding[$size], token.tagsInput.contentPaddingRight[$size], $hasValue && (0, _styledComponents.css)(["max-height:", "px;padding-left:0;"], $maxVerticalTagCount * (token.inputHeight[$size] - 10) + Math.ceil($maxVerticalTagCount - 1) * 4 + ((0, _isInteger2["default"])($maxVerticalTagCount) ? 5 : 0)), $horizontalScroll && (0, _styledComponents.css)(["padding-right:0;overflow-x:auto;"]), (0, _styled.scrollBarStyle)({
    theme: props.theme,
    hideEndButton: true,
    hidden: $horizontalScroll
  }), StyledTagsInputSuffixNodeWrapper, $horizontalScroll ? (0, _styledComponents.css)(["margin:0 ", "px ", "px ", "px;color:", ";flex-shrink:0;outline:none;cursor:default;user-select:none;display:inline-flex;align-items:center;line-height:1;"], $hasValue ? token.inputPadding[$size] - 4 : token.inputPadding[$size], $hasValue ? 4 : 0, token.inputPadding[$size], token.color) : (0, _styledComponents.css)(["position:absolute;top:50%;right:", "px;transform:translateY(-50%);margin:auto;"], token.inputPadding[$size]), StyledTagsInput, colorsPattern.text.text, fontSize[fontLevel], $disabled ? 'not-allowed' : 'pointer', colorsPattern.text.placeholder, StyledInputSizer, fontSize[fontLevel], StyledTagsWrapper, !$hasValue && (0, _styledComponents.css)(["height:", "px;", "{margin-bottom:0;}"], token.inputHeight[$size] - 2, StyledTagsInput), $horizontalScroll && "flex-wrap: nowrap", _InputTag["default"], computedMaxTagWidth);
};

var StyledTagsInputWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledTagsInputWrapper"
})(["", ""], tagsInputCss);

exports.StyledTagsInputWrapper = StyledTagsInputWrapper;

var StyledPopperInputWrapper = _styledComponents["default"].div.withConfig({
  displayName: "StyledPopperInputWrapper"
})(["position:relative;display:inline-block;outline:none;", "{", ";}", ""], StyledInput, _ellipsis["default"], function (props) {
  var $pointer = props.$pointer,
      $popupVisible = props.$popupVisible,
      $width = props.$width,
      $status = props.$status,
      $bordered = props.$bordered,
      theme = props.theme;
  var openStyle;
  var widthStyle;
  var pointerStyle;
  var borderStyle;

  if ($popupVisible && $status !== 'error') {
    openStyle = (0, _styledComponents.css)(["", ",", "{border-color:", ";}"], StyledTagsInputWrapper, StyledInputWrapper, theme.components.Input.borderColor.focused);
  }

  if ($width) {
    widthStyle = (0, _styledComponents.css)(["width:", ";"], $width);
  }

  if ($pointer) {
    pointerStyle = (0, _styledComponents.css)(["", ",", "{cursor:pointer;input,svg{cursor:pointer;}}"], StyledInputWrapper, StyledTagsInputWrapper);
  }

  if ($bordered === false) {
    borderStyle = (0, _styledComponents.css)(["", ",", "{border:none;}"], StyledInputWrapper, StyledTagsInputWrapper);
  }

  return (0, _styledComponents.css)(["", ";", ";", ";", ";"], widthStyle, openStyle, pointerStyle, borderStyle);
});

exports.StyledPopperInputWrapper = StyledPopperInputWrapper;