"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useInput;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _useForkRef2 = require("@muya-ui/utils/lib/hooks/useForkRef");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = require("react");

var _resizeObserverPolyfill = _interopRequireDefault(require("resize-observer-polyfill"));

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _calculateNodeHeight = _interopRequireDefault(require("./calculateNodeHeight"));

var _useInputBase = _interopRequireDefault(require("./useInputBase"));

function useInput(props, isControlled) {
  var _props$defaultValue = props.defaultValue,
      defaultValue = _props$defaultValue === void 0 ? '' : _props$defaultValue,
      _props$value = props.value,
      value = _props$value === void 0 ? '' : _props$value,
      _props$autoFocus = props.autoFocus,
      autoFocus = _props$autoFocus === void 0 ? false : _props$autoFocus,
      disabled = props.disabled,
      inputRefProp = props.inputRef,
      onClear = props.onClear,
      onChange = props.onChange,
      onClick = props.onClick,
      onKeyDown = props.onKeyDown,
      onPressEnter = props.onPressEnter,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      focusWhenClear = props.focusWhenClear,
      multiline = props.multiline,
      autosize = props.autosize,
      minRows = props.minRows,
      maxRows = props.maxRows,
      formatter = props.formatter,
      parser = props.parser;

  var _useState = (0, _react.useState)(defaultValue),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      innerValue = _useState2[0],
      setInnerValue = _useState2[1];

  var _useState3 = (0, _react.useState)(defaultValue),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      showValue = _useState4[0],
      setShowValue = _useState4[1];

  var inputRef = (0, _react.useRef)(null);
  var handleInputRef = (0, _useForkRef2.useForkRef)(inputRef, inputRefProp);
  var observerRef = (0, _react.useRef)();

  var _useState5 = (0, _react.useState)({}),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      autosizeStyles = _useState6[0],
      setTextareaStyles = _useState6[1];

  var baseResult = (0, _useInputBase["default"])({
    autoFocus: autoFocus,
    disabled: disabled,
    onFocus: onFocus,
    onBlur: onBlur,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave
  });
  var handleClick = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function (e) {
      if (inputRef.current && e.currentTarget === e.target) {
        inputRef.current.focus();
      }
    }, onClick);
  }, [onClick]);
  var setInputValue = (0, _react.useCallback)(function (value, e) {
    var isBlur = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    var newValue = value;

    if (!isBlur) {
      newValue = parser ? parser(value) : value;
      setShowValue(value);

      if (!isControlled) {
        setInnerValue(newValue);
      }
    }

    if (onChange) {
      var event = e;

      if (e.type === 'click') {
        // click clear icon
        var inputTarget = inputRef.current;
        event = Object.create(e);
        event.target = inputTarget;
        event.currentTarget = inputTarget;
        var originalInputValue = inputTarget.value; // change input value cause e.target.value should be '' when clear input

        inputTarget.value = '';
        onChange(event); // reset input value

        inputTarget.value = originalInputValue;
        return;
      } else {
        // @ts-ignore
        event.target.value = value;
      }

      onChange(event);
    }
  }, [isControlled, onChange, parser]);
  var resizeTextarea = (0, _react.useCallback)(function () {
    if (autosize && multiline && inputRef.current) {
      setTextareaStyles((0, _calculateNodeHeight["default"])(inputRef.current, isControlled, minRows, maxRows));
    }
  }, [autosize, isControlled, maxRows, minRows, multiline]);
  var handleChange = (0, _react.useCallback)(function (e) {
    setInputValue(e.target.value, e);
  }, [setInputValue]);
  var handleClear = (0, _react.useCallback)(function (e) {
    e.stopPropagation();
    setInputValue('', e);

    if (onClear) {
      onClear(e);
    } // 重新聚焦的动作等清楚动作完成后再聚焦，不然会导致点一下没有办法清空值


    setTimeout(function () {
      if (focusWhenClear && inputRef.current) {
        inputRef.current.focus();
      }
    }, 0);
  }, [focusWhenClear, onClear, setInputValue]);
  var handleKeyDown = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function (e) {
      if (e.keyCode === 13 && onPressEnter) {
        onPressEnter(e);
      }
    }, onKeyDown);
  }, [onKeyDown, onPressEnter]);
  var handleFocus = (0, _react.useCallback)(function (e) {
    var val = isControlled ? value : innerValue;
    var showVal = formatter ? formatter(val) : val;
    setShowValue(showVal);
    baseResult.handleFocus(e);
  }, [baseResult, formatter, innerValue, isControlled, value]);
  var handleBlur = (0, _react.useCallback)(function (e) {
    if (parser) {
      setInputValue(parser(showValue), e, true);
    }

    baseResult.handleBlur(e);
  }, [baseResult, parser, setInputValue, showValue]);

  if (disabled) {
    inputRef.current && inputRef.current.blur();
  }

  (0, _react.useEffect)(function () {
    // textarea autosize
    if (!observerRef.current && inputRef.current && multiline && autosize) {
      observerRef.current = new _resizeObserverPolyfill["default"](resizeTextarea);
      observerRef.current.observe(inputRef.current);
    }

    return function () {
      if (observerRef.current) {
        observerRef.current.disconnect();
        observerRef.current = undefined;
      }
    };
  }, [autosize, multiline, resizeTextarea]);
  (0, _react.useEffect)(function () {
    resizeTextarea();
  }, [resizeTextarea]);
  (0, _react.useEffect)(function () {
    // formatter 函数存在且受控的情况下，只有失焦的时候才更新showValue
    if (isControlled && formatter && !baseResult.focus) {
      setShowValue(formatter(value));
    } else if (isControlled && !formatter) {
      // 其他受控情况下直接更新值即可
      setShowValue(value);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [value]);
  var currentValue = isControlled ? value : innerValue;
  var finalValue = baseResult.focus ? showValue : formatter ? formatter(currentValue) : currentValue;
  (0, _react.useEffect)(function () {
    resizeTextarea(); // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [finalValue]);
  return (0, _extends2["default"])({}, baseResult, {
    finalValue: finalValue,
    handleClick: handleClick,
    handleChange: handleChange,
    handleClear: handleClear,
    handleKeyDown: handleKeyDown,
    handleInputRef: handleInputRef,
    handleFocus: handleFocus,
    handleBlur: handleBlur,
    autosizeStyles: autosizeStyles
  });
}