"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useRangeInput;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = require("react");

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _useInputBase = _interopRequireDefault(require("./useInputBase"));

var _utils = require("./utils");

function useRangeInput(props) {
  var autoFocus = props.autoFocus,
      disabled = props.disabled,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      onClick = props.onClick,
      value = props.value,
      defaultValue = props.defaultValue,
      onChange = props.onChange,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      focusWhenClear = props.focusWhenClear,
      onKeyDown = props.onKeyDown,
      onPressEnter = props.onPressEnter,
      onClear = props.onClear;
  var leftInputRef = (0, _react.useRef)(null);
  var rightInputRef = (0, _react.useRef)(null);
  var baseResult = (0, _useInputBase["default"])({
    autoFocus: autoFocus,
    disabled: disabled,
    onBlur: onBlur,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave
  });
  var valueFromProps = (0, _react.useMemo)(function () {
    if (value) {
      return (0, _utils.getRangeDefaultValue)(value);
    }

    if (defaultValue) {
      return (0, _utils.getRangeDefaultValue)(defaultValue);
    }

    return ['', ''];
  }, [defaultValue, value]);

  var _useState = (0, _react.useState)(valueFromProps),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      innerValue = _useState2[0],
      setInnerValue = _useState2[1];

  var finalValue = 'value' in props ? valueFromProps : innerValue;
  var handleClick = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function (e) {
      if (leftInputRef.current && e.currentTarget === e.target) {
        leftInputRef.current.focus();
      }
    }, onClick);
  }, [onClick]);
  var updateValue = (0, _react.useCallback)(function (newValue) {
    setInnerValue(newValue);

    if (onChange) {
      // TODO: 把
      onChange({
        target: {
          type: 'range-input'
        },
        value: newValue
      });
    }
  }, [onChange]);
  var setInputValue = (0, _react.useCallback)(function (v, inputPos) {
    var _finalValue = (0, _slicedToArray2["default"])(finalValue, 2),
        oldLeft = _finalValue[0],
        oldRight = _finalValue[1];

    var newValue = [oldLeft, oldRight];
    newValue[inputPos] = v;
    updateValue(newValue);
  }, [finalValue, updateValue]);
  var handleLeftChange = (0, _react.useCallback)(function (e) {
    setInputValue(e.target.value, 0);
  }, [setInputValue]);
  var handleRightChange = (0, _react.useCallback)(function (e) {
    setInputValue(e.target.value, 1);
  }, [setInputValue]);

  if (disabled) {
    leftInputRef.current && leftInputRef.current.blur();
    rightInputRef.current && rightInputRef.current.blur();
  }

  var handleClear = (0, _react.useCallback)(function (e) {
    e.stopPropagation();
    updateValue(['', '']);

    if (focusWhenClear && leftInputRef.current) {
      leftInputRef.current.focus();
    }

    if (onClear) {
      onClear({
        value: ['', '']
      });
    }
  }, [focusWhenClear, onClear, updateValue]);
  var handleKeyDown = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function (e) {
      if (e.keyCode === 13 && onPressEnter) {
        onPressEnter({
          value: finalValue
        });
      }
    }, onKeyDown);
  }, [finalValue, onKeyDown, onPressEnter]);
  var handleLeftFocus = (0, _react.useCallback)(function (e) {
    onFocus && onFocus(e, 0);
  }, [onFocus]);
  var handleRightFocus = (0, _react.useCallback)(function (e) {
    onFocus && onFocus(e, 1);
  }, [onFocus]);
  return (0, _extends2["default"])({}, baseResult, {
    finalValue: finalValue,
    leftInputRef: leftInputRef,
    rightInputRef: rightInputRef,
    handleClick: handleClick,
    handleLeftChange: handleLeftChange,
    handleRightChange: handleRightChange,
    handleClear: handleClear,
    handleKeyDown: handleKeyDown,
    handleLeftFocus: handleLeftFocus,
    handleRightFocus: handleRightFocus
  });
}