"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useTagsInput;
exports.isTagDisabled = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _usePrevious2 = require("@muya-ui/utils/lib/hooks/usePrevious");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _findLastIndex2 = _interopRequireDefault(require("lodash/findLastIndex"));

var _findIndex2 = _interopRequireDefault(require("lodash/findIndex"));

var _react = require("react");

var _shallowEqual = require("shallow-equal");

var _keyCode = require("../utils/keyCode");

var _useInputBase = _interopRequireDefault(require("./useInputBase"));

var isTagDisabled = function isTagDisabled(disabledValue, value) {
  return (0, _findIndex2["default"])(disabledValue, function (v) {
    return value && (0, _typeof2["default"])(value) === 'object' ? value.value === v : value === v;
  }) >= 0;
};

exports.isTagDisabled = isTagDisabled;

function useTagsInput(props, contentRef, sizerRef, inputRef) {
  var _props$allowAdd = props.allowAdd,
      allowAdd = _props$allowAdd === void 0 ? false : _props$allowAdd,
      _props$autoFocus = props.autoFocus,
      autoFocus = _props$autoFocus === void 0 ? false : _props$autoFocus,
      _props$focusWhenClear = props.focusWhenClear,
      focusWhenClear = _props$focusWhenClear === void 0 ? true : _props$focusWhenClear,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      _props$scrollTopAfter = props.scrollTopAfterEffect,
      scrollTopAfterEffect = _props$scrollTopAfter === void 0 ? false : _props$scrollTopAfter,
      _props$allowInput = props.allowInput,
      allowInput = _props$allowInput === void 0 ? true : _props$allowInput,
      _props$value = props.value,
      value = _props$value === void 0 ? [] : _props$value,
      _props$defaultValue = props.defaultValue,
      defaultValue = _props$defaultValue === void 0 ? [] : _props$defaultValue,
      _props$disabledValue = props.disabledValue,
      disabledValue = _props$disabledValue === void 0 ? [] : _props$disabledValue,
      _props$inputValue = props.inputValue,
      inputValue = _props$inputValue === void 0 ? '' : _props$inputValue,
      _props$defaultInputVa = props.defaultInputValue,
      defaultInputValue = _props$defaultInputVa === void 0 ? '' : _props$defaultInputVa,
      maxTagTextLength = props.maxTagTextLength,
      _props$horizontalScro = props.horizontalScroll,
      horizontalScroll = _props$horizontalScro === void 0 ? false : _props$horizontalScro,
      placeholder = props.placeholder,
      status = props.status,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      onClick = props.onClick,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      onChange = props.onChange,
      onClear = props.onClear,
      onRemoveTag = props.onRemoveTag,
      onAddTag = props.onAddTag,
      onInputChange = props.onInputChange,
      onInput = props.onInput,
      onInputKeyDown = props.onInputKeyDown;
  var baseResult = (0, _useInputBase["default"])({
    autoFocus: autoFocus,
    disabled: disabled,
    onFocus: onFocus,
    onBlur: onBlur,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave
  });

  var _useRef = (0, _react.useRef)('value' in props),
      isControlled = _useRef.current;

  var _useRef2 = (0, _react.useRef)('inputValue' in props),
      isInputControlled = _useRef2.current;

  var _useState = (0, _react.useState)(isControlled ? value : defaultValue),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      valueState = _useState2[0],
      setValueState = _useState2[1];

  var _useState3 = (0, _react.useState)(isInputControlled ? inputValue : defaultInputValue),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      inputValueState = _useState4[0],
      setInputValueState = _useState4[1];

  var preValueState = (0, _usePrevious2.usePrevious)(valueState);
  var hasValue = valueState && valueState.length > 0;
  var hasError = status === 'error';
  var finalPlaceholder = !inputValue && !hasValue ? placeholder : undefined;

  var handleInputChange = function handleInputChange(value) {
    if (!isInputControlled) {
      setInputValueState(value);
    }

    if (onInputChange) {
      onInputChange(value);
    }
  };

  var handleClick = function handleClick(e) {
    e.stopPropagation();

    if (onClick) {
      onClick(e);
    }

    inputRef.current && inputRef.current.focus();
  };

  var handleClear = function handleClear(e) {
    e.stopPropagation(); // 清空 tag

    if (!isControlled) {
      setValueState([]);
    }

    if (onChange) {
      onChange([]);
    }

    if (onClear) {
      onClear(e);
    } // 清空 input


    handleInputChange('');

    if (focusWhenClear && inputRef.current) {
      inputRef.current.focus();
    }
  };

  var handleInput = function handleInput(e) {
    var newInputValue = e.target.value;

    if (allowInput && (typeof maxTagTextLength === 'undefined' || newInputValue.length <= maxTagTextLength)) {
      handleInputChange(newInputValue);
    }

    if (onInput) {
      onInput(e);
    }
  };

  var handleInputKeyDown = function handleInputKeyDown(e) {
    if (inputValueState.length > 0 && e.keyCode === _keyCode.KeyCode.BACKSPACE) {
      return;
    }

    if (e.keyCode === _keyCode.KeyCode.BACKSPACE) {
      var index = (0, _findLastIndex2["default"])(valueState, function (v) {
        return !isTagDisabled(disabledValue, v);
      });

      if (index >= 0) {
        handleRemoveTag(index, e);
      }
    }

    if (e.keyCode === _keyCode.KeyCode.ENTER) {
      e.preventDefault();
      handleAddTag(e);
    }

    if (onInputKeyDown) {
      onInputKeyDown(e);
    }
  };

  var handleRemoveTag = function handleRemoveTag(index, e) {
    var newValueState = (0, _toConsumableArray2["default"])(valueState);
    newValueState.splice(index, 1);

    if (!isControlled) {
      setValueState(newValueState);
    }

    if (onChange) {
      onChange(newValueState);
    }

    if (onRemoveTag) {
      onRemoveTag(valueState[index], index, e);
    }
  };

  var handleAddTag = function handleAddTag(e) {
    if (allowAdd && inputValueState) {
      var newValueState = [].concat((0, _toConsumableArray2["default"])(valueState), [inputValueState]);

      if (!isControlled) {
        setValueState(newValueState);
      }

      if (onChange) {
        onChange(newValueState);
      }

      handleInputChange('');

      if (onAddTag) {
        onAddTag(inputValueState, e);
      }
    }
  };

  var handleFocus = function handleFocus(e) {
    baseResult.handleFocus(e);

    if (contentRef.current) {
      contentRef.current.scrollTop = contentRef.current.scrollHeight;
    }
  };

  var handleBlur = function handleBlur(e) {
    baseResult.handleBlur(e);
    handleAddTag(e);
  };

  if (isControlled && !(0, _shallowEqual.shallowEqualArrays)(value, valueState)) {
    setValueState(value);
  }

  if (isInputControlled && inputValue !== inputValueState) {
    setInputValueState(inputValue);
  }

  (0, _react.useEffect)(function () {
    // resize input
    if (inputRef.current && sizerRef.current) {
      var inputWidth = Math.ceil(sizerRef.current.scrollWidth);

      if (inputWidth > 0) {
        inputRef.current.style.width = "".concat(inputWidth, "px");
      } else {
        inputRef.current.style.width = '';
      }
    }

    if (scrollTopAfterEffect && !baseResult.focus && contentRef.current) {
      contentRef.current.scrollTop = 0;
    }
  });
  (0, _react.useEffect)(function () {
    if (baseResult.focus && !(0, _shallowEqual.shallowEqualArrays)(preValueState, valueState) && contentRef.current) {
      if (horizontalScroll) {
        contentRef.current.scrollLeft = contentRef.current.scrollWidth;
      } else {
        contentRef.current.scrollTop = contentRef.current.scrollHeight;
      }
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [baseResult.focus, contentRef, preValueState, valueState]);
  return (0, _extends2["default"])({}, baseResult, {
    value: valueState,
    inputValue: inputValueState,
    placeholder: finalPlaceholder,
    hasValue: hasValue,
    hasError: hasError,
    handleFocus: handleFocus,
    handleBlur: handleBlur,
    handleClick: handleClick,
    handleClear: handleClear,
    handleInput: handleInput,
    handleInputKeyDown: handleInputKeyDown,
    handleRemoveTag: handleRemoveTag,
    handleAddTag: handleAddTag
  });
}