"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useInputNumber;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _includes2 = _interopRequireDefault(require("lodash/includes"));

var _react = require("react");

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _keyCode = require("../utils/keyCode");

var _decimal = _interopRequireDefault(require("./decimal"));

var _handleNumber = _interopRequireDefault(require("./handleNumber"));

function useInputNumber(props, isControlled) {
  var focus = props.focus,
      _props$defaultValue = props.defaultValue,
      defaultValue = _props$defaultValue === void 0 ? '' : _props$defaultValue,
      onChange = props.onChange,
      onKeyDown = props.onKeyDown,
      onBlur = props.onBlur,
      onFocus = props.onFocus,
      value = props.value,
      onMouseEnter = props.onMouseEnter,
      onMouseLeave = props.onMouseLeave,
      onDoubleClick = props.onDoubleClick,
      _props$formatter = props.formatter,
      formatter = _props$formatter === void 0 ? function (value) {
    return value || value === 0 ? value.toString() : '';
  } : _props$formatter,
      _props$parser = props.parser,
      parser = _props$parser === void 0 ? function (value) {
    return value;
  } : _props$parser,
      _props$min = props.min,
      min = _props$min === void 0 ? -Infinity : _props$min,
      _props$max = props.max,
      max = _props$max === void 0 ? Infinity : _props$max,
      step = props.step,
      _props$precision = props.precision,
      precision = _props$precision === void 0 ? 1 : _props$precision,
      fixPrecision = props.fixPrecision,
      _props$disableBlurFor = props.disableBlurFormat,
      disableBlurFormat = _props$disableBlurFor === void 0 ? false : _props$disableBlurFor,
      disabled = props.disabled;

  var _useMemo = (0, _react.useMemo)(function () {
    return (0, _handleNumber["default"])({
      min: min,
      max: max,
      precision: precision,
      fixPrecision: fixPrecision
    });
  }, [fixPrecision, max, min, precision]),
      getValidValue = _useMemo.getValidValue,
      toNumber = _useMemo.toNumber;

  var formatterFunc = (0, _react.useCallback)(function (val) {
    // 如果格式化后的值只有数字，允许清空输入框
    if (val === '' && /^\d+$/.test(formatter(val))) {
      return val;
    }

    return formatter(val);
  }, [formatter]);
  var parserFunc = (0, _react.useCallback)(function (val) {
    if (val === '') {
      return val;
    }

    return parser(val).toString();
  }, [parser]); // 格式化后的 value 值

  var validValue = getValidValue(value); // 输入框实际的值

  var _useState = (0, _react.useState)(isControlled ? validValue : getValidValue(defaultValue)),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      realValue = _useState2[0],
      setRealValue = _useState2[1]; // 输入过程中显示的值


  var _useState3 = (0, _react.useState)(formatterFunc(realValue)),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      showValue = _useState4[0],
      setShowValue = _useState4[1];

  var _useState5 = (0, _react.useState)(false),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      entered = _useState6[0],
      setEntered = _useState6[1];

  var _useState7 = (0, _react.useState)(false),
      _useState8 = (0, _slicedToArray2["default"])(_useState7, 2),
      isFocus = _useState8[0],
      setFocus = _useState8[1];

  var _useState9 = (0, _react.useState)(false),
      _useState10 = (0, _slicedToArray2["default"])(_useState9, 2),
      stepEntered = _useState10[0],
      setStepEntered = _useState10[1]; // 标记是步进器改变值的情况


  var _useState11 = (0, _react.useState)(false),
      _useState12 = (0, _slicedToArray2["default"])(_useState11, 2),
      isStepChange = _useState12[0],
      setIsStepChange = _useState12[1]; // 最终使用及显示的值


  var currentValue; // 判断在输入 focus 过程中，value 是否和 showValue 不同

  if (isFocus) {
    currentValue = showValue;
  } else {
    var realVal = isControlled ? validValue : getValidValue(realValue);
    currentValue = formatterFunc(realVal);
  }

  var getArrowStatus = (0, _react.useCallback)(function (val) {
    var realVal = val;

    if (max !== Infinity && (0, _decimal["default"])(max).lessEquals((0, _decimal["default"])(realVal))) {
      return {
        up: true,
        down: false
      };
    } else if (min !== -Infinity && (0, _decimal["default"])(realVal).lessEquals((0, _decimal["default"])(min))) {
      return {
        down: true,
        up: false
      };
    } else {
      return {
        down: false,
        up: false
      };
    }
  }, [max, min]);
  var emitChangeEvent = (0, _react.useCallback)(function (value, formattedValue) {
    var isBlur = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    if (value === '') {
      onChange && onChange('', NaN);
    } else {
      var decimal = (0, _decimal["default"])(formattedValue.toString());

      if (isBlur) {
        var newValue = decimal.isNaN() ? '' : toNumber(value);
        onChange && onChange(newValue, decimal.toNumber());
      } else {
        onChange && onChange(value, decimal.toNumber());
      }
    }
  }, [onChange, toNumber]);
  var setCurrentValue = (0, _react.useCallback)(function (val) {
    if (onChange) {
      var number = val === '' ? NaN : toNumber(val);
      emitChangeEvent(val, number);
    }
  }, [emitChangeEvent, onChange, toNumber]);
  var updateValue = (0, _react.useCallback)(function (val) {
    setCurrentValue(val.toString());
  }, [setCurrentValue]);
  var changeStep = (0, _react.useCallback)(function (type) {
    focus();
    setIsStepChange(true);
    var arrowDisabled = getArrowStatus(realValue);
    if (type === 'up' && arrowDisabled.up) return;

    if (type === 'down' && arrowDisabled.down) {
      if (realValue === '') {
        setRealValue(min.toString());
        emitChangeEvent(min.toString(), min);
      }

      return;
    }

    var newValue = parserFunc(showValue);
    var val;

    if (type === 'up') {
      val = (0, _decimal["default"])(newValue || 0).add(step);
    } else {
      val = (0, _decimal["default"])(newValue || 0).add(-step);
    }

    val = toNumber(val.toString());
    updateValue(val); // 受控模式下以外部的value为准

    if (!isControlled) {
      setRealValue(val.toString());
      setShowValue(formatterFunc(val));
    }
  }, [emitChangeEvent, focus, formatterFunc, getArrowStatus, isControlled, min, parserFunc, realValue, showValue, step, toNumber, updateValue]);
  var handleChange = (0, _react.useCallback)(function (event) {
    setShowValue(event.target.value);
    updateValue(parserFunc(event.target.value));
  }, [parserFunc, updateValue]);
  var handleMouseEnter = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function () {
      !disabled && setEntered(true);
    }, onMouseEnter);
  }, [disabled, onMouseEnter]);
  var handleFocus = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function () {
      setFocus(true);
    }, onFocus);
  }, [onFocus]);
  var handleMouseLeave = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function () {
      !disabled && setEntered(false);
    }, onMouseLeave);
  }, [disabled, onMouseLeave]);
  var handleDoubleClick = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function (e) {
      var input = e.target; // 找到第一个空格所在的index，更多空格的情况暂时不支持

      var spaceIndex = input.value.indexOf(' '); // 查看当前选中的字符在哪个空格范围之间

      if (input.selectionStart !== null && input.selectionStart < spaceIndex) {
        input.setSelectionRange(0, spaceIndex);
      } else if (input.selectionStart) {
        input.setSelectionRange(spaceIndex + 1, input.value.length);
      }
    }, onDoubleClick);
  }, [onDoubleClick]);
  var handleKeyDown = (0, _react.useMemo)(function () {
    return (// eslint-disable-next-line complexity
      (0, _forkHandler["default"])(function (e) {
        var keyCode = e.keyCode; // 支持组合键

        if (e.metaKey || e.ctrlKey) {
          return;
        } // 支持 tab 键


        if (e.keyCode === _keyCode.KeyCode.TAB) {
          return;
        } // 输入左右箭头及退格键


        if (keyCode === _keyCode.KeyCode.LEFT || keyCode === _keyCode.KeyCode.RIGHT || keyCode === _keyCode.KeyCode.BACKSPACE) return; // 输入上下箭头

        if (keyCode === _keyCode.KeyCode.UP) {
          changeStep('up');
          e.preventDefault();
        } else if (keyCode === _keyCode.KeyCode.DOWN) {
          changeStep('down');
          e.preventDefault();
        } // 输入 -


        if (keyCode === _keyCode.KeyCode.NUM_MINUS || keyCode === _keyCode.KeyCode.DASH) {
          if (currentValue.startsWith('-') || e.target.selectionStart !== 0) {
            e.preventDefault();
          }

          return;
        } // 输入 . 。


        if (keyCode === _keyCode.KeyCode.PERIOD || keyCode === _keyCode.KeyCode.NUM_PERIOD) {
          if ((0, _includes2["default"])(currentValue, '.') || precision === 0) {
            e.preventDefault();
          }

          return;
        } // 输入数字


        if (keyCode >= _keyCode.KeyCode.ZERO && keyCode <= _keyCode.KeyCode.NINE || keyCode >= _keyCode.KeyCode.NUM_ZERO && keyCode <= _keyCode.KeyCode.NUM_NINE) {
          return;
        } // e.preventDefault();

      }, onKeyDown)
    );
  }, [changeStep, currentValue, onKeyDown, precision]);
  var handleBlur = (0, _react.useCallback)(function (e) {
    if (stepEntered) {
      focus();
      return;
    }

    onBlur && onBlur(e);
    setFocus(false);
    var val = parserFunc(showValue);
    var decimal = (0, _decimal["default"])(val);
    var formattedString = getValidValue(decimal.toString());
    var validFormattedValue = decimal.isNaN() ? '' : formattedString;
    var value = isControlled ? formattedString : validFormattedValue; // 加入校验逻辑，如果当前值是非法值，则显示成最近一次有效值

    if (isNaN(Number(val))) {
      value = realValue;
    } else {
      setRealValue(toNumber(value));
    }

    setShowValue(formatterFunc(value));
    !disableBlurFormat && emitChangeEvent(value, toNumber(value), true);
  }, [disableBlurFormat, emitChangeEvent, focus, formatterFunc, getValidValue, isControlled, onBlur, parserFunc, realValue, showValue, stepEntered, toNumber]);
  var handleStepMouseEnter = (0, _react.useCallback)(function () {
    setStepEntered(true);
  }, []);
  var handleStepMouseLeave = (0, _react.useCallback)(function () {
    setStepEntered(false);
  }, []);
  (0, _react.useEffect)(function () {
    if (isControlled && value !== realValue && !isFocus) {
      setRealValue(validValue);
      setShowValue(formatterFunc(validValue));
    } else if (isStepChange && isControlled) {
      // 如果当前值是步进器改变的，且当前是受控状态，需要将realValue和showValue更新一下
      setRealValue(validValue.toString());
      setShowValue(formatterFunc(validValue));
      setIsStepChange(false);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [isStepChange, validValue]);
  return {
    handleBlur: handleBlur,
    handleChange: handleChange,
    handleKeyDown: handleKeyDown,
    setRealValue: setRealValue,
    arrowDisabled: getArrowStatus(realValue),
    inputValue: currentValue,
    entered: entered,
    setEntered: setEntered,
    handleMouseEnter: handleMouseEnter,
    handleMouseLeave: handleMouseLeave,
    handleStepMouseEnter: handleStepMouseEnter,
    handleStepMouseLeave: handleStepMouseLeave,
    handleDoubleClick: handleDoubleClick,
    changeStep: changeStep,
    getValidValue: getValidValue,
    handleFocus: handleFocus,
    // for test
    showValue: showValue,
    realValue: realValue
  };
}