"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useRangeInputNumber;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = require("react");

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _utils = require("./utils");

function useRangeInputNumber(props) {
  var disabled = props.disabled,
      value = props.value,
      defaultValue = props.defaultValue,
      onChange = props.onChange,
      _props$inputProps = props.inputProps,
      inputProps = _props$inputProps === void 0 ? {} : _props$inputProps,
      onKeyDown = props.onKeyDown,
      onPressEnter = props.onPressEnter,
      onFocus = props.onFocus,
      onBlur = props.onBlur;
  var autoFocus = inputProps.autoFocus;
  var leftInputRef = (0, _react.useRef)(null);
  var rightInputRef = (0, _react.useRef)(null);
  var valueFromProps = (0, _react.useMemo)(function () {
    if (value) {
      return (0, _utils.getRangeDefaultValue)(value);
    }

    if (defaultValue) {
      return (0, _utils.getRangeDefaultValue)(defaultValue);
    }

    return ['', ''];
  }, [defaultValue, value]);

  var _useState = (0, _react.useState)(valueFromProps),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      innerValue = _useState2[0],
      setInnerValue = _useState2[1];

  var finalValue = 'value' in props ? valueFromProps : innerValue;
  var updateValue = (0, _react.useCallback)(function (newValue) {
    setInnerValue(newValue);

    if (onChange) {
      var formatterValue = [parseFloat(newValue[0]), parseFloat(newValue[1])];
      onChange(newValue, formatterValue);
    }
  }, [onChange]);
  var setInputValue = (0, _react.useCallback)(function (v, inputPos) {
    var _finalValue = (0, _slicedToArray2["default"])(finalValue, 2),
        oldLeft = _finalValue[0],
        oldRight = _finalValue[1];

    var newValue = [oldLeft, oldRight];
    newValue[inputPos] = v;
    updateValue(newValue);
  }, [finalValue, updateValue]);
  var handleLeftChange = (0, _react.useCallback)(function (displayedValue, formatterValue) {
    setInputValue(displayedValue, 0);
  }, [setInputValue]);
  var handleRightChange = (0, _react.useCallback)(function (displayedValue, formatterValue) {
    setInputValue(displayedValue, 1);
  }, [setInputValue]);

  if (disabled) {
    leftInputRef.current && leftInputRef.current.blur();
    rightInputRef.current && rightInputRef.current.blur();
  }

  var handleKeyDown = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function (e) {
      if (e.keyCode === 13 && onPressEnter) {
        var formatterValue = [parseFloat(finalValue[0]), parseFloat(finalValue[1])];
        onPressEnter(finalValue, formatterValue);
      }
    }, onKeyDown);
  }, [finalValue, onKeyDown, onPressEnter]);

  var _useState3 = (0, _react.useState)(autoFocus),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      focus = _useState4[0],
      setFocus = _useState4[1];

  var handleFocus = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function () {
      setFocus(true);
    }, onFocus);
  }, [onFocus]);
  var handleBlur = (0, _react.useMemo)(function () {
    return (0, _forkHandler["default"])(function () {
      setFocus(false);
    }, onBlur);
  }, [onBlur]);
  return {
    focus: focus,
    finalValue: finalValue,
    leftInputRef: leftInputRef,
    rightInputRef: rightInputRef,
    handleFocus: handleFocus,
    handleBlur: handleBlur,
    handleLeftChange: handleLeftChange,
    handleRightChange: handleRightChange,
    handleKeyDown: handleKeyDown
  };
}