"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useMenuBase = void 0;

var _react = require("react");

var _utils = require("./utils");

var useMenuBase = function useMenuBase(props) {
  var level = props.level,
      eventKey = props.eventKey,
      hasScrollBar = props.hasScrollBar,
      parentIsRootMenu = props.parentIsRootMenu,
      parentMenuOpen = props.parentMenuOpen,
      menuHasIcon = props.menuHasIcon,
      children = props.children,
      onClick = props.onClick,
      onItemSelect = props.onItemSelect,
      onItemDeselect = props.onItemDeselect,
      _props$autoScrollToSe = props.autoScrollToSelection,
      autoScrollToSelection = _props$autoScrollToSe === void 0 ? true : _props$autoScrollToSe; // 获取子节点

  var elements = (0, _react.useMemo)(function () {
    return (0, _utils.getValidChildElements)(children);
  }, [children]); // 渲染子节点

  var renderMenuWidget = function renderMenuWidget(child, i) {
    if (!child) {
      return null;
    }

    if (child.type && child.type.displayName === 'MenuDivider') {
      return child;
    }

    var key = (0, _utils.getKeyFromChildrenIndex)(child, eventKey, i);
    var childProps = child.props;

    if (!childProps || typeof child.type === 'string') {
      return child;
    }

    return /*#__PURE__*/(0, _react.cloneElement)(child, {
      level: level,
      index: i,
      eventKey: key,
      parentEventKey: eventKey,
      parentMenuOpen: parentMenuOpen,
      parentIsRootMenu: parentIsRootMenu,
      parentMenuScrollable: hasScrollBar && autoScrollToSelection,
      menuHasIcon: menuHasIcon,
      originOnClick: childProps.onClick,
      onClick: onClick,
      onItemSelect: onItemSelect,
      onItemDeselect: onItemDeselect
    });
  };

  return {
    elements: elements,
    renderMenuWidget: renderMenuWidget
  };
};

exports.useMenuBase = useMenuBase;