"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useSubMenu = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = _interopRequireWildcard(require("react"));

var _const = require("./const");

var _utils = require("./utils");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/* istanbul ignore next */
var useSubMenu = function useSubMenu(props, context, titleRef) {
  var openKeys = context.openKeys,
      selectedKeys = context.selectedKeys,
      mode = context.mode,
      multiple = context.multiple,
      inlineMode = context.inlineMode,
      horizontalMode = context.horizontalMode,
      inlineCollapsed = context.inlineCollapsed,
      triggerSubMenuAction = context.triggerSubMenuAction,
      handleOpenChange = context.handleOpenChange;
  var eventKey = props.eventKey,
      onClick = props.onClick,
      originOnClick = props.originOnClick,
      onTitleMouseEnter = props.onTitleMouseEnter,
      onTitleMouseLeave = props.onTitleMouseLeave,
      onTitleClick = props.onTitleClick,
      parentIsRootMenu = props.parentIsRootMenu,
      disabled = props.disabled,
      children = props.children,
      onItemSelect = props.onItemSelect,
      onItemDeselect = props.onItemDeselect;

  var _useState = (0, _react.useState)(false),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      hasScrollBar = _useState2[0],
      setHasScrollBar = _useState2[1];

  var _useState3 = (0, _react.useState)(false),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      appear = _useState4[0],
      setAppear = _useState4[1];

  var menuWidthRef = (0, _react.useRef)(0);
  var mouseenterTimeoutRef = (0, _react.useRef)();
  var isOpen = (0, _react.useMemo)(function () {
    return openKeys.indexOf(eventKey) > -1;
  }, [eventKey, openKeys]);

  var _menuId = "".concat(eventKey, "$Menu");

  var placement = _const.placementMap[mode]; // item 的选中状态

  var isSelected = (0, _react.useMemo)(function () {
    var ret = {
      find: false
    };
    (0, _utils.loopMenuItemRecursively)(children, selectedKeys, ret);
    return selectedKeys.indexOf(eventKey) !== -1 || ret.find;
  }, [children, eventKey, selectedKeys]);
  var handleMenuScrollRef = (0, _react.useCallback)(function (el) {
    if (el) {
      setHasScrollBar(el.scrollHeight > el.offsetHeight);
    }
  }, []);
  /**
   * 触发 open 事件，点击 title 或者 popup 展现
   * 如果没有 children 那么还会触发选中行为
   */

  var triggerOpenChange = (0, _react.useCallback)(function (open, type) {
    if (open === isOpen) return; // 第一次打开是将 appear 设为 true，使得后面的动画正常

    if (inlineMode && !inlineCollapsed && !appear) {
      setAppear(true);
    }

    var changeInfo = {
      key: eventKey,
      item: titleRef,
      trigger: type,
      open: open,
      parentIsRootMenu: parentIsRootMenu
    };

    if (type === 'mouseenter') {
      // make sure mouseenter happen after other menu item's mouseleave
      mouseenterTimeoutRef.current = setTimeout(function () {
        handleOpenChange(changeInfo);
      }, 0);
    } else {
      handleOpenChange(changeInfo);
    }
  }, [appear, eventKey, handleOpenChange, inlineCollapsed, inlineMode, isOpen, parentIsRootMenu, titleRef]);
  var addKeyPath = (0, _react.useCallback)(function (info) {
    return (0, _extends2["default"])({}, info, {
      keyPath: (info.keyPath || []).concat([eventKey])
    });
  }, [eventKey]); // title 的事件

  var handleTitleMouseEnter = (0, _react.useCallback)(function (e) {
    if (disabled) return;

    if (onTitleMouseEnter) {
      onTitleMouseEnter({
        key: eventKey,
        nativeEvent: e
      });
    }
  }, [disabled, eventKey, onTitleMouseEnter]);
  var handleTitleMouseLeave = (0, _react.useCallback)(function (e) {
    if (disabled) return;

    if (onTitleMouseLeave) {
      onTitleMouseLeave({
        key: eventKey,
        nativeEvent: e
      });
    }
  }, [disabled, eventKey, onTitleMouseLeave]);
  var handleItemSelect = (0, _react.useCallback)(function (info) {
    if (onItemSelect) {
      onItemSelect(addKeyPath(info));
    }
  }, [addKeyPath, onItemSelect]);
  var handleItemDeselect = (0, _react.useCallback)(function (info) {
    if (onItemDeselect) {
      onItemDeselect(addKeyPath(info));
    }
  }, [addKeyPath, onItemDeselect]);
  var handleTitleClick = (0, _react.useCallback)(function (e) {
    e.stopPropagation();
    if (disabled) return;

    if (onTitleClick) {
      onTitleClick({
        key: eventKey,
        nativeEvent: e
      });
    }

    if (horizontalMode && titleRef.current) {
      menuWidthRef.current = parseFloat(getComputedStyle(titleRef.current).width || '0');
    }

    if (inlineMode && !inlineCollapsed) {
      triggerOpenChange(!isOpen, 'click');
    } // 没有 children 那么触发选中


    if (_react["default"].Children.count(children) === 0) {
      var info = {
        key: eventKey,
        keyPath: [eventKey],
        item: titleRef,
        nativeEvent: e
      };

      if (multiple) {
        if (isSelected) {
          handleItemDeselect(info);
        } else {
          handleItemSelect(info);
        }
      } else if (!isSelected) {
        handleItemSelect(info);
      }
    }
  }, [children, disabled, eventKey, handleItemDeselect, handleItemSelect, horizontalMode, inlineCollapsed, inlineMode, isOpen, isSelected, multiple, onTitleClick, titleRef, triggerOpenChange]);
  var handleVisibleChange = (0, _react.useCallback)(function (visible) {
    if (triggerSubMenuAction === 'hover') {
      triggerOpenChange(visible, visible ? 'mouseenter' : 'mouseleave');
    } else {
      triggerOpenChange(visible, 'click');
    }
  }, [triggerOpenChange, triggerSubMenuAction]);
  var handleSubMenuClick = (0, _react.useCallback)(function (info) {
    if (onClick) {
      onClick(addKeyPath(info));
    }

    if (originOnClick) {
      originOnClick(addKeyPath(info));
    }
  }, [addKeyPath, onClick, originOnClick]);
  (0, _react.useEffect)(function () {
    return function () {
      if (mouseenterTimeoutRef.current) {
        clearTimeout(mouseenterTimeoutRef.current);
      }
    };
  });
  return {
    isOpen: isOpen,
    _menuId: _menuId,
    placement: placement,
    appear: appear,
    hasScrollBar: hasScrollBar,
    handleMenuScrollRef: handleMenuScrollRef,
    handleTitleMouseEnter: handleTitleMouseEnter,
    handleTitleMouseLeave: handleTitleMouseLeave,
    handleTitleClick: handleTitleClick,
    handleVisibleChange: handleVisibleChange,
    handleSubMenuClick: handleSubMenuClick,
    handleItemSelect: handleItemSelect,
    handleItemDeselect: handleItemDeselect,
    isSelected: isSelected,
    menuWidth: menuWidthRef.current
  };
};

exports.useSubMenu = useSubMenu;