"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.applyStyle = applyStyle;
exports.getKeyFromChildrenIndex = getKeyFromChildrenIndex;
exports.getValidChildElement = getValidChildElement;
exports.getValidChildElements = getValidChildElements;
exports.loopMenuItemRecursively = loopMenuItemRecursively;

var _isNumber2 = _interopRequireDefault(require("lodash/isNumber"));

var _react = _interopRequireDefault(require("react"));

var _addPx = _interopRequireDefault(require("../utils/addPx"));

var _scrollIntoView = require("../utils/scrollIntoView");

/**
 * 获取菜单节点的 eventKey
 */
function getKeyFromChildrenIndex(child, parentEventKey, index) {
  var prefix = parentEventKey || '';

  if (child.key) {
    return "".concat(child.key);
  }

  return "".concat(prefix, "_item_").concat(index);
}
/**
 * 递归子节点寻找是否有子节点的 key 存在于 keys 数组中，用于计算子节点是否选中
 **/


function loopMenuItemRecursively(children, keys, ret) {
  if (!children || ret.find) {
    return;
  }

  _react["default"].Children.forEach(children, function (child) {
    if (child) {
      var construct = child.type;

      if (!construct || !(construct.displayName === 'SubMenu' || construct.displayName === 'MenuItem' || construct.displayName === 'MenuItemGroup' || construct.displayName === 'MenuDivider')) {
        return;
      }

      if (keys.indexOf("".concat(child.key)) !== -1) {
        ret.find = true;
      } else if (child.props && child.props.children) {
        loopMenuItemRecursively(child.props.children, keys, ret);
      }
    }
  });
}
/**
 * 过滤掉外层包裹的 div、fragment 等非合法的菜单节点
 */


function getValidChildElement(child) {
  if (child) {
    var construct = child.type;

    if (!construct || !(construct.displayName === 'SubMenu' || construct.displayName === 'MenuItem' || construct.displayName === 'MenuItemGroup' || construct.displayName === 'MenuDivider')) {
      if (child.props && child.props.children) {
        return getValidChildElements(child.props.children);
      }
    }

    return child;
  }
}
/**
 * 从 children 中解析出合法的菜单节点
 */


function getValidChildElements(children) {
  var result = [];

  _react["default"].Children.forEach(children, function (child) {
    var elements = getValidChildElement(child);

    if (Array.isArray(elements)) {
      result = result.concat(elements);
    } else if (elements) {
      result = result.concat([elements]);
    }
  });

  return result;
}

function setStyles(element, styles) {
  Object.keys(styles).forEach(function (prop) {
    if (['width', 'height', 'top', 'right', 'bottom', 'left'].indexOf(prop) !== -1 && (0, _isNumber2["default"])(styles[prop])) {
      // @ts-ignore
      element.style[prop] = (0, _addPx["default"])(styles[prop]);
    } else {
      // @ts-ignore
      element.style[prop] = styles[prop];
    }
  });
}

function setAttributes(element, attributes) {
  Object.keys(attributes).forEach(function (prop) {
    var value = attributes[prop];

    if (value !== false) {
      element.setAttribute(prop, attributes[prop]);
    } else {
      element.removeAttribute(prop);
    }
  });
}

function applyStyle(data) {
  var popperNode = data.instance.popper;
  var refNode = data.instance.reference;
  var subMenuNode = refNode.parentElement;

  if (subMenuNode) {
    var parentMenuNode = (0, _scrollIntoView.findFirstScrollableParent)(subMenuNode);

    if (parentMenuNode) {
      var parentTop = parentMenuNode.scrollTop + parentMenuNode.offsetTop;
      var subMenuBottom = parentMenuNode.offsetHeight + parentMenuNode.scrollTop + parentMenuNode.offsetTop - subMenuNode.offsetTop;
      var popperHeight = popperNode.clientHeight;
      var _data$offsets$popper = data.offsets.popper,
          left = _data$offsets$popper.left,
          top = _data$offsets$popper.top; // 上边界溢出

      if (subMenuNode.offsetTop < parentTop) {
        data.styles.transform = "translate3d(".concat(left, "px, ").concat(top + parentTop - subMenuNode.offsetTop, "px, 0)"); // 下边界溢出
      } else if (subMenuBottom < popperHeight && top + popperHeight <= subMenuBottom) {
        data.styles.transform = "translate3d(".concat(left, "px, ").concat(top - popperHeight + subMenuBottom, "px, 0)");
      }
    }
  }

  setStyles(data.instance.popper, data.styles);
  setAttributes(data.instance.popper, data.attributes);

  if (data.arrowElement && Object.keys(data.arrowStyles).length) {
    setStyles(data.arrowElement, data.arrowStyles);
  }

  return data;
}