"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useRangeDateTimePicker;

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _isEqual2 = _interopRequireDefault(require("lodash/isEqual"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _useRangeDateTimeCalendar = _interopRequireDefault(require("./useRangeDateTimeCalendar"));

var _Locale = require("../Locale");

var _useDateTimePicker = require("../DateTimePicker/useDateTimePicker");

var _excluded = ["todayDate", "viewType", "viewDate", "disableDate", "options", "defaultViewDate", "getPopupContainer", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "placeholder", "inputReadOnly", "timeRowNum", "hourStep", "minuteStep", "secondStep", "disabledHours", "disabledMinutes", "disabledSeconds", "hideDisabledOptions", "disabled", "readOnly", "onFocus", "onClear", "defaultValue", "value", "format", "timeFormat", "allowClear", "popupOpen", "onConfirm", "onChange", "onDateChange", "onUIChange", "onCalendarChange", "placement", "triggerId", "popperProps", "flip", "arrowPointAtCenter", "calendarStyles", "styles", "defaultTime", "disableTimeAfter", "disableTimeBefore"];
var defaultDateTimeFormat = 'YYYY-MM-DD HH:mm:ss';

function useRangeDateTimePicker(props) {
  var locale = (0, _Locale.useLocale)();
  var todayDate = props.todayDate,
      viewType = props.viewType,
      viewDate = props.viewDate,
      disableDate = props.disableDate,
      options = props.options,
      defaultViewDate = props.defaultViewDate,
      getPopupContainer = props.getPopupContainer,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      _props$placeholder = props.placeholder,
      placeholder = _props$placeholder === void 0 ? [locale['RangeDateTimePicker.startPlaceholder'], locale['RangeDateTimePicker.endPlaceholder']] : _props$placeholder,
      inputReadOnly = props.inputReadOnly,
      timeRowNum = props.timeRowNum,
      hourStep = props.hourStep,
      minuteStep = props.minuteStep,
      secondStep = props.secondStep,
      disabledHours = props.disabledHours,
      disabledMinutes = props.disabledMinutes,
      disabledSeconds = props.disabledSeconds,
      hideDisabledOptions = props.hideDisabledOptions,
      disabled = props.disabled,
      readOnly = props.readOnly,
      onFocus = props.onFocus,
      onClear = props.onClear,
      defaultValue = props.defaultValue,
      value = props.value,
      _props$format = props.format,
      format = _props$format === void 0 ? defaultDateTimeFormat : _props$format,
      timeFormat = props.timeFormat,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      popupOpen = props.popupOpen,
      onConfirm = props.onConfirm,
      onChange = props.onChange,
      onDateChange = props.onDateChange,
      onUIChange = props.onUIChange,
      onCalendarChange = props.onCalendarChange,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      triggerId = props.triggerId,
      popperProps = props.popperProps,
      flip = props.flip,
      arrowPointAtCenter = props.arrowPointAtCenter,
      calendarStyles = props.calendarStyles,
      styles = props.styles,
      defaultTime = props.defaultTime,
      disableTimeAfter = props.disableTimeAfter,
      disableTimeBefore = props.disableTimeBefore,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var isControlled = ('value' in props);
  var finalTimeFormat = timeFormat || (0, _useDateTimePicker.getTimeFormatByFormat)(format);
  var getRangeValue = (0, _react.useCallback)(function (value) {
    var range;
    var start;
    var end;

    if (value) {
      var innerDate = value.map(function (date) {
        return date && (0, _dayjs["default"])(date);
      });

      var _innerDate = (0, _slicedToArray2["default"])(innerDate, 2),
          innerStart = _innerDate[0],
          innerEnd = _innerDate[1];

      start = innerStart && innerStart.isValid() ? innerStart : undefined;
      end = innerEnd && innerEnd.isValid() ? innerEnd : undefined;

      if (start && end) {
        range = innerDate;
      }
    }

    return {
      range: range,
      start: start,
      end: end
    };
  }, []);
  var valueFromProps = (0, _react.useMemo)(function () {
    if (isControlled) {
      if (value) {
        return getRangeValue(value);
      }
    } else if (defaultValue) {
      return getRangeValue(defaultValue);
    }

    return {
      range: undefined,
      start: undefined,
      end: undefined
    };
  }, [defaultValue, getRangeValue, isControlled, value]);
  var formatDateStr = (0, _react.useMemo)(function () {
    var tplFn;

    if (typeof format === 'function') {
      tplFn = format;
    } else {
      tplFn = function tplFn(date) {
        return date.format(format);
      };
    }

    return function (date) {
      if (date) {
        return date.map(function (d) {
          return d ? tplFn((0, _dayjs["default"])(d)) : '';
        });
      }

      return [];
    };
  }, [format]);

  var _useState = (0, _react.useState)(false),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      statePopupOpen = _useState2[0],
      setPopupOpen = _useState2[1];

  var _useState3 = (0, _react.useState)('none'),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      focusState = _useState4[0],
      setFocusState = _useState4[1];

  var _useState5 = (0, _react.useState)('none'),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      nextFocusStatus = _useState6[0],
      setNextFocusState = _useState6[1];

  var finalPopupOpen = popupOpen !== undefined ? popupOpen : statePopupOpen;

  var _useState7 = (0, _react.useState)(valueFromProps),
      _useState8 = (0, _slicedToArray2["default"])(_useState7, 2),
      calendarDate = _useState8[0],
      setCalendarDate = _useState8[1];

  var initDate = [valueFromProps.start, valueFromProps.end]; // 面板关闭时，记录下的值

  var _useState9 = (0, _react.useState)(initDate),
      _useState10 = (0, _slicedToArray2["default"])(_useState9, 2),
      inputValue = _useState10[0],
      setInputValue = _useState10[1]; // 面板打开时，输入框的值


  var _useState11 = (0, _react.useState)(initDate),
      _useState12 = (0, _slicedToArray2["default"])(_useState11, 2),
      inputDate = _useState12[0],
      setInputDate = _useState12[1];

  var finalInputDate = isControlled ? value : inputDate;
  var calendarProps = (0, _extends2["default"])({
    allowClear: allowClear,
    todayDate: todayDate,
    disableTimeBefore: disableTimeBefore,
    disableTimeAfter: disableTimeAfter,
    disableDate: disableDate,
    options: options,
    defaultViewDate: defaultViewDate,
    defaultDate: defaultValue,
    // selectedDate: calendarDate as any,
    defaultTime: defaultTime
  }, calendarDate);

  if ('viewType' in props) {
    calendarProps.viewType = viewType;
  }

  if ('viewDate' in props) {
    calendarProps.viewDate = viewDate;
  }
  /**
   * 这里复用 hooks 的逻辑需要注意的点：
   * * 事件处理一定要在这个 hook 里处理
   */


  var calendarState = (0, _useRangeDateTimeCalendar["default"])(calendarProps);
  var finalUIState = (0, _extends2["default"])({}, calendarState.finalUIState, {
    popupOpen: finalPopupOpen
  }); // 切换日历面板的显示隐藏

  var updateUI = function updateUI(innerPopupOpen, status) {
    var newState = (0, _extends2["default"])({}, calendarState.finalUIState, {
      popupOpen: innerPopupOpen
    });
    setPopupOpen(innerPopupOpen);

    if (onUIChange) {
      onUIChange(newState);
    }

    if (innerPopupOpen && status && status !== 'none' && calendarDate[status]) {
      calendarState.calendarState.updateUI({
        viewDate: calendarDate[status]
      });
    }
  };
  /**
   * 关闭日历面板
   */


  function closePopup() {
    if (!finalUIState.popupOpen) {
      return;
    }

    setFocusState('none');
    updateUI(false);
  }
  /**
   * 更新日历的值
   */


  var updateDate = (0, _react.useCallback)(function (value) {
    // 置空
    if (!value) {
      setCalendarDate({});
      setInputDate(undefined);
      setInputValue(undefined);

      if (onCalendarChange) {
        onCalendarChange([]);
      }

      if (onChange) {
        onChange([]);
      }

      if (onDateChange) {
        onDateChange(['', ''], []);
      }

      return;
    }

    var start = value.start,
        end = value.end,
        range = value.range;
    var finalRange = range || calendarDate.range;
    var finalEnd = end || calendarDate.end;
    var finalStart = start || calendarDate.start;

    if (finalEnd && start && finalEnd.isBefore(start)) {
      // 如果是编辑开始时间, 选择的开始时间大于结束时间时，清空结束时间
      finalEnd = undefined;
    }

    if (finalStart && end && end.isBefore(finalStart)) {
      // 如果是编辑结束时间, 选择的开始时间大于结束时间时，清空开始时间
      finalStart = undefined;
    } // 如果range中已有值，但是本次更新没有传入range，要将range传入的值对应更新


    if (!range && finalRange) {
      finalRange = [finalStart, finalEnd];
    }

    setCalendarDate({
      range: finalRange,
      start: finalStart,
      end: finalEnd
    });
    setInputDate([finalStart, finalEnd]);

    if (onCalendarChange) {
      onCalendarChange([finalStart, finalEnd]);
    }

    var result = [finalStart, finalEnd];

    if (onChange) {
      onChange(result);
    }

    if (onDateChange) {
      onDateChange(formatDateStr(result), result);
    }
  }, [calendarDate.end, calendarDate.range, calendarDate.start, formatDateStr, onCalendarChange, onChange, onDateChange]);
  var handleCalendarUIChange = (0, _useEventCallback2.useEventCallback)(function (inputState) {
    updateUI(finalPopupOpen);
    calendarState.calendarState.updateUI(inputState);
  });
  var handleCalendarChange = (0, _useEventCallback2.useEventCallback)(function (value) {
    updateDate(value);
  });
  var handleFocus = (0, _useEventCallback2.useEventCallback)(function (status) {
    setFocusState(status);
    updateUI(true, status);
  });
  var handleStartFocus = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function (e) {
    handleFocus('start');
    setNextFocusState(focusState === 'none' ? 'end' : 'none');
  }, onFocus));
  var handleEndFocus = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function (e) {
    handleFocus('end');
    setNextFocusState(focusState === 'none' ? 'start' : 'none');
  }, onFocus));
  var handleStartClear = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    var end = inputDate && inputDate[1];
    setCalendarDate({
      end: calendarDate.end
    });
    var newDate = [undefined, end];
    setInputDate(newDate);
    setInputValue(newDate);

    if (onChange) {
      onChange(newDate);
    }

    if (onDateChange) {
      onDateChange(formatDateStr(newDate), newDate);
    }
  }, onClear));
  var handleEndClear = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    var start = inputDate && inputDate[0];
    setCalendarDate({
      start: calendarDate.start
    });
    var newDate = start && [start, undefined];
    setInputDate(newDate);
    setInputValue(newDate);

    if (onChange) {
      onChange(newDate);
    }

    if (onDateChange) {
      onDateChange(formatDateStr(newDate), newDate);
    }
  }, onClear));
  var handleClickAway = (0, _useEventCallback2.useEventCallback)(function () {
    if (finalPopupOpen) {
      closePopup();

      var _ref = inputValue || [undefined, undefined],
          _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
          start = _ref2[0],
          end = _ref2[1];

      var prevValue = inputValue ? {
        start: start && (0, _dayjs["default"])(start),
        end: end && (0, _dayjs["default"])(end),
        range: start && end && inputValue
      } : {};
      setCalendarDate(prevValue);
      var newDate = !start && !end ? undefined : inputValue;
      setInputDate(newDate);

      if (onChange) {
        onChange(newDate || []);
      }

      if (onDateChange) {
        onDateChange(formatDateStr(newDate), newDate);
      }
    }
  });
  var handleConfirm = (0, _useEventCallback2.useEventCallback)(function (range) {
    if (nextFocusStatus !== 'none') {
      setFocusState(nextFocusStatus); // 改变一下当前的viewDate

      updateUI(finalPopupOpen, nextFocusStatus);
      setNextFocusState('none');
    } else {
      closePopup();

      if (onConfirm) {
        onConfirm(range);
      }
    }

    setInputValue(finalInputDate);
  });
  var finalValue = finalPopupOpen ? formatDateStr(finalInputDate) // 打开
  : formatDateStr(inputValue); // 关闭

  (0, _react.useEffect)(function () {
    if (isControlled && !(0, _isEqual2["default"])(value, finalValue)) {
      // 根据传入的value判断要不要改变日历面板的值。
      var _ref3 = value || [undefined, undefined],
          _ref4 = (0, _slicedToArray2["default"])(_ref3, 2),
          start = _ref4[0],
          end = _ref4[1]; // 略过hover的状态


      if (finalPopupOpen && (start && !end || !start || end)) {
        return;
      }

      var data = getRangeValue(value);
      setCalendarDate(data);
      setInputValue([data.start, data.end]);
    } // eslint-disable-next-line

  }, [value]);
  return {
    focusState: focusState,
    // 处理后的数据
    finalUIState: finalUIState,
    finalInputDate: finalInputDate,
    calendarDate: calendarDate,
    calendarState: calendarState,
    finalValue: finalValue,
    otherProps: otherProps,
    finalTimeFormat: finalTimeFormat,
    // 事件处理
    handleStartFocus: handleStartFocus,
    handleEndFocus: handleEndFocus,
    handleCalendarChange: handleCalendarChange,
    handleCalendarUIChange: handleCalendarUIChange,
    handleClickAway: handleClickAway,
    handleStartClear: handleStartClear,
    handleEndClear: handleEndClear,
    handleConfirm: handleConfirm,
    // 直接透传
    styles: styles,
    disabled: disabled,
    readOnly: readOnly,
    allowClear: allowClear,
    calendarStyles: calendarStyles,
    placeholder: placeholder,
    disableTimeAfter: disableTimeAfter,
    disableTimeBefore: disableTimeBefore,
    defaultTime: defaultTime,
    container: getPopupContainer,
    inputReadOnly: inputReadOnly,
    // ---- calendar props
    todayDate: todayDate,
    disableDate: disableDate,
    options: options,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // ---- timepicker panel props
    timeRowNum: timeRowNum,
    hourStep: hourStep,
    minuteStep: minuteStep,
    secondStep: secondStep,
    disabledHours: disabledHours,
    disabledMinutes: disabledMinutes,
    disabledSeconds: disabledSeconds,
    hideDisabledOptions: hideDisabledOptions,
    // triggerProps
    placement: placement,
    popperProps: popperProps,
    arrowPointAtCenter: arrowPointAtCenter,
    triggerId: triggerId,
    flip: flip
  };
}