"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useSelectInput;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _react = _interopRequireDefault(require("react"));

var _keyCode = require("../utils/keyCode");

var _utils = require("./utils");

function useSelectInput(props, inputRef, sizerRef) {
  var maxTagTextLength = props.maxTagTextLength,
      inputValue = props.inputValue,
      value = props.value,
      allOptions = props.allOptions,
      showSearch = props.showSearch,
      onInput = props.onInput,
      onInputChange = props.onInputChange,
      onFocus = props.onFocus,
      onBlur = props.onBlur,
      onKeyDown = props.onKeyDown,
      onAddTag = props.onAddTag,
      onSearch = props.onSearch,
      onDeselect = props.onDeselect,
      mode = props.mode,
      autoResize = props.autoResize,
      menuRef = props.menuRef,
      isMultiple = props.isMultiple,
      tokenSeparators = props.tokenSeparators;

  var pastedTextRef = _react["default"].useRef();

  var tokenWithEnter = _react["default"].useMemo(function () {
    return (tokenSeparators || []).some(function (tokenSeparator) {
      return ['\n', '\r\n'].includes(tokenSeparator);
    });
  }, [tokenSeparators]);
  /**
   * default 模式下获取展示的 label
   */


  var getDefaultModeLabel = function getDefaultModeLabel(combobox) {
    var optionValue = value[0];
    var option;

    if (typeof optionValue === 'undefined') {
      return '';
    } else {
      option = optionByValue(optionValue, combobox);

      if (option) {
        return option.label || option.value;
      }

      return '';
    }
  };
  /**
   * 获取 value 对应的 option 对象
   */


  var optionByValue = function optionByValue(value, combobox) {
    if (value && (0, _typeof2["default"])(value) === 'object') {
      return allOptions[value.value] || (0, _extends2["default"])({}, value);
    } else {
      // TODO: 大版本在外层判断，此版本先做兼容
      return allOptions[value] || {
        value: value === null || combobox ? '' : value
      };
    }
  };
  /**
   * 获取 value 对应的 label 值
   */


  var labelByValue = function labelByValue(value) {
    var option = optionByValue(value);
    return option.label || option.value;
  };

  var disabledValue = _react["default"].useMemo(function () {
    if (!isMultiple) {
      return [];
    }

    return Object.keys(allOptions).reduce(function (ret, key) {
      var option = allOptions[key];

      if (option && option.disabled) {
        ret.push(option.value);
      }

      return ret;
    }, []);
  }, [allOptions, isMultiple]);

  var readOnlyValue = _react["default"].useMemo(function () {
    if (!isMultiple) {
      return [];
    }

    return Object.keys(allOptions).reduce(function (ret, key) {
      var option = allOptions[key];

      if (option && option.readOnly) {
        ret.push(option.value);
      }

      return ret;
    }, []);
  }, [allOptions, isMultiple]); // tag event


  var handleAddTag = function handleAddTag(inputValue, e) {
    onAddTag(inputValue, e);
  };

  var handleRemoveTag = function handleRemoveTag(value, index, e) {
    onDeselect(optionByValue(value), index, e);
  };

  var handleInputPaste = function handleInputPaste(e) {
    var clipboardData = e.clipboardData;
    var value = clipboardData.getData('text');
    pastedTextRef.current = value;
  }; // input event


  var handleInput = function handleInput(e) {
    var newInputValue = e.target.value;

    if (tokenWithEnter && pastedTextRef.current && /[\r\n]/.test(pastedTextRef.current)) {
      // CRLF will be treated as a single space for input element
      var replacedText = pastedTextRef.current.replace(/[\r\n]+$/, '').replace(/\r\n/g, ' ').replace(/[\r\n]/g, ' ');
      newInputValue = newInputValue.replace(replacedText, pastedTextRef.current);
    }

    pastedTextRef.current = null;
    var patchLabels = tokenSeparators ? (0, _utils.getSeparatedContent)(newInputValue, tokenSeparators) : null;

    if (mode === 'tags' && patchLabels) {
      newInputValue = '';
      onAddTag(patchLabels);
    } else if (mode !== 'tags') {
      onInputChange(newInputValue);
    } else {
      if (typeof maxTagTextLength === 'undefined' || newInputValue.length <= maxTagTextLength) {
        onInputChange(newInputValue);
      }
    }

    if (showSearch && onSearch) {
      onSearch(newInputValue);
    }

    if (onInput) {
      onInput(e);
    }
  };

  var handleInputKeyDown = function handleInputKeyDown(e) {
    if (!showSearch && e.keyCode !== _keyCode.KeyCode.TAB) {
      e.preventDefault();
    }

    if (onKeyDown) {
      onKeyDown(e);
    }
  };

  var handleInputBlur = function handleInputBlur(e) {
    if (mode === 'tags' && inputValue) {
      onAddTag(inputValue, e);
    }

    if (e.relatedTarget !== menuRef.current) {
      onBlur && onBlur(e);
    }
  };

  _react["default"].useEffect(function () {
    if (autoResize && inputRef.current && sizerRef.current) {
      var inputWidth = Math.ceil(sizerRef.current.offsetWidth);

      if (inputWidth > 0) {
        inputRef.current.style.width = "".concat(inputWidth, "px");
      } else {
        inputRef.current.style.width = '';
      }
    }
  });

  return {
    // utils
    getDefaultModeLabel: getDefaultModeLabel,
    optionByValue: optionByValue,
    labelByValue: labelByValue,
    disabledValue: disabledValue,
    readOnlyValue: readOnlyValue,
    // tag
    handleRemoveTag: handleRemoveTag,
    handleAddTag: handleAddTag,
    // input
    handleInput: handleInput,
    handleInputFocus: onFocus,
    handleInputBlur: handleInputBlur,
    handleInputKeyDown: handleInputKeyDown,
    handleInputPaste: handleInputPaste
  };
}