"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getChildType = exports.getActiveKeyFromValue = void 0;
exports.getOptionPropValue = getOptionPropValue;
exports.getOptionsFromChildren = void 0;
exports.getSeparatedContent = getSeparatedContent;
exports.isOptionGroup = exports.isOptionDivider = exports.isOption = exports.getValueFromProps = void 0;
exports.isSelectChildNode = isSelectChildNode;
exports.toArray = exports.noop = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _toArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _get2 = _interopRequireDefault(require("lodash/get"));

var _react = _interopRequireDefault(require("react"));

var noop = function noop() {};
/**
 * value 在 select 内部都通过数组保存，因此提供转换函数
 */


exports.noop = noop;

var toArray = function toArray(value) {
  var ret = [];

  if (typeof value === 'undefined') {
    ret = [];
  } else if (!Array.isArray(value)) {
    ret = [value];
  } else {
    ret = value;
  }

  return ret;
};
/**
 * 获取 child 的组件类型，对于函数组件目前通过 displayName 进行判断
 */


exports.toArray = toArray;

var getChildType = function getChildType(child) {
  if (child.type && child.type.displayName) {
    return child.type.displayName;
  }

  return '';
};

exports.getChildType = getChildType;

var isOption = function isOption(type) {
  return type === 'Option';
};

exports.isOption = isOption;

var isOptionGroup = function isOptionGroup(type) {
  return type === 'OptionGroup';
};

exports.isOptionGroup = isOptionGroup;

var isOptionDivider = function isOptionDivider(type) {
  return type === 'OptionDivider';
};

exports.isOptionDivider = isOptionDivider;

function getOptionPropValue(child, prop) {
  return child.props[prop];
}

function isSelectChildNode(child) {
  var type = getChildType(child);
  return isOption(type) || isOptionGroup(type) || isOptionDivider(type);
}
/**
 * 从 props 获取 value
 */


var getValueFromProps = function getValueFromProps(props) {
  var value = [];

  if ('value' in props) {
    value = toArray(props.value);
  } else if ('defaultValue' in props) {
    value = toArray(props.defaultValue);
  }

  return value;
};

exports.getValueFromProps = getValueFromProps;

var getActiveKeyFromValue = function getActiveKeyFromValue(valueState) {
  var value = (0, _get2["default"])(valueState, '0', '');

  if (value && (0, _typeof2["default"])(value) === 'object') {
    return value.value;
  } else {
    return value;
  }
};
/**
 * 从 children 中读取 option 节点
 */


exports.getActiveKeyFromValue = getActiveKeyFromValue;

var getOptionsFromChildren = function getOptionsFromChildren(children) {
  var optionInfos = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var keys = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];

  _react["default"].Children.forEach(children, function (child) {
    if (!child) {
      return;
    }

    var type = getChildType(child);

    if (isOptionGroup(type) || !isOption(type) && child.props && Array.isArray(child.props.children)) {
      getOptionsFromChildren(child.props.children, optionInfos, keys);
    } else if (isOption(type)) {
      var value = getOptionPropValue(child, 'value');
      optionInfos[value] = {
        value: value,
        label: getOptionPropValue(child, 'label') || getOptionPropValue(child, 'children'),
        disabled: child.props.disabled,
        readOnly: child.props.readOnly
      };
      keys.push(value);
    }
  });

  return {
    options: optionInfos,
    optionKeysOrder: keys.reduce(function (ans, cur, index) {
      ans[cur] = index;
      return ans;
    }, {})
  };
};

exports.getOptionsFromChildren = getOptionsFromChildren;

function getSeparatedContent(text, tokens) {
  if (!tokens || !tokens.length) {
    return null;
  }

  var match = false;

  function separate(str, _ref) {
    var _ref2 = (0, _toArray2["default"])(_ref),
        token = _ref2[0],
        restTokens = _ref2.slice(1);

    if (!token) {
      return [str];
    }

    var list = str.split(token);
    match = match || list.length > 1;
    return list.reduce(function (prevList, unitStr) {
      return [].concat((0, _toConsumableArray2["default"])(prevList), (0, _toConsumableArray2["default"])(separate(unitStr, restTokens)));
    }, []).filter(function (unit) {
      return unit;
    });
  }

  var list = separate(text, tokens);
  return match ? list : null;
}