"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireWildcard(require("react"));

var _Tooltip = _interopRequireDefault(require("../Tooltip"));

var _memoForwardRef = _interopRequireDefault(require("../utils/memoForwardRef"));

var _mergeStyleItem = _interopRequireDefault(require("../utils/mergeStyleItem"));

var _useStyles = _interopRequireDefault(require("../utils/useStyles"));

var _useTheme = _interopRequireDefault(require("../utils/useTheme"));

var _styled = require("./styled");

var _useBaseSlider2 = _interopRequireDefault(require("./useBaseSlider"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function defaultTipFormatter(str) {
  return str.toString();
}

var defaultStyles = {
  startCircle: '',
  endCircle: '',
  markPoint: '',
  markPointInclude: '',
  marksRow: '',
  markLabel: '',
  track: '',
  range: ''
};
var BaseSlider = (0, _memoForwardRef["default"])(function (props, ref) {
  var _useBaseSlider = (0, _useBaseSlider2["default"])(props),
      startCircleStyle = _useBaseSlider.startCircleStyle,
      endCircleStyle = _useBaseSlider.endCircleStyle,
      rangeStyle = _useBaseSlider.rangeStyle,
      start = _useBaseSlider.start,
      end = _useBaseSlider.end,
      innerMarksOnly = _useBaseSlider.innerMarksOnly,
      finalMarks = _useBaseSlider.finalMarks,
      markPropsMap = _useBaseSlider.markPropsMap,
      markNums = _useBaseSlider.markNums,
      movingType = _useBaseSlider.movingType,
      trackRef = _useBaseSlider.trackRef,
      handleStartMoveStartCircle = _useBaseSlider.handleStartMoveStartCircle,
      handleStartMoveEndCircle = _useBaseSlider.handleStartMoveEndCircle,
      handleTrackClick = _useBaseSlider.handleTrackClick,
      handleEnterStart = _useBaseSlider.handleEnterStart,
      handleLeaveStart = _useBaseSlider.handleLeaveStart,
      handleEnterEnd = _useBaseSlider.handleEnterEnd,
      handleLeaveEnd = _useBaseSlider.handleLeaveEnd,
      endTooltipOpen = _useBaseSlider.endTooltipOpen,
      startTooltipOpen = _useBaseSlider.startTooltipOpen,
      hideStart = _useBaseSlider.hideStart,
      vertical = _useBaseSlider.vertical,
      tooltipVisible = _useBaseSlider.tooltipVisible,
      otherProps = _useBaseSlider.otherProps,
      styles = _useBaseSlider.styles,
      disabled = _useBaseSlider.disabled,
      tooltipPlacement = _useBaseSlider.tooltipPlacement,
      markLabelDisabled = _useBaseSlider.markLabelDisabled,
      _useBaseSlider$tipFor = _useBaseSlider.tipFormatter,
      tipFormatter = _useBaseSlider$tipFor === void 0 ? defaultTipFormatter : _useBaseSlider$tipFor;

  var innerStyles = (0, _useStyles["default"])('slider', defaultStyles, styles);
  var theme = (0, _useTheme["default"])();
  var memoStartCircleStyle = (0, _react.useMemo)(function () {
    if (!innerStyles.startCircle.style) {
      return startCircleStyle;
    }

    return (0, _extends2["default"])({}, innerStyles.startCircle.style, startCircleStyle);
  }, [innerStyles.startCircle.style, startCircleStyle]);
  var startCircleNode = (0, _react.useMemo)(function () {
    return hideStart ? null : /*#__PURE__*/_react["default"].createElement(_styled.StyledCircle, {
      className: innerStyles.startCircle.className,
      $moving: movingType === 'start',
      theme: theme,
      $disabled: disabled,
      $vertical: vertical,
      style: memoStartCircleStyle,
      onMouseDown: handleStartMoveStartCircle,
      onMouseEnter: handleEnterStart,
      onMouseLeave: handleLeaveStart
    });
  }, [disabled, handleEnterStart, handleLeaveStart, handleStartMoveStartCircle, hideStart, innerStyles.startCircle.className, memoStartCircleStyle, movingType, theme, vertical]);
  var memoEndCircleStyle = (0, _react.useMemo)(function () {
    if (!innerStyles.endCircle.style) {
      return endCircleStyle;
    }

    return (0, _extends2["default"])({}, innerStyles.endCircle.style, endCircleStyle);
  }, [endCircleStyle, innerStyles.endCircle.style]); // 第二个控制点的渲染

  var endCircleNode = (0, _react.useMemo)(function () {
    return /*#__PURE__*/_react["default"].createElement(_styled.StyledCircle, {
      className: innerStyles.endCircle.className,
      $moving: movingType === 'end',
      theme: theme,
      $disabled: disabled,
      $vertical: vertical,
      style: memoEndCircleStyle,
      onMouseDown: handleStartMoveEndCircle,
      onMouseEnter: handleEnterEnd,
      onMouseLeave: handleLeaveEnd
    });
  }, [disabled, handleEnterEnd, handleLeaveEnd, handleStartMoveEndCircle, innerStyles.endCircle.className, memoEndCircleStyle, movingType, theme, vertical]);
  var startNode = startCircleNode;
  var endNode = endCircleNode;

  if (tooltipVisible) {
    startNode = startCircleNode && /*#__PURE__*/_react["default"].createElement(_Tooltip["default"], {
      title: tipFormatter(start),
      placement: tooltipPlacement,
      open: startTooltipOpen
    }, startCircleNode);
    endNode = /*#__PURE__*/_react["default"].createElement(_Tooltip["default"], {
      title: tipFormatter(end),
      placement: tooltipPlacement,
      open: endTooltipOpen
    }, endCircleNode);
  }

  var markPointsNode;
  var marksRowNode;

  if (finalMarks) {
    markPointsNode = markNums.map(function (keyNum) {
      var $include = keyNum > start && keyNum < end;
      var markProps = markPropsMap[keyNum];
      var innerStyle = markProps.style;

      if ($include && innerStyles.markPointInclude.style) {
        innerStyle = (0, _extends2["default"])({}, innerStyles.markPointInclude.style, innerStyle);
      } else if (!$include && innerStyles.markPoint.style) {
        innerStyle = (0, _extends2["default"])({}, innerStyles.markPoint.style, innerStyle);
      }

      var pointClassName = $include ? innerStyles.markPointInclude.className : innerStyles.markPoint.className;
      return /*#__PURE__*/_react["default"].createElement(_styled.StyledMarkPoint, {
        className: pointClassName,
        theme: theme,
        $include: $include,
        $disabled: disabled,
        key: keyNum,
        $vertical: vertical,
        style: innerStyle,
        onClick: markProps.onClick
      });
    });
  }

  var markLabelPropsMap = (0, _react.useMemo)(function () {
    var map = {};

    if (!finalMarks) {
      return map;
    }

    markNums.forEach(function (keyNum) {
      var label = finalMarks[keyNum];
      var markItem = label;
      var markProps = markPropsMap[keyNum];
      var markStyle = markProps.style;
      var markLabel = label;

      if (markItem.label) {
        markLabel = markItem.label;
      }

      if (markItem.style) {
        markStyle = (0, _extends2["default"])({}, markItem.style, markStyle);
      }

      if (innerStyles.markLabel.style) {
        markStyle = (0, _extends2["default"])({}, innerStyles.markLabel.style, markStyle);
      }

      map[keyNum] = {
        markProps: {
          className: innerStyles.markLabel.className,
          style: markStyle,
          onClick: markProps.onClick
        },
        markLabel: markLabel
      };
    });
    return map;
  }, [finalMarks, innerStyles.markLabel.className, innerStyles.markLabel.style, markNums, markPropsMap]);

  if (finalMarks && !markLabelDisabled) {
    marksRowNode = /*#__PURE__*/_react["default"].createElement(_styled.StyledMarksRow, {
      theme: theme,
      $vertical: vertical
    }, markNums.map(function (keyNum) {
      var _markLabelPropsMap$ke = markLabelPropsMap[keyNum],
          markProps = _markLabelPropsMap$ke.markProps,
          markLabel = _markLabelPropsMap$ke.markLabel;
      var labelNode;

      if ( /*#__PURE__*/_react["default"].isValidElement(markLabel)) {
        labelNode = markLabel;
      } else {
        labelNode = /*#__PURE__*/_react["default"].createElement(_styled.StyledLabelNode, {
          theme: theme,
          size: "s",
          fontWeight: "lighter"
        }, markLabel);
      }

      return /*#__PURE__*/_react["default"].createElement(_styled.StyledMarkLabel, (0, _extends2["default"])({
        theme: theme,
        $disabled: disabled,
        key: keyNum,
        $vertical: vertical
      }, markProps), labelNode);
    }));
  }

  var rangeStyleItem = (0, _react.useMemo)(function () {
    return (0, _mergeStyleItem["default"])({
      style: rangeStyle
    }, innerStyles.range);
  }, [innerStyles.range, rangeStyle]);
  return /*#__PURE__*/_react["default"].createElement(_styled.StyledRoot, (0, _extends2["default"])({
    theme: theme,
    $disabled: disabled,
    $vertical: vertical,
    $moving: movingType !== 'none',
    $disableClick: innerMarksOnly
  }, otherProps, {
    ref: ref
  }), /*#__PURE__*/_react["default"].createElement(_styled.StyledBaseRootTrack, (0, _extends2["default"])({}, innerStyles.track, {
    theme: theme,
    $disabled: disabled,
    $disableClick: innerMarksOnly,
    $moving: movingType !== 'none',
    $vertical: vertical,
    ref: trackRef,
    onClick: handleTrackClick
  }), startNode, endNode, markPointsNode, /*#__PURE__*/_react["default"].createElement(_styled.StyledRange, (0, _extends2["default"])({
    $moving: movingType !== 'none',
    theme: theme,
    $disabled: disabled,
    $vertical: vertical,
    $disableClick: innerMarksOnly
  }, rangeStyleItem))), marksRowNode);
});
var _default = BaseSlider;
exports["default"] = _default;