"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findClosest = findClosest;
exports.formatValue = formatValue;
exports.offsetCss = offsetCss;
exports.toPercent = toPercent;
exports.validNum = validNum;

function findClosest(value, nums, snapSize, marksOnly) {
  if (marksOnly) {
    nums.sort(function (a, b) {
      return Math.abs(a - value) - Math.abs(b - value);
    });
    return nums[0];
  }

  for (var i = 0; i < nums.length; i++) {
    var item = nums[i];

    if (Math.abs(value - item) < snapSize) {
      return item;
    }
  }

  return value;
}

function validNum(num, max, min) {
  if (num > max) {
    return max;
  }

  if (num < min) {
    return min;
  }

  return num;
}
/**
 * 计算偏移量
 * @param vertical
 * @param offset
 * @param numRange
 * @param start
 * @param size
 */


function offsetCss(vertical, offset, numRange, start, reverse, size) {
  var offsetPercent = size && reverse ? toPercent(offset + size - start, numRange) : toPercent(offset - start, numRange);
  var css = {};

  if (vertical) {
    css.top = reverse ? "".concat(100 - offsetPercent, "%") : "".concat(offsetPercent, "%");
  } else {
    css.left = reverse ? "".concat(100 - offsetPercent, "%") : "".concat(offsetPercent, "%");
  }

  if (!size) {
    return css;
  }

  var sizePercent = toPercent(size, numRange);

  if (vertical) {
    css.height = "".concat(sizePercent, "%");
  } else {
    css.width = "".concat(sizePercent, "%");
  }

  return css;
}

function formatValue(value, fractionDigits) {
  // 解决 js 0.2 + 0.1 !== 0.3 的问题
  return Number(value.toFixed(fractionDigits));
}

function toPercent(num, total) {
  return formatValue(num / total * 100, 3);
}