"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useSticky;

var _useEffectOnce2 = require("@muya-ui/utils/lib/hooks/useEffectOnce");

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = require("react");

var _utils = require("./utils");

function useSticky(props, divEl) {
  var offsetTop = props.offsetTop,
      target = props.target,
      onStatusChange = props.onStatusChange;

  var _useState = (0, _react.useState)(false),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      fixed = _useState2[0],
      setFixed = _useState2[1];

  var _useState3 = (0, _react.useState)(0),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      transform = _useState4[0],
      setTransform = _useState4[1];

  var _useState5 = (0, _react.useState)(false),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      scrollTick = _useState6[0],
      setScrollTick = _useState6[1];

  var _useState7 = (0, _react.useState)(false),
      _useState8 = (0, _slicedToArray2["default"])(_useState7, 2),
      targetScrollTick = _useState8[0],
      setTargetScrollTick = _useState8[1];

  var handleChange = (0, _react.useCallback)(function (status) {
    if (onStatusChange && fixed !== status) {
      onStatusChange(status);
    }
  }, [fixed, onStatusChange]);
  var calculate = (0, _react.useCallback)(function (top, target, current) {
    var transform = target.getBoundingClientRect().top;
    var distance = transform - current.getBoundingClientRect().top;
    var originalPosition = Math.abs(distance - (0, _utils.getScrollTop)(target));
    var scrollTop = (0, _utils.getScrollTop)(target);

    if (scrollTop === 0) {
      setFixed(false);
      handleChange(false);
      return;
    }

    if (originalPosition - scrollTop <= top) {
      setTransform(transform);
    } else {
      setTransform(transform + originalPosition - scrollTop - top);
    }

    setFixed(true);
    handleChange(true);
  }, [handleChange]);
  var onScroll = (0, _useEventCallback2.useEventCallback)(function () {
    setScrollTick(false);
    var current = divEl.current;
    if (!current) return;

    if ((0, _utils.isHidden)(current)) {
      return;
    }

    var top = Number(offsetTop);
    var scrollTop = (0, _utils.getScrollTop)(window);
    var topToPageTop = (0, _utils.getElementTop)(current);

    if (target) {
      if (!fixed) {
        return;
      }

      calculate(top, target(), current);
      return;
    }

    if (scrollTop + top > topToPageTop) {
      setFixed(true);
      handleChange(true);
      setTransform(0);
    } else {
      setFixed(false);
      handleChange(false);
    }
  }, [divEl, target, fixed, offsetTop]);
  var handleScroll = (0, _react.useCallback)(function () {
    !scrollTick && requestAnimationFrame(onScroll);
    setScrollTick(true);
  }, [onScroll, scrollTick]);
  var onTargetScroll = (0, _react.useCallback)(function () {
    setTargetScrollTick(false);
    var current = divEl.current;
    if (!current || !target) return;

    if ((0, _utils.isHidden)(current)) {
      return;
    }

    var top = Number(offsetTop);
    calculate(top, target(), current);
  }, [divEl, target, offsetTop, calculate]);
  var handleTargetScroll = (0, _react.useCallback)(function () {
    !targetScrollTick && requestAnimationFrame(onTargetScroll);
    setTargetScrollTick(true);
  }, [onTargetScroll, targetScrollTick]);
  (0, _useEffectOnce2.useEffectOnce)(function () {
    var current = divEl.current;
    var scroller = window;

    if (current) {
      scroller = (0, _utils.getScroller)(current);

      if (target) {
        window.addEventListener('scroll', handleScroll);
        target().addEventListener('scroll', handleTargetScroll, true);
      } else {
        scroller.addEventListener('scroll', handleScroll);
      }
    }

    return function () {
      if (target) {
        window.removeEventListener('scroll', handleScroll);
        target().removeEventListener('scroll', handleTargetScroll, true);
      }

      scroller.removeEventListener('scroll', handleScroll);
    };
  });
  var style = (0, _react.useMemo)(function () {
    if (!fixed) {
      return;
    }

    var style = {};

    if (Number(offsetTop)) {
      style.top = "".concat(offsetTop, "px");
    }

    if (transform) {
      style.transform = "translate3d(0, ".concat(transform, "px, 0)");
      style.WebkitTransform = "translate3d(0, ".concat(transform, "px, 0)");
      style.willChange = 'transform';
    }

    return style;
  }, [fixed, offsetTop, transform]);
  return {
    style: style,
    fixed: fixed,
    divEl: divEl
  };
}