"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getElementTop = getElementTop;
exports.getRootScrollTop = getRootScrollTop;
exports.getScrollTop = getScrollTop;
exports.getScroller = getScroller;
exports.isHidden = isHidden;

var _scrollIntoView = require("../utils/scrollIntoView");

function isWindow(val) {
  return val === window;
}

function getScroller(el) {
  var root = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : window;
  var node = el;

  while (node && node.tagName !== 'HTML' && node.nodeType === 1 && node !== root) {
    if ((0, _scrollIntoView.isScrollable)(node)) {
      if (node.tagName !== 'BODY') {
        return node;
      }

      if ((0, _scrollIntoView.isScrollable)(node.parentNode)) {
        return node;
      }
    }

    node = node.parentNode;
  }

  return root;
}

function isHidden(el) {
  var style = window.getComputedStyle(el);
  var hidden = style.display === 'none';
  var parentHidden = el.offsetParent === null && style.position !== 'fixed';
  return hidden || parentHidden;
}

function getScrollTop(el) {
  return 'scrollTop' in el ? el.scrollTop : el.pageYOffset;
}

function getRootScrollTop() {
  return window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;
}

function getElementTop(el, scroller) {
  if (isWindow(el)) {
    return 0;
  }

  var scrollTop = scroller ? getScrollTop(scroller) : getRootScrollTop();
  return el.getBoundingClientRect().top + scrollTop;
}