"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useInnerLoopSwipe;

var _useDebounce4 = require("@muya-ui/utils/lib/hooks/useDebounce");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _useForceUpdate2 = require("@muya-ui/utils/lib/hooks/useForceUpdate");

var _react = require("react");

var _generateLoopSwipeManager = _interopRequireWildcard(require("./generateLoopSwipeManager"));

var _useBaseSwipe2 = _interopRequireDefault(require("./useBaseSwipe"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function useInnerLoopSwipe(props, containerRef) {
  var _props$direction = props.direction,
      direction = _props$direction === void 0 ? 'horizontal' : _props$direction,
      _props$stepIndex = props.stepIndex,
      stepIndex = _props$stepIndex === void 0 ? -1 : _props$stepIndex,
      _props$itemIndex = props.itemIndex,
      itemIndex = _props$itemIndex === void 0 ? -1 : _props$itemIndex,
      equalNum = props.equalNum,
      onStepsChange = props.onStepsChange,
      gutter = props.gutter,
      _props$defaultIndex = props.defaultIndex,
      defaultIndex = _props$defaultIndex === void 0 ? 0 : _props$defaultIndex,
      children = props.children,
      transitionStatus = props.transitionStatus,
      enableDiffChildren = props.enableDiffChildren;
  var swipeManager = (0, _react.useRef)();
  var isH = direction === 'horizontal';
  var innerGutter = gutter || 0;
  var forceUpdate = (0, _useForceUpdate2.useForceUpdate)();
  var isForceUpdate = (0, _react.useRef)(false);

  var forceRefreshSwipe = function forceRefreshSwipe() {
    isForceUpdate.current = true;
    forceUpdate();
  };

  var endRectChange = function endRectChange() {
    if (onStepsChange && swipeManager.current) {
      onStepsChange(swipeManager.current);
    }

    forceRefreshSwipe();
  };

  var handleRectChange = function handleRectChange(itemRects, containerRect, cloneItemRects) {
    // 如果没有 clone 的项，表面当前的面板是无法进行循环的
    // 不需要做任何处理
    if (cloneItemRects.length <= 0) {
      swipeManager.current = (0, _generateLoopSwipeManager.generateDisabledLoopSwipeManager)();
      endRectChange();
      return;
    }

    var containerSize = isH ? containerRect.width : containerRect.height;
    swipeManager.current = (0, _generateLoopSwipeManager["default"])({
      items: itemRects,
      cloneItemRects: cloneItemRects,
      containerSize: containerSize,
      direction: direction,
      gutter: innerGutter,
      stepIndex: stepIndex,
      itemIndex: itemIndex,
      defaultIndex: defaultIndex
    });
    endRectChange();
  };

  var _useBaseSwipe = (0, _useBaseSwipe2["default"])({
    loop: true,
    children: children,
    direction: direction,
    equalNum: equalNum,
    gutter: gutter,
    containerRef: containerRef,
    onRectChange: handleRectChange,
    enableDiffChildren: enableDiffChildren
  }),
      nodes = _useBaseSwipe.nodes,
      finalDuration = _useBaseSwipe.finalDuration;

  var innerOffset = 0;
  var innerTransitionStatus = transitionStatus || 'on';

  if (swipeManager.current && itemIndex >= 0) {
    innerOffset = swipeManager.current.getItemOffset(itemIndex);
  } else if (swipeManager.current && stepIndex >= 0) {
    innerOffset = swipeManager.current.getStepOffset(stepIndex);
  } // 如果移到了临界点，需要解除锁定，并forceUpdate


  var resetPosition = function resetPosition() {
    if (!swipeManager.current) {
      return;
    }

    if (swipeManager.current.locked) {
      swipeManager.current.unlock();
      forceRefreshSwipe();
    }
  };

  if (isForceUpdate.current) {
    innerTransitionStatus = 'off';
  }

  var _useDebounce = (0, _useDebounce4.useDebounce)(resetPosition, finalDuration),
      _useDebounce2 = (0, _slicedToArray2["default"])(_useDebounce, 1),
      debounceResetPosition = _useDebounce2[0];

  (0, _react.useEffect)(function () {
    debounceResetPosition();
  });
  isForceUpdate.current = false;
  var position = isH ? "".concat(-innerOffset, "px, 0") : "0, ".concat(-innerOffset, "px");
  var panelStyle = (0, _react.useMemo)(function () {
    return {
      transform: "translate(".concat(position, ")")
    };
  }, [position]);
  return {
    finalDuration: finalDuration,
    manager: swipeManager,
    panelStyle: panelStyle,
    nodes: nodes,
    transitionStatus: innerTransitionStatus,
    // 只是为了测试而返回的一些操作
    handleRectChange: handleRectChange,
    swipeManager: swipeManager,
    resetPosition: resetPosition
  };
}