"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useSwipe;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _useLockScroll2 = require("@muya-ui/utils/lib/hooks/useLockScroll");

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = require("react");

var _shallowEqual = require("shallow-equal");

function useSwipe() {
  var _useState = (0, _react.useState)({
    itemIndex: 0,
    stepIndex: 0,
    offset: 0,
    hasNext: false,
    hasPrev: false,
    stepSize: 0,
    hasNextItem: false,
    hasPrevItem: false,
    itemSize: 0,
    transitionStatus: 'off'
  }),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      state = _useState2[0],
      setState = _useState2[1];

  var _useState3 = (0, _react.useState)(false),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      active = _useState4[0],
      setActive = _useState4[1];

  var swipeManager = (0, _react.useRef)();

  var updateState = function updateState(newOffset) {
    var manager = swipeManager.current;
    var index = manager.index,
        hasNext = manager.hasNext,
        hasPrev = manager.hasPrev,
        size = manager.size,
        itemIndex = manager.itemIndex,
        hasNextItem = manager.hasNextItem,
        hasPrevItem = manager.hasPrevItem,
        itemSize = manager.itemSize;
    setState({
      hasNext: hasNext,
      hasPrev: hasPrev,
      stepIndex: index,
      stepSize: size,
      offset: newOffset,
      itemIndex: itemIndex,
      hasNextItem: hasNextItem,
      hasPrevItem: hasPrevItem,
      itemSize: itemSize,
      transitionStatus: 'on'
    });
  };

  var firstStepsChange = (0, _react.useRef)(true);
  var onStepsChange = (0, _useEventCallback2.useEventCallback)(function (manager) {
    swipeManager.current = manager;
    var newState = {
      transitionStatus: 'off',
      stepIndex: firstStepsChange.current ? manager.index : state.stepIndex,
      stepSize: manager.size,
      hasNext: manager.hasNext,
      hasPrev: manager.hasPrev,
      itemIndex: firstStepsChange.current ? manager.itemIndex : state.itemIndex,
      hasNextItem: manager.hasNextItem,
      hasPrevItem: manager.hasPrevItem,
      itemSize: manager.itemSize,
      offset: manager.offset
    };
    firstStepsChange.current = false; // 步长发生了变化，极限情况处理一下

    if (newState.stepIndex !== undefined && newState.stepIndex >= manager.size) {
      newState.stepIndex = manager.size - 1;
    }

    if (!(0, _shallowEqual.shallowEqualObjects)(state, newState)) {
      setState(newState);
    }
  }); // 获取指定 item 所在的 step

  var getItemStep = function getItemStep(itemIndex) {
    var manager = swipeManager.current;

    if (manager) {
      return manager.getItemStep(itemIndex);
    }

    return 0;
  };

  var onPrev = (0, _useEventCallback2.useEventCallback)(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.prev();
      updateState();
    }
  });
  var onNext = (0, _useEventCallback2.useEventCallback)(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.next();
      updateState();
    }
  });
  var onGoTo = (0, _useEventCallback2.useEventCallback)(function (stepIndex) {
    var manager = swipeManager.current;

    if (manager && manager.hasStep(stepIndex)) {
      manager.goTo(stepIndex);
      updateState();
    }
  });
  var onItemPrev = (0, _useEventCallback2.useEventCallback)(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.prevItem();
      updateState();
    }
  });
  var onItemNext = (0, _useEventCallback2.useEventCallback)(function () {
    var manager = swipeManager.current;

    if (manager) {
      manager.nextItem();
      updateState();
    }
  });
  var onItemGoTo = (0, _useEventCallback2.useEventCallback)(function (itemIndex) {
    var manager = swipeManager.current;

    if (manager && manager.hasItem(itemIndex)) {
      manager.goToItem(itemIndex);
      updateState();
    }
  });
  var onWheel = (0, _useEventCallback2.useEventCallback)(function (e) {
    var manager = swipeManager.current;

    if (!manager || !active) {
      return;
    }

    var diff = manager.direction === 'horizontal' ? e.deltaX : e.deltaY;
    var currentOffset = state.offset || manager.offset;
    var newOffset = currentOffset + diff;

    if (newOffset < 0 || newOffset > manager.maxOffset) {
      return;
    }

    manager.updateOffset(newOffset);
    updateState(newOffset);
  });
  var onWheelActive = (0, _react.useCallback)(function () {
    setActive(true);
  }, []);
  var onWheelDisable = (0, _react.useCallback)(function () {
    setActive(false);
  }, []);
  (0, _useLockScroll2.useLockScroll)(active);
  return (0, _extends2["default"])({}, state, {
    manager: swipeManager.current,
    getItemStep: getItemStep,
    onStepsChange: onStepsChange,
    onNext: onNext,
    onPrev: onPrev,
    onGoTo: onGoTo,
    onItemNext: onItemNext,
    onItemPrev: onItemPrev,
    onItemGoTo: onItemGoTo,
    onWheel: onWheel,
    onWheelActive: onWheelActive,
    onWheelDisable: onWheelDisable
  });
}