"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calculateCloneNum = calculateCloneNum;
exports.calculateSize = calculateSize;
exports.cloneChildren = cloneChildren;
exports.getChildStyle = getChildStyle;
exports.rectSizeEqual = rectSizeEqual;
exports.transformItems = transformItems;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _useForkRef = require("@muya-ui/utils/lib/hooks/useForkRef");

var _react = require("react");

var _excluded = ["style"];

function rectSizeEqual(rectA, rectB) {
  if (!rectA || !rectB) {
    return false;
  }

  return rectA.width === rectB.width && rectA.height === rectB.height;
}

function transformItems(items) {
  return items.map(function (item) {
    return item.current.getBoundingClientRect();
  });
}

function getStepPoints(items, isH) {
  var firstPoint = isH ? items[0].left : items[0].top;
  return items.map(function (item) {
    return isH ? item.right - firstPoint : item.bottom - firstPoint;
  });
}
/**
 * 根据一个个 bounding rect 来计算，要覆盖掉 size 需要多少个 rect
 * @param rects 输入的矩形数组
 * @param isH 水平还是垂直
 * @param size 要覆盖到的区间
 */


function calculateCloneNum(rects, isH, size) {
  var items = getStepPoints(rects, isH);

  for (var i = 0, l = items.length; i < l; i++) {
    var item = items[i];

    if (item >= size) {
      return i + 1;
    }
  }
}

function calculateSize(items, isH) {
  var firstItem = items[0];
  var lastItem = items[items.length - 1];

  if (isH) {
    return lastItem.right - firstItem.left;
  }

  return lastItem.bottom - firstItem.top;
}

function notNull(value) {
  return value !== null;
}

function cloneMore(arr, num, refs, cloneItems) {
  var i = 0;

  while (i < num) {
    var child = arr[i];
    var innerChildRef = /*#__PURE__*/(0, _react.createRef)();
    var handleChildRef = (0, _useForkRef.forkRef)(refs[i], innerChildRef);
    cloneItems.push(innerChildRef);
    arr.push( /*#__PURE__*/(0, _react.cloneElement)(child, (0, _extends2["default"])({}, child.props, {
      key: arr.length,
      ref: handleChildRef
    })));
    i++;
  }
}

function cloneChildren(children) {
  var containerStyle = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var cloneNum = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var innerItems = [];
  var cloneItems = [];
  var newChildren = [];
  var formerRefs = [];

  _react.Children.forEach(children, function (child) {
    var _child$props = child.props,
        style = _child$props.style,
        otherChildProps = (0, _objectWithoutProperties2["default"])(_child$props, _excluded);
    var innerChildRef = /*#__PURE__*/(0, _react.createRef)();
    var handleChildRef = (0, _useForkRef.forkRef)(child.ref, innerChildRef);
    var childStyle = (0, _extends2["default"])({}, style, containerStyle);
    innerItems.push(innerChildRef);
    formerRefs.push(child.ref);
    newChildren.push( /*#__PURE__*/(0, _react.cloneElement)(child, (0, _extends2["default"])({}, otherChildProps, {
      key: notNull(child.key) ? child.key : newChildren.length,
      style: childStyle,
      ref: handleChildRef
    })));
  });

  var num = cloneNum;
  var realItemNum = newChildren.length;

  if (num > realItemNum && realItemNum > 0) {
    while (num > realItemNum) {
      num -= realItemNum;
      cloneMore(newChildren, realItemNum, formerRefs, cloneItems);
    }
  }

  if (num > 0) {
    cloneMore(newChildren, num, formerRefs, cloneItems);
  }

  return {
    refs: innerItems,
    cloneRefs: cloneItems,
    nodes: newChildren
  };
}
/**
 * 根据容器节点来设置子容器的样式，只有设置的了等分的时候需要
 * @param isH
 * @param containerRect
 * @param equalNum
 * @param gutter
 */


function getChildStyle(isH, containerRect, equalNum) {
  var gutter = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
  var childStyle = {};

  if (equalNum && containerRect && isH) {
    var containerSize = containerRect.width;
    childStyle.width = (containerSize - gutter * (equalNum - 1)) / equalNum;
    gutter && (childStyle.marginRight = gutter);
  } else if (equalNum && containerRect) {
    var _containerSize = containerRect.height;
    childStyle.height = (_containerSize - gutter * (equalNum - 1)) / equalNum;
    gutter && (childStyle.marginBottom = gutter);
  }

  return childStyle;
}