"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useSwitch;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = require("react");

function useSwitch(props, nodeRef) {
  var disabled = props.disabled,
      checked = props.checked,
      defaultChecked = props.defaultChecked,
      loading = props.loading,
      onChange = props.onChange,
      onClickProp = props.onClick,
      onMouseUpProp = props.onMouseUp,
      readOnly = props.readOnly;

  var _useRef = (0, _react.useRef)('checked' in props),
      isControlled = _useRef.current;

  var _useState = (0, _react.useState)(checked || defaultChecked || false),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      checkedState = _useState2[0],
      setCheckedState = _useState2[1];

  var setChecked = (0, _react.useCallback)(function (checked, e) {
    if (disabled) {
      return;
    }

    if (!isControlled) {
      setCheckedState(checked);
    }

    if (onChange) {
      onChange(checked, e);
    }
  }, [disabled, isControlled, onChange]);
  var onKeyDown = (0, _react.useCallback)(function (e) {
    if (e.keyCode === 37) {
      setChecked(false, e);
      /* istanbul ignore else */
    } else if (e.keyCode === 39) {
      setChecked(true, e);
    }
  }, [setChecked]);
  var onClick = (0, _react.useCallback)(function (e) {
    if (readOnly) return;

    if (disabled || loading) {
      e.stopPropagation();
      return;
    }

    var newChecked = !checkedState;
    setChecked(newChecked, e);
    /* istanbul ignore else */

    if (onClickProp) {
      onClickProp(newChecked, e);
    }
  }, [checkedState, disabled, loading, onClickProp, readOnly, setChecked]);
  var blur = (0, _react.useCallback)(function () {
    /* istanbul ignore next */
    if (nodeRef.current) {
      nodeRef.current.blur();
    }
  }, [nodeRef]); // Handle auto focus when click switch in Chrome

  var onMouseUp = (0, _react.useCallback)(function (e) {
    blur();
    /* istanbul ignore else */

    if (onMouseUpProp) {
      onMouseUpProp(e);
    }
  }, [blur, onMouseUpProp]);
  var focus = (0, _react.useCallback)(function () {
    /* istanbul ignore else */
    if (nodeRef.current) {
      nodeRef.current.focus();
    }
  }, [nodeRef]);
  return {
    checkedState: checkedState,
    setCheckedState: setCheckedState,
    onKeyDown: onKeyDown,
    onClick: onClick,
    onMouseUp: onMouseUp,
    focus: focus,
    blur: blur
  };
}