"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _uniq2 = _interopRequireDefault(require("lodash/uniq"));

var _pick2 = _interopRequireDefault(require("lodash/pick"));

var _get2 = _interopRequireDefault(require("lodash/get"));

var _findIndex2 = _interopRequireDefault(require("lodash/findIndex"));

var _react = _interopRequireWildcard(require("react"));

var _styledComponents = _interopRequireWildcard(require("styled-components"));

var _Animation = _interopRequireDefault(require("../Animation"));

var _memoForwardRef = _interopRequireDefault(require("../utils/memoForwardRef"));

var _typeHelper = require("../utils/type-helper");

var _useStyles = _interopRequireDefault(require("../utils/useStyles"));

var _useTheme = _interopRequireDefault(require("../utils/useTheme"));

var _withTheme = require("../utils/withTheme");

var _CardTabs = _interopRequireDefault(require("./CardTabs"));

var _const = require("./const");

var _LineTabs = _interopRequireDefault(require("./LineTabs"));

var _styled = require("./styled");

var _Tab = _interopRequireDefault(require("./Tab"));

var _TabPanel = _interopRequireDefault(require("./TabPanel"));

var _Img = require("../Img");

var _CapsuleTabs = _interopRequireDefault(require("./CapsuleTabs"));

var _excluded = ["visible", "children", "keepMounted", "theme"],
    _excluded2 = ["type", "panelAnimation", "defaultIndex", "panelTriggerResize", "panelKeepMounted", "tabPosition", "style"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var PanelItem = function PanelItem(props) {
  var visible = props.visible,
      children = props.children,
      type = props.type,
      theme = props.theme,
      _props$keepMounted = props.keepMounted,
      keepMounted = _props$keepMounted === void 0 ? false : _props$keepMounted;

  var hasVisibleRef = _react["default"].useRef(false);

  var mountedEl = /*#__PURE__*/_react["default"].createElement(_styled.BaseNode, {
    style: visible ? {} : {
      display: 'none'
    }
  }, type === 'fade' ? /*#__PURE__*/_react["default"].createElement(_Animation["default"].Fade, {
    "in": visible,
    timeout: theme.components.Tabs.panel.fade
  }, children) : /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, children));

  if (!keepMounted) {
    if (!visible) {
      return null;
    }

    if (type === 'fade') {
      return /*#__PURE__*/_react["default"].createElement(_Animation["default"].Fade, {
        "in": true,
        timeout: theme.components.Tabs.panel.fade
      }, children);
    } else {
      return /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, children);
    }
  } else if (keepMounted === 'lazy') {
    if (hasVisibleRef.current) {
      return mountedEl;
    } else if (visible) {
      hasVisibleRef.current = true;
      return mountedEl;
    } else {
      return null;
    }
  } else {
    return mountedEl;
  }
};

var SwipePanelItem = /*#__PURE__*/_react["default"].forwardRef(function (props, ref) {
  var visible = props.visible,
      children = props.children,
      _props$keepMounted2 = props.keepMounted,
      keepMounted = _props$keepMounted2 === void 0 ? false : _props$keepMounted2,
      theme = props.theme,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);

  var hasVisibleRef = _react["default"].useRef(false);

  var childNode = null;

  if (!keepMounted) {
    if (visible) {
      childNode = children;
    } else {
      childNode = null;
    }
  } else if (keepMounted === 'lazy') {
    if (hasVisibleRef.current) {
      childNode = children;
    } else if (visible) {
      hasVisibleRef.current = true;
      childNode = children;
    }
  } else {
    childNode = children;
  }

  return /*#__PURE__*/_react["default"].createElement("div", (0, _extends2["default"])({}, otherProps, {
    ref: ref
  }), childNode);
});

var TabsPure = (0, _memoForwardRef["default"])(function (props, ref) {
  var _props$type = props.type,
      type = _props$type === void 0 ? 'line' : _props$type,
      _props$panelAnimation = props.panelAnimation,
      panelAnimation = _props$panelAnimation === void 0 ? 'static' : _props$panelAnimation,
      defaultIndex = props.defaultIndex,
      _props$panelTriggerRe = props.panelTriggerResize,
      panelTriggerResize = _props$panelTriggerRe === void 0 ? false : _props$panelTriggerRe,
      panelKeepMountedProp = props.panelKeepMounted,
      tabPositionProp = props.tabPosition,
      style = props.style,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded2);
  var tabPosition = (0, _react.useMemo)(function () {
    if (typeof tabPositionProp === 'undefined' || type === 'card') {
      return 'top';
    }

    return tabPositionProp;
  }, [tabPositionProp, type]);
  var theme = (0, _useTheme["default"])();
  var panelKeepMounted = (0, _react.useMemo)(function () {
    // panelKeepMounted 计算逻辑，swipe 时默认值为 true，其他为 false
    if (typeof panelKeepMountedProp === 'undefined') {
      if (panelAnimation === 'swipe') {
        return true;
      } else {
        return false;
      }
    }

    return panelKeepMountedProp;
  }, [panelAnimation, panelKeepMountedProp]);
  var vertical = (0, _react.useMemo)(function () {
    return tabPosition === 'left' || tabPosition === 'right';
  }, [tabPosition]);

  var tabPanelChildrenArr = _react["default"].useMemo(function () {
    return _react["default"].Children.toArray(props.children).filter(function (child) {
      return /*#__PURE__*/_react["default"].isValidElement(child) && child.type.__MUYA_TABPANEL;
    });
  }, [props.children]);

  var _React$useState = _react["default"].useState(tabPanelChildrenArr.length ? 'tabPanel' : 'tab'),
      _React$useState2 = (0, _slicedToArray2["default"])(_React$useState, 2),
      renderMode = _React$useState2[0],
      setRenderMode = _React$useState2[1];

  var _React$useState3 = _react["default"].useState(defaultIndex || (0, _get2["default"])(tabPanelChildrenArr, '0.props.index', undefined)),
      _React$useState4 = (0, _slicedToArray2["default"])(_React$useState3, 2),
      _currentIdxKey = _React$useState4[0],
      setCurrentIdxKey = _React$useState4[1];

  var _React$useState5 = _react["default"].useState([_currentIdxKey]),
      _React$useState6 = (0, _slicedToArray2["default"])(_React$useState5, 2),
      initialIdxKeys = _React$useState6[0],
      setInitialIdxKeys = _React$useState6[1]; // 缓存上次选中的 tab，用于 swipe 动画


  var _React$useState7 = _react["default"].useState(undefined),
      _React$useState8 = (0, _slicedToArray2["default"])(_React$useState7, 2),
      preIdxKey = _React$useState8[0],
      setPreIdxKey = _React$useState8[1];

  var clearPreIdxKey = _react["default"].useCallback(function () {
    setPreIdxKey(undefined);
  }, [setPreIdxKey]); // TabPanel.props中的index的值


  var currentIdxKey = 'index' in props ? props['index'] : _currentIdxKey; // 当前选中currenIdxKey在tabPanelChildrenArr中的索引值，用于swipe定位

  var currentIdx = _react["default"].useMemo(function () {
    var _currentIdx = (0, _findIndex2["default"])(tabPanelChildrenArr, function (child) {
      return child.props.index === currentIdxKey;
    });

    return _currentIdx;
  }, [currentIdxKey, tabPanelChildrenArr]);

  var enableBadge = _react["default"].useMemo(function () {
    var badgeTabLen = 0;

    if (renderMode === 'tabPanel') {
      badgeTabLen = tabPanelChildrenArr.filter(function (child) {
        return !!(0, _get2["default"])(child, 'props.tabProps.badgeProps', false);
      }).length;
    } else {
      badgeTabLen = _react["default"].Children.toArray(props.children).filter(function (child) {
        return /*#__PURE__*/_react["default"].isValidElement(child) && child.props.badgeProps;
      }).length;
    }

    return !!badgeTabLen;
  }, [props.children, renderMode, tabPanelChildrenArr]); // 为了兼容只对开启了 badge 的 Tab 设置 overflow


  var innerStyle = (0, _react.useMemo)(function () {
    if (!enableBadge) {
      return (0, _extends2["default"])({
        overflow: 'hidden'
      }, style);
    }

    return style;
  }, [enableBadge, style]);
  var innerStyles = (0, _useStyles["default"])('tabs', _const.defaultStyles, props.styles);
  var pool = (0, _react.useContext)(_Img.ImgPoolContext);
  /** 用于切换 tab 时触发 resize 重新计算 */

  (0, _react.useEffect)(function () {
    if (panelTriggerResize) {
      pool.throttleCheck({
        type: 'resize'
      });
    }
  }, [currentIdxKey, panelTriggerResize, pool]);
  (0, _react.useEffect)(function () {
    // children中任一节点为TabPanel则认为开启了tabPanel模式
    if (tabPanelChildrenArr.length && renderMode === 'tab') {
      setRenderMode('tabPanel');
    } else if (!tabPanelChildrenArr.length && renderMode === 'tabPanel') {
      setRenderMode('tab');
    }

    var tabPanelKeyUniq = (0, _uniq2["default"])(tabPanelChildrenArr.map(function (child) {
      return child.props.index;
    }));

    if (tabPanelKeyUniq.length !== tabPanelChildrenArr.length) {
      (0, _devWarning2.devWarning)('[TabPanel]: duplicate indexes');
    }
  }, [props.children, renderMode, tabPanelChildrenArr]);

  var wrapChange = _react["default"].useCallback(function (index) {
    setPreIdxKey(currentIdxKey);
    setCurrentIdxKey(index);
    setInitialIdxKeys((0, _uniq2["default"])([].concat((0, _toConsumableArray2["default"])(initialIdxKeys), [index])));
    props.onChange && props.onChange(index);
  }, [currentIdxKey, initialIdxKeys, props]);

  if (renderMode === 'tab') {
    if (type === 'card') {
      return /*#__PURE__*/_react["default"].createElement(_CardTabs["default"], (0, _extends2["default"])({
        ref: ref,
        type: "card",
        defaultIndex: defaultIndex,
        enableBadge: enableBadge,
        vertical: vertical,
        tabPosition: tabPosition,
        style: innerStyle
      }, otherProps));
    } else if (type === 'capsule') {
      return /*#__PURE__*/_react["default"].createElement(_CapsuleTabs["default"], (0, _extends2["default"])({
        ref: ref,
        type: "capsule",
        defaultIndex: defaultIndex,
        enableBadge: enableBadge,
        vertical: vertical,
        tabPosition: tabPosition,
        style: innerStyle
      }, otherProps));
    } else {
      return /*#__PURE__*/_react["default"].createElement(_LineTabs["default"], (0, _extends2["default"])({
        ref: ref,
        type: "line",
        defaultIndex: defaultIndex,
        enableBadge: enableBadge,
        vertical: vertical,
        tabPosition: tabPosition,
        style: innerStyle
      }, otherProps));
    }
  } else {
    var TypedTabs = _LineTabs["default"];

    switch (type) {
      case 'card':
        TypedTabs = _CardTabs["default"];
        break;

      case 'line':
        TypedTabs = _LineTabs["default"];
        break;

      case 'capsule':
        TypedTabs = _CapsuleTabs["default"];
        break;

      default:
        break;
    }

    var tabsNode = /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, /*#__PURE__*/_react["default"].createElement(TypedTabs, (0, _extends2["default"])({
      ref: ref,
      type: type
    }, otherProps, {
      onChange: wrapChange,
      index: currentIdxKey,
      enableBadge: enableBadge,
      vertical: vertical,
      tabPosition: tabPosition,
      style: innerStyle
    }), tabPanelChildrenArr.map(function (child) {
      var _child$props = child.props,
          tab = _child$props.tab,
          index = _child$props.index,
          tabProps = _child$props.tabProps;
      return /*#__PURE__*/_react["default"].createElement(_Tab["default"], (0, _extends2["default"])({
        key: index,
        index: index
      }, tabProps), tab);
    })), panelAnimation === 'swipe' ? /*#__PURE__*/_react["default"].createElement(_styled.StyledPanelSwipeContainer, (0, _extends2["default"])({
      theme: theme,
      equalNum: 1,
      stepIndex: currentIdx // TODO: 去掉 swipePanel，把styles.panel提升到这层节点
      ,
      styles: (0, _pick2["default"])(props.styles, 'panel'),
      onTransitionEnd: clearPreIdxKey
    }, innerStyles.swipePanel), tabPanelChildrenArr.map(function (child) {
      return /*#__PURE__*/_react["default"].createElement(SwipePanelItem, {
        key: child.props.index,
        keepMounted: panelKeepMounted,
        visible: panelKeepMounted === 'lazy' ? initialIdxKeys.includes(child.props.index) : child.props.index === currentIdxKey || child.props.index === preIdxKey,
        theme: theme
      }, child);
    })) : /*#__PURE__*/_react["default"].createElement(_styled.StyledPanelDefaultContainer, (0, _extends2["default"])({}, innerStyles.panel, {
      theme: theme
    }), tabPanelChildrenArr.map(function (child) {
      return /*#__PURE__*/_react["default"].createElement(PanelItem, {
        type: panelAnimation,
        visible: child.props.index === currentIdxKey,
        key: child.props.index,
        theme: theme,
        keepMounted: panelKeepMounted
      }, child);
    })));

    if (tabPosition !== 'top') {
      return /*#__PURE__*/_react["default"].createElement(_styled.StyledTabsWrapper, (0, _extends2["default"])({}, innerStyles.wrapper, {
        $tabPosition: tabPosition
      }), tabsNode);
    } else {
      return tabsNode;
    }
  }
});
var TabsWithNoTheme = (0, _styledComponents["default"])(TabsPure).withConfig({
  displayName: "TabsWithNoTheme"
})(["position:relative;display:flex;", ""], function (props) {
  var theme = props.theme,
      _props$size = props.size,
      size = _props$size === void 0 ? 'm' : _props$size,
      _props$type2 = props.type,
      type = _props$type2 === void 0 ? 'line' : _props$type2,
      _props$tabPosition = props.tabPosition,
      tabPosition = _props$tabPosition === void 0 ? 'top' : _props$tabPosition;
  var token = theme.components.Tabs;
  var padding = token.padding[size];
  var paddingCss = (0, _styledComponents.css)(["padding-bottom:", "px;"], token.indicator.bgHeight); // line indicator 需要为extraContent增加paddingBottom，确保内容对齐

  if (type === 'line' && padding) {
    paddingCss = (0, _styledComponents.css)(["padding:", ";"], padding);
  }

  return (0, _styledComponents.css)(["", " ", " ", " & ", "{", ";}"], (tabPosition === 'right' || tabPosition === 'bottom') && 'order: 1;', (tabPosition === 'left' || tabPosition === 'right') && 'width: fit-content;', (tabPosition === 'top' || tabPosition === 'bottom') && 'align-items: center;', _styled.ExtraContentWrapper, paddingCss);
});
var Tabs = (0, _withTheme.withThemeForStyled)(TabsWithNoTheme);

var _default = (0, _typeHelper.withProperties)(Tabs, {
  Tab: _Tab["default"],
  TabPanel: _TabPanel["default"]
});

exports["default"] = _default;