"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useTabsContainer;

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _react = require("react");

var _Swipe = require("../Swipe");

function useTabsContainer(props) {
  var swipe = props.swipe,
      selectedIndex = props.selectedIndex,
      onChange = props.onChange;

  var _useSwipe = (0, _Swipe.useSwipe)(),
      onPrev = _useSwipe.onPrev,
      onNext = _useSwipe.onNext,
      onGoTo = _useSwipe.onGoTo,
      onStepsChange = _useSwipe.onStepsChange,
      getItemStep = _useSwipe.getItemStep,
      hasNext = _useSwipe.hasNext,
      hasPrev = _useSwipe.hasPrev,
      stepIndex = _useSwipe.stepIndex,
      manager = _useSwipe.manager;

  var containerOffset = (0, _react.useRef)(0);

  var updateContainerOffset = function updateContainerOffset() {
    if (manager) {
      containerOffset.current = manager.offset;
    }
  };

  var btnPrev = (0, _useEventCallback2.useEventCallback)(function () {
    updateContainerOffset();
    onPrev();
  });
  var btnNext = (0, _useEventCallback2.useEventCallback)(function () {
    updateContainerOffset();
    onNext();
  });

  var innerOnStepsChange = function innerOnStepsChange(manager) {
    onStepsChange(manager);
    updateContainerOffset();
    onChange && onChange();
  };

  var onTransitionEnd = function onTransitionEnd() {
    updateContainerOffset();
    onChange && onChange();
  };

  (0, _react.useEffect)(function () {
    if (swipe) {
      setTimeout(function () {
        var newStepIndex = getItemStep(selectedIndex);

        if (newStepIndex !== stepIndex) {
          updateContainerOffset();
          onGoTo(newStepIndex);
        }
      }, 500);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [selectedIndex]);
  (0, _react.useEffect)(function () {
    if (manager) {
      var newOffset = manager.offset;

      if (newOffset !== containerOffset.current) {
        onChange && onChange(newOffset - containerOffset.current);
        containerOffset.current = newOffset;
      }
    }
  });
  return {
    onStepsChange: innerOnStepsChange,
    onPrev: btnPrev,
    onNext: btnNext,
    hasNext: hasNext,
    hasPrev: hasPrev,
    stepIndex: stepIndex,
    onTransitionEnd: onTransitionEnd
  };
}