"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useTimePicker;

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _useDebounce4 = require("@muya-ui/utils/lib/hooks/useDebounce");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var _utils = require("../Calendar/utils");

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _utils2 = require("./utils");

var timeFormat = 'HH:mm:ss';

function useTimePicker(props, inputRef) {
  var value = props.value,
      defaultValue = props.defaultValue,
      onChange = props.onChange,
      onUIChange = props.onUIChange,
      popupOpen = props.popupOpen,
      _props$format = props.format,
      format = _props$format === void 0 ? timeFormat : _props$format,
      onFocus = props.onFocus,
      onClear = props.onClear,
      onPressEnter = props.onPressEnter,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      _props$inputDelay = props.inputDelay,
      inputDelay = _props$inputDelay === void 0 ? 200 : _props$inputDelay,
      options = props.options;
  var propsDate = (0, _react.useMemo)(function () {
    var v = value || defaultValue || '';

    var _transformConfigType = (0, _utils2.transformConfigType2Time)(v),
        _transformConfigType2 = (0, _slicedToArray2["default"])(_transformConfigType, 1),
        pDate = _transformConfigType2[0];

    return pDate;
  }, [defaultValue, value]);
  var propsValue = propsDate.isValid() ? propsDate.format(format) : '';

  var _useState = (0, _react.useState)(propsDate),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      stateDate = _useState2[0],
      setStateDate = _useState2[1];

  var _useState3 = (0, _react.useState)(false),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      statePopupOpen = _useState4[0],
      setPopupOpen = _useState4[1];

  var _useState5 = (0, _react.useState)(propsValue),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      stateValue = _useState6[0],
      setStateValue = _useState6[1];

  var propsDateIsValid = propsDate.isValid();
  var stateDateIsValid = stateDate.isValid();

  if ('value' in props && (propsDateIsValid && !stateDateIsValid || !propsDateIsValid && stateDateIsValid || propsDateIsValid && stateDateIsValid && !propsDate.isSame(stateDate, 's'))) {
    setStateDate(propsDate);
  }

  var finalPopupOpen = popupOpen !== undefined ? popupOpen : statePopupOpen;
  var finalValue = stateValue;

  if ('value' in props && !finalPopupOpen) {
    finalValue = propsValue;
  }

  var updateTime = function updateTime(newTime) {
    setStateDate(newTime);

    if (onChange) {
      onChange(newTime);
    }
  }; // 切换日历面板的显示隐藏


  var updateUI = function updateUI(innerPopupOpen) {
    var newState = {
      popupOpen: innerPopupOpen
    };
    setPopupOpen(innerPopupOpen);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var lastValidDate = (0, _react.useRef)(propsDate.isValid() ? propsDate : undefined);

  var selectByValue = function selectByValue(newValue) {
    var valueDate = (0, _utils2.transformStr2Date)(newValue, stateDate);
    var valid = valueDate.isValid();

    if (valid) {
      lastValidDate.current = valueDate;
    }

    if (newValue === '' && allowClear) {
      updateTime((0, _dayjs["default"])(''));
      return '';
    }

    if (lastValidDate.current) {
      updateTime(lastValidDate.current);
      return lastValidDate.current.format(format);
    }

    return '';
  };

  var _useDebounce = (0, _useDebounce4.useDebounce)(selectByValue, inputDelay),
      _useDebounce2 = (0, _slicedToArray2["default"])(_useDebounce, 2),
      debounceUpdateInputSelected = _useDebounce2[0],
      clearUpdateInputSelected = _useDebounce2[1];
  /**
   * 关闭日历面板
   */


  var closePopup = function closePopup() {
    if (!finalPopupOpen) {
      return;
    }

    updateUI(false);
    clearUpdateInputSelected();
  };

  var handleInputChange = (0, _useEventCallback2.useEventCallback)(function (e) {
    debounceUpdateInputSelected(e.target.value);
    setStateValue(e.target.value);
  });
  var handlePanelChange = (0, _useEventCallback2.useEventCallback)(function (date) {
    updateTime(date);
    setStateValue(date.format(format));
  });
  var handleFocus = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    updateUI(true);
  }, onFocus));
  var handleClear = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    clearUpdateInputSelected();
    setStateValue('');
    updateTime((0, _dayjs["default"])(''));
  }, onClear));
  var handleClickAway = (0, _useEventCallback2.useEventCallback)(closePopup);
  var handlePressEnter = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    if (inputRef.current) {
      inputRef.current.blur();
    }

    var newValue = selectByValue(finalValue);
    closePopup();
    setStateValue(newValue);
  }, onPressEnter));
  var handleOptionSelect = (0, _useEventCallback2.useEventCallback)(function (newDate) {
    if (!stateDate.isSame(newDate, 's')) {
      updateTime(newDate);
      setStateValue(newDate.format(format));
    }
  });
  var finalOptions = (0, _react.useMemo)(function () {
    return (0, _utils.formatCalendarOptions)(options, handleOptionSelect, function (opt) {
      if (typeof opt.date === 'function') {
        (0, _devWarning2.devWarning)('[TimePicker]: option.date() return invalid date.');
      } else {
        (0, _devWarning2.devWarning)('[TimePicker]: options passed invalid date.');
      }
    });
  }, [handleOptionSelect, options]);
  return {
    finalPopupOpen: finalPopupOpen,
    finalValue: finalValue,
    stateDate: stateDate,
    finalOptions: finalOptions,
    handleFocus: handleFocus,
    handleInputChange: handleInputChange,
    handlePressEnter: handlePressEnter,
    handleClickAway: handleClickAway,
    handleClear: handleClear,
    handlePanelChange: handlePanelChange
  };
}