"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useTimePickerNumberList;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _react = require("react");

var _scrollIntoView = _interopRequireDefault(require("../utils/scrollIntoView"));

var _useTheme = _interopRequireDefault(require("../utils/useTheme"));

var _excluded = ["children", "label", "max", "min", "step", "rowNum", "selected", "onChange", "disableNum", "hideDisabledNum", "defaultScrollBehavior", "styles"];

function useTimePickerNumberList(props) {
  var children = props.children,
      label = props.label,
      _props$max = props.max,
      max = _props$max === void 0 ? 59 : _props$max,
      _props$min = props.min,
      min = _props$min === void 0 ? 0 : _props$min,
      _props$step = props.step,
      step = _props$step === void 0 ? 1 : _props$step,
      _props$rowNum = props.rowNum,
      rowNum = _props$rowNum === void 0 ? 6 : _props$rowNum,
      propSelected = props.selected,
      onChange = props.onChange,
      disableNum = props.disableNum,
      hideDisabledNum = props.hideDisabledNum,
      _props$defaultScrollB = props.defaultScrollBehavior,
      defaultScrollBehavior = _props$defaultScrollB === void 0 ? 'auto' : _props$defaultScrollB,
      styles = props.styles,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var theme = (0, _useTheme["default"])();
  var innerStep = step >= 1 ? step : 1;
  var innerDefaultSelected = min - 1;

  var _useState = (0, _react.useState)(innerDefaultSelected),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      selected = _useState2[0],
      setSelected = _useState2[1];

  var finalSelected = selected;

  if (propSelected !== undefined) {
    finalSelected = propSelected;
  }

  var updateSelected = (0, _react.useCallback)(function (num) {
    setSelected(num);
    onChange && onChange(num);
  }, [onChange]);
  var selectedRef = (0, _react.useRef)(null);
  var scrollContainerRef = (0, _react.useRef)(null);
  var scrollBehavior = (0, _react.useRef)(defaultScrollBehavior);
  var items = (0, _react.useMemo)(function () {
    var nums = [];
    var start = min;

    while (start <= max) {
      nums.push(start);
      start += innerStep;
    }

    if (hideDisabledNum && disableNum) {
      nums = nums.filter(function (num) {
        return !disableNum(num);
      });
    }

    return nums.map(function (num) {
      var isSelected = num === finalSelected;
      var itemRef = isSelected ? selectedRef : undefined;
      var disabled = false;

      if (!hideDisabledNum && disableNum) {
        disabled = disableNum(num);
      }

      return {
        num: num,
        isSelected: isSelected,
        itemRef: itemRef,
        disabled: disabled,
        handleClick: function handleClick() {
          if (isSelected || disabled) {
            return;
          }

          scrollBehavior.current = 'smooth';
          updateSelected(num);
        }
      };
    });
  }, [disableNum, finalSelected, hideDisabledNum, innerStep, max, min, updateSelected]);
  (0, _react.useEffect)(function () {
    if (finalSelected > innerDefaultSelected && selectedRef.current && scrollContainerRef.current) {
      (0, _scrollIntoView["default"])(selectedRef.current, {
        parentNode: scrollContainerRef.current,
        behavior: scrollBehavior.current,
        time: theme.transition.spec.duration.normal,
        align: {
          top: 0
        }
      });
    }
  }, [finalSelected, innerDefaultSelected, min, selected, theme.transition.spec.duration.normal]);
  return {
    rowNum: rowNum,
    selectedRef: selectedRef,
    scrollContainerRef: scrollContainerRef,
    otherProps: otherProps,
    items: items,
    finalSelected: finalSelected
  };
}