"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useTimePickerPanel;
exports.getDisableTimeFns = getDisableTimeFns;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _includes2 = _interopRequireDefault(require("lodash/includes"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = require("react");

var _mergeBooleanFns = _interopRequireDefault(require("../utils/mergeBooleanFns"));

var _mergeBooleanHocFns = _interopRequireDefault(require("../utils/mergeBooleanHocFns"));

var _utils = require("./utils");

var _excluded = ["hourStep", "minuteStep", "secondStep", "rowNum", "format", "disabledHours", "disabledMinutes", "disabledSeconds", "hideDisabledOptions", "defaultScrollBehavior", "value", "defaultValue", "styles", "onChange", "disableTimeBefore", "disableTimeAfter"];

function isSameDate(dateA, dateB) {
  if (dateA && dateB && dateA.isSame(dateB, 'd')) {
    return true;
  }

  return false;
}

/**
 * 注意时间的边界
 * @param disableTimeBefore
 * @param disableTimeAfter
 * @param currentSelectedDate
 * @param ignoreBeforeDate 忽略日期，比如 disableTimeBefore 设置了 '2020-08-15 20:30:30' 只有 '20:30:30' 是生效的
 * @param ignoreAfterDate 忽略日期，比如 disableTimeAfter 设置了 '2020-08-15 20:30:30' 只有 '20:30:30' 是生效的
 */
function getDisableTimeFns(disableTimeBefore, disableTimeAfter, currentSelectedDate) {
  var ignoreBeforeDate = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  var ignoreAfterDate = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;
  var emptyFns = [undefined, undefined, undefined]; // 没有选择，则不可能有禁用函数
  // 没设置具体禁用时间也不可能有禁用函数

  if (!disableTimeAfter && !disableTimeBefore) {
    return emptyFns;
  } // 如果 ignoreDate 了，那默认就认为


  var beforeIsSameDate = true;
  var afterIsSameDate = true;

  if (!ignoreBeforeDate) {
    beforeIsSameDate = isSameDate(currentSelectedDate, disableTimeBefore);
  }

  if (!ignoreAfterDate) {
    afterIsSameDate = isSameDate(currentSelectedDate, disableTimeAfter);
  }

  if (!beforeIsSameDate && !afterIsSameDate) {
    return emptyFns;
  }

  var disabledHours = function disabledHours(num) {
    var hourIsAfter = false;
    var hourIsBefore = false;

    if (disableTimeAfter && afterIsSameDate) {
      var _hour = disableTimeAfter.hour();

      hourIsAfter = num > _hour;
    }

    if (disableTimeBefore && beforeIsSameDate) {
      var _hour2 = disableTimeBefore.hour();

      hourIsBefore = num < _hour2;
    }

    return hourIsAfter || hourIsBefore;
  };

  var disabledMinutes = function disabledMinutes(selectedHour) {
    return function (num) {
      var minIsAfter = false;
      var minIsBefore = false;

      if (disableTimeAfter && afterIsSameDate) {
        var _hour3 = disableTimeAfter.hour();

        if (_hour3 === selectedHour) {
          var min = disableTimeAfter.minute();
          minIsAfter = num > min;
        }
      }

      if (disableTimeBefore && beforeIsSameDate) {
        var _hour4 = disableTimeBefore.hour();

        if (_hour4 === selectedHour) {
          var _min = disableTimeBefore.minute();

          minIsBefore = num < _min;
        }
      }

      return minIsAfter || minIsBefore;
    };
  };

  var disabledSeconds = function disabledSeconds(selectedHour, selectedMinute) {
    return function (num) {
      var senIsAfter = false;
      var senIsBefore = false;

      if (disableTimeAfter && afterIsSameDate) {
        var _hour5 = disableTimeAfter.hour();

        var min = disableTimeAfter.minute();

        if (_hour5 === selectedHour && selectedMinute === min) {
          var sec = disableTimeAfter.second();
          senIsAfter = num > sec;
        }
      }

      if (disableTimeBefore && beforeIsSameDate) {
        var _hour6 = disableTimeBefore.hour();

        var _min2 = disableTimeBefore.minute();

        if (_hour6 === selectedHour && selectedMinute === _min2) {
          var _sec = disableTimeBefore.second();

          senIsBefore = num < _sec;
        }
      }

      return senIsAfter || senIsBefore;
    };
  };

  return [disabledHours, disabledMinutes, disabledSeconds];
}

function useTimePickerPanel(props) {
  var _props$hourStep = props.hourStep,
      hourStep = _props$hourStep === void 0 ? 1 : _props$hourStep,
      _props$minuteStep = props.minuteStep,
      minuteStep = _props$minuteStep === void 0 ? 1 : _props$minuteStep,
      _props$secondStep = props.secondStep,
      secondStep = _props$secondStep === void 0 ? 1 : _props$secondStep,
      rowNum = props.rowNum,
      _props$format = props.format,
      format = _props$format === void 0 ? 'HH:mm:ss' : _props$format,
      disabledHours = props.disabledHours,
      propDisabledMinutes = props.disabledMinutes,
      propDisabledSeconds = props.disabledSeconds,
      hideDisabledOptions = props.hideDisabledOptions,
      defaultScrollBehavior = props.defaultScrollBehavior,
      value = props.value,
      defaultValue = props.defaultValue,
      styles = props.styles,
      onChange = props.onChange,
      disableTimeBefore = props.disableTimeBefore,
      disableTimeAfter = props.disableTimeAfter,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var showMinute = (0, _includes2["default"])(format, 'm');
  var showSecond = (0, _includes2["default"])(format, 's');
  var propsDate = (0, _react.useMemo)(function () {
    var v = value || defaultValue || '';

    var _transformConfigType = (0, _utils.transformConfigType2Time)(v),
        _transformConfigType2 = (0, _slicedToArray2["default"])(_transformConfigType, 1),
        pDate = _transformConfigType2[0];

    return pDate;
  }, [defaultValue, value]);

  var _useMemo = (0, _react.useMemo)(function () {
    if (disableTimeBefore) {
      var _transformConfigType3 = (0, _utils.transformConfigType2Time)(disableTimeBefore),
          _transformConfigType4 = (0, _slicedToArray2["default"])(_transformConfigType3, 2),
          innerDate = _transformConfigType4[0],
          ignore = _transformConfigType4[1];

      if (innerDate.isValid()) {
        return [innerDate, ignore];
      }
    }

    return [undefined, false];
  }, [disableTimeBefore]),
      _useMemo2 = (0, _slicedToArray2["default"])(_useMemo, 2),
      memoDisableTimeBefore = _useMemo2[0],
      ignoreBefore = _useMemo2[1];

  var _useMemo3 = (0, _react.useMemo)(function () {
    if (disableTimeAfter) {
      var _transformConfigType5 = (0, _utils.transformConfigType2Time)(disableTimeAfter),
          _transformConfigType6 = (0, _slicedToArray2["default"])(_transformConfigType5, 2),
          innerDate = _transformConfigType6[0],
          ignore = _transformConfigType6[1];

      if (innerDate.isValid()) {
        return [innerDate, ignore];
      }
    }

    return [undefined, false];
  }, [disableTimeAfter]),
      _useMemo4 = (0, _slicedToArray2["default"])(_useMemo3, 2),
      memoDisableTimeAfter = _useMemo4[0],
      ignoreAfter = _useMemo4[1];

  var _useState = (0, _react.useState)(propsDate),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      stateDate = _useState2[0],
      setStateDate = _useState2[1];

  var currentSelectedDate = (0, _react.useMemo)(function () {
    if (stateDate.isValid()) {
      return stateDate;
    }

    return (0, _dayjs["default"])();
  }, [stateDate]);

  var _useMemo5 = (0, _react.useMemo)(function () {
    return getDisableTimeFns(memoDisableTimeBefore, memoDisableTimeAfter, currentSelectedDate, ignoreBefore, ignoreAfter);
  }, [currentSelectedDate, ignoreAfter, ignoreBefore, memoDisableTimeAfter, memoDisableTimeBefore]),
      _useMemo6 = (0, _slicedToArray2["default"])(_useMemo5, 3),
      innerDisabledHours = _useMemo6[0],
      innerDisabledMinutes = _useMemo6[1],
      innerDisabledSeconds = _useMemo6[2];

  var memoDisabledHours = (0, _react.useMemo)(function () {
    return (0, _mergeBooleanFns["default"])(disabledHours, innerDisabledHours);
  }, [disabledHours, innerDisabledHours]);
  var memoDisabledMinutes = (0, _react.useMemo)(function () {
    return (0, _mergeBooleanHocFns["default"])(propDisabledMinutes, innerDisabledMinutes);
  }, [innerDisabledMinutes, propDisabledMinutes]);
  var memoDisabledSeconds = (0, _react.useMemo)(function () {
    return (0, _mergeBooleanHocFns["default"])(propDisabledSeconds, innerDisabledSeconds);
  }, [innerDisabledSeconds, propDisabledSeconds]);
  var propsDateIsValid = propsDate.isValid();
  var stateDateIsValid = stateDate.isValid();
  var propSelectedHour = propsDateIsValid ? propsDate.hour() : -1;
  var propSelectedMinute = propsDateIsValid ? propsDate.minute() : -1;
  var propSelectedSecond = propsDateIsValid ? propsDate.second() : -1;

  if (!showMinute) {
    propSelectedMinute = 0;
  }

  if (!showSecond) {
    propSelectedSecond = 0;
  }

  var _useState3 = (0, _react.useState)(propSelectedHour),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      selectedHour = _useState4[0],
      setSelectedHour = _useState4[1];

  var _useState5 = (0, _react.useState)(propSelectedMinute),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      selectedMinute = _useState6[0],
      setSelectedMinute = _useState6[1];

  var _useState7 = (0, _react.useState)(propSelectedSecond),
      _useState8 = (0, _slicedToArray2["default"])(_useState7, 2),
      selectedSecond = _useState8[0],
      setSelectedSecond = _useState8[1];

  if ('value' in props && (propsDateIsValid && !stateDateIsValid || !propsDateIsValid && stateDateIsValid || propsDateIsValid && stateDateIsValid && !propsDate.isSame(stateDate, 's'))) {
    setStateDate(propsDate);
    setSelectedHour(propSelectedHour);
    setSelectedMinute(propSelectedMinute);
    setSelectedSecond(propSelectedSecond);
  }

  var disabledMinutes = (0, _react.useMemo)(function () {
    if (memoDisabledMinutes) {
      return memoDisabledMinutes(selectedHour);
    }
  }, [memoDisabledMinutes, selectedHour]);
  var disabledSeconds = (0, _react.useMemo)(function () {
    if (memoDisabledSeconds) {
      return memoDisabledSeconds(selectedHour, selectedMinute);
    }
  }, [memoDisabledSeconds, selectedHour, selectedMinute]);
  (0, _react.useEffect)(function () {
    if (onChange && selectedHour >= 0 && selectedMinute >= 0 && selectedSecond >= 0) {
      var newDate = stateDateIsValid ? stateDate.clone() : (0, _dayjs["default"])();
      newDate = newDate.set('h', selectedHour).set('m', selectedMinute).set('s', selectedSecond);

      if (!stateDateIsValid || !newDate.isSame(stateDate, 's')) {
        onChange(newDate);
      }
    }
  }, [onChange, selectedHour, selectedMinute, selectedSecond, stateDate, stateDateIsValid]);
  return {
    otherProps: otherProps,
    selectedHour: selectedHour,
    selectedMinute: selectedMinute,
    selectedSecond: selectedSecond,
    disabledHours: memoDisabledHours,
    disabledMinutes: disabledMinutes,
    disabledSeconds: disabledSeconds,
    hideDisabledOptions: hideDisabledOptions,
    showMinute: showMinute,
    showSecond: showSecond,
    hourStep: hourStep,
    minuteStep: minuteStep,
    secondStep: secondStep,
    rowNum: rowNum,
    styles: styles,
    defaultScrollBehavior: defaultScrollBehavior,
    handleHourChange: setSelectedHour,
    handleMinuteChange: setSelectedMinute,
    handleSecondChange: setSelectedSecond
  };
}