"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _useForkRef2 = require("@muya-ui/utils/lib/hooks/useForkRef");

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _omit2 = _interopRequireDefault(require("lodash/omit"));

var _react = _interopRequireWildcard(require("react"));

var _useStyles = _interopRequireDefault(require("../utils/useStyles"));

var _useTheme = _interopRequireDefault(require("../utils/useTheme"));

var _const = require("./const");

var _styled = require("./styled");

var _TransitionTreeNode = _interopRequireDefault(require("./TransitionTreeNode"));

var _TreeContext = _interopRequireDefault(require("./TreeContext"));

var _TreeNode = _interopRequireDefault(require("./TreeNode"));

var _useTree2 = _interopRequireDefault(require("./useTree"));

var _excluded = ["className", "style", "styles", "treeRef", "scrollable"],
    _excluded2 = ["transitionType", "transitionNodes", "onNodeTransitionEnd", "list", "containerProps", "wrapperProps", "handleScroll"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var defaultStyles = {
  scrollWrapper: '',
  wrapper: ''
};

var Tree = function Tree(props) {
  var className = props.className,
      style = props.style,
      styles = props.styles,
      treeRef = props.treeRef,
      _props$scrollable = props.scrollable,
      scrollable = _props$scrollable === void 0 ? false : _props$scrollable,
      restProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var treeScrollWrapperRef = (0, _react.useRef)(null);
  var innerTreeRef = (0, _react.useRef)(null);
  var theme = (0, _useTheme["default"])();

  var _useTree = (0, _useTree2["default"])(restProps, theme, innerTreeRef),
      transitionType = _useTree.transitionType,
      transitionNodes = _useTree.transitionNodes,
      onNodeTransitionEnd = _useTree.onNodeTransitionEnd,
      list = _useTree.list,
      containerProps = _useTree.containerProps,
      wrapperProps = _useTree.wrapperProps,
      handleScroll = _useTree.handleScroll,
      contextValue = (0, _objectWithoutProperties2["default"])(_useTree, _excluded2);

  var domProps = (0, _omit2["default"])(props, _const.allTreeProps);
  var virtual = contextValue.virtual,
      size = contextValue.size,
      treeData = contextValue.treeData;
  var handleChildrenRef = (0, _useForkRef2.useForkRef)(containerProps.ref, innerTreeRef);
  var handleRef = (0, _useForkRef2.useForkRef)(handleChildrenRef, treeRef);
  var treeStyles = (0, _react.useMemo)(function () {
    var treeStyles;

    if (styles) {
      var wrapper = styles.wrapper,
          scrollWrapper = styles.scrollWrapper;
      treeStyles = {
        wrapper: wrapper,
        scrollWrapper: scrollWrapper
      };
    }

    return treeStyles;
  }, [styles]);
  var innerStyles = (0, _useStyles["default"])('tree', defaultStyles, treeStyles);
  var innerStyle = (0, _react.useMemo)(function () {
    var innerStyle = (0, _extends2["default"])({}, innerStyles.wrapper.style, style);

    if (virtual) {
      return (0, _extends2["default"])({}, innerStyle, containerProps.style);
    }

    return innerStyle;
  }, [containerProps.style, innerStyles.wrapper.style, style, virtual]);
  var children = (0, _react.useMemo)(function () {
    return treeData.map(function (node) {
      return /*#__PURE__*/_react["default"].createElement(_TreeNode["default"], (0, _extends2["default"])({
        styles: styles,
        key: node.key,
        nodeKey: node.key,
        data: node
      }, node));
    });
  }, [styles, treeData]);
  var virtualChildren = (0, _react.useMemo)(function () {
    return /*#__PURE__*/_react["default"].createElement("div", wrapperProps, list.map(function (_ref) {
      var item = _ref.data;
      var data = item.data;
      var key = data.key;
      return /*#__PURE__*/_react["default"].createElement(_TransitionTreeNode["default"], (0, _extends2["default"])({
        styles: styles,
        key: key,
        nodeKey: key,
        data: data,
        lastChildFlags: item.lastChildFlags || [],
        transitionType: transitionType,
        transitionNodes: key === _const.TRANSITION_KEY ? transitionNodes : null,
        onTransitionEnd: onNodeTransitionEnd
      }, data));
    }));
  }, [list, onNodeTransitionEnd, styles, transitionNodes, transitionType, wrapperProps]);
  var treeNode = (0, _react.useMemo)(function () {
    return /*#__PURE__*/_react["default"].createElement(_styled.StyledTree, (0, _extends2["default"])({
      ref: handleRef,
      role: "tree",
      $size: size,
      $scrollable: scrollable,
      theme: theme
    }, domProps, {
      onScroll: handleScroll,
      className: [innerStyles.wrapper.className, className].join(' ').trim(),
      style: innerStyle
    }), virtual ? virtualChildren : children);
  }, [children, className, domProps, handleRef, handleScroll, innerStyle, innerStyles.wrapper.className, scrollable, size, theme, virtual, virtualChildren]);
  (0, _react.useLayoutEffect)(function () {
    if (scrollable && treeScrollWrapperRef.current && innerTreeRef.current) {
      if (treeScrollWrapperRef.current.scrollWidth <= treeScrollWrapperRef.current.offsetWidth) {
        innerTreeRef.current.style.width = '';
      } else {
        innerTreeRef.current.style.width = 'fit-content';
      }
    }
  });
  return /*#__PURE__*/_react["default"].createElement(_TreeContext["default"].Provider, {
    value: contextValue
  }, scrollable ? /*#__PURE__*/_react["default"].createElement(_styled.StyledTreeScrollWrapper, (0, _extends2["default"])({
    ref: treeScrollWrapperRef
  }, innerStyles.scrollWrapper, {
    theme: theme
  }), treeNode) : treeNode);
};

var _default = Tree;
exports["default"] = _default;