"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _typeof = require("@babel/runtime/helpers/typeof");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _RightIcon2 = require("@muya-ui/theme-light/lib/icons/RightIcon");

var _range2 = _interopRequireDefault(require("lodash/range"));

var _pick2 = _interopRequireDefault(require("lodash/pick"));

var _isEmpty2 = _interopRequireDefault(require("lodash/isEmpty"));

var _get2 = _interopRequireDefault(require("lodash/get"));

var _react = _interopRequireWildcard(require("react"));

var _Animation = _interopRequireDefault(require("../Animation"));

var _Checkbox = _interopRequireDefault(require("../Checkbox"));

var _ExpandIcon = _interopRequireDefault(require("../ExpandIcon"));

var _Spin = _interopRequireDefault(require("../Spin"));

var _ExpandWrapper = _interopRequireDefault(require("../styled/components/ExpandWrapper"));

var _mergeStyleItem = _interopRequireDefault(require("../utils/mergeStyleItem"));

var _useStyles = _interopRequireDefault(require("../utils/useStyles"));

var _useTheme = _interopRequireDefault(require("../utils/useTheme"));

var _styled = require("./styled");

var _TreeContext = require("./TreeContext");

var _useTreeNode2 = require("./useTreeNode");

var _excluded = ["data", "children", "title", "nodeKey", "isLeaf", "disabled", "checkable", "selectable", "draggable", "disableCheckbox", "icon", "expandIcon", "nodeRef", "styles", "lastChildFlags", "content"];

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

var defaultStyles = {
  node: '',
  selectedNode: '',
  checkedNode: '',
  nodeContent: '',
  nodeSelector: '',
  nodeIconWrapper: '',
  nodeTitle: '',
  childTree: ''
};

var ExpandHotZone = function ExpandHotZone(props) {
  var loading = props.loading,
      selected = props.selected,
      noChild = props.noChild,
      onExpand = props.onExpand,
      showLine = props.showLine,
      theme = props.theme,
      isLeaf = props.isLeaf,
      expanded = props.expanded,
      customIcon = props.customIcon;
  var token = theme.components.Tree;

  if (loading && noChild) {
    return /*#__PURE__*/_react["default"].createElement(_styled.StyleExpandHotZone, null, /*#__PURE__*/_react["default"].createElement(_Spin["default"], null));
  }

  if (!isLeaf) {
    return /*#__PURE__*/_react["default"].createElement(_styled.StyleExpandHotZone, {
      onClick: onExpand
    }, /*#__PURE__*/_react["default"].createElement(_ExpandWrapper["default"], {
      theme: theme,
      $expanded: expanded,
      $expandRotate: 90
    }, /*#__PURE__*/_react["default"].createElement(_ExpandIcon["default"], {
      icon: customIcon,
      iconSize: token.expandIconSize,
      DefaultIcon: token.expandIcon || _RightIcon2.RightIcon,
      buttonType: selected ? 'primary' : 'weak'
    })));
  } else {
    return /*#__PURE__*/_react["default"].createElement(_styled.StyleExpandHotZone, null, showLine && /*#__PURE__*/_react["default"].createElement(_styled.StyledExpandHotZoneLine, null));
  }
};

var NodeSelector = function NodeSelector(props) {
  var title = props.title,
      disableNativeTitle = props.disableNativeTitle,
      styles = props.styles,
      customIcon = props.customIcon,
      contentNode = props.contentNode;
  return /*#__PURE__*/_react["default"].createElement(_styled.StyledTreeNodeSelectorWrapper, (0, _extends2["default"])({
    title: typeof title === 'string' && !disableNativeTitle ? title : ''
  }, styles.nodeSelector), customIcon && /*#__PURE__*/_react["default"].createElement(_styled.StyledTreeIconWrapper, styles.nodeIconWrapper, customIcon), contentNode || /*#__PURE__*/_react["default"].createElement(_styled.StyledTitle, styles.nodeTitle, title));
};

var Line = function Line(props) {
  var level = props.level,
      size = props.size,
      lastChildFlags = props.lastChildFlags,
      contentHeight = props.contentHeight,
      theme = props.theme;
  var levels = (0, _range2["default"])(level);
  return /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, levels.map(function (l) {
    var isLastLevel = l === level - 1;
    return (!lastChildFlags[l + 1] || isLastLevel) && /*#__PURE__*/_react["default"].createElement(_styled.StyledLine, {
      key: l,
      theme: theme,
      $level: l,
      $size: size,
      $isLastChild: lastChildFlags[l + 1],
      $contentHeight: contentHeight
    });
  }));
};

var IndicatorLine = function IndicatorLine(props) {
  var theme = props.theme;
  return /*#__PURE__*/_react["default"].createElement(_styled.StyledIndicator, {
    theme: theme
  });
};

var TreeNode = function TreeNode(props) {
  var data = props.data,
      children = props.children,
      title = props.title,
      nodeKey = props.nodeKey,
      isLeafProp = props.isLeaf,
      nodeDisabled = props.disabled,
      nodeCheckable = props.checkable,
      nodeSelectable = props.selectable,
      nodeDraggable = props.draggable,
      disableCheckbox = props.disableCheckbox,
      nodeIcon = props.icon,
      nodeExpandIcon = props.expandIcon,
      nodeRef = props.nodeRef,
      styles = props.styles,
      lastChildFlags = props.lastChildFlags,
      content = props.content,
      domProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var contentRef = (0, _react.useRef)(null);
  var lastChildRef = (0, _react.useRef)(null);
  var childTreeRenderedRef = (0, _react.useRef)(false);
  var theme = (0, _useTheme["default"])();
  var context = (0, _TreeContext.useTreeContext)();
  var size = context.size,
      keyEntities = context.keyEntities,
      treeDraggable = context.draggable,
      treeIcon = context.icon,
      treeExpandIcon = context.expandIcon,
      renderNodeContent = context.renderNodeContent,
      renderNodeRightArea = context.renderNodeRightArea,
      renderCheckbox = context.renderCheckbox,
      showLine = context.showLine,
      disableNativeTitle = context.disableNativeTitle,
      renderAfterExpand = context.renderAfterExpand,
      virtual = context.virtual,
      dropLevelOffset = context.dropLevelOffset;
  var draggable = treeDraggable && nodeDraggable !== false; // 通过 tree 状态计算得到的节点状态

  var _useTreeNode = (0, _useTreeNode2.useTreeNode)(props, context, contentRef),
      dragNodeHighlight = _useTreeNode.dragNodeHighlight,
      nodeState = _useTreeNode.nodeState,
      expandAppear = _useTreeNode.expandAppear,
      isLeaf = _useTreeNode.isLeaf,
      disabled = _useTreeNode.disabled,
      checkable = _useTreeNode.checkable,
      onDragEnter = _useTreeNode.onDragEnter,
      onDragOver = _useTreeNode.onDragOver,
      onDragLeave = _useTreeNode.onDragLeave,
      onDrop = _useTreeNode.onDrop,
      onDragEnd = _useTreeNode.onDragEnd,
      onCheck = _useTreeNode.onCheck,
      onExpand = _useTreeNode.onExpand,
      onMouseEnter = _useTreeNode.onMouseEnter,
      onMouseLeave = _useTreeNode.onMouseLeave,
      onContextMenu = _useTreeNode.onContextMenu,
      onClick = _useTreeNode.onClick,
      onDoubleClick = _useTreeNode.onDoubleClick,
      onDragStart = _useTreeNode.onDragStart;

  var treeNodeStyles = (0, _react.useMemo)(function () {
    if (styles) {
      return (0, _pick2["default"])(styles, ['node', 'selectedNode', 'checkedNode', 'nodeContent', 'nodeSelector', 'nodeTitle', 'childTree']);
    }
  }, [styles]); // 连线，需要动态计算连线距底部的距离

  var lineBottom;

  if (showLine && !virtual) {
    var lastChildHeight = (0, _get2["default"])(lastChildRef, 'current.offsetHeight', 0);
    var lastChildContentHeight = (0, _get2["default"])(lastChildRef, 'current.firstChild.offsetHeight', 0);
    lineBottom = lastChildHeight / 2;

    if (lastChildHeight > lastChildContentHeight + 4) {
      lineBottom = lastChildHeight - (lastChildContentHeight + 4) / 2;
    }
  }

  var innerStyles = (0, _useStyles["default"])('tree', defaultStyles, treeNodeStyles);
  var selected = nodeState.selected,
      checked = nodeState.checked,
      halfChecked = nodeState.halfChecked,
      expanded = nodeState.expanded,
      loading = nodeState.loading,
      dragOver = nodeState.dragOver,
      dragOverGapTop = nodeState.dragOverGapTop,
      dragOverGapBottom = nodeState.dragOverGapBottom;

  var _ref = keyEntities[nodeKey] || {},
      _ref$level = _ref.level,
      level = _ref$level === void 0 ? 0 : _ref$level;

  var finalNodeStyleItem = innerStyles.node;

  if (selected) {
    finalNodeStyleItem = (0, _mergeStyleItem["default"])(finalNodeStyleItem, innerStyles.selectedNode);
  }

  if (checked) {
    finalNodeStyleItem = (0, _mergeStyleItem["default"])(finalNodeStyleItem, innerStyles.checkedNode);
  }

  var checkboxNode = (0, _react.useMemo)(function () {
    if (checkable) {
      if (renderCheckbox) {
        return renderCheckbox((0, _extends2["default"])({}, data, {
          checked: checked,
          halfChecked: halfChecked,
          disabled: disabled,
          disableCheckbox: disableCheckbox,
          onCheck: onCheck
        }));
      }

      return /*#__PURE__*/_react["default"].createElement(_Checkbox["default"], {
        size: "s",
        indeterminate: !checked && halfChecked,
        checked: checked,
        onChange: onCheck,
        disabled: disabled || disableCheckbox
      });
    }
  }, [checkable, checked, data, disableCheckbox, disabled, halfChecked, onCheck, renderCheckbox]);
  var customContentNode = (0, _react.useMemo)(function () {
    var rightAreaNode;

    if (renderNodeRightArea) {
      rightAreaNode = renderNodeRightArea((0, _extends2["default"])({}, props, nodeState, {
        isLeaf: isLeaf
      }));
    } // 自定义所有内容


    if (renderNodeContent) {
      return renderNodeContent((0, _extends2["default"])({}, props, nodeState, {
        isLeaf: isLeaf
      })); // 自定义右侧操作区
    } else if (rightAreaNode) {
      return /*#__PURE__*/_react["default"].createElement(_styled.StyledTreeNodeSelector, null, /*#__PURE__*/_react["default"].createElement(_styled.StyledTitle, innerStyles.nodeTitle, title), rightAreaNode);
    }
  }, [innerStyles.nodeTitle, isLeaf, nodeState, props, renderNodeContent, renderNodeRightArea, title]);
  var customExpandIcon = (0, _react.useMemo)(function () {
    var expandIcon = nodeExpandIcon || treeExpandIcon;

    if (expandIcon) {
      return /*#__PURE__*/_react["default"].createElement("span", {
        onClick: onExpand
      }, typeof expandIcon === 'function' ? expandIcon((0, _extends2["default"])({}, props, nodeState, {
        isLeaf: isLeaf
      })) : expandIcon);
    }
  }, [isLeaf, nodeExpandIcon, nodeState, onExpand, props, treeExpandIcon]);
  var customIcon = (0, _react.useMemo)(function () {
    var icon = nodeIcon || treeIcon;

    if (icon) {
      return typeof icon === 'function' ? icon((0, _extends2["default"])({}, props, nodeState, {
        isLeaf: isLeaf
      })) : icon;
    }
  }, [isLeaf, nodeIcon, nodeState, props, treeIcon]);
  var contentNode = (0, _react.useMemo)(function () {
    return /*#__PURE__*/_react["default"].createElement(_styled.StyledTreeNodeContent, (0, _extends2["default"])({
      ref: contentRef
    }, innerStyles.nodeContent, {
      draggable: !disabled && draggable || undefined,
      "aria-grabbed": !disabled && draggable || undefined,
      onDragStart: draggable ? onDragStart : undefined,
      onMouseEnter: onMouseEnter,
      onMouseLeave: onMouseLeave,
      onContextMenu: onContextMenu,
      onClick: onClick,
      onDoubleClick: onDoubleClick
    }), (dragOverGapTop || dragOverGapBottom) && /*#__PURE__*/_react["default"].createElement(IndicatorLine, {
      theme: theme
    }), /*#__PURE__*/_react["default"].createElement(ExpandHotZone, {
      loading: loading,
      selected: selected,
      noChild: (0, _isEmpty2["default"])(children),
      onExpand: onExpand,
      showLine: showLine,
      expanded: expanded,
      theme: theme,
      isLeaf: isLeaf,
      customIcon: customExpandIcon
    }), checkboxNode, /*#__PURE__*/_react["default"].createElement(NodeSelector, {
      customIcon: customIcon,
      contentNode: customContentNode,
      title: typeof customContentNode === 'string' ? customContentNode : title,
      disableNativeTitle: disableNativeTitle,
      styles: innerStyles
    }));
  }, [checkboxNode, children, customContentNode, customExpandIcon, customIcon, disableNativeTitle, disabled, dragOverGapBottom, dragOverGapTop, draggable, expanded, innerStyles, isLeaf, loading, onClick, onContextMenu, onDoubleClick, onDragStart, onExpand, onMouseEnter, onMouseLeave, selected, showLine, theme, title]);
  var childTreeNode = (0, _react.useMemo)(function () {
    var childTree;

    if (virtual || renderAfterExpand && !expanded && !childTreeRenderedRef.current) {
      childTree = null;
    } else if (children && children.length > 0) {
      childTreeRenderedRef.current = true;
      childTree = /*#__PURE__*/_react["default"].createElement(_Animation["default"].Collapse, {
        appear: expandAppear,
        "in": expanded
      }, /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, children.map(function (node, index) {
        if (index === children.length - 1 && showLine) {
          return /*#__PURE__*/_react["default"].createElement(TreeNode, (0, _extends2["default"])({
            key: node.key,
            nodeKey: node.key,
            parentKey: nodeKey,
            data: node,
            styles: styles,
            nodeRef: lastChildRef
          }, node));
        } else {
          return /*#__PURE__*/_react["default"].createElement(TreeNode, (0, _extends2["default"])({
            key: node.key,
            nodeKey: node.key,
            parentKey: nodeKey,
            data: node,
            styles: styles
          }, node));
        }
      })));
    }

    return /*#__PURE__*/_react["default"].createElement(_styled.StyledChildTree, innerStyles.childTree, childTree);
  }, [children, expandAppear, expanded, innerStyles.childTree, nodeKey, renderAfterExpand, showLine, styles, virtual]);
  return /*#__PURE__*/_react["default"].createElement(_styled.StyledTreeNode, (0, _extends2["default"])({}, domProps, finalNodeStyleItem, {
    role: "treeitem",
    theme: theme,
    $size: size,
    $isLeaf: isLeaf,
    $level: level,
    $selected: selected,
    $disabled: disabled,
    $dragOver: dragOver,
    $dragOverGapTop: dragOverGapTop,
    $dragOverGapBottom: dragOverGapBottom,
    $dragNodeHighlight: dragNodeHighlight,
    $dropLevelOffset: dropLevelOffset,
    $showLine: showLine,
    $virtual: virtual,
    $lineBottom: lineBottom,
    ref: nodeRef,
    onDragEnter: draggable ? onDragEnter : undefined,
    onDragOver: draggable ? onDragOver : undefined,
    onDragLeave: draggable ? onDragLeave : undefined,
    onDrop: draggable ? onDrop : undefined,
    onDragEnd: draggable ? onDragEnd : undefined
  }), showLine && virtual && level > 0 && /*#__PURE__*/_react["default"].createElement(Line, {
    theme: theme,
    level: level,
    size: size,
    lastChildFlags: lastChildFlags,
    contentHeight: (0, _get2["default"])(contentRef, 'current.offsetHeight', 0)
  }), contentNode, childTreeNode);
};

var _default = TreeNode;
exports["default"] = _default;