"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useTreeNode = void 0;

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _isUndefined2 = _interopRequireDefault(require("lodash/isUndefined"));

var _react = require("react");

var _types = require("./types");

var _utils = require("./utils");

/* istanbul ignore next */
var useTreeNode = function useTreeNode(props, context, contentRef) {
  var nodeKey = props.nodeKey,
      isLeafProp = props.isLeaf,
      nodeDisabled = props.disabled,
      nodeCheckable = props.checkable,
      nodeSelectable = props.selectable,
      disableCheckbox = props.disableCheckbox;

  var _useState = (0, _react.useState)(false),
      _useState2 = (0, _slicedToArray2["default"])(_useState, 2),
      hovering = _useState2[0],
      setHovering = _useState2[1];

  var state = (0, _react.useMemo)(function () {
    var state = (0, _utils.getTreeNodeState)(nodeKey, context);
    state.hovering = hovering;
    return state;
  }, [context, hovering, nodeKey]);
  var loaded = state.loaded,
      expanded = state.expanded,
      loading = state.loading;
  var expandOnClick = context.expandOnClick,
      checkOnClick = context.checkOnClick,
      keyEntities = context.keyEntities,
      loadData = context.loadData,
      treeDisabled = context.disabled,
      treeCheckable = context.checkable,
      notLeafCheckable = context.notLeafCheckable,
      treeSelectable = context.selectable,
      onNodeLoad = context.onNodeLoad,
      onNodeClick = context.onNodeClick,
      onNodeCheck = context.onNodeCheck,
      onNodeSelect = context.onNodeSelect,
      onNodeDoubleClick = context.onNodeDoubleClick,
      onNodeMouseEnter = context.onNodeMouseEnter,
      onNodeMouseLeave = context.onNodeMouseLeave,
      onNodeContextMenu = context.onNodeContextMenu,
      onNodeDragStart = context.onNodeDragStart,
      onNodeDragEnter = context.onNodeDragEnter,
      onNodeDragOver = context.onNodeDragOver,
      onNodeDragLeave = context.onNodeDragLeave,
      onNodeDragEnd = context.onNodeDragEnd,
      onNodeDrop = context.onNodeDrop,
      onNodeExpand = context.onNodeExpand; // 首次渲染不需要展开动画

  var _useState3 = (0, _react.useState)(false),
      _useState4 = (0, _slicedToArray2["default"])(_useState3, 2),
      expandAppear = _useState4[0],
      setExpandAppear = _useState4[1];

  var _useState5 = (0, _react.useState)(false),
      _useState6 = (0, _slicedToArray2["default"])(_useState5, 2),
      dragNodeHighlight = _useState6[0],
      setDragNodeHighlight = _useState6[1];

  var disabled = !!(treeDisabled || nodeDisabled);
  var hasChildren = (0, _react.useMemo)(function () {
    var _ref = keyEntities[nodeKey] || {},
        _ref$children = _ref.children,
        children = _ref$children === void 0 ? [] : _ref$children;

    return !!children.length;
  }, [keyEntities, nodeKey]);
  var isLeaf = (0, _react.useMemo)(function () {
    if (isLeafProp === false) {
      return false;
    }

    return isLeafProp || !loadData && !hasChildren || loadData && loaded && !hasChildren;
  }, [hasChildren, isLeafProp, loadData, loaded]);
  var checkable = (0, _react.useMemo)(function () {
    /**
     * 1. 树或者树节点 checkable 为 false，那么当前节点不能选中
     * 2. 树可以选中且当前为非叶子节点，且 notLeafCheckable 为 false，那么当前节点不能选中
     */
    if (!treeCheckable || nodeCheckable === false || treeCheckable && (nodeCheckable || (0, _isUndefined2["default"])(nodeCheckable)) && !isLeaf && !notLeafCheckable) {
      return false;
    }

    return treeCheckable;
  }, [treeCheckable, nodeCheckable, isLeaf, notLeafCheckable]);
  var selectable = (0, _react.useMemo)(function () {
    if (typeof nodeSelectable === 'boolean') {
      return nodeSelectable;
    }

    return treeSelectable;
  }, [nodeSelectable, treeSelectable]);
  var eventData = (0, _react.useMemo)(function () {
    return (0, _extends2["default"])({}, state, props, {
      nodeRef: contentRef
    });
  }, [contentRef, props, state]);
  var onExpand = (0, _useEventCallback2.useEventCallback)(function (e) {
    var trigger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _types.ExpandEventTrigger.ExpandIcon;
    e.stopPropagation(); // 事件触发的第一次展开需要动画

    if (!expandAppear) {
      setExpandAppear(true);
    }

    onNodeExpand(e, eventData, trigger);
  });
  var onSelect = (0, _useEventCallback2.useEventCallback)(function (e) {
    if (disabled) return;
    onNodeSelect(e, eventData);
  }, [disabled, eventData, onNodeSelect]);
  var onCheck = (0, _useEventCallback2.useEventCallback)(function (e) {
    var trigger = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _types.CheckEventTrigger.Checkbox;
    if (disabled) return;
    if (!checkable || disableCheckbox) return;
    onNodeCheck(e, eventData, trigger);
  });
  var onClick = (0, _useEventCallback2.useEventCallback)(function (e) {
    onNodeClick(e, eventData);

    if (selectable) {
      onSelect(e);
    }

    if (checkOnClick) {
      onCheck(e, _types.CheckEventTrigger.TreeNode);
    }

    if (expandOnClick) {
      onExpand(e, _types.ExpandEventTrigger.TreeNode);
    }
  });
  var onDoubleClick = (0, _useEventCallback2.useEventCallback)(function (e) {
    onNodeDoubleClick(e, eventData);
  });
  var onMouseEnter = (0, _useEventCallback2.useEventCallback)(function (e) {
    if (!hovering) {
      setHovering(true);
    }

    onNodeMouseEnter(e, eventData);
  });
  var onMouseLeave = (0, _useEventCallback2.useEventCallback)(function (e) {
    if (hovering) {
      setHovering(false);
    }

    onNodeMouseLeave(e, eventData);
  });
  var onContextMenu = (0, _useEventCallback2.useEventCallback)(function (e) {
    onNodeContextMenu(e, eventData);
  });
  var onDragStart = (0, _useEventCallback2.useEventCallback)(function (e) {
    e.stopPropagation();
    setDragNodeHighlight(true);
    onNodeDragStart(e, eventData);

    try {
      e.dataTransfer.setData('text/plain', '');
    } catch (error) {}
  });
  var onDragEnter = (0, _useEventCallback2.useEventCallback)(function (e) {
    e.preventDefault();
    e.stopPropagation();
    onNodeDragEnter(e, eventData);
  });
  var onDragOver = (0, _useEventCallback2.useEventCallback)(function (e) {
    e.preventDefault();
    e.stopPropagation();
    onNodeDragOver(e, eventData);
  });
  var onDragLeave = (0, _useEventCallback2.useEventCallback)(function (e) {
    e.stopPropagation();
    onNodeDragLeave(e, eventData);
  });
  var onDragEnd = (0, _useEventCallback2.useEventCallback)(function (e) {
    e.stopPropagation();
    setDragNodeHighlight(false);
    onNodeDragEnd(e, eventData);
  });
  var onDrop = (0, _useEventCallback2.useEventCallback)(function (e) {
    e.preventDefault();
    e.stopPropagation();
    setDragNodeHighlight(false);
    onNodeDrop(e, eventData);
  });
  (0, _react.useEffect)(function () {
    if (loading) return;

    if (loadData && expanded && !isLeaf && !loaded && !hasChildren) {
      onNodeLoad(eventData);
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [expanded, hasChildren, isLeaf, loadData, loaded]);
  return {
    dragNodeHighlight: dragNodeHighlight,
    nodeState: state,
    expandAppear: expandAppear,
    disabled: disabled,
    checkable: checkable,
    selectable: selectable,
    isLeaf: isLeaf,
    onSelect: onSelect,
    onCheck: onCheck,
    onClick: onClick,
    onDoubleClick: onDoubleClick,
    onMouseEnter: onMouseEnter,
    onMouseLeave: onMouseLeave,
    onContextMenu: onContextMenu,
    onDragStart: onDragStart,
    onDragEnter: onDragEnter,
    onDragOver: onDragOver,
    onDragLeave: onDragLeave,
    onDragEnd: onDragEnd,
    onDrop: onDrop,
    onExpand: onExpand
  };
};

exports.useTreeNode = useTreeNode;