"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calcDropPosition = calcDropPosition;
exports.calcSelectedKeys = calcSelectedKeys;
exports.conductCheck = conductCheck;
exports.conductExpandParent = conductExpandParent;
exports.convertDataToEntities = convertDataToEntities;
exports.convertDataToEventData = convertDataToEventData;
exports.diffCheckedKeys = diffCheckedKeys;
exports.getDragNodesKeys = getDragNodesKeys;
exports.getExpandRange = getExpandRange;
exports.getFlattenExpandedTreeData = getFlattenExpandedTreeData;
exports.getInitExpandedKeys = exports.getInitCheckedKeys = void 0;
exports.getKey = getKey;
exports.getMinimumRangeTransitionRange = getMinimumRangeTransitionRange;
exports.getPosition = getPosition;
exports.getTreeNodeState = getTreeNodeState;
exports.isCheckDisabled = isCheckDisabled;
exports.isFirstChild = isFirstChild;
exports.isLastChild = isLastChild;
exports.parseCheckedKeys = parseCheckedKeys;
exports.posToArray = posToArray;
exports.transformNode = transformNode;
exports.transformTreeData = transformTreeData;
exports.traverseDataNodes = traverseDataNodes;
exports.warningWithoutKey = warningWithoutKey;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _uniq2 = _interopRequireDefault(require("lodash/uniq"));

var _findIndex2 = _interopRequireDefault(require("lodash/findIndex"));

var _assignIn2 = _interopRequireDefault(require("lodash/assignIn"));

var _deepmerge = _interopRequireDefault(require("deepmerge"));

var _shallowEqual = require("shallow-equal");

var _const = require("./const");

var DRAG_SIDE_RANGE = 0.15;
var DRAG_MIN_GAP = 2;

function isCheckDisabled(node) {
  var _ref = node || {},
      disabled = _ref.disabled,
      disableCheckbox = _ref.disableCheckbox,
      checkable = _ref.checkable;

  return !!(disabled || disableCheckbox) || checkable === false;
}

function getKey(key, pos) {
  if (typeof key !== 'undefined') {
    return key;
  }

  return pos;
}

function getPosition(level, index) {
  return "".concat(level, "-").concat(index);
}
/**
 * 检测 key 存在和重复
 * @param treeData
 */


function warningWithoutKey() {
  var treeData = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var fieldNames = arguments.length > 1 ? arguments[1] : undefined;
  var keys = {};

  function dfs(list) {
    var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
    (list || []).forEach(function (treeNode) {
      var key = treeNode[fieldNames.key];
      var children = treeNode[fieldNames.children] || [];
      (0, _devWarning2.devWarning)("[Tree]: TreeNode must have a certain key: [".concat(path).concat(key, "]"), typeof key === 'undefined');
      var recordKey = String(key);
      (0, _devWarning2.devWarning)("[Tree]: Same 'key' exist in the Tree: ".concat(recordKey), !!keys[recordKey] && typeof key !== 'undefined');
      keys[recordKey] = true;
      dfs(children, "".concat(path).concat(recordKey, " > "));
    });
  }

  dfs(treeData);
}
/**
 * 遍历 tree
 * @param dataNodes
 * @param callback
 */


function traverseDataNodes(dataNodes, fieldNames, callback, externalGetKey) {
  var syntheticGetKey;

  if (externalGetKey) {
    if (typeof externalGetKey === 'string') {
      syntheticGetKey = function syntheticGetKey(node) {
        return node[externalGetKey];
      };
    } else if (typeof externalGetKey === 'function') {
      syntheticGetKey = function syntheticGetKey(node, index) {
        return externalGetKey(node, index);
      };
    }
  }

  function dfs(node, index, parent) {
    var children = node ? node[fieldNames.children] : dataNodes;
    var pos = node ? getPosition(parent.pos, index) : '0';

    if (node) {
      var key = syntheticGetKey ? syntheticGetKey(node, index) : typeof node[fieldNames.key] !== 'undefined' ? node[fieldNames.key] : pos;
      var _data = {
        node: node,
        index: index,
        pos: pos,
        key: key,
        parentPos: parent.pos,
        level: parent.level + 1
      };
      callback(_data);
    }

    if (children) {
      children.forEach(function (subNode, subIndex) {
        dfs(subNode, subIndex, {
          node: node,
          pos: pos,
          level: parent ? parent.level + 1 : -1
        });
      });
    }
  }

  dfs(null);
}
/**
 * Convert `treeData` into entity records.
 */


function convertDataToEntities(dataNodes, fieldNames) {
  var needTransition = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var externalGetKey = arguments.length > 3 ? arguments[3] : undefined;
  var posEntities = {};
  var keyEntities = {};
  var wrapper = {
    posEntities: posEntities,
    keyEntities: keyEntities
  };
  traverseDataNodes(dataNodes, fieldNames, function (item) {
    var node = item.node,
        index = item.index,
        pos = item.pos,
        key = item.key,
        parentPos = item.parentPos,
        level = item.level;
    var entity = {
      node: node,
      index: index,
      key: key,
      pos: pos,
      level: level
    };
    posEntities[pos] = entity;
    keyEntities[key] = entity;
    entity.parent = posEntities[parentPos];

    if (entity.parent) {
      entity.parent.children = entity.parent.children || [];
      entity.parent.children.push(entity);
    }
  }, externalGetKey);

  if (needTransition) {
    wrapper.keyEntities[_const.TRANSITION_KEY] = _const.TRANSITION_ENTITY;
  }

  return wrapper;
}
/**
 * Return selectedKeys according with multiple prop
 * @param selectedKeys
 * @param props
 * @returns [string]
 */


function calcSelectedKeys(selectedKeys, multiple) {
  if (Array.isArray(selectedKeys)) {
    if (multiple) {
      return selectedKeys;
    } else if (selectedKeys.length) {
      return [selectedKeys[0]];
    }
  } else {
    (0, _devWarning2.devWarning)('[Tree]: `selectedKeys` is not an array');
  }

  return [];
}
/**
 * If user use `autoExpandParent` we should get the list of parent node
 * @param keyList
 * @param keyEntities
 */


function conductExpandParent(keyList, keyEntities) {
  var expandedKeys = {};

  function conductUp(key) {
    if (expandedKeys[key]) return;
    var entity = keyEntities[key];
    if (!entity) return;
    expandedKeys[key] = true;
    var parent = entity.parent,
        node = entity.node;
    if (node.disabled) return;

    if (parent) {
      conductUp(parent.key);
    }
  }

  (keyList || []).forEach(function (key) {
    conductUp(key);
  });
  return Object.keys(expandedKeys).filter(function (key) {
    return !!keyEntities[key];
  }).map(function (key) {
    return keyEntities[key].key;
  });
}
/**
 * Parse `checkedKeys` to { checkedKeys, halfCheckedKeys } style
 */


function parseCheckedKeys(keys) {
  if (!keys) {
    return null;
  }

  var keyProps;

  if (Array.isArray(keys)) {
    keyProps = {
      checked: keys,
      halfChecked: []
    };
  } else if ((0, _typeof2["default"])(keys) === 'object') {
    keyProps = {
      checked: keys.checked || [],
      halfChecked: keys.halfChecked || []
    };
  } else {
    (0, _devWarning2.devWarning)('[Tree]: `checkedKeys` is not an array or an object');
    return null;
  }

  return keyProps;
}
/**
 * Conduct with keys.
 * @param keyList current key list
 * @param keyEntities key - dataEntity map
 * @param mode `fill` to fill missing key, `clean` to remove useless key
 */


function conductCheck(keyList, checked, keyEntities) {
  var checkStatus = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var checkedKeys = {};
  var halfCheckedKeys = {};
  (checkStatus.checkedKeys || []).forEach(function (key) {
    checkedKeys[key] = true;
  });
  (checkStatus.halfCheckedKeys || []).forEach(function (key) {
    halfCheckedKeys[key] = true;
  }); // Conduct up

  function conductUp(key) {
    if (checkedKeys[key] === checked) return;
    var entity = keyEntities[key];
    if (!entity) return;
    var children = entity.children,
        parent = entity.parent,
        node = entity.node;
    if (isCheckDisabled(node)) return; // Check child node checked status

    var everyChildChecked = true;
    var someChildChecked = false; // Child checked or half checked

    (children || []).filter(function (child) {
      return !isCheckDisabled(child.node);
    }).forEach(function (_ref2) {
      var childKey = _ref2.key;
      var childChecked = checkedKeys[childKey];
      var childHalfChecked = halfCheckedKeys[childKey];
      if (childChecked || childHalfChecked) someChildChecked = true;
      if (!childChecked) everyChildChecked = false;
    }); // Update checked status

    if (checked) {
      checkedKeys[key] = everyChildChecked;
    } else {
      checkedKeys[key] = false;
    }

    halfCheckedKeys[key] = someChildChecked;

    if (parent) {
      conductUp(parent.key);
    }
  } // Conduct down


  function conductDown(key) {
    if (checkedKeys[key] === checked) return;
    var entity = keyEntities[key];
    if (!entity) return;
    var children = entity.children,
        node = entity.node;
    if (isCheckDisabled(node)) return;
    checkedKeys[key] = checked;
    (children || []).forEach(function (child) {
      conductDown(child.key);
    });
  }

  function conduct(key) {
    var entity = keyEntities[key];

    if (!entity) {
      (0, _devWarning2.devWarning)("[Tree]: '".concat(key, "' does not exist in the tree."));
      return;
    }

    var children = entity.children,
        parent = entity.parent,
        node = entity.node;
    checkedKeys[key] = checked;
    if (isCheckDisabled(node)) return; // Conduct down

    (children || []).filter(function (child) {
      return !isCheckDisabled(child.node);
    }).forEach(function (child) {
      conductDown(child.key);
    }); // Conduct up

    if (parent) {
      conductUp(parent.key);
    }
  }

  (keyList || []).forEach(function (key) {
    conduct(key);
  });
  var checkedKeyList = [];
  var halfCheckedKeyList = []; // Fill checked list

  Object.keys(checkedKeys).forEach(function (key) {
    if (checkedKeys[key] && keyEntities[key]) {
      checkedKeyList.push(keyEntities[key].key);
    }
  }); // Fill half checked list

  Object.keys(halfCheckedKeys).forEach(function (key) {
    if (!checkedKeys[key] && halfCheckedKeys[key] && keyEntities[key]) {
      halfCheckedKeyList.push(keyEntities[key].key);
    }
  });
  return {
    checked: checkedKeyList,
    halfChecked: halfCheckedKeyList
  };
}

function getDragNodesKeys(dragNodeKey, keyEntities) {
  var dragNodesKeys = [dragNodeKey];
  var entity = keyEntities[dragNodeKey];

  function dfs() {
    var list = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    list.forEach(function (_ref3) {
      var key = _ref3.key,
          children = _ref3.children;
      dragNodesKeys.push(key);
      dfs(children);
    });
  }

  dfs(entity.children);
  return dragNodesKeys;
}

function posToArray(pos) {
  return pos.split('-');
} // eslint-disable-next-line max-params, complexity


function calcDropPosition(event, treeNode, prevNodeKey, startMousePosition) {
  var flattenExpandedTreeData = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : [];
  var expandedKeys = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : [];
  var keyEntities = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {};
  var virtual = arguments.length > 7 && arguments[7] !== undefined ? arguments[7] : false;
  var clientY = event.clientY,
      clientX = event.clientX,
      currentTarget = event.currentTarget;
  var nodeRef = treeNode.nodeRef,
      nodeKey = treeNode.nodeKey;

  var _getBoundingClientRec = nodeRef.current.getBoundingClientRect(),
      top = _getBoundingClientRec.top,
      bottom = _getBoundingClientRec.bottom,
      height = _getBoundingClientRec.height;

  var _getBoundingClientRec2 = currentTarget.getBoundingClientRect(),
      targetBottom = _getBoundingClientRec2.bottom;

  var horizontalMouseOffset = -clientX;

  if (startMousePosition) {
    horizontalMouseOffset = startMousePosition.x - clientX;
  }

  var rawDropLevelOffset = Math.floor(horizontalMouseOffset / 10); // 10 为每个层级偏移的距离

  var filteredExpandKeys = expandedKeys.filter(function (key) {
    return keyEntities[key] && keyEntities[key].children ? keyEntities[key].children.length : false;
  });
  var des = Math.max(height * DRAG_SIDE_RANGE, DRAG_MIN_GAP);
  var dropPosition = 0;
  var abstractDropNodeEntity = keyEntities[nodeKey];
  var dragOverNode = abstractDropNodeEntity;
  var dropAllowed = true; // 如果节点位置在元素的上半部分，判定为上一个元素的下方

  var nodeIndex = flattenExpandedTreeData.findIndex(function (flattenedNode) {
    return flattenedNode.data.key === abstractDropNodeEntity.key;
  });
  var prevNodeIndex = flattenExpandedTreeData.findIndex(function (flattenedNode) {
    return flattenedNode.data.key === prevNodeKey;
  });
  var dropLevelOffset = 0; // 在元素的上半部分

  if (clientY <= top + des) {
    // 每个层级的第一个节点，直接识别为他的上方
    if (isFirstChild(abstractDropNodeEntity) || prevNodeIndex > nodeIndex) {
      dropPosition = -1;
    } else {
      var _prevNodeIndex = nodeIndex <= 0 ? 0 : nodeIndex - 1;

      var _prevNodeKey = flattenExpandedTreeData[_prevNodeIndex].data.key;
      abstractDropNodeEntity = keyEntities[_prevNodeKey]; // 如果上一个节点的层级小于当前节点，然后滑动了

      for (var i = 0; i < rawDropLevelOffset; i++) {
        var parent = abstractDropNodeEntity.parent;

        if (parent && parent.level >= dragOverNode.level) {
          abstractDropNodeEntity = parent;
          dropLevelOffset += 1;
        } else {
          break;
        }
      }

      dropPosition = 1;
      if (virtual) dragOverNode = keyEntities[_prevNodeKey];
    }
  } else if (clientY >= bottom - des) {
    // 在元素的下半部分
    if ((abstractDropNodeEntity.children || []).length && filteredExpandKeys.includes(abstractDropNodeEntity.key) && nodeKey === abstractDropNodeEntity.key && (virtual || clientY < targetBottom - height)) {
      // 如果滑到最外层了那么不行
      // 如果节点元素是展开的那么在元素的下半部分时目标为下个元素的上方
      dropPosition = 0;
      dropAllowed = false;
    } else if (isLastChild(abstractDropNodeEntity) && clientY >= targetBottom - height && rawDropLevelOffset > -0.15) {
      // 因为此处虚拟树和普通树DOM结构不一致需要做特殊处理
      if (virtual) {
        var nextNodeIndex = nodeIndex >= flattenExpandedTreeData.length - 1 ? flattenExpandedTreeData.length - 1 : nodeIndex + 1;
        var nextNodeKey = flattenExpandedTreeData[nextNodeIndex].data.key;
        var nextNode = keyEntities[nextNodeKey]; // 如果上一个节点的层级小于当前节点，然后滑动了

        for (var _i = 0; _i < rawDropLevelOffset; _i++) {
          var _parent = abstractDropNodeEntity.parent;

          if (_parent && _parent.level >= nextNode.level) {
            abstractDropNodeEntity = _parent;
            dropLevelOffset += 1;
          } else {
            break;
          }
        }
      } else {
        var _parent2 = abstractDropNodeEntity.parent; // 最后一个节点的地方支持滑动，然后分别加入到每个父节点的下面

        if (_parent2 && _parent2.level >= dragOverNode.level) {
          abstractDropNodeEntity = _parent2;
        }
      }

      dropPosition = 1;
    } else {
      dropPosition = 1;
    }
  }

  return {
    dropPosition: dropPosition,
    dropTargetKey: abstractDropNodeEntity.key,
    dragOverNodeKey: dragOverNode.key,
    dropAllowed: dropAllowed,
    dropLevelOffset: dropLevelOffset
  };
}

function getTreeNodeState(key, state) {
  var expandedKeys = state.expandedKeys,
      selectedKeys = state.selectedKeys,
      loadKeys = state.loadKeys,
      checkedKeys = state.checkedKeys,
      dragOverNodeKey = state.dragOverNodeKey,
      dropPosition = state.dropPosition,
      keyEntities = state.keyEntities;
  var entity = keyEntities[key];
  var treeNodeState = {
    expanded: expandedKeys.indexOf(key) !== -1,
    selected: selectedKeys.indexOf(key) !== -1,
    loaded: loadKeys.loaded.indexOf(key) !== -1,
    loading: loadKeys.loading.indexOf(key) !== -1,
    checked: checkedKeys.checked.indexOf(key) !== -1,
    halfChecked: checkedKeys.halfChecked.indexOf(key) !== -1,
    pos: String(entity ? entity.pos : ''),
    dragOver: dragOverNodeKey === key && dropPosition === 0,
    dragOverGapTop: dragOverNodeKey === key && dropPosition === -1,
    dragOverGapBottom: dragOverNodeKey === key && dropPosition === 1,
    hovering: false
  };
  return treeNodeState;
}

var getInitExpandedKeys = function getInitExpandedKeys(props, keyEntities) {
  var _props$expandedKeys = props.expandedKeys,
      expandedKeys = _props$expandedKeys === void 0 ? [] : _props$expandedKeys,
      _props$defaultExpande = props.defaultExpandedKeys,
      defaultExpandedKeys = _props$defaultExpande === void 0 ? [] : _props$defaultExpande,
      autoExpandParent = props.autoExpandParent,
      _props$defaultExpandP = props.defaultExpandParent,
      defaultExpandParent = _props$defaultExpandP === void 0 ? true : _props$defaultExpandP,
      defaultExpandAll = props.defaultExpandAll;

  if ('expandedKeys' in props) {
    return expandedKeys;
  } else if (defaultExpandAll) {
    return Object.keys(keyEntities).filter(function (key) {
      return !!keyEntities[key];
    }).map(function (key) {
      return keyEntities[key].key;
    });
  } else {
    return autoExpandParent || defaultExpandParent ? conductExpandParent(defaultExpandedKeys, keyEntities) : defaultExpandedKeys;
  }
};

exports.getInitExpandedKeys = getInitExpandedKeys;

var getInitCheckedKeys = function getInitCheckedKeys(checkedKeysProp, checkStrictly, keyEntities) {
  var loadData = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  var checkedKeyEntity = parseCheckedKeys(checkedKeysProp) || {
    checked: [],
    halfChecked: []
  };
  var checked = checkedKeyEntity.checked,
      halfChecked = checkedKeyEntity.halfChecked;

  if (!checkStrictly) {
    var _conductCheck = conductCheck(checked, true, keyEntities);

    checked = _conductCheck.checked;
    halfChecked = _conductCheck.halfChecked;

    // 动态加载的情况下，halfChecked 在计算的基础上与 props 中的再做一次合并
    if (loadData) {
      halfChecked = (0, _uniq2["default"])([].concat((0, _toConsumableArray2["default"])(halfChecked), (0, _toConsumableArray2["default"])(checkedKeyEntity.halfChecked)));
    }
  }

  return {
    checked: checked,
    halfChecked: halfChecked
  };
};

exports.getInitCheckedKeys = getInitCheckedKeys;

function getFlattenExpandedTreeData(treeData, expandedKeys, fieldNames, dynamicPropsForNode) {
  var data = [];

  function dfs(list) {
    var parent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
    var len = list.length;
    return list.map(function (treeNode, index) {
      var pos = getPosition(parent ? parent.pos : '0', index);
      var mergedKey = getKey(treeNode[fieldNames.key], pos);
      var flattenNode = {
        parent: parent,
        pos: pos,
        children: [],
        data: transformNode(treeNode, fieldNames, dynamicPropsForNode),
        lastChildFlags: parent ? [].concat((0, _toConsumableArray2["default"])(parent.lastChildFlags), [index === len - 1]) : [index === len - 1]
      };
      data.push(flattenNode);

      if (expandedKeys.indexOf(mergedKey) > -1) {
        flattenNode.children = dfs(treeNode[fieldNames.children] || [], flattenNode);
      }

      return flattenNode;
    });
  }

  dfs(treeData);
  return data;
}

function diffCheckedKeys(checkedKeys, checkedKeysProp) {
  if (Array.isArray(checkedKeys) && Array.isArray(checkedKeysProp)) {
    return (0, _shallowEqual.shallowEqualArrays)(checkedKeys, checkedKeysProp);
  } else if (checkedKeys && !Array.isArray(checkedKeys) && !Array.isArray(checkedKeysProp)) {
    return (0, _shallowEqual.shallowEqualArrays)(checkedKeys.checked, checkedKeysProp.checked) && (0, _shallowEqual.shallowEqualArrays)(checkedKeys.halfChecked, checkedKeysProp.halfChecked);
  }

  return false;
}

function getExpandRange(shorter, longer, key) {
  var shorterStartIndex = (0, _findIndex2["default"])(shorter, function (_ref4) {
    var data = _ref4.data;
    return data.key === key;
  });
  var shorterEndNode = shorter[shorterStartIndex + 1];
  var longerStartIndex = (0, _findIndex2["default"])(longer, function (_ref5) {
    var data = _ref5.data;
    return data.key === key;
  });

  if (shorterEndNode) {
    var longerEndIndex = (0, _findIndex2["default"])(longer, function (_ref6) {
      var data = _ref6.data;
      return data.key === shorterEndNode.data.key;
    });
    return longer.slice(longerStartIndex + 1, longerEndIndex);
  }

  return longer.slice(longerStartIndex + 1);
}

function getMinimumRangeTransitionRange(list, height, itemHeight) {
  if (!height) {
    return list;
  }

  return list.slice(0, Math.ceil(height / itemHeight) + 1);
}

function transformNode(node, fieldNames, dynamicPropsForNode) {
  if (fieldNames.key !== 'key' && typeof node[fieldNames.key] !== 'undefined') {
    node.key = node[fieldNames.key];
  }

  if (typeof node.key === 'undefined') {
    (0, _devWarning2.devWarning)('[Tree]: node does not exist key.');
  }

  if (fieldNames.title !== 'title' && node[fieldNames.title]) {
    node.title = node[fieldNames.title];
  }

  if (fieldNames.children !== 'children' && node[fieldNames.children]) {
    node.children = node[fieldNames.children];
  }

  if (fieldNames.isLeaf !== 'isLeaf' && node[fieldNames.isLeaf]) {
    node.isLeaf = node[fieldNames.isLeaf];
  }

  if (typeof dynamicPropsForNode === 'function') {
    (0, _assignIn2["default"])(node, dynamicPropsForNode(node));
  }

  return node;
}

function transformTreeData(dataNodes, fieldNames, dynamicPropsForNode) {
  var data = (0, _deepmerge["default"])([], dataNodes);

  function dfs(node) {
    var children = node ? node[fieldNames.children] : data;

    if (node) {
      transformNode(node, fieldNames, dynamicPropsForNode);
    }

    if (children) {
      children.forEach(function (subNode) {
        dfs(subNode);
      });
    }
  }

  dfs(null);
  return data;
}

function isFirstChild(treeNodeEntity) {
  var posArr = posToArray(treeNodeEntity.pos);
  return Number(posArr[posArr.length - 1]) === 0;
}

function isLastChild(treeNodeEntity) {
  if (treeNodeEntity.parent) {
    var posArr = posToArray(treeNodeEntity.pos);
    return Number(posArr[posArr.length - 1]) === treeNodeEntity.parent.children.length - 1;
  }

  return false;
}

function convertDataToEventData(nodeKey, state) {
  var entity = state.keyEntities[nodeKey];
  var props = (0, _extends2["default"])({
    data: entity.node,
    nodeKey: entity.key,
    key: entity.key
  }, entity.node);
  return (0, _extends2["default"])({}, getTreeNodeState(nodeKey, state), props, {
    nodeRef: entity.node.ref
  });
}