"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.defaultFilterFunc = defaultFilterFunc;
exports.getCheckedKeysFromValue = void 0;
exports.getFilterTree = getFilterTree;
exports.toArray = exports.getValueFromProps = void 0;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _deepmerge = _interopRequireDefault(require("deepmerge"));

var toArray = function toArray(value) {
  var ret = [];

  if (typeof value === 'undefined') {
    ret = [];
  } else if (!Array.isArray(value)) {
    ret = [value];
  } else {
    ret = value;
  }

  return ret;
};

exports.toArray = toArray;

var getValueFromProps = function getValueFromProps(props) {
  var value = [];

  if ('value' in props) {
    value = toArray(props.value);
  } else if ('defaultValue' in props) {
    value = toArray(props.defaultValue);
  }

  return value;
};

exports.getValueFromProps = getValueFromProps;

var getCheckedKeysFromValue = function getCheckedKeysFromValue(props) {
  return getValueFromProps(props).map(function (item) {
    if (item && (0, _typeof2["default"])(item) === 'object') {
      return item.value;
    } else {
      return item;
    }
  });
};

exports.getCheckedKeysFromValue = getCheckedKeysFromValue;

function defaultFilterFunc(searchValue, node) {
  var match = false;

  if (searchValue) {
    if ("".concat(node.key).indexOf(searchValue) > -1 || "".concat(node.title).indexOf(searchValue) > -1) {
      match = true;
    }
  }

  return match;
}

function getFilterTree(treeNodes, searchValue, filterFunc) {
  if (!searchValue) {
    return treeNodes;
  }

  function mapFilteredNodeToData(node) {
    if (!node) return null;
    var match = false;

    if (filterFunc(searchValue, node)) {
      match = true;
    }

    var children = [];

    if (node.children) {
      children = node.children.map(mapFilteredNodeToData).filter(function (n) {
        return n;
      });
    }

    if (children.length || match) {
      return (0, _deepmerge["default"])({}, (0, _extends2["default"])({}, node, {
        children: children
      }));
    }

    return null;
  }

  return treeNodes.map(mapFilteredNodeToData).filter(function (node) {
    return node;
  });
}