"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useCalendar;

var _devWarning2 = require("@muya-ui/utils/lib/utils/devWarning");

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = _interopRequireDefault(require("react"));

var _utils = require("../Calendar/utils");

var _Locale = require("../Locale");

var _utils2 = require("./utils");

var _excluded = ["selectedWeek", "defaultWeek", "todayDate", "viewType", "viewDate", "options", "onUIChange", "onChange", "onSelect", "styles", "allowClear", "defaultViewDate", "disableWeek", "hideItemOutside", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel"];

function useCalendar(props) {
  var selectedWeek = props.selectedWeek,
      defaultWeek = props.defaultWeek,
      todayDate = props.todayDate,
      _props$viewType = props.viewType,
      viewType = _props$viewType === void 0 ? 'month' : _props$viewType,
      viewDate = props.viewDate,
      options = props.options,
      onUIChange = props.onUIChange,
      onChange = props.onChange,
      onSelect = props.onSelect,
      styles = props.styles,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      defaultViewDate = props.defaultViewDate,
      disableWeek = props.disableWeek,
      hideItemOutside = props.hideItemOutside,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var isControlled = ('selectedWeek' in props);

  var finalTodayDate = _react["default"].useMemo(function () {
    return (0, _dayjs["default"])(todayDate);
  }, [todayDate]);

  var weekFromProps = _react["default"].useMemo(function () {
    return (0, _utils2.formatWeek)(selectedWeek || defaultWeek);
  }, [defaultWeek, selectedWeek]);

  var uiStateFromProps = _react["default"].useMemo(function () {
    var innerViewDate;

    if (viewDate) {
      innerViewDate = viewDate;
    } else if (weekFromProps) {
      innerViewDate = (0, _utils2.getViewDateByWeek)(weekFromProps);
    } else if (defaultViewDate) {
      innerViewDate = defaultViewDate;
    } else {
      innerViewDate = finalTodayDate;
    }

    return {
      viewDate: (0, _dayjs["default"])(innerViewDate),
      viewType: viewType
    };
  }, [defaultViewDate, finalTodayDate, viewDate, viewType, weekFromProps]);

  var _React$useState = _react["default"].useState(uiStateFromProps),
      _React$useState2 = (0, _slicedToArray2["default"])(_React$useState, 2),
      uiState = _React$useState2[0],
      setUIState = _React$useState2[1];

  var _React$useState3 = _react["default"].useState(weekFromProps),
      _React$useState4 = (0, _slicedToArray2["default"])(_React$useState3, 2),
      stateSelectedWeek = _React$useState4[0],
      setSelectedWeek = _React$useState4[1];

  var _React$useState5 = _react["default"].useState(),
      _React$useState6 = (0, _slicedToArray2["default"])(_React$useState5, 2),
      stateHoveredWeek = _React$useState6[0],
      setHoveredWeek = _React$useState6[1];

  var finalUIState = (0, _extends2["default"])({}, uiState);

  if ('viewDate' in props) {
    finalUIState.viewDate = uiStateFromProps.viewDate;
  }

  if ('viewType' in props) {
    finalUIState.viewType = uiStateFromProps.viewType;
  }

  var updateUI = function updateUI(inputState) {
    var newState = (0, _extends2["default"])({}, finalUIState, inputState);
    setUIState(newState);

    if (onUIChange) {
      onUIChange(newState);
    }
  };

  var updateSelectedWeek = function updateSelectedWeek(week) {
    setSelectedWeek(week);

    if (onChange) {
      onChange(week);
    }

    if (onSelect && week) {
      onSelect(week);
    }
  };

  var selectWeek = function selectWeek(week) {
    if (allowClear && (0, _utils2.isSameWeek)(week, stateSelectedWeek)) {
      updateSelectedWeek();
      return;
    }

    updateSelectedWeek(week);
    var newViewDate = (0, _utils2.getViewDateByWeek)(week);

    if (!(0, _utils.isSame)(newViewDate, finalUIState.viewDate, finalUIState.viewType)) {
      updateUI({
        viewDate: newViewDate
      });
    }
  };

  var handleItemClick = (0, _useEventCallback2.useEventCallback)(function (date) {
    if (finalUIState.viewType === 'year') {
      updateUI({
        viewType: 'month',
        viewDate: finalUIState.viewDate.set('month', date.get('month'))
      });
      return;
    }

    if (finalUIState.viewType === 'decade') {
      updateUI({
        viewType: 'month',
        viewDate: finalUIState.viewDate.set('year', date.get('year'))
      });
      return;
    }

    var selectedWeek = (0, _utils2.formatWeek)(date); // 从面板上点击，这里不可能是 undefined ，看看后续是否要处理

    if (selectedWeek) {
      selectWeek(selectedWeek);
    }
  });
  var handleItemHover = (0, _useEventCallback2.useEventCallback)(function (date, status) {
    if (status === 'normal' || status === 'outside' || status === 'range') {
      setHoveredWeek([date.startOf('w'), date.endOf('w')]);
    }
  });
  var handleLeavePanel = (0, _useEventCallback2.useEventCallback)(function () {
    if (stateHoveredWeek) {
      setHoveredWeek(undefined);
    }
  });
  var handleOption = (0, _useEventCallback2.useEventCallback)(function (week) {
    selectWeek(week);
  });

  var handleMove = function handleMove(step, type) {
    return function () {
      updateUI({
        viewDate: (0, _utils.moveOne)(finalUIState.viewDate, type, step)
      });
    };
  };

  var handlePrevMonth = (0, _useEventCallback2.useEventCallback)(handleMove(-1, 'month'));
  var handleNextMonth = (0, _useEventCallback2.useEventCallback)(handleMove(1, 'month'));
  var yearBtnType = finalUIState.viewType === 'decade' ? 'decade' : 'year';
  var handlePrevYear = (0, _useEventCallback2.useEventCallback)(handleMove(-1, yearBtnType));
  var handleNextYear = (0, _useEventCallback2.useEventCallback)(handleMove(1, yearBtnType));
  var handleSwitch = (0, _useEventCallback2.useEventCallback)(function (type) {
    updateUI({
      viewType: type
    });
  });
  var locale = (0, _Locale.useLocale)();

  var finalOptions = _react["default"].useMemo(function () {
    if (!options || options.length < 1) {
      return;
    }

    var innerOpts = [];
    options.forEach(function (opt) {
      var innerWeek = (0, _utils2.formatWeek)(opt.week);

      if (!innerWeek) {
        (0, _devWarning2.devWarning)('[WeekCalendar]: options passed invalid week.');
        return;
      }

      innerOpts.push({
        label: opt.label,
        disabled: opt.disabled,
        onClick: function onClick() {
          handleOption(innerWeek);
        }
      });
    });
    return innerOpts;
  }, [handleOption, options]); // 当国际化变了，得更新内存中 dayjs 对象的国际化


  var updateLocale = function updateLocale() {
    var localeKey = locale.dayjsLocaleKey;
    updateUI({
      viewDate: finalUIState.viewDate.locale(localeKey)
    });

    if (stateSelectedWeek) {
      updateSelectedWeek([stateSelectedWeek[0].locale(localeKey), stateSelectedWeek[1].locale(localeKey)]);
    }

    if (stateHoveredWeek) {
      setHoveredWeek([stateHoveredWeek[0].locale(localeKey), stateHoveredWeek[1].locale(localeKey)]);
    }
  };

  _react["default"].useEffect(function () {
    var localeKey = locale.dayjsLocaleKey; // 任一变了就可以了

    var localeChange = localeKey !== finalUIState.viewDate.locale();

    if (!localeChange) {
      return;
    }

    updateLocale();
  });

  _react["default"].useEffect(function () {
    if (isControlled) {
      setSelectedWeek(weekFromProps);
    }
  }, [isControlled, weekFromProps]);

  return {
    // 处理过的数据
    finalUIState: finalUIState,
    finalSelectedWeek: stateSelectedWeek,
    stateHoveredWeek: stateHoveredWeek,
    finalOptions: finalOptions,
    finalTodayDate: finalTodayDate,
    otherProps: otherProps,
    // 事件处理
    handleItemClick: handleItemClick,
    handlePrevMonth: handlePrevMonth,
    handleNextMonth: handleNextMonth,
    handlePrevYear: handlePrevYear,
    handleNextYear: handleNextYear,
    handleSwitch: handleSwitch,
    handleItemHover: handleItemHover,
    handleLeavePanel: handleLeavePanel,
    // 透传
    styles: styles,
    disableWeek: disableWeek,
    hideItemOutside: hideItemOutside,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    // 另外一个 hooks 中使用
    updateSelectedWeek: updateSelectedWeek,
    // for test
    updateUI: updateUI,
    updateLocale: updateLocale,
    selectWeek: selectWeek
  };
}