"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = useWeekPicker;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _useEventCallback2 = require("@muya-ui/utils/lib/hooks/useEventCallback");

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _pick2 = _interopRequireDefault(require("lodash/pick"));

var _dayjs = _interopRequireDefault(require("dayjs"));

var _react = _interopRequireDefault(require("react"));

var _forkHandler = _interopRequireDefault(require("../utils/forkHandler"));

var _useWeekCalendar = _interopRequireDefault(require("./useWeekCalendar"));

var _utils = require("./utils");

var _excluded = ["options", "todayDate", "disableWeek", "renderDecadeItem", "renderMonthItem", "renderYearItem", "renderMonthLabel", "getPopupContainer", "onSelect", "viewType", "viewDate", "defaultViewDate", "disabled", "defaultValue", "value", "allowClear", "onFocus", "onClear", "format", "formatSeparator", "popupOpen", "onChange", "onUIChange", "onCalendarChange", "suffixNode", "placement", "triggerId", "popperProps", "flip", "arrowPointAtCenter", "calendarStyles", "styles"];

function useWeekPicker(props) {
  var options = props.options,
      todayDate = props.todayDate,
      disableWeek = props.disableWeek,
      renderDecadeItem = props.renderDecadeItem,
      renderMonthItem = props.renderMonthItem,
      renderYearItem = props.renderYearItem,
      renderMonthLabel = props.renderMonthLabel,
      getPopupContainer = props.getPopupContainer,
      onSelect = props.onSelect,
      viewType = props.viewType,
      viewDate = props.viewDate,
      defaultViewDate = props.defaultViewDate,
      disabled = props.disabled,
      defaultValue = props.defaultValue,
      value = props.value,
      _props$allowClear = props.allowClear,
      allowClear = _props$allowClear === void 0 ? true : _props$allowClear,
      onFocus = props.onFocus,
      onClear = props.onClear,
      _props$format = props.format,
      format = _props$format === void 0 ? 'YYYY-MM-DD' : _props$format,
      _props$formatSeparato = props.formatSeparator,
      formatSeparator = _props$formatSeparato === void 0 ? ' - ' : _props$formatSeparato,
      popupOpen = props.popupOpen,
      onChange = props.onChange,
      onUIChange = props.onUIChange,
      onCalendarChange = props.onCalendarChange,
      suffixNode = props.suffixNode,
      _props$placement = props.placement,
      placement = _props$placement === void 0 ? 'bottom-start' : _props$placement,
      triggerId = props.triggerId,
      popperProps = props.popperProps,
      flip = props.flip,
      arrowPointAtCenter = props.arrowPointAtCenter,
      calendarStyles = props.calendarStyles,
      styles = props.styles,
      otherProps = (0, _objectWithoutProperties2["default"])(props, _excluded);
  var isControlled = ('value' in props);

  var valuePropWeek = _react["default"].useMemo(function () {
    return (0, _utils.formatWeek)(value || defaultValue);
  }, [defaultValue, value]);

  var formatWeekStr = _react["default"].useMemo(function () {
    var tplFn;

    if (typeof format === 'function') {
      tplFn = format;
    } else {
      tplFn = function tplFn(week) {
        return "".concat(week[0].format(format)).concat(formatSeparator).concat(week[1].format(format));
      };
    }

    return function (week) {
      if (week) {
        return tplFn(week);
      }

      return '';
    };
  }, [format, formatSeparator]);

  var _React$useState = _react["default"].useState(false),
      _React$useState2 = (0, _slicedToArray2["default"])(_React$useState, 2),
      statePopupOpen = _React$useState2[0],
      setPopupOpen = _React$useState2[1]; // const [stateValue, setStateValue] = useState(formatWeekStr(valuePropWeek));


  var finalPopupOpen = popupOpen !== undefined ? popupOpen : statePopupOpen;
  var calendarProps = (0, _pick2["default"])(props, ['selectedWeek', 'viewType', 'viewDate', 'disableWeek', 'defaultViewDate']);

  if (defaultValue) {
    calendarProps.defaultWeek = defaultValue;
  }

  if (isControlled && !('selectedWeek' in props) && !finalPopupOpen) {
    calendarProps.selectedWeek = valuePropWeek;
  }

  calendarProps.onUIChange = (0, _useEventCallback2.useEventCallback)(function (inputState) {
    var newState = (0, _extends2["default"])({
      popupOpen: finalPopupOpen
    }, inputState);

    if (onUIChange) {
      onUIChange(newState);
    }
  });
  calendarProps.onChange = (0, _useEventCallback2.useEventCallback)(function (week) {
    if (!onChange) {
      return;
    }

    if (onChange) {
      onChange(week ? week : [(0, _dayjs["default"])(''), (0, _dayjs["default"])('')]);
    }
  });
  /**
   * 这里复用 hooks 的逻辑需要注意的点：
   * * 事件处理一定要在这个 hook 里处理
   */

  var calendarState = (0, _useWeekCalendar["default"])(calendarProps);
  var finalSelectedWeek = calendarState.finalSelectedWeek;
  var finalUIState = (0, _extends2["default"])({}, calendarState.finalUIState, {
    popupOpen: finalPopupOpen
  });
  var finalValue = formatWeekStr(finalSelectedWeek); // 切换日历面板的显示隐藏

  var updateUI = function updateUI(innerPopupOpen) {
    var newState = (0, _extends2["default"])({}, calendarState.finalUIState, {
      popupOpen: innerPopupOpen
    });
    setPopupOpen(innerPopupOpen);

    if (onUIChange) {
      onUIChange(newState);
    }
  };
  /**
   * 关闭日历面板
   */


  var closePopup = function closePopup() {
    if (!finalUIState.popupOpen) {
      return;
    }

    updateUI(false);
  };

  var handleCalendarUIChange = (0, _useEventCallback2.useEventCallback)(function (inputState) {
    calendarState.updateUI(inputState);
  });
  var handleCalendarChange = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function (week) {
    calendarState.updateSelectedWeek(week);

    if (week) {
      closePopup();
    }
  }, onCalendarChange));
  var handleFocus = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    updateUI(true);
  }, onFocus));
  var handleClear = (0, _useEventCallback2.useEventCallback)((0, _forkHandler["default"])(function () {
    calendarState.updateSelectedWeek();
  }, onClear));
  var handleClickAway = (0, _useEventCallback2.useEventCallback)(function () {
    if (finalPopupOpen) {
      closePopup();
    }
  });
  return {
    // 处理后的数据
    finalUIState: finalUIState,
    finalSelectedWeek: finalSelectedWeek,
    finalValue: finalValue,
    otherProps: otherProps,
    // 事件处理
    handleFocus: handleFocus,
    handleCalendarChange: handleCalendarChange,
    handleCalendarUIChange: handleCalendarUIChange,
    handleClickAway: handleClickAway,
    handleClear: handleClear,
    // 直接透传
    styles: styles,
    disabled: disabled,
    options: options,
    todayDate: todayDate,
    disableWeek: disableWeek,
    allowClear: allowClear,
    container: getPopupContainer,
    renderDecadeItem: renderDecadeItem,
    renderMonthItem: renderMonthItem,
    renderYearItem: renderYearItem,
    renderMonthLabel: renderMonthLabel,
    onSelect: onSelect,
    calendarStyles: calendarStyles,
    // triggerProps
    placement: placement,
    popperProps: popperProps,
    arrowPointAtCenter: arrowPointAtCenter,
    triggerId: triggerId,
    flip: flip,
    // for test
    formatWeekStr: formatWeekStr
  };
}