"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _theme = require("@muya-ui/theme-light/lib/theme");

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _styledComponents = require("styled-components");

var _Notification = require("../Notification");

var _ToastItem = _interopRequireDefault(require("./ToastItem"));

var toast = {
  _expQueue: new _Notification.ExpireQueue(),
  _autoInit: function _autoInit() {
    if (this._init) {
      return;
    }

    this._init = true;
    var opt = this._opt || {};
    var doc = window.document;
    this._mountEl = opt.container || doc.body;
    var containerElement = doc.createElement('div');

    this._mountEl.appendChild(containerElement);

    this._container = containerElement;
    var theme = opt.theme || _theme.muyaThemeLight;
    var poolSettings = opt.setting || theme.components.Notification.toast.queueSetting;
    var expireQueue = new _Notification.ExpireQueue();
    this._expQueue = expireQueue;

    if (poolSettings) {
      expireQueue.reset(poolSettings);
    }

    var children = /*#__PURE__*/_react["default"].createElement(_styledComponents.ThemeProvider, {
      theme: theme
    }, /*#__PURE__*/_react["default"].createElement(_Notification.NotificationList, {
      position: "top-center",
      className: "".concat(theme.prefix, "-toast-list"),
      expireQueue: expireQueue,
      container: containerElement
    }, function (state, item) {
      return /*#__PURE__*/_react["default"].createElement(_ToastItem["default"], {
        item: item,
        state: state,
        timeout: expireQueue.setting.timeout
      });
    }));

    if (opt.styleSheet) {
      _reactDom["default"].render( /*#__PURE__*/_react["default"].createElement(_styledComponents.StyleSheetManager, {
        target: opt.styleSheet
      }, children), containerElement);
    } else {
      _reactDom["default"].render(children, containerElement);
    }
  },
  destroy: function destroy() {
    if (this._container) {
      _reactDom["default"].unmountComponentAtNode(this._container);

      if (this._mountEl) {
        this._mountEl.removeChild(this._container);
      }

      this._container = undefined;
    }

    this._init = false;
  },
  config: function config(opt) {
    this.destroy();
    this._opt = opt;

    this._autoInit();
  },
  _unshiftWithType: function _unshiftWithType(type, msg) {
    var item;

    if (typeof msg === 'string') {
      item = {
        content: msg,
        type: type
      };
    } else {
      item = msg;
      item.type = type;
    }

    return this.add(item);
  },
  success: function success(msg) {
    return this._unshiftWithType('success', msg);
  },
  error: function error(msg) {
    return this._unshiftWithType('error', msg);
  },
  info: function info(msg) {
    return this._unshiftWithType('info', msg);
  },
  warning: function warning(msg) {
    return this._unshiftWithType('warning', msg);
  },
  loading: function loading(msg) {
    return this._unshiftWithType('loading', msg);
  },
  close: function close(id) {
    this._expQueue.remove(id);
  },
  add: function add(item) {
    return this._expQueue.unshift(item);
  }
}; // 保证第一次进入的时候动画效果

toast.config({});
var _default = toast;
exports["default"] = _default;