import { IComponentSizeSpec, ITheme } from '@muya-ui/theme-light';
import React from 'react';
import { StyledComponent } from 'styled-components';
import { IInlineButtonProps } from '../Button';
import { ICustomStyleBaseProps, IMuyaPureComponent, ISizeSpecBaseProps } from '../types';
/**
 *  Breadcrumbs style keys
 * @styles item 每个子项
 * @styles headSeparator 第一个的分割符
 * @styles separator 默认的分隔符
 */
export declare type IBreadcrumbsStyleKeys = 'item' | 'headSeparator' | 'separator';
export interface IBreadcrumbItem {
    /**
     * 文字内容
     * @default -
     * */
    label: React.ReactNode;
    /**
     * 文字链接
     * @default -
     * */
    url?: string;
    /**
     * 点击事件
     * @default -
     * */
    onClick?: IInlineButtonProps['onClick'];
    /**
     * 鼠标进入事件
     * @default -
     * */
    onMouseEnter?: IInlineButtonProps['onMouseEnter'];
    /**
     * 鼠标离开事件
     * @default -
     * */
    onMouseLeave?: IInlineButtonProps['onMouseLeave'];
}
declare type IPickFromInlineButtonForBreadcrumbs = Pick<IInlineButtonProps, 'fontWeight'>;
export interface IBreadcrumbsPureProps extends ISizeSpecBaseProps, IPickFromInlineButtonForBreadcrumbs, ICustomStyleBaseProps<IBreadcrumbsStyleKeys> {
    /**
     * 分割符
     * @default ['>']
     **/
    separators?: React.ReactNode[];
    /**
     * 第一个元素单独设置
     * @default -
     * */
    headItem?: IBreadcrumbItem;
    /**
     * 可以传数组
     * @default -
     * */
    items?: IBreadcrumbItem[];
    /**
     * 面包屑最多显示个数，超出部分会被隐藏
     * @default 4
     */
    maxNode?: number;
    /**
     * 面包屑数量多于maxNode后的隐藏规则, 接受一个2个number的元组，第一个值为省略号前的数量，第二个值为省略号后的数量
     * @default [1,2]
     */
    ellipsisRule?: [number, number];
}
export interface IDropdownConatinerProp {
    size: IComponentSizeSpec;
}
export declare type IBreadcrumbsProps = IBreadcrumbsPureProps & React.HTMLAttributes<HTMLDivElement>;
/**
 * @dts[only]
 */
export declare const Breadcrumbs: StyledComponent<IMuyaPureComponent<IBreadcrumbsProps, HTMLDivElement>, ITheme, IBreadcrumbsProps>;
export {};
