import React from 'react';
import { IButtonProps } from '../Button';
import { IMenuProps, IMenuSelectInfo } from '../Menu';
import { IPopperPlacement } from '../Popper';
import { ITriggerAction, ITriggerContainerFunc, ITriggerProps } from '../Trigger';
import { IMuyaPureComponent, IReactElementWithRef, ISizeSpecBaseProps, Omit } from '../types';
export declare type IOverlayFunc = () => React.ReactElement<IMenuProps>;
export declare type IOverlay = React.ReactElement<IMenuProps> | IOverlayFunc;
export interface IDropdownProps extends ISizeSpecBaseProps, Omit<ITriggerProps, 'popup' | 'open' | 'children' | 'container' | 'placement' | 'triggerActions' | 'TransitionComponent' | 'hideArrow'> {
    /**
     * trigger id
     * @default -
     * */
    id?: string;
    /**
     * 是否禁用
     * @default false
     **/
    disabled?: boolean;
    /**
     * 菜单
     * @typeText React.ReactElement<IMenuProps> | () => React.ReactElement<IMenuProps>
     * @default -
     **/
    overlay: IOverlay;
    /**
     * 菜单弹出位置
     * @default 'bottom-start'
     **/
    placement?: IPopperPlacement;
    /**
     * 下拉触发行为
     * @typeText Array<'hover' | 'click' | 'focus'>
     * @default ['hover']
     **/
    triggerActions?: Array<ITriggerAction>;
    /**
     * 下拉触发行为，优先级低于 triggerActions
     * @default 'hover'
     **/
    triggerAction?: ITriggerAction;
    /**
     * 菜单是否可见
     * @default -
     */
    visible?: boolean;
    /**
     * 菜单是否默认可见
     * @default false
     **/
    defaultVisible?: boolean;
    /**
     * 过渡组件
     * @default Animation.Slide
     **/
    TransitionComponent?: React.ComponentType<any>;
    /**
     * 子节点
     * @default -
     *  */
    children: IReactElementWithRef;
    /**
     * 隐藏箭头
     * @default true
     */
    hideArrow?: boolean;
    /** 菜单绑定的节点 */
    getPopupContainer?: ITriggerContainerFunc;
    /** 点击菜单 */
    onOverlayClick?: (selectInfo: IMenuSelectInfo) => void;
}
export interface IDropdownButtonProps extends Omit<IDropdownProps, 'children'>, Omit<IButtonProps, 'theme'> {
    /**
     * 按钮类型
     * @default false
     **/
    plain?: boolean;
    /**
     * 是否是按钮组
     * @default false
     **/
    group?: boolean;
    /**
     * 展开的 icon
     * @default FoldIcon
     */
    expandIcon?: React.ReactNode;
    /**
     *  group icon
     * @default MoreIcon
     */
    groupIcon?: React.ReactNode;
    /**
     * 子节点
     * @default -
     */
    children: React.ReactNode;
    /**
     * dropdown 节点的 ref
     */
    dropdownRef?: React.Ref<unknown>;
}
/**
 * @dts[only]
 */
export declare const DropdownButton: IMuyaPureComponent<IDropdownButtonProps, HTMLElement>;
export declare const Dropdown: IMuyaPureComponent<IDropdownProps, HTMLElement, {
    Button: typeof DropdownButton;
}>;
