import React from 'react';
import { IPopperPlacement } from '../Popper';
import { ITooltipProps } from '../Tooltip';
import { Omit } from '../types';
import { ITypographyBaseProps } from '../Typography';
export interface IEllipsisContentProps extends ITypographyBaseProps {
    /**
     * 自定义 Tooltip 或者 title 展示的内容
     */
    customContent?: React.ReactNode;
    /**
     * 启用 tooltip，大部分场景建议使用原生的 Title，下个大版本默认值将改为 false
     * @default true
     */
    enableTooltip?: boolean;
    /**
     * 启用原生的 title 属性，下个大版本默认值将改为 true
     */
    enableTitle?: boolean;
    /**
     * 透传给 Tooltip 的 props，仅 enableTooltip 为 true 时有效
     */
    tooltipProps?: Omit<ITooltipProps, 'title' | 'children'>;
    /**
     * 弹出的位置
     */
    placement?: IPopperPlacement;
    /**
     * 子节点
     */
    children: React.ReactNode;
    /**
     * 透传给 Typography 的样式对象
     */
    style?: React.CSSProperties;
    /**
     * 透传给 Typography 的类名
     */
    className?: string;
    /**
     * 渲染的节点名
     */
    as?: keyof JSX.IntrinsicElements | React.ComponentType<any>;
}
/**
 * @dts[only]
 */
export declare const EllipsisContent: React.MemoExoticComponent<(props: IEllipsisContentProps) => JSX.Element>;
