import React from 'react';
import { ICustomStyleBaseProps, ICustomStyleItem, IMuyaPureComponent, Omit } from '../types';
export declare type ILayoutSplitterSetSizesFunc = (sizes: string[] | number[]) => void;
export interface ILayoutSplitterGutterDraggerProps {
    index: number;
    setSizes: ILayoutSplitterSetSizesFunc;
}
export interface ILayoutSplitterGutterProps {
    styles: Record<'gutter' | 'dragger', ICustomStyleItem>;
    direction?: ILayoutSplitDirection;
    onMouseDown?: (e: any) => void;
    draggable: boolean;
    gutterColor?: string;
    gutterDragger?: React.ReactNode;
}
export interface ILayoutContext {
    siderHook: {
        hasSider: boolean;
        addSider: (hasSider?: boolean) => void;
    };
}
export interface ILayoutPair {
    idx: number;
    a: HTMLElement;
    b: HTMLElement;
    aMini: number;
    bMini: number;
    closeToA: boolean;
    closeToB: boolean;
    gutter: HTMLElement;
    parent: HTMLElement;
    start: number;
    end: number;
    size: number;
    gutterSize: number;
    aSizePct: number;
    bSizePct: number;
}
export interface ILayoutPureProps {
    /**
     * 自定义高度
     */
    height?: string | number;
    /**
     * 自定义宽度
     */
    width?: string | number;
    /**
     * 背景色
     */
    background?: string;
}
export declare type ILayoutProps = ILayoutPureProps & React.HTMLAttributes<HTMLDivElement>;
export interface ILayoutHeaderProps extends React.HTMLAttributes<HTMLDivElement> {
    /**
     * 自定义高度
     */
    height?: string | number;
    /**
     * 背景色
     */
    background?: string;
}
export interface ILayoutFooterProps extends React.HTMLAttributes<HTMLDivElement> {
    /**
     * 自定义高度
     */
    height?: string | number;
    /**
     * 背景色
     */
    background?: string;
}
export interface ILayoutSiderProps extends React.HTMLAttributes<HTMLDivElement> {
    /**
     * 自定义宽度
     */
    width?: string | number;
    /**
     * 自定义高度
     */
    height?: string | number;
    /**
     * 背景色
     */
    background?: string;
    /**
     * 当前收起状态
     */
    collapsed?: boolean;
    /**
     * 收缩宽度
     */
    collapsedWidth?: number;
    /**
     * 是否可收起
     * @default false
     */
    collapsible?: boolean;
    /**
     * 是否默认收起
     * @default false
     */
    defaultCollapsed?: boolean;
    /**
     * 翻转折叠提示箭头的方向，当 Sider 在右边时可以使用
     * @default false
     */
    reverseArrow?: boolean;
    /**
     * 自定义 trigger，设置为 null 时隐藏 trigger
     */
    trigger?: React.ReactNode;
    /**
     * 展开-收起时的回调函数
     */
    onCollapse?: (collapsed: boolean) => void;
}
export interface ILayoutContentProps extends React.HTMLAttributes<HTMLDivElement> {
    /**
     * 自定义宽度
     */
    width?: string | number;
    /**
     * 自定义高度
     */
    height?: string | number;
    /**
     * 背景色
     */
    background?: string;
}
export declare type ILayoutSplitDirection = 'horizontal' | 'vertical';
export declare type ILayoutSplitterStyleKeys = 'content' | 'gutter' | 'dragger';
export interface ILayoutSplitterPureProps extends ICustomStyleBaseProps<ILayoutSplitterStyleKeys> {
    /**
     * 子面板布局方向
     * @default 'horizontal'
     */
    direction?: ILayoutSplitDirection;
    /**
     * 子面板在布局方向上的最小高度/宽度, 单位像素, 传数组会从前到后对应到每一个的最小尺寸, 传数值则应用到所有子布局上
     * @default 0
     */
    miniSizes?: number | number[];
    /**
     * 初始子面板在布局方向上占据的空间，可以是 px、%，如果为空的话会自适应剩余空间，传入数字的话单位默认为 px
     */
    initialSizes?: number[] | string[];
    /**
     * 是否可拖拽
     * @default true
     */
    draggable?: boolean;
    /**
     * 间隔线颜色
     */
    gutterColor?: string;
    /**
     * 自定义 gutter 交互节点
     */
    renderGutterDragger?: (props: ILayoutSplitterGutterDraggerProps) => React.ReactNode;
    /**
     * 子节点
     */
    children: React.ReactElement[];
    /**
     * 鼠标在 gutter 上按下时的回调
     */
    onGutterMouseDown?: (e: MouseEvent, pairs: ILayoutPair[]) => void;
    /**
     * 鼠标在 gutter 上放开时的回调
     */
    onGutterMouseUp?: (e: MouseEvent, pairs: ILayoutPair[]) => void;
}
export declare type ILayoutSplitterProps = ILayoutSplitterPureProps & Omit<React.HTMLAttributes<HTMLDivElement>, 'children'>;
/**
 * @dts[only]
 */
export declare const Layout: IMuyaPureComponent<ILayoutProps, HTMLDivElement, {
    Content: IMuyaPureComponent<ILayoutContentProps, HTMLDivElement>;
    Header: IMuyaPureComponent<ILayoutHeaderProps, HTMLDivElement>;
    Sider: IMuyaPureComponent<ILayoutSiderProps, HTMLDivElement>;
    Footer: IMuyaPureComponent<ILayoutFooterProps, HTMLDivElement>;
    Splitter: IMuyaPureComponent<ILayoutSplitterProps, HTMLDivElement & {
        setSizes: ILayoutSplitterSetSizesFunc;
    }>;
}>;
