import React, { HTMLAttributes } from 'react';
import { ICustomStyleBaseProps, IMuyaPureComponent, ISizeSpecBaseProps } from '../types';
export declare type IPaginationNode = HTMLUListElement;
export declare type IPaginationAlign = 'left' | 'center' | 'right';
/**
 * Pagination style keys
 * @styles wrapper 容器
 * @styles input 简洁型页码输入框
 * @styles numberItem 非简洁型数字及箭头 Item
 * @styles simpleNumberItem 简洁型数字 Item
 * @styles simpleArrow 简洁型箭头
 * @styles jumpText 跳至 / 页文字
 */
export declare type IPaginationStyleKeys = 'wrapper' | 'input' | 'numberItem' | 'simpleNumberItem' | 'simpleArrow' | 'jumpText';
/**
 * Base Props
 */
export interface IPaginationPureProps extends ISizeSpecBaseProps, ICustomStyleBaseProps<IPaginationStyleKeys> {
    /**
     * 默认当前第几页
     * @default 1
     */
    defaultCurrent?: number;
    /**
     * 当前第几页, 受控
     */
    current?: number;
    /**
     * 总记录数
     * @default 0
     */
    totalRecords?: number;
    /**
     * 是否显示总条数
     */
    showTotal?: boolean | ((total: number) => React.ReactNode);
    /**
     * 当前页码改变事件
     */
    onChange?: (page: number) => void;
    /**
     * 是否展示跳页输入框
     * @default false
     */
    showQuickJumper?: boolean;
    /**
     * 简洁型
     * @default false
     */
    simple?: boolean;
    /**
     * 只有一页时是否隐藏分页器, `showPageSizeChanger` 为 `true` 时不生效
     * @default false
     */
    hideOnSinglePage?: boolean;
    /**
     * 选中页两侧展示几项
     * @default 2
     */
    midSideWidth?: number;
    /**
     * 是否是深色背景
     * @default false
     */
    isDarkBackground?: boolean;
    /**
     * 每页数量
     */
    pageSize?: number;
    /**
     * 每页数量
     * @default 10
     */
    defaultPageSize?: number;
    /**
     * 是否展示 `pageSize` 切换器
     * @default false
     */
    showPageSizeChanger?: boolean;
    /**
     * 可以切换的 `pageSize` 列表
     * @default [10, 20, 50, 100]
     */
    pageSizeOptions?: number[];
    /**
     * 对齐方式
     * @default 'left'
     */
    align?: IPaginationAlign;
    /**
     * 当 `pageSize` 变化时触发
     */
    onPageSizeChange?: (pageSize: number) => void;
    /**
     * 用于自定义页码的结构，可用于优化 SEO
     */
    itemRender?: (page: number, type: 'page' | 'prev' | 'next' | 'jump-prev' | 'jump-next', originalElement: React.ReactElement<HTMLElement>) => React.ReactNode;
}
export declare type IPaginationItemType = 'page' | 'prev' | 'next' | 'jump-prev' | 'jump-next';
export declare type IPaginationItemRender = (page: number, type: IPaginationItemType, originalElement: React.ReactElement<HTMLElement>) => React.ReactNode;
export declare type IPaginationProps = IPaginationPureProps & Omit<HTMLAttributes<HTMLUListElement>, 'onChange'>;
/**
 * @dts[only]
 */
export declare const Pagination: IMuyaPureComponent<IPaginationProps, IPaginationNode>;
