import { CSSProperties, ReactNode } from 'react';
import { ITriggerProps } from '../Trigger';
import { ICustomStyleBaseProps, IMuyaPureComponent, Omit } from '../types';
/**
 * @styles wrapper 容器
 * @styles titleWrapper 标题容器
 * @styles actionsWrapper 操作区容器
 * @styles contentWrapper 内容容器
 */
export declare type IPopoverCardStyleKeys = 'wrapper' | 'titleWrapper' | 'actionsWrapper' | 'contentWrapper';
export interface IPopoverCardProps extends Omit<ITriggerProps, 'popup'>, ICustomStyleBaseProps<IPopoverCardStyleKeys> {
    /**
     * 卡片标题
     *
     * @type {React.ReactNode}
     * @memberof ITooltipProps
     */
    title?: ReactNode;
    /**
     * 卡片内容
     *
     * @type {ReactNode}
     * @memberof IPopoverCardProps
     */
    content?: ReactNode;
    /**
     * 卡片行动区域
     *
     * @type {ReactNode}
     * @memberof IPopoverCardProps
     */
    actions?: ReactNode;
    /**
     * 弹出层容器的样式
     *
     * @type {CSSProperties}
     * @memberof ITooltipProps
     */
    style?: CSSProperties;
    /**
     * 弹出层容器的class
     *
     * @type {string}
     * @memberof ITooltipProps
     */
    className?: string;
    /**
     * 组件默认会将 content 使用Typography包裹一次，该属性可以禁用此行为
     * @default false
     */
    disableTypography?: boolean;
}
export declare const PopoverCard: IMuyaPureComponent<IPopoverCardProps, HTMLDivElement>;
