import React from 'react';
import { ICustomStyleBaseProps, IMuyaPureComponent, ISizeSpecBaseProps, Omit } from '../types';
/**
 * @styles wrapper 容器
 * @styles inputRadio 内部的 input
 * @styles innerRadio 内部的radio
 * @styles text 文本
 */
export declare type IRadioStyleKeys = 'wrapper' | 'inputRadio' | 'innerRadio' | 'text';
/**
 * Base Props
 */
export interface IRadioPureProps extends ISizeSpecBaseProps, ICustomStyleBaseProps<IRadioStyleKeys> {
    /**
     * 设置自动溢出省略
     * @default false
     */
    ellipsis?: boolean;
    /** 容器宽度 */
    width?: string | number;
    /**
     * 是否默认聚焦
     * @default false
     */
    autoFocus?: boolean;
    /**
     * 占满容器宽度
     */
    /**
     * 指定当前是否选中, 受控用法
     */
    checked?: boolean;
    /**
     * 子节点
     */
    children?: React.ReactNode;
    /**
     * 初始是否选中
     * @default false
     */
    defaultChecked?: boolean;
    /**
     * 是否禁用
     * @default false
     */
    disabled?: boolean;
    /**
     * 自定义选中状态Node
     */
    checkedIcon?: React.ReactNode;
    /**
     * 自定义未选中状态Node
     */
    unCheckedIcon?: React.ReactNode;
    /**
     * checked 改变事件
     */
    onChange?: React.ChangeEventHandler<HTMLInputElement>;
    /**
     * 鼠标进入事件
     */
    onMouseEnter?: React.MouseEventHandler<HTMLElement>;
    /**
     * 鼠标移出事件
     */
    onMouseLeave?: React.MouseEventHandler<HTMLElement>;
    /**
     * 鼠标点击事件
     */
    onClick?: React.MouseEventHandler<HTMLElement>;
    /**
     * 根据 value 进行比较，判断是否选中
     */
    value?: any;
    /**
     * 内部 input 的 ref
     */
    inputRef?: React.RefObject<HTMLInputElement>;
}
export declare type IRadioProps = IRadioPureProps & Omit<React.InputHTMLAttributes<HTMLInputElement>, 'size' | 'onChange' | 'onMouseEnter' | 'onMouseLeave' | 'value' | 'onClick'>;
export declare type IRadioGroupValue = string | number | null;
export interface IRadioGroupOption extends Pick<IRadioPureProps, 'width' | 'ellipsis'> {
    /**
     * 变化事件
     */
    onChange?: React.ChangeEventHandler<HTMLInputElement>;
    /**
     * 是否禁用
     * @default false
     */
    disabled?: boolean;
    /**
     * 该项内容
     */
    label: React.ReactNode;
    /**
     * 该项的值
     */
    value: IRadioGroupValue;
    /**
     * 挂载自定义数据
     */
    data?: any;
}
export interface IRadioGroupPureProps extends ISizeSpecBaseProps, ICustomStyleBaseProps<IRadioStyleKeys> {
    /** 透传 radio 的 ellipsis */
    radioEllipsis?: boolean;
    /** 透传 radio 的 width */
    radioWidth?: string | number;
    /**
     * 默认值
     * @default null
     */
    defaultValue?: IRadioGroupValue;
    /**
     * 是否整体禁用
     * @default false
     */
    disabled?: boolean;
    /**
     * 变化事件
     * @param checkedValue
     */
    onChange?: (checkedValue: IRadioGroupValue) => void;
    /**
     * RadioGroup 具体参数
     */
    options?: Array<IRadioGroupOption | string>;
    /**
     * 当前选中值
     */
    value?: IRadioGroupValue;
    /**
     * 自定义 radio 的渲染
     * @default (radioNode: React.ReactElement) => React.ReactElement
     */
    renderRadio?: (radioNode: React.ReactElement, option: IRadioGroupOption, optionIndex: number) => React.ReactNode;
}
export declare type IRadioGroupProps = IRadioGroupPureProps & Omit<React.HTMLAttributes<HTMLDivElement>, 'onChange' | 'value' | 'defaultValue'>;
export interface IUseRadioResult {
    checkedState: boolean;
    handleInputChange: React.ChangeEventHandler<HTMLInputElement>;
    handleInputClick: React.MouseEventHandler;
    handleClick: React.MouseEventHandler;
    setCheckedState: React.Dispatch<React.SetStateAction<boolean>>;
}
export interface IUseRadioGroupResult<T> {
    opts: IRadioGroupOption[];
    valueState?: IRadioGroupValue;
    restProps: Partial<IRadioGroupProps>;
    children?: React.ReactNode;
    radioEllipsis?: boolean;
    radioWidth?: string | number;
    handleChange: (option: IRadioGroupOption, e: React.ChangeEvent<HTMLInputElement>) => void;
    handleClick: (option: IRadioGroupOption & React.HTMLAttributes<T>, e: React.MouseEvent<T, MouseEvent>) => void;
    renderRadio: (radioNode: React.ReactElement, option: IRadioGroupOption, optionIndex: number) => React.ReactNode;
}
/**
 * @dts[only]
 */
export declare const useRadio: <T extends HTMLElement>(props: IRadioProps, isControlled: boolean) => IUseRadioResult;
export declare const Radio: IMuyaPureComponent<IRadioProps, HTMLLabelElement, {
    useRadio: typeof useRadio;
    __MUYA_RADIO: true;
}>;
export declare const useRadioGroup: <T extends HTMLElement>(props: IRadioGroupProps) => IUseRadioGroupResult<T>;
export declare const RadioGroup: IMuyaPureComponent<IRadioGroupProps, HTMLDivElement, {
    useRadioGroup: typeof useRadioGroup;
}>;
