import { CSSProperties, ReactNode } from 'react';
import { ICustomStyleBaseProps, IMuyaPureComponent } from '../types';
/**
 * @styles wrapper 最外层容器
 * @styles item 每一个child的容器
 */
export declare type IRateStyleKeys = 'wrapper' | 'item';
export interface IconFuncProps {
    index: number;
}
export declare type IconFunc = (props: IconFuncProps) => ReactNode;
export interface IRateProps extends ICustomStyleBaseProps<IRateStyleKeys> {
    /**
     * 当前值，受控
     *
     * @default -
     * @type {number}
     * @memberof IRateProps
     */
    value?: number;
    /**
     * 默认值
     *
     * @default -
     * @type {number}
     * @memberof IRateProps
     */
    defaultValue?: number;
    /**
     * 图标总数
     *
     * @default 5
     * @type {number}
     * @memberof IRateProps
     */
    count?: number;
    /**
     * 只读，做展示使用
     *
     * @default -
     * @type {boolean}
     * @memberof IRateProps
     */
    readOnly?: boolean;
    /**
     * 图标
     *
     * @default -
     * @type {ReactNode}
     * @memberof IRateProps
     */
    icon?: ReactNode | IconFunc;
    /**
     * 显示结果
     *
     * @default -
     * @type {boolean}
     * @memberof IRateProps
     */
    showResult?: boolean;
    /**
     * 自定义每项的文字说明
     *
     * @default 5
     * @type {string[]}
     * @memberof IRateProps
     */
    infos?: string[];
    /**
     * 是否展示说明信息
     *
     * @default false
     * @type {boolean}
     * @memberof IRateProps
     */
    showInfo?: boolean;
    /**
     * 是否显示Tooltip，开启了该选项，选项下面的文字说明将隐藏
     *
     * @default false
     * @type {boolean}
     * @memberof IRateProps
     */
    showTooltip?: boolean;
    /**
     * 点亮的图标的颜色
     *
     * @default -
     * @type {string}
     * @memberof IRateProps
     */
    color?: string;
    /**
     * 自定义图标样式
     *
     * @default -
     * @type {CSSProperties}
     * @memberof IRateProps
     */
    style?: CSSProperties;
    /**
     * 是否只高亮当前选中项
     *
     * @default false
     * @type {boolean}
     * @memberof IRateProps
     */
    selectSingle?: boolean;
    /**
     * 选中事件回调
     */
    onChange?: (value: number) => void;
    /**
     * 鼠标经过时回调
     */
    onHoverChange?: (value: number) => void;
}
/**
 * @dts[only]
 */
export declare const Rate: IMuyaPureComponent<IRateProps, HTMLDivElement, {
    __MUYA_RATE: true;
}>;
