import React from 'react';
import { ITheme } from '@muya-ui/theme-light';
import { StyledComponent } from 'styled-components';
import { ICustomStyleBaseProps, IMuyaPureComponent, ISizeSpecBaseProps } from '../types';
/**
 * ScrollView style keys
 *
 * @styles content 内容节点
 */
export declare type IScrollViewStyleKeys = 'content';
export interface IScrollViewBaseProps extends ISizeSpecBaseProps, ICustomStyleBaseProps<IScrollViewStyleKeys> {
    /**
     * 横向滚动
     */
    scrollX?: boolean;
    /**
     * 纵向滚动
     * @default true
     */
    scrollY?: boolean;
    /**
     * 容器宽度
     */
    width?: string | number;
    /**
     * 容器高度
     */
    height?: string | number;
    /**
     * 容器最大宽度
     */
    maxWidth?: string | number;
    /**
     * 容器最大高度
     */
    maxHeight?: string | number;
    /**
     * 内容容器节点
     */
    onScroll?: React.UIEventHandler<HTMLDivElement>;
    /**
     * 获取内部content滚动容器的ref
     *
     * @type {React.Ref<HTMLDivElement>}
     * @memberof IScrollViewBaseProps
     */
    contentRef?: React.Ref<HTMLDivElement>;
}
export declare type IScrollViewProps = IScrollViewBaseProps & Omit<React.HTMLAttributes<HTMLDivElement>, 'onScroll'>;
/**
 * @dts[only]
 */
export declare const ScrollView: StyledComponent<IMuyaPureComponent<IScrollViewProps, HTMLDivElement>, ITheme, IScrollViewProps>;
