import { IComponentSizeSpec } from '@muya-ui/theme-light';
import React, { CSSProperties, HTMLAttributes, Key, RefAttributes } from 'react';
import { ICheckboxProps } from '../Checkbox';
import { IPaginationProps } from '../Pagination';
import { IPopoverCardProps } from '../PopoverCard';
import { IRadioProps } from '../Radio';
import { ISpinProps } from '../Spin';
import { ICustomStyleBaseProps, ISizeSpecBaseProps, Omit } from '../types';
import { ITypographyEllipsis } from '../Typography';
import { IPopperProps } from '../Popper';
export declare type ITableRowKey = string | number;
export interface ITablePaginationState {
    current?: number;
    totalRecords?: number;
    pageSize?: number;
}
export interface ITableSelectedRowKeys {
    checked: ITableRowKey[];
    halfChecked: ITableRowKey[];
}
export declare type ITableRowProps = HTMLAttributes<HTMLTableRowElement> & RefAttributes<HTMLTableRowElement> & Record<string, any>;
export declare type ITableCellProps = React.TdHTMLAttributes<HTMLTableDataCellElement> & RefAttributes<HTMLTableCellElement> & Record<string, any>;
export interface ITableExpandableConfig<D> {
    /**
     * 自定义style
     *
     * @type {CSSProperties}
     * @memberof ITableExpandableConfig
     */
    style?: CSSProperties;
    /**
     * 默认展开所有行
     *
     * @type {boolean}
     * @memberof ITableExpandableConfig
     */
    defaultExpandAllRows?: boolean;
    /**
     * 默认展开的rowKeys
     *
     * @type {ITableRowKey}
     * @memberof ITableExpandableConfig
     */
    defaultExpandedRowKeys?: ITableRowKey[];
    /**
     * 展开的rowKeys，受控模式使用
     *
     * @type {ITableRowKey}
     * @memberof ITableExpandableConfig
     */
    expandedRowKeys?: ITableRowKey[];
    /**
     * 展开行数据变化时触发该事件
     *
     * @memberof ITableExpandableConfig
     */
    onExpandedRowsChange?: (expandedRowKeys: ITableRowKey[]) => void;
    /**
     * 任意行展开/收起时触发该事件
     *
     * @memberof ITableExpandableConfig
     */
    onExpand?: (expanded: boolean, record: D) => void;
    /**
     * 展开列的渲染逻辑
     *
     * @memberof ITableExpandableConfig
     */
    expandedRowRender?: (record: D, rowIndex: number, expanded: boolean) => React.ReactNode;
    /**
     * 控制列是否可以展开的逻辑
     *
     * @memberof ITableExpandableConfig
     */
    rowExpandable?: (record: D) => boolean;
    /**
     * 缩进宽度
     * @default 16
     * @type {number}
     * @memberof ITableExpandableConfig
     */
    indentSize?: number;
    /**
     * 指定树形结构的列名
     *
     * @default 'children'
     * @type {string}
     * @memberof ITableExpandableConfig
     */
    childrenColumnName?: string;
    /**
     * 展开行的背景色
     *
     * @default {theme.components.Table.trBackground.stripe}
     * @type {string}
     * @memberof ITableExpandableConfig
     */
    expandedRowBackgroundColor?: string;
}
export interface ITableBodyRowProps<D> {
    /**
     * 当前行序号
     *
     * @type {number}
     * @memberof ITableBodyRowProps
     */
    rowIndex: number;
    /**
     * 当前行对应的key
     *
     * @type {*}
     * @memberof ITableBodyRowProps
     */
    rowKey: any;
    /**
     * 当前行对应的数据
     *
     * @type {D}
     * @memberof ITableBodyRowProps
     */
    data: D;
    /**
     * 传给tr元素的props
     *
     * @type {(ITableRowProps)}
     * @memberof ITableBodyRowProps
     */
    rowProps?: ITableRowProps;
    /**
     * 子节点
     *
     * @type {React.ReactNode}
     * @memberof ITableBodyRowProps
     */
    children?: React.ReactNode;
}
export interface ITableBodyCellProps<D> extends ITableBodyRowProps<D> {
    /**
     * 当前单元格对应的列序号
     *
     * @type {number}
     * @memberof ITableBodyCellProps
     */
    columnIndex: number;
    /**
     * 单元格所在列对应的key
     *
     * @type {*}
     * @memberof ITableBodyCellProps
     */
    columnKey: any;
    /**
     * 单元格所处的column数据
     *
     * @type {ITableColumn<D>}
     * @memberof ITableBodyCellProps
     */
    column: ITableColumn<D>;
    /**
     * 传给td的props
     *
     * @type {(ITableCellProps)}
     * @memberof ITableBodyCellProps
     */
    cellProps?: ITableCellProps;
}
interface CustomizeBodyFnProps {
    setChildren: any;
    ref: React.RefObject<any>;
    onScroll: (event: React.UIEvent<HTMLDivElement>) => void;
}
export declare type CustomizeBodyFn = (props: CustomizeBodyFnProps) => React.ReactNode;
export declare type IBodyComponent<D> = React.ComponentType<{
    ref: any;
    onScroll: (event: React.UIEvent<HTMLDivElement>) => void;
    dataSource: D[];
    columns: ITableColumn<D>[];
    children: (customChildren: React.ReactNode) => React.ReactNode;
    [prop: string]: any;
}>;
export declare type IHeaderComponent = React.ComponentType<{
    onScroll?: (event: React.UIEvent<HTMLDivElement>) => void;
}> | React.ComponentType<HTMLAttributes<any>>;
export declare type IBody<D> = React.ComponentType<HTMLAttributes<any>> | CustomizeBodyFn | {
    /**
     * Body 实际渲染的组件
     *
     * @type {IBodyComponent<D>}
     */
    component: IBodyComponent<D>;
    /**
     * 滚动容器的 ref
     *
     * @type {React.RefObject<any>}
     */
    ref: React.RefObject<any>;
    /**
     * 自定义透传的 props
     *
     * @type {Record<string, any>}
     */
    props: Record<string, any>;
    [key: string]: any;
};
export interface ITableComponents<D> {
    BodyRow?: React.ComponentType<ITableBodyRowProps<D>>;
    BodyCell?: React.ComponentType<ITableBodyCellProps<D>>;
    HeaderRow?: React.ComponentType<ITableRowProps>;
    HeaderCell?: React.ComponentType<HTMLAttributes<HTMLTableHeaderCellElement>>;
    Table?: React.ComponentType<HTMLAttributes<any>>;
    Header?: IHeaderComponent;
    Body?: IBody<D>;
}
export declare type TableFilterValue = string | number;
export interface ITableRowSelection<D> {
    /**
     * checkable 状态下节点选择完全受控（父子数据选中状态不再关联）
     * @default true
     */
    checkStrictly?: boolean;
    /**
     * 受控模式使用，设置当前已选中的row keys
     *
     * @type {ITableRowKey[]}
     * @memberof ITableRowSelection
     */
    selectedRowKeys?: ITableRowKey[];
    /**
     * 选择列的宽度
     *
     * @default 60
     * @type {(string | number)}
     * @memberof ITableRowSelection
     */
    columnWidth?: string | number;
    /**
     * 选择列的表头，默认为全选/取消全选的Checkbox
     *
     * @type {(React.ReactNode)}
     * @memberof ITableRowSelection
     */
    columnTitle?: React.ReactNode;
    /**
     * 选中的row keys发生变化时的回调函数
     *
     * @memberof ITableRowSelection
     */
    onChange?: (selectedRowKeys: ITableRowKey[], selectedRows: D[]) => void;
    /**
     * 设置Checkbox的属性
     *
     * @memberof ITableRowSelection
     */
    getCheckboxProps?: (data: D) => ICheckboxProps;
    /**
     * 设置radio的属性
     *
     * @memberof ITableRowSelection
     */
    getRadioProps?: (data: D) => IRadioProps;
    /**
     * Checkbox 选中/取消时的回调
     *
     * @memberof ITableRowSelection
     */
    onSelect?: (selected: boolean, selectedRow: D) => void;
    /**
     * 全选/取消全选时的回调
     *
     * @memberof ITableRowSelection
     */
    onSelectAll?: (selected: boolean, selectedRows: D[]) => void;
    /**
     * 是否单选，默认多选
     * @default 'checkbox'
     */
    type?: 'checkbox' | 'radio';
    /**
     * 自定义节点渲染
     * @argument checked 当前状态是否被选中
     * @argument record 当前单元格所在行的数据
     * @argument index 当前单元格所在行的index
     * @argument node muya 对选择框默认渲染的节点，一般情况下函数返回值是对该节点的一次包装
     */
    renderCell?: (checked: boolean, record: D, index: number, node: React.ReactElement) => React.ReactElement;
}
export interface ITableFilterColumn {
    /**
     * 唯一key值
     *
     * @type {ITableRowKey}
     * @memberof ITableFilterColumn
     */
    key: ITableRowKey;
    /**
     * 筛选项的数据
     *
     * @type {TableFilterValue[]}
     * @memberof ITableFilterColumn
     */
    filterValues: TableFilterValue[];
    /**
     * 已选中的筛选项数据
     *
     * @type {TableFilterValue[]}
     * @memberof ITableFilterColumn
     */
    selectedFilterValues: TableFilterValue[];
}
export interface ColumnRenderSpanValue {
    children: React.ReactNode;
    props: ITableCellProps;
}
/**
 *
 * table column 结构
 *
 * @export
 * @interface ITableColumn
 * @template D
 */
export interface ITableColumn<D> {
    /**
     * 当前列的表头内容
     *
     * @type {React.ReactNode}
     * @memberof ITableColumn
     */
    title?: React.ReactNode;
    /**
     * 当前列的表尾内容
     *
     * @type {React.ReactNode}
     * @memberof ITableColumn
     */
    footer?: React.ReactNode;
    /**
     * 每一列所对应的key
     *
     * @type {Key}
     * @memberof ITableColumn
     */
    key: Key;
    /**
     * 表格内容默认为: dataSource[index][dataIndex]
     *
     * @type {keyof D}
     * @memberof ITableColumn
     */
    dataIndex?: keyof D;
    /**
     * 自定义表格内容渲染逻辑
     *
     * @argument data 当前单元格的数据
     * @argument columnIndex 当前单元格所在列的index
     * @argument rowIndex 当前单元格所在行的index
     * @memberof ITableColumn
     */
    render?: (data: D, columnIndex: number, rowIndex: number) => React.ReactNode | ColumnRenderSpanValue;
    /**
     * 开启table排序功能，并指定排序函数
     *
     * @memberof ITableColumn
     */
    sorter?: (a: D, b: D, sortOrder?: ITableSortOrder) => number;
    /**
     * 树形 Table 支持深度排序，默认值排序第一层级，优先级高于直接在 Table 上面设置的 deepSort
     * @default false
     */
    deepSort?: boolean;
    /**
     * 设置筛选功能
     *
     * @type {Array<{
     *     text: React.ReactNode;
     *     value: TableFilterValue;
     *   }>}
     * @memberof ITableColumn
     */
    filters?: Array<{
        text: React.ReactNode;
        value: TableFilterValue;
    }>;
    /**
     * 自定义筛选逻辑
     *
     * @memberof ITableColumn
     */
    onFilter?: (value: TableFilterValue, record: D) => boolean;
    /**
     * 筛选面板展开/收起时的事件
     *
     * @memberof ITableColumn
     */
    onFilterVisibleChange?: (visible: boolean) => void;
    /**
     * 筛选面板展开动画完成事件
     *
     * @memberof ITableColumn
     */
    onFilterEntered?: () => void;
    /**
     * 筛选面板收起动画完成事件
     *
     * @memberof ITableColumn
     */
    onFilterExited?: () => void;
    /**
     * 自定义筛选面板内容的渲染
     *
     * @memberof IFilterPopoverCardProps
     */
    renderFilterContent?: (props: IRenderFilterContentProps) => React.ReactNode;
    /**
     * 自定义筛选面板行动区域的渲染
     *
     * @memberof IFilterPopoverCardProps
     */
    renderFilterActions?: (props: IRenderFilterContentProps) => React.ReactNode;
    /**
     * 自定义筛选按钮
     *
     * @type {React.ReactNode}
     * @memberof ITableColumn
     */
    filterIcon?: React.ReactNode;
    /**
     * 手动控制筛选面板展开状态
     *
     * @type {boolean}
     * @memberof ITableColumn
     */
    filterVisible?: boolean;
    /**
     * 是否多选
     *
     * @default true
     * @type {boolean}
     * @memberof ITableColumn
     */
    filterMultiple?: boolean;
    /**
     * 弹出层的控制
     *
     * @type {Partial<IPopperProps>}
     * @memberof ITableColumn
     */
    filterPopperProps?: Partial<IPopperProps>;
    /**
     * 是否高亮内部筛选图标
     * @default false
     * @type {boolean}
     * @memberof ITableColumn
     */
    filterIconHighlight?: boolean;
    /**
     * 支持的排序方式，取值为 'ascend' 'descend'
     *
     * @type {ITableSortOrder[]}
     * @memberof ITableColumn
     */
    sortDirections?: ITableSortOrder[];
    /**
     * 列定宽
     *
     * @type {number}
     * @memberof ITableColumn
     */
    width?: number | string;
    /**
     * 超过宽度将自动省略，暂不支持和排序筛选一起使用。
     * 设置为 true 时，表格布局将变成 tableLayout="fixed"。
     *
     * @default false
     * @type {boolean}
     * @memberof ITableColumn
     */
    ellipsis?: boolean | ITypographyEllipsis;
    /**
     * ellipsis情况下，是否使用Tooltip展示详细内容
     *
     * @default true
     * @type {boolean}
     * @memberof ITableColumn
     */
    ellipsisTooltip?: boolean;
    /**
     * ellipsis情况下，是否使用原生 title 的 tip，优先级高于 ellipsisTooltip
     *
     * @type {boolean}
     * @memberof ITableColumn
     */
    ellipsisTitle?: boolean;
    /**
     * ellipsis情况下，默认会将溢出的节点作为 Tooltip 或者 title 属性的内容，但是一些情况下节点是链接等，此时如果不希望带上链接的行为或者样式，可以单独设置溢出的提示文案
     * @default -
     */
    renderEllipsisContent?: (data: D, columnIndex: number, rowIndex: number) => React.ReactNode;
    /**
     * 固定当前列，可配置固定的方向
     *
     * @type {('left' | 'right')}
     * @memberof ITableColumn
     */
    fixed?: 'left' | 'right';
    /**
     * 多级表头
     *
     * @memberof ITableColumn
     */
    children?: ITableColumn<D>[];
    /**
     * 列的对齐
     * @default 'left'
     */
    align?: 'left' | 'center' | 'right';
}
export declare type ITableSortOrder = 'descend' | 'ascend';
export declare type ITableStyleKeys = 'spinWrapper' | 'wrapper' | 'paginationWrapper';
export declare type TablePaginationPosition = 'topLeft' | 'topCenter' | 'topRight' | 'bottomLeft' | 'bottomCenter' | 'bottomRight';
export interface ITablePaginationProps extends Omit<IPaginationProps, 'onChange' | 'onPageSizeChange'> {
    /**
     * 分页器相对于表格的位置
     *
     * @type {TablePaginationPosition}
     * @memberof ITablePaginationProps
     */
    position?: TablePaginationPosition;
}
export interface ITableState {
    /**
     * 当前处于排序状态的Column
     *
     * @type {ITableRowKey}
     * @memberof ITableState
     */
    sortColumnKey?: ITableRowKey;
    /**
     * 当前处于筛选状态的Column
     *
     * @type {ITableRowKey}
     * @memberof ITableState
     */
    filterColumnKey?: ITableRowKey;
    /**
     * 当前排序的类型
     *
     * @type {ITableSortOrder}
     * @memberof ITableState
     */
    sortOrder?: ITableSortOrder;
    /**
     * 当前列筛选的数据
     *
     * @type {Array<ITableFilterColumnState>}
     * @memberof ITableState
     */
    filterColumnList?: Array<ITableFilterColumn>;
    /**
     * 当前分页的信息
     *
     * @type {ITablePaginationState}
     * @memberof ITableState
     */
    pagination?: ITablePaginationState;
}
/**
 *
 * Table组件参数
 *
 * @export
 * @interface ITableProps
 * @extends {HTMLAttributes<HTMLDivElement>}
 * @template D dataSource中子项的类型
 *
 */
export interface ITableProps<D> extends ISizeSpecBaseProps, ICustomStyleBaseProps<ITableStyleKeys>, Omit<HTMLAttributes<HTMLDivElement>, 'onChange'> {
    /**
     * 表格数据源
     *
     * @type {D[]}
     * @memberof ITableProps
     */
    dataSource: D[];
    /**
     * 表格列配置
     *
     * @type {ITableColumn<D>[]}
     * @memberof ITableProps
     */
    columns: ITableColumn<D>[];
    /**
     * 使用带边框的表格
     *
     * @type {boolean}
     * @memberof ITableProps
     * @default false
     */
    bordered?: boolean;
    /**
     * 使用斑马条纹的表格
     *
     * @type {boolean}
     * @memberof ITableProps
     * @default false
     */
    stripe?: boolean;
    /**
     * 空状态时提示文字
     *
     * @type {React.ReactNode}
     * @memberof ITableProps
     */
    emptyText?: React.ReactNode;
    /**
     * 渲染row时的key，默认取dataSource中子项的key
     *
     * @default 'key'
     * @memberof ITableProps
     */
    rowKey?: keyof D | ((data: D, index: number) => string);
    /**
     * 表格加载状态
     *
     * @type {(boolean | ISpinProps)}
     * @memberof ITableProps
     * @default false
     */
    loading?: boolean | Partial<ISpinProps>;
    /**
     * 固定表头
     *
     * @type {boolean}
     * @memberof ITableProps
     */
    fixedHeader?: boolean;
    /**
     * 固定表尾
     *
     * @type {boolean}
     * @memberof ITableProps
     */
    fixedFooter?: boolean;
    /**
     * 默认排序规则
     *
     * @type {ITableSortOrder[]}
     * @memberof ITableProps
     * @default ['ascend', 'descend']
     */
    defaultSortDirections?: ITableSortOrder[];
    /**
     * 设置默认的tableState
     *
     * @type {ITableState}
     * @memberof ITableProps
     */
    defaultTableState?: ITableState;
    /**
     * 表单的tableState，搭配onChange受控使用
     *
     * @type {ITableState}
     * @memberof ITableProps
     */
    tableState?: ITableState;
    /**
     * 页码配置
     *
     * @type {ITablePaginationProps}
     * @memberof ITableProps
     * @default false
     */
    pagination?: ITablePaginationProps;
    /**
     * Table内部状态发生变化时的回调函数
     *
     * @memberof ITableProps
     */
    onChange?: (state: ITableState) => void;
    /**
     * Table数据源发生变化时的回调函数
     *
     * @memberof ITableProps
     */
    onDataSourceChange?: (currentDataSource: D[]) => void;
    /**
     * 1. 表格元素的 table-layout 属性，设为 fixed 表示内容不会影响列的布局
     * 2. 固定表头/列或使用了 column.ellipsis 时，默认值为 fixed
     *
     * @type {('auto' | 'fixed')}
     * @memberof ITableProps
     */
    tableLayout?: 'auto' | 'fixed';
    /**
     * 设置table columns默认的ellipsis属性
     *
     * @default false
     * @type {boolean}
     * @memberof ITableProps
     */
    ellipsis?: boolean | ITypographyEllipsis;
    /**
     * 自定义style
     *
     * @type {CSSProperties}
     * @memberof ITableProps
     */
    style?: CSSProperties;
    /**
     * 自定义类名
     *
     * @type {string}
     * @memberof ITableProps
     */
    className?: string;
    /**
     * 表格是否可以选择
     *
     * @type {ITableRowSelection<D>}
     * @memberof ITableProps
     */
    rowSelection?: ITableRowSelection<D>;
    /**
     * 自定义表格行的属性
     *
     * @memberof ITableProps
     */
    onRow?: (data: D, rowIndex: number) => ITableRowProps;
    /**
     * 自定义单元格的属性(通过返回 `colSpan: 0` 或 `rowSpan: 0` 的方式让单元格不再渲染)
     *
     * @memberof ITableProps
     */
    onCell?: (data: D, rowIndex: number, column: ITableColumn<D>, columnIndex: number) => ITableCellProps;
    /**
     * 自定义表头行的属性
     *
     * @memberof ITableProps
     */
    onHeaderRow?: () => ITableRowProps;
    /**
     * 自定义表头单元格的属性
     *
     * @memberof ITableProps
     */
    onHeaderCell?: (column: ITableColumn<D>, columnIndex: number) => ITableCellProps;
    /**
     * 覆盖默认的 table 元素
     *
     * @default {}
     * @type {ITableComponents}
     * @memberof ITableProps
     */
    components?: ITableComponents<D>;
    /**
     * 展开功能的配置
     *
     * @type {ITableExpandableConfig<D>}
     * @memberof ITableProps
     * @default {}
     */
    expandable?: ITableExpandableConfig<D>;
    /**
     * 滚动条尺寸，局部的 table 建议使用小号 s/m 的滚动条，全屏的 table 建议使用大号 l/xl 的滚动条
     * @default 'l'
     */
    scrollBarSize?: IComponentSizeSpec;
    /**
     * 空状态高度
     * @default 'auto'
     */
    emptyHeight?: string | number;
    /**
     * 树形 Table 支持深度排序，默认值排序第一层级
     * @default false
     */
    deepSort?: boolean;
    /**
     * 传递到滚动容器节点的 ref
     */
    wrapRef?: React.Ref<HTMLDivElement>;
    /**
     * table 节点渲染模式, 默认单表，声明为 double 后 header 和 body 各为一个 table 节点
     * 一般场景下 muya 会根据需求自动切换渲染模式，不需要用户手动指定
     *
     * @default 'single'
     */
    renderMode?: ITableMode;
}
export interface IFilterPopoverCardProps extends Omit<IPopoverCardProps, 'children'>, ISizeSpecBaseProps {
    /**
     * 筛选面板所在列对应的column数据
     *
     * @type {ITableColumn<any>}
     * @memberof IFilterPopoverCardProps
     */
    column: ITableColumn<any>;
    /**
     * 当前所激活的column.key
     *
     * @type {ITableRowKey}
     * @memberof IFilterPopoverCardProps
     */
    activeColumnKey?: ITableRowKey;
    /**
     * 当前列的筛选数据
     *
     * @type {ITableFilterColumn}
     * @memberof IFilterPopoverCardProps
     */
    filterColumnData: ITableFilterColumn;
    /**
     * 设置当前激活筛选项的column.key
     *
     * @memberof IFilterPopoverCardProps
     */
    onFilterColumnKeyChange: (column: ITableColumn<any>) => void;
    /**
     * 将selectedFilterValues同步到tableState.filterColumnList中，调用后会触发一次`Table.onChange`事件
     *
     * @memberof IFilterPopoverCardProps
     */
    onFilterColumnDataChange: (filterColumn: ITableFilterColumn) => void;
    /**
     * 下个大版本移除，请勿使用
     */
    onFilterColumnListChange: (outerFilterColumnList: ITableFilterColumn[]) => void;
}
export interface IRenderFilterContentProps {
    /**
     * 确认方法，建议确定时手动调用
     *
     * @memberof IRenderFilterContentProps
     */
    confirm: () => void;
    /**
     * 重置方法，建议重置时手动调用
     *
     * @memberof IRenderFilterContentProps
     */
    reset: () => void;
    /**
     * 当前选中的数据，多选/单选组会使用该数据渲染
     */
    selectedFilterValues: TableFilterValue[];
    /**
     * 设置内部`selectedFilterValues`的数据
     */
    setSelectedFilterValues: (filterValues: TableFilterValue[], cb?: (newFilterColumnsList: ITableFilterColumn[]) => void) => void;
    /**
     * 设置当前激活筛选项的column.key
     */
    setFilterColumnKey: (column: ITableColumn<any>) => void;
    /**
     * 将selectedFilterValues同步到tableState.filterColumnList中，调用后会触发一次`Table.onChange`事件
     */
    syncFilterColumnsList: (outerFilterColumnList?: ITableFilterColumn[]) => void;
}
export declare type IWrapTableColumn<T> = ITableColumn<T> & {
    colspan?: number;
    rowspan?: number;
    parent?: IWrapTableColumn<T>;
};
/** 表格的Dom结构类型，分为单table和双table */
export declare type ITableMode = 'double' | 'single';
/**
 * @dts[only]
 */
export declare const Table: <D>(props: ITableProps<D>) => JSX.Element;
export {};
