import { ITableColumn, IWrapTableColumn, ITableSortOrder } from './types';
/**
 * 获取columns的最深children层数
 *
 * @export
 * @template T
 * @param {ITableColumn<T>[]} columns
 * @returns
 */
export declare function setParentAndGetDepth<T>(columns: ITableColumn<T>[]): number;
/**
 * 将columns处理为用于渲染header使用和用于渲染body使用的两个数组
 * 如：
 * 1      3      5
 * ├── 2  ├── 4
 *        ├── 6
 *            ├── 7
 * 返回的第一个数组为[[1, 3, 5], [2, 4, 6], [7]]
 * 返回的第二个数组为[2, 4, 7, 5]
 *
 * @export
 * @template T
 * @param {IWrapTableColumn<T>[]} columns
 * @param {number} maxDepth children的最大深度，可以从 setParentAndGetDepth 获得
 * @returns {[IWrapTableColumn<T>[][], IWrapTableColumn<T>[]]}
 */
export declare function genHeaderArr<T>(columns: IWrapTableColumn<T>[], maxDepth: number): [IWrapTableColumn<T>[][], IWrapTableColumn<T>[]];
/** 获得 column 列表，[[columnA, columnB], [columnA, columnC]]
 * 如：
 * 1      3      5
 *        ├── 4
 *        ├── 6
 *            ├── 7
 * 返回为[[1,3,5], [1,4,6,5], [1,4,7,5]]
 *
 * 用于计算宽度
 *
 */
export declare function getChildIndexes(lastIndexes: ITableColumn<any>[][], column: ITableColumn<any>, direction: 'left' | 'right', depth: number): ITableColumn<any>[][];
export declare function getHasSort<T>(sorter: ITableColumn<T>['sorter'], children: ITableColumn<T>['children']): boolean;
export declare function getHasFilter<T>(filters: ITableColumn<T>['filters'], renderFilterContent: ITableColumn<T>['renderFilterContent'], children: ITableColumn<T>['children']): boolean;
export declare function sort<T>(props: {
    data: T[];
    sorter: (a: T, b: T, sortOrder?: ITableSortOrder) => number;
    childrenColumnName: string;
    deepSort: boolean;
}): T[];
