import { ReactNode } from 'react';
import { ICustomStyleBaseProps, IMuyaPureComponent, ISizeSpecBaseProps } from '../types';
export declare type ITagShape = 'circle' | 'rect';
/**
 * Tag style keys
 * @styles childrenWrapper 子节点容器
 * @styles closeIcon 关闭按钮
 */
export declare type ITagStyleKeys = 'childrenWrapper' | 'closeIcon';
export declare type ITagType = 'normal' | 'primary' | 'danger' | 'success' | 'warning' | 'mask';
export interface ITagBaseProps {
    /**
     * 最大宽度
     */
    maxWidth?: number;
    /**
     * 形状
     * @default 'circle'
     */
    shape?: ITagShape;
    /**
     * 禁用状态
     * @default false
     */
    disabled?: boolean;
    /**
     * 是否有边框
     * @default false
     */
    bordered?: boolean;
    /**
     * 边框颜色
     */
    borderColor?: string;
    /**
     * 子节点
     */
    children: ReactNode;
    /**
     * Tag 的类型
     * * `normal`: 默认
     * * `primary`: 主要
     * * `danger`: 危险
     * * `success`: 成功
     * * `warning`: 警告
     * * `mask`: 蒙层
     * @default 'normal'
     */
    type?: ITagType;
}
export interface ITagPureProps extends ISizeSpecBaseProps, ITagBaseProps, ICustomStyleBaseProps<ITagStyleKeys> {
    /** 颜色 */
    color?: string;
    /**
     * 彩色低显示亮色
     * @default true
     */
    colorInverse?: boolean;
    /**
     * 是否可关闭
     * @default false
     **/
    closable?: boolean;
    /**
     * hover 是否高亮标签
     * @default true
     **/
    hoverable?: boolean;
    /**
     * 是否可见
     * @default true
     **/
    visible?: boolean;
    /**
     * 关闭的回调
     */
    onClose?: React.MouseEventHandler;
}
export declare type ITagProps = ITagPureProps & React.HTMLAttributes<HTMLSpanElement>;
/**
 * @dts[only]
 */
export declare const Tag: IMuyaPureComponent<ITagProps, HTMLSpanElement>;
