import { HTMLAttributes, ReactNode } from 'react';
import { ITheme } from '@muya-ui/theme-light';
import { StyledComponent } from 'styled-components';
import { ICustomStyleBaseProps, IMuyaPureComponent } from '../types';
/**
 * @styles wrapper 外层容器
 * @styles content 内容
 * @styles date 日期
 * @styles item 每项
 */
export declare type ITimelineStyleKeys = 'wrapper' | 'content' | 'date' | 'item';
export declare type ITimelineItemPositionType = 'left' | 'right' | 'top' | 'bottom';
export declare type ITimelineModeType = 'left' | 'right' | 'alternate' | 'alternate-left' | 'alternate-right';
export interface ITimelinePureProps extends ICustomStyleBaseProps<ITimelineStyleKeys> {
    /**
     * 设置内容相对于时间轴的位置
     * @default 'left'
     */
    mode?: ITimelineModeType;
}
export interface ITimelineContext {
    mode?: ITimelineModeType;
}
export declare type ITimelineProps = ITimelinePureProps & HTMLAttributes<HTMLDivElement>;
export interface ITimelineItemPureProps extends ICustomStyleBaseProps<ITimelineStyleKeys> {
    /**
     * 指定圆圈颜色
     */
    color?: string;
    /**
     * 自定义图标
     */
    dot?: ReactNode;
    /**
     * 时间相对内容的位置
     * @default 'left'
     */
    position?: ITimelineItemPositionType;
    /**
     * 时间
     */
    date?: string;
}
/**
 * @dts[only]
 */
export declare type ITimelineItemProps = ITimelineItemPureProps & HTMLAttributes<HTMLDivElement>;
export declare const TimelineItem: IMuyaPureComponent<ITimelineItemProps, HTMLDivElement>;
export declare const Timeline: StyledComponent<IMuyaPureComponent<ITimelineProps, HTMLDivElement>, ITheme, ITimelineProps> & {
    Item: typeof TimelineItem;
};
