import React from 'react';
import { IFlattenNode, ITreeCheckedKeys, ITreeCheckedKeysObject, ITreeContextValue, ITreeFilledFieldNamesType, ITreeNodeData, ITreeNodeDataEntity, ITreeNodeEventData, ITreeNodeKey, ITreeProps } from './types';
interface IWrapper<T> {
    posEntities: Record<string, ITreeNodeDataEntity<T>>;
    keyEntities: Record<ITreeNodeKey, ITreeNodeDataEntity<T>>;
}
export declare type IGetTreeKey<T> = (d: T, index?: number) => ITreeNodeKey;
export declare function isCheckDisabled(node: ITreeNodeData): boolean;
export declare function getKey(key: ITreeNodeKey | undefined, pos: string): React.ReactText;
export declare function getPosition(level: string | number, index: number): string;
/**
 * 检测 key 存在和重复
 * @param treeData
 */
export declare function warningWithoutKey(treeData: ITreeNodeData[] | undefined, fieldNames: ITreeFilledFieldNamesType): void;
/**
 * 遍历 tree
 * @param dataNodes
 * @param callback
 */
export declare function traverseDataNodes<T extends ITreeNodeData>(dataNodes: T[], fieldNames: ITreeFilledFieldNamesType, callback: (data: {
    node: T;
    index: number;
    pos: string;
    key: ITreeNodeKey;
    parentPos: string | number;
    level: number;
}) => void, externalGetKey?: IGetTreeKey<T> | string): void;
/**
 * Convert `treeData` into entity records.
 */
export declare function convertDataToEntities<T>(dataNodes: T[], fieldNames: ITreeFilledFieldNamesType, needTransition?: boolean, externalGetKey?: IGetTreeKey<T> | string): IWrapper<T>;
/**
 * Return selectedKeys according with multiple prop
 * @param selectedKeys
 * @param props
 * @returns [string]
 */
export declare function calcSelectedKeys(selectedKeys: ITreeNodeKey[] | undefined, multiple: boolean): React.ReactText[];
/**
 * If user use `autoExpandParent` we should get the list of parent node
 * @param keyList
 * @param keyEntities
 */
export declare function conductExpandParent(keyList: ITreeNodeKey[], keyEntities: Record<ITreeNodeKey, ITreeNodeDataEntity>): React.ReactText[];
/**
 * Parse `checkedKeys` to { checkedKeys, halfCheckedKeys } style
 */
export declare function parseCheckedKeys(keys: ITreeCheckedKeys | undefined): {
    checked: React.ReactText[];
    halfChecked: React.ReactText[];
} | null;
/**
 * Conduct with keys.
 * @param keyList current key list
 * @param keyEntities key - dataEntity map
 * @param mode `fill` to fill missing key, `clean` to remove useless key
 */
export declare function conductCheck(keyList: ITreeNodeKey[], checked: boolean, keyEntities: Record<ITreeNodeKey, ITreeNodeDataEntity>, checkStatus?: {
    checkedKeys?: ITreeNodeKey[];
    halfCheckedKeys?: ITreeNodeKey[];
}): ITreeCheckedKeysObject;
export declare function getDragNodesKeys(dragNodeKey: ITreeNodeKey, keyEntities: Record<ITreeNodeKey, ITreeNodeDataEntity>): ITreeNodeKey[];
export declare function posToArray(pos: string): string[];
export declare function calcDropPosition<T extends ITreeNodeData = ITreeNodeData>(event: React.MouseEvent, treeNode: ITreeNodeEventData<T>, prevNodeKey: ITreeNodeKey, startMousePosition?: {
    x: number;
    y: number;
}, flattenExpandedTreeData?: IFlattenNode<T>[], expandedKeys?: ITreeNodeKey[], keyEntities?: Record<ITreeNodeKey, ITreeNodeDataEntity>, virtual?: boolean): {
    dropPosition: 0 | 1 | -1;
    dropTargetKey: React.ReactText;
    dragOverNodeKey: React.ReactText;
    dropAllowed: boolean;
    dropLevelOffset: number;
};
export declare function getTreeNodeState<T>(key: ITreeNodeKey, state: ITreeContextValue<T>): {
    expanded: boolean;
    selected: boolean;
    loaded: boolean;
    loading: boolean;
    checked: boolean;
    halfChecked: boolean;
    pos: string;
    dragOver: boolean;
    dragOverGapTop: boolean;
    dragOverGapBottom: boolean;
    hovering: boolean;
};
export declare const getInitExpandedKeys: <T>(props: ITreeProps<T>, keyEntities: Record<React.ReactText, ITreeNodeDataEntity<ITreeNodeData>>) => React.ReactText[];
export declare const getInitCheckedKeys: (checkedKeysProp: React.ReactText[] | ITreeCheckedKeysObject | undefined, checkStrictly: boolean, keyEntities: Record<React.ReactText, ITreeNodeDataEntity<ITreeNodeData>>, loadData?: boolean) => ITreeCheckedKeysObject;
export declare function getFlattenExpandedTreeData<T extends ITreeNodeData>(treeData: T[], expandedKeys: ITreeNodeKey[], fieldNames: ITreeFilledFieldNamesType, dynamicPropsForNode?: (node: T) => Partial<Record<'checkable' | 'selectable' | 'disabled' | 'disableCheckbox', boolean>>): IFlattenNode<T>[];
export declare function diffCheckedKeys(checkedKeys: ITreeCheckedKeys | undefined, checkedKeysProp: ITreeCheckedKeys): boolean;
export declare function getExpandRange<T extends ITreeNodeData>(shorter: T[], longer: T[], key: ITreeNodeKey): T[];
export declare function getMinimumRangeTransitionRange<T>(list: T[], height: number, itemHeight: number): T[];
export declare function transformNode<T extends ITreeNodeData>(node: T, fieldNames: ITreeFilledFieldNamesType, dynamicPropsForNode?: (node: T) => Partial<Record<'checkable' | 'selectable' | 'disabled' | 'disableCheckbox', boolean>>): T;
export declare function transformTreeData<T extends ITreeNodeData>(dataNodes: T[], fieldNames: ITreeFilledFieldNamesType, dynamicPropsForNode?: (node: T) => Partial<Record<'checkable' | 'selectable' | 'disabled' | 'disableCheckbox', boolean>>): T[];
export declare function isFirstChild<T extends ITreeNodeData = ITreeNodeData>(treeNodeEntity: ITreeNodeDataEntity<T>): boolean;
export declare function isLastChild<T extends ITreeNodeData = ITreeNodeData>(treeNodeEntity: ITreeNodeDataEntity<T>): boolean;
export declare function convertDataToEventData<T extends ITreeNodeData = ITreeNodeData>(nodeKey: ITreeNodeKey, state: ITreeContextValue<T>): ITreeNodeEventData<T>;
export {};
