import { ConfigType, Dayjs } from 'dayjs';
import React from 'react';
import { ICalendarBaseOption, ICalendarPanelBaseProps, ICalendarPureProps, ICalendarStyleKeys, ICalendarType, ICalendarUIChangeEvent } from '../Calendar';
import { IInputOnlyInputStyleKeys, IInputPureProps } from '../Input';
import { ITriggerContainerFunc, ITriggerProps } from '../Trigger';
import { ICommonTriggerProps, ICustomStyleBaseProps, IMuyaPureComponent, Omit } from '../types';
export declare type IWeekPickerValue = [ConfigType, ConfigType] | ConfigType;
declare type IPropsFromCalendar = Omit<IWeekCalendarPureProps, 'defaultWeek' | 'styles' | 'onChange' | 'onUIChange' | 'allowClear'>;
declare type IPropsFromInput = Omit<IInputPureProps, 'defaultValue' | 'value' | 'limit' | 'multiline' | 'autosize' | 'allowClear' | 'maxRows' | 'minRows' | 'focusWhenClear' | 'styles' | 'readOnly'>;
/**
 * ScrollView style keys
 *
 * @styles content 内容节点
 */
export declare type IWeekPickerStyleKeys = 'popPanel' | IInputOnlyInputStyleKeys;
export interface IWeekPickerUIChangeEvent extends ICalendarUIChangeEvent {
    /** 面板是否显示 */
    popupOpen: boolean;
}
/**
 * 日期的快捷选项
 */
export interface IWeekCalendarOption extends ICalendarBaseOption {
    /**
     * 时间
     * @default dayjs() 今天
     */
    week?: IWeekPickerValue;
}
/**
 * 日期的快捷选项的事件
 */
export interface IWeekCalendarOptionEvent {
    label: string;
    week: [Dayjs, Dayjs];
}
export interface IWeekCalendarPureProps extends ICalendarPanelBaseProps, ICustomStyleBaseProps<ICalendarStyleKeys> {
    /** 选择的日期 */
    selectedWeek?: IWeekPickerValue;
    /** 默认日期 */
    defaultWeek?: IWeekPickerValue;
    /** 周禁用 */
    disableWeek?: (week: [Dayjs, Dayjs]) => boolean;
    /**
     * 用于展示的类型
     * @default 'month'
     **/
    viewType?: ICalendarType;
    /** 用于展示的日期  */
    viewDate?: ConfigType;
    /** 今天的日期 */
    todayDate?: ConfigType;
    /**
     * 是否允许清空选择的日期
     * @default true
     */
    allowClear?: boolean;
    /** 设置默认的面板展示的日期 */
    defaultViewDate?: ConfigType;
    /** 日期的快捷选项 */
    options?: IWeekCalendarOption[];
    /** UI的状态变化，包括面板的翻页、切换 */
    onUIChange?: (event: ICalendarUIChangeEvent) => void;
    /** 选中则触发onSelect，取消时会 date 为 undefined */
    onChange?: (week?: [Dayjs, Dayjs]) => void;
    /** 日历区间选中 */
    onSelect?: (week: [Dayjs, Dayjs]) => void;
}
export declare type IWeekCalendarProps = IWeekCalendarPureProps & Omit<React.HTMLAttributes<HTMLDivElement>, 'onChange' | 'onSelect' | 'defaultValue'>;
export interface IWeekPickerBaseProps extends ICustomStyleBaseProps<IWeekPickerStyleKeys>, IPropsFromInput, ICommonTriggerProps, IPropsFromCalendar {
    /**
     * 日期格式化，参考 https://github.com/iamkun/dayjs
     * @default 'YYYY-MM-DD'
     */
    format?: string | ((date: [Dayjs, Dayjs]) => string);
    /**
     * 日期格式化分割符
     * @default ' - '
     */
    formatSeparator?: string;
    /**
     * 日期的位置
     * @default 'bottom'
     */
    placement?: ITriggerProps['placement'];
    /** 日期面板的显示/隐藏 */
    popupOpen?: boolean;
    /** 日期面板绑定的容器 */
    getPopupContainer?: ITriggerContainerFunc;
    /**
     * 日历组件的样式重置，详见 [日历样式重置](../components/calendar#styles)
     */
    calendarStyles?: ICalendarPureProps['styles'];
    /**
     * input 输入框 debounce 延时触发日历面板的UI同步
     * @default 200
     */
    /**
     * 是否允许清空选择的日期
     * @default true
     */
    allowClear?: boolean;
    /** WeekPicker 的值 */
    value?: IWeekPickerValue;
    /** WeekPicker 的默认值 */
    defaultValue?: IWeekPickerValue;
    /** UI的状态变化，包括面板的翻页、切换、展开收起 */
    onUIChange?: (event: IWeekPickerUIChangeEvent) => void;
    /** DatePicker 变化的值，注意 value === '' 时，date.isValid() === false */
    onChange?: (date: [Dayjs, Dayjs]) => void;
    /**
     * Calender的 onChange 透传
     */
    onCalendarChange?: (week?: [Dayjs, Dayjs]) => void;
}
export declare type IWeekPickerProps = IWeekPickerBaseProps & Omit<React.InputHTMLAttributes<HTMLInputElement>, 'size' | 'onClick' | 'onMouseLeave' | 'onMouseEnter' | 'value' | 'defaultValue' | 'onChange' | 'onSelect'>;
/**
 * @dts[only]
 */
export declare const WeekPicker: IMuyaPureComponent<IWeekPickerProps, HTMLDivElement, {
    __MUYA_WEEKPICKER: true;
}>;
export {};
